import * as C from '../common';
export declare namespace AWS.ApiGateway {
    /**
     * The AWS::ApiGateway::Account resource specifies the AWS Identity and Access Management  (IAM) role that Amazon
    API Gateway (API Gateway) uses to write API logs to Amazon CloudWatch Logs (CloudWatch
    Logs).
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-account.html
     */
    interface Account {
        Type: 'AWS::ApiGateway::Account';
        Properties: AWS.ApiGateway.Account.Properties;
    }
    /**
     * The AWS::ApiGateway::Account resource specifies the AWS Identity and Access Management  (IAM) role that Amazon
    API Gateway (API Gateway) uses to write API logs to Amazon CloudWatch Logs (CloudWatch
    Logs).
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-account.html
     */
    function Account(props: C.Omit<Account, 'Type'>): Account;
}
export declare namespace AWS.ApiGateway.Account {
    interface Properties {
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of an IAM role that has write access to CloudWatch
 Logs in your account.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-account.html#cfn-apigateway-account-cloudwatchrolearn
         */
        CloudWatchRoleArn?: C.YieldsString;
    }
}
export declare namespace AWS.ApiGateway {
    /**
     * The AWS::ApiGateway::ApiKey resource creates a unique key that you can distribute to clients who are executing
    Amazon API Gateway (API Gateway) Method resources that require an API key. To specify which API key clients must use, map
    the API key with the RestApi and Stage resources that include the methods requiring a key.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-apikey.html
     */
    interface ApiKey {
        Type: 'AWS::ApiGateway::ApiKey';
        Properties: AWS.ApiGateway.ApiKey.Properties;
    }
    /**
     * The AWS::ApiGateway::ApiKey resource creates a unique key that you can distribute to clients who are executing
    Amazon API Gateway (API Gateway) Method resources that require an API key. To specify which API key clients must use, map
    the API key with the RestApi and Stage resources that include the methods requiring a key.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-apikey.html
     */
    function ApiKey(props: C.Omit<ApiKey, 'Type'>): ApiKey;
}
export declare namespace AWS.ApiGateway.ApiKey {
    interface Properties {
        /**
         * String

         *
         * A description of the purpose of the API key.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-apikey.html#cfn-apigateway-apigateway-apikey-description
         */
        Description?: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether the API key can be used by clients.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-apikey.html#cfn-apigateway-apigateway-apikey-enabled
         */
        Enabled?: C.YieldsBoolean;
        /**
         * String

         *
         * A name for the API key. If you don't specify a name, AWS CloudFormation generates
 a unique physical ID and uses that ID for the API key name. For more information,
 see Name Type.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-apikey.html#cfn-apigateway-apigateway-apikey-name
         */
        Name?: C.YieldsString;
        /**
         * List of Amazon API Gateway ApiKey StageKey
         *
         * A list of stages to associated with this API key.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-apikey.html#cfn-apigateway-apigateway-apikey-stagekeys
         */
        StageKeys?: C.Yields<C.CFList<C.Yields<AWS.ApiGateway.ApiKey.StageKey>>>;
    }
}
export declare namespace AWS.ApiGateway.ApiKey {
    interface StageKey {
        /**
         * String

         *
         * The ID of a RestApi resource that includes the stage with which you want to associate the API key.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-apikey-stagekey.html#cfn-apigateway-apikey-stagekey-restapiid
         */
        RestApiId?: C.YieldsString;
        /**
         * String

         *
         * The name of the stage with which to associate the API key. The stage must be included
 in the RestApi resource that you specified in the RestApiId property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-apikey-stagekey.html#cfn-apigateway-apikey-stagekey-stagename
         */
        StageName?: C.YieldsString;
    }
}
export declare namespace AWS.ApiGateway {
    /**
     * The AWS::ApiGateway::Authorizer resource creates an authorization layer that Amazon API Gateway (API Gateway) activates
    for methods that have authorization enabled. API Gateway activates the authorizer
    when a client calls those methods.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html
     */
    interface Authorizer {
        Type: 'AWS::ApiGateway::Authorizer';
        Properties: AWS.ApiGateway.Authorizer.Properties;
    }
    /**
     * The AWS::ApiGateway::Authorizer resource creates an authorization layer that Amazon API Gateway (API Gateway) activates
    for methods that have authorization enabled. API Gateway activates the authorizer
    when a client calls those methods.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html
     */
    function Authorizer(props: C.Omit<Authorizer, 'Type'>): Authorizer;
}
export declare namespace AWS.ApiGateway.Authorizer {
    interface Properties {
        /**
         * String

         *
         * The credentials required for the authorizer. To specify an AWS Identity and Access
 Management (IAM) role that API Gateway assumes, specify the role's Amazon Resource
 Name (ARN). To use resource-based permissions on the AWS Lambda (Lambda) function,
 specify null.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-authorizercredentials
         */
        AuthorizerCredentials?: C.YieldsString;
        /**
         * Integer

         *
         * The time-to-live (TTL) period, in seconds, that specifies how long API Gateway caches
 authorizer results. If you specify a value greater than 0, API Gateway caches the authorizer responses. By default, API Gateway sets this property
 to 300. The maximum value is 3600, or 1 hour.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-authorizerresultttlinseconds
         */
        AuthorizerResultTtlInSeconds?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The authorizer's Uniform Resource Identifier (URI). If you specify TOKEN for the authorizer's Type property, specify a Lambda function URI, which has the form arn:aws:apigateway:region:lambda:path/path. The path usually has the form /2015-03-31/functions/LambdaFunctionARN/invocations.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-authorizeruri
         */
        AuthorizerUri?: C.YieldsString;
        /**
         * String

         *
         * The source of the identity in an incoming request. If you specify TOKEN for the authorizer's Type property, specify a mapping expression. The custom header mapping expression has
 the form method.request.header.name, where name is the name of a custom authorization header that clients submit as part of their
 requests.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-identitysource
         */
        IdentitySource?: C.YieldsString;
        /**
         * String

         *
         * A validation expression for the incoming identity. If you specify TOKEN for the authorizer's Type property, specify a regular expression. API Gateway uses the expression to attempt
 to match the incoming client token, and proceeds if the token matches. If the token
 doesn't match, API Gateway responds with a 401 (unauthorized request) error code.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-identityvalidationexpression
         */
        IdentityValidationExpression?: C.YieldsString;
        /**
         * String

         *
         * The name of the authorizer.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-name
         */
        Name?: C.YieldsString;
        /**
         * List of strings

         *
         * A list of the Amazon Cognito user pool Amazon Resource Names (ARNs) to associate with
 this authorizer. For more information, see Use Amazon Cognito Your User Pool in the API Gateway Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-providerarns
         */
        ProviderARNs?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The ID of the RestApi resource in which API Gateway creates the authorizer.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-restapiid
         */
        RestApiId: C.YieldsString;
        /**
         * String

         *
         * The type of authorizer:
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html#cfn-apigateway-authorizer-type
         */
        Type?: C.YieldsString;
    }
}
export declare namespace AWS.ApiGateway {
    /**
     * The AWS::ApiGateway::BasePathMapping resource creates a base path that clients who call your Amazon API Gateway API must
    use in the invocation URL.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-basepathmapping.html
     */
    interface BasePathMapping {
        Type: 'AWS::ApiGateway::BasePathMapping';
        Properties: AWS.ApiGateway.BasePathMapping.Properties;
    }
    /**
     * The AWS::ApiGateway::BasePathMapping resource creates a base path that clients who call your Amazon API Gateway API must
    use in the invocation URL.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-basepathmapping.html
     */
    function BasePathMapping(props: C.Omit<BasePathMapping, 'Type'>): BasePathMapping;
}
export declare namespace AWS.ApiGateway.BasePathMapping {
    interface Properties {
        /**
         * String

         *
         * The base path name that callers of the API must provide in the URL after the domain
 name.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-basepathmapping.html#cfn-apigateway-basepathmapping-basepath
         */
        BasePath?: C.YieldsString;
        /**
         * String

         *
         * The name of a DomainName resource.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-basepathmapping.html#cfn-apigateway-basepathmapping-domainname
         */
        DomainName: C.YieldsString;
        /**
         * String

         *
         * The name of the API.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-basepathmapping.html#cfn-apigateway-basepathmapping-restapiid
         */
        RestApiId?: C.YieldsString;
        /**
         * String

         *
         * The name of the API's stage.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-basepathmapping.html#cfn-apigateway-basepathmapping-stage
         */
        Stage?: C.YieldsString;
    }
}
export declare namespace AWS.ApiGateway {
    /**
     * The AWS::ApiGateway::ClientCertificate resource creates a client certificate that Amazon API Gateway (API Gateway) uses
    to configure client-side SSL authentication for sending requests to the integration
    endpoint.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-clientcertificate.html
     */
    interface ClientCertificate {
        Type: 'AWS::ApiGateway::ClientCertificate';
        Properties: AWS.ApiGateway.ClientCertificate.Properties;
    }
    /**
     * The AWS::ApiGateway::ClientCertificate resource creates a client certificate that Amazon API Gateway (API Gateway) uses
    to configure client-side SSL authentication for sending requests to the integration
    endpoint.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-clientcertificate.html
     */
    function ClientCertificate(props: C.Omit<ClientCertificate, 'Type'>): ClientCertificate;
}
export declare namespace AWS.ApiGateway.ClientCertificate {
    interface Properties {
        /**
         * String

         *
         * A description of the client certificate.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-clientcertificate.html#cfn-apigateway-clientcertificate-description
         */
        Description?: C.YieldsString;
    }
}
export declare namespace AWS.ApiGateway {
    /**
     * The AWS::ApiGateway::Deployment resource deploys an Amazon API Gateway (API Gateway) RestApi resource to a stage so that clients can call the API over the Internet. The stage
    acts as an environment.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-deployment.html
     */
    interface Deployment {
        Type: 'AWS::ApiGateway::Deployment';
        Properties: AWS.ApiGateway.Deployment.Properties;
    }
    /**
     * The AWS::ApiGateway::Deployment resource deploys an Amazon API Gateway (API Gateway) RestApi resource to a stage so that clients can call the API over the Internet. The stage
    acts as an environment.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-deployment.html
     */
    function Deployment(props: C.Omit<Deployment, 'Type'>): Deployment;
}
export declare namespace AWS.ApiGateway.Deployment {
    interface Properties {
        /**
         * String

         *
         * A description of the purpose of the API Gateway deployment.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-deployment.html#cfn-apigateway-deployment-description
         */
        Description?: C.YieldsString;
        /**
         * String

         *
         * The ID of the RestApi resource to deploy.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-deployment.html#cfn-apigateway-deployment-restapiid
         */
        RestApiId: C.YieldsString;
        /**
         * Amazon API Gateway Deployment StageDescription
         *
         * Configures the stage that API Gateway creates with this deployment.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-deployment.html#cfn-apigateway-deployment-stagedescription
         */
        StageDescription?: C.Yields<AWS.ApiGateway.Deployment.StageDescription>;
        /**
         * String

         *
         * A name for the stage that API Gateway creates with this deployment. Use only alphanumeric
 characters.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-deployment.html#cfn-apigateway-deployment-stagename
         */
        StageName?: C.YieldsString;
    }
}
export declare namespace AWS.ApiGateway.Deployment {
    interface MethodSetting {
        /**
         * Boolean

         *
         * Indicates whether the cached responses are encrypted.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription-methodsetting.html#cfn-apigateway-deployment-stagedescription-methodsetting-cachedataencrypted
         */
        CacheDataEncrypted?: C.YieldsBoolean;
        /**
         * Integer

         *
         * The time-to-live (TTL) period, in seconds, that specifies how long API Gateway caches
 responses.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription-methodsetting.html#cfn-apigateway-deployment-stagedescription-methodsetting-cachettlinseconds
         */
        CacheTtlInSeconds?: C.Yields<C.CFInteger>;
        /**
         * Boolean

         *
         * Indicates whether responses are cached and returned for requests. You must enable
 a cache cluster on the stage to cache responses. For more information, see Enable API Gateway Caching in a Stage to Enhance API Performance in the API Gateway Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription-methodsetting.html#cfn-apigateway-deployment-stagedescription-methodsetting-cachingenabled
         */
        CachingEnabled?: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Indicates whether data trace logging is enabled for methods in the stage. API Gateway
 pushes these logs to Amazon CloudWatch Logs.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription-methodsetting.html#cfn-apigateway-deployment-stagedescription-methodsetting-datatraceenabled
         */
        DataTraceEnabled?: C.YieldsBoolean;
        /**
         * String

         *
         * The HTTP method.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription-methodsetting.html#cfn-apigateway-deployment-stagedescription-methodsetting-httpmethod
         */
        HttpMethod?: C.YieldsString;
        /**
         * String

         *
         * The logging level for this method. For valid values, see the loggingLevel property of the Stage resource in the Amazon API Gateway API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription-methodsetting.html#cfn-apigateway-deployment-stagedescription-methodsetting-logginglevel
         */
        LoggingLevel?: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether Amazon CloudWatch metrics are enabled for methods in the stage.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription-methodsetting.html#cfn-apigateway-deployment-stagedescription-methodsetting-metricsenabled
         */
        MetricsEnabled?: C.YieldsBoolean;
        /**
         * String

         *
         * The resource path for this method. Forward slashes (/) are encoded as ~1 and the initial slash must include a forward slash. For example, the path value /resource/subresource must be encoded as /~1resource~1subresource. To specify the root path, use only a slash (/).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription-methodsetting.html#cfn-apigateway-deployment-stagedescription-methodsetting-resourcepath
         */
        ResourcePath?: C.YieldsString;
        /**
         * Integer

         *
         * The number of burst requests per second that API Gateway permits across all APIs,
 stages, and methods in your AWS account. For more information, see Manage API Request Throttling in the API Gateway Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription-methodsetting.html#cfn-apigateway-deployment-stagedescription-methodsetting-throttlingburstlimit
         */
        ThrottlingBurstLimit?: C.Yields<C.CFInteger>;
        /**
         * Number

         *
         * The number of steady-state requests per second that API Gateway permits across all
 APIs, stages, and methods in your AWS account. For more information, see Manage API Request Throttling in the API Gateway Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription-methodsetting.html#cfn-apigateway-deployment-stagedescription-methodsetting-throttlingratelimit
         */
        ThrottlingRateLimit?: C.Yields<C.CFDouble>;
    }
}
export declare namespace AWS.ApiGateway.Deployment {
    interface StageDescription {
        /**
         * Boolean

         *
         * Indicates whether cache clustering is enabled for the stage.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription.html#cfn-apigateway-deployment-stagedescription-cacheclusterenabled
         */
        CacheClusterEnabled?: C.YieldsBoolean;
        /**
         * String

         *
         * The size of the stage's cache cluster.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription.html#cfn-apigateway-deployment-stagedescription-cacheclustersize
         */
        CacheClusterSize?: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether the cached responses are encrypted.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription.html#cfn-apigateway-deployment-stagedescription-cachedataencrypted
         */
        CacheDataEncrypted?: C.YieldsBoolean;
        /**
         * Integer

         *
         * The time-to-live (TTL) period, in seconds, that specifies how long API Gateway caches
 responses.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription.html#cfn-apigateway-deployment-stagedescription-cachettlinseconds
         */
        CacheTtlInSeconds?: C.Yields<C.CFInteger>;
        /**
         * Boolean

         *
         * Indicates whether responses are cached and returned for requests. You must enable
 a cache cluster on the stage to cache responses. For more information, see Enable API Gateway Caching in a Stage to Enhance API Performance in the API Gateway Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription.html#cfn-apigateway-deployment-stagedescription-cachingenabled
         */
        CachingEnabled?: C.YieldsBoolean;
        /**
         * String

         *
         * The identifier of the client certificate that API Gateway uses to call your integration
 endpoints in the stage.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription.html#cfn-apigateway-deployment-stagedescription-clientcertificateid
         */
        ClientCertificateId?: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether data trace logging is enabled for methods in the stage. API Gateway
 pushes these logs to Amazon CloudWatch Logs.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription.html#cfn-apigateway-deployment-stagedescription-datatraceenabled
         */
        DataTraceEnabled?: C.YieldsBoolean;
        /**
         * String

         *
         * A description of the purpose of the stage.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription.html#cfn-apigateway-deployment-stagedescription-description
         */
        Description?: C.YieldsString;
        /**
         * String

         *
         * The version identifier of the API documentation snapshot.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription.html#cfn-apigateway-deployment-stagedescription-documentationversion
         */
        DocumentationVersion?: C.YieldsString;
        /**
         * String

         *
         * The logging level for this method. For valid values, see the loggingLevel property of the Stage resource in the Amazon API Gateway API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription.html#cfn-apigateway-deployment-stagedescription-logginglevel
         */
        LoggingLevel?: C.YieldsString;
        /**
         * Amazon API Gateway Deployment StageDescription MethodSetting
         *
         * Configures settings for all of the stage's methods.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription.html#cfn-apigateway-deployment-stagedescription-methodsettings
         */
        MethodSettings?: C.Yields<C.CFList<C.Yields<AWS.ApiGateway.Deployment.MethodSetting>>>;
        /**
         * Boolean

         *
         * Indicates whether Amazon CloudWatch metrics are enabled for methods in the stage.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription.html#cfn-apigateway-deployment-stagedescription-metricsenabled
         */
        MetricsEnabled?: C.YieldsBoolean;
        /**
         * String

         *
         * The name of the stage, which API Gateway uses as the first path segment in the invoke
 Uniform Resource Identifier (URI).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription.html#cfn-apigateway-deployment-stagedescription-stagename
         */
        StageName?: C.YieldsString;
        /**
         * Integer

         *
         * The number of burst requests per second that API Gateway permits across all APIs,
 stages, and methods in your AWS account. For more information, see Manage API Request Throttling in the API Gateway Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription.html#cfn-apigateway-deployment-stagedescription-throttlingburstlimit
         */
        ThrottlingBurstLimit?: C.Yields<C.CFInteger>;
        /**
         * Number

         *
         * The number of steady-state requests per second that API Gateway permits across all
 APIs, stages, and methods in your AWS account. For more information, see Manage API Request Throttling in the API Gateway Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription.html#cfn-apigateway-deployment-stagedescription-throttlingratelimit
         */
        ThrottlingRateLimit?: C.Yields<C.CFDouble>;
        /**
         * Mapping of key-value pairs

         *
         * A map that defines the stage variables. Variable names must consist of alphanumeric
 characters, and the values must match the following regular expression: [A-Za-z0-9-._~:/?#&amp;=,]+.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription.html#cfn-apigateway-deployment-stagedescription-variables
         */
        Variables?: C.Yields<C.CFMap<C.YieldsString>>;
    }
}
export declare namespace AWS.ApiGateway {
    /**
     * The AWS::ApiGateway::DocumentationPart resource creates a documentation
    part for an Amazon API Gateway API entity. For more information, see  Representation of API Documentation in API Gateway in the
    API Gateway Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-documentationpart.html
     */
    interface DocumentationPart {
        Type: 'AWS::ApiGateway::DocumentationPart';
        Properties: AWS.ApiGateway.DocumentationPart.Properties;
    }
    /**
     * The AWS::ApiGateway::DocumentationPart resource creates a documentation
    part for an Amazon API Gateway API entity. For more information, see  Representation of API Documentation in API Gateway in the
    API Gateway Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-documentationpart.html
     */
    function DocumentationPart(props: C.Omit<DocumentationPart, 'Type'>): DocumentationPart;
}
export declare namespace AWS.ApiGateway.DocumentationPart {
    interface Properties {
        /**
         *                             Type: Amazon API Gateway DocumentationPart Location
         *
         * The location of the API entity that the documentation applies to.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-documentationpart.html#cfn-apigateway-documentationpart-location
         */
        Location: C.Yields<AWS.ApiGateway.DocumentationPart.Location>;
        /**
         * String

         *
         * The documentation content map of the targeted API entity.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-documentationpart.html#cfn-apigateway-documentationpart-properties
         */
        Properties: C.YieldsString;
        /**
         * String

         *
         * The identifier of the targeted API entity.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-documentationpart.html#cfn-apigateway-documentationpart-restapiid
         */
        RestApiId: C.YieldsString;
    }
}
export declare namespace AWS.ApiGateway.DocumentationPart {
    interface Location {
        /**
         * String

         *
         * The HTTP verb of a method.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-documentationpart-location.html#cfn-apigateway-documentationpart-location-method
         */
        Method?: C.YieldsString;
        /**
         * String

         *
         * The name of the targeted API entity.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-documentationpart-location.html#cfn-apigateway-documentationpart-location-name
         */
        Name?: C.YieldsString;
        /**
         * String

         *
         * The URL path of the target.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-documentationpart-location.html#cfn-apigateway-documentationpart-location-path
         */
        Path?: C.YieldsString;
        /**
         * String

         *
         * The HTTP status code of a response.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-documentationpart-location.html#cfn-apigateway-documentationpart-location-statuscode
         */
        StatusCode?: C.YieldsString;
        /**
         * String

         *
         * The type of API entity that the documentation content applies to.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-documentationpart-location.html#cfn-apigateway-documentationpart-location-type
         */
        Type?: C.YieldsString;
    }
}
export declare namespace AWS.ApiGateway {
    /**
     * The AWS::ApiGateway::DocumentationVersion resource creates a snapshot of
    the documentation for an Amazon API Gateway API entity. For more information, see
    Representation of API Documentation in API Gateway in the
    API Gateway Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-documentationversion.html
     */
    interface DocumentationVersion {
        Type: 'AWS::ApiGateway::DocumentationVersion';
        Properties: AWS.ApiGateway.DocumentationVersion.Properties;
    }
    /**
     * The AWS::ApiGateway::DocumentationVersion resource creates a snapshot of
    the documentation for an Amazon API Gateway API entity. For more information, see
    Representation of API Documentation in API Gateway in the
    API Gateway Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-documentationversion.html
     */
    function DocumentationVersion(props: C.Omit<DocumentationVersion, 'Type'>): DocumentationVersion;
}
export declare namespace AWS.ApiGateway.DocumentationVersion {
    interface Properties {
        /**
         * String

         *
         * The description of the API documentation snapshot.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-documentationversion.html#cfn-apigateway-documentationversion-description
         */
        Description?: C.YieldsString;
        /**
         * String

         *
         * The version identifier of the API documentation snapshot.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-documentationversion.html#cfn-apigateway-documentationversion-documentationversion
         */
        DocumentationVersion: C.YieldsString;
        /**
         * String

         *
         * The identifier of the targeted API entity.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-documentationversion.html#cfn-apigateway-documentationversion-restapiid
         */
        RestApiId: C.YieldsString;
    }
}
export declare namespace AWS.ApiGateway {
    /**
     * The AWS::ApiGateway::DomainName resource specifies a custom domain name for your API in Amazon API Gateway (API Gateway).
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-domainname.html
     */
    interface DomainName {
        Type: 'AWS::ApiGateway::DomainName';
        Properties: AWS.ApiGateway.DomainName.Properties;
    }
    /**
     * The AWS::ApiGateway::DomainName resource specifies a custom domain name for your API in Amazon API Gateway (API Gateway).
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-domainname.html
     */
    function DomainName(props: C.Omit<DomainName, 'Type'>): DomainName;
}
export declare namespace AWS.ApiGateway.DomainName {
    interface Properties {
        /**
         * String

         *
         * The
 Amazon Resource Name (ARN) of an AWS-managed certificate stored in
 AWS Certificate Manager. For requirements and additional information about setting
 up certificates, see Get Certificates Ready in AWS Certificate Manager in the
 API Gateway Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-domainname.html#cfn-apigateway-domainname-certificatearn
         */
        CertificateArn: C.YieldsString;
        /**
         * String

         *
         * The custom domain name for your API in Amazon API Gateway.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-domainname.html#cfn-apigateway-domainname-domainname
         */
        DomainName: C.YieldsString;
    }
}
export declare namespace AWS.ApiGateway {
    /**
     * The
    AWS::ApiGateway::GatewayResponse resource creates
    a custom response for your API Gateway API. For more information, see API Gateway Responses
        in the API Gateway Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-gatewayresponse.html
     */
    interface GatewayResponse {
        Type: 'AWS::ApiGateway::GatewayResponse';
        Properties: AWS.ApiGateway.GatewayResponse.Properties;
    }
    /**
     * The
    AWS::ApiGateway::GatewayResponse resource creates
    a custom response for your API Gateway API. For more information, see API Gateway Responses
        in the API Gateway Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-gatewayresponse.html
     */
    function GatewayResponse(props: C.Omit<GatewayResponse, 'Type'>): GatewayResponse;
}
export declare namespace AWS.ApiGateway.GatewayResponse {
    interface Properties {
        /**
         *                             Type: String to string map

         *
         * The response parameters (paths, query strings, and headers) for the response. Duplicates
 not allowed.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-gatewayresponse.html#cfn-apigateway-gatewayresponse-responseparameters
         */
        ResponseParameters?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         *                             Type: String to string map

         *
         * The response templates for the response. Duplicates not allowed.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-gatewayresponse.html#cfn-apigateway-gatewayresponse-responsetemplates
         */
        ResponseTemplates?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         *                             Type: String

         *
         * The response type. For valid values, see GatewayResponse in the API Gateway API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-gatewayresponse.html#cfn-apigateway-gatewayresponse-responsetype
         */
        ResponseType: C.YieldsString;
        /**
         *                             Type: String

         *
         * The identifier of the targeted API entity.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-gatewayresponse.html#cfn-apigateway-gatewayresponse-restapiid
         */
        RestApiId: C.YieldsString;
        /**
         *                             Type: String

         *
         * The HTTP status code for the response.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-gatewayresponse.html#cfn-apigateway-gatewayresponse-statuscode
         */
        StatusCode?: C.YieldsString;
    }
}
export declare namespace AWS.ApiGateway {
    /**
     * The AWS::ApiGateway::Method resource creates Amazon API Gateway (API Gateway) methods that define the parameters
    and body that clients must send in their requests.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html
     */
    interface Method {
        Type: 'AWS::ApiGateway::Method';
        Properties: AWS.ApiGateway.Method.Properties;
    }
    /**
     * The AWS::ApiGateway::Method resource creates Amazon API Gateway (API Gateway) methods that define the parameters
    and body that clients must send in their requests.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html
     */
    function Method(props: C.Omit<Method, 'Type'>): Method;
}
export declare namespace AWS.ApiGateway.Method {
    interface Properties {
        /**
         * Boolean

         *
         * Indicates whether the method requires clients to submit a valid API key.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html#cfn-apigateway-method-apikeyrequired
         */
        ApiKeyRequired?: C.YieldsBoolean;
        /**
         * String

         *
         * The method's authorization type.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html#cfn-apigateway-method-authorizationtype
         */
        AuthorizationType?: C.YieldsString;
        /**
         * String

         *
         * The identifier of the authorizer to use on this method. If you specify this property, specify CUSTOM for the AuthorizationType property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html#cfn-apigateway-method-authorizerid
         */
        AuthorizerId?: C.YieldsString;
        /**
         * String

         *
         * The HTTP method that clients will use to call this method.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html#cfn-apigateway-method-httpmethod
         */
        HttpMethod: C.YieldsString;
        /**
         * Amazon API Gateway Method Integration
         *
         * The back-end system that the method calls when it receives a request.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html#cfn-apigateway-method-integration
         */
        Integration?: C.Yields<AWS.ApiGateway.Method.Integration>;
        /**
         * List of Amazon API Gateway Method MethodResponse
         *
         * The responses that can be sent to the client who calls the method.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html#cfn-apigateway-method-methodresponses
         */
        MethodResponses?: C.Yields<C.CFList<C.Yields<AWS.ApiGateway.Method.MethodResponse>>>;
        /**
         * Mapping of key-value pairs

         *
         * The resources used for the response's content type. Specify response models as key-value
 pairs (string-to-string map), with a content type as the key and a Model resource name as the value.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html#cfn-apigateway-method-requestmodels
         */
        RequestModels?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         * Mapping of key-value pairs

         *
         * Request parameters that API Gateway accepts. Specify request parameters as key-value
 pairs (string-to-Boolean map), with a source as the key and a Boolean as the value.
 The Boolean specifies whether a parameter is required. A source must match the following
 format method.request.location.name, where the location is querystring, path, or header, and
 name is a valid, unique parameter name.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html#cfn-apigateway-method-requestparameters
         */
        RequestParameters?: C.Yields<C.CFMap<C.YieldsBoolean>>;
        /**
         * String

         *
         * The ID of an API Gateway resource. For root resource methods, specify the RestApi root resource ID, such as { "Fn::GetAtt": ["MyRestApi", "RootResourceId"] }.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html#cfn-apigateway-method-resourceid
         */
        ResourceId: C.YieldsString;
        /**
         * String

         *
         * The ID of the RestApi resource in which API Gateway creates the method.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html#cfn-apigateway-method-restapiid
         */
        RestApiId: C.YieldsString;
    }
}
export declare namespace AWS.ApiGateway.Method {
    interface Integration {
        /**
         * List of strings

         *
         * A list of request parameters whose values API Gateway will cache.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-method-integration.html#cfn-apigateway-method-integration-cachekeyparameters
         */
        CacheKeyParameters?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * An API-specific tag group of related cached parameters.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-method-integration.html#cfn-apigateway-method-integration-cachenamespace
         */
        CacheNamespace?: C.YieldsString;
        /**
         * String

         *
         * The credentials required for the integration. To specify an AWS Identity and Access
 Management (IAM) role that API Gateway assumes, specify the role's Amazon Resource
 Name (ARN). To require that the caller's identity be passed through from the request,
 specify arn:aws:iam::*:user/*.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-method-integration.html#cfn-apigateway-method-integration-credentials
         */
        Credentials?: C.YieldsString;
        /**
         * String

         *
         * The integration's HTTP method type.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-method-integration.html#cfn-apigateway-method-integration-integrationhttpmethod
         */
        IntegrationHttpMethod?: C.YieldsString;
        /**
         * List of Amazon API Gateway Method Integration IntegrationResponse
         *
         * The response that API Gateway provides after a method's back end completes processing
 a request. API Gateway intercepts the back end's response so that you can control
 how API Gateway surfaces back-end responses. For example, you can map the back-end
 status codes to codes that you define.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-method-integration.html#cfn-apigateway-method-integration-integrationresponses
         */
        IntegrationResponses?: C.Yields<C.CFList<C.Yields<AWS.ApiGateway.Method.IntegrationResponse>>>;
        /**
         * String

         *
         * Indicates when API Gateway passes requests to the targeted back end. This behavior
 depends on the request's Content-Type header and whether you defined a mapping template for it.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-method-integration.html#cfn-apigateway-method-integration-passthroughbehavior
         */
        PassthroughBehavior?: C.YieldsString;
        /**
         * Mapping of key-value pairs

         *
         * The request parameters that API Gateway sends with the back-end request. Specify request
 parameters as key-value pairs (string-to-string maps), with a destination as the key
 and a source as the value.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-method-integration.html#cfn-apigateway-method-integration-requestparameters
         */
        RequestParameters?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         * Mapping of key-value pairs

         *
         * A map of Apache Velocity templates that are applied on the request payload. The template
 that API Gateway uses is based on the value of the Content-Type header sent by the
 client. The content type value is the key, and the template is the value (specified
 as a string), such as the following snippet:

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-method-integration.html#cfn-apigateway-method-integration-requesttemplates
         */
        RequestTemplates?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         * String

         *
         * The type of back end your method is running, such as HTTP or
 MOCK. For all of the valid values, see the type property for the Integration resource in the
 Amazon API Gateway REST API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-method-integration.html#cfn-apigateway-method-integration-type
         */
        Type?: C.YieldsString;
        /**
         * String

         *
         * The integration's Uniform Resource Identifier (URI).
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-method-integration.html#cfn-apigateway-method-integration-uri
         */
        Uri?: C.YieldsString;
    }
}
export declare namespace AWS.ApiGateway.Method {
    interface IntegrationResponse {
        /**
         * Mapping of key-value pairs

         *
         * The response parameters from the back-end response that API Gateway sends to the method
 response. Specify response parameters as key-value pairs (string-to-string mappings).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-method-integration-integrationresponse.html#cfn-apigateway-method-integration-integrationresponse-responseparameters
         */
        ResponseParameters?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         * Mapping of key-value pairs

         *
         * The templates used to transform the integration response body. Specify templates as
 key-value pairs (string-to-string maps), with a content type as the key and a template
 as the value. For more information, see API Gateway API Request and Response Payload-Mapping Template Reference in the API Gateway Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-method-integration-integrationresponse.html#cfn-apigateway-method-integration-integrationresponse-responsetemplates
         */
        ResponseTemplates?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         * String

         *
         * A regular expression that specifies which error strings or status codes from the back end map to the integration
 response.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-method-integration-integrationresponse.html#cfn-apigateway-method-integration-integrationresponse-selectionpattern
         */
        SelectionPattern?: C.YieldsString;
        /**
         * String

         *
         * The status code that API Gateway uses to map the integration response to a MethodResponse status code.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-method-integration-integrationresponse.html#cfn-apigateway-method-integration-integrationresponse-statuscode
         */
        StatusCode: C.YieldsString;
    }
}
export declare namespace AWS.ApiGateway.Method {
    interface MethodResponse {
        /**
         * Mapping of key-value pairs

         *
         * The resources used for the response's content type. Specify response models as key-value
 pairs (string-to-string maps), with a content type as the key and a Model resource name as the value.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-method-methodresponse.html#cfn-apigateway-method-methodresponse-responsemodels
         */
        ResponseModels?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         * Mapping of key-value pairs

         *
         * Response parameters that API Gateway sends to the client that called a method. Specify
 response parameters as key-value pairs (string-to-Boolean maps), with a destination
 as the key and a Boolean as the value. Specify the destination using the following
 pattern: method.response.header.name, where the name is a valid, unique header name. The Boolean specifies whether a parameter is required.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-method-methodresponse.html#cfn-apigateway-method-methodresponse-responseparameters
         */
        ResponseParameters?: C.Yields<C.CFMap<C.YieldsBoolean>>;
        /**
         * String

         *
         * The method response's status code, which you map to an IntegrationResponse.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-method-methodresponse.html#cfn-apigateway-method-methodresponse-statuscode
         */
        StatusCode: C.YieldsString;
    }
}
export declare namespace AWS.ApiGateway {
    /**
     * The AWS::ApiGateway::Model resource defines the structure of a request or response payload for an Amazon API
    Gateway (API Gateway) method.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-model.html
     */
    interface Model {
        Type: 'AWS::ApiGateway::Model';
        Properties: AWS.ApiGateway.Model.Properties;
    }
    /**
     * The AWS::ApiGateway::Model resource defines the structure of a request or response payload for an Amazon API
    Gateway (API Gateway) method.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-model.html
     */
    function Model(props: C.Omit<Model, 'Type'>): Model;
}
export declare namespace AWS.ApiGateway.Model {
    interface Properties {
        /**
         * String

         *
         * The content type for the model.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-model.html#cfn-apigateway-model-contenttype
         */
        ContentType?: C.YieldsString;
        /**
         * String

         *
         * A description that identifies this model.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-model.html#cfn-apigateway-model-description
         */
        Description?: C.YieldsString;
        /**
         * String

         *
         * A name for the model. If you don't specify a name, AWS CloudFormation
 generates a unique physical ID and uses that ID for the model name.
 For more information, see Name Type.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-model.html#cfn-apigateway-model-name
         */
        Name?: C.YieldsString;
        /**
         * String

         *
         * The ID of a REST API with which to associate this model.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-model.html#cfn-apigateway-model-restapiid
         */
        RestApiId: C.YieldsString;
        /**
         * JSON object

         *
         * The schema to use to transform data to one or more output formats. Specify null
 ({}) if you don't want to specify a schema.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-model.html#cfn-apigateway-model-schema
         */
        Schema?: C.Yields<C.CFJson>;
    }
}
export declare namespace AWS.ApiGateway {
    /**
     * The AWS::ApiGateway::RequestValidator resource sets up basic validation rules for incoming
    requests to your API Gateway API.
    For more information, see
    
       Enable Basic Request Validation for an API in API Gateway in the API Gateway Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-requestvalidator.html
     */
    interface RequestValidator {
        Type: 'AWS::ApiGateway::RequestValidator';
        Properties: AWS.ApiGateway.RequestValidator.Properties;
    }
    /**
     * The AWS::ApiGateway::RequestValidator resource sets up basic validation rules for incoming
    requests to your API Gateway API.
    For more information, see
    
       Enable Basic Request Validation for an API in API Gateway in the API Gateway Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-requestvalidator.html
     */
    function RequestValidator(props: C.Omit<RequestValidator, 'Type'>): RequestValidator;
}
export declare namespace AWS.ApiGateway.RequestValidator {
    interface Properties {
        /**
         * String

         *
         * The name of this request validator.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-requestvalidator.html#cfn-apigateway-requestvalidator-name
         */
        Name?: C.YieldsString;
        /**
         * String

         *
         * The identifier of the targeted API entity.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-requestvalidator.html#cfn-apigateway-requestvalidator-restapiid
         */
        RestApiId: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether to validate the request body according to the configured
 schema for the targeted API and method.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-requestvalidator.html#cfn-apigateway-requestvalidator-validaterequestbody
         */
        ValidateRequestBody?: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Indicates whether to validate request parameters.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-requestvalidator.html#cfn-apigateway-requestvalidator-validaterequestparameters
         */
        ValidateRequestParameters?: C.YieldsBoolean;
    }
}
export declare namespace AWS.ApiGateway {
    /**
     * The AWS::ApiGateway::Resource resource creates a resource in an Amazon API Gateway (API Gateway) API.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-resource.html
     */
    interface Resource {
        Type: 'AWS::ApiGateway::Resource';
        Properties: AWS.ApiGateway.Resource.Properties;
    }
    /**
     * The AWS::ApiGateway::Resource resource creates a resource in an Amazon API Gateway (API Gateway) API.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-resource.html
     */
    function Resource(props: C.Omit<Resource, 'Type'>): Resource;
}
export declare namespace AWS.ApiGateway.Resource {
    interface Properties {
        /**
         * String

         *
         * If you want to create a child resource, the ID of the parent resource. For resources
 without a parent, specify the RestApi root resource ID, such as { "Fn::GetAtt": ["MyRestApi", "RootResourceId"] }.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-resource.html#cfn-apigateway-resource-parentid
         */
        ParentId: C.YieldsString;
        /**
         * String

         *
         * A path name for the resource.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-resource.html#cfn-apigateway-resource-pathpart
         */
        PathPart: C.YieldsString;
        /**
         * String

         *
         * The ID of the RestApi resource in which you want to create this resource.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-resource.html#cfn-apigateway-resource-restapiid
         */
        RestApiId: C.YieldsString;
    }
}
export declare namespace AWS.ApiGateway {
    /**
     * The AWS::ApiGateway::RestApi resource contains a collection of Amazon API Gateway
    resources and methods that can be invoked through HTTPS endpoints. For more information,
    see
    restapi:create in the Amazon API Gateway REST API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-restapi.html
     */
    interface RestApi {
        Type: 'AWS::ApiGateway::RestApi';
        Properties: AWS.ApiGateway.RestApi.Properties;
    }
    /**
     * The AWS::ApiGateway::RestApi resource contains a collection of Amazon API Gateway
    resources and methods that can be invoked through HTTPS endpoints. For more information,
    see
    restapi:create in the Amazon API Gateway REST API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-restapi.html
     */
    function RestApi(props: C.Omit<RestApi, 'Type'>): RestApi;
}
export declare namespace AWS.ApiGateway.RestApi {
    interface Properties {
        /**
         * List of strings

         *
         * The list of binary media types supported by the RestApi resource, such as image/png or application/octet-stream. By default, RestApi supports only UTF-8-encoded
 text payloads. For more information, see Enable Support for Binary Payloads in API Gateway in the API Gateway Developer Guide. Duplicates are not allowed.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-restapi.html#cfn-apigateway-restapi-binarymediatypes
         */
        BinaryMediaTypes?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * JSON object

         *
         * An OpenAPI specification that defines a set of RESTful APIs in the JSON format.
 For YAML templates, you can also provide the specification in the YAML format.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-restapi.html#cfn-apigateway-restapi-body
         */
        Body?: C.Yields<C.CFJson>;
        /**
         * Amazon API Gateway RestApi S3Location
         *
         * The Amazon Simple Storage Service (Amazon S3) location that points to a OpenAPI file,
 which defines a set
 of RESTful APIs in JSON or YAML format.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-restapi.html#cfn-apigateway-restapi-bodys3location
         */
        BodyS3Location?: C.Yields<AWS.ApiGateway.RestApi.S3Location>;
        /**
         * String

         *
         * The ID of the API Gateway RestApi resource that you want to
 clone.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-restapi.html#cfn-apigateway-restapi-clonefrom
         */
        CloneFrom?: C.YieldsString;
        /**
         * String

         *
         * A description of the purpose of this API Gateway RestApi resource.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-restapi.html#cfn-apigateway-restapi-description
         */
        Description?: C.YieldsString;
        /**
         * Boolean

         *
         * If a warning occurs while API Gateway is creating the RestApi resource, indicates whether to roll back the resource.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-restapi.html#cfn-apigateway-restapi-failonwarnings
         */
        FailOnWarnings?: C.YieldsBoolean;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-restapi.html#cfn-apigateway-restapi-mode
         */
        Mode?: C.YieldsString;
        /**
         * String

         *
         * A name for the API Gateway RestApi resource.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-restapi.html#cfn-apigateway-restapi-name
         */
        Name?: C.YieldsString;
        /**
         * List of strings

         *
         * Custom header parameters for the request.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-restapi.html#cfn-apigateway-restapi-parameters
         */
        Parameters?: C.Yields<C.CFMap<C.YieldsString>>;
    }
}
export declare namespace AWS.ApiGateway.RestApi {
    interface S3Location {
        /**
         * String

         *
         * The name of the S3 bucket where the OpenAPI file is stored.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-restapi-bodys3location.html#cfn-apigateway-restapi-s3location-bucket
         */
        Bucket?: C.YieldsString;
        /**
         * String

         *
         * The Amazon S3 ETag (a file checksum) of the OpenAPI file. If you don't specify a value,
 API Gateway skips ETag validation of your OpenAPI file.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-restapi-bodys3location.html#cfn-apigateway-restapi-s3location-etag
         */
        ETag?: C.YieldsString;
        /**
         * String

         *
         * The file name of the OpenAPI file (Amazon S3 object name).
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-restapi-bodys3location.html#cfn-apigateway-restapi-s3location-key
         */
        Key?: C.YieldsString;
        /**
         * String

         *
         * For versioning-enabled buckets, a specific version of the OpenAPI file.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-restapi-bodys3location.html#cfn-apigateway-restapi-s3location-version
         */
        Version?: C.YieldsString;
    }
}
export declare namespace AWS.ApiGateway {
    /**
     * The AWS::ApiGateway::Stage resource creates a stage for an Amazon API Gateway (API Gateway) deployment.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-stage.html
     */
    interface Stage {
        Type: 'AWS::ApiGateway::Stage';
        Properties: AWS.ApiGateway.Stage.Properties;
    }
    /**
     * The AWS::ApiGateway::Stage resource creates a stage for an Amazon API Gateway (API Gateway) deployment.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-stage.html
     */
    function Stage(props: C.Omit<Stage, 'Type'>): Stage;
}
export declare namespace AWS.ApiGateway.Stage {
    interface Properties {
        /**
         * Boolean

         *
         * Indicates whether cache clustering is enabled for the stage.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-stage.html#cfn-apigateway-stage-cacheclusterenabled
         */
        CacheClusterEnabled?: C.YieldsBoolean;
        /**
         * String

         *
         * The stage's cache cluster size.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-stage.html#cfn-apigateway-stage-cacheclustersize
         */
        CacheClusterSize?: C.YieldsString;
        /**
         * String

         *
         * The identifier of the client certificate that API Gateway uses to call your integration
 endpoints in the stage.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-stage.html#cfn-apigateway-stage-clientcertificateid
         */
        ClientCertificateId?: C.YieldsString;
        /**
         * String

         *
         * The ID of the deployment that the stage points to.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-stage.html#cfn-apigateway-stage-deploymentid
         */
        DeploymentId?: C.YieldsString;
        /**
         * String

         *
         * A description of the stage's purpose.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-stage.html#cfn-apigateway-stage-description
         */
        Description?: C.YieldsString;
        /**
         * String

         *
         * The version identifier of the API documentation snapshot.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-stage.html#cfn-apigateway-stage-documentationversion
         */
        DocumentationVersion?: C.YieldsString;
        /**
         * Amazon API Gateway Stage MethodSetting
         *
         * Settings for all methods in the stage.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-stage.html#cfn-apigateway-stage-methodsettings
         */
        MethodSettings?: C.Yields<C.CFList<C.Yields<AWS.ApiGateway.Stage.MethodSetting>>>;
        /**
         * String

         *
         * The ID of the RestApi resource that you're deploying with this stage.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-stage.html#cfn-apigateway-stage-restapiid
         */
        RestApiId: C.YieldsString;
        /**
         * String

         *
         * The name of the stage, which API Gateway uses as the first path segment in the invoked
 Uniform Resource Identifier (URI).

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-stage.html#cfn-apigateway-stage-stagename
         */
        StageName?: C.YieldsString;
        /**
         * Mapping of key-value pairs

         *
         * A map (string-to-string map) that defines the stage variables, where the variable
 name is the key and the variable value is the value. Variable names are limited to
 alphanumeric characters. Values must match the following regular expression:
 [A-Za-z0-9-._~:/?#&amp;=,]+.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-stage.html#cfn-apigateway-stage-variables
         */
        Variables?: C.Yields<C.CFMap<C.YieldsString>>;
    }
}
export declare namespace AWS.ApiGateway.Stage {
    interface MethodSetting {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-stage-methodsetting.html#cfn-apigateway-stage-methodsetting-cachedataencrypted
         */
        CacheDataEncrypted?: C.YieldsBoolean;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-stage-methodsetting.html#cfn-apigateway-stage-methodsetting-cachettlinseconds
         */
        CacheTtlInSeconds?: C.Yields<C.CFInteger>;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-stage-methodsetting.html#cfn-apigateway-stage-methodsetting-cachingenabled
         */
        CachingEnabled?: C.YieldsBoolean;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-stage-methodsetting.html#cfn-apigateway-stage-methodsetting-datatraceenabled
         */
        DataTraceEnabled?: C.YieldsBoolean;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-stage-methodsetting.html#cfn-apigateway-stage-methodsetting-httpmethod
         */
        HttpMethod?: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-stage-methodsetting.html#cfn-apigateway-stage-methodsetting-logginglevel
         */
        LoggingLevel?: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-stage-methodsetting.html#cfn-apigateway-stage-methodsetting-metricsenabled
         */
        MetricsEnabled?: C.YieldsBoolean;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-stage-methodsetting.html#cfn-apigateway-stage-methodsetting-resourcepath
         */
        ResourcePath?: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-stage-methodsetting.html#cfn-apigateway-stage-methodsetting-throttlingburstlimit
         */
        ThrottlingBurstLimit?: C.Yields<C.CFInteger>;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apitgateway-stage-methodsetting.html#cfn-apigateway-stage-methodsetting-throttlingratelimit
         */
        ThrottlingRateLimit?: C.Yields<C.CFDouble>;
    }
}
export declare namespace AWS.ApiGateway {
    /**
     * The AWS::ApiGateway::UsagePlan resource specifies a usage plan for deployed Amazon API Gateway (API Gateway) APIs.
    A usage plan enforces throttling and quota limits on individual client API keys. For
    more information, see Creating and Using API Usage Plans in Amazon API Gateway in the API Gateway Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-usageplan.html
     */
    interface UsagePlan {
        Type: 'AWS::ApiGateway::UsagePlan';
        Properties: AWS.ApiGateway.UsagePlan.Properties;
    }
    /**
     * The AWS::ApiGateway::UsagePlan resource specifies a usage plan for deployed Amazon API Gateway (API Gateway) APIs.
    A usage plan enforces throttling and quota limits on individual client API keys. For
    more information, see Creating and Using API Usage Plans in Amazon API Gateway in the API Gateway Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-usageplan.html
     */
    function UsagePlan(props: C.Omit<UsagePlan, 'Type'>): UsagePlan;
}
export declare namespace AWS.ApiGateway.UsagePlan {
    interface Properties {
        /**
         * List of Amazon API Gateway UsagePlan ApiStage
         *
         * The API stages to associate with this usage plan.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-usageplan.html#cfn-apigateway-usageplan-apistages
         */
        ApiStages?: C.Yields<C.CFList<C.Yields<AWS.ApiGateway.UsagePlan.ApiStage>>>;
        /**
         * String

         *
         * The purpose of this usage plan.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-usageplan.html#cfn-apigateway-usageplan-description
         */
        Description?: C.YieldsString;
        /**
         * Amazon API Gateway UsagePlan QuotaSettings
         *
         * Configures the number of requests that users can make within a given interval.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-usageplan.html#cfn-apigateway-usageplan-quota
         */
        Quota?: C.Yields<AWS.ApiGateway.UsagePlan.QuotaSettings>;
        /**
         * Amazon API Gateway UsagePlan ThrottleSettings
         *
         * Configures the overall request rate (average requests per second) and burst capacity.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-usageplan.html#cfn-apigateway-usageplan-throttle
         */
        Throttle?: C.Yields<AWS.ApiGateway.UsagePlan.ThrottleSettings>;
        /**
         * String

         *
         * A name for this usage plan.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-usageplan.html#cfn-apigateway-usageplan-usageplanname
         */
        UsagePlanName?: C.YieldsString;
    }
}
export declare namespace AWS.ApiGateway.UsagePlan {
    interface ApiStage {
        /**
         * String

         *
         * The ID of an API that is in the specified Stage property that you want to associate with the usage plan.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-usageplan-apistage.html#cfn-apigateway-usageplan-apistage-apiid
         */
        ApiId?: C.YieldsString;
        /**
         * String

         *
         * The name of an API Gateway stage to associate with the usage plan.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-usageplan-apistage.html#cfn-apigateway-usageplan-apistage-stage
         */
        Stage?: C.YieldsString;
    }
}
export declare namespace AWS.ApiGateway.UsagePlan {
    interface QuotaSettings {
        /**
         * Integer

         *
         * The maximum number of requests that users can make within the specified time period.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-usageplan-quotasettings.html#cfn-apigateway-usageplan-quotasettings-limit
         */
        Limit?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * For the initial time period, the number of requests to subtract from the specified
 limit. When you first implement a usage plan, the plan might start in the middle of
 the week or month. With this property, you can decrease the limit for this initial
 time period.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-usageplan-quotasettings.html#cfn-apigateway-usageplan-quotasettings-offset
         */
        Offset?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The time period for which the maximum limit of requests applies, such as DAY or WEEK. For valid values, see the period property for the UsagePlan resource in the Amazon API Gateway REST API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-usageplan-quotasettings.html#cfn-apigateway-usageplan-quotasettings-period
         */
        Period?: C.YieldsString;
    }
}
export declare namespace AWS.ApiGateway.UsagePlan {
    interface ThrottleSettings {
        /**
         * Integer

         *
         * The maximum API request rate limit over a time ranging from one to a few seconds.
 The maximum API request rate limit depends on whether the underlying token bucket
 is at its full capacity. For more information about request throttling, see Manage API Request Throttling in the API Gateway Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-usageplan-throttlesettings.html#cfn-apigateway-usageplan-throttlesettings-burstlimit
         */
        BurstLimit?: C.Yields<C.CFInteger>;
        /**
         * Number

         *
         * The API request steady-state rate limit (average requests per second over an extended
 period of time). For more information about request throttling, see Manage API Request Throttling in the API Gateway Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-usageplan-throttlesettings.html#cfn-apigateway-usageplan-throttlesettings-ratelimit
         */
        RateLimit?: C.Yields<C.CFDouble>;
    }
}
export declare namespace AWS.ApiGateway {
    /**
     * The AWS::ApiGateway::UsagePlanKey resource associates an Amazon API Gateway API key with
    an API Gateway usage plan. This association determines which users the usage plan
    is applied
    to.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-usageplankey.html
     */
    interface UsagePlanKey {
        Type: 'AWS::ApiGateway::UsagePlanKey';
        Properties: AWS.ApiGateway.UsagePlanKey.Properties;
    }
    /**
     * The AWS::ApiGateway::UsagePlanKey resource associates an Amazon API Gateway API key with
    an API Gateway usage plan. This association determines which users the usage plan
    is applied
    to.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-usageplankey.html
     */
    function UsagePlanKey(props: C.Omit<UsagePlanKey, 'Type'>): UsagePlanKey;
}
export declare namespace AWS.ApiGateway.UsagePlanKey {
    interface Properties {
        /**
         * String

         *
         * The ID of the usage plan key.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-usageplankey.html#cfn-apigateway-usageplankey-keyid
         */
        KeyId: C.YieldsString;
        /**
         * String

         *
         * The type of usage plan key. Currently, the valid key type is
 API_KEY.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-usageplankey.html#cfn-apigateway-usageplankey-keytype
         */
        KeyType: C.YieldsString;
        /**
         * String

         *
         * The
 value of the usage plan key.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-usageplankey.html#cfn-apigateway-usageplankey-usageplanid
         */
        UsagePlanId: C.YieldsString;
    }
}
export declare namespace AWS.ApplicationAutoScaling {
    /**
     * The AWS::ApplicationAutoScaling::ScalableTarget resource specifies a resource that Application Auto Scaling can scale up or down.
    For more information, see the RegisterScalableTarget action in the Application Auto Scaling API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalabletarget.html
     */
    interface ScalableTarget {
        Type: 'AWS::ApplicationAutoScaling::ScalableTarget';
        Properties: AWS.ApplicationAutoScaling.ScalableTarget.Properties;
    }
    /**
     * The AWS::ApplicationAutoScaling::ScalableTarget resource specifies a resource that Application Auto Scaling can scale up or down.
    For more information, see the RegisterScalableTarget action in the Application Auto Scaling API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalabletarget.html
     */
    function ScalableTarget(props: C.Omit<ScalableTarget, 'Type'>): ScalableTarget;
}
export declare namespace AWS.ApplicationAutoScaling.ScalableTarget {
    interface Properties {
        /**
         * Integer

         *
         * The maximum value that Application Auto Scaling can use to scale a target during a
 scaling activity.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalabletarget.html#cfn-applicationautoscaling-scalabletarget-maxcapacity
         */
        MaxCapacity: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The minimum value that Application Auto Scaling can use to scale a target during a
 scaling activity.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalabletarget.html#cfn-applicationautoscaling-scalabletarget-mincapacity
         */
        MinCapacity: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The resource identifier to associate with this scalable target. This string consists
 of the resource type and unique identifier.
 For more information, see the ResourceId parameter for the RegisterScalableTarget
 action in the Application Auto Scaling API Reference, or see the ScalableTarget examples.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalabletarget.html#cfn-applicationautoscaling-scalabletarget-resourceid
         */
        ResourceId: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of an AWS Identity and Access Management (IAM) role
 that allows Application Auto Scaling to modify your scalable target.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalabletarget.html#cfn-applicationautoscaling-scalabletarget-rolearn
         */
        RoleARN: C.YieldsString;
        /**
         * String

         *
         * The scalable dimension associated with the scalable target. Specify the service namespace,
 resource type, and scaling property, such as ecs:service:DesiredCount for the desired task count of an Amazon EC2 Container Service service. For valid
 values, see the ScalableDimension content for the ScalingPolicy data type in the Application Auto Scaling API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalabletarget.html#cfn-applicationautoscaling-scalabletarget-scalabledimension
         */
        ScalableDimension: C.YieldsString;
        /**
         * String

         *
         * The AWS service namespace of the scalable target. For a list of service namespaces,
 see AWS Service Namespaces in the AWS General Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalabletarget.html#cfn-applicationautoscaling-scalabletarget-servicenamespace
         */
        ServiceNamespace: C.YieldsString;
    }
}
export declare namespace AWS.ApplicationAutoScaling {
    /**
     * The AWS::ApplicationAutoScaling::ScalingPolicy resource defines an Application Auto Scaling scaling policy that Application Auto
    Scaling uses to adjust your application resources.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalingpolicy.html
     */
    interface ScalingPolicy {
        Type: 'AWS::ApplicationAutoScaling::ScalingPolicy';
        Properties: AWS.ApplicationAutoScaling.ScalingPolicy.Properties;
    }
    /**
     * The AWS::ApplicationAutoScaling::ScalingPolicy resource defines an Application Auto Scaling scaling policy that Application Auto
    Scaling uses to adjust your application resources.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalingpolicy.html
     */
    function ScalingPolicy(props: C.Omit<ScalingPolicy, 'Type'>): ScalingPolicy;
}
export declare namespace AWS.ApplicationAutoScaling.ScalingPolicy {
    interface Properties {
        /**
         * String

         *
         * A name for the scaling policy.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalingpolicy.html#cfn-applicationautoscaling-scalingpolicy-policyname
         */
        PolicyName: C.YieldsString;
        /**
         * String

         *
         * An Application Auto Scaling policy type.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalingpolicy.html#cfn-applicationautoscaling-scalingpolicy-policytype
         */
        PolicyType: C.YieldsString;
        /**
         * String

         *
         * The unique resource identifier for the scalable target that this scaling policy applies
 to. For more information, see the ResourceId parameter for the PutScalingPolicy action in the Application Auto Scaling API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalingpolicy.html#cfn-applicationautoscaling-scalingpolicy-resourceid
         */
        ResourceId?: C.YieldsString;
        /**
         * String

         *
         * The scalable dimension of the scalable target that this scaling policy applies to.
 The scalable dimension contains the service namespace, resource type, and scaling
 property, such as ecs:service:DesiredCount for the desired task count of an Amazon
 ECS service.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalingpolicy.html#cfn-applicationautoscaling-scalingpolicy-scalabledimension
         */
        ScalableDimension?: C.YieldsString;
        /**
         * String

         *
         * The AWS CloudFormation-generated ID of an Application Auto Scaling scalable target.
 For more information about
 the ID, see the Return Value section of the AWS::ApplicationAutoScaling::ScalableTarget resource.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalingpolicy.html#cfn-applicationautoscaling-scalingpolicy-scalingtargetid
         */
        ScalingTargetId?: C.YieldsString;
        /**
         * String

         *
         * The AWS service namespace of the scalable target that this scaling policy applies
 to. For a list of service namespaces, see AWS Service Namespaces in the AWS General Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalingpolicy.html#cfn-applicationautoscaling-scalingpolicy-servicenamespace
         */
        ServiceNamespace?: C.YieldsString;
        /**
         * Application Auto Scaling ScalingPolicy StepScalingPolicyConfiguration
         *
         * A step policy that configures when Application Auto Scaling scales resources up or
 down, and by how much.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalingpolicy.html#cfn-applicationautoscaling-scalingpolicy-stepscalingpolicyconfiguration
         */
        StepScalingPolicyConfiguration?: C.Yields<AWS.ApplicationAutoScaling.ScalingPolicy.StepScalingPolicyConfiguration>;
        /**
         * Application Auto Scaling ScalingPolicy TargetTrackingScalingPolicyConfiguration
         *
         * Configures
 a target tracking scaling
 policy.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-applicationautoscaling-scalingpolicy.html#cfn-applicationautoscaling-scalingpolicy-targettrackingscalingpolicyconfiguration
         */
        TargetTrackingScalingPolicyConfiguration?: C.Yields<AWS.ApplicationAutoScaling.ScalingPolicy.TargetTrackingScalingPolicyConfiguration>;
    }
}
export declare namespace AWS.ApplicationAutoScaling.ScalingPolicy {
    interface CustomizedMetricSpecification {
        /**
         * List of Application Auto Scaling ScalingPolicy MetricDimension
         *
         * The
 dimensions
 of the metric. Duplicates not allowed.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-applicationautoscaling-scalingpolicy-customizedmetricspecification.html#cfn-applicationautoscaling-scalingpolicy-customizedmetricspecification-dimensions
         */
        Dimensions?: C.Yields<C.CFList<C.Yields<AWS.ApplicationAutoScaling.ScalingPolicy.MetricDimension>>>;
        /**
         * String

         *
         * The name of the metric.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-applicationautoscaling-scalingpolicy-customizedmetricspecification.html#cfn-applicationautoscaling-scalingpolicy-customizedmetricspecification-metricname
         */
        MetricName: C.YieldsString;
        /**
         * String

         *
         * The namespace of the metric.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-applicationautoscaling-scalingpolicy-customizedmetricspecification.html#cfn-applicationautoscaling-scalingpolicy-customizedmetricspecification-namespace
         */
        Namespace: C.YieldsString;
        /**
         * String

         *
         * The
 statistic
 of the metric.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-applicationautoscaling-scalingpolicy-customizedmetricspecification.html#cfn-applicationautoscaling-scalingpolicy-customizedmetricspecification-statistic
         */
        Statistic: C.YieldsString;
        /**
         * String

         *
         * The
 unit
 of the metric.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-applicationautoscaling-scalingpolicy-customizedmetricspecification.html#cfn-applicationautoscaling-scalingpolicy-customizedmetricspecification-unit
         */
        Unit?: C.YieldsString;
    }
}
export declare namespace AWS.ApplicationAutoScaling.ScalingPolicy {
    interface MetricDimension {
        /**
         * String

         *
         * The name of the dimension.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-applicationautoscaling-scalingpolicy-metricdimension.html#cfn-applicationautoscaling-scalingpolicy-metricdimension-name
         */
        Name: C.YieldsString;
        /**
         * String

         *
         * The
 value of the dimension.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-applicationautoscaling-scalingpolicy-metricdimension.html#cfn-applicationautoscaling-scalingpolicy-metricdimension-value
         */
        Value: C.YieldsString;
    }
}
export declare namespace AWS.ApplicationAutoScaling.ScalingPolicy {
    interface PredefinedMetricSpecification {
        /**
         * String

         *
         * The
 metric
 type.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-applicationautoscaling-scalingpolicy-predefinedmetricspecification.html#cfn-applicationautoscaling-scalingpolicy-predefinedmetricspecification-predefinedmetrictype
         */
        PredefinedMetricType: C.YieldsString;
        /**
         * String

         *
         * This
 property is reserved for future use.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-applicationautoscaling-scalingpolicy-predefinedmetricspecification.html#cfn-applicationautoscaling-scalingpolicy-predefinedmetricspecification-resourcelabel
         */
        ResourceLabel?: C.YieldsString;
    }
}
export declare namespace AWS.ApplicationAutoScaling.ScalingPolicy {
    interface StepAdjustment {
        /**
         * Number

         *
         * The lower bound of the breach size. The lower bound is the difference between the
 breach threshold and the aggregated CloudWatch metric value. If the metric value is
 within the lower and upper bounds, Application Auto Scaling triggers this step adjustment.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-applicationautoscaling-scalingpolicy-stepscalingpolicyconfiguration-stepadjustment.html#cfn-applicationautoscaling-scalingpolicy-stepscalingpolicyconfiguration-stepadjustment-metricintervallowerbound
         */
        MetricIntervalLowerBound?: C.Yields<C.CFDouble>;
        /**
         * Number

         *
         * The upper bound of the breach size. The upper bound is the difference between the
 breach threshold and the CloudWatch metric value. If the metric value is within the
 lower and upper bounds, Application Auto Scaling triggers this step adjustment.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-applicationautoscaling-scalingpolicy-stepscalingpolicyconfiguration-stepadjustment.html#cfn-applicationautoscaling-scalingpolicy-stepscalingpolicyconfiguration-stepadjustment-metricintervalupperbound
         */
        MetricIntervalUpperBound?: C.Yields<C.CFDouble>;
        /**
         * Integer

         *
         * The amount by which to scale. The adjustment is based on the value that you specified
 in the AdjustmentType property (either an absolute number or a percentage). A positive value adds to the
 current capacity and a negative number subtracts from the current capacity.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-applicationautoscaling-scalingpolicy-stepscalingpolicyconfiguration-stepadjustment.html#cfn-applicationautoscaling-scalingpolicy-stepscalingpolicyconfiguration-stepadjustment-scalingadjustment
         */
        ScalingAdjustment: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.ApplicationAutoScaling.ScalingPolicy {
    interface StepScalingPolicyConfiguration {
        /**
         * String

         *
         * Specifies whether the ScalingAdjustment value in the StepAdjustment property is an absolute number or a percentage of the current capacity. For valid
 values, see the AdjustmentType content for the StepScalingPolicyConfiguration data type in the Application Auto Scaling API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-applicationautoscaling-scalingpolicy-stepscalingpolicyconfiguration.html#cfn-applicationautoscaling-scalingpolicy-stepscalingpolicyconfiguration-adjustmenttype
         */
        AdjustmentType?: C.YieldsString;
        /**
         * Integer

         *
         * The amount of time, in seconds, after a scaling activity completes before any
 further trigger-related scaling activities can start. For more information, see the
 Cooldown content for the StepScalingPolicyConfiguration data type in the
 Application Auto Scaling API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-applicationautoscaling-scalingpolicy-stepscalingpolicyconfiguration.html#cfn-applicationautoscaling-scalingpolicy-stepscalingpolicyconfiguration-cooldown
         */
        Cooldown?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The aggregation type for the CloudWatch metrics. You can specify Minimum, Maximum, or Average. By default, AWS CloudFormation specifies Average. For more information, see Aggregation in the Amazon CloudWatch User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-applicationautoscaling-scalingpolicy-stepscalingpolicyconfiguration.html#cfn-applicationautoscaling-scalingpolicy-stepscalingpolicyconfiguration-metricaggregationtype
         */
        MetricAggregationType?: C.YieldsString;
        /**
         * Integer

         *
         * The minimum number of resources to adjust when a scaling activity is triggered. If
 you specify PercentChangeInCapacity for the adjustment type, the scaling policy scales the target by this amount.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-applicationautoscaling-scalingpolicy-stepscalingpolicyconfiguration.html#cfn-applicationautoscaling-scalingpolicy-stepscalingpolicyconfiguration-minadjustmentmagnitude
         */
        MinAdjustmentMagnitude?: C.Yields<C.CFInteger>;
        /**
         * List of Application Auto Scaling ScalingPolicy StepScalingPolicyConfiguration StepAdjustment
         *
         * A set of adjustments that enable you to scale based on the size of the alarm breach.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-applicationautoscaling-scalingpolicy-stepscalingpolicyconfiguration.html#cfn-applicationautoscaling-scalingpolicy-stepscalingpolicyconfiguration-stepadjustments
         */
        StepAdjustments?: C.Yields<C.CFList<C.Yields<AWS.ApplicationAutoScaling.ScalingPolicy.StepAdjustment>>>;
    }
}
export declare namespace AWS.ApplicationAutoScaling.ScalingPolicy {
    interface TargetTrackingScalingPolicyConfiguration {
        /**
         * Application Auto Scaling ScalingPolicy CustomizedMetricSpecification
         *
         * This
 property is reserved for future use.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-applicationautoscaling-scalingpolicy-targettrackingscalingpolicyconfiguration.html#cfn-applicationautoscaling-scalingpolicy-targettrackingscalingpolicyconfiguration-customizedmetricspecification
         */
        CustomizedMetricSpecification?: C.Yields<AWS.ApplicationAutoScaling.ScalingPolicy.CustomizedMetricSpecification>;
        /**
         * Application Auto Scaling ScalingPolicy PredefinedMetricSpecification
         *
         * A predefined metric.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-applicationautoscaling-scalingpolicy-targettrackingscalingpolicyconfiguration.html#cfn-applicationautoscaling-scalingpolicy-targettrackingscalingpolicyconfiguration-predefinedmetricspecification
         */
        PredefinedMetricSpecification?: C.Yields<AWS.ApplicationAutoScaling.ScalingPolicy.PredefinedMetricSpecification>;
        /**
         * Integer

         *
         * The amount of time, in seconds, after a
 scale
 in activity completes before another scale in activity
 can start.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-applicationautoscaling-scalingpolicy-targettrackingscalingpolicyconfiguration.html#cfn-applicationautoscaling-scalingpolicy-targettrackingscalingpolicyconfiguration-scaleincooldown
         */
        ScaleInCooldown?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The amount of time, in seconds, after a
 scale
 out activity completes before another scale out activity
 can start.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-applicationautoscaling-scalingpolicy-targettrackingscalingpolicyconfiguration.html#cfn-applicationautoscaling-scalingpolicy-targettrackingscalingpolicyconfiguration-scaleoutcooldown
         */
        ScaleOutCooldown?: C.Yields<C.CFInteger>;
        /**
         * Double

         *
         * The target value for the metric.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-applicationautoscaling-scalingpolicy-targettrackingscalingpolicyconfiguration.html#cfn-applicationautoscaling-scalingpolicy-targettrackingscalingpolicyconfiguration-targetvalue
         */
        TargetValue: C.Yields<C.CFDouble>;
    }
}
export declare namespace AWS.AutoScaling {
    /**
     * The AWS::AutoScaling::AutoScalingGroup type creates an Auto Scaling group.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-group.html
     */
    interface AutoScalingGroup {
        Type: 'AWS::AutoScaling::AutoScalingGroup';
        Properties: AWS.AutoScaling.AutoScalingGroup.Properties;
    }
    /**
     * The AWS::AutoScaling::AutoScalingGroup type creates an Auto Scaling group.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-group.html
     */
    function AutoScalingGroup(props: C.Omit<AutoScalingGroup, 'Type'>): AutoScalingGroup;
}
export declare namespace AWS.AutoScaling.AutoScalingGroup {
    interface Properties {
        /**
         * List of strings

         *
         * Contains a list of availability zones for the group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-group.html#cfn-as-group-availabilityzones
         */
        AvailabilityZones?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The number of seconds after a scaling activity is completed before any further
 scaling activities can start.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-group.html#cfn-as-group-cooldown
         */
        Cooldown?: C.YieldsString;
        /**
         * String

         *
         * Specifies the desired capacity for the Auto Scaling group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-group.html#cfn-as-group-desiredcapacity
         */
        DesiredCapacity?: C.YieldsString;
        /**
         * Integer

         *
         * The length of time in seconds after a new EC2 instance comes into service that
 Auto Scaling starts checking its health.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-group.html#cfn-as-group-healthcheckgraceperiod
         */
        HealthCheckGracePeriod?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The service you want the health status from, Amazon EC2 or Elastic Load
 Balancer. Valid values are EC2 or ELB.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-group.html#cfn-as-group-healthchecktype
         */
        HealthCheckType?: C.YieldsString;
        /**
         * String

         *
         * The ID of the Amazon EC2 instance you want to use to create the Auto Scaling group.
 Use this
 property if you want to create an Auto Scaling group that uses an existing Amazon
 EC2 instance
 instead of a launch configuration.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-group.html#cfn-as-group-instanceid
         */
        InstanceId?: C.YieldsString;
        /**
         * String

         *
         * Specifies the name of the associated AWS::AutoScaling::LaunchConfiguration.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-group.html#cfn-as-group-launchconfigurationname
         */
        LaunchConfigurationName?: C.YieldsString;
        /**
         * List of strings

         *
         * A list of Classic load balancers associated with this Auto Scaling group. To specify
 Application load balancers, use TargetGroupARNs.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-group.html#cfn-as-group-loadbalancernames
         */
        LoadBalancerNames?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The maximum size of the Auto Scaling group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-group.html#cfn-as-group-maxsize
         */
        MaxSize: C.YieldsString;
        /**
         * A list of  Auto Scaling MetricsCollection
         *
         * Enables the monitoring of group metrics of an Auto Scaling group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-group.html#cfn-as-group-metricscollection
         */
        MetricsCollection?: C.Yields<C.CFList<C.Yields<AWS.AutoScaling.AutoScalingGroup.MetricsCollection>>>;
        /**
         * String

         *
         * The minimum size of the Auto Scaling group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-group.html#cfn-as-group-minsize
         */
        MinSize: C.YieldsString;
        /**
         * List of Auto Scaling
    NotificationConfigurations
         *
         * An embedded property that configures an Auto Scaling group to send notifications when
 specified events take place.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-group.html#cfn-as-group-notificationconfigurations
         */
        NotificationConfigurations?: C.Yields<C.CFList<C.Yields<AWS.AutoScaling.AutoScalingGroup.NotificationConfiguration>>>;
        /**
         * String

         *
         * The name of an existing cluster placement group into which you want to launch
 your instances. A placement group is a logical grouping of instances within a
 single Availability Zone. You cannot specify multiple Availability Zones and a
 placement group.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-group.html#cfn-as-group-placementgroup
         */
        PlacementGroup?: C.YieldsString;
        /**
         * List of Auto Scaling
    Tags
         *
         * The tags you want to attach to this resource.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-group.html#cfn-as-group-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<AWS.AutoScaling.AutoScalingGroup.TagProperty>>>;
        /**
         * List of strings

         *
         * A list of Amazon Resource Names (ARN) of target groups to associate with the Auto
 Scaling group.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-group.html#cfn-as-group-targetgrouparns
         */
        TargetGroupARNs?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of strings

         *
         * A policy or a list of policies that are used to select the instances to
 terminate. The policies are executed in the order that you list them.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-group.html#cfn-as-group-termpolicy
         */
        TerminationPolicies?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of strings

         *
         * A list of subnet identifiers of Amazon Virtual Private Cloud (Amazon VPCs).
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-group.html#cfn-as-group-vpczoneidentifier
         */
        VPCZoneIdentifier?: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.AutoScaling.AutoScalingGroup {
    interface MetricsCollection {
        /**
         * String

         *
         * The frequency at which Auto Scaling sends aggregated data to CloudWatch. For example,
 you can
 specify 1Minute to send aggregated data to CloudWatch every minute.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-metricscollection.html#cfn-as-metricscollection-granularity
         */
        Granularity: C.YieldsString;
        /**
         * List of strings

         *
         * The list of metrics to collect. If you don't specify any metrics, all metrics
 are enabled.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-metricscollection.html#cfn-as-metricscollection-metrics
         */
        Metrics?: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.AutoScaling.AutoScalingGroup {
    interface NotificationConfiguration {
        /**
         * List of strings

         *
         * A list of event types that trigger a notification. Event types can include any
 of the following types: autoscaling:EC2_INSTANCE_LAUNCH,
 autoscaling:EC2_INSTANCE_LAUNCH_ERROR,
 autoscaling:EC2_INSTANCE_TERMINATE,
 autoscaling:EC2_INSTANCE_TERMINATE_ERROR, and
 autoscaling:TEST_NOTIFICATION. For more information about event
 types, see DescribeAutoScalingNotificationTypes in the
 Auto Scaling API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-notificationconfigurations.html#cfn-as-group-notificationconfigurations-notificationtypes
         */
        NotificationTypes?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the Amazon Simple Notification Service (SNS)
 topic.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-notificationconfigurations.html#cfn-autoscaling-autoscalinggroup-notificationconfigurations-topicarn
         */
        TopicARN: C.YieldsString;
    }
}
export declare namespace AWS.AutoScaling.AutoScalingGroup {
    interface TagProperty {
        /**
         * String

         *
         * The key name of the tag.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-tags.html#cfn-as-tags-Key
         */
        Key: C.YieldsString;
        /**
         * Boolean

         *
         * Set to true if you want AWS CloudFormation to copy the tag to EC2 instances that are launched
 as
 part of the auto scaling group. Set to false if you want the tag attached only to the
 auto scaling group and not copied to any instances launched as part of the auto scaling
 group.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-tags.html#cfn-as-tags-PropagateAtLaunch
         */
        PropagateAtLaunch: C.YieldsBoolean;
        /**
         * String

         *
         * The value for the tag.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-tags.html#cfn-as-tags-Value
         */
        Value: C.YieldsString;
    }
}
export declare namespace AWS.AutoScaling {
    /**
     * The AWS::AutoScaling::LaunchConfiguration type creates an Auto Scaling launch
    configuration that can be used by an Auto Scaling group to configure Amazon EC2 instances
    in the Auto Scaling
    group.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig.html
     */
    interface LaunchConfiguration {
        Type: 'AWS::AutoScaling::LaunchConfiguration';
        Properties: AWS.AutoScaling.LaunchConfiguration.Properties;
    }
    /**
     * The AWS::AutoScaling::LaunchConfiguration type creates an Auto Scaling launch
    configuration that can be used by an Auto Scaling group to configure Amazon EC2 instances
    in the Auto Scaling
    group.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig.html
     */
    function LaunchConfiguration(props: C.Omit<LaunchConfiguration, 'Type'>): LaunchConfiguration;
}
export declare namespace AWS.AutoScaling.LaunchConfiguration {
    interface Properties {
        /**
         * Boolean

         *
         * For Amazon EC2 instances in a VPC, indicates whether instances in the Auto Scaling
 group
 receive public IP addresses. If you specify true, each instance in
 the Auto Scaling receives a unique public IP address.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig.html#cf-as-launchconfig-associatepubip
         */
        AssociatePublicIpAddress?: C.YieldsBoolean;
        /**
         * A list of BlockDeviceMappings.

         *
         * Specifies how block devices are exposed to the instance. You can specify
 virtual devices and EBS volumes.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig.html#cfn-as-launchconfig-blockdevicemappings
         */
        BlockDeviceMappings?: C.Yields<C.CFList<C.Yields<AWS.AutoScaling.LaunchConfiguration.BlockDeviceMapping>>>;
        /**
         * String

         *
         * The ID of a ClassicLink-enabled VPC to link your EC2-Classic instances to. You
 can specify this property only for EC2-Classic instances. For more information,
 see ClassicLink in the
 Amazon Elastic Compute Cloud User
    Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig.html#cfn-as-launchconfig-classiclinkvpcid
         */
        ClassicLinkVPCId?: C.YieldsString;
        /**
         * List of strings

         *
         * The IDs of one or more security groups for the VPC that you specified in the
 ClassicLinkVPCId property.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig.html#cfn-as-launchconfig-classiclinkvpcsecuritygroups
         */
        ClassicLinkVPCSecurityGroups?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * Boolean

         *
         * Specifies whether the launch configuration is optimized for EBS I/O. This
 optimization provides dedicated throughput to Amazon EBS and an optimized
 configuration stack to provide optimal EBS I/O performance.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig.html#cfn-as-launchconfig-ebsoptimized
         */
        EbsOptimized?: C.YieldsBoolean;
        /**
         * String (1–1600 chars)

         *
         * Provides the name or the Amazon Resource Name (ARN) of the instance profile
 associated with the IAM role for the instance. The instance profile contains the
 IAM role.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig.html#cfn-as-launchconfig-iaminstanceprofile
         */
        IamInstanceProfile?: C.YieldsString;
        /**
         * String

         *
         * Provides the unique ID of the Amazon Machine Image (AMI) that was assigned
 during registration.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig.html#cfn-as-launchconfig-imageid
         */
        ImageId: C.YieldsString;
        /**
         * String

         *
         * The ID of the Amazon EC2 instance you want to use to create the launch
 configuration. Use this property if you want the launch configuration to use
 settings from an existing Amazon EC2 instance.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig.html#cfn-as-launchconfig-instanceid
         */
        InstanceId?: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether detailed instance monitoring is enabled for the Auto Scaling group.
 By default, this property is set to true (enabled).

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig.html#cfn-as-launchconfig-instancemonitoring
         */
        InstanceMonitoring?: C.YieldsBoolean;
        /**
         * String

         *
         * Specifies the instance type of the EC2 instance.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig.html#cfn-as-launchconfig-instancetype
         */
        InstanceType: C.YieldsString;
        /**
         * String

         *
         * Provides the ID of the kernel associated with the EC2 AMI.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig.html#cfn-as-launchconfig-kernelid
         */
        KernelId?: C.YieldsString;
        /**
         * String

         *
         * Provides the name of the EC2 key pair.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig.html#cfn-as-launchconfig-keyname
         */
        KeyName?: C.YieldsString;
        /**
         * String

         *
         * The tenancy of the instance. An instance with a tenancy of
 dedicated runs on single-tenant hardware and can only be launched
 in a VPC. You must set the value of this parameter to dedicated if
 want to launch dedicated instances in a shared tenancy VPC (a VPC with the
 instance placement tenancy attribute set to default). For more information, see
 CreateLaunchConfiguration in the
 Auto Scaling API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig.html#cfn-as-launchconfig-placementtenancy
         */
        PlacementTenancy?: C.YieldsString;
        /**
         * String

         *
         * The ID of the RAM disk to select. Some kernels require additional drivers at
 launch. Check the kernel requirements for information about whether you need to
 specify a RAM disk. To find kernel requirements, refer to the AWS Resource Center
 and search for the kernel ID.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig.html#cfn-as-launchconfig-ramdiskid
         */
        RamDiskId?: C.YieldsString;
        /**
         * A list of EC2 security groups.

         *
         * A list that contains the EC2 security groups to assign to the Amazon EC2
 instances in the Auto Scaling group. The list can contain the name of existing EC2
 security groups or references to AWS::EC2::SecurityGroup resources
 created in the template. If your instances are launched within VPC, specify Amazon
 VPC
 security group IDs.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig.html#cfn-as-launchconfig-securitygroups
         */
        SecurityGroups?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The spot price for this autoscaling group. If a spot price is set, then the
 autoscaling group will launch when the current spot price is less than the amount
 specified in the template.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig.html#cfn-as-launchconfig-spotprice
         */
        SpotPrice?: C.YieldsString;
        /**
         * String

         *
         * The user data available to the launched EC2 instances.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig.html#cfn-as-launchconfig-userdata
         */
        UserData?: C.YieldsString;
    }
}
export declare namespace AWS.AutoScaling.LaunchConfiguration {
    interface BlockDevice {
        /**
         * Boolean

         *
         * Indicates whether to delete the volume when the instance is terminated. By
 default, Auto Scaling uses true.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig-blockdev-template.html#cfn-as-launchconfig-blockdev-template-deleteonterm
         */
        DeleteOnTermination?: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Indicates whether the volume is encrypted. Encrypted EBS volumes must be
 attached to instances that support Amazon EBS encryption. Volumes that you create
 from
 encrypted snapshots are automatically encrypted. You cannot create an encrypted
 volume from an unencrypted snapshot or an unencrypted volume from an encrypted
 snapshot.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig-blockdev-template.html#cfn-as-launchconfig-blockdev-template-encrypted
         */
        Encrypted?: C.YieldsBoolean;
        /**
         * Integer.

         *
         * The number of I/O operations per second (IOPS) that the volume supports. The
 maximum ratio of IOPS to volume size is 30.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig-blockdev-template.html#cfn-as-launchconfig-blockdev-template-iops
         */
        Iops?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The snapshot ID of the volume to use.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig-blockdev-template.html#cfn-as-launchconfig-blockdev-template-snapshotid
         */
        SnapshotId?: C.YieldsString;
        /**
         * Integer.

         *
         * The volume size, in Gibibytes (GiB). This can be a number from 1 – 1024.
 If the volume type is EBS optimized, the minimum value is 10. For more information
 about specifying the volume type, see EbsOptimized in AWS::AutoScaling::LaunchConfiguration.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig-blockdev-template.html#cfn-as-launchconfig-blockdev-template-volumesize
         */
        VolumeSize?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The volume type. By default, Auto Scaling uses the standard volume type.
 For more information, see Ebs in the Auto Scaling API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig-blockdev-template.html#cfn-as-launchconfig-blockdev-template-volumetype
         */
        VolumeType?: C.YieldsString;
    }
}
export declare namespace AWS.AutoScaling.LaunchConfiguration {
    interface BlockDeviceMapping {
        /**
         * String

         *
         * The name of the device within Amazon EC2.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig-blockdev-mapping.html#cfn-as-launchconfig-blockdev-mapping-devicename
         */
        DeviceName: C.YieldsString;
        /**
         * AutoScaling EBS Block Device.

         *
         * The Amazon Elastic Block Store volume information.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig-blockdev-mapping.html#cfn-as-launchconfig-blockdev-mapping-ebs
         */
        Ebs?: C.Yields<AWS.AutoScaling.LaunchConfiguration.BlockDevice>;
        /**
         * Boolean

         *
         * Suppresses the device mapping. If NoDevice is set to true for the
 root device, the instance might fail the Amazon EC2 health check. Auto Scaling launches
 a
 replacement instance if the instance fails the health check.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig-blockdev-mapping.html#cfn-as-launchconfig-blockdev-mapping-nodevice
         */
        NoDevice?: C.YieldsBoolean;
        /**
         * String

         *
         * The name of the virtual device. The name must be in the form
 ephemeralX
     where X is a number starting from zero (0), for
 example, ephemeral0.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig-blockdev-mapping.html#cfn-as-launchconfig-blockdev-mapping-virtualname
         */
        VirtualName?: C.YieldsString;
    }
}
export declare namespace AWS.AutoScaling {
    /**
     * Use AWS::AutoScaling::LifecycleHook to control the state of an instance in an Auto Scaling group after it is launched
    or terminated. When you use a lifecycle hook, the Auto Scaling group either pauses
    the instance after it is launched (before it is put into service) or pauses the instance
    as it is terminated (before it is fully terminated). For more information, see Examples of How to Use Lifecycle Hooks in the
    Auto Scaling User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-as-lifecyclehook.html
     */
    interface LifecycleHook {
        Type: 'AWS::AutoScaling::LifecycleHook';
        Properties: AWS.AutoScaling.LifecycleHook.Properties;
    }
    /**
     * Use AWS::AutoScaling::LifecycleHook to control the state of an instance in an Auto Scaling group after it is launched
    or terminated. When you use a lifecycle hook, the Auto Scaling group either pauses
    the instance after it is launched (before it is put into service) or pauses the instance
    as it is terminated (before it is fully terminated). For more information, see Examples of How to Use Lifecycle Hooks in the
    Auto Scaling User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-as-lifecyclehook.html
     */
    function LifecycleHook(props: C.Omit<LifecycleHook, 'Type'>): LifecycleHook;
}
export declare namespace AWS.AutoScaling.LifecycleHook {
    interface Properties {
        /**
         * String

         *
         * The name of the Auto Scaling group for the lifecycle hook.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-as-lifecyclehook.html#cfn-as-lifecyclehook-autoscalinggroupname
         */
        AutoScalingGroupName: C.YieldsString;
        /**
         * String

         *
         * The action the Auto Scaling group takes when the lifecycle hook timeout elapses or
 if
 an unexpected failure occurs.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-as-lifecyclehook.html#cfn-as-lifecyclehook-defaultresult
         */
        DefaultResult?: C.YieldsString;
        /**
         * Integer

         *
         * The amount of time that can elapse before the lifecycle hook times out. When
 the lifecycle hook times out, Auto Scaling performs the action that you specified
 in the
 DefaultResult property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-as-lifecyclehook.html#cfn-as-lifecyclehook-heartbeattimeout
         */
        HeartbeatTimeout?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The state of the Amazon EC2 instance to which you want to attach the lifecycle hook.
 For valid values, see the LifecycleTransition content for the LifecycleHook data type in the
 Auto Scaling API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-as-lifecyclehook.html#cfn-as-lifecyclehook-lifecycletransition
         */
        LifecycleTransition: C.YieldsString;
        /**
         * String

         *
         * Additional information that you want to include when Auto Scaling sends a message
 to
 the notification target.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-as-lifecyclehook.html#cfn-as-lifecyclehook-notificationmetadata
         */
        NotificationMetadata?: C.YieldsString;
        /**
         * String

         *
         * The Amazon resource name (ARN) of the notification target that Auto Scaling uses to
 notify you when an instance is in the transition state for the lifecycle hook. You
 can specify an Amazon SQS queue or an Amazon SNS topic. The notification message includes
 the following information: lifecycle action token, user account ID, Auto Scaling group
 name, lifecycle hook name, instance ID, lifecycle transition, and notification
 metadata.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-as-lifecyclehook.html#cfn-as-lifecyclehook-notificationtargetarn
         */
        NotificationTargetARN?: C.YieldsString;
        /**
         * String

         *
         * The ARN of the IAM role that allows the Auto Scaling group to publish to the
 specified notification target. The role requires permissions to Amazon SNS and
 Amazon SQS.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-as-lifecyclehook.html#cfn-as-lifecyclehook-rolearn
         */
        RoleARN?: C.YieldsString;
    }
}
export declare namespace AWS.AutoScaling {
    /**
     * The AWS::AutoScaling::ScalingPolicy resource adds a scaling policy to an auto scaling group. A
    scaling policy specifies whether to scale the auto scaling group up or down, and by
    how much. For more information
    on scaling policies, see Scaling by Policy in the Auto Scaling Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-policy.html
     */
    interface ScalingPolicy {
        Type: 'AWS::AutoScaling::ScalingPolicy';
        Properties: AWS.AutoScaling.ScalingPolicy.Properties;
    }
    /**
     * The AWS::AutoScaling::ScalingPolicy resource adds a scaling policy to an auto scaling group. A
    scaling policy specifies whether to scale the auto scaling group up or down, and by
    how much. For more information
    on scaling policies, see Scaling by Policy in the Auto Scaling Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-policy.html
     */
    function ScalingPolicy(props: C.Omit<ScalingPolicy, 'Type'>): ScalingPolicy;
}
export declare namespace AWS.AutoScaling.ScalingPolicy {
    interface Properties {
        /**
         * String

         *
         * Specifies whether the ScalingAdjustment is an
 absolute number or a percentage of the current capacity. Valid values are
 ChangeInCapacity, ExactCapacity,
 and PercentChangeInCapacity.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-policy.html#cfn-as-scalingpolicy-adjustmenttype
         */
        AdjustmentType?: C.YieldsString;
        /**
         * String

         *
         * The name or Amazon Resource Name (ARN) of the Auto Scaling Group that you
 want to attach the policy to.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-policy.html#cfn-as-scalingpolicy-autoscalinggroupname
         */
        AutoScalingGroupName: C.YieldsString;
        /**
         * String

         *
         * The amount of time, in seconds, after a scaling activity completes before
 any further trigger-related scaling activities can start.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-policy.html#cfn-as-scalingpolicy-cooldown
         */
        Cooldown?: C.YieldsString;
        /**
         * Integer

         *
         * The estimated time, in seconds, until a newly launched instance can send
 metrics to CloudWatch. By default, Auto Scaling uses the cooldown period, as specified
 in the
 Cooldown property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-policy.html#cfn-as-scalingpolicy-estimatedinstancewarmup
         */
        EstimatedInstanceWarmup?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The aggregation type for the CloudWatch metrics. You can specify
 Minimum, Maximum, or Average. By default,
 AWS CloudFormation specifies Average.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-policy.html#cfn-as-scalingpolicy-metricaggregationtype
         */
        MetricAggregationType?: C.YieldsString;
        /**
         * Integer

         *
         * For the PercentChangeInCapacity adjustment type, the minimum
 number of instances to scale. The scaling policy changes the desired capacity of
 the Auto Scaling group by a minimum of this many instances. This property replaces
 the
 MinAdjustmentStep property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-policy.html#cfn-as-scalingpolicy-minadjustmentmagnitude
         */
        MinAdjustmentMagnitude?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * An Auto Scaling policy type. You can specify SimpleScaling,
 StepScaling, or TargetTrackingScaling. By default, AWS CloudFormation specifies
 SimpleScaling. For more information, see Scaling Policy
    Types in the Auto Scaling User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-policy.html#cfn-as-scalingpolicy-policytype
         */
        PolicyType?: C.YieldsString;
        /**
         * Integer

         *
         * The number of instances by which to scale. The AdjustmentType property determines if AWS CloudFormation interprets this number as an absolute number
 (when the ExactCapacity value is specified), increase or decrease capacity by a specified number (when the
 ChangeInCapacity value is specified), or increase or decrease capacity as a percentage of the existing
 Auto Scaling group size (when the PercentChangeInCapacity value is
 specified). A positive value adds to the current capacity and a negative value subtracts
 from the current capacity. For exact capacity, you must specify a positive value.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-policy.html#cfn-as-scalingpolicy-scalingadjustment
         */
        ScalingAdjustment?: C.Yields<C.CFInteger>;
        /**
         * List of Auto Scaling ScalingPolicy
    StepAdjustments
         *
         * A set of adjustments that enable you to scale based on the size of the alarm
 breach.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-policy.html#cfn-as-scalingpolicy-stepadjustments
         */
        StepAdjustments?: C.Yields<C.CFList<C.Yields<AWS.AutoScaling.ScalingPolicy.StepAdjustment>>>;
        /**
         * Auto Scaling ScalingPolicy
    TargetTrackingConfiguration
         *
         * Configures a target tracking scaling policy.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-policy.html#cfn-autoscaling-scalingpolicy-targettrackingconfiguration
         */
        TargetTrackingConfiguration?: C.Yields<AWS.AutoScaling.ScalingPolicy.TargetTrackingConfiguration>;
    }
}
export declare namespace AWS.AutoScaling.ScalingPolicy {
    interface CustomizedMetricSpecification {
        /**
         * List of Auto Scaling ScalingPolicy MetricDimension
         *
         * The dimensions of the metric. Duplicates not allowed.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-autoscaling-scalingpolicy-customizedmetricspecification.html#cfn-autoscaling-scalingpolicy-customizedmetricspecification-dimensions
         */
        Dimensions?: C.Yields<C.CFList<C.Yields<AWS.AutoScaling.ScalingPolicy.MetricDimension>>>;
        /**
         * String

         *
         * The name of the metric.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-autoscaling-scalingpolicy-customizedmetricspecification.html#cfn-autoscaling-scalingpolicy-customizedmetricspecification-metricname
         */
        MetricName: C.YieldsString;
        /**
         * String

         *
         * The namespace of the metric.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-autoscaling-scalingpolicy-customizedmetricspecification.html#cfn-autoscaling-scalingpolicy-customizedmetricspecification-namespace
         */
        Namespace: C.YieldsString;
        /**
         * String

         *
         * The statistic of the metric.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-autoscaling-scalingpolicy-customizedmetricspecification.html#cfn-autoscaling-scalingpolicy-customizedmetricspecification-statistic
         */
        Statistic: C.YieldsString;
        /**
         * String

         *
         * The unit of the metric.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-autoscaling-scalingpolicy-customizedmetricspecification.html#cfn-autoscaling-scalingpolicy-customizedmetricspecification-unit
         */
        Unit?: C.YieldsString;
    }
}
export declare namespace AWS.AutoScaling.ScalingPolicy {
    interface MetricDimension {
        /**
         * String

         *
         * The name of the dimension.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-autoscaling-scalingpolicy-metricdimension.html#cfn-autoscaling-scalingpolicy-metricdimension-name
         */
        Name: C.YieldsString;
        /**
         * String

         *
         * The value of the dimension.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-autoscaling-scalingpolicy-metricdimension.html#cfn-autoscaling-scalingpolicy-metricdimension-value
         */
        Value: C.YieldsString;
    }
}
export declare namespace AWS.AutoScaling.ScalingPolicy {
    interface PredefinedMetricSpecification {
        /**
         * String

         *
         * The metric type.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-autoscaling-scalingpolicy-predefinedmetricspecification.html#cfn-autoscaling-scalingpolicy-predefinedmetricspecification-predefinedmetrictype
         */
        PredefinedMetricType: C.YieldsString;
        /**
         * String

         *
         * Identifies the resource associated with the metric type.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-autoscaling-scalingpolicy-predefinedmetricspecification.html#cfn-autoscaling-scalingpolicy-predefinedmetricspecification-resourcelabel
         */
        ResourceLabel?: C.YieldsString;
    }
}
export declare namespace AWS.AutoScaling.ScalingPolicy {
    interface StepAdjustment {
        /**
         * Number

         *
         * The lower bound of the breach size. The lower bound is the difference between the
 breach threshold and the aggregated CloudWatch metric value. If the metric value is
 within the lower and upper bounds, Auto Scaling triggers this step adjustment.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-autoscaling-scalingpolicy-stepadjustments.html#cfn-autoscaling-scalingpolicy-stepadjustment-metricintervallowerbound
         */
        MetricIntervalLowerBound?: C.Yields<C.CFDouble>;
        /**
         * Number

         *
         * The upper bound of the breach size. The upper bound is the difference between the
 breach threshold and the CloudWatch metric value. If the metric value is within the
 lower and upper bounds, Auto Scaling triggers this step adjustment.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-autoscaling-scalingpolicy-stepadjustments.html#cfn-autoscaling-scalingpolicy-stepadjustment-metricintervalupperbound
         */
        MetricIntervalUpperBound?: C.Yields<C.CFDouble>;
        /**
         * Integer

         *
         * The amount by which to scale. The adjustment is based on the value that you specified
 in the AdjustmentType property (either an absolute number or a percentage). A positive value adds to the
 current capacity and a negative number subtracts from the current capacity.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-autoscaling-scalingpolicy-stepadjustments.html#cfn-autoscaling-scalingpolicy-stepadjustment-scalingadjustment
         */
        ScalingAdjustment: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.AutoScaling.ScalingPolicy {
    interface TargetTrackingConfiguration {
        /**
         * Auto Scaling ScalingPolicy CustomizedMetricSpecification
         *
         * A customized metric.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-autoscaling-scalingpolicy-targettrackingconfiguration.html#cfn-autoscaling-scalingpolicy-targettrackingconfiguration-customizedmetricspecification
         */
        CustomizedMetricSpecification?: C.Yields<AWS.AutoScaling.ScalingPolicy.CustomizedMetricSpecification>;
        /**
         * Boolean

         *
         * Indicates whether to disable scale-in for the target tracking policy. If true,
 the target tracking policy will not scale in the Auto Scaling group. The default value
 is false.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-autoscaling-scalingpolicy-targettrackingconfiguration.html#cfn-autoscaling-scalingpolicy-targettrackingconfiguration-disablescalein
         */
        DisableScaleIn?: C.YieldsBoolean;
        /**
         * Auto Scaling ScalingPolicy PredefinedMetricSpecification
         *
         * A predefined metric.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-autoscaling-scalingpolicy-targettrackingconfiguration.html#cfn-autoscaling-scalingpolicy-targettrackingconfiguration-predefinedmetricspecification
         */
        PredefinedMetricSpecification?: C.Yields<AWS.AutoScaling.ScalingPolicy.PredefinedMetricSpecification>;
        /**
         * Double

         *
         * The target value for the metric.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-autoscaling-scalingpolicy-targettrackingconfiguration.html#cfn-autoscaling-scalingpolicy-targettrackingconfiguration-targetvalue
         */
        TargetValue: C.Yields<C.CFDouble>;
    }
}
export declare namespace AWS.AutoScaling {
    /**
     * Creates a scheduled scaling action for an Auto Scaling group, changing the number
    of servers
    available for your application in response to predictable load changes.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-as-scheduledaction.html
     */
    interface ScheduledAction {
        Type: 'AWS::AutoScaling::ScheduledAction';
        Properties: AWS.AutoScaling.ScheduledAction.Properties;
    }
    /**
     * Creates a scheduled scaling action for an Auto Scaling group, changing the number
    of servers
    available for your application in response to predictable load changes.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-as-scheduledaction.html
     */
    function ScheduledAction(props: C.Omit<ScheduledAction, 'Type'>): ScheduledAction;
}
export declare namespace AWS.AutoScaling.ScheduledAction {
    interface Properties {
        /**
         * String

         *
         * The name or ARN of the Auto Scaling group.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-as-scheduledaction.html#cfn-as-scheduledaction-asgname
         */
        AutoScalingGroupName: C.YieldsString;
        /**
         * Integer

         *
         * The number of Amazon EC2 instances that should be running in the
 Auto Scaling group. At least one of MaxSize, MinSize, or
 DesiredCapacity must be specified.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-as-scheduledaction.html#cfn-as-scheduledaction-desiredcapacity
         */
        DesiredCapacity?: C.Yields<C.CFInteger>;
        /**
         * Time stamp

         *
         * The time in UTC for this schedule to end. For example,
 2010-06-01T00:00:00Z.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-as-scheduledaction.html#cfn-as-scheduledaction-endtime
         */
        EndTime?: C.YieldsString;
        /**
         * Integer

         *
         * The maximum number of Amazon EC2 instances in the Auto Scaling group. At least one
 of MaxSize, MinSize, or
 DesiredCapacity must be specified.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-as-scheduledaction.html#cfn-as-scheduledaction-maxsize
         */
        MaxSize?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The minimum number of Amazon EC2 instances in the Auto Scaling group. At least one
 of MaxSize, MinSize, or
 DesiredCapacity must be specified.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-as-scheduledaction.html#cfn-as-scheduledaction-minsize
         */
        MinSize?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The time in UTC when recurring future actions will start. You specify the start
 time by following the Unix cron syntax format. For more information about cron
 syntax, go to http://en.wikipedia.org/wiki/Cron.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-as-scheduledaction.html#cfn-as-scheduledaction-recurrence
         */
        Recurrence?: C.YieldsString;
        /**
         * Time stamp

         *
         * The time in UTC for this schedule to start. For example,
 2010-06-01T00:00:00Z.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-as-scheduledaction.html#cfn-as-scheduledaction-starttime
         */
        StartTime?: C.YieldsString;
    }
}
export declare namespace AWS.Batch {
    /**
     * The AWS::Batch::ComputeEnvironment resource to define your AWS Batch compute
    environment. For more information, see Compute Environments in
    the AWS Batch User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-computeenvironment.html
     */
    interface ComputeEnvironment {
        Type: 'AWS::Batch::ComputeEnvironment';
        Properties: AWS.Batch.ComputeEnvironment.Properties;
    }
    /**
     * The AWS::Batch::ComputeEnvironment resource to define your AWS Batch compute
    environment. For more information, see Compute Environments in
    the AWS Batch User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-computeenvironment.html
     */
    function ComputeEnvironment(props: C.Omit<ComputeEnvironment, 'Type'>): ComputeEnvironment;
}
export declare namespace AWS.Batch.ComputeEnvironment {
    interface Properties {
        /**
         * String

         *
         * The type of the compute environment.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-computeenvironment.html#cfn-batch-computeenvironment-type
         */
        Type: C.YieldsString;
        /**
         * String

         *
         * The service role associated with the compute environment that allows AWS Batch to
 make
 calls to AWS API operations on your behalf.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-computeenvironment.html#cfn-batch-computeenvironment-servicerole
         */
        ServiceRole: C.YieldsString;
        /**
         * String

         *
         * The name of the compute environment.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-computeenvironment.html#cfn-batch-computeenvironment-computeenvironmentname
         */
        ComputeEnvironmentName?: C.YieldsString;
        /**
         *                             Type: AWS Batch
    ComputeEnvironment ComputeResources
         *
         * The compute resources defined for the compute environment.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-computeenvironment.html#cfn-batch-computeenvironment-computeresources
         */
        ComputeResources: C.Yields<AWS.Batch.ComputeEnvironment.ComputeResources>;
        /**
         * String

         *
         * The state of the compute environment. The valid values are ENABLED or
 DISABLED. An ENABLED state indicates that you can register
 instances with the compute environment and that the associated instances can accept
 jobs.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-computeenvironment.html#cfn-batch-computeenvironment-state
         */
        State?: C.YieldsString;
    }
}
export declare namespace AWS.Batch.ComputeEnvironment {
    interface ComputeResources {
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the Amazon EC2 Spot Fleet IAM role applied to a
 SPOT compute environment.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-computeenvironment-computeresources.html#cfn-batch-computeenvironment-computeresources-spotiamfleetrole
         */
        SpotIamFleetRole?: C.YieldsString;
        /**
         * Integer

         *
         * The maximum number of EC2 vCPUs that an environment can reach.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-computeenvironment-computeresources.html#cfn-batch-computeenvironment-computeresources-maxvcpus
         */
        MaxvCpus: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The minimum percentage that a Spot Instance price must be when compared with the On-Demand
 
 price for that instance type before instances are launched. For example, if your bid
 percentage is 20%, then the Spot price must be below 20% of the current On-Demand
 price for
 that EC2 instance.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-computeenvironment-computeresources.html#cfn-batch-computeenvironment-computeresources-bidpercentage
         */
        BidPercentage?: C.Yields<C.CFInteger>;
        /**
         * List of strings

         *
         * The EC2 security group that is associated with instances launched in the compute
 environment.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-computeenvironment-computeresources.html#cfn-batch-computeenvironment-computeresources-securitygroupids
         */
        SecurityGroupIds: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of strings

         *
         * The VPC subnets into which the compute resources are launched.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-computeenvironment-computeresources.html#cfn-batch-computeenvironment-computeresources-subnets
         */
        Subnets: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The type of compute environment: EC2 or SPOT.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-computeenvironment-computeresources.html#cfn-batch-computeenvironment-computeresources-type
         */
        Type: C.YieldsString;
        /**
         * Integer

         *
         * The minimum number of EC2 vCPUs that an environment should maintain.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-computeenvironment-computeresources.html#cfn-batch-computeenvironment-computeresources-minvcpus
         */
        MinvCpus: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The Amazon Machine Image (AMI) ID used for instances launched in the compute
 environment.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-computeenvironment-computeresources.html#cfn-batch-computeenvironment-computeresources-imageid
         */
        ImageId?: C.YieldsString;
        /**
         * String

         *
         * The Amazon ECS instance profile applied to Amazon EC2 instances in a compute
 environment.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-computeenvironment-computeresources.html#cfn-batch-computeenvironment-computeresources-instancerole
         */
        InstanceRole: C.YieldsString;
        /**
         * List of strings

         *
         * The instances types that may launched.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-computeenvironment-computeresources.html#cfn-batch-computeenvironment-computeresources-instancetypes
         */
        InstanceTypes: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The EC2 key pair that is used for instances launched in the compute
 environment.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-computeenvironment-computeresources.html#cfn-batch-computeenvironment-computeresources-ec2keypair
         */
        Ec2KeyPair?: C.YieldsString;
        /**
         * AWS CloudFormation Resource Tags
         *
         * Key-value pair tags to be applied to resources that are launched in the compute
 environment.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-computeenvironment-computeresources.html#cfn-batch-computeenvironment-computeresources-tags
         */
        Tags?: C.Yields<C.CFJson>;
        /**
         * Integer

         *
         * The desired number of EC2 vCPUS in the compute environment.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-computeenvironment-computeresources.html#cfn-batch-computeenvironment-computeresources-desiredvcpus
         */
        DesiredvCpus?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.Batch {
    /**
     * The AWS::Batch::JobDefinition resource specifies the parameters for an
    AWS Batch job definition. For more information, see Job Definitions in the
    AWS Batch User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-jobdefinition.html
     */
    interface JobDefinition {
        Type: 'AWS::Batch::JobDefinition';
        Properties: AWS.Batch.JobDefinition.Properties;
    }
    /**
     * The AWS::Batch::JobDefinition resource specifies the parameters for an
    AWS Batch job definition. For more information, see Job Definitions in the
    AWS Batch User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-jobdefinition.html
     */
    function JobDefinition(props: C.Omit<JobDefinition, 'Type'>): JobDefinition;
}
export declare namespace AWS.Batch.JobDefinition {
    interface Properties {
        /**
         * String

         *
         * The type of job definition.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-jobdefinition.html#cfn-batch-jobdefinition-type
         */
        Type: C.YieldsString;
        /**
         * JSON object

         *
         * Default parameters or parameter substitution placeholders that are set in the job
 definition. Parameters are specified as a key-value pair mapping.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-jobdefinition.html#cfn-batch-jobdefinition-parameters
         */
        Parameters?: C.Yields<C.CFJson>;
        /**
         *                             Type: AWS Batch
    JobDefinition ContainerProperties
         *
         * An object with various properties specific to container-based jobs.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-jobdefinition.html#cfn-batch-jobdefinition-containerproperties
         */
        ContainerProperties: C.Yields<AWS.Batch.JobDefinition.ContainerProperties>;
        /**
         * String

         *
         * The name of the job definition.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-jobdefinition.html#cfn-batch-jobdefinition-jobdefinitionname
         */
        JobDefinitionName?: C.YieldsString;
        /**
         *                             Type: AWS Batch JobDefinition
    RetryStrategy
         *
         * The retry strategy to use for failed jobs that are submitted with this job
 definition.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-jobdefinition.html#cfn-batch-jobdefinition-retrystrategy
         */
        RetryStrategy?: C.Yields<AWS.Batch.JobDefinition.RetryStrategy>;
    }
}
export declare namespace AWS.Batch.JobDefinition {
    interface ContainerProperties {
        /**
         *                             Type: List of AWS Batch JobDefinition
    MountPoints
         *
         * The mount points for data volumes in your container. This parameter maps to
 Volumes in the Create a container section of the
 Docker Remote API and the --volume option to docker
    run.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-containerproperties.html#cfn-batch-jobdefinition-containerproperties-mountpoints
         */
        MountPoints?: C.Yields<C.CFList<C.Yields<AWS.Batch.JobDefinition.MountPoints>>>;
        /**
         * String

         *
         * The user name to use inside the container. This parameter maps to User
 in the Create a container section of the Docker Remote API and the
 --user option to docker
    run.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-containerproperties.html#cfn-batch-jobdefinition-containerproperties-user
         */
        User?: C.YieldsString;
        /**
         *                             Type: List of AWS Batch JobDefinition
    Volumes
         *
         * A list of data volumes used in a job.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-containerproperties.html#cfn-batch-jobdefinition-containerproperties-volumes
         */
        Volumes?: C.Yields<C.CFList<C.Yields<AWS.Batch.JobDefinition.Volumes>>>;
        /**
         * List of strings

         *
         * The command that is passed to the container. This parameter maps to
 Cmd in the Create a container section of the
 Docker Remote API and the COMMAND parameter to docker run.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-containerproperties.html#cfn-batch-jobdefinition-containerproperties-command
         */
        Command?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * Integer

         *
         * The hard limit (in MiB) of memory to present to the container. If your container
 attempts to exceed the memory specified here, the container is killed. This parameter
 maps to Memory in the Create a container section of the
 Docker Remote API and the --memory option to docker
    run.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-containerproperties.html#cfn-batch-jobdefinition-containerproperties-memory
         */
        Memory: C.Yields<C.CFInteger>;
        /**
         * Boolean

         *
         * When this parameter is true, the container is given elevated privileges on the host
 container instance (similar to the root user). This parameter maps to
 Privileged in the Create a container section of the
 Docker Remote API and the --privileged option to docker
    run.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-containerproperties.html#cfn-batch-jobdefinition-containerproperties-privileged
         */
        Privileged?: C.YieldsBoolean;
        /**
         *                             Type: List of AWS Batch JobDefinition
    Environment
         *
         * The environment variables to pass to a container. This parameter maps to
 Env in the Create a container section of the
 Docker Remote API and the --env option to docker
    run.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-containerproperties.html#cfn-batch-jobdefinition-containerproperties-environment
         */
        Environment?: C.Yields<C.CFList<C.Yields<AWS.Batch.JobDefinition.Environment>>>;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the IAM role that the container can assume
 for AWS permissions.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-containerproperties.html#cfn-batch-jobdefinition-containerproperties-jobrolearn
         */
        JobRoleArn?: C.YieldsString;
        /**
         * Boolean

         *
         * When this parameter is true, the container is given read-only access to its root
 file system. This parameter maps to ReadonlyRootfs in the
 Create a container section of the Docker Remote API and the
 --read-only option to docker run.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-containerproperties.html#cfn-batch-jobdefinition-containerproperties-readonlyrootfilesystem
         */
        ReadonlyRootFilesystem?: C.YieldsBoolean;
        /**
         *                             Type: List of AWS Batch JobDefinition
    Ulimit
         *
         * A list of ulimits to set in the container. This parameter maps to
 Ulimits in the Create a container section of the
 Docker Remote API and the --ulimit option to docker run.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-containerproperties.html#cfn-batch-jobdefinition-containerproperties-ulimits
         */
        Ulimits?: C.Yields<C.CFList<C.Yields<AWS.Batch.JobDefinition.Ulimit>>>;
        /**
         * Integer

         *
         * The number of vCPUs reserved for the container. This parameter maps to CpuShares in the
 Create a container section of the Docker Remote API and the
 --cpu-shares option to docker
    run. Each vCPU is equivalent to 1,024 CPU shares.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-containerproperties.html#cfn-batch-jobdefinition-containerproperties-vcpus
         */
        Vcpus: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The image used to start a container. This string is passed directly to the Docker
 daemon. Images in the Docker Hub registry are available by default. Other repositories
 are specified with
    repository-url/image:tag
    . Up to 255 letters (uppercase and lowercase), numbers, hyphens, underscores, colons,
 periods, forward slashes, and number signs are allowed. This parameter maps to Image in the
 Create a container section of the Docker Remote API and the
 IMAGE parameter of docker
    run.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-containerproperties.html#cfn-batch-jobdefinition-containerproperties-image
         */
        Image: C.YieldsString;
    }
}
export declare namespace AWS.Batch.JobDefinition {
    interface Environment {
        /**
         * String

         *
         * The value of the environment variable.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-environment.html#cfn-batch-jobdefinition-environment-value
         */
        Value?: C.YieldsString;
        /**
         * String

         *
         * The name of the environment variable.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-environment.html#cfn-batch-jobdefinition-environment-name
         */
        Name?: C.YieldsString;
    }
}
export declare namespace AWS.Batch.JobDefinition {
    interface MountPoints {
        /**
         * Boolean

         *
         * If this value is true, the container has read-only access to the volume;
 otherwise, the container can write to the volume. The
 default value is false.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-mountpoints.html#cfn-batch-jobdefinition-mountpoints-readonly
         */
        ReadOnly?: C.YieldsBoolean;
        /**
         * String

         *
         * The name of the volume to mount.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-mountpoints.html#cfn-batch-jobdefinition-mountpoints-sourcevolume
         */
        SourceVolume?: C.YieldsString;
        /**
         * String

         *
         * The path on the container at which to mount the host volume.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-mountpoints.html#cfn-batch-jobdefinition-mountpoints-containerpath
         */
        ContainerPath?: C.YieldsString;
    }
}
export declare namespace AWS.Batch.JobDefinition {
    interface RetryStrategy {
        /**
         * Integer

         *
         * The number of times to move a job to the RUNNABLE status. You may
 specify between 1 and 10 attempts. If attempts is greater than one, the job is
 retried if it fails until it has moved to RUNNABLE that many times.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-retrystrategy.html#cfn-batch-jobdefinition-retrystrategy-attempts
         */
        Attempts?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.Batch.JobDefinition {
    interface Ulimit {
        /**
         * Integer

         *
         * The soft limit for the ulimit type.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-ulimit.html#cfn-batch-jobdefinition-ulimit-softlimit
         */
        SoftLimit: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The hard limit for the ulimit type.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-ulimit.html#cfn-batch-jobdefinition-ulimit-hardlimit
         */
        HardLimit: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The type of the ulimit.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-ulimit.html#cfn-batch-jobdefinition-ulimit-name
         */
        Name: C.YieldsString;
    }
}
export declare namespace AWS.Batch.JobDefinition {
    interface Volumes {
        /**
         *                             Type: AWS Batch JobDefinition VolumesHost
         *
         * The contents of the Host parameter determine whether your data
 volume persists on the host container instance and where it is stored. If the host
 parameter is empty, then the Docker daemon assigns a host path for your data volume,
 but
 the data is not guaranteed to persist after the containers associated with it stop
 running.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-volumes.html#cfn-batch-jobdefinition-volumes-host
         */
        Host?: C.Yields<AWS.Batch.JobDefinition.VolumesHost>;
        /**
         * String

         *
         * The name of the volume. Up to 255 letters (uppercase and lowercase), numbers, hyphens,
 and underscores are allowed. This name is referenced in the
 SourceVolume parameter of container definition
 MountPoints.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-volumes.html#cfn-batch-jobdefinition-volumes-name
         */
        Name?: C.YieldsString;
    }
}
export declare namespace AWS.Batch.JobDefinition {
    interface VolumesHost {
        /**
         * String

         *
         * The path on the host container instance that is presented to the container.
 If this parameter is empty, then the Docker daemon has assigned a host path for you.
 If
 the VolumesHost parameter contains a SourcePath file location,
 then the data volume persists at the specified location on the host container instance
 until you delete it manually. If the SourcePath value does not exist on the
 host container instance, the Docker daemon creates it. If the location does exist,
 the
 contents of the source path folder are exported.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobdefinition-volumeshost.html#cfn-batch-jobdefinition-volumeshost-sourcepath
         */
        SourcePath?: C.YieldsString;
    }
}
export declare namespace AWS.Batch {
    /**
     * The AWS::Batch::JobQueue resource defines your AWS Batch job queue. For more
    information, see Job
       Queues in the AWS Batch User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-jobqueue.html
     */
    interface JobQueue {
        Type: 'AWS::Batch::JobQueue';
        Properties: AWS.Batch.JobQueue.Properties;
    }
    /**
     * The AWS::Batch::JobQueue resource defines your AWS Batch job queue. For more
    information, see Job
       Queues in the AWS Batch User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-jobqueue.html
     */
    function JobQueue(props: C.Omit<JobQueue, 'Type'>): JobQueue;
}
export declare namespace AWS.Batch.JobQueue {
    interface Properties {
        /**
         *                             Type: List of AWS Batch JobQueue
    ComputeEnvironmentOrder
         *
         * The compute environments that are attached to the job queue and the order in
 which job placement is preferred. Compute environments are selected for job placement
 in
 ascending order.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-jobqueue.html#cfn-batch-jobqueue-computeenvironmentorder
         */
        ComputeEnvironmentOrder: C.Yields<C.CFList<C.Yields<AWS.Batch.JobQueue.ComputeEnvironmentOrder>>>;
        /**
         * Integer

         *
         * The priority of the job queue.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-jobqueue.html#cfn-batch-jobqueue-priority
         */
        Priority: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The status of the job queue (for example, CREATING or
 VALID).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-jobqueue.html#cfn-batch-jobqueue-state
         */
        State?: C.YieldsString;
        /**
         * String

         *
         * The name of the job queue.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-batch-jobqueue.html#cfn-batch-jobqueue-jobqueuename
         */
        JobQueueName?: C.YieldsString;
    }
}
export declare namespace AWS.Batch.JobQueue {
    interface ComputeEnvironmentOrder {
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the compute environment.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobqueue-computeenvironmentorder.html#cfn-batch-jobqueue-computeenvironmentorder-computeenvironment
         */
        ComputeEnvironment: C.YieldsString;
        /**
         * Integer

         *
         * The order of the compute environment.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-batch-jobqueue-computeenvironmentorder.html#cfn-batch-jobqueue-computeenvironmentorder-order
         */
        Order: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.CertificateManager {
    /**
     * The AWS::CertificateManager::Certificate resource requests an AWS Certificate Manager
    (ACM) certificate that you can use with AWS services to enable secure connections.
    For
    example, you can deploy an ACM certificate to an Elastic Load Balancing load balancer
    to enable HTTPS
    support. For more information, see the RequestCertificate action
    in the AWS Certificate Manager API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-certificatemanager-certificate.html
     */
    interface Certificate {
        Type: 'AWS::CertificateManager::Certificate';
        Properties: AWS.CertificateManager.Certificate.Properties;
    }
    /**
     * The AWS::CertificateManager::Certificate resource requests an AWS Certificate Manager
    (ACM) certificate that you can use with AWS services to enable secure connections.
    For
    example, you can deploy an ACM certificate to an Elastic Load Balancing load balancer
    to enable HTTPS
    support. For more information, see the RequestCertificate action
    in the AWS Certificate Manager API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-certificatemanager-certificate.html
     */
    function Certificate(props: C.Omit<Certificate, 'Type'>): Certificate;
}
export declare namespace AWS.CertificateManager.Certificate {
    interface Properties {
        /**
         * String

         *
         * Fully qualified domain name (FQDN), such as www.example.com, of
 the site that you want to secure with the ACM certificate. To protect several
 sites in the same domain, use an asterisk (*) to specify a wildcard.
 For example, *.example.com protects www.example.com,
 site.example.com, and images.example.com.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-certificatemanager-certificate.html#cfn-certificatemanager-certificate-domainname
         */
        DomainName: C.YieldsString;
        /**
         * List of AWS Certificate Manager Certificate DomainValidationOption
         *
         * Domain information that domain name registrars use to verify your identity. For more
 information and the default values, see Configure Email for Your Domain and Validate Domain Ownership in the AWS Certificate Manager User Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-certificatemanager-certificate.html#cfn-certificatemanager-certificate-domainvalidationoptions
         */
        DomainValidationOptions?: C.Yields<C.CFList<C.Yields<AWS.CertificateManager.Certificate.DomainValidationOption>>>;
        /**
         * List of strings

         *
         * FQDNs to be included in the Subject Alternative Name extension of the ACM certificate.
 For example, you can add www.example.net to a certificate for the www.example.com domain name so that users can reach your site by using either name.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-certificatemanager-certificate.html#cfn-certificatemanager-certificate-subjectalternativenames
         */
        SubjectAlternativeNames?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key–value pairs) for this ACM certificate.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-certificatemanager-certificate.html#cfn-certificatemanager-certificate-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
    }
}
export declare namespace AWS.CertificateManager.Certificate {
    interface DomainValidationOption {
        /**
         * String

         *
         * Fully Qualified Domain Name (FQDN) of the Certificate that you are requesting.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-certificatemanager-certificate-domainvalidationoption.html#cfn-certificatemanager-certificate-domainvalidationoptions-domainname
         */
        DomainName: C.YieldsString;
        /**
         * String

         *
         * The domain that domain name registrars use to send validation emails. Registrars use
 this value as the email address suffix when sending emails to verify your identity.
 This value must be the same as the domain name or a superdomain of the domain name.
 For more information, see the ValidationDomain content for the DomainValidationOption data type in the AWS Certificate Manager API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-certificatemanager-certificate-domainvalidationoption.html#cfn-certificatemanager-certificate-domainvalidationoption-validationdomain
         */
        ValidationDomain: C.YieldsString;
    }
}
export declare namespace AWS.CloudFormation {
    /**
     * In an AWS CloudFormation template, you use the AWS::CloudFormation::CustomResource or Custom::String resource type to specify custom resources.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cfn-customresource.html
     */
    interface CustomResource {
        Type: 'AWS::CloudFormation::CustomResource';
        Properties: AWS.CloudFormation.CustomResource.Properties;
    }
    /**
     * In an AWS CloudFormation template, you use the AWS::CloudFormation::CustomResource or Custom::String resource type to specify custom resources.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cfn-customresource.html
     */
    function CustomResource(props: C.Omit<CustomResource, 'Type'>): CustomResource;
}
export declare namespace AWS.CloudFormation.CustomResource {
    interface Properties {
        /**
         * String

         *
         * The service token that was given to the template developer by the service provider
 to access the service, such as an Amazon SNS topic ARN or Lambda function ARN. The
 service token must be from the same region in which you are creating the
 stack.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cfn-customresource.html#cfn-customresource-servicetoken
         */
        ServiceToken: C.YieldsString;
    }
}
export declare namespace AWS.CloudFormation {
    /**
     * The AWS::CloudFormation::Stack type nests a stack as a resource in a top-level
    template.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-stack.html
     */
    interface Stack {
        Type: 'AWS::CloudFormation::Stack';
        Properties: AWS.CloudFormation.Stack.Properties;
    }
    /**
     * The AWS::CloudFormation::Stack type nests a stack as a resource in a top-level
    template.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-stack.html
     */
    function Stack(props: C.Omit<Stack, 'Type'>): Stack;
}
export declare namespace AWS.CloudFormation.Stack {
    interface Properties {
        /**
         * List of strings

         *
         * A list of existing Amazon SNS topics where notifications about stack events are
 sent.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-stack.html#cfn-cloudformation-stack-notificationarns
         */
        NotificationARNs?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * AWS CloudFormation Stack Parameters
         *
         * The set of parameters passed to AWS CloudFormation when this nested stack is created.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-stack.html#cfn-cloudformation-stack-parameters
         */
        Parameters?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key–value pairs) to describe this
 stack.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-stack.html#cfn-cloudformation-stack-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * String

         *
         * The URL of a template that specifies the stack that you want to create as a
 resource. The template must be stored on an Amazon S3 bucket, so the URL must have
 the
 form:
 https://s3.amazonaws.com/.../TemplateName.template
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-stack.html#cfn-cloudformation-stack-templateurl
         */
        TemplateURL: C.YieldsString;
        /**
         * Integer

         *
         * The length of time, in minutes, that AWS CloudFormation waits for the nested stack
 to reach
 the CREATE_COMPLETE state. The default is no timeout. When AWS CloudFormation detects that the
 nested stack has reached the CREATE_COMPLETE state, it marks the nested stack
 resource as CREATE_COMPLETE in the parent stack and resumes creating the parent
 stack. If the timeout period expires before the nested stack reaches
 CREATE_COMPLETE, AWS CloudFormation marks the nested stack as failed and rolls back both the
 nested stack and parent stack.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-stack.html#cfn-cloudformation-stack-timeoutinminutes
         */
        TimeoutInMinutes?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.CloudFormation {
    /**
     *
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waitcondition.html
     */
    interface WaitCondition {
        Type: 'AWS::CloudFormation::WaitCondition';
        Properties: AWS.CloudFormation.WaitCondition.Properties;
    }
    /**
     *
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waitcondition.html
     */
    function WaitCondition(props: C.Omit<WaitCondition, 'Type'>): WaitCondition;
}
export declare namespace AWS.CloudFormation.WaitCondition {
    interface Properties {
        /**
         * Integer

         *
         * The number of success signals that AWS CloudFormation must receive before it continues
 the stack creation process. When the wait condition receives the requisite
 number of success signals, AWS CloudFormation resumes the creation of the stack. If
 the
 wait condition does not receive the specified number of success signals
 before the Timeout period expires, AWS CloudFormation assumes that the wait condition
 has
 failed and rolls the stack back.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waitcondition.html#cfn-waitcondition-count
         */
        Count?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * A reference to the wait condition handle used to signal this wait
 condition. Use the Ref intrinsic function to specify an
 AWS::CloudFormation::WaitConditionHandle resource.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waitcondition.html#cfn-waitcondition-handle
         */
        Handle: C.YieldsString;
        /**
         * String

         *
         * The length of time (in seconds) to wait for the number of signals that the
 Count property specifies. Timeout is a
 minimum-bound property, meaning the timeout occurs no sooner than the time
 you specify, but can occur shortly thereafter. The maximum time that can be
 specified for this property is 12 hours (43200 seconds).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waitcondition.html#cfn-waitcondition-timeout
         */
        Timeout: C.YieldsString;
    }
}
export declare namespace AWS.CloudFormation {
    /**
     *
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waitconditionhandle.html
     */
    interface WaitConditionHandle {
        Type: 'AWS::CloudFormation::WaitConditionHandle';
        Properties: AWS.CloudFormation.WaitConditionHandle.Properties;
    }
    /**
     *
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waitconditionhandle.html
     */
    function WaitConditionHandle(props: C.Omit<WaitConditionHandle, 'Type'>): WaitConditionHandle;
}
export declare namespace AWS.CloudFormation.WaitConditionHandle {
    interface Properties {
    }
}
export declare namespace AWS.CloudFront {
    /**
     * Creates an Amazon CloudFront web distribution. For general information about CloudFront
    distributions, see
    the Introduction to Amazon CloudFront in
    the Amazon CloudFront Developer Guide. For specific information about creating CloudFront web
    distributions, see CreateDistribution in the Amazon CloudFront API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distribution.html
     */
    interface Distribution {
        Type: 'AWS::CloudFront::Distribution';
        Properties: AWS.CloudFront.Distribution.Properties;
    }
    /**
     * Creates an Amazon CloudFront web distribution. For general information about CloudFront
    distributions, see
    the Introduction to Amazon CloudFront in
    the Amazon CloudFront Developer Guide. For specific information about creating CloudFront web
    distributions, see CreateDistribution in the Amazon CloudFront API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distribution.html
     */
    function Distribution(props: C.Omit<Distribution, 'Type'>): Distribution;
}
export declare namespace AWS.CloudFront.Distribution {
    interface Properties {
        /**
         * DistributionConfig type

         *
         * The distribution's configuration information.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distribution.html#cfn-cloudfront-distribution-distributionconfig
         */
        DistributionConfig: C.Yields<AWS.CloudFront.Distribution.DistributionConfig>;
    }
}
export declare namespace AWS.CloudFront.Distribution {
    interface CacheBehavior {
        /**
         * List of strings

         *
         * HTTP methods that CloudFront processes and forwards to your Amazon S3 bucket or your
 custom origin. You can specify ["HEAD", "GET"], ["GET", "HEAD",
    "OPTIONS"], or ["DELETE", "GET", "HEAD", "OPTIONS", "PATCH",
    "POST", "PUT"]. If you don't specify a value, AWS CloudFormation specifies
 ["HEAD", "GET"].

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-cachebehavior.html#cfn-cloudfront-cachebehavior-allowedmethods
         */
        AllowedMethods?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of strings

         *
         * HTTP methods for which CloudFront caches responses. You can specify ["HEAD",
    "GET"] or ["GET", "HEAD", "OPTIONS"]. If you don't specify
 a value, AWS CloudFormation specifies ["HEAD", "GET"].

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-cachebehavior.html#cfn-cloudfront-cachebehavior-cachedmethods
         */
        CachedMethods?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * Boolean

         *
         * Indicates whether CloudFront automatically compresses certain files for this cache
 behavior. For more information, see Serving Compressed Files
 in the Amazon CloudFront Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-cachebehavior.html#cfn-cloudfront-cachebehavior-compress
         */
        Compress?: C.YieldsBoolean;
        /**
         * Number

         *
         * The default time in seconds that objects stay in CloudFront caches before CloudFront
 forwards another request to your custom origin to determine whether the object has
 been updated. This value applies only when your custom origin does not add HTTP
 headers, such as Cache-Control max-age, Cache-Control
    s-maxage, and Expires to objects.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-cachebehavior.html#cfn-cloudfront-cachebehavior-defaultttl
         */
        DefaultTTL?: C.Yields<C.CFLong>;
        /**
         * ForwardedValues
 type

         *
         * Specifies how CloudFront handles query strings or cookies.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-cachebehavior.html#cfn-cloudfront-cachebehavior-forwardedvalues
         */
        ForwardedValues: C.Yields<AWS.CloudFront.Distribution.ForwardedValues>;
        /**
         * Number

         *
         * The maximum time in seconds that objects stay in CloudFront caches before CloudFront
 forwards
 another request to your custom origin to determine whether the object has been
 updated. This value applies only when your custom origin does not add HTTP
 headers, such as Cache-Control max-age, Cache-Control
    s-maxage, and Expires to objects.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-cachebehavior.html#cfn-cloudfront-cachebehavior-maxttl
         */
        MaxTTL?: C.Yields<C.CFLong>;
        /**
         * Number

         *
         * The minimum amount of time that you want objects to stay in the cache before
 CloudFront queries your origin to see whether the object has been updated.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-cachebehavior.html#cfn-cloudfront-cachebehavior-minttl
         */
        MinTTL?: C.Yields<C.CFLong>;
        /**
         * String

         *
         * The pattern to which this cache behavior applies. For example, you can specify
 images/*.jpg.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-cachebehavior.html#cfn-cloudfront-cachebehavior-pathpattern
         */
        PathPattern: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether to use the origin that is associated with this cache behavior
 to distribute media files in the Microsoft Smooth Streaming format. If you specify
 true, you can still use this cache behavior to distribute other
 content if the content matches the PathPattern value.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-cachebehavior.html#cfn-cloudfront-cachebehavior-smoothstreaming
         */
        SmoothStreaming?: C.YieldsBoolean;
        /**
         * String

         *
         * The ID value of the origin to which you want CloudFront to route requests when a
 request matches the value of the PathPattern property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-cachebehavior.html#cfn-cloudfront-cachebehavior-targetoriginid
         */
        TargetOriginId: C.YieldsString;
        /**
         * List of strings

         *
         * A list of AWS accounts that can create signed URLs in order to access private
 content.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-cachebehavior.html#cfn-cloudfront-cachebehavior-trustedsigners
         */
        TrustedSigners?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The protocol that users can use to access the files in the origin that you
 specified in the TargetOriginId property when a request matches the
 value of the PathPattern property. For more information about the
 valid values, see the ViewerProtocolPolicy content for the CacheBehavior data type in the
 Amazon CloudFront API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-cachebehavior.html#cfn-cloudfront-cachebehavior-viewerprotocolpolicy
         */
        ViewerProtocolPolicy: C.YieldsString;
    }
}
export declare namespace AWS.CloudFront.Distribution {
    interface Cookies {
        /**
         * String

         *
         * The cookies to forward to the origin of the cache behavior. You can specify
 none, all, or whitelist.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-forwardedvalues-cookies.html#cfn-cloudfront-forwardedvalues-cookies-forward
         */
        Forward: C.YieldsString;
        /**
         * List of strings

         *
         * The names of cookies to forward to the origin for the cache behavior.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-forwardedvalues-cookies.html#cfn-cloudfront-forwardedvalues-cookies-whitelistednames
         */
        WhitelistedNames?: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.CloudFront.Distribution {
    interface CustomErrorResponse {
        /**
         * Integer

         *
         * The minimum amount of time, in seconds, that Amazon CloudFront caches the HTTP status
 code
 that you specified in the ErrorCode property. The default value is
 300.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig-customerrorresponse.html#cfn-cloudfront-distributionconfig-customerrorresponse-errorcachingminttl
         */
        ErrorCachingMinTTL?: C.Yields<C.CFLong>;
        /**
         * Integer

         *
         * An HTTP status code for which you want to specify a custom error page. You can
 specify 400, 403, 404, 405,
 414, 500, 501, 502,
 503, or 504.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig-customerrorresponse.html#cfn-cloudfront-distributionconfig-customerrorresponse-errorcode
         */
        ErrorCode: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The HTTP status code that CloudFront returns to viewer along with the custom error
 page. You can specify 200, 400, 403,
 404, 405, 414, 500,
 501, 502, 503, or
 504.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig-customerrorresponse.html#cfn-cloudfront-distributionconfig-customerrorresponse-responsecode
         */
        ResponseCode?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The path to the custom error page that CloudFront returns to a viewer when your
 origin returns the HTTP status code that you specified in the
 ErrorCode property. For example, you can specify
 /404-errors/403-forbidden.html.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig-customerrorresponse.html#cfn-cloudfront-distributionconfig-customerrorresponse-responsepagepath
         */
        ResponsePagePath?: C.YieldsString;
    }
}
export declare namespace AWS.CloudFront.Distribution {
    interface CustomOriginConfig {
        /**
         * Integer

         *
         * The HTTP port the custom origin listens on.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-customorigin.html#cfn-cloudfront-customorigin-httpport
         */
        HTTPPort?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The HTTPS port the custom origin listens on.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-customorigin.html#cfn-cloudfront-customorigin-httpsport
         */
        HTTPSPort?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The origin protocol policy to apply to your origin.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-customorigin.html#cfn-cloudfront-customorigin-originprotocolpolicy
         */
        OriginProtocolPolicy: C.YieldsString;
        /**
         * List of strings

         *
         * The SSL protocols that CloudFront can use when establishing an HTTPS connection with
 your origin. By default, AWS CloudFormation specifies the TLSv1 and SSLv3 protocols.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-customorigin.html#cfn-cloudfront-customorigin-originsslprotocols
         */
        OriginSSLProtocols?: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.CloudFront.Distribution {
    interface DefaultCacheBehavior {
        /**
         * List of strings

         *
         * HTTP methods that CloudFront processes and forwards to your Amazon S3 bucket or your
 custom origin. In AWS CloudFormation templates, you can specify ["HEAD", "GET"],
 ["GET", "HEAD", "OPTIONS"], or ["DELETE", "GET", "HEAD",
    "OPTIONS", "PATCH", "POST", "PUT"]. If you don't specify a value, AWS CloudFormation
 specifies ["HEAD", "GET"].

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-defaultcachebehavior.html#cfn-cloudfront-defaultcachebehavior-allowedmethods
         */
        AllowedMethods?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of strings

         *
         * HTTP methods for which CloudFront caches responses. In AWS CloudFormation templates,
 you can
 specify ["HEAD", "GET"] or ["GET", "HEAD", "OPTIONS"].
 If you don't specify a value, AWS CloudFormation specifies ["HEAD", "GET"].

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-defaultcachebehavior.html#cfn-cloudfront-defaultcachebehavior-cachedmethods
         */
        CachedMethods?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * Boolean

         *
         * Indicates whether CloudFront automatically compresses certain files for this cache
 behavior. For more information, see Serving Compressed Files
 in the Amazon CloudFront Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-defaultcachebehavior.html#cfn-cloudfront-defaultcachebehavior-compress
         */
        Compress?: C.YieldsBoolean;
        /**
         * Number

         *
         * The default time in seconds that objects stay in CloudFront caches before CloudFront
 forwards another request to your custom origin to determine whether the object has
 been updated. This value applies only when your custom origin does not add HTTP
 headers, such as Cache-Control max-age, Cache-Control
    s-maxage, and Expires to objects.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-defaultcachebehavior.html#cfn-cloudfront-defaultcachebehavior-defaultttl
         */
        DefaultTTL?: C.Yields<C.CFLong>;
        /**
         * ForwardedValues
 type

         *
         * Specifies how CloudFront handles query strings or cookies.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-defaultcachebehavior.html#cfn-cloudfront-defaultcachebehavior-forwardedvalues
         */
        ForwardedValues: C.Yields<AWS.CloudFront.Distribution.ForwardedValues>;
        /**
         * Number

         *
         * The maximum time in seconds that objects stay in CloudFront caches before CloudFront
 forwards another request to your custom origin to determine whether the object has
 been updated. This value applies only when your custom origin adds HTTP
 headers, such as Cache-Control max-age, Cache-Control
    s-maxage, and Expires to objects.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-defaultcachebehavior.html#cfn-cloudfront-defaultcachebehavior-maxttl
         */
        MaxTTL?: C.Yields<C.CFLong>;
        /**
         * String

         *
         * The minimum amount of time that you want objects to stay in the cache before
 CloudFront queries your origin to see whether the object has been updated.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-defaultcachebehavior.html#cfn-cloudfront-defaultcachebehavior-minttl
         */
        MinTTL?: C.Yields<C.CFLong>;
        /**
         * Boolean

         *
         * Indicates whether to use the origin that is associated with this cache behavior
 to distribute media files in the Microsoft Smooth Streaming format.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-defaultcachebehavior.html#cfn-cloudfront-defaultcachebehavior-smoothstreaming
         */
        SmoothStreaming?: C.YieldsBoolean;
        /**
         * String

         *
         * The value of ID for the origin that CloudFront routes requests to when the default
 cache behavior is applied to a request.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-defaultcachebehavior.html#cfn-cloudfront-defaultcachebehavior-targetoriginid
         */
        TargetOriginId: C.YieldsString;
        /**
         * List of strings

         *
         * A list of AWS accounts that can create signed URLs in order to access private
 content.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-defaultcachebehavior.html#cfn-cloudfront-defaultcachebehavior-trustedsigners
         */
        TrustedSigners?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The protocol that users can use to access the files in the origin that you
 specified in the TargetOriginId property when the default cache
 behavior is applied to a request. For more information about the valid values, see
 the ViewerProtocolPolicy content for the DefaultCacheBehavior
 data type in the Amazon CloudFront API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-defaultcachebehavior.html#cfn-cloudfront-defaultcachebehavior-viewerprotocolpolicy
         */
        ViewerProtocolPolicy: C.YieldsString;
    }
}
export declare namespace AWS.CloudFront.Distribution {
    interface DistributionConfig {
        /**
         * List of strings

         *
         * CNAMEs (alternate domain names), if any, for the distribution.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig.html#cfn-cloudfront-distributionconfig-aliases
         */
        Aliases?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of CacheBehavior
         *
         * A list of CacheBehavior types for the distribution.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig.html#cfn-cloudfront-distributionconfig-cachebehaviors
         */
        CacheBehaviors?: C.Yields<C.CFList<C.Yields<AWS.CloudFront.Distribution.CacheBehavior>>>;
        /**
         * String

         *
         * Any comments that you want to include about the distribution. Optional.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig.html#cfn-cloudfront-distributionconfig-comment
         */
        Comment?: C.YieldsString;
        /**
         * ist of CloudFront
    DistributionConfig CustomErrorResponse
         *
         * Whether CloudFront replaces HTTP status codes in the 4xx and
 5xx range with custom error messages before returning the response
 to the viewer.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig.html#cfn-cloudfront-distributionconfig-customerrorresponses
         */
        CustomErrorResponses?: C.Yields<C.CFList<C.Yields<AWS.CloudFront.Distribution.CustomErrorResponse>>>;
        /**
         * DefaultCacheBehavior type
         *
         * The default cache behavior that is triggered if you do not specify the
 CacheBehavior property or if files don't match any of the values
 of PathPattern in the CacheBehavior property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig.html#cfn-cloudfront-distributionconfig-defaultcachebehavior
         */
        DefaultCacheBehavior: C.Yields<AWS.CloudFront.Distribution.DefaultCacheBehavior>;
        /**
         * String

         *
         * The object (such as index.html) that you want CloudFront to request from
 your origin when the root URL for your distribution (such as
 http://example.com/) is requested.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig.html#cfn-cloudfront-distributionconfig-defaultrootobject
         */
        DefaultRootObject?: C.YieldsString;
        /**
         * Boolean

         *
         * Controls whether the distribution is enabled to accept end user requests for content.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig.html#cfn-cloudfront-distributionconfig-enabled
         */
        Enabled: C.YieldsBoolean;
        /**
         * String

         *
         * The latest HTTP version that viewers can use to communicate with CloudFront. Viewers
 that don't support the latest version automatically use an earlier HTTP version.
 By default, AWS CloudFormation specifies http1.1.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig.html#cfn-cloudfront-distributionconfig-httpversion
         */
        HttpVersion?: C.YieldsString;
        /**
         * Logging
 type

         *
         * Controls whether access logs are written for the distribution. To turn on
 access logs, specify this property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig.html#cfn-cloudfront-distributionconfig-logging
         */
        Logging?: C.Yields<AWS.CloudFront.Distribution.Logging>;
        /**
         * List of Origins.

         *
         * A list of origins for this CloudFront distribution. For each origin, you can specify
 whether it is an Amazon S3 or custom origin.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig.html#cfn-cloudfront-distributionconfig-origins
         */
        Origins: C.Yields<C.CFList<C.Yields<AWS.CloudFront.Distribution.Origin>>>;
        /**
         * String

         *
         * The price class that corresponds with the maximum price that you want to pay
 for the CloudFront service. For more information, see Choosing the Price Class in the
 Amazon CloudFront Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig.html#cfn-cloudfront-distributionconfig-priceclass
         */
        PriceClass?: C.YieldsString;
        /**
         * CloudFront
    DistributionConfiguration Restrictions
         *
         * Specifies restrictions on who or how viewers can access your content.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig.html#cfn-cloudfront-distributionconfig-restrictions
         */
        Restrictions?: C.Yields<AWS.CloudFront.Distribution.Restrictions>;
        /**
         * CloudFront
    DistributionConfiguration ViewerCertificate
         *
         * The certificate to use when viewers use HTTPS to request objects.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig.html#cfn-cloudfront-distributionconfig-viewercertificate
         */
        ViewerCertificate?: C.Yields<AWS.CloudFront.Distribution.ViewerCertificate>;
        /**
         * String

         *
         * The AWS WAF web ACL to associate
 with this distribution. AWS WAF is a web application firewall that enables you to
 monitor the HTTP and HTTPS requests that are forwarded to CloudFront and to control
 who
 can access your content. CloudFront permits or forbids requests based on conditions
 that
 you specify, such as the IP addresses from which requests originate or the values
 of query strings.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig.html#cfn-cloudfront-distributionconfig-webaclid
         */
        WebACLId?: C.YieldsString;
    }
}
export declare namespace AWS.CloudFront.Distribution {
    interface ForwardedValues {
        /**
         * CloudFront ForwardedValues
    Cookies
         *
         * Forwards specified cookies to the origin of the cache behavior. For more information,
 see Configuring CloudFront to Cache Based on Cookies in the Amazon CloudFront Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-forwardedvalues.html#cfn-cloudfront-forwardedvalues-cookies
         */
        Cookies?: C.Yields<AWS.CloudFront.Distribution.Cookies>;
        /**
         * List of strings

         *
         * Specifies the headers that you want Amazon CloudFront to forward to the origin for
 this
 cache behavior (whitelisted headers). For the headers that you specify, Amazon CloudFront
 also caches separate versions of a specified object that is based on the header
 values in viewer requests.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-forwardedvalues.html#cfn-cloudfront-forwardedvalues-headers
         */
        Headers?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * Boolean

         *
         * Indicates whether you want CloudFront to forward query strings to the origin that
 is
 associated with this cache behavior. If so, specify true; if not,
 specify false. For more information about forwarding query strings,
 see the QueryString parameter for the ForwardedValues type in the
 Amazon CloudFront API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-forwardedvalues.html#cfn-cloudfront-forwardedvalues-querystring
         */
        QueryString: C.YieldsBoolean;
        /**
         * List of strings

         *
         * If you forward query strings to the origin, specifies the query string
 parameters that CloudFront uses to determine which content to cache. For more
 information, see Configuring
    CloudFront to Cache Based on Query String Parameters in the
 Amazon CloudFront Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-forwardedvalues.html#cfn-cloudfront-forwardedvalues-querystringcachekeys
         */
        QueryStringCacheKeys?: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.CloudFront.Distribution {
    interface GeoRestriction {
        /**
         * List of strings

         *
         * The two-letter, uppercase country code for a country that you want to include
 in your blacklist or whitelist.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig-restrictions-georestriction.html#cfn-cloudfront-distributionconfig-restrictions-georestriction-locations
         */
        Locations?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The method to restrict distribution of your content:
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig-restrictions-georestriction.html#cfn-cloudfront-distributionconfig-restrictions-georestriction-restrictiontype
         */
        RestrictionType: C.YieldsString;
    }
}
export declare namespace AWS.CloudFront.Distribution {
    interface Logging {
        /**
         * String

         *
         * The Amazon S3 bucket address where access logs are stored, for example,
 mybucket.s3.amazonaws.com.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-logging.html#cfn-cloudfront-logging-bucket
         */
        Bucket: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether CloudFront includes cookies in access logs.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-logging.html#cfn-cloudfront-logging-includecookies
         */
        IncludeCookies?: C.YieldsBoolean;
        /**
         * String

         *
         * A prefix for the access log file names for this distribution.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-logging.html#cfn-cloudfront-logging-prefix
         */
        Prefix?: C.YieldsString;
    }
}
export declare namespace AWS.CloudFront.Distribution {
    interface Origin {
        /**
         * CustomOrigin
 type

         *
         * Origin information to specify a custom origin.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-origin.html#cfn-cloudfront-origin-customorigin
         */
        CustomOriginConfig?: C.Yields<AWS.CloudFront.Distribution.CustomOriginConfig>;
        /**
         * String

         *
         * The DNS name of the Amazon Simple Storage Service (S3) bucket or the HTTP server from
 which you want
 CloudFront to get objects for this origin.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-origin.html#cfn-cloudfront-origin-domainname
         */
        DomainName: C.YieldsString;
        /**
         * String

         *
         * An identifier for the origin. The value of Id must be unique
 within the distribution.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-origin.html#cfn-cloudfront-origin-id
         */
        Id: C.YieldsString;
        /**
         * List of OriginCustomHeader type

         *
         * Custom headers that CloudFront includes when it forwards a request to your origin.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-origin.html#cfn-cloudfront-origin-origincustomheaders
         */
        OriginCustomHeaders?: C.Yields<C.CFList<C.Yields<AWS.CloudFront.Distribution.OriginCustomHeader>>>;
        /**
         * String

         *
         * The path that CloudFront uses to request content from an S3 bucket or custom origin.
 The combination of the DomainName and OriginPath
 properties must resolve to a valid path. The value must start with a slash mark
 (/) and cannot end with a slash mark.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-origin.html#cfn-cloudfront-origin-originpath
         */
        OriginPath?: C.YieldsString;
        /**
         * S3Origin
 type

         *
         * Origin information to specify an S3 origin.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-origin.html#cfn-cloudfront-origin-s3origin
         */
        S3OriginConfig?: C.Yields<AWS.CloudFront.Distribution.S3OriginConfig>;
    }
}
export declare namespace AWS.CloudFront.Distribution {
    interface OriginCustomHeader {
        /**
         * String

         *
         * The name of a header that CloudFront forwards to your origin. For more information,
 see Forwarding Custom Headers to Your Origin (Web Distributions Only) in the Amazon CloudFront Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-origin-origincustomheader.html#cfn-cloudfront-origin-origincustomheader-headername
         */
        HeaderName: C.YieldsString;
        /**
         * String

         *
         * The value for the header that you specified in the HeaderName property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-origin-origincustomheader.html#cfn-cloudfront-origin-origincustomheader-headervalue
         */
        HeaderValue: C.YieldsString;
    }
}
export declare namespace AWS.CloudFront.Distribution {
    interface Restrictions {
        /**
         * CloudFront
    DistributionConfig Restrictions GeoRestriction
         *
         * The countries in which viewers are able to access your content.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig-restrictions.html#cfn-cloudfront-distributionconfig-restrictions-georestriction
         */
        GeoRestriction: C.Yields<AWS.CloudFront.Distribution.GeoRestriction>;
    }
}
export declare namespace AWS.CloudFront.Distribution {
    interface S3OriginConfig {
        /**
         * String

         *
         * The CloudFront origin access identity to associate with the origin. This is used to
 configure the origin so that end users can access objects in an Amazon S3 bucket
 through CloudFront only.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-s3origin.html#cfn-cloudfront-s3origin-originaccessidentity
         */
        OriginAccessIdentity?: C.YieldsString;
    }
}
export declare namespace AWS.CloudFront.Distribution {
    interface ViewerCertificate {
        /**
         * String

         *
         * If you're using an alternate domain name, the Amazon Resource Name (ARN) of an AWS
 Certificate Manager (ACM) certificate. Use the ACM service to provision and manage
 your certificates. For more information, see the AWS Certificate Manager User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig-viewercertificate.html#cfn-cloudfront-distributionconfig-viewercertificate-acmcertificatearn
         */
        AcmCertificateArn?: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether to use the default certificate for your CloudFront domain name when
 viewers use HTTPS to request your content.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig-viewercertificate.html#cfn-cloudfront-distributionconfig-viewercertificate-cloudfrontdefaultcertificate
         */
        CloudFrontDefaultCertificate?: C.YieldsBoolean;
        /**
         * String

         *
         * If you're using an alternate domain name, the ID of a server certificate that was
 purchased from a certificate authority. This ID is the ServerCertificateId value, which AWS Identity and Access Management (IAM) returns when the certificate
 is added to the IAM certificate store, such as ASCACKCEVSQ6CEXAMPLE1.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig-viewercertificate.html#cfn-cloudfront-distributionconfig-viewercertificate-iamcertificateid
         */
        IamCertificateId?: C.YieldsString;
        /**
         * String

         *
         * The minimum version of the SSL protocol that you want CloudFront to use for HTTPS
 connections. CloudFront serves your objects only to browsers or devices that support
 at
 least the SSL version that you specify. For valid values, see the
 MinimumProtocolVersion content for the ViewerCertificate data
 type in the Amazon CloudFront API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig-viewercertificate.html#cfn-cloudfront-distributionconfig-viewercertificate-sslsupportmethod
         */
        MinimumProtocolVersion?: C.YieldsString;
        /**
         * String

         *
         * Specifies how CloudFront serves HTTPS requests. For valid values, see the
 SslSupportMethod content for the ViewerCertificate data
 type in the Amazon CloudFront API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudfront-distributionconfig-viewercertificate.html#cfn-cloudfront-distributionconfig-viewercertificate-minimumprotocolversion
         */
        SslSupportMethod?: C.YieldsString;
    }
}
export declare namespace AWS.CloudTrail {
    /**
     * Use the AWS::CloudTrail::Trail resource to create a trail and specify where
    logs are published. An AWS CloudTrail (CloudTrail) trail can capture AWS API calls
    made by your AWS account
    and publish the logs to an Amazon S3 bucket. For more information, see What is AWS CloudTrail? in the
    AWS CloudTrail User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudtrail-trail.html
     */
    interface Trail {
        Type: 'AWS::CloudTrail::Trail';
        Properties: AWS.CloudTrail.Trail.Properties;
    }
    /**
     * Use the AWS::CloudTrail::Trail resource to create a trail and specify where
    logs are published. An AWS CloudTrail (CloudTrail) trail can capture AWS API calls
    made by your AWS account
    and publish the logs to an Amazon S3 bucket. For more information, see What is AWS CloudTrail? in the
    AWS CloudTrail User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudtrail-trail.html
     */
    function Trail(props: C.Omit<Trail, 'Type'>): Trail;
}
export declare namespace AWS.CloudTrail.Trail {
    interface Properties {
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of a log group to which CloudTrail logs will be
 delivered.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudtrail-trail.html#cfn-cloudtrail-trail-cloudwatchlogsloggrouparn
         */
        CloudWatchLogsLogGroupArn?: C.YieldsString;
        /**
         * String

         *
         * The role ARN that Amazon CloudWatch Logs (CloudWatch Logs) assumes to write logs to
 a log group. For more information,
 see Role
    Policy Document for CloudTrail to Use CloudWatch Logs for Monitoring in the
 AWS CloudTrail User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudtrail-trail.html#cfn-cloudtrail-trail-cloudwatchlogsrolearn
         */
        CloudWatchLogsRoleArn?: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether CloudTrail validates the integrity of log files. By default, AWS
 CloudFormation sets
 this value to false. When you disable log file integrity validation, CloudTrail
 stops creating digest files. For more information, see CreateTrail in the
 AWS CloudTrail API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudtrail-trail.html#cfn-cloudtrail-trail-enablelogfilevalidation
         */
        EnableLogFileValidation?: C.YieldsBoolean;
        /**
         * List of CloudTrail Trail EventSelector
         *
         * Configures logging for management and data events.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudtrail-trail.html#cfn-cloudtrail-trail-eventselectors
         */
        EventSelectors?: C.Yields<C.CFList<C.Yields<AWS.CloudTrail.Trail.EventSelector>>>;
        /**
         * Boolean

         *
         * Indicates whether the trail is publishing events from global services, such as
 IAM, to the log files. By default, AWS CloudFormation sets this value to false.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudtrail-trail.html#cfn-cloudtrail-trail-includeglobalserviceevents
         */
        IncludeGlobalServiceEvents?: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Indicates whether the CloudTrail trail is currently logging AWS API calls.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudtrail-trail.html#cfn-cloudtrail-trail-islogging
         */
        IsLogging: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Indicates whether the CloudTrail trail is created in the region in which you create
 the
 stack (false) or in all regions (true). By default, AWS CloudFormation sets
 this value to false. For more information, see How Does CloudTrail Behave Regionally and Globally? in the
 AWS CloudTrail User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudtrail-trail.html#cfn-cloudtrail-trail-ismultiregiontrail
         */
        IsMultiRegionTrail?: C.YieldsBoolean;
        /**
         * String

         *
         * The AWS Key Management Service (AWS KMS) key ID that you want to use to encrypt CloudTrail
 logs. You can
 specify an alias name (prefixed with alias/), an alias ARN, a key ARN, or a
 globally unique identifier.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudtrail-trail.html#cfn-cloudtrail-trail-kmskeyid
         */
        KMSKeyId?: C.YieldsString;
        /**
         * String

         *
         * The name of the Amazon S3 bucket where CloudTrail publishes log files.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudtrail-trail.html#cfn-cloudtrail-trail-s3bucketname
         */
        S3BucketName: C.YieldsString;
        /**
         * String

         *
         * An Amazon S3 object key prefix that precedes the name of all log files.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudtrail-trail.html#cfn-cloudtrail-trail-s3keyprefix
         */
        S3KeyPrefix?: C.YieldsString;
        /**
         * String

         *
         * The name of an Amazon SNS topic that is notified when new log files are
 published.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudtrail-trail.html#cfn-cloudtrail-trail-snstopicname
         */
        SnsTopicName?: C.YieldsString;
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key–value pairs) for this trail.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudtrail-trail.html#cfn-cloudtrail-trail-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * String

         *
         * The name of the trail. For constraint information, see CreateTrail in the
 AWS CloudTrail API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudtrail-trail.html#cfn-cloudtrail-trail-trailname
         */
        TrailName?: C.YieldsString;
    }
}
export declare namespace AWS.CloudTrail.Trail {
    interface DataResource {
        /**
         *                             Type: String

         *
         * The resource type to log data events for. You can specify only the following value:
 AWS::S3::Object.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudtrail-trail-dataresource.html#cfn-cloudtrail-trail-dataresource-type
         */
        Type: C.YieldsString;
        /**
         *                             Type: List of strings

         *
         * A list of ARN-like strings for the specified Amazon S3 objects.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudtrail-trail-dataresource.html#cfn-cloudtrail-trail-dataresource-values
         */
        Values?: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.CloudTrail.Trail {
    interface EventSelector {
        /**
         *                             Type: List of CloudTrail Trail DataResource
         *
         * The resources for data events. CloudTrail supports logging data events for Amazon
 S3 objects only.
 For more information, see Data Events
 in the AWS CloudTrail User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudtrail-trail-eventselector.html#cfn-cloudtrail-trail-eventselector-dataresources
         */
        DataResources?: C.Yields<C.CFList<C.Yields<AWS.CloudTrail.Trail.DataResource>>>;
        /**
         *                             Type: Boolean

         *
         * Specifies whether the event selector includes management events for the trail. The
 default value is true.
 For more information, see Management  Events
 in the AWS CloudTrail User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudtrail-trail-eventselector.html#cfn-cloudtrail-trail-eventselector-includemanagementevents
         */
        IncludeManagementEvents?: C.YieldsBoolean;
        /**
         *                             Type: String

         *
         * Specifies whether to log read-only events, write-only events, or all events. The default
 value is All.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cloudtrail-trail-eventselector.html#cfn-cloudtrail-trail-eventselector-readwritetype
         */
        ReadWriteType?: C.YieldsString;
    }
}
export declare namespace AWS.CloudWatch {
    /**
     * The AWS::CloudWatch::Alarm type creates a CloudWatch alarm.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cw-alarm.html
     */
    interface Alarm {
        Type: 'AWS::CloudWatch::Alarm';
        Properties: AWS.CloudWatch.Alarm.Properties;
    }
    /**
     * The AWS::CloudWatch::Alarm type creates a CloudWatch alarm.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cw-alarm.html
     */
    function Alarm(props: C.Omit<Alarm, 'Type'>): Alarm;
}
export declare namespace AWS.CloudWatch.Alarm {
    interface Properties {
        /**
         * Boolean

         *
         * Indicates whether
 actions
 should be executed during changes to the CloudWatch alarm's state.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cw-alarm.html#cfn-cloudwatch-alarms-actionsenabled
         */
        ActionsEnabled?: C.YieldsBoolean;
        /**
         * List of strings

         *
         * The list of actions to execute when this alarm transitions into an ALARM state
 from any other state. Specify each action as an Amazon Resource Number (ARN). For
 more information about creating alarms and the actions that you can specify, see PutMetricAlarm in the Amazon CloudWatch API Reference and
 Creating Amazon CloudWatch
    Alarms in the Amazon CloudWatch User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cw-alarm.html#cfn-cloudwatch-alarms-alarmactions
         */
        AlarmActions?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The description of the alarm.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cw-alarm.html#cfn-cloudwatch-alarms-alarmdescription
         */
        AlarmDescription?: C.YieldsString;
        /**
         * String

         *
         * A name for the alarm. If you don't specify a name, AWS CloudFormation generates a
 unique
 physical ID and uses that ID for the alarm name. For more information, see Name Type.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cw-alarm.html#cfn-cloudwatch-alarms-alarmname
         */
        AlarmName?: C.YieldsString;
        /**
         * String

         *
         * The arithmetic operation to use when comparing the specified
 Statistic and Threshold. AWS CloudFormation uses the value of
 Statistic as the first operand.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cw-alarm.html#cfn-cloudwatch-alarms-comparisonoperator
         */
        ComparisonOperator: C.YieldsString;
        /**
         * List of Metric Dimension
         *
         * The
 dimensions
 of the metric for the alarm.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cw-alarm.html#cfn-cloudwatch-alarms-dimension
         */
        Dimensions?: C.Yields<C.CFList<C.Yields<AWS.CloudWatch.Alarm.Dimension>>>;
        /**
         * String

         *
         * Used only for alarms that are
 based
 on percentiles. Specifies whether to evaluate the data and
 potentially change the alarm state if there are too few data points to be
 statistically significant.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cw-alarm.html#cfn-cloudwatch-alarms-evaluatelowsamplecountpercentile
         */
        EvaluateLowSampleCountPercentile?: C.YieldsString;
        /**
         * Integer

         *
         * The number of
 periods
 over which data is compared to the specified
 threshold.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cw-alarm.html#cfn-cloudwatch-alarms-evaluationperiods
         */
        EvaluationPeriods: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The
 percentile statistic for the metric. Specify a value between
 p0.0 and p100.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cw-alarm.html#cfn-cloudwatch-alarms-extendedstatistic
         */
        ExtendedStatistic?: C.YieldsString;
        /**
         * List of strings

         *
         * The list of actions to execute when this alarm transitions into an
 INSUFFICIENT_DATA state. Specify each action as an Amazon Resource Number (ARN).
 Currently,
 the only action supported is publishing to an Amazon SNS topic or an Auto Scaling
 policy.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cw-alarm.html#cfn-cloudwatch-alarms-insufficientdataactions
         */
        InsufficientDataActions?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The name of the metric associated with the alarm. For more information about
 the metrics that you can specify, see Amazon CloudWatch Namespaces,
    Dimensions, and Metrics Reference in the
 Amazon CloudWatch User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cw-alarm.html#cfn-cloudwatch-alarms-metricname
         */
        MetricName: C.YieldsString;
        /**
         * String

         *
         * The namespace of the metric that is associated with the alarm.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cw-alarm.html#cfn-cloudwatch-alarms-namespace
         */
        Namespace: C.YieldsString;
        /**
         * List of strings

         *
         * The list of actions to execute when this alarm transitions into an OK state.
 Specify each action as an Amazon Resource Number (ARN).
 Currently,
 the only action supported is publishing to an SNS topic or an Auto Scaling
 policy.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cw-alarm.html#cfn-cloudwatch-alarms-okactions
         */
        OKActions?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * Integer

         *
         * The time over which the specified statistic is applied. Specify time in
 seconds, in multiples of 60.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cw-alarm.html#cfn-cloudwatch-alarms-period
         */
        Period: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The statistic to apply to the alarm's associated metric.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cw-alarm.html#cfn-cloudwatch-alarms-statistic
         */
        Statistic?: C.YieldsString;
        /**
         * Double

         *
         * The value against which the specified statistic is compared.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cw-alarm.html#cfn-cloudwatch-alarms-threshold
         */
        Threshold: C.Yields<C.CFDouble>;
        /**
         * String

         *
         * Sets how this alarm is to handle missing data points. If
 TreatMissingData is omitted, the default behavior of
 missing is used. For more information, see PutMetricAlarm
 in the Amazon CloudWatch API Reference and Configuring How CloudWatch Alarms Treats Missing Data
 in the Amazon CloudWatch User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cw-alarm.html#cfn-cloudwatch-alarms-treatmissingdata
         */
        TreatMissingData?: C.YieldsString;
        /**
         * String

         *
         * The unit for the metric that is associated with the alarm.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cw-alarm.html#cfn-cloudwatch-alarms-unit
         */
        Unit?: C.YieldsString;
    }
}
export declare namespace AWS.CloudWatch.Alarm {
    interface Dimension {
        /**
         * String

         *
         * The name of the dimension, from 1–255 characters in length.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cw-dimension.html#cfn-cloudwatch-alarm-dimension-name
         */
        Name: C.YieldsString;
        /**
         * String

         *
         * The value representing the dimension measurement, from 1–255 characters in length.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cw-dimension.html#cfn-cloudwatch-alarm-dimension-value
         */
        Value: C.YieldsString;
    }
}
export declare namespace AWS.CloudWatch {
    /**
     *
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudwatch-dashboard.html
     */
    interface Dashboard {
        Type: 'AWS::CloudWatch::Dashboard';
        Properties: AWS.CloudWatch.Dashboard.Properties;
    }
    /**
     *
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudwatch-dashboard.html
     */
    function Dashboard(props: C.Omit<Dashboard, 'Type'>): Dashboard;
}
export declare namespace AWS.CloudWatch.Dashboard {
    interface Properties {
        /**
         *
         *
         *
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudwatch-dashboard.html#cfn-cloudwatch-dashboard-dashboardname
         */
        DashboardName?: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cloudwatch-dashboard.html#cfn-cloudwatch-dashboard-dashboardbody
         */
        DashboardBody: C.YieldsString;
    }
}
export declare namespace AWS.CodeBuild {
    /**
     * The AWS::CodeBuild::Project resource configures how AWS CodeBuild builds your source
    code. For example, it tells AWS CodeBuild where to get the source code and which build
    environment to
    use.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codebuild-project.html
     */
    interface Project {
        Type: 'AWS::CodeBuild::Project';
        Properties: AWS.CodeBuild.Project.Properties;
    }
    /**
     * The AWS::CodeBuild::Project resource configures how AWS CodeBuild builds your source
    code. For example, it tells AWS CodeBuild where to get the source code and which build
    environment to
    use.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codebuild-project.html
     */
    function Project(props: C.Omit<Project, 'Type'>): Project;
}
export declare namespace AWS.CodeBuild.Project {
    interface Properties {
        /**
         * AWS CodeBuild Project Artifacts
         *
         * The output settings for artifacts that the project generates during a build.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codebuild-project.html#cfn-codebuild-project-artifacts
         */
        Artifacts: C.Yields<AWS.CodeBuild.Project.Artifacts>;
        /**
         * String

         *
         * A description of the project. Use the description to identify the purpose of the
 project.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codebuild-project.html#cfn-codebuild-project-description
         */
        Description?: C.YieldsString;
        /**
         * String

         *
         * The ARN of the service role that AWS CodeBuild uses to interact with services on your
 behalf.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codebuild-project.html#cfn-codebuild-project-servicerole
         */
        ServiceRole: C.YieldsString;
        /**
         * AWS CodeBuild Project Environment
         *
         * The build environment settings for the project, such as the environment type or the
 environment variables to use for the build environment.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codebuild-project.html#cfn-codebuild-project-environment
         */
        Environment: C.Yields<AWS.CodeBuild.Project.Environment>;
        /**
         * String

         *
         * The alias or Amazon Resource Name (ARN) of the AWS Key Management Service (AWS KMS)
 customer master key
 (CMK) that AWS CodeBuild uses to encrypt the build output. If you don't specify a
 value, AWS CodeBuild
 uses the AWS-managed CMK for Amazon Simple Storage Service.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codebuild-project.html#cfn-codebuild-project-encryptionkey
         */
        EncryptionKey?: C.YieldsString;
        /**
         * AWS CodeBuild Project Source
         *
         * The source code settings for the project, such as the source code's repository type
 and location.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codebuild-project.html#cfn-codebuild-project-source
         */
        Source: C.Yields<AWS.CodeBuild.Project.Source>;
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key-value pairs) for the AWS CodeBuild project.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codebuild-project.html#cfn-codebuild-project-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * String

         *
         * A name for the project. The name must be unique across all of the projects in your
 AWS account.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codebuild-project.html#cfn-codebuild-project-name
         */
        Name?: C.YieldsString;
        /**
         * Integer

         *
         * The number of minutes after which AWS CodeBuild stops the build if it's not complete.
 For
 valid values, see the timeoutInMinutes field in the
 AWS CodeBuild User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codebuild-project.html#cfn-codebuild-project-timeoutinminutes
         */
        TimeoutInMinutes?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.CodeBuild.Project {
    interface Artifacts {
        /**
         * String

         *
         * The path to the build output folder where AWS CodeBuild saves the build output
 artifacts.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codebuild-project-artifacts.html#cfn-codebuild-project-artifacts-path
         */
        Path?: C.YieldsString;
        /**
         * String

         *
         * The type of build output artifact. For valid values, see the artifacts-type field
 in the AWS CodeBuild User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codebuild-project-artifacts.html#cfn-codebuild-project-artifacts-type
         */
        Type: C.YieldsString;
        /**
         * String

         *
         * Indicates how AWS CodeBuild packages the build output artifacts. For valid values,
 see the
 packaging field in the
 AWS CodeBuild User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codebuild-project-artifacts.html#cfn-codebuild-project-artifacts-packaging
         */
        Packaging?: C.YieldsString;
        /**
         * String

         *
         * The location where AWS CodeBuild saves the build output artifacts. For valid values,
 see the
 artifacts-location field in the
 AWS CodeBuild User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codebuild-project-artifacts.html#cfn-codebuild-project-artifacts-location
         */
        Location?: C.YieldsString;
        /**
         * String

         *
         * The name of the build output folder where AWS CodeBuild saves the build output artifacts.
 For .zip packages, the name of the build output .zip file that contains the build
 output
 artifacts.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codebuild-project-artifacts.html#cfn-codebuild-project-artifacts-name
         */
        Name?: C.YieldsString;
        /**
         * String

         *
         * The information AWS CodeBuild adds to the build output path, such as a build ID. For
 more
 information, see the namespaceType field in the
 AWS CodeBuild User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codebuild-project-artifacts.html#cfn-codebuild-project-artifacts-namespacetype
         */
        NamespaceType?: C.YieldsString;
    }
}
export declare namespace AWS.CodeBuild.Project {
    interface Environment {
        /**
         * String

         *
         * The type of build environment. For valid values, see the environment-type field in the
 AWS CodeBuild User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codebuild-project-environment.html#cfn-codebuild-project-environment-type
         */
        Type: C.YieldsString;
        /**
         * List of AWS CodeBuild Project Environment
    EnvironmentVariables
         *
         * The environment variables that your builds can use. For more information, see the
 environmentVariables field in the
 AWS CodeBuild User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codebuild-project-environment.html#cfn-codebuild-project-environment-environmentvariables
         */
        EnvironmentVariables?: C.Yields<C.CFList<C.Yields<AWS.CodeBuild.Project.EnvironmentVariable>>>;
        /**
         * Boolean

         *
         * Indicates how the project builds Docker images. Specify true to enable running the Docker daemon inside a Docker container.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codebuild-project-environment.html#cfn-codebuild-project-environment-privilegedmode
         */
        PrivilegedMode?: C.YieldsBoolean;
        /**
         * String

         *
         * The Docker image identifier that the build environment uses. For more information,
 see the image field in the
 AWS CodeBuild User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codebuild-project-environment.html#cfn-codebuild-project-environment-image
         */
        Image: C.YieldsString;
        /**
         * String

         *
         * The type of compute environment, such as BUILD_GENERAL1_SMALL. The
 compute type determines the number of CPU cores and memory the build environment uses.
 For valid values, see the computeType field in the
 AWS CodeBuild User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codebuild-project-environment.html#cfn-codebuild-project-environment-computetype
         */
        ComputeType: C.YieldsString;
    }
}
export declare namespace AWS.CodeBuild.Project {
    interface EnvironmentVariable {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codebuild-project-environmentvariable.html#cfn-codebuild-project-environmentvariable-type
         */
        Type?: C.YieldsString;
        /**
         * String

         *
         * The value of the environment variable.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codebuild-project-environmentvariable.html#cfn-codebuild-project-environmentvariable-value
         */
        Value: C.YieldsString;
        /**
         * String

         *
         * The name of an environment variable.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codebuild-project-environmentvariable.html#cfn-codebuild-project-environmentvariable-name
         */
        Name: C.YieldsString;
    }
}
export declare namespace AWS.CodeBuild.Project {
    interface Source {
        /**
         * String

         *
         * The type of repository that contains your source code. For valid values, see the
 source-type field in the
 AWS CodeBuild User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codebuild-project-source.html#cfn-codebuild-project-source-type
         */
        Type: C.YieldsString;
        /**
         *                             Type: AWS CodeBuild Project SourceAuth
         *
         * Information about the authorization settings for AWS CodeBuild to access the source
 code to be built.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codebuild-project-source.html#cfn-codebuild-project-source-auth
         */
        Auth?: C.Yields<AWS.CodeBuild.Project.SourceAuth>;
        /**
         * String

         *
         * The build specification for the project. If this value is not provided, then the source
 code must contain a build spec
 file named buildspec.yml at the root level. If this value is provided, it can be either a single string containing
 the entire build
 specification, or the path to an alternate build spec file relative to the value of
 the built-in environment variable
 CODEBUILD_SRC_DIR. The alternate build spec file can have a name other than buildspec.yml, for example myspec.yml or
 build_spec_qa.yml or similar. For more information, see the
 Build Spec
       Reference in the AWS CodeBuild User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codebuild-project-source.html#cfn-codebuild-project-source-buildspec
         */
        BuildSpec?: C.YieldsString;
        /**
         * String

         *
         * The location of the source code in the specified repository type. For more
 information, see the source-location field in the
 AWS CodeBuild User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codebuild-project-source.html#cfn-codebuild-project-source-location
         */
        Location?: C.YieldsString;
    }
}
export declare namespace AWS.CodeBuild.Project {
    interface SourceAuth {
        /**
         *                             Type: String

         *
         * The authorization type to use. The only valid value is OAUTH, which represents the
 OAuth authorization type.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codebuild-project-sourceauth.html#cfn-codebuild-project-sourceauth-type
         */
        Type: C.YieldsString;
        /**
         *                             Type: String

         *
         * The resource value that applies to the specified authorization type.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codebuild-project-sourceauth.html#cfn-codebuild-project-sourceauth-resource
         */
        Resource?: C.YieldsString;
    }
}
export declare namespace AWS.CodeCommit {
    /**
     * The AWS::CodeCommit::Repository resource creates an AWS CodeCommit repository that is
    hosted by Amazon Web Services. For more information, see Create an AWS CodeCommit Repository in the
    AWS CodeCommit User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codecommit-repository.html
     */
    interface Repository {
        Type: 'AWS::CodeCommit::Repository';
        Properties: AWS.CodeCommit.Repository.Properties;
    }
    /**
     * The AWS::CodeCommit::Repository resource creates an AWS CodeCommit repository that is
    hosted by Amazon Web Services. For more information, see Create an AWS CodeCommit Repository in the
    AWS CodeCommit User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codecommit-repository.html
     */
    function Repository(props: C.Omit<Repository, 'Type'>): Repository;
}
export declare namespace AWS.CodeCommit.Repository {
    interface Properties {
        /**
         * String

         *
         * A name for the AWS CodeCommit repository.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codecommit-repository.html#cfn-codecommit-repository-repositoryname
         */
        RepositoryName: C.YieldsString;
        /**
         * List of AWS CodeCommit Repository Trigger
         *
         * Defines the actions to take in response to events that occur in the repository.
 For example, you can send email notifications when someone pushes to the
 repository.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codecommit-repository.html#cfn-codecommit-repository-triggers
         */
        Triggers?: C.Yields<C.CFList<C.Yields<AWS.CodeCommit.Repository.RepositoryTrigger>>>;
        /**
         * String

         *
         * A description about the AWS CodeCommit repository. For constraints, see the CreateRepository action in
 the AWS CodeCommit API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codecommit-repository.html#cfn-codecommit-repository-repositorydescription
         */
        RepositoryDescription?: C.YieldsString;
    }
}
export declare namespace AWS.CodeCommit.Repository {
    interface RepositoryTrigger {
        /**
         * List of strings

         *
         * The repository events for which AWS CodeCommit sends information to the target, which
 you
 specified in the DestinationArn property. If you don't specify events, the
 trigger runs for all repository events. For valid values, see the RepositoryTrigger data type in
 the AWS CodeCommit API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codecommit-repository-repositorytrigger.html#cfn-codecommit-repository-repositorytrigger-events
         */
        Events?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of strings

         *
         * The names of the branches in the AWS CodeCommit repository that contain events that
 you want
 to include in the trigger. If you don't specify at least one branch, the trigger applies
 to all branches.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codecommit-repository-repositorytrigger.html#cfn-codecommit-repository-repositorytrigger-branches
         */
        Branches?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * When an event is triggered, additional information that AWS CodeCommit includes when
 it sends
 information to the target.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codecommit-repository-repositorytrigger.html#cfn-codecommit-repository-repositorytrigger-customdata
         */
        CustomData?: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the resource that is the target for this trigger.
 For valid targets, see Manage Triggers for
    an AWS CodeCommit Repository in the AWS CodeCommit User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codecommit-repository-repositorytrigger.html#cfn-codecommit-repository-repositorytrigger-destinationarn
         */
        DestinationArn?: C.YieldsString;
        /**
         * String

         *
         * A name for the trigger.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codecommit-repository-repositorytrigger.html#cfn-codecommit-repository-repositorytrigger-name
         */
        Name?: C.YieldsString;
    }
}
export declare namespace AWS.CodeDeploy {
    /**
     * The AWS::CodeDeploy::Application resource creates an AWS CodeDeploy application. In AWS CodeDeploy, an application
    is a name that functions as a container
    to ensure that the correct combination of revision, deployment configuration, and
    deployment group are referenced during a deployment.
    You can use the AWS::CodeDeploy::DeploymentGroup resource to associate the application with an AWS CodeDeploy deployment group.
    For more information, see AWS CodeDeploy Deployments in the AWS CodeDeploy User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-application.html
     */
    interface Application {
        Type: 'AWS::CodeDeploy::Application';
        Properties: AWS.CodeDeploy.Application.Properties;
    }
    /**
     * The AWS::CodeDeploy::Application resource creates an AWS CodeDeploy application. In AWS CodeDeploy, an application
    is a name that functions as a container
    to ensure that the correct combination of revision, deployment configuration, and
    deployment group are referenced during a deployment.
    You can use the AWS::CodeDeploy::DeploymentGroup resource to associate the application with an AWS CodeDeploy deployment group.
    For more information, see AWS CodeDeploy Deployments in the AWS CodeDeploy User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-application.html
     */
    function Application(props: C.Omit<Application, 'Type'>): Application;
}
export declare namespace AWS.CodeDeploy.Application {
    interface Properties {
        /**
         * String

         *
         * A name for the application. If you don't specify a name, AWS CloudFormation generates
 a unique
 physical ID and uses that ID for the application name. For more information, see Name Type.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-application.html#cfn-codedeploy-application-applicationname
         */
        ApplicationName?: C.YieldsString;
    }
}
export declare namespace AWS.CodeDeploy {
    /**
     * The AWS::CodeDeploy::DeploymentConfig resource creates a set of deployment
    rules, deployment success conditions, and deployment failure conditions that AWS CodeDeploy
    uses during a
    deployment. The deployment configuration specifies, through the use of a MinimumHealthyHosts value,
    the number or percentage of instances that must remain available at any time during
    a deployment.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-deploymentconfig.html
     */
    interface DeploymentConfig {
        Type: 'AWS::CodeDeploy::DeploymentConfig';
        Properties: AWS.CodeDeploy.DeploymentConfig.Properties;
    }
    /**
     * The AWS::CodeDeploy::DeploymentConfig resource creates a set of deployment
    rules, deployment success conditions, and deployment failure conditions that AWS CodeDeploy
    uses during a
    deployment. The deployment configuration specifies, through the use of a MinimumHealthyHosts value,
    the number or percentage of instances that must remain available at any time during
    a deployment.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-deploymentconfig.html
     */
    function DeploymentConfig(props: C.Omit<DeploymentConfig, 'Type'>): DeploymentConfig;
}
export declare namespace AWS.CodeDeploy.DeploymentConfig {
    interface Properties {
        /**
         * String

         *
         * A name for the deployment configuration. If you don't specify a name, AWS CloudFormation
 generates a unique physical ID and uses that ID for the deployment configuration
 name. For more information, see Name Type.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-deploymentconfig.html#cfn-codedeploy-deploymentconfig-deploymentconfigname
         */
        DeploymentConfigName?: C.YieldsString;
        /**
         * AWS CodeDeploy
    DeploymentConfig MinimumHealthyHosts
         *
         * The minimum number of healthy instances that must be available at any time
 during an AWS CodeDeploy deployment. For example, for a fleet of nine instances, if
 you
 specify a minimum of six healthy instances, AWS CodeDeploy deploys your application
 up to
 three instances at a time so that you always have six healthy instances. The
 deployment succeeds if your application successfully deploys to six or more
 instances; otherwise, the deployment fails.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-deploymentconfig.html#cfn-codedeploy-deploymentconfig-minimumhealthyhosts
         */
        MinimumHealthyHosts?: C.Yields<AWS.CodeDeploy.DeploymentConfig.MinimumHealthyHosts>;
    }
}
export declare namespace AWS.CodeDeploy.DeploymentConfig {
    interface MinimumHealthyHosts {
        /**
         * String

         *
         * The type of count to use, such as an absolute value or a percentage of the
 total number of instances in the deployment. For valid values, see MinimumHealthyHosts in
 the AWS CodeDeploy API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentconfig-minimumhealthyhosts.html#cfn-codedeploy-deploymentconfig-minimumhealthyhosts-type
         */
        Type: C.YieldsString;
        /**
         * Integer

         *
         * The minimum number of healthy instances.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentconfig-minimumhealthyhosts.html#cfn-codedeploy-deploymentconfig-minimumhealthyhosts-value
         */
        Value: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.CodeDeploy {
    /**
     * The AWS::CodeDeploy::DeploymentGroup resource creates an AWS CodeDeploy deployment
    group that specifies which instances your application revisions are deployed to, along
    with other deployment options.
    For more information, see CreateDeploymentGroup in the AWS CodeDeploy API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-deploymentgroup.html
     */
    interface DeploymentGroup {
        Type: 'AWS::CodeDeploy::DeploymentGroup';
        Properties: AWS.CodeDeploy.DeploymentGroup.Properties;
    }
    /**
     * The AWS::CodeDeploy::DeploymentGroup resource creates an AWS CodeDeploy deployment
    group that specifies which instances your application revisions are deployed to, along
    with other deployment options.
    For more information, see CreateDeploymentGroup in the AWS CodeDeploy API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-deploymentgroup.html
     */
    function DeploymentGroup(props: C.Omit<DeploymentGroup, 'Type'>): DeploymentGroup;
}
export declare namespace AWS.CodeDeploy.DeploymentGroup {
    interface Properties {
        /**
         * AWS CodeDeploy DeploymentGroup AlarmConfiguration
         *
         * Information about the Amazon CloudWatch alarms that are associated with the deployment
 group.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-deploymentgroup.html#cfn-codedeploy-deploymentgroup-alarmconfiguration
         */
        AlarmConfiguration?: C.Yields<AWS.CodeDeploy.DeploymentGroup.AlarmConfiguration>;
        /**
         * String

         *
         * The name of an existing AWS CodeDeploy application to associate this deployment group
 with.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-deploymentgroup.html#cfn-codedeploy-deploymentgroup-applicationname
         */
        ApplicationName: C.YieldsString;
        /**
         *                             Type: AWS CodeDeploy DeploymentGroup AutoRollbackConfiguration
         *
         * Information about the automatic rollback configuration that is associated with the
 deployment group.
 If you specify this property, don't specify the Deployment property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-deploymentgroup.html#cfn-codedeploy-deploymentgroup-autorollbackconfiguration
         */
        AutoRollbackConfiguration?: C.Yields<AWS.CodeDeploy.DeploymentGroup.AutoRollbackConfiguration>;
        /**
         * List of strings

         *
         * A list of associated Auto Scaling groups that AWS CodeDeploy automatically deploys
 revisions to
 when new instances are created. Duplicates are not allowed.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-deploymentgroup.html#cfn-codedeploy-deploymentgroup-autoscalinggroups
         */
        AutoScalingGroups?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * AWS CodeDeploy DeploymentGroup
    Deployment
         *
         * The application revision to deploy to this deployment group. If you specify this property,
 your target application revision will be
 deployed as soon as the provisioning process is complete. If you specify this property,
 
 don't specify the AutoRollbackConfiguration property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-deploymentgroup.html#cfn-codedeploy-deploymentgroup-deployment
         */
        Deployment?: C.Yields<AWS.CodeDeploy.DeploymentGroup.Deployment>;
        /**
         * String

         *
         * A deployment configuration name or a predefined configuration name. With
 predefined configurations, you can deploy application revisions to one instance at
 a time, half of the instances at a time, or all the instances at once. For more
 information and valid values, see Working with Deployment Configurations
 in the AWS CodeDeploy User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-deploymentgroup.html#cfn-codedeploy-deploymentgroup-deploymentconfigname
         */
        DeploymentConfigName?: C.YieldsString;
        /**
         * String

         *
         * A name for the deployment group. If you don't specify a name, AWS CloudFormation generates
 a
 unique physical ID and uses that ID for the deployment group name. For more
 information, see Name Type.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-deploymentgroup.html#cfn-codedeploy-deploymentgroup-deploymentgroupname
         */
        DeploymentGroupName?: C.YieldsString;
        /**
         *                             Type: AWS CodeDeploy DeploymentGroup DeploymentStyle
         *
         * Attributes that determine the type of deployment,
 including specifying whether to route deployment traffic behind a load balancer for
 an AWS CodeDeploy deployment group.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-deploymentgroup.html#cfn-codedeploy-deploymentgroup-deploymentstyle
         */
        DeploymentStyle?: C.Yields<AWS.CodeDeploy.DeploymentGroup.DeploymentStyle>;
        /**
         * List of AWS CodeDeploy DeploymentGroup
    Ec2TagFilters
         *
         * The EC2 tags that are already applied to EC2 instances that you want to include in
 the deployment group. AWS CodeDeploy includes all
 EC2 instances identified by any of the tags you specify in this deployment group.
 Duplicates are not allowed.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-deploymentgroup.html#cfn-codedeploy-deploymentgroup-ec2tagfilters
         */
        Ec2TagFilters?: C.Yields<C.CFList<C.Yields<AWS.CodeDeploy.DeploymentGroup.EC2TagFilter>>>;
        /**
         *                             Type: AWS CodeDeploy DeploymentGroup LoadBalancerInfo
         *
         * Information about the load balancer to use in an in-place deployment. For more information,
 see
    Integrating AWS CodeDeploy with Elastic Load Balancing in the AWS CodeDeploy User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-deploymentgroup.html#cfn-codedeploy-deploymentgroup-loadbalancerinfo
         */
        LoadBalancerInfo?: C.Yields<AWS.CodeDeploy.DeploymentGroup.LoadBalancerInfo>;
        /**
         * List of AWS CodeDeploy
    DeploymentGroup OnPremisesInstanceTagFilters
         *
         * The on-premises instance tags already applied to on-premises instances that you want
 to include in the deployment group.
 AWS CodeDeploy includes all on-premises instances identified by any of the tags you
 specify in this deployment group. To register
 on-premises instances with AWS CodeDeploy, see Working with On-Premises Instances for AWS CodeDeploy in the
 AWS CodeDeploy User Guide. Duplicates are not allowed.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-deploymentgroup.html#cfn-codedeploy-deploymentgroup-onpremisesinstancetagfilters
         */
        OnPremisesInstanceTagFilters?: C.Yields<C.CFList<C.Yields<AWS.CodeDeploy.DeploymentGroup.TagFilter>>>;
        /**
         * String

         *
         * A service role Amazon Resource Name (ARN) that grants AWS CodeDeploy permission to
 make
 calls to AWS services on your behalf. For more information, see Create a Service Role for
    AWS CodeDeploy in the AWS CodeDeploy User Guide.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-deploymentgroup.html#cfn-codedeploy-deploymentgroup-servicerolearn
         */
        ServiceRoleArn: C.YieldsString;
        /**
         * List of AWS CodeDeploy DeploymentGroup TriggerConfig
         *
         * Information
 about the notification triggers for the deployment group.
 Duplicates are not allowed.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codedeploy-deploymentgroup.html#cfn-codedeploy-deploymentgroup-triggerconfigurations
         */
        TriggerConfigurations?: C.Yields<C.CFList<C.Yields<AWS.CodeDeploy.DeploymentGroup.TriggerConfig>>>;
    }
}
export declare namespace AWS.CodeDeploy.DeploymentGroup {
    interface Alarm {
        /**
         * String

         *
         * The
 name of the alarm. For more information, see Alarm in the AWS CodeDeploy API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-alarm.html#cfn-codedeploy-deploymentgroup-alarm-name
         */
        Name?: C.YieldsString;
    }
}
export declare namespace AWS.CodeDeploy.DeploymentGroup {
    interface AlarmConfiguration {
        /**
         * List of AWS CodeDeploy DeploymentGroup Alarm
         *
         * The list of alarms configured for the deployment group. Duplicates are not allowed.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-alarmconfiguration.html#cfn-codedeploy-deploymentgroup-alarmconfiguration-alarms
         */
        Alarms?: C.Yields<C.CFList<C.Yields<AWS.CodeDeploy.DeploymentGroup.Alarm>>>;
        /**
         * Boolean

         *
         * Indicates whether the
 alarm
 configuration is enabled.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-alarmconfiguration.html#cfn-codedeploy-deploymentgroup-alarmconfiguration-enabled
         */
        Enabled?: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Indicates whether a deployment should continue if
 information
 about the current state of alarms cannot be retrieved from
 CloudWatch. The default value is false.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-alarmconfiguration.html#cfn-codedeploy-deploymentgroup-alarmconfiguration-ignorepollalarmfailure
         */
        IgnorePollAlarmFailure?: C.YieldsBoolean;
    }
}
export declare namespace AWS.CodeDeploy.DeploymentGroup {
    interface AutoRollbackConfiguration {
        /**
         *                             Type: Boolean

         *
         * Indicates whether a defined automatic rollback configuration is currently enabled.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-autorollbackconfiguration.html#cfn-codedeploy-deploymentgroup-autorollbackconfiguration-enabled
         */
        Enabled?: C.YieldsBoolean;
        /**
         *                             Type: List of String

         *
         * The event type or types that trigger a rollback. Valid values are DEPLOYMENT_FAILURE,
 DEPLOYMENT_STOP_ON_ALARM, or DEPLOYMENT_STOP_ON_REQUEST.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-autorollbackconfiguration.html#cfn-codedeploy-deploymentgroup-autorollbackconfiguration-events
         */
        Events?: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.CodeDeploy.DeploymentGroup {
    interface Deployment {
        /**
         * String

         *
         * A description about this deployment.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-deployment.html#cfn-properties-codedeploy-deploymentgroup-deployment-description
         */
        Description?: C.YieldsString;
        /**
         * Boolean

         *
         * Whether to continue the deployment if the
 ApplicationStop deployment lifecycle event fails. If you
 want AWS CodeDeploy to continue the deployment lifecycle even if the
 ApplicationStop event fails on an instance, specify
 true. The deployment continues to the BeforeInstall
 deployment lifecycle event. If you want AWS CodeDeploy to stop deployment on the instance
 if the ApplicationStop event fails, specify false or do
 not specify a value.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-deployment.html#cfn-properties-codedeploy-deploymentgroup-deployment-ignoreapplicationstopfailures
         */
        IgnoreApplicationStopFailures?: C.YieldsBoolean;
        /**
         * AWS CodeDeploy
    DeploymentGroup Deployment Revision
         *
         * The location of the application revision to deploy.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-deployment.html#cfn-properties-codedeploy-deploymentgroup-deployment-revision
         */
        Revision: C.Yields<AWS.CodeDeploy.DeploymentGroup.RevisionLocation>;
    }
}
export declare namespace AWS.CodeDeploy.DeploymentGroup {
    interface DeploymentStyle {
        /**
         *                             Type: String

         *
         * Indicates whether to route deployment traffic behind a load balancer.
 For valid values, see
    DeploymentStyle in the AWS CodeDeploy API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-deploymentstyle.html#cfn-codedeploy-deploymentgroup-deploymentstyle-deploymentoption
         */
        DeploymentOption?: C.YieldsString;
    }
}
export declare namespace AWS.CodeDeploy.DeploymentGroup {
    interface EC2TagFilter {
        /**
         * String

         *
         * Filter instances with this key.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-ec2tagfilters.html#cfn-properties-codedeploy-deploymentgroup-ec2tagfilters-key
         */
        Key?: C.YieldsString;
        /**
         * String

         *
         * The filter type. For example, you can filter instances by the key, tag value,
 or both. For valid values, see EC2TagFilter in the AWS CodeDeploy API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-ec2tagfilters.html#cfn-properties-codedeploy-deploymentgroup-ec2tagfilters-type
         */
        Type?: C.YieldsString;
        /**
         * String

         *
         * Filter instances with this tag value.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-ec2tagfilters.html#cfn-properties-codedeploy-deploymentgroup-ec2tagfilters-value
         */
        Value?: C.YieldsString;
    }
}
export declare namespace AWS.CodeDeploy.DeploymentGroup {
    interface ELBInfo {
        /**
         *                             Type: String

         *
         * The name of the load balancer that instances are deregistered from so they are not
 serving traffic during a deployment,
 and then re-registered with after the deployment completes.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-elbinfo.html#cfn-codedeploy-deploymentgroup-elbinfo-name
         */
        Name?: C.YieldsString;
    }
}
export declare namespace AWS.CodeDeploy.DeploymentGroup {
    interface GitHubLocation {
        /**
         * String

         *
         * The SHA1 commit ID of the GitHub commit to use as your application
 revision.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-deployment-revision-githublocation.html#cfn-properties-codedeploy-deploymentgroup-deployment-revision-githublocation-commitid
         */
        CommitId: C.YieldsString;
        /**
         * String

         *
         * The GitHub account and repository name that includes the application revision.
 Specify the value as
 account/repository_name.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-deployment-revision-githublocation.html#cfn-properties-codedeploy-deploymentgroup-deployment-revision-githublocation-repository
         */
        Repository: C.YieldsString;
    }
}
export declare namespace AWS.CodeDeploy.DeploymentGroup {
    interface LoadBalancerInfo {
        /**
         *                             Type: List of AWS CodeDeploy DeploymentGroup ELBInfo
         *
         * Information about the Elastic Load Balancing load balancer to use in the deployment.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-loadbalancerinfo.html#cfn-codedeploy-deploymentgroup-loadbalancerinfo-elbinfolist
         */
        ElbInfoList?: C.Yields<C.CFList<C.Yields<AWS.CodeDeploy.DeploymentGroup.ELBInfo>>>;
    }
}
export declare namespace AWS.CodeDeploy.DeploymentGroup {
    interface RevisionLocation {
        /**
         * AWS CodeDeploy DeploymentGroup Deployment Revision GitHubLocation
         *
         * If your application revision is stored in GitHub, information about the
 location where it is stored.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-deployment-revision.html#cfn-properties-codedeploy-deploymentgroup-deployment-revision-githublocation
         */
        GitHubLocation?: C.Yields<AWS.CodeDeploy.DeploymentGroup.GitHubLocation>;
        /**
         * String

         *
         * The application revision's location, such as in an S3 bucket or GitHub
 repository. For valid values, see RevisionLocation in the
 AWS CodeDeploy API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-deployment-revision.html#cfn-properties-codedeploy-deploymentgroup-deployment-revision-revisiontype
         */
        RevisionType?: C.YieldsString;
        /**
         * AWS CodeDeploy
    DeploymentGroup Deployment Revision S3Location
         *
         * If the application revision is stored in an S3 bucket, information about the
 location.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-deployment-revision.html#cfn-properties-codedeploy-deploymentgroup-deployment-revision-s3location
         */
        S3Location?: C.Yields<AWS.CodeDeploy.DeploymentGroup.S3Location>;
    }
}
export declare namespace AWS.CodeDeploy.DeploymentGroup {
    interface S3Location {
        /**
         * String

         *
         * The name of the S3 bucket where the application revision is stored.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-deployment-revision-s3location.html#cfn-properties-codedeploy-deploymentgroup-deployment-revision-s3location-bucket
         */
        Bucket: C.YieldsString;
        /**
         * String

         *
         * The file type of the application revision, such as tar,
 tgz, or zip. For valid values, see S3Location in the
 AWS CodeDeploy API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-deployment-revision-s3location.html#cfn-properties-codedeploy-deploymentgroup-deployment-revision-s3location-bundletype
         */
        BundleType?: C.YieldsString;
        /**
         * String

         *
         * The Amazon S3 ETag (a file checksum) of the application revision. If you don't
 specify a value, AWS CodeDeploy skips the ETag validation of your application
 revision.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-deployment-revision-s3location.html#cfn-properties-codedeploy-deploymentgroup-deployment-revision-s3location-etag
         */
        ETag?: C.YieldsString;
        /**
         * String

         *
         * The file name of the application revision (Amazon S3 object name).
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-deployment-revision-s3location.html#cfn-properties-codedeploy-deploymentgroup-deployment-revision-s3location-key
         */
        Key: C.YieldsString;
        /**
         * String

         *
         * For versioning-enabled buckets, a specific version of the application
 revision.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-deployment-revision-s3location.html#cfn-properties-codedeploy-deploymentgroup-deployment-revision-s3location-value
         */
        Version?: C.YieldsString;
    }
}
export declare namespace AWS.CodeDeploy.DeploymentGroup {
    interface TagFilter {
        /**
         * String

         *
         * Filter on-premises instances with this key.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-onpremisesinstancetagfilters.html#cfn-properties-codedeploy-deploymentgroup-onpremisesinstancetagfilters-key
         */
        Key?: C.YieldsString;
        /**
         * String

         *
         * The filter type. For example, you can filter on-premises instances by the key,
 tag value, or both. For valid values, see EC2TagFilter in the
 AWS CodeDeploy API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-onpremisesinstancetagfilters.html#cfn-properties-codedeploy-deploymentgroup-onpremisesinstancetagfilters-type
         */
        Type?: C.YieldsString;
        /**
         * String

         *
         * Filter on-premises instances with this tag value.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-onpremisesinstancetagfilters.html#cfn-properties-codedeploy-deploymentgroup-onpremisesinstancetagfilters-value
         */
        Value?: C.YieldsString;
    }
}
export declare namespace AWS.CodeDeploy.DeploymentGroup {
    interface TriggerConfig {
        /**
         * List of strings

         *
         * The event type or types
 that
 trigger
 notifications.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-triggerconfig.html#cfn-codedeploy-deploymentgroup-triggerconfig-triggerevents
         */
        TriggerEvents?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The name of the notification trigger.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-triggerconfig.html#cfn-codedeploy-deploymentgroup-triggerconfig-triggername
         */
        TriggerName?: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the Amazon Simple Notification Service topic through
 which
 notifications about deployment
 or
 instance events are sent.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codedeploy-deploymentgroup-triggerconfig.html#cfn-codedeploy-deploymentgroup-triggerconfig-triggertargetarn
         */
        TriggerTargetArn?: C.YieldsString;
    }
}
export declare namespace AWS.CodePipeline {
    /**
     * The AWS::CodePipeline::CustomActionType resource creates a custom action for
    activities that aren't included in the AWS CodePipeline default actions, such as running
    an internally
    developed build process or a test suite. You can use these custom actions in the stage
    of a
    pipeline. For more information,
    see Create and Add a Custom Action
       in AWS CodePipeline in the AWS CodePipeline User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-customactiontype.html
     */
    interface CustomActionType {
        Type: 'AWS::CodePipeline::CustomActionType';
        Properties: AWS.CodePipeline.CustomActionType.Properties;
    }
    /**
     * The AWS::CodePipeline::CustomActionType resource creates a custom action for
    activities that aren't included in the AWS CodePipeline default actions, such as running
    an internally
    developed build process or a test suite. You can use these custom actions in the stage
    of a
    pipeline. For more information,
    see Create and Add a Custom Action
       in AWS CodePipeline in the AWS CodePipeline User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-customactiontype.html
     */
    function CustomActionType(props: C.Omit<CustomActionType, 'Type'>): CustomActionType;
}
export declare namespace AWS.CodePipeline.CustomActionType {
    interface Properties {
        /**
         * String

         *
         * The category of the custom action, such as a source action or a build action.
 For valid values, see CreateCustomActionType in the
 AWS CodePipeline API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-customactiontype.html#cfn-codepipeline-customactiontype-category
         */
        Category: C.YieldsString;
        /**
         * List of AWS CodePipeline
    CustomActionType ConfigurationProperties
         *
         * The configuration properties for the custom action.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-customactiontype.html#cfn-codepipeline-customactiontype-configurationproperties
         */
        ConfigurationProperties?: C.Yields<C.CFList<C.Yields<AWS.CodePipeline.CustomActionType.ConfigurationProperties>>>;
        /**
         * AWS CodePipeline
    CustomActionType ArtifactDetails
         *
         * The input artifact details for this custom action.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-customactiontype.html#cfn-codepipeline-customactiontype-inputartifactdetails
         */
        InputArtifactDetails: C.Yields<AWS.CodePipeline.CustomActionType.ArtifactDetails>;
        /**
         * AWS CodePipeline
    CustomActionType ArtifactDetails
         *
         * The output artifact details for this custom action.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-customactiontype.html#cfn-codepipeline-customactiontype-outputartifactdetails
         */
        OutputArtifactDetails: C.Yields<AWS.CodePipeline.CustomActionType.ArtifactDetails>;
        /**
         * String

         *
         * The name of the service provider that AWS CodePipeline uses for this custom action.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-customactiontype.html#cfn-codepipeline-customactiontype-provider
         */
        Provider: C.YieldsString;
        /**
         * AWS CodePipeline CustomActionType
    Settings
         *
         * URLs that provide users information about this custom action.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-customactiontype.html#cfn-codepipeline-customactiontype-settings
         */
        Settings?: C.Yields<AWS.CodePipeline.CustomActionType.Settings>;
        /**
         * String

         *
         * The version number of this custom action.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-customactiontype.html#cfn-codepipeline-customactiontype-version
         */
        Version?: C.YieldsString;
    }
}
export declare namespace AWS.CodePipeline.CustomActionType {
    interface ArtifactDetails {
        /**
         * Integer

         *
         * The maximum number of artifacts allowed for the action type.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-customactiontype-artifactdetails.html#cfn-codepipeline-customactiontype-artifactdetails-maximumcount
         */
        MaximumCount: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The minimum number of artifacts allowed for the action type.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-customactiontype-artifactdetails.html#cfn-codepipeline-customactiontype-artifactdetails-minimumcount
         */
        MinimumCount: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.CodePipeline.CustomActionType {
    interface ConfigurationProperties {
        /**
         * String

         *
         * A description of this configuration property that will be displayed to
 users.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-customactiontype-configurationproperties.html#cfn-codepipeline-customactiontype-configurationproperties-description
         */
        Description?: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether the configuration property is a key.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-customactiontype-configurationproperties.html#cfn-codepipeline-customactiontype-configurationproperties-key
         */
        Key: C.YieldsBoolean;
        /**
         * String

         *
         * A name for this configuration property.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-customactiontype-configurationproperties.html#cfn-codepipeline-customactiontype-configurationproperties-name
         */
        Name: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether the configuration property will be used with the
 PollForJobs call. A custom action can have one queryable property.
 The queryable property must be required (see the Required property)
 and must not be secret (see the Secret property). For more
 information, see the queryable contents for the ActionConfigurationProperty data type in the
 AWS CodePipeline API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-customactiontype-configurationproperties.html#cfn-codepipeline-customactiontype-configurationproperties-queryable
         */
        Queryable?: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Indicates whether the configuration property is a required value.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-customactiontype-configurationproperties.html#cfn-codepipeline-customactiontype-configurationproperties-required
         */
        Required: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Indicates whether the configuration property is secret. Secret configuration
 properties are hidden from all AWS CodePipeline calls except for GetJobDetails,
 GetThirdPartyJobDetails, PollForJobs, and
 PollForThirdPartyJobs.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-customactiontype-configurationproperties.html#cfn-codepipeline-customactiontype-configurationproperties-secret
         */
        Secret: C.YieldsBoolean;
        /**
         * String

         *
         * The type of the configuration property, such as String,
 Number, or Boolean.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-customactiontype-configurationproperties.html#cfn-codepipeline-customactiontype-configurationproperties-type
         */
        Type?: C.YieldsString;
    }
}
export declare namespace AWS.CodePipeline.CustomActionType {
    interface Settings {
        /**
         * String

         *
         * The URL that is returned to the AWS CodePipeline console that links to the resources
 of
 the external system, such as the configuration page for an AWS CodeDeploy deployment
 group.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-customactiontype-settings.html#cfn-codepipeline-customactiontype-settings-entityurltemplate
         */
        EntityUrlTemplate?: C.YieldsString;
        /**
         * String

         *
         * The URL that is returned to the AWS CodePipeline console that links to the top-level
 landing page for the external system, such as the console page for AWS CodeDeploy.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-customactiontype-settings.html#cfn-codepipeline-customactiontype-settings-executionurltemplate
         */
        ExecutionUrlTemplate?: C.YieldsString;
        /**
         * String

         *
         * The URL that is returned to the AWS CodePipeline console that links to the page where
 customers can update or change the configuration of the external action.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-customactiontype-settings.html#cfn-codepipeline-customactiontype-settings-revisionurltemplate
         */
        RevisionUrlTemplate?: C.YieldsString;
        /**
         * String

         *
         * The URL of a sign-up page where users can sign up for an external service and
 specify the initial configurations for the service's action.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-customactiontype-settings.html#cfn-codepipeline-customactiontype-settings-thirdpartyconfigurationurl
         */
        ThirdPartyConfigurationUrl?: C.YieldsString;
    }
}
export declare namespace AWS.CodePipeline {
    /**
     * The AWS::CodePipeline::Pipeline resource creates an AWS CodePipeline pipeline that
    describes how software changes go through a release process. For more information,
    see What Is AWS CodePipeline? in the
    AWS CodePipeline User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-pipeline.html
     */
    interface Pipeline {
        Type: 'AWS::CodePipeline::Pipeline';
        Properties: AWS.CodePipeline.Pipeline.Properties;
    }
    /**
     * The AWS::CodePipeline::Pipeline resource creates an AWS CodePipeline pipeline that
    describes how software changes go through a release process. For more information,
    see What Is AWS CodePipeline? in the
    AWS CodePipeline User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-pipeline.html
     */
    function Pipeline(props: C.Omit<Pipeline, 'Type'>): Pipeline;
}
export declare namespace AWS.CodePipeline.Pipeline {
    interface Properties {
        /**
         * AWS CodePipeline Pipeline
    ArtifactStore
         *
         * The Amazon Simple Storage Service (Amazon S3) location where AWS CodePipeline stores
 pipeline artifacts. For more
 information, see Create an Amazon S3 Bucket
    for Your Application in the AWS CodePipeline User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-pipeline.html#cfn-codepipeline-pipeline-artifactstore
         */
        ArtifactStore: C.Yields<AWS.CodePipeline.Pipeline.ArtifactStore>;
        /**
         * List of AWS CodePipeline
    Pipeline DisableInboundStageTransitions
         *
         * Prevents artifacts in a pipeline from transitioning to the stage that you
 specified. This enables you to manually control transitions.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-pipeline.html#cfn-codepipeline-pipeline-disableinboundstagetransitions
         */
        DisableInboundStageTransitions?: C.Yields<C.CFList<C.Yields<AWS.CodePipeline.Pipeline.StageTransition>>>;
        /**
         * String

         *
         * The name of your AWS CodePipeline pipeline.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-pipeline.html#cfn-codepipeline-pipeline-name
         */
        Name?: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether to rerun the AWS CodePipeline pipeline after you update it.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-pipeline.html#cfn-codepipeline-pipeline-restartexecutiononupdate
         */
        RestartExecutionOnUpdate?: C.YieldsBoolean;
        /**
         * String

         *
         * A service role Amazon Resource Name (ARN) that grants AWS CodePipeline permission
 to
 make calls to AWS services on your behalf. For more information, see AWS CodePipeline Access Permissions
    Reference in the AWS CodePipeline User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-pipeline.html#cfn-codepipeline-pipeline-rolearn
         */
        RoleArn: C.YieldsString;
        /**
         * AWS CodePipeline Pipeline Stages
         *
         * Defines the AWS CodePipeline pipeline stages.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-pipeline.html#cfn-codepipeline-pipeline-stages
         */
        Stages: C.Yields<C.CFList<C.Yields<AWS.CodePipeline.Pipeline.StageDeclaration>>>;
    }
}
export declare namespace AWS.CodePipeline.Pipeline {
    interface ActionDeclaration {
        /**
         * AWS CodePipeline Pipeline
    Stages Actions ActionTypeId
         *
         * Specifies the action type and the provider of the action.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages-actions.html#cfn-codepipeline-pipeline-stages-actions-actiontypeid
         */
        ActionTypeId: C.Yields<AWS.CodePipeline.Pipeline.ActionTypeId>;
        /**
         * JSON object

         *
         * The action's configuration. These are key-value pairs that specify input values
 for an action. For more information, see
    Action Structure Requirements in AWS CodePipeline in the AWS CodePipeline User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages-actions.html#cfn-codepipeline-pipeline-stages-actions-configuration
         */
        Configuration?: C.Yields<C.CFJson>;
        /**
         * List of AWS CodePipeline
    Pipeline Stages Actions InputArtifacts
         *
         * The name or ID of the artifact that the action consumes, such as a test or
 build artifact.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages-actions.html#cfn-codepipeline-pipeline-stages-actions-inputartifacts
         */
        InputArtifacts?: C.Yields<C.CFList<C.Yields<AWS.CodePipeline.Pipeline.InputArtifact>>>;
        /**
         * String

         *
         * The action name.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages-actions.html#cfn-codepipeline-pipeline-stages-actions-name
         */
        Name: C.YieldsString;
        /**
         * List of AWS CodePipeline
    Pipeline Stages Actions OutputArtifacts
         *
         * The artifact name or ID that is a result of the action, such as a test or build
 artifact.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages-actions.html#cfn-codepipeline-pipeline-stages-actions-outputartifacts
         */
        OutputArtifacts?: C.Yields<C.CFList<C.Yields<AWS.CodePipeline.Pipeline.OutputArtifact>>>;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of a service role that the action uses. The
 pipeline's role assumes this role.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages-actions.html#cfn-codepipeline-pipeline-stages-actions-rolearn
         */
        RoleArn?: C.YieldsString;
        /**
         * Integer

         *
         * The order in which AWS CodePipeline runs this action.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages-actions.html#cfn-codepipeline-pipeline-stages-actions-runorder
         */
        RunOrder?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.CodePipeline.Pipeline {
    interface ActionTypeId {
        /**
         * String

         *
         * A category that defines which action type the owner (the entity that performs
 the action) performs. The category that you select determine the providers that
 you can specify for the Provider property. For valid values, see
 ActionTypeId in the
 AWS CodePipeline API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages-actions-actiontypeid.html#cfn-codepipeline-pipeline-stages-actions-actiontypeid-category
         */
        Category: C.YieldsString;
        /**
         * String

         *
         * The entity that performs the action. For valid values, see ActionTypeId in the
 AWS CodePipeline API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages-actions-actiontypeid.html#cfn-codepipeline-pipeline-stages-actions-actiontypeid-owner
         */
        Owner: C.YieldsString;
        /**
         * String

         *
         * The service provider that the action calls. The providers that you can specify
 are determined by the category that you select. For example, a valid provider for
 the Deploy category is AWS CodeDeploy, which you would specify as
 CodeDeploy.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages-actions-actiontypeid.html#cfn-codepipeline-pipeline-stages-actions-actiontypeid-provider
         */
        Provider: C.YieldsString;
        /**
         * String

         *
         * A version identifier for this action.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages-actions-actiontypeid.html#cfn-codepipeline-pipeline-stages-actions-actiontypeid-version
         */
        Version: C.YieldsString;
    }
}
export declare namespace AWS.CodePipeline.Pipeline {
    interface ArtifactStore {
        /**
         * AWS CodePipeline Pipeline
    ArtifactStore EncryptionKey
         *
         * The encryption key AWS CodePipeline uses to encrypt the data in the artifact store,
 such
 as an AWS Key Management Service (AWS KMS) key. If you don't specify a key, AWS CodePipeline
 uses the default
 key for Amazon Simple Storage Service (Amazon S3).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-artifactstore.html#cfn-codepipeline-pipeline-artifactstore-encryptionkey
         */
        EncryptionKey?: C.Yields<AWS.CodePipeline.Pipeline.EncryptionKey>;
        /**
         * String

         *
         * The location where AWS CodePipeline stores artifacts for a pipeline, such as an S3
 bucket.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-artifactstore.html#cfn-codepipeline-pipeline-artifactstore-location
         */
        Location: C.YieldsString;
        /**
         * String

         *
         * The type of the artifact store, such as Amazon S3. For valid values, see ArtifactStore in the
 AWS CodePipeline API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-artifactstore.html#cfn-codepipeline-pipeline-artifactstore-type
         */
        Type: C.YieldsString;
    }
}
export declare namespace AWS.CodePipeline.Pipeline {
    interface BlockerDeclaration {
        /**
         * String

         *
         * The name of the gate declaration.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages-blockers.html#cfn-codepipeline-pipeline-stages-blockers-name
         */
        Name: C.YieldsString;
        /**
         * String

         *
         * The type of gate declaration. For valid values, see BlockerDeclaration in
 the AWS CodePipeline API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages-blockers.html#cfn-codepipeline-pipeline-stages-blockers-type
         */
        Type: C.YieldsString;
    }
}
export declare namespace AWS.CodePipeline.Pipeline {
    interface EncryptionKey {
        /**
         * String

         *
         * The ID of the key. For an AWS KMS key, specify the key ID or key Amazon Resource
 Number (ARN).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-artifactstore-encryptionkey.html#cfn-codepipeline-pipeline-artifactstore-encryptionkey-id
         */
        Id: C.YieldsString;
        /**
         * String

         *
         * The type of encryption key, such as KMS. For valid values, see
 EncryptionKey in the
 AWS CodePipeline API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-artifactstore-encryptionkey.html#cfn-codepipeline-pipeline-artifactstore-encryptionkey-type
         */
        Type: C.YieldsString;
    }
}
export declare namespace AWS.CodePipeline.Pipeline {
    interface InputArtifact {
        /**
         * String

         *
         * The name of the artifact that the AWS CodePipeline action works on, such as My
    App.The input artifact of an action must match the output artifact from
 any preceding action.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages-actions-inputartifacts.html#cfn-codepipeline-pipeline-stages-actions-inputartifacts-name
         */
        Name: C.YieldsString;
    }
}
export declare namespace AWS.CodePipeline.Pipeline {
    interface OutputArtifact {
        /**
         * String

         *
         * The name of the artifact that is the result of an AWS CodePipeline action, such as
 My App. Output artifact names must be unique within a
 pipeline.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages-actions-outputartifacts.html#cfn-codepipeline-pipeline-stages-actions-outputartifacts-name
         */
        Name: C.YieldsString;
    }
}
export declare namespace AWS.CodePipeline.Pipeline {
    interface StageDeclaration {
        /**
         * List of AWS CodePipeline Pipeline Stages
    Actions
         *
         * The actions to include in this stage.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages.html#cfn-codepipeline-pipeline-stages-actions
         */
        Actions: C.Yields<C.CFList<C.Yields<AWS.CodePipeline.Pipeline.ActionDeclaration>>>;
        /**
         * List of AWS CodePipeline Pipeline Stages
    Blockers
         *
         * The gates included in a stage.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages.html#cfn-codepipeline-pipeline-stages-blockers
         */
        Blockers?: C.Yields<C.CFList<C.Yields<AWS.CodePipeline.Pipeline.BlockerDeclaration>>>;
        /**
         * String

         *
         * A name for this stage.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-stages.html#cfn-codepipeline-pipeline-stages-name
         */
        Name: C.YieldsString;
    }
}
export declare namespace AWS.CodePipeline.Pipeline {
    interface StageTransition {
        /**
         * String

         *
         * An explanation of why the transition between two stages of a pipeline was
 disabled.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-disableinboundstagetransitions.html#cfn-codepipeline-pipeline-disableinboundstagetransitions-reason
         */
        Reason: C.YieldsString;
        /**
         * String

         *
         * The name of the stage to which transitions are disabled.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-codepipeline-pipeline-disableinboundstagetransitions.html#cfn-codepipeline-pipeline-disableinboundstagetransitions-stagename
         */
        StageName: C.YieldsString;
    }
}
export declare namespace AWS.Cognito {
    /**
     * The AWS::Cognito::IdentityPool resource creates an Amazon Cognito
    identity
    pool.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-identitypool.html
     */
    interface IdentityPool {
        Type: 'AWS::Cognito::IdentityPool';
        Properties: AWS.Cognito.IdentityPool.Properties;
    }
    /**
     * The AWS::Cognito::IdentityPool resource creates an Amazon Cognito
    identity
    pool.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-identitypool.html
     */
    function IdentityPool(props: C.Omit<IdentityPool, 'Type'>): IdentityPool;
}
export declare namespace AWS.Cognito.IdentityPool {
    interface Properties {
        /**
         * Amazon Cognito IdentityPool PushSync
         *
         * Configuration options to be applied to the identity pool.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-identitypool.html#cfn-cognito-identitypool-pushsync
         */
        PushSync?: C.Yields<AWS.Cognito.IdentityPool.PushSync>;
        /**
         * An array of Amazon Cognito IdentityPool CognitoIdentityProvider.

         *
         * An array of Amazon Cognito user pools and their client IDs.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-identitypool.html#cfn-cognito-identitypool-cognitoidentityproviders
         */
        CognitoIdentityProviders?: C.Yields<C.CFList<C.Yields<AWS.Cognito.IdentityPool.CognitoIdentityProvider>>>;
        /**
         *
         *
         * The events to configure.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-identitypool.html#cfn-cognito-identitypool-cognitoevents
         */
        CognitoEvents?: C.Yields<C.CFJson>;
        /**
         * String

         *
         * The "domain" by which Amazon Cognito will refer to your users. This name acts as a
 placeholder that allows your backend and the Amazon Cognito service to communicate
 about the
 developer provider. For the DeveloperProviderName, you can use letters and
 periods (.), underscores (_), and dashes
 (-).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-identitypool.html#cfn-cognito-identitypool-developerprovidername
         */
        DeveloperProviderName?: C.YieldsString;
        /**
         * Amazon Cognito IdentityPool CognitoStreams
         *
         * Configuration options for configuring Amazon Cognito streams.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-identitypool.html#cfn-cognito-identitypool-cognitostreams
         */
        CognitoStreams?: C.Yields<AWS.Cognito.IdentityPool.CognitoStreams>;
        /**
         * String

         *
         * The name of your Amazon Cognito identity pool.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-identitypool.html#cfn-cognito-identitypool-identitypoolname
         */
        IdentityPoolName?: C.YieldsString;
        /**
         * Boolean

         *
         * Specifies whether the identity pool supports unauthenticated logins.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-identitypool.html#cfn-cognito-identitypool-allowunauthenticatedidentities
         */
        AllowUnauthenticatedIdentities: C.YieldsBoolean;
        /**
         * String to string map

         *
         * Key-value pairs that map provider names to provider app IDs.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-identitypool.html#cfn-cognito-identitypool-supportedloginproviders
         */
        SupportedLoginProviders?: C.Yields<C.CFJson>;
        /**
         * List of strings

         *
         * A list of Amazon Resource Names (ARNs) of Security Assertion Markup Language (SAML)
 providers.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-identitypool.html#cfn-cognito-identitypool-samlproviderarns
         */
        SamlProviderARNs?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of strings

         *
         * A list of  ARNs for the OpendID Connect provider.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-identitypool.html#cfn-cognito-identitypool-openidconnectproviderarns
         */
        OpenIdConnectProviderARNs?: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.Cognito.IdentityPool {
    interface CognitoIdentityProvider {
        /**
         * Boolean

         *
         * TRUE if server-side token validation is enabled for the identity provider’s token.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-identitypool-cognitoidentityprovider.html#cfn-cognito-identitypool-cognitoidentityprovider-serversidetokencheck
         */
        ServerSideTokenCheck?: C.YieldsBoolean;
        /**
         * String

         *
         * The provider name for an Amazon Cognito user pool. For example, cognito-idp.us-east-2.amazonaws.com/us-east-2_123456789.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-identitypool-cognitoidentityprovider.html#cfn-cognito-identitypool-cognitoidentityprovider-providername
         */
        ProviderName?: C.YieldsString;
        /**
         * String

         *
         * The client ID for the Amazon Cognito user pool.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-identitypool-cognitoidentityprovider.html#cfn-cognito-identitypool-cognitoidentityprovider-clientid
         */
        ClientId?: C.YieldsString;
    }
}
export declare namespace AWS.Cognito.IdentityPool {
    interface CognitoStreams {
        /**
         * String

         *
         * Status of the Cognito streams. Valid values are: ENABLED or DISABLED.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-identitypool-cognitostreams.html#cfn-cognito-identitypool-cognitostreams-streamingstatus
         */
        StreamingStatus?: C.YieldsString;
        /**
         * String

         *
         * The name of the Amazon Cognito stream to receive updates. This stream must be in the
 developer's account and in the same region as the identity pool.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-identitypool-cognitostreams.html#cfn-cognito-identitypool-cognitostreams-streamname
         */
        StreamName?: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the role Amazon Cognito can assume to publish to
 the stream. This role must grant access to Amazon Cognito (cognito-sync) to invoke
 PutRecord on your Amazon Cognito stream.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-identitypool-cognitostreams.html#cfn-cognito-identitypool-cognitostreams-rolearn
         */
        RoleArn?: C.YieldsString;
    }
}
export declare namespace AWS.Cognito.IdentityPool {
    interface PushSync {
        /**
         * List of strings

         *
         * List of Amazon SNS platform application ARNs that could be used by clients.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-identitypool-pushsync.html#cfn-cognito-identitypool-pushsync-applicationarns
         */
        ApplicationArns?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * An IAM role configured to allow Amazon Cognito to call SNS on behalf of the developer.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-identitypool-pushsync.html#cfn-cognito-identitypool-pushsync-rolearn
         */
        RoleArn?: C.YieldsString;
    }
}
export declare namespace AWS.Cognito {
    /**
     * The AWS::Cognito::IdentityPoolRoleAttachment resource manages the role configuration for an Amazon Cognito identity pool.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-identitypoolroleattachment.html
     */
    interface IdentityPoolRoleAttachment {
        Type: 'AWS::Cognito::IdentityPoolRoleAttachment';
        Properties: AWS.Cognito.IdentityPoolRoleAttachment.Properties;
    }
    /**
     * The AWS::Cognito::IdentityPoolRoleAttachment resource manages the role configuration for an Amazon Cognito identity pool.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-identitypoolroleattachment.html
     */
    function IdentityPoolRoleAttachment(props: C.Omit<IdentityPoolRoleAttachment, 'Type'>): IdentityPoolRoleAttachment;
}
export declare namespace AWS.Cognito.IdentityPoolRoleAttachment {
    interface Properties {
        /**
         * String to Amazon Cognito IdentityPoolRoleAttachment RoleMapping object map.

         *
         * How users for a specific identity provider are to mapped to roles. This is a string
 to RoleMapping object map. The string identifies the identity provider, for example,
 "graph.facebook.com" or "cognito-idp-east-1.amazonaws.com/us-east-1_abcdefghi:app_client_id"

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-identitypoolroleattachment.html#cfn-cognito-identitypoolroleattachment-rolemappings
         */
        RoleMappings?: C.Yields<C.CFJson>;
        /**
         * String

         *
         * An identity pool ID in the format REGION:GUID.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-identitypoolroleattachment.html#cfn-cognito-identitypoolroleattachment-identitypoolid
         */
        IdentityPoolId: C.YieldsString;
        /**
         *
         *
         * The map of roles associated with this pool. For a given role, the key will be either
 "authenticated" or "unauthenticated" and the value will be the Role ARN.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-identitypoolroleattachment.html#cfn-cognito-identitypoolroleattachment-roles
         */
        Roles?: C.Yields<C.CFJson>;
    }
}
export declare namespace AWS.Cognito.IdentityPoolRoleAttachment {
    interface MappingRule {
        /**
         * String

         *
         * The match condition that specifies how closely the claim value in the IdP token must
 match Value.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-identitypoolroleattachment-mappingrule.html#cfn-cognito-identitypoolroleattachment-mappingrule-matchtype
         */
        MatchType: C.YieldsString;
        /**
         * String

         *
         * A brief string that the claim must match, for example, "paid" or "yes."
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-identitypoolroleattachment-mappingrule.html#cfn-cognito-identitypoolroleattachment-mappingrule-value
         */
        Value: C.YieldsString;
        /**
         * String

         *
         * The claim name that must be present in the token, for example, "isAdmin" or "paid."
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-identitypoolroleattachment-mappingrule.html#cfn-cognito-identitypoolroleattachment-mappingrule-claim
         */
        Claim: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the role.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-identitypoolroleattachment-mappingrule.html#cfn-cognito-identitypoolroleattachment-mappingrule-rolearn
         */
        RoleARN: C.YieldsString;
    }
}
export declare namespace AWS.Cognito.IdentityPoolRoleAttachment {
    interface RoleMapping {
        /**
         * String

         *
         * The role mapping type. Token will use cognito:roles and cognito:preferred_role claims from the Amazon Cognito identity provider token to map groups to roles. Rules will attempt to match claims from the token to map to a role.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-identitypoolroleattachment-rolemapping.html#cfn-cognito-identitypoolroleattachment-rolemapping-type
         */
        Type: C.YieldsString;
        /**
         * String

         *
         * Specifies the action to be taken if either no rules match the claim value for the
 Rules type, or there is no cognito:preferred_role claim and there are multiple cognito:roles matches for the Token type. If you specify Token or Rules as the Type, AmbiguousRoleResolution
 is required.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-identitypoolroleattachment-rolemapping.html#cfn-cognito-identitypoolroleattachment-rolemapping-ambiguousroleresolution
         */
        AmbiguousRoleResolution?: C.YieldsString;
        /**
         * Amazon Cognito IdentityPoolRoleAttachment RoleMapping RulesConfiguration
         *
         * The rules to be used for mapping users to roles. If you specify Rules as the role
 mapping type, RulesConfiguration is required.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-identitypoolroleattachment-rolemapping.html#cfn-cognito-identitypoolroleattachment-rolemapping-rulesconfiguration
         */
        RulesConfiguration?: C.Yields<AWS.Cognito.IdentityPoolRoleAttachment.RulesConfigurationType>;
    }
}
export declare namespace AWS.Cognito.IdentityPoolRoleAttachment {
    interface RulesConfigurationType {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-identitypoolroleattachment-rulesconfigurationtype.html#cfn-cognito-identitypoolroleattachment-rulesconfigurationtype-rules
         */
        Rules: C.Yields<C.CFList<C.Yields<AWS.Cognito.IdentityPoolRoleAttachment.MappingRule>>>;
    }
}
export declare namespace AWS.Cognito {
    /**
     * The AWS::Cognito::UserPool resource creates an Amazon Cognito user pool.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpool.html
     */
    interface UserPool {
        Type: 'AWS::Cognito::UserPool';
        Properties: AWS.Cognito.UserPool.Properties;
    }
    /**
     * The AWS::Cognito::UserPool resource creates an Amazon Cognito user pool.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpool.html
     */
    function UserPool(props: C.Omit<UserPool, 'Type'>): UserPool;
}
export declare namespace AWS.Cognito.UserPool {
    interface Properties {
        /**
         * String to string map

         *
         * The cost allocation tags for the user pool. For more information, see Adding Cost Allocation Tags to Your User Pool in the Amazon Cognito Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpool.html#cfn-cognito-userpool-userpooltags
         */
        UserPoolTags?: C.Yields<C.CFJson>;
        /**
         * Amazon Cognito UserPool Policies
         *
         * The policies associated with the Amazon Cognito user pool.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpool.html#cfn-cognito-userpool-policies
         */
        Policies?: C.Yields<AWS.Cognito.UserPool.Policies>;
        /**
         * String

         *
         * Specifies multi-factor authentication (MFA) configuration details. Can be one of the
 following values:

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpool.html#cfn-cognito-userpool-mfaconfiguration
         */
        MfaConfiguration?: C.YieldsString;
        /**
         * List of SchemaAttribute
         *
         * A list of schema attributes for the new user pool. These attributes can be standard
 or custom attributes.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpool.html#cfn-cognito-userpool-schema
         */
        Schema?: C.Yields<C.CFList<C.Yields<AWS.Cognito.UserPool.SchemaAttribute>>>;
        /**
         * Amazon Cognito UserPool AdminCreateUserConfig
         *
         * The type of configuration for creating a new user profile.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpool.html#cfn-cognito-userpool-admincreateuserconfig
         */
        AdminCreateUserConfig?: C.Yields<AWS.Cognito.UserPool.AdminCreateUserConfig>;
        /**
         * String

         *
         * A string representing the SMS authentication message. Must contain {####} in the message.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpool.html#cfn-cognito-userpool-smsauthenticationmessage
         */
        SmsAuthenticationMessage?: C.YieldsString;
        /**
         * String

         *
         * A string used to name the user pool.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpool.html#cfn-cognito-userpool-userpoolname
         */
        UserPoolName?: C.YieldsString;
        /**
         * String

         *
         * A string representing the SMS verification message. Must contain {####} in the message.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpool.html#cfn-cognito-userpool-smsverificationmessage
         */
        SmsVerificationMessage?: C.YieldsString;
        /**
         * Amazon Cognito UserPool EmailConfiguration
         *
         * The email configuration.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpool.html#cfn-cognito-userpool-emailconfiguration
         */
        EmailConfiguration?: C.Yields<AWS.Cognito.UserPool.EmailConfiguration>;
        /**
         * Amazon Cognito UserPool SmsConfiguration
         *
         * The Short Message Service (SMS) configuration.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpool.html#cfn-cognito-userpool-smsconfiguration
         */
        SmsConfiguration?: C.Yields<AWS.Cognito.UserPool.SmsConfiguration>;
        /**
         * List of strings

         *
         * Attributes supported as an alias for this user pool. Possible values: phone_number, email, and/or preferred_username.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpool.html#cfn-cognito-userpool-aliasattributes
         */
        AliasAttributes?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * A string representing the email verification subject.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpool.html#cfn-cognito-userpool-emailverificationsubject
         */
        EmailVerificationSubject?: C.YieldsString;
        /**
         * Amazon Cognito UserPool LambdaConfig
         *
         * The AWS Lambda trigger configuration information for the Amazon Cognito user pool.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpool.html#cfn-cognito-userpool-lambdaconfig
         */
        LambdaConfig?: C.Yields<AWS.Cognito.UserPool.LambdaConfig>;
        /**
         * List of strings

         *
         * The attributes to be auto-verified. Possible values: email and/or phone_number.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpool.html#cfn-cognito-userpool-autoverifiedattributes
         */
        AutoVerifiedAttributes?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * Amazon Cognito UserPool DeviceConfiguration
         *
         * The type of configuration for the user pool's device tracking.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpool.html#cfn-cognito-userpool-deviceconfiguration
         */
        DeviceConfiguration?: C.Yields<AWS.Cognito.UserPool.DeviceConfiguration>;
        /**
         * String

         *
         * A string representing the email verification message. Must contain {####} in the description.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpool.html#cfn-cognito-userpool-emailverificationmessage
         */
        EmailVerificationMessage?: C.YieldsString;
    }
}
export declare namespace AWS.Cognito.UserPool {
    interface AdminCreateUserConfig {
        /**
         * Amazon Cognito UserPool InviteMessageTemplate
         *
         * The message template to be used for the welcome message to new users.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-admincreateuserconfig.html#cfn-cognito-userpool-admincreateuserconfig-invitemessagetemplate
         */
        InviteMessageTemplate?: C.Yields<AWS.Cognito.UserPool.InviteMessageTemplate>;
        /**
         * Number

         *
         * The user account expiration limit, in days, after which the account is no longer usable.
 To reset the account after that time limit, you must call AdminCreateUser again, specifying
 RESEND for the MessageAction parameter. The default value for this parameter is 7.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-admincreateuserconfig.html#cfn-cognito-userpool-admincreateuserconfig-unusedaccountvaliditydays
         */
        UnusedAccountValidityDays?: C.Yields<C.CFDouble>;
        /**
         * Boolean

         *
         * Set to True if only the administrator is allowed to create user profiles. Set to False
 if users can sign themselves up via an app.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-admincreateuserconfig.html#cfn-cognito-userpool-admincreateuserconfig-allowadmincreateuseronly
         */
        AllowAdminCreateUserOnly?: C.YieldsBoolean;
    }
}
export declare namespace AWS.Cognito.UserPool {
    interface DeviceConfiguration {
        /**
         * Boolean

         *
         * If true, a device is only remembered on user prompt.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-deviceconfiguration.html#cfn-cognito-userpool-deviceconfiguration-deviceonlyrememberedonuserprompt
         */
        DeviceOnlyRememberedOnUserPrompt?: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Indicates whether a challenge is required on a new device. Only applicable to a new
 device.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-deviceconfiguration.html#cfn-cognito-userpool-deviceconfiguration-challengerequiredonnewdevice
         */
        ChallengeRequiredOnNewDevice?: C.YieldsBoolean;
    }
}
export declare namespace AWS.Cognito.UserPool {
    interface EmailConfiguration {
        /**
         * String

         *
         * The REPLY-TO email address.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-emailconfiguration.html#cfn-cognito-userpool-emailconfiguration-replytoemailaddress
         */
        ReplyToEmailAddress?: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the email source.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-emailconfiguration.html#cfn-cognito-userpool-emailconfiguration-sourcearn
         */
        SourceArn?: C.YieldsString;
    }
}
export declare namespace AWS.Cognito.UserPool {
    interface InviteMessageTemplate {
        /**
         * String

         *
         * The message template for email messages.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-invitemessagetemplate.html#cfn-cognito-userpool-invitemessagetemplate-emailmessage
         */
        EmailMessage?: C.YieldsString;
        /**
         * String

         *
         * The message template for SMS messages.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-invitemessagetemplate.html#cfn-cognito-userpool-invitemessagetemplate-smsmessage
         */
        SMSMessage?: C.YieldsString;
        /**
         * String

         *
         * The subject line for email messages.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-invitemessagetemplate.html#cfn-cognito-userpool-invitemessagetemplate-emailsubject
         */
        EmailSubject?: C.YieldsString;
    }
}
export declare namespace AWS.Cognito.UserPool {
    interface LambdaConfig {
        /**
         * String

         *
         * Creates an authentication challenge.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-lambdaconfig.html#cfn-cognito-userpool-lambdaconfig-createauthchallenge
         */
        CreateAuthChallenge?: C.YieldsString;
        /**
         * String

         *
         * A pre-authentication AWS Lambda trigger.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-lambdaconfig.html#cfn-cognito-userpool-lambdaconfig-preauthentication
         */
        PreAuthentication?: C.YieldsString;
        /**
         * String

         *
         * Defines the authentication challenge.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-lambdaconfig.html#cfn-cognito-userpool-lambdaconfig-defineauthchallenge
         */
        DefineAuthChallenge?: C.YieldsString;
        /**
         * String

         *
         * A pre-registration AWS Lambda trigger.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-lambdaconfig.html#cfn-cognito-userpool-lambdaconfig-presignup
         */
        PreSignUp?: C.YieldsString;
        /**
         * String

         *
         * A post-authentication AWS Lambda trigger.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-lambdaconfig.html#cfn-cognito-userpool-lambdaconfig-postauthentication
         */
        PostAuthentication?: C.YieldsString;
        /**
         * String

         *
         * A post-confirmation AWS Lambda trigger.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-lambdaconfig.html#cfn-cognito-userpool-lambdaconfig-postconfirmation
         */
        PostConfirmation?: C.YieldsString;
        /**
         * String

         *
         * A custom Message AWS Lambda trigger.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-lambdaconfig.html#cfn-cognito-userpool-lambdaconfig-custommessage
         */
        CustomMessage?: C.YieldsString;
        /**
         * String

         *
         * Verifies the authentication challenge response.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-lambdaconfig.html#cfn-cognito-userpool-lambdaconfig-verifyauthchallengeresponse
         */
        VerifyAuthChallengeResponse?: C.YieldsString;
    }
}
export declare namespace AWS.Cognito.UserPool {
    interface NumberAttributeConstraints {
        /**
         * String

         *
         * The minimum value of an attribute that is of the number data type.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-numberattributeconstraints.html#cfn-cognito-userpool-numberattributeconstraints-minvalue
         */
        MinValue?: C.YieldsString;
        /**
         * String

         *
         * The maximum value of an attribute that is of the number data type.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-numberattributeconstraints.html#cfn-cognito-userpool-numberattributeconstraints-maxvalue
         */
        MaxValue?: C.YieldsString;
    }
}
export declare namespace AWS.Cognito.UserPool {
    interface PasswordPolicy {
        /**
         * Boolean

         *
         * In the password policy that you have set, refers to whether you have required users
 to use at least one number in their password.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-passwordpolicy.html#cfn-cognito-userpool-passwordpolicy-requirenumbers
         */
        RequireNumbers?: C.YieldsBoolean;
        /**
         * Integer

         *
         * The minimum length of the password policy that you have set. Cannot be less than 6.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-passwordpolicy.html#cfn-cognito-userpool-passwordpolicy-minimumlength
         */
        MinimumLength?: C.Yields<C.CFInteger>;
        /**
         * Boolean

         *
         * In the password policy that you have set, refers to whether you have required users
 to use at least one uppercase letter in their password.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-passwordpolicy.html#cfn-cognito-userpool-passwordpolicy-requireuppercase
         */
        RequireUppercase?: C.YieldsBoolean;
        /**
         * Boolean

         *
         * In the password policy that you have set, refers to whether you have required users
 to use at least one lowercase letter in their password.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-passwordpolicy.html#cfn-cognito-userpool-passwordpolicy-requirelowercase
         */
        RequireLowercase?: C.YieldsBoolean;
        /**
         * Boolean

         *
         * In the password policy that you have set, refers to whether you have required users
 to use at least one symbol in their password.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-passwordpolicy.html#cfn-cognito-userpool-passwordpolicy-requiresymbols
         */
        RequireSymbols?: C.YieldsBoolean;
    }
}
export declare namespace AWS.Cognito.UserPool {
    interface Policies {
        /**
         * Amazon Cognito UserPool PasswordPolicy
         *
         * Specifies information about the user pool password policy.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-policies.html#cfn-cognito-userpool-policies-passwordpolicy
         */
        PasswordPolicy?: C.Yields<AWS.Cognito.UserPool.PasswordPolicy>;
    }
}
export declare namespace AWS.Cognito.UserPool {
    interface SchemaAttribute {
        /**
         * Boolean

         *
         * Specifies whether the attribute type is developer only.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-schemaattribute.html#cfn-cognito-userpool-schemaattribute-developeronlyattribute
         */
        DeveloperOnlyAttribute?: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Specifies whether the attribute can be changed after it has been created. True means mutable and False means immutable.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-schemaattribute.html#cfn-cognito-userpool-schemaattribute-mutable
         */
        Mutable?: C.YieldsBoolean;
        /**
         * String

         *
         * The attribute data type. Can be one of the following: String, Number, DateTime, or Boolean.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-schemaattribute.html#cfn-cognito-userpool-schemaattribute-attributedatatype
         */
        AttributeDataType?: C.YieldsString;
        /**
         * Amazon Cognito UserPool StringAttributeConstraints
         *
         * Specifies the constraints for an attribute of the string type.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-schemaattribute.html#cfn-cognito-userpool-schemaattribute-stringattributeconstraints
         */
        StringAttributeConstraints?: C.Yields<AWS.Cognito.UserPool.StringAttributeConstraints>;
        /**
         * Boolean

         *
         * Specifies whether a user pool attribute is required. If the attribute is required
 and the user does not provide a value, registration or sign-in fails.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-schemaattribute.html#cfn-cognito-userpool-schemaattribute-required
         */
        Required?: C.YieldsBoolean;
        /**
         * Amazon Cognito UserPool NumberAttributeConstraints
         *
         * Specifies the constraints for an attribute of the number type.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-schemaattribute.html#cfn-cognito-userpool-schemaattribute-numberattributeconstraints
         */
        NumberAttributeConstraints?: C.Yields<AWS.Cognito.UserPool.NumberAttributeConstraints>;
        /**
         * String

         *
         * A schema attribute of the name type.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-schemaattribute.html#cfn-cognito-userpool-schemaattribute-name
         */
        Name?: C.YieldsString;
    }
}
export declare namespace AWS.Cognito.UserPool {
    interface SmsConfiguration {
        /**
         * String

         *
         * The external ID used in IAM role trust relationships.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-smsconfiguration.html#cfn-cognito-userpool-smsconfiguration-externalid
         */
        ExternalId?: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the Amazon Simple Notification Service (SNS) caller.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-smsconfiguration.html#cfn-cognito-userpool-smsconfiguration-snscallerarn
         */
        SnsCallerArn?: C.YieldsString;
    }
}
export declare namespace AWS.Cognito.UserPool {
    interface StringAttributeConstraints {
        /**
         * String

         *
         * The minimum value of an attribute that is of the string data type.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-stringattributeconstraints.html#cfn-cognito-userpool-stringattributeconstraints-minlength
         */
        MinLength?: C.YieldsString;
        /**
         * String

         *
         * The maximum value of an attribute that is of the string data type.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-stringattributeconstraints.html#cfn-cognito-userpool-stringattributeconstraints-maxlength
         */
        MaxLength?: C.YieldsString;
    }
}
export declare namespace AWS.Cognito {
    /**
     * The AWS::Cognito::UserPoolClient resource creates an Amazon Cognito user pool client.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpoolclient.html
     */
    interface UserPoolClient {
        Type: 'AWS::Cognito::UserPoolClient';
        Properties: AWS.Cognito.UserPoolClient.Properties;
    }
    /**
     * The AWS::Cognito::UserPoolClient resource creates an Amazon Cognito user pool client.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpoolclient.html
     */
    function UserPoolClient(props: C.Omit<UserPoolClient, 'Type'>): UserPoolClient;
}
export declare namespace AWS.Cognito.UserPoolClient {
    interface Properties {
        /**
         * Boolean

         *
         * Specifies whether you want to generate a secret for the user pool client being
 created.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpoolclient.html#cfn-cognito-userpoolclient-generatesecret
         */
        GenerateSecret?: C.YieldsBoolean;
        /**
         * String

         *
         * The client name for the user pool client that you want to create.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpoolclient.html#cfn-cognito-userpoolclient-clientname
         */
        ClientName?: C.YieldsString;
        /**
         * String

         *
         * The user pool ID for the user pool where you want to create a client.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpoolclient.html#cfn-cognito-userpoolclient-userpoolid
         */
        UserPoolId: C.YieldsString;
        /**
         * List of Strings

         *
         * The explicit authentication flows, which can be one of the following:
 ADMIN_NO_SRP_AUTH or CUSTOM_AUTH_FLOW_ONLY.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpoolclient.html#cfn-cognito-userpoolclient-explicitauthflows
         */
        ExplicitAuthFlows?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * Integer

         *
         * The time limit, in days, after which the refresh token is no longer valid.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpoolclient.html#cfn-cognito-userpoolclient-refreshtokenvalidity
         */
        RefreshTokenValidity?: C.Yields<C.CFDouble>;
        /**
         * List of Strings

         *
         * The read attributes.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpoolclient.html#cfn-cognito-userpoolclient-readattributes
         */
        ReadAttributes?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of Strings

         *
         * The write attributes.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpoolclient.html#cfn-cognito-userpoolclient-writeattributes
         */
        WriteAttributes?: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.Cognito {
    /**
     * The AWS::Cognito::UserPoolGroup resource creates a user group in an Amazon Cognito user pool.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpoolgroup.html
     */
    interface UserPoolGroup {
        Type: 'AWS::Cognito::UserPoolGroup';
        Properties: AWS.Cognito.UserPoolGroup.Properties;
    }
    /**
     * The AWS::Cognito::UserPoolGroup resource creates a user group in an Amazon Cognito user pool.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpoolgroup.html
     */
    function UserPoolGroup(props: C.Omit<UserPoolGroup, 'Type'>): UserPoolGroup;
}
export declare namespace AWS.Cognito.UserPoolGroup {
    interface Properties {
        /**
         * String

         *
         * The name of the user group. GroupName
 must
 be unique.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpoolgroup.html#cfn-cognito-userpoolgroup-groupname
         */
        GroupName?: C.YieldsString;
        /**
         * String

         *
         * A description of the user group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpoolgroup.html#cfn-cognito-userpoolgroup-description
         */
        Description?: C.YieldsString;
        /**
         * String

         *
         * The user pool ID.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpoolgroup.html#cfn-cognito-userpoolgroup-userpoolid
         */
        UserPoolId: C.YieldsString;
        /**
         * Number

         *
         * A nonnegative integer value that specifies the precedence of this group relative to
 the other groups that a user can belong to in the user pool. Zero is the highest
 Precedence value. Groups with lower Precedence values take
 precedence over groups with higher or null Precedence values. If a user
 belongs to two or more groups, the  role ARN of the group with the lowest precedence
 value is used in the cognito:roles and cognito:preferred_role
 claims in the user's tokens.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpoolgroup.html#cfn-cognito-userpoolgroup-precedence
         */
        Precedence?: C.Yields<C.CFDouble>;
        /**
         * String

         *
         * The role ARN for the group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpoolgroup.html#cfn-cognito-userpoolgroup-rolearn
         */
        RoleArn?: C.YieldsString;
    }
}
export declare namespace AWS.Cognito {
    /**
     * The AWS::Cognito::UserPoolUser resource creates an Amazon Cognito user pool user.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpooluser.html
     */
    interface UserPoolUser {
        Type: 'AWS::Cognito::UserPoolUser';
        Properties: AWS.Cognito.UserPoolUser.Properties;
    }
    /**
     * The AWS::Cognito::UserPoolUser resource creates an Amazon Cognito user pool user.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpooluser.html
     */
    function UserPoolUser(props: C.Omit<UserPoolUser, 'Type'>): UserPoolUser;
}
export declare namespace AWS.Cognito.UserPoolUser {
    interface Properties {
        /**
         * List of Amazon Cognito UserPoolUser AttributeType
         *
         * The user's validation data. This is a list of name-value pairs that contain user
 attributes and attribute values that you can use for custom validation, such as
 restricting the types of user accounts that can be registered. For example, you might
 choose to allow or disallow user sign-up based on the user's domain.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpooluser.html#cfn-cognito-userpooluser-validationdata
         */
        ValidationData?: C.Yields<C.CFList<C.Yields<AWS.Cognito.UserPoolUser.AttributeType>>>;
        /**
         * String

         *
         * The ID for the user pool where the user will be created.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpooluser.html#cfn-cognito-userpooluser-userpoolid
         */
        UserPoolId: C.YieldsString;
        /**
         * String

         *
         * The user name for the user. Username must be unique within the user
 pool. It must be a UTF-8 string between 1 and 128 characters. You can't change the
 username.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpooluser.html#cfn-cognito-userpooluser-username
         */
        Username?: C.YieldsString;
        /**
         * String

         *
         * Specifies the action you'd like to take for the message. Valid values are RESEND and SUPPRESS.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpooluser.html#cfn-cognito-userpooluser-messageaction
         */
        MessageAction?: C.YieldsString;
        /**
         * List of strings

         *
         * Specifies how the welcome message will be sent. For email, specify
 EMAIL. To use a phone number, specify SMS. You can specify
 more than one value. The default value is SMS.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpooluser.html#cfn-cognito-userpooluser-desireddeliverymediums
         */
        DesiredDeliveryMediums?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * Boolean

         *
         * Use this parameter only if the phone_number_verified attribute or the
 email_verified attribute is set to True. Otherwise, it is
 ignored. The default value is False.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpooluser.html#cfn-cognito-userpooluser-forcealiascreation
         */
        ForceAliasCreation?: C.YieldsBoolean;
        /**
         * List of Amazon Cognito UserPoolUser AttributeType
         *
         * A list of name-value pairs that contain user attributes and attribute values to be
 set for the user that you are creating. You can create a user without specifying any
 attributes other than Username. However, any attributes that you specify as
 required (in CreateUserPool or in the Attributes tab
 of the console) must be supplied either by you (in your call to
 AdminCreateUser) or by the user (when signing up in response to your
 welcome message).

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpooluser.html#cfn-cognito-userpooluser-userattributes
         */
        UserAttributes?: C.Yields<C.CFList<C.Yields<AWS.Cognito.UserPoolUser.AttributeType>>>;
    }
}
export declare namespace AWS.Cognito.UserPoolUser {
    interface AttributeType {
        /**
         * String

         *
         * The value of the attribute.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpooluser-attributetype.html#cfn-cognito-userpooluser-attributetype-value
         */
        Value?: C.YieldsString;
        /**
         * String

         *
         * The name of the attribute.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpooluser-attributetype.html#cfn-cognito-userpooluser-attributetype-name
         */
        Name?: C.YieldsString;
    }
}
export declare namespace AWS.Cognito {
    /**
     * The AWS::Cognito::UserPoolUserToGroupAttachment resource attaches a user to an Amazon Cognito user pool user group.
    
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpoolusertogroupattachment.html
     */
    interface UserPoolUserToGroupAttachment {
        Type: 'AWS::Cognito::UserPoolUserToGroupAttachment';
        Properties: AWS.Cognito.UserPoolUserToGroupAttachment.Properties;
    }
    /**
     * The AWS::Cognito::UserPoolUserToGroupAttachment resource attaches a user to an Amazon Cognito user pool user group.
    
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpoolusertogroupattachment.html
     */
    function UserPoolUserToGroupAttachment(props: C.Omit<UserPoolUserToGroupAttachment, 'Type'>): UserPoolUserToGroupAttachment;
}
export declare namespace AWS.Cognito.UserPoolUserToGroupAttachment {
    interface Properties {
        /**
         * String

         *
         * The name of the group.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpoolusertogroupattachment.html#cfn-cognito-userpoolusertogroupattachment-groupname
         */
        GroupName: C.YieldsString;
        /**
         * String

         *
         * The ID of the user pool.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpoolusertogroupattachment.html#cfn-cognito-userpoolusertogroupattachment-userpoolid
         */
        UserPoolId: C.YieldsString;
        /**
         * String

         *
         * The user's user name.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-cognito-userpoolusertogroupattachment.html#cfn-cognito-userpoolusertogroupattachment-username
         */
        Username: C.YieldsString;
    }
}
export declare namespace AWS.Config {
    /**
     * The AWS::Config::ConfigRule resource uses an AWS Lambda (Lambda) function that
    evaluates configuration items to assess whether your AWS resources comply with your
    specified
    configurations. This function can run when AWS Config detects a configuration change
    or delivers a
    configuration snapshot. The resources this function evaluates must be in the recording
    group.
    For more information, see Evaluating AWS Resource
       Configurations with AWS Config in the AWS Config Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-configrule.html
     */
    interface ConfigRule {
        Type: 'AWS::Config::ConfigRule';
        Properties: AWS.Config.ConfigRule.Properties;
    }
    /**
     * The AWS::Config::ConfigRule resource uses an AWS Lambda (Lambda) function that
    evaluates configuration items to assess whether your AWS resources comply with your
    specified
    configurations. This function can run when AWS Config detects a configuration change
    or delivers a
    configuration snapshot. The resources this function evaluates must be in the recording
    group.
    For more information, see Evaluating AWS Resource
       Configurations with AWS Config in the AWS Config Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-configrule.html
     */
    function ConfigRule(props: C.Omit<ConfigRule, 'Type'>): ConfigRule;
}
export declare namespace AWS.Config.ConfigRule {
    interface Properties {
        /**
         * String

         *
         * A name for the AWS Config rule. If you don't specify a name, AWS CloudFormation generates
 a unique
 physical ID and uses that ID for the rule name. For more information, see Name Type.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-configrule.html#cfn-config-configrule-configrulename
         */
        ConfigRuleName?: C.YieldsString;
        /**
         * String

         *
         * A description about this AWS Config rule.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-configrule.html#cfn-config-configrule-description
         */
        Description?: C.YieldsString;
        /**
         * JSON object

         *
         * Input parameter values that are passed to the AWS Config rule (Lambda function).
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-configrule.html#cfn-config-configrule-inputparameters
         */
        InputParameters?: C.Yields<C.CFJson>;
        /**
         * String

         *
         * The maximum frequency at which the AWS Config rule runs evaluations. For valid values,
 see
 the ConfigRule data type in the
 AWS Config API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-configrule.html#cfn-config-configrule-maximumexecutionfrequency
         */
        MaximumExecutionFrequency?: C.YieldsString;
        /**
         * AWS Config ConfigRule Scope
         *
         * Defines which AWS resources will trigger an evaluation when their configurations
 change. The scope can include one or more resource types, a combination of a tag key
 and
 value, or a combination of one resource type and one resource ID. Specify a scope
 to
 constrain the resources that are evaluated. If you don't specify a scope, the rule
 evaluates all resources in the recording group.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-configrule.html#cfn-config-configrule-scope
         */
        Scope?: C.Yields<AWS.Config.ConfigRule.Scope>;
        /**
         * AWS Config ConfigRule Source
         *
         * Specifies the rule owner, the rule identifier, and the events that cause the
 function to evaluate your AWS resources.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-configrule.html#cfn-config-configrule-source
         */
        Source: C.Yields<AWS.Config.ConfigRule.Source>;
    }
}
export declare namespace AWS.Config.ConfigRule {
    interface Scope {
        /**
         * String

         *
         * The ID of an AWS resource that you want AWS Config to evaluate against a rule. If
 you
 specify an ID, you must also specify a resource type for the
 ComplianceResourceTypes property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-config-configrule-scope.html#cfn-config-configrule-scope-complianceresourceid
         */
        ComplianceResourceId?: C.YieldsString;
        /**
         * List of strings

         *
         * The types of AWS resources that you want AWS Config to evaluate against the rule.
 If you specify the
 ComplianceResourceId property, specify only one resource type. For more information, see
 Supported Resources, Configuration Items, and Relationships.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-config-configrule-scope.html#cfn-config-configrule-scope-complianceresourcetypes
         */
        ComplianceResourceTypes?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The tag key that is applied to the AWS resources that you want AWS Config to evaluate
 against the rule.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-config-configrule-scope.html#cfn-config-configrule-scope-tagkey
         */
        TagKey?: C.YieldsString;
        /**
         * String

         *
         * The tag value that is applied to the AWS resources that you want AWS Config to evaluate
 against the rule.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-config-configrule-scope.html#cfn-config-configrule-scope-tagvalue
         */
        TagValue?: C.YieldsString;
    }
}
export declare namespace AWS.Config.ConfigRule {
    interface Source {
        /**
         * String

         *
         * Indicates who owns and manages the AWS Config rule. For valid values, see the Source data type in the
 AWS Config API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-config-configrule-source.html#cfn-config-configrule-source-owner
         */
        Owner: C.YieldsString;
        /**
         * List of AWS Config ConfigRule
    SourceDetails
         *
         * Provides the source and type of event that triggers AWS Config to evaluate your AWS
 resources.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-config-configrule-source.html#cfn-config-configrule-source-sourcedetails
         */
        SourceDetails?: C.Yields<C.CFList<C.Yields<AWS.Config.ConfigRule.SourceDetail>>>;
        /**
         * String

         *
         * For AWS managed rules, the identifier of the rule. For a list of identifiers, see
 AWS Managed
    Rules in the AWS Config Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-config-configrule-source.html#cfn-config-configrule-source-sourceidentifier
         */
        SourceIdentifier: C.YieldsString;
    }
}
export declare namespace AWS.Config.ConfigRule {
    interface SourceDetail {
        /**
         * String

         *
         * The source, such as an AWS service, that generate events, triggering AWS Config to
 evaluate your AWS resources.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-config-configrule-source-sourcedetails.html#cfn-config-configrule-source-sourcedetail-eventsource
         */
        EventSource: C.YieldsString;
        /**
         * String

         *
         * The frequency that you want AWS Config to run evaluations for a custom rule with a
 periodic trigger. By default, rules with a periodic trigger are evaluated every 24
 hours.
 If you specify a value for MaximumExecutionFrequency, then MessageType must use the ScheduledNotification value.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-config-configrule-source-sourcedetails.html#cfn-config-configrule-sourcedetail-maximumexecutionfrequency
         */
        MaximumExecutionFrequency?: C.YieldsString;
        /**
         * String

         *
         * The type of Amazon Simple Notification Service (Amazon SNS) message that triggers
 AWS Config to run an
 evaluation. For more information, see the SourceDetail data type in the
 AWS Config API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-config-configrule-source-sourcedetails.html#cfn-config-configrule-source-sourcedetail-messagetype
         */
        MessageType: C.YieldsString;
    }
}
export declare namespace AWS.Config {
    /**
     * The AWS::Config::ConfigurationRecorder resource describes the AWS resource
    types for which AWS Config records configuration changes. The configuration recorder
    stores the
    configurations of the supported resources in your account as configuration items.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-configurationrecorder.html
     */
    interface ConfigurationRecorder {
        Type: 'AWS::Config::ConfigurationRecorder';
        Properties: AWS.Config.ConfigurationRecorder.Properties;
    }
    /**
     * The AWS::Config::ConfigurationRecorder resource describes the AWS resource
    types for which AWS Config records configuration changes. The configuration recorder
    stores the
    configurations of the supported resources in your account as configuration items.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-configurationrecorder.html
     */
    function ConfigurationRecorder(props: C.Omit<ConfigurationRecorder, 'Type'>): ConfigurationRecorder;
}
export declare namespace AWS.Config.ConfigurationRecorder {
    interface Properties {
        /**
         * String

         *
         * A name for the configuration recorder. If you don't specify a name, AWS CloudFormation
 generates
 a unique physical ID and uses that ID for the configuration recorder name. For more
 information, see Name Type.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-configurationrecorder.html#cfn-config-configurationrecorder-name
         */
        Name?: C.YieldsString;
        /**
         * AWS Config
    ConfigurationRecorder RecordingGroup
         *
         * Indicates whether to record configurations for all supported resources or for a list
 of resource types. The resource types that you list must be supported by AWS Config.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-configurationrecorder.html#cfn-config-configurationrecorder-recordinggroup
         */
        RecordingGroup?: C.Yields<AWS.Config.ConfigurationRecorder.RecordingGroup>;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role
 that is used to make
 read or write requests to the delivery channel that you specify and to get configuration
 details for supported AWS resources. For more information, see Permissions for the IAM Role Assigned to
    AWS Config in the AWS Config Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-configurationrecorder.html#cfn-config-configurationrecorder-rolearn
         */
        RoleARN: C.YieldsString;
    }
}
export declare namespace AWS.Config.ConfigurationRecorder {
    interface RecordingGroup {
        /**
         * Boolean

         *
         * Indicates whether to record all supported resource types. If you specify this
 property, do not specify the ResourceTypes property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-config-configurationrecorder-recordinggroup.html#cfn-config-configurationrecorder-recordinggroup-allsupported
         */
        AllSupported?: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Indicates whether AWS Config records all supported global resource types. When AWS
 Config
 supports new global resource types, AWS Config will automatically start recording
 them if you
 enable this property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-config-configurationrecorder-recordinggroup.html#cfn-config-configurationrecorder-recordinggroup-includeglobalresourcetypes
         */
        IncludeGlobalResourceTypes?: C.YieldsBoolean;
        /**
         * List of strings

         *
         * A list of valid AWS resource types to include in this recording group, such as
 AWS::EC2::Instance or
 AWS::CloudTrail::Trail. If you specify this property, do not
 specify the AllSupported property. For a list of supported resource types,
 see Supported resource types in the AWS Config Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-config-configurationrecorder-recordinggroup.html#cfn-config-configurationrecorder-recordinggroup-resourcetypes
         */
        ResourceTypes?: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.Config {
    /**
     * The AWS::Config::DeliveryChannel resource describes where AWS Config sends
    notifications and updated configuration states for AWS resources.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-deliverychannel.html
     */
    interface DeliveryChannel {
        Type: 'AWS::Config::DeliveryChannel';
        Properties: AWS.Config.DeliveryChannel.Properties;
    }
    /**
     * The AWS::Config::DeliveryChannel resource describes where AWS Config sends
    notifications and updated configuration states for AWS resources.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-deliverychannel.html
     */
    function DeliveryChannel(props: C.Omit<DeliveryChannel, 'Type'>): DeliveryChannel;
}
export declare namespace AWS.Config.DeliveryChannel {
    interface Properties {
        /**
         * AWS Config
    DeliveryChannel ConfigSnapshotDeliveryProperties
         *
         * Provides options for how AWS Config delivers configuration snapshots to the S3 bucket
 in
 your delivery channel.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-deliverychannel.html#cfn-config-deliverychannel-configsnapshotdeliveryproperties
         */
        ConfigSnapshotDeliveryProperties?: C.Yields<AWS.Config.DeliveryChannel.ConfigSnapshotDeliveryProperties>;
        /**
         * String

         *
         * A name for the delivery channel. If you don't specify a name, AWS CloudFormation generates
 a
 unique physical ID and uses that ID for the delivery channel name. For more information,
 see Name Type.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-deliverychannel.html#cfn-config-deliverychannel-name
         */
        Name?: C.YieldsString;
        /**
         * String

         *
         * The name of an S3 bucket where you want to store configuration history for the
 delivery channel.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-deliverychannel.html#cfn-config-deliverychannel-s3bucketname
         */
        S3BucketName: C.YieldsString;
        /**
         * String

         *
         * A key prefix (folder) for the specified S3 bucket.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-deliverychannel.html#cfn-config-deliverychannel-s3keyprefix
         */
        S3KeyPrefix?: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the Amazon Simple Notification Service (Amazon SNS)
 topic that AWS Config delivers
 notifications to.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-config-deliverychannel.html#cfn-config-deliverychannel-snstopicarn
         */
        SnsTopicARN?: C.YieldsString;
    }
}
export declare namespace AWS.Config.DeliveryChannel {
    interface ConfigSnapshotDeliveryProperties {
        /**
         * String

         *
         * The frequency with which AWS Config delivers configuration snapshots. For valid
 values, see ConfigSnapshotDeliveryProperties in the
 AWS Config API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-config-deliverychannel-configsnapshotdeliveryproperties.html#cfn-config-deliverychannel-configsnapshotdeliveryproperties-deliveryfrequency
         */
        DeliveryFrequency?: C.YieldsString;
    }
}
export declare namespace AWS.DAX {
    /**
     * Use the AWS::DAX::Cluster resource to create a DAX cluster for use
    with Amazon DynamoDB.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-cluster.html
     */
    interface Cluster {
        Type: 'AWS::DAX::Cluster';
        Properties: AWS.DAX.Cluster.Properties;
    }
    /**
     * Use the AWS::DAX::Cluster resource to create a DAX cluster for use
    with Amazon DynamoDB.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-cluster.html
     */
    function Cluster(props: C.Omit<Cluster, 'Type'>): Cluster;
}
export declare namespace AWS.DAX.Cluster {
    interface Properties {
        /**
         * String

         *
         * A description of the cluster.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-cluster.html#cfn-dax-cluster-description
         */
        Description?: C.YieldsString;
        /**
         * Integer

         *
         * The number of nodes in the DAX cluster. A replication factor of 1 will create a
 single-node cluster, without any read replicas. For additional fault tolerance,
 you can create a multiple node cluster with one or more read replicas. To do this,
 set ReplicationFactor to 2 or more.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-cluster.html#cfn-dax-cluster-replicationfactor
         */
        ReplicationFactor: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The parameter group to be associated with the DAX cluster.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-cluster.html#cfn-dax-cluster-parametergroupname
         */
        ParameterGroupName?: C.YieldsString;
        /**
         * List of strings

         *
         * The Availability Zones (AZs) in which the cluster nodes will be created. All
 nodes belonging to the cluster are placed in these Availability Zones. Use this
 parameter if you want to distribute the nodes across multiple AZs.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-cluster.html#cfn-dax-cluster-availabilityzones
         */
        AvailabilityZones?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The compute and memory capacity of the nodes in the cluster.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-cluster.html#cfn-dax-cluster-nodetype
         */
        NodeType: C.YieldsString;
        /**
         * String

         *
         * A valid Amazon Resource Name (ARN) that identifies an IAM role. At runtime, DAX
 will assume this role and use the role's permissions to access DynamoDB on your
 behalf.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-cluster.html#cfn-dax-cluster-iamrolearn
         */
        IAMRoleARN: C.YieldsString;
        /**
         * String

         *
         * The name of the subnet group to be used for the replication group.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-cluster.html#cfn-dax-cluster-subnetgroupname
         */
        SubnetGroupName?: C.YieldsString;
        /**
         * String

         *
         * The cluster identifier. This parameter is stored as a lowercase string.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-cluster.html#cfn-dax-cluster-clustername
         */
        ClusterName?: C.YieldsString;
        /**
         * String

         *
         * Specifies the weekly time range during which maintenance on the DAX cluster is
 performed. It is specified as a range in the format ddd:hh24:mi-ddd:hh24:mi (24H
 Clock UTC). The minimum maintenance window is a 60 minute period. Valid values for
 ddd are:

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-cluster.html#cfn-dax-cluster-preferredmaintenancewindow
         */
        PreferredMaintenanceWindow?: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the Amazon SNS topic to which notifications
 will be sent.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-cluster.html#cfn-dax-cluster-notificationtopicarn
         */
        NotificationTopicARN?: C.YieldsString;
        /**
         * List of strings

         *
         * A list of security group IDs to be assigned to each node in the DAX cluster.
 (Each of the security group ID is system-generated.)

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-cluster.html#cfn-dax-cluster-securitygroupids
         */
        SecurityGroupIds?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String to string map

         *
         * A map of tags to associate with the DAX cluster.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-cluster.html#cfn-dax-cluster-tags
         */
        Tags?: C.Yields<C.CFJson>;
    }
}
export declare namespace AWS.DAX {
    /**
     * Use the AWS CloudFormation AWS::DAX::ParameterGroup resource to create a parameter group
    for use with Amazon DynamoDB.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-parametergroup.html
     */
    interface ParameterGroup {
        Type: 'AWS::DAX::ParameterGroup';
        Properties: AWS.DAX.ParameterGroup.Properties;
    }
    /**
     * Use the AWS CloudFormation AWS::DAX::ParameterGroup resource to create a parameter group
    for use with Amazon DynamoDB.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-parametergroup.html
     */
    function ParameterGroup(props: C.Omit<ParameterGroup, 'Type'>): ParameterGroup;
}
export declare namespace AWS.DAX.ParameterGroup {
    interface Properties {
        /**
         * String to string map

         *
         * A map of DAX parameter names and values.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-parametergroup.html#cfn-dax-parametergroup-parameternamevalues
         */
        ParameterNameValues?: C.Yields<C.CFJson>;
        /**
         * String

         *
         * A description of the parameter group.
 

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-parametergroup.html#cfn-dax-parametergroup-description
         */
        Description?: C.YieldsString;
        /**
         * String

         *
         * The name of the parameter group.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-parametergroup.html#cfn-dax-parametergroup-parametergroupname
         */
        ParameterGroupName?: C.YieldsString;
    }
}
export declare namespace AWS.DAX {
    /**
     * Use the AWS CloudFormation AWS::DAX::SubnetGroup resource to create a subnet group for use
    with DAX (DynamoDB Accelerator).
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-subnetgroup.html
     */
    interface SubnetGroup {
        Type: 'AWS::DAX::SubnetGroup';
        Properties: AWS.DAX.SubnetGroup.Properties;
    }
    /**
     * Use the AWS CloudFormation AWS::DAX::SubnetGroup resource to create a subnet group for use
    with DAX (DynamoDB Accelerator).
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-subnetgroup.html
     */
    function SubnetGroup(props: C.Omit<SubnetGroup, 'Type'>): SubnetGroup;
}
export declare namespace AWS.DAX.SubnetGroup {
    interface Properties {
        /**
         * String

         *
         * The description of the subnet group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-subnetgroup.html#cfn-dax-subnetgroup-description
         */
        Description?: C.YieldsString;
        /**
         * String

         *
         * The name of the subnet group.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-subnetgroup.html#cfn-dax-subnetgroup-subnetgroupname
         */
        SubnetGroupName?: C.YieldsString;
        /**
         * List of strings;

         *
         * A list of subnets associated with the subnet group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dax-subnetgroup.html#cfn-dax-subnetgroup-subnetids
         */
        SubnetIds: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.DMS {
    /**
     * The AWS::DMS::Certificate resource creates an SSL certificate that encrypts connections between AWS DMS endpoints
    and the replication instance.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-certificate.html
     */
    interface Certificate {
        Type: 'AWS::DMS::Certificate';
        Properties: AWS.DMS.Certificate.Properties;
    }
    /**
     * The AWS::DMS::Certificate resource creates an SSL certificate that encrypts connections between AWS DMS endpoints
    and the replication instance.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-certificate.html
     */
    function Certificate(props: C.Omit<Certificate, 'Type'>): Certificate;
}
export declare namespace AWS.DMS.Certificate {
    interface Properties {
        /**
         * String

         *
         * The customer-assigned name of the certificate. Valid characters are A-z and 0-9.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-certificate.html#cfn-dms-certificate-certificateidentifier
         */
        CertificateIdentifier?: C.YieldsString;
        /**
         * String

         *
         * The contents of the .pem X.509 certificate file for the certificate.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-certificate.html#cfn-dms-certificate-certificatepem
         */
        CertificatePem?: C.YieldsString;
        /**
         *
         *
         * The location of the imported Oracle Wallet certificate for use with SSL.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-certificate.html#cfn-dms-certificate-certificatewallet
         */
        CertificateWallet?: C.YieldsString;
    }
}
export declare namespace AWS.DMS {
    /**
     * The AWS::DMS::Endpoint resource creates an AWS DMS endpoint.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-endpoint.html
     */
    interface Endpoint {
        Type: 'AWS::DMS::Endpoint';
        Properties: AWS.DMS.Endpoint.Properties;
    }
    /**
     * The AWS::DMS::Endpoint resource creates an AWS DMS endpoint.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-endpoint.html
     */
    function Endpoint(props: C.Omit<Endpoint, 'Type'>): Endpoint;
}
export declare namespace AWS.DMS.Endpoint {
    interface Properties {
        /**
         * String

         *
         * The KMS key identifier that will be used to encrypt the connection parameters.
 If you do not specify a value for the KmsKeyId parameter, then AWS DMS will use
 your default encryption key. AWS KMS creates the default encryption key for your
 AWS account. Your AWS account has a different default encryption key for each AWS
 region.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-endpoint.html#cfn-dms-endpoint-kmskeyid
         */
        KmsKeyId?: C.YieldsString;
        /**
         * Integer

         *
         * The port used by the endpoint database.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-endpoint.html#cfn-dms-endpoint-port
         */
        Port?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The name of the endpoint database.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-endpoint.html#cfn-dms-endpoint-databasename
         */
        DatabaseName?: C.YieldsString;
        /**
         *
         *
         * Settings in JSON format for the target Amazon S3 endpoint. For more information about
 the available settings, see the Extra Connection
    Attributes section at
    Using Amazon S3 as a Target for AWS Database Migration Service in the AWS Database Migration Service User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-endpoint.html#cfn-dms-endpoint-s3settings
         */
        S3Settings?: C.Yields<AWS.DMS.Endpoint.S3Settings>;
        /**
         * String

         *
         * The type of engine for the endpoint. Valid values depend on the EndPointType
 and include MYSQL, ORACLE, POSTGRES, MARIADB, AURORA, REDSHIFT, S3, SYBASE,
 DYNAMODB, MONGODB, and SQLSERVER.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-endpoint.html#cfn-dms-endpoint-enginename
         */
        EngineName: C.YieldsString;
        /**
         *
         *
         * Settings in JSON format for the target DynamoDB endpoint. For more
 information about the available settings, see the Using
    Object Mapping to Migrate Data to DynamoDB section at  Using an Amazon DynamoDB Database as a Target for AWS Database Migration
    Service.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-endpoint.html#cfn-dms-endpoint-dynamodbsettings
         */
        DynamoDbSettings?: C.Yields<AWS.DMS.Endpoint.DynamoDbSettings>;
        /**
         * String

         *
         * The user name to be used to login to the endpoint database.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-endpoint.html#cfn-dms-endpoint-username
         */
        Username?: C.YieldsString;
        /**
         * String

         *
         * The SSL mode to use for the SSL connection.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-endpoint.html#cfn-dms-endpoint-sslmode
         */
        SslMode?: C.YieldsString;
        /**
         * String

         *
         * The name of the server where the endpoint database resides.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-endpoint.html#cfn-dms-endpoint-servername
         */
        ServerName?: C.YieldsString;
        /**
         * String

         *
         * Additional attributes associated with the connection.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-endpoint.html#cfn-dms-endpoint-extraconnectionattributes
         */
        ExtraConnectionAttributes?: C.YieldsString;
        /**
         * String

         *
         * The type of endpoint. Valid values are source and target.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-endpoint.html#cfn-dms-endpoint-endpointtype
         */
        EndpointType: C.YieldsString;
        /**
         * List of resource tags in key-value
 format

         *
         * The tags that you want to attach to the DMS endpoint.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-endpoint.html#cfn-dms-endpoint-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * String

         *
         * The database endpoint identifier. Identifiers must begin with a letter; must
 contain only ASCII letters, digits, and hyphens; and must not end with a hyphen or
 contain two consecutive hyphens.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-endpoint.html#cfn-dms-endpoint-endpointidentifier
         */
        EndpointIdentifier?: C.YieldsString;
        /**
         * String

         *
         * The password to be used to login to the endpoint database. Do not use this
 parameter directly. Use Password as an input parameter with noEcho as shown in
 the Parameters. For best practices information, see Do Not Embed Credentials in Your Templates.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-endpoint.html#cfn-dms-endpoint-password
         */
        Password?: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Number (ARN) for the certificate.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-endpoint.html#cfn-dms-endpoint-certificatearn
         */
        CertificateArn?: C.YieldsString;
        /**
         *
         *
         * Settings in JSON format for the source MongoDB endpoint. For more information
 about the available settings, see the Configuration
    Properties When Using MongoDB as a Source for AWS Database Migration
    Service section at
    Using Amazon S3 as a Target for AWS Database Migration Service.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-endpoint.html#cfn-dms-endpoint-mongodbsettings
         */
        MongoDbSettings?: C.Yields<AWS.DMS.Endpoint.MongoDbSettings>;
    }
}
export declare namespace AWS.DMS.Endpoint {
    interface DynamoDbSettings {
        /**
         * String

         *
         * The Amazon Resource Name (ARN) used by the service access IAM role.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dms-endpoint-dynamodbsettings.html#cfn-dms-endpoint-dynamodbsettings-serviceaccessrolearn
         */
        ServiceAccessRoleArn?: C.YieldsString;
    }
}
export declare namespace AWS.DMS.Endpoint {
    interface MongoDbSettings {
        /**
         * String

         *
         *  The authentication type you use to access the MongoDB source endpoint.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dms-endpoint-mongodbsettings.html#cfn-dms-endpoint-mongodbsettings-authsource
         */
        AuthSource?: C.YieldsString;
        /**
         * String

         *
         *  The authentication mechanism you use to access the MongoDB source endpoint.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dms-endpoint-mongodbsettings.html#cfn-dms-endpoint-mongodbsettings-authmechanism
         */
        AuthMechanism?: C.YieldsString;
        /**
         * String

         *
         * The user name you use to access the MongoDB source endpoint.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dms-endpoint-mongodbsettings.html#cfn-dms-endpoint-mongodbsettings-username
         */
        Username?: C.YieldsString;
        /**
         * String

         *
         *  Indicates the number of documents to preview to determine the document organization.
 
 Use this attribute when NestingLevel is set to ONE.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dms-endpoint-mongodbsettings.html#cfn-dms-endpoint-mongodbsettings-docstoinvestigate
         */
        DocsToInvestigate?: C.YieldsString;
        /**
         * String

         *
         * The name of the server on the MongoDB source endpoint.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dms-endpoint-mongodbsettings.html#cfn-dms-endpoint-mongodbsettings-servername
         */
        ServerName?: C.YieldsString;
        /**
         * Integer

         *
         * The port value for the MongoDB source endpoint.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dms-endpoint-mongodbsettings.html#cfn-dms-endpoint-mongodbsettings-port
         */
        Port?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         *  Specifies the document ID. Use this attribute when NestingLevel is set to NONE.
 
 Default value is false.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dms-endpoint-mongodbsettings.html#cfn-dms-endpoint-mongodbsettings-extractdocid
         */
        ExtractDocId?: C.YieldsString;
        /**
         * String

         *
         * The database name on the MongoDB source endpoint.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dms-endpoint-mongodbsettings.html#cfn-dms-endpoint-mongodbsettings-databasename
         */
        DatabaseName?: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dms-endpoint-mongodbsettings.html#cfn-dms-endpoint-mongodbsettings-authtype
         */
        AuthType?: C.YieldsString;
        /**
         * String

         *
         *  The password for the user account you use to access the MongoDB source endpoint.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dms-endpoint-mongodbsettings.html#cfn-dms-endpoint-mongodbsettings-password
         */
        Password?: C.YieldsString;
        /**
         * String

         *
         *  Specifies either document or table mode.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dms-endpoint-mongodbsettings.html#cfn-dms-endpoint-mongodbsettings-nestinglevel
         */
        NestingLevel?: C.YieldsString;
    }
}
export declare namespace AWS.DMS.Endpoint {
    interface S3Settings {
        /**
         * String

         *
         * The definition of the external table.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dms-endpoint-s3settings.html#cfn-dms-endpoint-s3settings-externaltabledefinition
         */
        ExternalTableDefinition?: C.YieldsString;
        /**
         * String

         *
         * The name of the Amazon S3 bucket.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dms-endpoint-s3settings.html#cfn-dms-endpoint-s3settings-bucketname
         */
        BucketName?: C.YieldsString;
        /**
         * String

         *
         * An optional parameter to set a folder name in the S3 bucket. If provided, tables are
 created in the path <bucketFolder>/<schema_name>/<table_name>/. If this
 parameter is not specified, then the path used is <schema_name>/<table_name>/.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dms-endpoint-s3settings.html#cfn-dms-endpoint-s3settings-bucketfolder
         */
        BucketFolder?: C.YieldsString;
        /**
         * String

         *
         *  The delimiter used to separate rows in the source files. The default is a carriage
 return (\n).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dms-endpoint-s3settings.html#cfn-dms-endpoint-s3settings-csvrowdelimiter
         */
        CsvRowDelimiter?: C.YieldsString;
        /**
         * String

         *
         *  The delimiter used to separate columns in the source files. The default is a comma.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dms-endpoint-s3settings.html#cfn-dms-endpoint-s3settings-csvdelimiter
         */
        CsvDelimiter?: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) used by the service access IAM role.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dms-endpoint-s3settings.html#cfn-dms-endpoint-s3settings-serviceaccessrolearn
         */
        ServiceAccessRoleArn?: C.YieldsString;
        /**
         * String

         *
         *  An optional parameter to use GZIP to compress the target files. Set to GZIP to
 compress the target files. Set to NONE (the default) or do not use to leave the files
 uncompressed.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dms-endpoint-s3settings.html#cfn-dms-endpoint-s3settings-compressiontype
         */
        CompressionType?: C.YieldsString;
    }
}
export declare namespace AWS.DMS {
    /**
     * Use the AWS::DMS::EventSubscription resource to get notifications for
    AWS Database Migration Service events through the Amazon Simple Notification Service.
    For more information, see Using AWS DMS Event Notification in the
    AWS Database Migration Service User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-eventsubscription.html
     */
    interface EventSubscription {
        Type: 'AWS::DMS::EventSubscription';
        Properties: AWS.DMS.EventSubscription.Properties;
    }
    /**
     * Use the AWS::DMS::EventSubscription resource to get notifications for
    AWS Database Migration Service events through the Amazon Simple Notification Service.
    For more information, see Using AWS DMS Event Notification in the
    AWS Database Migration Service User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-eventsubscription.html
     */
    function EventSubscription(props: C.Omit<EventSubscription, 'Type'>): EventSubscription;
}
export declare namespace AWS.DMS.EventSubscription {
    interface Properties {
        /**
         * String

         *
         * The type of source for which AWS DMS provides notification events.
 For example, if you want to be notified of events generated by a
 database instance, set this parameter to replication-instance.
 If you don't specify a value, notifications are provided for all
 source types. For valid values, see the SourceType
 parameter for
 the
    CreateEventSubscription action in the
 AWS Database Migration Service API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-eventsubscription.html#cfn-dms-eventsubscription-sourcetype
         */
        SourceType?: C.YieldsString;
        /**
         * List of strings

         *
         * A list of event categories that you want to subscribe
 to for a given source type. If you don't specify this
 property, you are notified about all event categories.
 For more information,
 see Using
    AWS DMS Event Notification in the AWS Database Migration Service User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-eventsubscription.html#cfn-dms-eventsubscription-eventcategories
         */
        EventCategories?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * Boolean

         *
         * Indicates whether to activate the subscription. If you don't specify this
 property, AWS CloudFormation activates the subscription.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-eventsubscription.html#cfn-dms-eventsubscription-enabled
         */
        Enabled?: C.YieldsBoolean;
        /**
         * List of strings

         *
         * The subscription name.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-eventsubscription.html#cfn-dms-eventsubscription-subscriptionname
         */
        SubscriptionName?: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of an Amazon SNS topic that you want to send event
 notifications to.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-eventsubscription.html#cfn-dms-eventsubscription-snstopicarn
         */
        SnsTopicArn: C.YieldsString;
        /**
         * List of strings

         *
         * A list of identifiers for which AWS DMS provides notification events.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-eventsubscription.html#cfn-dms-eventsubscription-sourceids
         */
        SourceIds?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of resource tags in key-value
 format

         *
         * The tags that you want to attach to the DMS event subscription.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-eventsubscription.html#cfn-dms-eventsubscription-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
    }
}
export declare namespace AWS.DMS {
    /**
     * The AWS::DMS::ReplicationInstance resource creates an AWS DMS replication instance.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationinstance.html
     */
    interface ReplicationInstance {
        Type: 'AWS::DMS::ReplicationInstance';
        Properties: AWS.DMS.ReplicationInstance.Properties;
    }
    /**
     * The AWS::DMS::ReplicationInstance resource creates an AWS DMS replication instance.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationinstance.html
     */
    function ReplicationInstance(props: C.Omit<ReplicationInstance, 'Type'>): ReplicationInstance;
}
export declare namespace AWS.DMS.ReplicationInstance {
    interface Properties {
        /**
         * String

         *
         * A name for the replication instance. If you specify a name, AWS CloudFormation
 converts it to lower case. If you don't specify a name, AWS CloudFormation
 generates a unique physical ID and uses that ID for the replication instance
 identifier. For more information, see Name Type.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationinstance.html#cfn-dms-replicationinstance-replicationinstanceidentifier
         */
        ReplicationInstanceIdentifier?: C.YieldsString;
        /**
         * String

         *
         * The engine version number of the replication instance.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationinstance.html#cfn-dms-replicationinstance-engineversion
         */
        EngineVersion?: C.YieldsString;
        /**
         * String

         *
         * The KMS key identifier that will be used to encrypt the content on the
 replication instance. If you do not specify a value for the KmsKeyId parameter,
 then AWS DMS will use your default encryption key. AWS KMS creates the default
 encryption key for your AWS account. Your AWS account has a different default
 encryption key for each AWS region.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationinstance.html#cfn-dms-replicationinstance-kmskeyid
         */
        KmsKeyId?: C.YieldsString;
        /**
         * String

         *
         * The EC2 Availability Zone that the replication instance will be created
 in. The default value is a random, system-chosen Availability Zone in the endpoint's
 region.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationinstance.html#cfn-dms-replicationinstance-availabilityzone
         */
        AvailabilityZone?: C.YieldsString;
        /**
         * String

         *
         * The weekly time range during which system maintenance can occur, in Universal
 Coordinated Time (UTC).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationinstance.html#cfn-dms-replicationinstance-preferredmaintenancewindow
         */
        PreferredMaintenanceWindow?: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates that minor engine upgrades will be applied automatically to the
 replication instance during the maintenance window.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationinstance.html#cfn-dms-replicationinstance-autominorversionupgrade
         */
        AutoMinorVersionUpgrade?: C.YieldsBoolean;
        /**
         * String

         *
         * A subnet group to associate with the replication instance.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationinstance.html#cfn-dms-replicationinstance-replicationsubnetgroupidentifier
         */
        ReplicationSubnetGroupIdentifier?: C.YieldsString;
        /**
         * Integer

         *
         * The amount of storage (in gigabytes) to be initially allocated for the
 replication instance.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationinstance.html#cfn-dms-replicationinstance-allocatedstorage
         */
        AllocatedStorage?: C.Yields<C.CFInteger>;
        /**
         * List of strings

         *
         * Specifies the VPC security group to be used with the replication instance. The
 VPC security group must work with the VPC containing the replication
 instance.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationinstance.html#cfn-dms-replicationinstance-vpcsecuritygroupids
         */
        VpcSecurityGroupIds?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationinstance.html#cfn-dms-replicationinstance-allowmajorversionupgrade
         */
        AllowMajorVersionUpgrade?: C.YieldsBoolean;
        /**
         * String

         *
         * The compute and memory capacity of the replication instance as specified by the
 replication instance class.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationinstance.html#cfn-dms-replicationinstance-replicationinstanceclass
         */
        ReplicationInstanceClass: C.YieldsString;
        /**
         * Boolean

         *
         * Specifies the accessibility options for the replication instance. A value of
 true
 represents an instance with a public IP address. A value of
 false
 represents an instance with a private IP address. The default value is
 true
 .

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationinstance.html#cfn-dms-replicationinstance-publiclyaccessible
         */
        PubliclyAccessible?: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Specifies if the replication instance is a Multi-AZ deployment. You cannot set
 the
 AvailabilityZone
 parameter if the MultiAZ parameter is set to
 true
 .

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationinstance.html#cfn-dms-replicationinstance-multiaz
         */
        MultiAZ?: C.YieldsBoolean;
        /**
         * List of resource tags in key-value
 format

         *
         * The tags that you want to attach to the DMS endpoint.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationinstance.html#cfn-dms-replicationinstance-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
    }
}
export declare namespace AWS.DMS {
    /**
     *
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationsubnetgroup.html
     */
    interface ReplicationSubnetGroup {
        Type: 'AWS::DMS::ReplicationSubnetGroup';
        Properties: AWS.DMS.ReplicationSubnetGroup.Properties;
    }
    /**
     *
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationsubnetgroup.html
     */
    function ReplicationSubnetGroup(props: C.Omit<ReplicationSubnetGroup, 'Type'>): ReplicationSubnetGroup;
}
export declare namespace AWS.DMS.ReplicationSubnetGroup {
    interface Properties {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationsubnetgroup.html#cfn-dms-replicationsubnetgroup-replicationsubnetgroupdescription
         */
        ReplicationSubnetGroupDescription: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationsubnetgroup.html#cfn-dms-replicationsubnetgroup-replicationsubnetgroupidentifier
         */
        ReplicationSubnetGroupIdentifier?: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationsubnetgroup.html#cfn-dms-replicationsubnetgroup-subnetids
         */
        SubnetIds: C.Yields<C.CFList<C.YieldsString>>;
        /**
         *
         *
         *
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationsubnetgroup.html#cfn-dms-replicationsubnetgroup-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
    }
}
export declare namespace AWS.DMS {
    /**
     * The AWS::DMS::ReplicationTask resource creates an AWS DMS replication task.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationtask.html
     */
    interface ReplicationTask {
        Type: 'AWS::DMS::ReplicationTask';
        Properties: AWS.DMS.ReplicationTask.Properties;
    }
    /**
     * The AWS::DMS::ReplicationTask resource creates an AWS DMS replication task.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationtask.html
     */
    function ReplicationTask(props: C.Omit<ReplicationTask, 'Type'>): ReplicationTask;
}
export declare namespace AWS.DMS.ReplicationTask {
    interface Properties {
        /**
         * String

         *
         * Settings for the task, such as target metadata settings. For a complete list of
 task settings, see Task Settings for AWS Database Migration Service Tasks in the AWS Database Migration Service User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationtask.html#cfn-dms-replicationtask-replicationtasksettings
         */
        ReplicationTaskSettings?: C.YieldsString;
        /**
         * String

         *
         * The JSON that contains additional parameter values.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationtask.html#cfn-dms-replicationtask-tablemappings
         */
        TableMappings: C.YieldsString;
        /**
         * String

         *
         * The ARN string that uniquely identifies the
 endpoint.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationtask.html#cfn-dms-replicationtask-replicationtaskidentifier
         */
        ReplicationTaskIdentifier?: C.YieldsString;
        /**
         * String

         *
         * The ARN string that uniquely identifies the
 endpoint.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationtask.html#cfn-dms-replicationtask-sourceendpointarn
         */
        SourceEndpointArn: C.YieldsString;
        /**
         * String

         *
         * The migration type.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationtask.html#cfn-dms-replicationtask-migrationtype
         */
        MigrationType: C.YieldsString;
        /**
         * String

         *
         * The ARN string that uniquely identifies the
 endpoint.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationtask.html#cfn-dms-replicationtask-targetendpointarn
         */
        TargetEndpointArn: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the replication instance.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationtask.html#cfn-dms-replicationtask-replicationinstancearn
         */
        ReplicationInstanceArn: C.YieldsString;
        /**
         * List of resource tags in key-value
 format

         *
         * The tags that you want to attach to the migration task.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationtask.html#cfn-dms-replicationtask-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         *
         *
         * The start time for the Change Data Capture (CDC) operation.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dms-replicationtask.html#cfn-dms-replicationtask-cdcstarttime
         */
        CdcStartTime?: C.Yields<C.CFDouble>;
    }
}
export declare namespace AWS.DataPipeline {
    /**
     * Creates a data pipeline that you can use to automate the movement and transformation
    of
    data. In each pipeline, you define pipeline objects, such as activities, schedules,
    data nodes,
    and resources. For information about pipeline objects and components that you can
    use, see
    Pipeline Object Reference in the
    AWS Data Pipeline Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-datapipeline-pipeline.html
     */
    interface Pipeline {
        Type: 'AWS::DataPipeline::Pipeline';
        Properties: AWS.DataPipeline.Pipeline.Properties;
    }
    /**
     * Creates a data pipeline that you can use to automate the movement and transformation
    of
    data. In each pipeline, you define pipeline objects, such as activities, schedules,
    data nodes,
    and resources. For information about pipeline objects and components that you can
    use, see
    Pipeline Object Reference in the
    AWS Data Pipeline Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-datapipeline-pipeline.html
     */
    function Pipeline(props: C.Omit<Pipeline, 'Type'>): Pipeline;
}
export declare namespace AWS.DataPipeline.Pipeline {
    interface Properties {
        /**
         * Boolean

         *
         * Indicates whether to validate and start the pipeline or stop an active pipeline. By
 default, the value is set to true.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-datapipeline-pipeline.html#cfn-datapipeline-pipeline-activate
         */
        Activate?: C.YieldsBoolean;
        /**
         * String

         *
         * A description for the pipeline.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-datapipeline-pipeline.html#cfn-datapipeline-pipeline-description
         */
        Description?: C.YieldsString;
        /**
         * String

         *
         * A name for the pipeline. Because AWS CloudFormation assigns each new pipeline a unique
 identifier, you can use the same name for multiple pipelines that are
 associated with your AWS account.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-datapipeline-pipeline.html#cfn-datapipeline-pipeline-name
         */
        Name: C.YieldsString;
        /**
         * AWS Data Pipeline Pipeline
    ParameterObjects
         *
         * Defines the variables that are in the pipeline definition. For more
 information, see Creating
    a Pipeline Using Parameterized Templates in the
 AWS Data Pipeline Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-datapipeline-pipeline.html#cfn-datapipeline-pipeline-parameterobjects
         */
        ParameterObjects: C.Yields<C.CFList<C.Yields<AWS.DataPipeline.Pipeline.ParameterObject>>>;
        /**
         * AWS Data Pipeline Pipeline
    ParameterValues
         *
         * Defines the values for the parameters that are defined in the
 ParameterObjects property. For more information, see Creating a Pipeline Using Parameterized
    Templates in the AWS Data Pipeline Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-datapipeline-pipeline.html#cfn-datapipeline-pipeline-parametervalues
         */
        ParameterValues?: C.Yields<C.CFList<C.Yields<AWS.DataPipeline.Pipeline.ParameterValue>>>;
        /**
         * A list of AWS Data Pipeline
    PipelineObject
         *
         * A list of pipeline objects that make up the pipeline. For more information about
 pipeline objects and a description of each object, see Pipeline Object Reference in the
 AWS Data Pipeline Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-datapipeline-pipeline.html#cfn-datapipeline-pipeline-pipelineobjects
         */
        PipelineObjects?: C.Yields<C.CFList<C.Yields<AWS.DataPipeline.Pipeline.PipelineObject>>>;
        /**
         * AWS Data Pipeline Pipeline
    PipelineTags
         *
         * A list of arbitrary tags (key-value pairs) to associate with the pipeline, which you
 can use to control permissions. For more information, see Controlling Access to
    Pipelines and Resources in the
 AWS Data Pipeline Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-datapipeline-pipeline.html#cfn-datapipeline-pipeline-pipelinetags
         */
        PipelineTags?: C.Yields<C.CFList<C.Yields<AWS.DataPipeline.Pipeline.PipelineTag>>>;
    }
}
export declare namespace AWS.DataPipeline.Pipeline {
    interface Field {
        /**
         * String

         *
         * Specifies the name of a field for a particular object. To view fields for a
 data pipeline object, see Pipeline Object Reference in the
 AWS Data Pipeline Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datapipeline-pipeline-pipelineobjects-fields.html#cfn-datapipeline-pipeline-pipelineobjects-fields-key
         */
        Key: C.YieldsString;
        /**
         * String

         *
         * A field value that you specify as an identifier of another object in the same
 pipeline definition.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datapipeline-pipeline-pipelineobjects-fields.html#cfn-datapipeline-pipeline-pipelineobjects-fields-refvalue
         */
        RefValue?: C.YieldsString;
        /**
         * String

         *
         * A field value that you specify as a string. To view valid values for a
 particular field, see Pipeline
    Object Reference in the AWS Data Pipeline Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datapipeline-pipeline-pipelineobjects-fields.html#cfn-datapipeline-pipeline-pipelineobjects-fields-stringvalue
         */
        StringValue?: C.YieldsString;
    }
}
export declare namespace AWS.DataPipeline.Pipeline {
    interface ParameterAttribute {
        /**
         * String

         *
         * Specifies the name of a parameter attribute. To view parameter attributes, see
 Creating a Pipeline Using
    Parameterized Templates in the
 AWS Data Pipeline Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datapipeline-pipeline-parameterobjects-attributes.html#cfn-datapipeline-pipeline-parameterobjects-attribtues-key
         */
        Key: C.YieldsString;
        /**
         * String

         *
         * A parameter attribute value.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datapipeline-pipeline-parameterobjects-attributes.html#cfn-datapipeline-pipeline-parameterobjects-attribtues-stringvalue
         */
        StringValue: C.YieldsString;
    }
}
export declare namespace AWS.DataPipeline.Pipeline {
    interface ParameterObject {
        /**
         * AWS Data Pipeline
    Parameter Objects Attributes
         *
         * Key-value pairs that define the attributes of the parameter object.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datapipeline-pipeline-parameterobjects.html#cfn-datapipeline-pipeline-parameterobjects-attributes
         */
        Attributes: C.Yields<C.CFList<C.Yields<AWS.DataPipeline.Pipeline.ParameterAttribute>>>;
        /**
         * String

         *
         * The identifier of the parameter object.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datapipeline-pipeline-parameterobjects.html#cfn-datapipeline-pipeline-parameterobjects-id
         */
        Id: C.YieldsString;
    }
}
export declare namespace AWS.DataPipeline.Pipeline {
    interface ParameterValue {
        /**
         * String

         *
         * The ID of a parameter object.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datapipeline-pipeline-parametervalues.html#cfn-datapipeline-pipeline-parametervalues-id
         */
        Id: C.YieldsString;
        /**
         * String

         *
         * A value to associate with the parameter object.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datapipeline-pipeline-parametervalues.html#cfn-datapipeline-pipeline-parametervalues-stringvalue
         */
        StringValue: C.YieldsString;
    }
}
export declare namespace AWS.DataPipeline.Pipeline {
    interface PipelineObject {
        /**
         * List of AWS Data Pipeline
    Pipeline Field
         *
         * Key-value pairs that define the properties of the object. Duplicates allowed. You
 can use the same key multiple times within a field to define array attributes.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datapipeline-pipeline-pipelineobjects.html#cfn-datapipeline-pipeline-pipelineobjects-fields
         */
        Fields: C.Yields<C.CFList<C.Yields<AWS.DataPipeline.Pipeline.Field>>>;
        /**
         * String

         *
         * Identifier of the object.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datapipeline-pipeline-pipelineobjects.html#cfn-datapipeline-pipeline-pipelineobjects-id
         */
        Id: C.YieldsString;
        /**
         * String

         *
         * Name of the object.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datapipeline-pipeline-pipelineobjects.html#cfn-datapipeline-pipeline-pipelineobjects-name
         */
        Name: C.YieldsString;
    }
}
export declare namespace AWS.DataPipeline.Pipeline {
    interface PipelineTag {
        /**
         * String

         *
         * The key name of a tag.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datapipeline-pipeline-pipelinetags.html#cfn-datapipeline-pipeline-pipelinetags-key
         */
        Key: C.YieldsString;
        /**
         * String

         *
         * The value to associate with the key name.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datapipeline-pipeline-pipelinetags.html#cfn-datapipeline-pipeline-pipelinetags-value
         */
        Value: C.YieldsString;
    }
}
export declare namespace AWS.DirectoryService {
    /**
     * The AWS::DirectoryService::MicrosoftAD resource creates a Microsoft Active
    Directory in AWS so that your directory users and groups can access the AWS Management
    Console and AWS
    applications using their existing credentials. For more information, see What Is AWS Directory Service? in the
    AWS Directory Service Administration Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-directoryservice-microsoftad.html
     */
    interface MicrosoftAD {
        Type: 'AWS::DirectoryService::MicrosoftAD';
        Properties: AWS.DirectoryService.MicrosoftAD.Properties;
    }
    /**
     * The AWS::DirectoryService::MicrosoftAD resource creates a Microsoft Active
    Directory in AWS so that your directory users and groups can access the AWS Management
    Console and AWS
    applications using their existing credentials. For more information, see What Is AWS Directory Service? in the
    AWS Directory Service Administration Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-directoryservice-microsoftad.html
     */
    function MicrosoftAD(props: C.Omit<MicrosoftAD, 'Type'>): MicrosoftAD;
}
export declare namespace AWS.DirectoryService.MicrosoftAD {
    interface Properties {
        /**
         * Boolean

         *
         * A unique alias to assign to the Microsoft Active Directory in AWS. AWS Directory Service
 uses
 the alias to construct the access URL for the directory, such as
 http://alias.awsapps.com. By
 default, AWS CloudFormation does not create an alias.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-directoryservice-microsoftad.html#cfn-directoryservice-microsoftad-createalias
         */
        CreateAlias?: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Whether to enable single sign-on for a Microsoft Active Directory in AWS.
 Single sign-on allows users in your directory to access certain AWS services from
 a computer joined to the directory without having to enter their credentials
 separately. If you don't specify a value, AWS CloudFormation disables single sign-on
 by
 default.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-directoryservice-microsoftad.html#cfn-directoryservice-microsoftad-enablesso
         */
        EnableSso?: C.YieldsBoolean;
        /**
         * String

         *
         * The fully qualified name for the Microsoft Active Directory in AWS, such as
 corp.example.com. The name doesn't need to be publicly resolvable;
 it will resolve inside your VPC only.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-directoryservice-microsoftad.html#cfn-directoryservice-microsoftad-name
         */
        Name: C.YieldsString;
        /**
         * String

         *
         * The password for the default administrative user, Admin.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-directoryservice-microsoftad.html#cfn-directoryservice-microsoftad-password
         */
        Password: C.YieldsString;
        /**
         * String

         *
         * The NetBIOS name for your domain, such as CORP. If you don't
 specify a value, AWS Directory Service uses the first part of your directory DNS server
 name. For
 example, if your directory DNS server name is corp.example.com, AWS Directory Service
 specifies CORP for the NetBIOS name.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-directoryservice-microsoftad.html#cfn-directoryservice-microsoftad-shortname
         */
        ShortName?: C.YieldsString;
        /**
         * AWS Directory Service MicrosoftAD
    VpcSettings
         *
         * Specifies the VPC settings of the Microsoft Active Directory server in
 AWS.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-directoryservice-microsoftad.html#cfn-directoryservice-microsoftad-vpcsettings
         */
        VpcSettings: C.Yields<AWS.DirectoryService.MicrosoftAD.VpcSettings>;
    }
}
export declare namespace AWS.DirectoryService.MicrosoftAD {
    interface VpcSettings {
        /**
         * List of strings

         *
         * A list of two subnet IDs for the directory servers. Each subnet must be in different
 Availability Zones (AZs). AWS Directory Service creates a directory server and a DNS
 server in each subnet.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-directoryservice-microsoftad-vpcsettings.html#cfn-directoryservice-microsoftad-vpcsettings-subnetids
         */
        SubnetIds: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The VPC ID in which to create the Microsoft Active Directory server.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-directoryservice-microsoftad-vpcsettings.html#cfn-directoryservice-microsoftad-vpcsettings-vpcid
         */
        VpcId: C.YieldsString;
    }
}
export declare namespace AWS.DirectoryService {
    /**
     * The AWS::DirectoryService::SimpleAD resource creates an AWS Directory Service Simple Active
    Directory (Simple AD) in AWS so that your directory users and groups can access the
    AWS Management Console
    and AWS applications using their existing credentials. Simple AD is a Microsoft Active
    Directory–compatible directory. For more information, see What Is
       AWS Directory Service? in the AWS Directory Service Administration Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-directoryservice-simplead.html
     */
    interface SimpleAD {
        Type: 'AWS::DirectoryService::SimpleAD';
        Properties: AWS.DirectoryService.SimpleAD.Properties;
    }
    /**
     * The AWS::DirectoryService::SimpleAD resource creates an AWS Directory Service Simple Active
    Directory (Simple AD) in AWS so that your directory users and groups can access the
    AWS Management Console
    and AWS applications using their existing credentials. Simple AD is a Microsoft Active
    Directory–compatible directory. For more information, see What Is
       AWS Directory Service? in the AWS Directory Service Administration Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-directoryservice-simplead.html
     */
    function SimpleAD(props: C.Omit<SimpleAD, 'Type'>): SimpleAD;
}
export declare namespace AWS.DirectoryService.SimpleAD {
    interface Properties {
        /**
         * Boolean

         *
         * A unique alias to assign to the directory. AWS Directory Service uses the alias to
 construct
 the access URL for the directory, such as
 http://alias.awsapps.com. By
 default, AWS CloudFormation does not create an alias.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-directoryservice-simplead.html#cfn-directoryservice-simplead-createalias
         */
        CreateAlias?: C.YieldsBoolean;
        /**
         * String

         *
         * A description of the directory.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-directoryservice-simplead.html#cfn-directoryservice-simplead-description
         */
        Description?: C.YieldsString;
        /**
         * Boolean

         *
         * Whether to enable single sign-on for a directory. If you don't specify a value,
 AWS CloudFormation disables single sign-on by default.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-directoryservice-simplead.html#cfn-directoryservice-simplead-enablesso
         */
        EnableSso?: C.YieldsBoolean;
        /**
         * String

         *
         * The fully qualified name for the directory, such as
 corp.example.com.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-directoryservice-simplead.html#cfn-directoryservice-simplead-name
         */
        Name: C.YieldsString;
        /**
         * String

         *
         * The password for the directory administrator. AWS Directory Service creates a directory
 administrator account with the user name Administrator and this
 password.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-directoryservice-simplead.html#cfn-directoryservice-simplead-password
         */
        Password: C.YieldsString;
        /**
         * String

         *
         * The NetBIOS name of the on-premises directory, such as
 CORP.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-directoryservice-simplead.html#cfn-directoryservice-simplead-shortname
         */
        ShortName?: C.YieldsString;
        /**
         * String

         *
         * The size of the directory. For valid values, see CreateDirectory in the
 AWS Directory Service API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-directoryservice-simplead.html#cfn-directoryservice-simplead-size
         */
        Size: C.YieldsString;
        /**
         * AWS Directory Service SimpleAD
    VpcSettings
         *
         * Specifies the VPC settings of the directory server.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-directoryservice-simplead.html#cfn-directoryservice-simplead-vpcsettings
         */
        VpcSettings: C.Yields<AWS.DirectoryService.SimpleAD.VpcSettings>;
    }
}
export declare namespace AWS.DirectoryService.SimpleAD {
    interface VpcSettings {
        /**
         * List of strings

         *
         * A list of two subnet IDs for the directory servers. Each subnet must be in different
 Availability Zones (AZ). AWS Directory Service creates a directory server and a DNS
 server in each subnet.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-directoryservice-simplead-vpcsettings.html#cfn-directoryservice-simplead-vpcsettings-subnetids
         */
        SubnetIds: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The VPC ID in which to create the Simple AD directory.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-directoryservice-simplead-vpcsettings.html#cfn-directoryservice-simplead-vpcsettings-vpcid
         */
        VpcId: C.YieldsString;
    }
}
export declare namespace AWS.DynamoDB {
    /**
     * The AWS::DynamoDB::Table resource creates a DynamoDB table.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html
     */
    interface Table {
        Type: 'AWS::DynamoDB::Table';
        Properties: AWS.DynamoDB.Table.Properties;
    }
    /**
     * The AWS::DynamoDB::Table resource creates a DynamoDB table.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html
     */
    function Table(props: C.Omit<Table, 'Type'>): Table;
}
export declare namespace AWS.DynamoDB.Table {
    interface Properties {
        /**
         * List of DynamoDB Table AttributeDefinitions
         *
         * A list of AttributeName and AttributeType objects
 that describe the key schema for the table and indexes.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-attributedef
         */
        AttributeDefinitions: C.Yields<C.CFList<C.Yields<AWS.DynamoDB.Table.AttributeDefinition>>>;
        /**
         * List of DynamoDB Table GlobalSecondaryIndexes
         *
         * Global secondary indexes to be created on the table. You can create up to 5 global
 secondary indexes.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-gsi
         */
        GlobalSecondaryIndexes?: C.Yields<C.CFList<C.Yields<AWS.DynamoDB.Table.GlobalSecondaryIndex>>>;
        /**
         * List of DynamoDB Table KeySchema
         *
         * Specifies the attributes that make up the primary key for the table. The
 attributes in the KeySchema property must also be defined in the
 AttributeDefinitions property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-keyschema
         */
        KeySchema: C.Yields<C.CFList<C.Yields<AWS.DynamoDB.Table.KeySchema>>>;
        /**
         * List of DynamoDB Table LocalSecondaryIndexes
         *
         * Local secondary indexes to be created on the table. You can create up to 5 local
 secondary indexes. Each index is scoped to a given hash key value. The size of each
 hash
 key can be up to 10 gigabytes.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-lsi
         */
        LocalSecondaryIndexes?: C.Yields<C.CFList<C.Yields<AWS.DynamoDB.Table.LocalSecondaryIndex>>>;
        /**
         * DynamoDB Table ProvisionedThroughput
         *
         * Throughput for the specified table, which consists of values for ReadCapacityUnits
 and WriteCapacityUnits. For more information about the contents of a provisioned
 throughput structure, see Amazon DynamoDB Table ProvisionedThroughput.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-provisionedthroughput
         */
        ProvisionedThroughput: C.Yields<AWS.DynamoDB.Table.ProvisionedThroughput>;
        /**
         * DynamoDB Table StreamSpecification
         *
         * The settings for the DynamoDB table stream, which capture changes to items stored
 in
 the table.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-streamspecification
         */
        StreamSpecification?: C.Yields<AWS.DynamoDB.Table.StreamSpecification>;
        /**
         * String

         *
         * A name for the table. If you don't specify a name, AWS CloudFormation generates a
 unique
 physical ID and uses that ID for the table name. For more information, see Name Type.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-tablename
         */
        TableName?: C.YieldsString;
        /**
         * AWS CloudFormation Resource Tags
         *
         * Specifies an arbitrary set of tags (key–value pairs) to associate with this
 table. Use tags to manage your resources.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * DynamoDB Table TimeToLiveSpecification
         *
         * Specifies the Time to Live (TTL)
 settings
 for the table.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-timetolivespecification
         */
        TimeToLiveSpecification?: C.Yields<AWS.DynamoDB.Table.TimeToLiveSpecification>;
    }
}
export declare namespace AWS.DynamoDB.Table {
    interface AttributeDefinition {
        /**
         * String

         *
         * The name of an attribute. Attribute names can be 1 – 255 characters long
 and have no character restrictions.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-attributedef.html#cfn-dynamodb-attributedef-attributename
         */
        AttributeName: C.YieldsString;
        /**
         * String

         *
         * The data type for the attribute. You can specify S for string
 data, N for numeric data, or B for binary data.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-attributedef.html#cfn-dynamodb-attributedef-attributename-attributetype
         */
        AttributeType: C.YieldsString;
    }
}
export declare namespace AWS.DynamoDB.Table {
    interface GlobalSecondaryIndex {
        /**
         * String

         *
         * The name of the global secondary index. The index name can be 3 – 255
 characters long and have no character restrictions.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-gsi.html#cfn-dynamodb-gsi-indexname
         */
        IndexName: C.YieldsString;
        /**
         * Amazon DynamoDB Table KeySchema
         *
         * The complete index key schema for the global secondary index, which consists of
 one or more pairs of attribute names and key types.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-gsi.html#cfn-dynamodb-gsi-keyschema
         */
        KeySchema: C.Yields<C.CFList<C.Yields<AWS.DynamoDB.Table.KeySchema>>>;
        /**
         * Amazon DynamoDB Table Projection
         *
         * Attributes that are copied (projected) from the source table into the index.
 These attributes are in addition to the primary key attributes and index key
 attributes, which are automatically projected.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-gsi.html#cfn-dynamodb-gsi-projection
         */
        Projection: C.Yields<AWS.DynamoDB.Table.Projection>;
        /**
         * Amazon DynamoDB Table ProvisionedThroughput
         *
         * The provisioned throughput settings for the index.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-gsi.html#cfn-dynamodb-gsi-provisionedthroughput
         */
        ProvisionedThroughput: C.Yields<AWS.DynamoDB.Table.ProvisionedThroughput>;
    }
}
export declare namespace AWS.DynamoDB.Table {
    interface KeySchema {
        /**
         * String

         *
         * The attribute name that is used as the primary key for this table. Primary key
 element names can be 1 – 255 characters long and have no character
 restrictions.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-keyschema.html#aws-properties-dynamodb-keyschema-attributename
         */
        AttributeName: C.YieldsString;
        /**
         * String

         *
         * Represents the attribute data, consisting of the data type and the attribute
 value itself. You can specify HASH or RANGE.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-keyschema.html#aws-properties-dynamodb-keyschema-keytype
         */
        KeyType: C.YieldsString;
    }
}
export declare namespace AWS.DynamoDB.Table {
    interface LocalSecondaryIndex {
        /**
         * String

         *
         * The name of the local secondary index. The index name can be 3 – 255
 characters long and have no character restrictions.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-lsi.html#cfn-dynamodb-lsi-indexname
         */
        IndexName: C.YieldsString;
        /**
         * Amazon DynamoDB Table KeySchema
         *
         * The complete index key schema for the local secondary index, which consists of
 one or more pairs of attribute names and key types. For local secondary indexes,
 the hash key must be the same as that of the source table.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-lsi.html#cfn-dynamodb-lsi-keyschema
         */
        KeySchema: C.Yields<C.CFList<C.Yields<AWS.DynamoDB.Table.KeySchema>>>;
        /**
         * Amazon DynamoDB Table Projection
         *
         * Attributes that are copied (projected) from the source table into the index.
 These attributes are additions to the primary key attributes and index key
 attributes, which are automatically projected.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-lsi.html#cfn-dynamodb-lsi-projection
         */
        Projection: C.Yields<AWS.DynamoDB.Table.Projection>;
    }
}
export declare namespace AWS.DynamoDB.Table {
    interface Projection {
        /**
         * List of strings

         *
         * The non-key attribute names that are projected into the index.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-projectionobject.html#cfn-dynamodb-projectionobj-nonkeyatt
         */
        NonKeyAttributes?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The set of attributes that are projected into the index:
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-projectionobject.html#cfn-dynamodb-projectionobj-projtype
         */
        ProjectionType?: C.YieldsString;
    }
}
export declare namespace AWS.DynamoDB.Table {
    interface ProvisionedThroughput {
        /**
         * Number

         *
         * Sets the desired minimum number of consistent reads of items (up to 1KB in
 size) per second for the specified table before Amazon DynamoDB balances the
 load.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-provisionedthroughput.html#cfn-dynamodb-provisionedthroughput-readcapacityunits
         */
        ReadCapacityUnits: C.Yields<C.CFLong>;
        /**
         * Number

         *
         * Sets the desired minimum number of consistent writes of items (up to 1KB in
 size) per second for the specified table before Amazon DynamoDB balances the
 load.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-provisionedthroughput.html#cfn-dynamodb-provisionedthroughput-writecapacityunits
         */
        WriteCapacityUnits: C.Yields<C.CFLong>;
    }
}
export declare namespace AWS.DynamoDB.Table {
    interface StreamSpecification {
        /**
         * String

         *
         * Determines the information that the stream captures when an item in the table
 is modified. For valid values, see StreamSpecification in
 the Amazon DynamoDB API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-streamspecification.html#cfn-dynamodb-streamspecification-streamviewtype
         */
        StreamViewType: C.YieldsString;
    }
}
export declare namespace AWS.DynamoDB.Table {
    interface TimeToLiveSpecification {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-timetolivespecification.html#cfn-dynamodb-timetolivespecification-attributename
         */
        AttributeName: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-timetolivespecification.html#cfn-dynamodb-timetolivespecification-enabled
         */
        Enabled: C.YieldsBoolean;
    }
}
export declare namespace AWS.EC2 {
    /**
     * Provides information to AWS about your VPN customer gateway device.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-customer-gateway.html
     */
    interface CustomerGateway {
        Type: 'AWS::EC2::CustomerGateway';
        Properties: AWS.EC2.CustomerGateway.Properties;
    }
    /**
     * Provides information to AWS about your VPN customer gateway device.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-customer-gateway.html
     */
    function CustomerGateway(props: C.Omit<CustomerGateway, 'Type'>): CustomerGateway;
}
export declare namespace AWS.EC2.CustomerGateway {
    interface Properties {
        /**
         * Number BgpAsn is always an integer value.

         *
         * The customer gateway's Border Gateway Protocol (BGP) Autonomous System Number (ASN).
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-customer-gateway.html#cfn-ec2-customergateway-bgpasn
         */
        BgpAsn: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The internet-routable IP address for the customer gateway's outside interface. The
 address must be
 static.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-customer-gateway.html#cfn-ec2-customergateway-ipaddress
         */
        IpAddress: C.YieldsString;
        /**
         * AWS CloudFormation Resource Tags.

         *
         * The tags that you want to attach to the resource.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-customer-gateway.html#cfn-ec2-customergateway-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * String

         *
         * The type of VPN connection that this customer gateway supports.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-customer-gateway.html#cfn-ec2-customergateway-type
         */
        Type: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * Creates a set of DHCP options for your VPC.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-dhcp-options.html
     */
    interface DHCPOptions {
        Type: 'AWS::EC2::DHCPOptions';
        Properties: AWS.EC2.DHCPOptions.Properties;
    }
    /**
     * Creates a set of DHCP options for your VPC.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-dhcp-options.html
     */
    function DHCPOptions(props: C.Omit<DHCPOptions, 'Type'>): DHCPOptions;
}
export declare namespace AWS.EC2.DHCPOptions {
    interface Properties {
        /**
         * String

         *
         * A domain name of your choice.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-dhcp-options.html#cfn-ec2-dhcpoptions-domainname
         */
        DomainName?: C.YieldsString;
        /**
         * List of strings

         *
         * The IP (IPv4) address of a domain name server. You can specify up to four addresses.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-dhcp-options.html#cfn-ec2-dhcpoptions-domainnameservers
         */
        DomainNameServers?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of strings

         *
         * The IP address (IPv4) of a NetBIOS name server. You can specify up to four addresses.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-dhcp-options.html#cfn-ec2-dhcpoptions-netbiosnameservers
         */
        NetbiosNameServers?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of numbers

         *
         * An integer value indicating the NetBIOS node type:
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-dhcp-options.html#cfn-ec2-dhcpoptions-netbiosnodetype
         */
        NetbiosNodeType?: C.Yields<C.CFInteger>;
        /**
         * List of strings

         *
         * The IP address (IPv4) of a Network Time Protocol (NTP) server. You can specify up
 to four
 addresses.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-dhcp-options.html#cfn-ec2-dhcpoptions-ntpservers
         */
        NtpServers?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key–value pairs) for this resource.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-dhcp-options.html#cfn-ec2-dhcpoptions-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
    }
}
export declare namespace AWS.EC2 {
    /**
     * The AWS::EC2::EIP resource allocates an Elastic IP (EIP) address and can, optionally,
    associate it with an Amazon EC2 instance.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-eip.html
     */
    interface EIP {
        Type: 'AWS::EC2::EIP';
        Properties: AWS.EC2.EIP.Properties;
    }
    /**
     * The AWS::EC2::EIP resource allocates an Elastic IP (EIP) address and can, optionally,
    associate it with an Amazon EC2 instance.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-eip.html
     */
    function EIP(props: C.Omit<EIP, 'Type'>): EIP;
}
export declare namespace AWS.EC2.EIP {
    interface Properties {
        /**
         * String

         *
         * Set to vpc to allocate the address to your Virtual Private Cloud
 (VPC). No other values are supported.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-eip.html#cfn-ec2-eip-domain
         */
        Domain?: C.YieldsString;
        /**
         * String

         *
         * The Instance ID of the Amazon EC2 instance that you want to associate with this
 Elastic IP address.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-eip.html#cfn-ec2-eip-instanceid
         */
        InstanceId?: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * The AWS::EC2::EIPAssociation resource type associates an Elastic IP address with an
    Amazon EC2
    instance. The Elastic IP address can be an existing Elastic IP address or an Elastic
    IP
    address allocated through an AWS::EC2::EIP
       resource.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-eip-association.html
     */
    interface EIPAssociation {
        Type: 'AWS::EC2::EIPAssociation';
        Properties: AWS.EC2.EIPAssociation.Properties;
    }
    /**
     * The AWS::EC2::EIPAssociation resource type associates an Elastic IP address with an
    Amazon EC2
    instance. The Elastic IP address can be an existing Elastic IP address or an Elastic
    IP
    address allocated through an AWS::EC2::EIP
       resource.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-eip-association.html
     */
    function EIPAssociation(props: C.Omit<EIPAssociation, 'Type'>): EIPAssociation;
}
export declare namespace AWS.EC2.EIPAssociation {
    interface Properties {
        /**
         * String

         *
         * [EC2-VPC] Allocation ID for the VPC Elastic IP address you want to associate
 with an Amazon EC2 instance in your VPC.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-eip-association.html#cfn-ec2-eipassociation-allocationid
         */
        AllocationId?: C.YieldsString;
        /**
         * String

         *
         * Elastic IP address that you want to associate with the Amazon EC2 instance specified
 by the InstanceId property. You can specify an existing Elastic IP
 address or a reference to an Elastic IP address allocated with a AWS::EC2::EIP resource.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-eip-association.html#cfn-ec2-eipassociation-eip
         */
        EIP?: C.YieldsString;
        /**
         * String

         *
         * Instance ID of the Amazon EC2 instance that you want to associate with the Elastic
 IP address specified by the EIP property. If the instance has more than one network
 interface, you must specify a network interface ID.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-eip-association.html#cfn-ec2-eipassociation-instanceid
         */
        InstanceId?: C.YieldsString;
        /**
         * String

         *
         * [EC2-VPC] The ID of the network interface to associate with the Elastic IP
 address. If the instance has more than one network interface, you must specify a
 network interface ID.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-eip-association.html#cfn-ec2-eipassociation-networkinterfaceid
         */
        NetworkInterfaceId?: C.YieldsString;
        /**
         * String

         *
         * [EC2-VPC] The private IP address that you want to associate with the Elastic IP
 address. The private IP address is restricted to the primary and secondary private
 IP addresses that are associated with the network interface. By default, the
 private IP address that is associated with the EIP is the primary private IP
 address of the network interface.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-eip-association.html#cfn-ec2-eipassociation-PrivateIpAddress
         */
        PrivateIpAddress?: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * The AWS::EC2::EgressOnlyInternetGateway resource creates an egress-only
    Internet gateway for your VPC (over IPv6 only). An egress-only Internet gateway enables
    outbound communication over IPv6 from instances in your VPC to the Internet. It also
    prevents
    hosts outside of your VPC from initiating an IPv6 connection with your instance.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-egressonlyinternetgateway.html
     */
    interface EgressOnlyInternetGateway {
        Type: 'AWS::EC2::EgressOnlyInternetGateway';
        Properties: AWS.EC2.EgressOnlyInternetGateway.Properties;
    }
    /**
     * The AWS::EC2::EgressOnlyInternetGateway resource creates an egress-only
    Internet gateway for your VPC (over IPv6 only). An egress-only Internet gateway enables
    outbound communication over IPv6 from instances in your VPC to the Internet. It also
    prevents
    hosts outside of your VPC from initiating an IPv6 connection with your instance.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-egressonlyinternetgateway.html
     */
    function EgressOnlyInternetGateway(props: C.Omit<EgressOnlyInternetGateway, 'Type'>): EgressOnlyInternetGateway;
}
export declare namespace AWS.EC2.EgressOnlyInternetGateway {
    interface Properties {
        /**
         * String

         *
         * The ID of the VPC for which to create the egress-only Internet gateway.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-egressonlyinternetgateway.html#cfn-ec2-egressonlyinternetgateway-vpcid
         */
        VpcId: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * The AWS::EC2::FlowLog resource creates an Amazon Elastic Compute Cloud (Amazon EC2) flow log that captures
    IP traffic for a specified network interface, subnet, or VPC. To view the log data,
    use Amazon CloudWatch Logs (CloudWatch Logs) to help troubleshoot connection issues.
    For example, you can use a flow log to investigate why certain traffic isn't reaching
    an instance, which can help you diagnose overly restrictive security group rules.
    For more information, see VPC Flow
       Logs in the Amazon VPC User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-flowlog.html
     */
    interface FlowLog {
        Type: 'AWS::EC2::FlowLog';
        Properties: AWS.EC2.FlowLog.Properties;
    }
    /**
     * The AWS::EC2::FlowLog resource creates an Amazon Elastic Compute Cloud (Amazon EC2) flow log that captures
    IP traffic for a specified network interface, subnet, or VPC. To view the log data,
    use Amazon CloudWatch Logs (CloudWatch Logs) to help troubleshoot connection issues.
    For example, you can use a flow log to investigate why certain traffic isn't reaching
    an instance, which can help you diagnose overly restrictive security group rules.
    For more information, see VPC Flow
       Logs in the Amazon VPC User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-flowlog.html
     */
    function FlowLog(props: C.Omit<FlowLog, 'Type'>): FlowLog;
}
export declare namespace AWS.EC2.FlowLog {
    interface Properties {
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of an AWS Identity and Access Management (IAM) role
 that permits Amazon EC2 to publish flow logs to a CloudWatch Logs log group in your
 account.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-flowlog.html#cfn-ec2-flowlog-deliverlogspermissionarn
         */
        DeliverLogsPermissionArn: C.YieldsString;
        /**
         * String

         *
         * The name of a new or existing CloudWatch Logs log group where Amazon EC2 publishes
 your flow logs.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-flowlog.html#cfn-ec2-flowlog-loggroupname
         */
        LogGroupName: C.YieldsString;
        /**
         * String

         *
         * The ID of the subnet, network interface, or VPC for which you want to create a flow
 log.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-flowlog.html#cfn-ec2-flowlog-resourceid
         */
        ResourceId: C.YieldsString;
        /**
         * String

         *
         * The type of resource that you specified in the ResourceId property. For example, if you specified a VPC ID for the ResourceId property, specify VPC for this property. For valid values, see the ResourceType parameter for the CreateFlowLogs action in the Amazon EC2 API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-flowlog.html#cfn-ec2-flowlog-resourcetype
         */
        ResourceType: C.YieldsString;
        /**
         * String

         *
         * The type of traffic to log. You can log traffic that the resource accepts or rejects,
 or all traffic. For valid values, see the TrafficType parameter for the CreateFlowLogs action in the Amazon EC2 API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-flowlog.html#cfn-ec2-flowlog-traffictype
         */
        TrafficType: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * The AWS::EC2::Host resource allocates a fully dedicated physical server for launching EC2 instances.
    Because the host is fully dedicated for your use, it can help you address compliance
    requirements and reduce costs by allowing you to use your existing server-bound software
    licenses. For more information, see Dedicated Hosts in the Amazon EC2 User Guide for Linux Instances.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-host.html
     */
    interface Host {
        Type: 'AWS::EC2::Host';
        Properties: AWS.EC2.Host.Properties;
    }
    /**
     * The AWS::EC2::Host resource allocates a fully dedicated physical server for launching EC2 instances.
    Because the host is fully dedicated for your use, it can help you address compliance
    requirements and reduce costs by allowing you to use your existing server-bound software
    licenses. For more information, see Dedicated Hosts in the Amazon EC2 User Guide for Linux Instances.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-host.html
     */
    function Host(props: C.Omit<Host, 'Type'>): Host;
}
export declare namespace AWS.EC2.Host {
    interface Properties {
        /**
         * String

         *
         * Indicates if the host accepts EC2 instances with only matching configurations or if
 instances must also specify the host ID. Instances that don't specify a host ID can't
 launch onto a host with AutoPlacement set to off. By default,
 AWS CloudFormation sets this property to on. For more information, see Understanding Instance
    Placement and Host Affinity in the
 Amazon EC2 User Guide for Linux Instances.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-host.html#cfn-ec2-host-autoplacement
         */
        AutoPlacement?: C.YieldsString;
        /**
         * String

         *
         * The Availability Zone (AZ) in which to launch the dedicated host.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-host.html#cfn-ec2-host-availabilityzone
         */
        AvailabilityZone: C.YieldsString;
        /**
         * String

         *
         * The instance type that the dedicated host accepts. Only instances of this type can
 be launched onto the host. For more information, see Supported Instance Types in the Amazon EC2 User Guide for Linux Instances.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-host.html#cfn-ec2-host-instancetype
         */
        InstanceType: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * The AWS::EC2::Instance resource creates an EC2  instance.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html
     */
    interface Instance {
        Type: 'AWS::EC2::Instance';
        Properties: AWS.EC2.Instance.Properties;
    }
    /**
     * The AWS::EC2::Instance resource creates an EC2  instance.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html
     */
    function Instance(props: C.Omit<Instance, 'Type'>): Instance;
}
export declare namespace AWS.EC2.Instance {
    interface Properties {
        /**
         * String

         *
         * Reserved.
         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-additionalinfo
         */
        AdditionalInfo?: C.YieldsString;
        /**
         * String

         *
         * Indicates whether Amazon Elastic Compute Cloud (Amazon EC2) always associates the
 instance with a dedicated host. If you want Amazon EC2 to always restart the instance (if it was stopped) onto the
 same host on which it was launched, specify host. If you want Amazon EC2 to restart the instance on any available host, but to try
 to launch the instance onto the last host it ran on (on a best-effort basis), specify
 default.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-affinity
         */
        Affinity?: C.YieldsString;
        /**
         * String

         *
         * Specifies the name of the Availability Zone in which the instance is
 located.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-availabilityzone
         */
        AvailabilityZone?: C.YieldsString;
        /**
         * A list of Amazon EC2 Block Device Mapping Property.

         *
         * Defines a set of Amazon Elastic Block Store block device mappings, ephemeral instance
 store
 block device mappings, or both. For more information, see Amazon Elastic Block Store or Amazon EC2 Instance Store in the
 Amazon EC2 User Guide for Linux Instances.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-blockdevicemappings
         */
        BlockDeviceMappings?: C.Yields<C.CFList<C.Yields<AWS.EC2.Instance.BlockDeviceMapping>>>;
        /**
         * Boolean

         *
         * Specifies whether the instance can be terminated through the API.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-disableapitermination
         */
        DisableApiTermination?: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Specifies whether the instance is optimized for Amazon Elastic Block Store I/O. This
 optimization provides dedicated throughput to Amazon EBS and an optimized configuration
 stack to provide optimal EBS I/O performance.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-ebsoptimized
         */
        EbsOptimized?: C.YieldsBoolean;
        /**
         * String

         *
         * If you specify host for the Affinity property, the ID of a dedicated host that the instance is associated with. If you
 don't specify an ID, Amazon EC2 launches the instance onto any available, compatible
 dedicated host in your account. This type of launch is called an untargeted launch.
 Note that for untargeted launches, you must have a compatible, dedicated host available
 to successfully launch instances.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-hostid
         */
        HostId?: C.YieldsString;
        /**
         * String

         *
         * The name of an instance profile or a reference to an AWS::IAM::InstanceProfile resource.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-iaminstanceprofile
         */
        IamInstanceProfile?: C.YieldsString;
        /**
         * String

         *
         * Provides the unique ID of the Amazon Machine Image (AMI) that was assigned
 during registration.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-imageid
         */
        ImageId: C.YieldsString;
        /**
         * String

         *
         * Indicates whether an instance stops or terminates when you shut down the
 instance from the instance's operating system shutdown command. You can specify
 stop or terminate. For more information, see the
 RunInstances command in the
 Amazon EC2 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-instanceinitiatedshutdownbehavior
         */
        InstanceInitiatedShutdownBehavior?: C.YieldsString;
        /**
         * String

         *
         * The instance type, such as t2.micro. The default type is
 "m3.medium". For a list of instance types, see Instance Families and
    Types.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-instancetype
         */
        InstanceType?: C.YieldsString;
        /**
         * Integer

         *
         * The number of IPv6 addresses to associate with the instance's primary network
 interface. Amazon EC2 automatically selects the IPv6 addresses from the subnet range.
 To specify specific IPv6 addresses, use the Ipv6Addresses property
 and don't specify this property.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-ipv6addresscount
         */
        Ipv6AddressCount?: C.Yields<C.CFInteger>;
        /**
         * List of EC2 NetworkInterface
    Ipv6Addresses
         *
         * One or more specific IPv6 addresses from the IPv6 CIDR block range of your
 subnet to associate with the instance's primary network interface. To specify a
 number of IPv6 addresses, use the Ipv6AddressCount property and don't
 specify this property.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-ipv6addresses
         */
        Ipv6Addresses?: C.Yields<C.CFList<C.Yields<AWS.EC2.Instance.InstanceIpv6Address>>>;
        /**
         * String

         *
         * The kernel ID.
         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-kernelid
         */
        KernelId?: C.YieldsString;
        /**
         * String

         *
         * Provides the name of the Amazon EC2 key pair.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-keyname
         */
        KeyName?: C.YieldsString;
        /**
         * Boolean

         *
         * Specifies whether detailed monitoring is enabled for the instance.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-monitoring
         */
        Monitoring?: C.YieldsBoolean;
        /**
         * A list of EC2 NetworkInterface Embedded
    Property Type
         *
         * A list of embedded objects that describes the network interfaces to associate
 with this instance.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-networkinterfaces
         */
        NetworkInterfaces?: C.Yields<C.CFList<C.Yields<AWS.EC2.Instance.NetworkInterface>>>;
        /**
         * String

         *
         * The name of an existing placement group that you want to launch the instance
 into (for cluster instances).

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-placementgroupname
         */
        PlacementGroupName?: C.YieldsString;
        /**
         * String

         *
         * The private IP address for this instance.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-privateipaddress
         */
        PrivateIpAddress?: C.YieldsString;
        /**
         * String

         *
         * The ID of the RAM disk to select. Some kernels require additional drivers at
 launch. Check the kernel requirements for information about whether you need to
 specify a RAM disk. To find kernel requirements, go to the AWS Resource Center and
 search for the kernel ID.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-ramdiskid
         */
        RamdiskId?: C.YieldsString;
        /**
         * List of strings

         *
         * A list that contains the security group IDs for VPC security groups to assign
 to the Amazon EC2 instance. If you specified the NetworkInterfaces
 property, do not specify this property.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-securitygroupids
         */
        SecurityGroupIds?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of strings

         *
         * Valid only for Amazon EC2 security groups. A list that contains the Amazon EC2 security
 groups to assign to the Amazon EC2 instance. The list can contain both the name of
 existing Amazon EC2 security groups or references to AWS::EC2::SecurityGroup resources
 created in the template.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-securitygroups
         */
        SecurityGroups?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * Boolean

         *
         * Controls whether source/destination checking is enabled on the instance. Also
 determines if an instance in a VPC will perform network address translation
 (NAT).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-sourcedestcheck
         */
        SourceDestCheck?: C.YieldsBoolean;
        /**
         * List of Amazon EC2 Instance
    SsmAssociations.

         *
         * The Amazon EC2 Systems Manager (SSM) document
 and parameter values to associate with this instance. To use this property, you
 must specify an IAM role for the instance. For more information, see Prerequisites for Remotely
    Running Commands on EC2 Instances in the
 Amazon EC2 User Guide for Windows Instances.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-ssmassociations
         */
        SsmAssociations?: C.Yields<C.CFList<C.Yields<AWS.EC2.Instance.SsmAssociation>>>;
        /**
         * String

         *
         * If you're using Amazon VPC, this property specifies the ID of the subnet that
 you want to launch the instance into. If you specified the
 NetworkInterfaces property, do not specify this property.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-subnetid
         */
        SubnetId?: C.YieldsString;
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key–value pairs) for this instance.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * String

         *
         * The tenancy of the instance that you want to launch, such as default, dedicated, or host. If you specify a tenancy value of dedicated or host, you must launch the instance in a VPC. For more information, see Dedicated Instances in the Amazon VPC User Guide.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-tenancy
         */
        Tenancy?: C.YieldsString;
        /**
         * String

         *
         * Base64-encoded MIME user data that is made available to the instances.
         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-userdata
         */
        UserData?: C.YieldsString;
        /**
         * A list of EC2 MountPoints.

         *
         * The Amazon EBS volumes to attach to the instance.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance.html#cfn-ec2-instance-volumes
         */
        Volumes?: C.Yields<C.CFList<C.Yields<AWS.EC2.Instance.Volume>>>;
    }
}
export declare namespace AWS.EC2.Instance {
    interface AssociationParameter {
        /**
         * String

         *
         * The name of an input parameter that is in the associated SSM document.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance-ssmassociations-associationparameters.html#cfn-ec2-instance-ssmassociations-associationparameters-key
         */
        Key: C.YieldsString;
        /**
         * List of strings

         *
         * The value of an input parameter.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance-ssmassociations-associationparameters.html#cfn-ec2-instance-ssmassociations-associationparameters-value
         */
        Value: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.EC2.Instance {
    interface BlockDeviceMapping {
        /**
         * String

         *
         * The name of the device within Amazon EC2.
 For more information, see Device Naming on Linux Instances in the Amazon EC2 User Guide for Linux Instances.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-blockdev-mapping.html#cfn-ec2-blockdev-mapping-devicename
         */
        DeviceName: C.YieldsString;
        /**
         * Amazon Elastic Block Store Block Device Property.

         *
         * Required: Conditional You can specify either VirtualName or
 Ebs, but not both.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-blockdev-mapping.html#cfn-ec2-blockdev-mapping-ebs
         */
        Ebs?: C.Yields<AWS.EC2.Instance.Ebs>;
        /**
         * Boolean

         *
         * This property can be used to unmap a defined device.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-blockdev-mapping.html#cfn-ec2-blockdev-mapping-nodevice
         */
        NoDevice?: C.Yields<AWS.EC2.Instance.NoDevice>;
        /**
         * String

         *
         * The name of the virtual device. The name must be in the form
 ephemeralX
     where X is a number starting from zero (0); for
 example, ephemeral0.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-blockdev-mapping.html#cfn-ec2-blockdev-mapping-virtualname
         */
        VirtualName?: C.YieldsString;
    }
}
export declare namespace AWS.EC2.Instance {
    interface Ebs {
        /**
         * Boolean

         *
         * Determines whether to delete the volume on instance termination. The default
 value is true.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-blockdev-template.html#cfn-ec2-blockdev-template-deleteontermination
         */
        DeleteOnTermination?: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Indicates whether the volume is encrypted. Encrypted Amazon EBS volumes can only be
 attached to instance types that support Amazon EBS encryption. Volumes that are created
 from encrypted snapshots are automatically encrypted. You cannot create an
 encrypted volume from an unencrypted snapshot or vice versa. If your AMI uses
 encrypted volumes, you can only launch the AMI on supported instance types. For
 more information, see Amazon EBS
    encryption in the Amazon EC2 User Guide for Linux Instances.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-blockdev-template.html#cfn-ec2-blockdev-template-encrypted
         */
        Encrypted?: C.YieldsBoolean;
        /**
         * Number

         *
         * The number of I/O operations per second (IOPS) that the volume supports. This can
 be an integer
 from 100 – 20000.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-blockdev-template.html#cfn-ec2-blockdev-template-iops
         */
        Iops?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The snapshot ID of the volume to use to create a block device.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-blockdev-template.html#cfn-ec2-blockdev-template-snapshotid
         */
        SnapshotId?: C.YieldsString;
        /**
         * String

         *
         * The volume size, in gibibytes (GiB). For valid values, see the Size parameter for the CreateVolume action in the Amazon EC2 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-blockdev-template.html#cfn-ec2-blockdev-template-volumesize
         */
        VolumeSize?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The volume type. If you set the type to io1, you must also set the Iops property. For valid values, see the VolumeType parameter for the CreateVolume action in the Amazon EC2 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-blockdev-template.html#cfn-ec2-blockdev-template-volumetype
         */
        VolumeType?: C.YieldsString;
    }
}
export declare namespace AWS.EC2.Instance {
    interface InstanceIpv6Address {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance-instanceipv6address.html#cfn-ec2-instance-instanceipv6address-ipv6address
         */
        Ipv6Address: C.YieldsString;
    }
}
export declare namespace AWS.EC2.Instance {
    interface NetworkInterface {
        /**
         * Boolean.

         *
         * Indicates whether the network interface receives a public IP address. You can
 associate a public IP address with a network interface only if it has a device
 index of eth0 and if it is a new network interface (not an existing
 one). In other words, if you specify true, don't specify a network interface ID.
 For more information, see Amazon EC2 Instance IP
    Addressing.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-network-iface-embedded.html#aws-properties-ec2-network-iface-embedded-associatepubip
         */
        AssociatePublicIpAddress?: C.YieldsBoolean;
        /**
         * Boolean.

         *
         * Whether to delete the network interface when the instance terminates.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-network-iface-embedded.html#aws-properties-ec2-network-iface-embedded-delete
         */
        DeleteOnTermination?: C.YieldsBoolean;
        /**
         * String

         *
         * The description of this network interface.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-network-iface-embedded.html#aws-properties-ec2-network-iface-embedded-description
         */
        Description?: C.YieldsString;
        /**
         * String

         *
         * The network interface's position in the attachment order.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-network-iface-embedded.html#aws-properties-ec2-network-iface-embedded-deviceindex
         */
        DeviceIndex: C.YieldsString;
        /**
         * List of strings.

         *
         * A list of security group IDs associated with this network interface.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-network-iface-embedded.html#aws-properties-ec2-network-iface-embedded-groupset
         */
        GroupSet?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * Integer

         *
         * The number of IPv6 addresses to associate with the network interface. Amazon EC2
 automatically selects the IPv6 addresses from the subnet range. To specify
 specific IPv6 addresses, use the Ipv6Addresses property and don't
 specify this property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-network-iface-embedded.html#cfn-ec2-instance-networkinterface-ipv6addresscount
         */
        Ipv6AddressCount?: C.Yields<C.CFInteger>;
        /**
         * List of EC2 NetworkInterface
    Ipv6Addresses
         *
         * One or more specific IPv6 addresses from the IPv6 CIDR block range of your
 subnet to associate with the network interface. To specify a number of IPv6
 addresses, use the Ipv6AddressCount property and don't specify this
 property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-network-iface-embedded.html#cfn-ec2-instance-networkinterface-ipv6addresses
         */
        Ipv6Addresses?: C.Yields<C.CFList<C.Yields<AWS.EC2.Instance.InstanceIpv6Address>>>;
        /**
         * String

         *
         * An existing network interface ID.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-network-iface-embedded.html#aws-properties-ec2-network-iface-embedded-network-iface
         */
        NetworkInterfaceId?: C.YieldsString;
        /**
         * String

         *
         * Assigns a single private IP address to the network interface, which is used as
 the primary private IP address. If you want to specify multiple private IP
 address, use the PrivateIpAddresses property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-network-iface-embedded.html#aws-properties-ec2-network-iface-embedded-privateipaddress
         */
        PrivateIpAddress?: C.YieldsString;
        /**
         * list of PrivateIpAddressSpecification
         *
         * Assigns a list of private IP addresses to the network interface. You can
 specify a primary private IP address by setting the value of the
 Primary property to true in the
 PrivateIpAddressSpecification property. If you want Amazon EC2 to
 automatically assign private IP addresses, use the
 SecondaryPrivateIpCount property and do not specify this
 property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-network-iface-embedded.html#aws-properties-ec2-network-iface-embedded-privateipaddresses
         */
        PrivateIpAddresses?: C.Yields<C.CFList<C.Yields<AWS.EC2.Instance.PrivateIpAddressSpecification>>>;
        /**
         * Integer.

         *
         * The number of secondary private IP addresses that Amazon EC2 auto assigns to the
 network interface. Amazon EC2 uses the value of the PrivateIpAddress
 property as the primary private IP address. If you don't specify that property,
 Amazon EC2 auto assigns both the primary and secondary private IP addresses.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-network-iface-embedded.html#aws-properties-ec2-network-iface-embedded-secondprivateip
         */
        SecondaryPrivateIpAddressCount?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The ID of the subnet to associate with the network interface.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-network-iface-embedded.html#aws-properties-ec2-network-iface-embedded-subnetid
         */
        SubnetId?: C.YieldsString;
    }
}
export declare namespace AWS.EC2.Instance {
    interface NoDevice {
    }
}
export declare namespace AWS.EC2.Instance {
    interface PrivateIpAddressSpecification {
        /**
         * Boolean

         *
         * Sets the private IP address as the primary private address. You can set only
 one primary private IP address. If you don't specify a primary private IP address,
 Amazon EC2 automatically assigns a primary private IP address.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-network-interface-privateipspec.html#cfn-ec2-networkinterface-privateipspecification-primary
         */
        Primary: C.YieldsBoolean;
        /**
         * String

         *
         * The private IP address of the network interface.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-network-interface-privateipspec.html#cfn-ec2-networkinterface-privateipspecification-privateipaddress
         */
        PrivateIpAddress: C.YieldsString;
    }
}
export declare namespace AWS.EC2.Instance {
    interface SsmAssociation {
        /**
         * List of Amazon EC2
    Instance SsmAssociations AssociationParameters
         *
         * The input parameter values to use with the associated SSM document.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance-ssmassociations.html#cfn-ec2-instance-ssmassociations-associationparameters
         */
        AssociationParameters?: C.Yields<C.CFList<C.Yields<AWS.EC2.Instance.AssociationParameter>>>;
        /**
         * String

         *
         * The name of an SSM document to associate with the instance.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-instance-ssmassociations.html#cfn-ec2-instance-ssmassociations-documentname
         */
        DocumentName: C.YieldsString;
    }
}
export declare namespace AWS.EC2.Instance {
    interface Volume {
        /**
         * String

         *
         * How the device is exposed to the instance (such as /dev/sdh, or xvdh).
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-mount-point.html#cfn-ec2-mountpoint-device
         */
        Device: C.YieldsString;
        /**
         * String

         *
         * The ID of the Amazon EBS volume. The volume and instance must be within the same Availability
 Zone
 and the instance must be running.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-mount-point.html#cfn-ec2-mountpoint-volumeid
         */
        VolumeId: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * Creates a new Internet gateway in your AWS account. After creating the Internet gateway,
    you then attach it to
    a VPC.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-internetgateway.html
     */
    interface InternetGateway {
        Type: 'AWS::EC2::InternetGateway';
        Properties: AWS.EC2.InternetGateway.Properties;
    }
    /**
     * Creates a new Internet gateway in your AWS account. After creating the Internet gateway,
    you then attach it to
    a VPC.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-internetgateway.html
     */
    function InternetGateway(props: C.Omit<InternetGateway, 'Type'>): InternetGateway;
}
export declare namespace AWS.EC2.InternetGateway {
    interface Properties {
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key–value pairs) for this resource.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-internetgateway.html#cfn-ec2-internetgateway-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
    }
}
export declare namespace AWS.EC2 {
    /**
     * The AWS::EC2::NatGateway resource creates a network address translation (NAT)
    gateway in the specified public subnet. Use a NAT gateway to allow instances in a
    private subnet
    to connect to the Internet or to other AWS services, but prevent the Internet from
    initiating a
    connection with those instances. For more information and a sample architectural diagram,
    see
    NAT Gateways in the
    Amazon VPC User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-natgateway.html
     */
    interface NatGateway {
        Type: 'AWS::EC2::NatGateway';
        Properties: AWS.EC2.NatGateway.Properties;
    }
    /**
     * The AWS::EC2::NatGateway resource creates a network address translation (NAT)
    gateway in the specified public subnet. Use a NAT gateway to allow instances in a
    private subnet
    to connect to the Internet or to other AWS services, but prevent the Internet from
    initiating a
    connection with those instances. For more information and a sample architectural diagram,
    see
    NAT Gateways in the
    Amazon VPC User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-natgateway.html
     */
    function NatGateway(props: C.Omit<NatGateway, 'Type'>): NatGateway;
}
export declare namespace AWS.EC2.NatGateway {
    interface Properties {
        /**
         * String

         *
         * The allocation ID of an Elastic IP address to associate with the NAT gateway. If the
 Elastic IP address is associated with another resource, you must first disassociate
 it.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-natgateway.html#cfn-ec2-natgateway-allocationid
         */
        AllocationId: C.YieldsString;
        /**
         * String

         *
         * The public subnet in which to create the NAT gateway.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-natgateway.html#cfn-ec2-natgateway-subnetid
         */
        SubnetId: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * Creates a new network ACL in a VPC.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-acl.html
     */
    interface NetworkAcl {
        Type: 'AWS::EC2::NetworkAcl';
        Properties: AWS.EC2.NetworkAcl.Properties;
    }
    /**
     * Creates a new network ACL in a VPC.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-acl.html
     */
    function NetworkAcl(props: C.Omit<NetworkAcl, 'Type'>): NetworkAcl;
}
export declare namespace AWS.EC2.NetworkAcl {
    interface Properties {
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key–value pairs) for this ACL.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-acl.html#cfn-ec2-networkacl-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * String

         *
         * The ID of the VPC where the network ACL will be created.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-acl.html#cfn-ec2-networkacl-vpcid
         */
        VpcId: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * Creates an entry (i.e., a rule) in a network ACL with a rule number you specify. Each
    network ACL has a set of numbered ingress rules and a separate set of numbered egress
    rules.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-acl-entry.html
     */
    interface NetworkAclEntry {
        Type: 'AWS::EC2::NetworkAclEntry';
        Properties: AWS.EC2.NetworkAclEntry.Properties;
    }
    /**
     * Creates an entry (i.e., a rule) in a network ACL with a rule number you specify. Each
    network ACL has a set of numbered ingress rules and a separate set of numbered egress
    rules.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-acl-entry.html
     */
    function NetworkAclEntry(props: C.Omit<NetworkAclEntry, 'Type'>): NetworkAclEntry;
}
export declare namespace AWS.EC2.NetworkAclEntry {
    interface Properties {
        /**
         * String

         *
         * The IPv4 CIDR range to allow or deny, in CIDR notation (e.g., 172.16.0.0/24).
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-acl-entry.html#cfn-ec2-networkaclentry-cidrblock
         */
        CidrBlock: C.YieldsString;
        /**
         * Boolean

         *
         * Whether this rule applies to egress traffic from the subnet (true)
 or ingress traffic to the subnet (false). By default, AWS CloudFormation specifies
 false.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-acl-entry.html#cfn-ec2-networkaclentry-egress
         */
        Egress?: C.YieldsBoolean;
        /**
         * EC2 NetworkAclEntry Icmp
         *
         * The Internet Control Message Protocol (ICMP) code and type.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-acl-entry.html#cfn-ec2-networkaclentry-icmp
         */
        Icmp?: C.Yields<AWS.EC2.NetworkAclEntry.Icmp>;
        /**
         * String

         *
         * The IPv6 CIDR range to allow or deny, in CIDR notation.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-acl-entry.html#cfn-ec2-networkaclentry-ipv6cidrblock
         */
        Ipv6CidrBlock?: C.YieldsString;
        /**
         * String

         *
         * ID of the ACL where the entry will be created.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-acl-entry.html#cfn-ec2-networkaclentry-networkaclid
         */
        NetworkAclId: C.YieldsString;
        /**
         * EC2 NetworkAclEntry
    PortRange
         *
         * The range of port numbers for the UDP/TCP protocol.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-acl-entry.html#cfn-ec2-networkaclentry-portrange
         */
        PortRange?: C.Yields<AWS.EC2.NetworkAclEntry.PortRange>;
        /**
         * Number

         *
         * The IP protocol that the rule applies to. You must specify -1 or a
 protocol number (go to Protocol Numbers at iana.org). You can specify -1 for all
 protocols.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-acl-entry.html#cfn-ec2-networkaclentry-protocol
         */
        Protocol: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * Whether to allow or deny traffic that matches the rule; valid values are "allow" or
 "deny".

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-acl-entry.html#cfn-ec2-networkaclentry-ruleaction
         */
        RuleAction: C.YieldsString;
        /**
         * Number

         *
         * Rule number to assign to the entry, such as 100. ACL entries are
 processed in ascending order by rule number. Entries can't use the same rule
 number unless one is an egress rule and the other is an ingress rule. For valid
 values, see the CreateNetworkAclEntry action in the
 Amazon EC2 API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-acl-entry.html#cfn-ec2-networkaclentry-rulenumber
         */
        RuleNumber: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.EC2.NetworkAclEntry {
    interface Icmp {
        /**
         * Integer

         *
         * The Internet Control Message Protocol (ICMP) code. You can use -1 to specify all ICMP
 codes
 for the given ICMP type.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-networkaclentry-icmp.html#cfn-ec2-networkaclentry-icmp-code
         */
        Code?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The Internet Control Message Protocol (ICMP) type. You can use -1 to specify all ICMP
 types.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-networkaclentry-icmp.html#cfn-ec2-networkaclentry-icmp-type
         */
        Type?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.EC2.NetworkAclEntry {
    interface PortRange {
        /**
         * Integer

         *
         * The first port in the range.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-networkaclentry-portrange.html#cfn-ec2-networkaclentry-portrange-from
         */
        From?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The last port in the range.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-networkaclentry-portrange.html#cfn-ec2-networkaclentry-portrange-to
         */
        To?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.EC2 {
    /**
     * Describes a network interface in an Elastic Compute Cloud (EC2) instance for AWS CloudFormation.
    This is provided in a list
    in the NetworkInterfaces property of AWS::EC2::Instance.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-interface.html
     */
    interface NetworkInterface {
        Type: 'AWS::EC2::NetworkInterface';
        Properties: AWS.EC2.NetworkInterface.Properties;
    }
    /**
     * Describes a network interface in an Elastic Compute Cloud (EC2) instance for AWS CloudFormation.
    This is provided in a list
    in the NetworkInterfaces property of AWS::EC2::Instance.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-interface.html
     */
    function NetworkInterface(props: C.Omit<NetworkInterface, 'Type'>): NetworkInterface;
}
export declare namespace AWS.EC2.NetworkInterface {
    interface Properties {
        /**
         * String

         *
         * The description of this network interface.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-interface.html#cfn-awsec2networkinterface-description
         */
        Description?: C.YieldsString;
        /**
         * List of strings.

         *
         * A list of security group IDs associated with this network interface.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-interface.html#cfn-awsec2networkinterface-groupset
         */
        GroupSet?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         *
         *
         *
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-interface.html#cfn-ec2-networkinterface-interfacetype
         */
        InterfaceType?: C.YieldsString;
        /**
         * Integer

         *
         * The number of IPv6 addresses to associate with the network interface. EC2
 automatically selects the IPv6 addresses from the subnet range. To specify
 specific IPv6 addresses, use the Ipv6Addresses property and don't
 specify this property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-interface.html#cfn-ec2-networkinterface-ipv6addresscount
         */
        Ipv6AddressCount?: C.Yields<C.CFInteger>;
        /**
         * List of EC2 NetworkInterface
    Ipv6Addresses
         *
         * One or more specific IPv6 addresses from the IPv6 CIDR block range of your
 subnet to associate with the network interface. If you're specifying a number of
 IPv6 addresses, use the Ipv6AddressCount property and don't specify
 this property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-interface.html#cfn-ec2-networkinterface-ipv6addresses
         */
        Ipv6Addresses?: C.Yields<AWS.EC2.NetworkInterface.InstanceIpv6Address>;
        /**
         * String

         *
         * Assigns a single private IP address to the network interface, which is used as
 the primary private IP address. If you want to specify multiple private IP
 address, use the PrivateIpAddresses property.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-interface.html#cfn-awsec2networkinterface-privateipaddress
         */
        PrivateIpAddress?: C.YieldsString;
        /**
         * list of PrivateIpAddressSpecification.

         *
         * Assigns a list of private IP addresses to the network interface. You can
 specify a primary private IP address by setting the value of the
 Primary property to true in the
 PrivateIpAddressSpecification property. If you want EC2 to
 automatically assign private IP addresses, use the
 SecondaryPrivateIpAddressCount property and do not specify this
 property.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-interface.html#cfn-awsec2networkinterface-privateipaddresses
         */
        PrivateIpAddresses?: C.Yields<C.CFList<C.Yields<AWS.EC2.NetworkInterface.PrivateIpAddressSpecification>>>;
        /**
         * Integer.

         *
         * The number of secondary private IP addresses that EC2 automatically assigns
 to the network interface. EC2 uses the value of the
 PrivateIpAddress property as the primary private IP address. If
 you don't specify that property, EC2 automatically assigns both the primary and
 secondary private IP addresses.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-interface.html#cfn-awsec2networkinterface-secondaryprivateipcount
         */
        SecondaryPrivateIpAddressCount?: C.Yields<C.CFInteger>;
        /**
         * Boolean

         *
         * Flag indicating whether traffic to or from the instance is validated.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-interface.html#cfn-awsec2networkinterface-sourcedestcheck
         */
        SourceDestCheck?: C.YieldsBoolean;
        /**
         * String

         *
         * The ID of the subnet to associate with the network interface.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-interface.html#cfn-awsec2networkinterface-subnetid
         */
        SubnetId: C.YieldsString;
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key–value pairs) for this network
 interface.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-interface.html#cfn-awsec2networkinterface-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
    }
}
export declare namespace AWS.EC2.NetworkInterface {
    interface InstanceIpv6Address {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-networkinterface-instanceipv6address.html#cfn-ec2-networkinterface-instanceipv6address-ipv6address
         */
        Ipv6Address: C.YieldsString;
    }
}
export declare namespace AWS.EC2.NetworkInterface {
    interface PrivateIpAddressSpecification {
        /**
         * Boolean

         *
         * Sets the private IP address as the primary private address. You can set only
 one primary private IP address. If you don't specify a primary private IP address,
 Amazon EC2 automatically assigns a primary private IP address.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-network-interface-privateipspec.html#cfn-ec2-networkinterface-privateipspecification-primary
         */
        Primary: C.YieldsBoolean;
        /**
         * String

         *
         * The private IP address of the network interface.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-network-interface-privateipspec.html#cfn-ec2-networkinterface-privateipspecification-privateipaddress
         */
        PrivateIpAddress: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * Attaches an elastic network interface (ENI) to an Amazon EC2 instance. You can use
    this resource
    type to attach additional network interfaces to an instances without interruption.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-interface-attachment.html
     */
    interface NetworkInterfaceAttachment {
        Type: 'AWS::EC2::NetworkInterfaceAttachment';
        Properties: AWS.EC2.NetworkInterfaceAttachment.Properties;
    }
    /**
     * Attaches an elastic network interface (ENI) to an Amazon EC2 instance. You can use
    this resource
    type to attach additional network interfaces to an instances without interruption.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-interface-attachment.html
     */
    function NetworkInterfaceAttachment(props: C.Omit<NetworkInterfaceAttachment, 'Type'>): NetworkInterfaceAttachment;
}
export declare namespace AWS.EC2.NetworkInterfaceAttachment {
    interface Properties {
        /**
         * Boolean.

         *
         * Whether to delete the network interface when the instance terminates. By
 default, this value is set to True.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-interface-attachment.html#cfn-ec2-network-interface-attachment-deleteonterm
         */
        DeleteOnTermination?: C.YieldsBoolean;
        /**
         * String.

         *
         * The network interface's position in the attachment order. For example, the
 first attached network interface has a DeviceIndex of
 0.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-interface-attachment.html#cfn-ec2-network-interface-attachment-deviceindex
         */
        DeviceIndex: C.YieldsString;
        /**
         * String.

         *
         * The ID of the instance to which you will attach the ENI.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-interface-attachment.html#cfn-ec2-network-interface-attachment-instanceid
         */
        InstanceId: C.YieldsString;
        /**
         * String.

         *
         * The ID of the ENI that you want to attach.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-network-interface-attachment.html#cfn-ec2-network-interface-attachment-networkinterfaceid
         */
        NetworkInterfaceId: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * The
    AWS::EC2::NetworkInterfacePermission resource specifies a permission for an Amazon EC2 network interface. For example,
    you can grant
    an AWS authorized partner account permission to attach the specified network interface
    to an instance in their account. For more information, see CreateNetworkInterfacePermission and NetworkInterfacePermission in the
    Amazon EC2 API Reference.
    
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-networkinterfacepermission.html
     */
    interface NetworkInterfacePermission {
        Type: 'AWS::EC2::NetworkInterfacePermission';
        Properties: AWS.EC2.NetworkInterfacePermission.Properties;
    }
    /**
     * The
    AWS::EC2::NetworkInterfacePermission resource specifies a permission for an Amazon EC2 network interface. For example,
    you can grant
    an AWS authorized partner account permission to attach the specified network interface
    to an instance in their account. For more information, see CreateNetworkInterfacePermission and NetworkInterfacePermission in the
    Amazon EC2 API Reference.
    
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-networkinterfacepermission.html
     */
    function NetworkInterfacePermission(props: C.Omit<NetworkInterfacePermission, 'Type'>): NetworkInterfacePermission;
}
export declare namespace AWS.EC2.NetworkInterfacePermission {
    interface Properties {
        /**
         *                             Type: String

         *
         * The AWS account ID.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-networkinterfacepermission.html#cfn-ec2-networkinterfacepermission-awsaccountid
         */
        AwsAccountId: C.YieldsString;
        /**
         *                             Type: String

         *
         * The ID of the network interface.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-networkinterfacepermission.html#cfn-ec2-networkinterfacepermission-networkinterfaceid
         */
        NetworkInterfaceId: C.YieldsString;
        /**
         *                             Type: String

         *
         * The type of permission to grant: INSTANCE-ATTACH or EIP-ASSOCIATE.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-networkinterfacepermission.html#cfn-ec2-networkinterfacepermission-permission
         */
        Permission: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * The AWS::EC2::PlacementGroup resource is a logical grouping of instances
    within a single Availability Zone (AZ) that enables applications to participate in
    a
    low-latency, 10 Gbps network. You create a placement group first, and then you can
    launch
    instances in the placement group.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-placementgroup.html
     */
    interface PlacementGroup {
        Type: 'AWS::EC2::PlacementGroup';
        Properties: AWS.EC2.PlacementGroup.Properties;
    }
    /**
     * The AWS::EC2::PlacementGroup resource is a logical grouping of instances
    within a single Availability Zone (AZ) that enables applications to participate in
    a
    low-latency, 10 Gbps network. You create a placement group first, and then you can
    launch
    instances in the placement group.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-placementgroup.html
     */
    function PlacementGroup(props: C.Omit<PlacementGroup, 'Type'>): PlacementGroup;
}
export declare namespace AWS.EC2.PlacementGroup {
    interface Properties {
        /**
         * String

         *
         * The placement strategy, which relates to the instance types that can be added
 to the placement group. For example, for the cluster strategy, you can
 cluster C4 instance types but not T2 instance types. For valid values, see CreatePlacementGroup in the
 Amazon EC2 API Reference. By default, AWS CloudFormation sets the value of this
 property to cluster.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-placementgroup.html#cfn-ec2-placementgroup-strategy
         */
        Strategy?: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * The AWS::EC2::Route resource creates a new route in a route table within a
    VPC. The route's target can be either a gateway attached to the VPC or a NAT instance
    in the
    VPC.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-route.html
     */
    interface Route {
        Type: 'AWS::EC2::Route';
        Properties: AWS.EC2.Route.Properties;
    }
    /**
     * The AWS::EC2::Route resource creates a new route in a route table within a
    VPC. The route's target can be either a gateway attached to the VPC or a NAT instance
    in the
    VPC.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-route.html
     */
    function Route(props: C.Omit<Route, 'Type'>): Route;
}
export declare namespace AWS.EC2.Route {
    interface Properties {
        /**
         * String

         *
         * The IPv4 CIDR address block used for the destination match. For example,
 0.0.0.0/0. Routing decisions are based on the most specific
 match.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-route.html#cfn-ec2-route-destinationcidrblock
         */
        DestinationCidrBlock?: C.YieldsString;
        /**
         * String

         *
         * The IPv6 CIDR address block used for the destination match. For example,
 ::/0. Routing decisions are based on the most specific
 match.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-route.html#cfn-ec2-route-destinationipv6cidrblock
         */
        DestinationIpv6CidrBlock?: C.YieldsString;
        /**
         * String

         *
         * The
 ID of an internet gateway or virtual private gateway that is attached to your VPC.
 For example:
 igw-eaad4883.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-route.html#cfn-ec2-route-gatewayid
         */
        GatewayId?: C.YieldsString;
        /**
         * String

         *
         * The
 ID of a NAT instance in your VPC. For example,
 i-1a2b3c4d.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-route.html#cfn-ec2-route-instanceid
         */
        InstanceId?: C.YieldsString;
        /**
         * String

         *
         * The
 ID of a NAT gateway. For example,
 nat-0a12bc456789de0fg.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-route.html#cfn-ec2-route-natgatewayid
         */
        NatGatewayId?: C.YieldsString;
        /**
         * String

         *
         * Allows
 the routing of network interface IDs.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-route.html#cfn-ec2-route-networkinterfaceid
         */
        NetworkInterfaceId?: C.YieldsString;
        /**
         * String

         *
         * The ID of the route table where the route will
 be added.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-route.html#cfn-ec2-route-routetableid
         */
        RouteTableId: C.YieldsString;
        /**
         * String

         *
         * The ID of a VPC peering connection.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-route.html#cfn-ec2-route-vpcpeeringconnectionid
         */
        VpcPeeringConnectionId?: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * Creates a new route table within a VPC. After you create a new route table, you can
    add routes and
    associate the table with a subnet.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-route-table.html
     */
    interface RouteTable {
        Type: 'AWS::EC2::RouteTable';
        Properties: AWS.EC2.RouteTable.Properties;
    }
    /**
     * Creates a new route table within a VPC. After you create a new route table, you can
    add routes and
    associate the table with a subnet.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-route-table.html
     */
    function RouteTable(props: C.Omit<RouteTable, 'Type'>): RouteTable;
}
export declare namespace AWS.EC2.RouteTable {
    interface Properties {
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key–value pairs) for this route table.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-route-table.html#cfn-ec2-routetable-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * String

         *
         * The ID of the VPC where the route table will be created.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-route-table.html#cfn-ec2-routetable-vpcid
         */
        VpcId: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * Creates an Amazon EC2 security group. To create a VPC security group, use the VpcId property.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group.html
     */
    interface SecurityGroup {
        Type: 'AWS::EC2::SecurityGroup';
        Properties: AWS.EC2.SecurityGroup.Properties;
    }
    /**
     * Creates an Amazon EC2 security group. To create a VPC security group, use the VpcId property.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group.html
     */
    function SecurityGroup(props: C.Omit<SecurityGroup, 'Type'>): SecurityGroup;
}
export declare namespace AWS.EC2.SecurityGroup {
    interface Properties {
        /**
         * String

         *
         * A description of the security group.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group.html#cfn-ec2-securitygroup-groupdescription
         */
        GroupDescription: C.YieldsString;
        /**
         * String

         *
         * The name of the security group. For valid values, see the GroupName parameter of the CreateSecurityGroup action in the Amazon EC2 API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group.html#cfn-ec2-securitygroup-groupname
         */
        GroupName?: C.YieldsString;
        /**
         * List of EC2 Security Group
    Rule
         *
         * A list of Amazon EC2 security group egress rules.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group.html#cfn-ec2-securitygroup-securitygroupegress
         */
        SecurityGroupEgress?: C.Yields<C.CFList<C.Yields<AWS.EC2.SecurityGroup.Egress>>>;
        /**
         * List of EC2 Security Group
    Rule
         *
         * A list of Amazon EC2 security group ingress rules.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group.html#cfn-ec2-securitygroup-securitygroupingress
         */
        SecurityGroupIngress?: C.Yields<C.CFList<C.Yields<AWS.EC2.SecurityGroup.Ingress>>>;
        /**
         * AWS CloudFormation Resource Tags.

         *
         * The tags that you want to attach to the resource.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group.html#cfn-ec2-securitygroup-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * String

         *
         * The physical ID of the VPC. You can obtain the physical ID by using a reference
 to an AWS::EC2::VPC, such as: {
    "Ref" : "myVPC" }.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group.html#cfn-ec2-securitygroup-vpcid
         */
        VpcId?: C.YieldsString;
    }
}
export declare namespace AWS.EC2.SecurityGroup {
    interface Egress {
        /**
         * String

         *
         * Specifies an IPv4 CIDR range.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-rule.html#cfn-ec2-security-group-rule-cidrip
         */
        CidrIp?: C.YieldsString;
        /**
         * String

         *
         * Specifies an IPv6 CIDR range.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-rule.html#cfn-ec2-security-group-rule-cidripv6
         */
        CidrIpv6?: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-rule.html#cfn-ec2-security-group-rule-destinationprefixlistid
         */
        DestinationPrefixListId?: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-rule.html#cfn-ec2-security-group-rule-destsecgroupid
         */
        DestinationSecurityGroupId?: C.YieldsString;
        /**
         * Integer

         *
         * The start of port range for the TCP and UDP protocols, or an ICMP type number.
 An ICMP type number of -1 indicates a wildcard (i.e., any ICMP type
 number).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-rule.html#cfn-ec2-security-group-rule-fromport
         */
        FromPort?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * An IP protocol name or number. For valid values, go to the IpProtocol parameter
 in AuthorizeSecurityGroupIngress
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-rule.html#cfn-ec2-security-group-rule-ipprotocol
         */
        IpProtocol: C.YieldsString;
        /**
         * Integer

         *
         * The end of port range for the TCP and UDP protocols, or an ICMP code. An ICMP
 code of -1 indicates a wildcard (i.e., any ICMP code).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-rule.html#cfn-ec2-security-group-rule-toport
         */
        ToPort?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.EC2.SecurityGroup {
    interface Ingress {
        /**
         * String

         *
         * Specifies an IPv4 CIDR range.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-rule.html#cfn-ec2-security-group-rule-cidrip
         */
        CidrIp?: C.YieldsString;
        /**
         * String

         *
         * Specifies an IPv6 CIDR range.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-rule.html#cfn-ec2-security-group-rule-cidripv6
         */
        CidrIpv6?: C.YieldsString;
        /**
         * Integer

         *
         * The start of port range for the TCP and UDP protocols, or an ICMP type number.
 An ICMP type number of -1 indicates a wildcard (i.e., any ICMP type
 number).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-rule.html#cfn-ec2-security-group-rule-fromport
         */
        FromPort?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * An IP protocol name or number. For valid values, go to the IpProtocol parameter
 in AuthorizeSecurityGroupIngress
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-rule.html#cfn-ec2-security-group-rule-ipprotocol
         */
        IpProtocol: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-rule.html#cfn-ec2-security-group-rule-sourcesecuritygroupid
         */
        SourceSecurityGroupId?: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-rule.html#cfn-ec2-security-group-rule-sourcesecuritygroupname
         */
        SourceSecurityGroupName?: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-rule.html#cfn-ec2-security-group-rule-sourcesecuritygroupownerid
         */
        SourceSecurityGroupOwnerId?: C.YieldsString;
        /**
         * Integer

         *
         * The end of port range for the TCP and UDP protocols, or an ICMP code. An ICMP
 code of -1 indicates a wildcard (i.e., any ICMP code).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-rule.html#cfn-ec2-security-group-rule-toport
         */
        ToPort?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.EC2 {
    /**
     * The AWS::EC2::SecurityGroupEgress resource adds an egress rule to an Amazon VPC
    security group.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-security-group-egress.html
     */
    interface SecurityGroupEgress {
        Type: 'AWS::EC2::SecurityGroupEgress';
        Properties: AWS.EC2.SecurityGroupEgress.Properties;
    }
    /**
     * The AWS::EC2::SecurityGroupEgress resource adds an egress rule to an Amazon VPC
    security group.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-security-group-egress.html
     */
    function SecurityGroupEgress(props: C.Omit<SecurityGroupEgress, 'Type'>): SecurityGroupEgress;
}
export declare namespace AWS.EC2.SecurityGroupEgress {
    interface Properties {
        /**
         * String

         *
         * An IPv4 CIDR range.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-security-group-egress.html#cfn-ec2-securitygroupegress-cidrip
         */
        CidrIp?: C.YieldsString;
        /**
         * String

         *
         * An IPv6 CIDR range.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-security-group-egress.html#cfn-ec2-securitygroupegress-cidripv6
         */
        CidrIpv6?: C.YieldsString;
        /**
         * String

         *
         * The AWS service prefix of an Amazon VPC endpoint. For more information, see VPC Endpoints in the
 Amazon VPC User Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-security-group-egress.html#cfn-ec2-securitygroupegress-destinationprefixlistid
         */
        DestinationPrefixListId?: C.YieldsString;
        /**
         * String

         *
         * Specifies the group ID of the destination Amazon VPC security group.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-security-group-egress.html#cfn-ec2-securitygroupegress-destinationsecuritygroupid
         */
        DestinationSecurityGroupId?: C.YieldsString;
        /**
         * Integer

         *
         * Start of port range for the TCP and UDP protocols, or an ICMP type number. If
 you specify icmp for the IpProtocol property, you can
 specify -1 as a wildcard (i.e., any ICMP type number).

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-security-group-egress.html#cfn-ec2-securitygroupegress-fromport
         */
        FromPort?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * ID of the Amazon VPC security group to modify. This value can be a reference to an
 AWS::EC2::SecurityGroup resource that has a
 valid VpcId property or the ID of an existing Amazon VPC security group.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-security-group-egress.html#cfn-ec2-securitygroupegress-groupid
         */
        GroupId: C.YieldsString;
        /**
         * String

         *
         * IP protocol name or number. For valid values, see the IpProtocol parameter in AuthorizeSecurityGroupIngress
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-security-group-egress.html#cfn-ec2-securitygroupegress-ipprotocol
         */
        IpProtocol: C.YieldsString;
        /**
         * Integer

         *
         * End of port range for the TCP and UDP protocols, or an ICMP code. If you
 specify icmp for the IpProtocol property, you can
 specify -1 as a wildcard (i.e., any ICMP code).

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-security-group-egress.html#cfn-ec2-securitygroupegress-toport
         */
        ToPort?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.EC2 {
    /**
     * The AWS::EC2::SecurityGroupIngress resource adds an ingress rule to an Amazon EC2
    or Amazon VPC security group.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-ingress.html
     */
    interface SecurityGroupIngress {
        Type: 'AWS::EC2::SecurityGroupIngress';
        Properties: AWS.EC2.SecurityGroupIngress.Properties;
    }
    /**
     * The AWS::EC2::SecurityGroupIngress resource adds an ingress rule to an Amazon EC2
    or Amazon VPC security group.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-ingress.html
     */
    function SecurityGroupIngress(props: C.Omit<SecurityGroupIngress, 'Type'>): SecurityGroupIngress;
}
export declare namespace AWS.EC2.SecurityGroupIngress {
    interface Properties {
        /**
         * String

         *
         * An IPv4 CIDR range.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-ingress.html#cfn-ec2-security-group-ingress-cidrip
         */
        CidrIp?: C.YieldsString;
        /**
         * String

         *
         * An IPv6 CIDR range.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-ingress.html#cfn-ec2-security-group-ingress-cidripv6
         */
        CidrIpv6?: C.YieldsString;
        /**
         * Integer

         *
         * Start of port range for the TCP and UDP protocols, or an ICMP type number.
 If you specify icmp for the IpProtocol property, you can
 specify -1 as a wildcard (i.e., any ICMP type number).

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-ingress.html#cfn-ec2-security-group-ingress-fromport
         */
        FromPort?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * ID of the Amazon EC2 or VPC security group to modify. The group must belong to your
 account.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-ingress.html#cfn-ec2-security-group-ingress-groupid
         */
        GroupId?: C.YieldsString;
        /**
         * String

         *
         * Name of the Amazon EC2 security group (non-VPC security group) to modify. This value
 can be a reference to an AWS::EC2::SecurityGroup resource or the name of an existing Amazon EC2
 security group.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-ingress.html#cfn-ec2-security-group-ingress-groupname
         */
        GroupName?: C.YieldsString;
        /**
         * String

         *
         * IP protocol name or number. For valid values, see the IpProtocol parameter
 in AuthorizeSecurityGroupIngress
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-ingress.html#cfn-ec2-security-group-ingress-ipprotocol
         */
        IpProtocol: C.YieldsString;
        /**
         * String

         *
         * Specifies the ID of the source security group or uses the Ref
 intrinsic function to refer to the logical ID of a security group defined in the
 same template.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-ingress.html#cfn-ec2-security-group-ingress-sourcesecuritygroupid
         */
        SourceSecurityGroupId?: C.YieldsString;
        /**
         * String

         *
         * Specifies the name of the Amazon EC2 security group (non-VPC security group) to
 allow access or use the Ref intrinsic function to refer to the logical ID
 of a security group defined in the same template. For instances in a VPC, specify
 the
 SourceSecurityGroupId property.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-ingress.html#cfn-ec2-security-group-ingress-sourcesecuritygroupname
         */
        SourceSecurityGroupName?: C.YieldsString;
        /**
         * String

         *
         * Specifies the AWS Account ID of the owner of the Amazon EC2 security group
 specified in the SourceSecurityGroupName property.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-ingress.html#cfn-ec2-security-group-ingress-sourcesecuritygroupownerid
         */
        SourceSecurityGroupOwnerId?: C.YieldsString;
        /**
         * Integer

         *
         * End of port range for the TCP and UDP protocols, or an ICMP code. If you
 specify icmp for the IpProtocol property, you can
 specify -1 as a wildcard (i.e., any ICMP code).

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group-ingress.html#cfn-ec2-security-group-ingress-toport
         */
        ToPort?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.EC2 {
    /**
     * The AWS::EC2::SpotFleet resource creates a request for a collection of Spot
    instances. The Spot fleet attempts to launch the number of Spot instances to meet
    the target
    capacity that you specified. For more information, see Spot Instances in the
    Amazon EC2 User Guide for Linux Instances.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-spotfleet.html
     */
    interface SpotFleet {
        Type: 'AWS::EC2::SpotFleet';
        Properties: AWS.EC2.SpotFleet.Properties;
    }
    /**
     * The AWS::EC2::SpotFleet resource creates a request for a collection of Spot
    instances. The Spot fleet attempts to launch the number of Spot instances to meet
    the target
    capacity that you specified. For more information, see Spot Instances in the
    Amazon EC2 User Guide for Linux Instances.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-spotfleet.html
     */
    function SpotFleet(props: C.Omit<SpotFleet, 'Type'>): SpotFleet;
}
export declare namespace AWS.EC2.SpotFleet {
    interface Properties {
        /**
         * Amazon EC2 SpotFleet
    SpotFleetRequestConfigData
         *
         * The configuration for a Spot fleet request.
         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-spotfleet.html#cfn-ec2-spotfleet-spotfleetrequestconfigdata
         */
        SpotFleetRequestConfigData: C.Yields<AWS.EC2.SpotFleet.SpotFleetRequestConfigData>;
    }
}
export declare namespace AWS.EC2.SpotFleet {
    interface BlockDeviceMapping {
        /**
         * String

         *
         * The name of the device within the EC2 instance, such as /dev/dsh
 or xvdh.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-blockdevicemappings.html#cfn-ec2-spotfleet-blockdevicemapping-devicename
         */
        DeviceName: C.YieldsString;
        /**
         * Amazon Elastic Compute Cloud SpotFleet SpotFleetRequestConfigData LaunchSpecifications
    BlockDeviceMappings
    Ebs
         *
         * The Amazon Elastic Block Store (Amazon EBS) volume information.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-blockdevicemappings.html#cfn-ec2-spotfleet-blockdevicemapping-ebs
         */
        Ebs?: C.Yields<AWS.EC2.SpotFleet.EbsBlockDevice>;
        /**
         * Boolean

         *
         * Suppresses the specified device that is included in the block device mapping of
 the Amazon Machine Image (AMI).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-blockdevicemappings.html#cfn-ec2-spotfleet-blockdevicemapping-nodevice
         */
        NoDevice?: C.YieldsString;
        /**
         * String

         *
         * The name of the virtual device. The name must be in the form
 ephemeralX where
 X is a number equal to or greater than zero (0), for
 example, ephemeral0.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-blockdevicemappings.html#cfn-ec2-spotfleet-blockdevicemapping-virtualname
         */
        VirtualName?: C.YieldsString;
    }
}
export declare namespace AWS.EC2.SpotFleet {
    interface EbsBlockDevice {
        /**
         * Boolean

         *
         * Indicates whether to delete the volume when the instance is terminated.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-blockdevicemappings-ebs.html#cfn-ec2-spotfleet-ebsblockdevice-deleteontermination
         */
        DeleteOnTermination?: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Indicates whether the EBS volume is encrypted. Encrypted Amazon EBS volumes can be
 attached only to instances that support Amazon EBS encryption.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-blockdevicemappings-ebs.html#cfn-ec2-spotfleet-ebsblockdevice-encrypted
         */
        Encrypted?: C.YieldsBoolean;
        /**
         * Integer

         *
         * The number of I/O operations per second (IOPS) that the volume supports. For
 more information, see Iops for the EbsBlockDevice action in the
 Amazon EC2 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-blockdevicemappings-ebs.html#cfn-ec2-spotfleet-ebsblockdevice-iops
         */
        Iops?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The snapshot ID of the volume that you want to use. If you specify both the
 SnapshotId and VolumeSize properties,
 VolumeSize must be equal to or greater than the size of the
 snapshot.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-blockdevicemappings-ebs.html#cfn-ec2-spotfleet-ebsblockdevice-snapshotid
         */
        SnapshotId?: C.YieldsString;
        /**
         * Integer

         *
         * The volume size, in Gibibytes (GiB). If you specify both the SnapshotId
 and VolumeSize properties, VolumeSize must be equal to
 or greater than the size of the snapshot. For more information about specifying
 the volume size, see VolumeSize for the EbsBlockDevice action in the
 Amazon EC2 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-blockdevicemappings-ebs.html#cfn-ec2-spotfleet-ebsblockdevice-volumesize
         */
        VolumeSize?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The volume type. For more information about specifying the volume type, see
 VolumeType for the
 EbsBlockDevice action in the
 Amazon EC2 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-blockdevicemappings-ebs.html#cfn-ec2-spotfleet-ebsblockdevice-volumetype
         */
        VolumeType?: C.YieldsString;
    }
}
export declare namespace AWS.EC2.SpotFleet {
    interface GroupIdentifier {
        /**
         * String

         *
         * The ID of a security group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-securitygroups.html#cfn-ec2-spotfleet-groupidentifier-groupid
         */
        GroupId: C.YieldsString;
    }
}
export declare namespace AWS.EC2.SpotFleet {
    interface IamInstanceProfileSpecification {
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the instance profile to associate with the
 instances. The instance profile contains the IAM role that is associated with
 the instances.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-iaminstanceprofile.html#cfn-ec2-spotfleet-iaminstanceprofilespecification-arn
         */
        Arn?: C.YieldsString;
    }
}
export declare namespace AWS.EC2.SpotFleet {
    interface InstanceIpv6Address {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-instanceipv6address.html#cfn-ec2-spotfleet-instanceipv6address-ipv6address
         */
        Ipv6Address: C.YieldsString;
    }
}
export declare namespace AWS.EC2.SpotFleet {
    interface InstanceNetworkInterfaceSpecification {
        /**
         * Boolean

         *
         * Indicates whether to assign a public IP address to an instance that you launch
 in a VPC. You can assign the public IP address can only to a network interface for
 eth0, and only to a new network interface, not an existing one.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-networkinterfaces.html#cfn-ec2-spotfleet-instancenetworkinterfacespecification-associatepublicipaddress
         */
        AssociatePublicIpAddress?: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Indicates whether to delete the network interface when the instance
 terminates.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-networkinterfaces.html#cfn-ec2-spotfleet-instancenetworkinterfacespecification-deleteontermination
         */
        DeleteOnTermination?: C.YieldsBoolean;
        /**
         * String

         *
         * The description of this network interface.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-networkinterfaces.html#cfn-ec2-spotfleet-instancenetworkinterfacespecification-description
         */
        Description?: C.YieldsString;
        /**
         * Integer

         *
         * The network interface's position in the attachment order.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-networkinterfaces.html#cfn-ec2-spotfleet-instancenetworkinterfacespecification-deviceindex
         */
        DeviceIndex?: C.Yields<C.CFInteger>;
        /**
         * List of strings

         *
         * A list of security group IDs to associate with this network interface.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-networkinterfaces.html#cfn-ec2-spotfleet-instancenetworkinterfacespecification-groups
         */
        Groups?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * Integer

         *
         * The number of IPv6 addresses to associate with the network interface. Amazon Elastic
 Compute Cloud
 automatically selects the IPv6 addresses from the subnet range. To specify
 specific IPv6 addresses, use the Ipv6Addresses property and don't
 specify this property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-networkinterfaces.html#cfn-ec2-spotfleet-instancenetworkinterfacespecification-ipv6addresscount
         */
        Ipv6AddressCount?: C.Yields<C.CFInteger>;
        /**
         * List of EC2 NetworkInterface
    Ipv6Addresses
         *
         * One or more specific IPv6 addresses from the IPv6 CIDR block range of your
 subnet to associate with the network interface. To specify a number of IPv6
 addresses, use the Ipv6AddressCount property and don't specify this
 property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-networkinterfaces.html#cfn-ec2-spotfleet-instancenetworkinterfacespecification-ipv6addresses
         */
        Ipv6Addresses?: C.Yields<C.CFList<C.Yields<AWS.EC2.SpotFleet.InstanceIpv6Address>>>;
        /**
         * String

         *
         * A network interface ID.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-networkinterfaces.html#cfn-ec2-spotfleet-instancenetworkinterfacespecification-networkinterfaceid
         */
        NetworkInterfaceId?: C.YieldsString;
        /**
         * List of Amazon Elastic Compute Cloud SpotFleet SpotFleetRequestConfigData LaunchSpecifications
    NetworkInterfaces
    PrivateIpAddresses
         *
         * One or more private IP addresses to assign to the network interface.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-networkinterfaces.html#cfn-ec2-spotfleet-instancenetworkinterfacespecification-privateipaddresses
         */
        PrivateIpAddresses?: C.Yields<C.CFList<C.Yields<AWS.EC2.SpotFleet.PrivateIpAddressSpecification>>>;
        /**
         * Integer

         *
         * The number of secondary private IP addresses that Amazon EC2 automatically assigns
 to the network interface.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-networkinterfaces.html#cfn-ec2-spotfleet-instancenetworkinterfacespecification-secondaryprivateipaddresscount
         */
        SecondaryPrivateIpAddressCount?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The ID of the subnet to associate with the network interface.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-networkinterfaces.html#cfn-ec2-spotfleet-instancenetworkinterfacespecification-subnetid
         */
        SubnetId?: C.YieldsString;
    }
}
export declare namespace AWS.EC2.SpotFleet {
    interface PrivateIpAddressSpecification {
        /**
         * Boolean

         *
         * Indicates whether the private IP address is the primary private IP address. You
 can designate only one IP address as primary.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-networkinterfaces-privateipaddresses.html#cfn-ec2-spotfleet-privateipaddressspecification-primary
         */
        Primary?: C.YieldsBoolean;
        /**
         * String

         *
         * The private IP address.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-networkinterfaces-privateipaddresses.html#cfn-ec2-spotfleet-privateipaddressspecification-privateipaddress
         */
        PrivateIpAddress: C.YieldsString;
    }
}
export declare namespace AWS.EC2.SpotFleet {
    interface SpotFleetLaunchSpecification {
        /**
         * List of Amazon Elastic Compute Cloud SpotFleet SpotFleetRequestConfigData LaunchSpecifications
    BlockDeviceMappings
         *
         * Defines the block devices that are mapped to the Spot instances.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications.html#cfn-ec2-spotfleet-spotfleetlaunchspecification-blockdevicemappings
         */
        BlockDeviceMappings?: C.Yields<C.CFList<C.Yields<AWS.EC2.SpotFleet.BlockDeviceMapping>>>;
        /**
         * Boolean

         *
         * Indicates whether the instances are optimized for Amazon Elastic Block Store (Amazon
 EBS) I/O. This
 optimization provides dedicated throughput to Amazon EBS and an optimized configuration
 stack to provide optimal EBS I/O performance. This optimization isn't available
 with all instance types. Additional usage charges apply when you use an
 Amazon EBS-optimized instance.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications.html#cfn-ec2-spotfleet-spotfleetlaunchspecification-ebsoptimized
         */
        EbsOptimized?: C.YieldsBoolean;
        /**
         * Amazon Elastic Compute Cloud SpotFleet SpotFleetRequestConfigData LaunchSpecifications
    IamInstanceProfile
         *
         * Defines the AWS Identity and Access Management (IAM) instance profile to associate
 with the
 instances.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications.html#cfn-ec2-spotfleet-spotfleetlaunchspecification-iaminstanceprofile
         */
        IamInstanceProfile?: C.Yields<AWS.EC2.SpotFleet.IamInstanceProfileSpecification>;
        /**
         * String

         *
         * The unique ID of the Amazon Machine Image (AMI) to launch on the
 instances.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications.html#cfn-ec2-spotfleet-spotfleetlaunchspecification-imageid
         */
        ImageId: C.YieldsString;
        /**
         * String

         *
         * Specifies the instance type of the EC2 instances.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications.html#cfn-ec2-spotfleet-spotfleetlaunchspecification-instancetype
         */
        InstanceType: C.YieldsString;
        /**
         * String

         *
         * The ID of the kernel that is associated with the Amazon Elastic Compute Cloud (Amazon
 EC2) AMI.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications.html#cfn-ec2-spotfleet-spotfleetlaunchspecification-kernelid
         */
        KernelId?: C.YieldsString;
        /**
         * String

         *
         * An Amazon EC2 key pair to associate with the instances.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications.html#cfn-ec2-spotfleet-spotfleetlaunchspecification-keyname
         */
        KeyName?: C.YieldsString;
        /**
         * Amazon EC2 SpotFleet SpotFleetRequestConfigData LaunchSpecifications Monitoring
         *
         * Enable or disable monitoring for the instances.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications.html#cfn-ec2-spotfleet-spotfleetlaunchspecification-monitoring
         */
        Monitoring?: C.Yields<AWS.EC2.SpotFleet.SpotFleetMonitoring>;
        /**
         * List of Amazon Elastic Compute Cloud SpotFleet SpotFleetRequestConfigData LaunchSpecifications
    NetworkInterfaces
         *
         * The network interfaces to associate with the instances.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications.html#cfn-ec2-spotfleet-spotfleetlaunchspecification-networkinterfaces
         */
        NetworkInterfaces?: C.Yields<C.CFList<C.Yields<AWS.EC2.SpotFleet.InstanceNetworkInterfaceSpecification>>>;
        /**
         * Amazon Elastic Compute Cloud SpotFleet SpotFleetRequestConfigData LaunchSpecifications
    Placement
         *
         * Defines a placement group, which is a logical grouping of instances within a
 single Availability Zone (AZ).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications.html#cfn-ec2-spotfleet-spotfleetlaunchspecification-placement
         */
        Placement?: C.Yields<AWS.EC2.SpotFleet.SpotPlacement>;
        /**
         * String

         *
         * The ID of the RAM disk to select. Some kernels require additional drivers at
 launch. Check the kernel requirements for information about whether you need to
 specify a RAM disk. To find kernel requirements, refer to the AWS Resource Center
 and search for the kernel ID.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications.html#cfn-ec2-spotfleet-spotfleetlaunchspecification-ramdiskid
         */
        RamdiskId?: C.YieldsString;
        /**
         * List of Amazon Elastic Compute Cloud SpotFleet SpotFleetRequestConfigData LaunchSpecifications
    SecurityGroups
         *
         * One or more security group IDs to associate with the instances.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications.html#cfn-ec2-spotfleet-spotfleetlaunchspecification-securitygroups
         */
        SecurityGroups?: C.Yields<C.CFList<C.Yields<AWS.EC2.SpotFleet.GroupIdentifier>>>;
        /**
         * String

         *
         * The bid price per unit hour for the specified instance type. If you don't specify
 a value, Amazon EC2 uses the Spot bid price for the fleet. For more information, see
 How Spot Fleet Works in the Amazon EC2 User Guide for Linux Instances.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications.html#cfn-ec2-spotfleet-spotfleetlaunchspecification-spotprice
         */
        SpotPrice?: C.YieldsString;
        /**
         * String

         *
         * The ID of the subnet in which to launch the instances.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications.html#cfn-ec2-spotfleet-spotfleetlaunchspecification-subnetid
         */
        SubnetId?: C.YieldsString;
        /**
         * String

         *
         * Base64-encoded MIME user data that instances use when starting up.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications.html#cfn-ec2-spotfleet-spotfleetlaunchspecification-userdata
         */
        UserData?: C.YieldsString;
        /**
         * Number

         *
         * The number of units provided by the specified instance type. These units are
 the same units that you chose to set the target capacity in terms of instances or
 a performance characteristic, such as vCPUs, memory, or I/O. For more information,
 see How Spot Fleet Works in the
 Amazon EC2 User Guide for Linux Instances.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications.html#cfn-ec2-spotfleet-spotfleetlaunchspecification-weightedcapacity
         */
        WeightedCapacity?: C.Yields<C.CFDouble>;
    }
}
export declare namespace AWS.EC2.SpotFleet {
    interface SpotFleetMonitoring {
        /**
         * Boolean

         *
         * Indicates whether monitoring is enabled for the instances.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-monitoring.html#cfn-ec2-spotfleet-spotfleetmonitoring-enabled
         */
        Enabled?: C.YieldsBoolean;
    }
}
export declare namespace AWS.EC2.SpotFleet {
    interface SpotFleetRequestConfigData {
        /**
         * String

         *
         * Indicates how to allocate the target capacity across the Spot pools that you
 specified in the Spot fleet request. For valid values, see SpotFleetRequestConfigData in the
 Amazon EC2 API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata.html#cfn-ec2-spotfleet-spotfleetrequestconfigdata-allocationstrategy
         */
        AllocationStrategy?: C.YieldsString;
        /**
         * String

         *
         * Indicates whether running Spot instances are terminated if you decrease the
 target capacity of the Spot fleet request below the current size of the Spot
 fleet. For valid values, see SpotFleetRequestConfigData in the
 Amazon EC2 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata.html#cfn-ec2-spotfleet-spotfleetrequestconfigdata-excesscapacityterminationpolicy
         */
        ExcessCapacityTerminationPolicy?: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of an AWS Identity and Access Management (IAM) role
 that grants the
 Spot fleet the ability to bid on, launch, and terminate instances on your behalf.
 For more information, see Spot
    Fleet Prerequisites in the
 Amazon EC2 User Guide for Linux Instances.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata.html#cfn-ec2-spotfleet-spotfleetrequestconfigdata-iamfleetrole
         */
        IamFleetRole: C.YieldsString;
        /**
         * List of Amazon Elastic Compute Cloud SpotFleet SpotFleetRequestConfigData LaunchSpecifications
         *
         * The launch specifications for the Spot fleet request.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata.html#cfn-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications
         */
        LaunchSpecifications: C.Yields<C.CFList<C.Yields<AWS.EC2.SpotFleet.SpotFleetLaunchSpecification>>>;
        /**
         * Boolean

         *
         * Indicates whether the Spot fleet should replace unhealthy instances.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata.html#cfn-ec2-spotfleet-spotfleetrequestconfigdata-replaceunhealthyinstances
         */
        ReplaceUnhealthyInstances?: C.YieldsBoolean;
        /**
         * String

         *
         * The bid price per unit hour. For more information, see How Spot Fleet Works in the
 Amazon EC2 User Guide for Linux Instances.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata.html#cfn-ec2-spotfleet-spotfleetrequestconfigdata-spotprice
         */
        SpotPrice: C.YieldsString;
        /**
         * Integer

         *
         * The number of units to request for the spot fleet. You can choose to set the
 target capacity as the number of instances or as a performance characteristic that
 is important to your application workload, such as vCPUs, memory, or I/O. For more
 information, see How Spot Fleet
    Works in the Amazon EC2 User Guide for Linux Instances.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata.html#cfn-ec2-spotfleet-spotfleetrequestconfigdata-targetcapacity
         */
        TargetCapacity: C.Yields<C.CFInteger>;
        /**
         * Boolean

         *
         * Indicates whether running Spot instances are terminated when the Spot fleet
 request expires.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata.html#cfn-ec2-spotfleet-spotfleetrequestconfigdata-terminateinstanceswithexpiration
         */
        TerminateInstancesWithExpiration?: C.YieldsBoolean;
        /**
         * String

         *
         * The type of request, which indicates whether the fleet will only request the target
 capacity or
 also attempt to maintain it.
 For more information, see SpotFleetRequestConfigData
 in the Amazon EC2 API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata.html#cfn-ec2-spotfleet-spotfleetrequestconfigdata-type
         */
        Type?: C.YieldsString;
        /**
         * String

         *
         * The start date and time of the request, in UTC format
 (YYYY-MM-DDTHH:MM:SSZ).
 By default, Amazon Elastic Compute Cloud (Amazon EC2 ) starts fulfilling the request
 immediately.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata.html#cfn-ec2-spotfleet-spotfleetrequestconfigdata-validfrom
         */
        ValidFrom?: C.YieldsString;
        /**
         * String

         *
         * The end date and time of the request, in UTC format
 (YYYY-MM-DDTHH:MM:SSZ).
 After the end date and time, Amazon EC2 doesn't request new Spot instances or enable
 them to fulfill the request.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata.html#cfn-ec2-spotfleet-spotfleetrequestconfigdata-validuntil
         */
        ValidUntil?: C.YieldsString;
    }
}
export declare namespace AWS.EC2.SpotFleet {
    interface SpotPlacement {
        /**
         * String

         *
         * The Availability Zone (AZ) of the placement group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-placement.html#cfn-ec2-spotfleet-spotplacement-availabilityzone
         */
        AvailabilityZone?: C.YieldsString;
        /**
         * String

         *
         * The name of the placement group (for cluster instances).
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-spotfleet-spotfleetrequestconfigdata-launchspecifications-placement.html#cfn-ec2-spotfleet-spotplacement-groupname
         */
        GroupName?: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * Creates a subnet in an existing VPC.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnet.html
     */
    interface Subnet {
        Type: 'AWS::EC2::Subnet';
        Properties: AWS.EC2.Subnet.Properties;
    }
    /**
     * Creates a subnet in an existing VPC.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnet.html
     */
    function Subnet(props: C.Omit<Subnet, 'Type'>): Subnet;
}
export declare namespace AWS.EC2.Subnet {
    interface Properties {
        /**
         * Boolean

         *
         * Indicates whether a network interface created in this subnet receives an IPv6 address.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnet.html#cfn-ec2-subnet-assignipv6addressoncreation
         */
        AssignIpv6AddressOnCreation?: C.YieldsBoolean;
        /**
         * String

         *
         * The availability zone in which you want the subnet. Default: AWS selects a zone
 for you (recommended).

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnet.html#cfn-ec2-subnet-availabilityzone
         */
        AvailabilityZone?: C.YieldsString;
        /**
         * String

         *
         * The CIDR block that you want the subnet to cover (for example,
 "10.0.0.0/24").

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnet.html#cfn-ec2-subnet-cidrblock
         */
        CidrBlock: C.YieldsString;
        /**
         * String

         *
         * The IPv6 network range for the subnet, in CIDR notation. The subnet size must use
 a /64 prefix length.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnet.html#cfn-ec2-subnet-ipv6cidrblock
         */
        Ipv6CidrBlock?: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether instances that are launched in this subnet receive a public
 IP address. By default, the value is false.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnet.html#cfn-ec2-subnet-mappubliciponlaunch
         */
        MapPublicIpOnLaunch?: C.YieldsBoolean;
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key–value pairs) for this subnet.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnet.html#cfn-ec2-subnet-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * Ref ID

         *
         * A Ref structure that contains the ID of the VPC on which you want to create the
 subnet. The VPC ID is provided as the value of the "Ref" property, as: {
    "Ref": "VPCID" }.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnet.html#cfn-awsec2subnet-prop-vpcid
         */
        VpcId: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * The AWS::EC2::SubnetCidrBlock resource associates a single IPv6 CIDR block
    with an Amazon VPC subnet.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnetcidrblock.html
     */
    interface SubnetCidrBlock {
        Type: 'AWS::EC2::SubnetCidrBlock';
        Properties: AWS.EC2.SubnetCidrBlock.Properties;
    }
    /**
     * The AWS::EC2::SubnetCidrBlock resource associates a single IPv6 CIDR block
    with an Amazon VPC subnet.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnetcidrblock.html
     */
    function SubnetCidrBlock(props: C.Omit<SubnetCidrBlock, 'Type'>): SubnetCidrBlock;
}
export declare namespace AWS.EC2.SubnetCidrBlock {
    interface Properties {
        /**
         * String

         *
         * The IPv6 CIDR block for the subnet. The CIDR block must have a prefix length of
 /64.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnetcidrblock.html#cfn-ec2-subnetcidrblock-ipv6cidrblock
         */
        Ipv6CidrBlock: C.YieldsString;
        /**
         * String

         *
         * The ID of the subnet to associate the IPv6 CIDR block with.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnetcidrblock.html#cfn-ec2-subnetcidrblock-subnetid
         */
        SubnetId: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * Associates a subnet with a network ACL.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnet-network-acl-assoc.html
     */
    interface SubnetNetworkAclAssociation {
        Type: 'AWS::EC2::SubnetNetworkAclAssociation';
        Properties: AWS.EC2.SubnetNetworkAclAssociation.Properties;
    }
    /**
     * Associates a subnet with a network ACL.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnet-network-acl-assoc.html
     */
    function SubnetNetworkAclAssociation(props: C.Omit<SubnetNetworkAclAssociation, 'Type'>): SubnetNetworkAclAssociation;
}
export declare namespace AWS.EC2.SubnetNetworkAclAssociation {
    interface Properties {
        /**
         * String

         *
         * The ID of the new ACL to associate with the subnet.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnet-network-acl-assoc.html#cfn-ec2-subnetnetworkaclassociation-networkaclid
         */
        NetworkAclId: C.YieldsString;
        /**
         * String

         *
         * The ID representing the current association between the original network ACL and the
 subnet.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnet-network-acl-assoc.html#cfn-ec2-subnetnetworkaclassociation-associationid
         */
        SubnetId: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * Associates a subnet with a route table.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnet-route-table-assoc.html
     */
    interface SubnetRouteTableAssociation {
        Type: 'AWS::EC2::SubnetRouteTableAssociation';
        Properties: AWS.EC2.SubnetRouteTableAssociation.Properties;
    }
    /**
     * Associates a subnet with a route table.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnet-route-table-assoc.html
     */
    function SubnetRouteTableAssociation(props: C.Omit<SubnetRouteTableAssociation, 'Type'>): SubnetRouteTableAssociation;
}
export declare namespace AWS.EC2.SubnetRouteTableAssociation {
    interface Properties {
        /**
         * String

         *
         * The ID of the route table. This is commonly written as a reference to a route table
 declared
 elsewhere in the template. For example:

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnet-route-table-assoc.html#cfn-ec2-subnetroutetableassociation-routetableid
         */
        RouteTableId: C.YieldsString;
        /**
         * String

         *
         * The ID of the subnet. This is commonly written as a reference to a subnet declared
 elsewhere in the
 template. For example:

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-subnet-route-table-assoc.html#cfn-ec2-subnetroutetableassociation-subnetid
         */
        SubnetId: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     *
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-trunkinterfaceassociation.html
     */
    interface TrunkInterfaceAssociation {
        Type: 'AWS::EC2::TrunkInterfaceAssociation';
        Properties: AWS.EC2.TrunkInterfaceAssociation.Properties;
    }
    /**
     *
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-trunkinterfaceassociation.html
     */
    function TrunkInterfaceAssociation(props: C.Omit<TrunkInterfaceAssociation, 'Type'>): TrunkInterfaceAssociation;
}
export declare namespace AWS.EC2.TrunkInterfaceAssociation {
    interface Properties {
        /**
         *
         *
         *
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-trunkinterfaceassociation.html#cfn-ec2-trunkinterfaceassociation-branchinterfaceid
         */
        BranchInterfaceId: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-trunkinterfaceassociation.html#cfn-ec2-trunkinterfaceassociation-grekey
         */
        GREKey?: C.Yields<C.CFInteger>;
        /**
         *
         *
         *
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-trunkinterfaceassociation.html#cfn-ec2-trunkinterfaceassociation-trunkinterfaceid
         */
        TrunkInterfaceId: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-trunkinterfaceassociation.html#cfn-ec2-trunkinterfaceassociation-vlanid
         */
        VLANId?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.EC2 {
    /**
     * Creates a Virtual Private Cloud (VPC) with the CIDR block that you specify. To name
    a VPC resource, use the Tags property and specify a value for the Name key.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpc.html
     */
    interface VPC {
        Type: 'AWS::EC2::VPC';
        Properties: AWS.EC2.VPC.Properties;
    }
    /**
     * Creates a Virtual Private Cloud (VPC) with the CIDR block that you specify. To name
    a VPC resource, use the Tags property and specify a value for the Name key.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpc.html
     */
    function VPC(props: C.Omit<VPC, 'Type'>): VPC;
}
export declare namespace AWS.EC2.VPC {
    interface Properties {
        /**
         * String

         *
         * The CIDR block you want the VPC to cover. For example: "10.0.0.0/16".

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpc.html#cfn-aws-ec2-vpc-cidrblock
         */
        CidrBlock: C.YieldsString;
        /**
         * Boolean

         *
         * Specifies whether the instances launched in the VPC get DNS hostnames. If this
 attribute is true, instances in the VPC get DNS hostnames; otherwise,
 they do not. You can only set EnableDnsHostnames to true
 if you also set the EnableDnsSupport attribute to true.
 By default, the value is set to false.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpc.html#cfn-aws-ec2-vpc-EnableDnsHostnames
         */
        EnableDnsHostnames?: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Specifies whether DNS resolution is supported for the VPC. If this attribute is
 true, the Amazon DNS server resolves DNS hostnames for your
 instances to their corresponding IP addresses; otherwise, it does not. By default
 the value is set to true.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpc.html#cfn-aws-ec2-vpc-EnableDnsSupport
         */
        EnableDnsSupport?: C.YieldsBoolean;
        /**
         * String

         *
         * The allowed tenancy of instances launched into the VPC.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpc.html#cfn-aws-ec2-vpc-instancetenancy
         */
        InstanceTenancy?: C.YieldsString;
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key–value pairs) for this VPC. To name a VPC resource, specify
 a value for the Name key.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpc.html#cfn-aws-ec2-vpc-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
    }
}
export declare namespace AWS.EC2 {
    /**
     * The AWS::EC2::VPCCidrBlock resource associates a single Amazon-provided IPv6
    CIDR block or a single user-specified IPv4 CIDR block with a Virtual Private Cloud
    (VPC).
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpccidrblock.html
     */
    interface VPCCidrBlock {
        Type: 'AWS::EC2::VPCCidrBlock';
        Properties: AWS.EC2.VPCCidrBlock.Properties;
    }
    /**
     * The AWS::EC2::VPCCidrBlock resource associates a single Amazon-provided IPv6
    CIDR block or a single user-specified IPv4 CIDR block with a Virtual Private Cloud
    (VPC).
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpccidrblock.html
     */
    function VPCCidrBlock(props: C.Omit<VPCCidrBlock, 'Type'>): VPCCidrBlock;
}
export declare namespace AWS.EC2.VPCCidrBlock {
    interface Properties {
        /**
         * Boolean

         *
         * Whether to request an Amazon-provided IPv6 CIDR block with a /56 prefix length
 for the VPC. You can't specify the range of IPv6 addresses or the size of the CIDR
 block.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpccidrblock.html#cfn-ec2-vpccidrblock-amazonprovidedipv6cidrblock
         */
        AmazonProvidedIpv6CidrBlock?: C.YieldsBoolean;
        /**
         * String

         *
         * An IPv4 CIDR block to associate with the VPC.
 

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpccidrblock.html#cfn-ec2-vpccidrblock-cidrblock
         */
        CidrBlock?: C.YieldsString;
        /**
         * String

         *
         * The ID of the VPC to associate the Amazon-provided IPv6 CIDR block with.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpccidrblock.html#cfn-ec2-vpccidrblock-vpcid
         */
        VpcId: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * Associates a set of DHCP options (that you've previously created) with the specified
    VPC.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpc-dhcp-options-assoc.html
     */
    interface VPCDHCPOptionsAssociation {
        Type: 'AWS::EC2::VPCDHCPOptionsAssociation';
        Properties: AWS.EC2.VPCDHCPOptionsAssociation.Properties;
    }
    /**
     * Associates a set of DHCP options (that you've previously created) with the specified
    VPC.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpc-dhcp-options-assoc.html
     */
    function VPCDHCPOptionsAssociation(props: C.Omit<VPCDHCPOptionsAssociation, 'Type'>): VPCDHCPOptionsAssociation;
}
export declare namespace AWS.EC2.VPCDHCPOptionsAssociation {
    interface Properties {
        /**
         * String

         *
         * The ID of the DHCP options you want to associate with the VPC. Specify
 default if you want the VPC to use no DHCP options.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpc-dhcp-options-assoc.html#cfn-ec2-vpcdhcpoptionsassociation-dhcpoptionsid
         */
        DhcpOptionsId: C.YieldsString;
        /**
         * String

         *
         * The ID of the VPC to associate with this DHCP options set.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpc-dhcp-options-assoc.html#cfn-ec2-vpcdhcpoptionsassociation-vpcid
         */
        VpcId: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * The AWS::EC2::VPCEndpoint resource creates a VPC endpoint that you can use to
    establish a private connection between your VPC and another AWS service without requiring
    access over the Internet, a VPN connection, or AWS Direct Connect.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpcendpoint.html
     */
    interface VPCEndpoint {
        Type: 'AWS::EC2::VPCEndpoint';
        Properties: AWS.EC2.VPCEndpoint.Properties;
    }
    /**
     * The AWS::EC2::VPCEndpoint resource creates a VPC endpoint that you can use to
    establish a private connection between your VPC and another AWS service without requiring
    access over the Internet, a VPN connection, or AWS Direct Connect.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpcendpoint.html
     */
    function VPCEndpoint(props: C.Omit<VPCEndpoint, 'Type'>): VPCEndpoint;
}
export declare namespace AWS.EC2.VPCEndpoint {
    interface Properties {
        /**
         * JSON object

         *
         * A policy to attach to the endpoint that controls access to the service. The
 policy must be valid JSON. The default policy allows full access to the AWS service.
 For
 more information, see Controlling Access to Services in the
 Amazon VPC User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpcendpoint.html#cfn-ec2-vpcendpoint-policydocument
         */
        PolicyDocument?: C.Yields<C.CFJson>;
        /**
         * List of strings

         *
         * One or more route table IDs that are used by the VPC to reach the
 endpoint.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpcendpoint.html#cfn-ec2-vpcendpoint-routetableids
         */
        RouteTableIds?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The AWS service to which you want to establish a connection. Specify the
 service name in the form of
 com.amazonaws.region.service.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpcendpoint.html#cfn-ec2-vpcendpoint-servicename
         */
        ServiceName: C.YieldsString;
        /**
         * String

         *
         * The ID of the VPC in which the endpoint is used.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpcendpoint.html#cfn-ec2-vpcendpoint-vpcid
         */
        VpcId: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * Attaches a gateway to a VPC.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpc-gateway-attachment.html
     */
    interface VPCGatewayAttachment {
        Type: 'AWS::EC2::VPCGatewayAttachment';
        Properties: AWS.EC2.VPCGatewayAttachment.Properties;
    }
    /**
     * Attaches a gateway to a VPC.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpc-gateway-attachment.html
     */
    function VPCGatewayAttachment(props: C.Omit<VPCGatewayAttachment, 'Type'>): VPCGatewayAttachment;
}
export declare namespace AWS.EC2.VPCGatewayAttachment {
    interface Properties {
        /**
         * String

         *
         * The ID of the Internet gateway.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpc-gateway-attachment.html#cfn-ec2-vpcgatewayattachment-internetgatewayid
         */
        InternetGatewayId?: C.YieldsString;
        /**
         * String

         *
         * The ID of the VPC to associate with this gateway.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpc-gateway-attachment.html#cfn-ec2-vpcgatewayattachment-vpcid
         */
        VpcId: C.YieldsString;
        /**
         * String

         *
         * The ID of the virtual private network (VPN) gateway to attach to the VPC.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpc-gateway-attachment.html#cfn-ec2-vpcgatewayattachment-vpngatewayid
         */
        VpnGatewayId?: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * A VPC peering connection enables a network connection between two virtual private
    clouds
    (VPCs) so that you can route traffic between them using a private IP address. For
    more
    information about VPC peering and its limitations, see VPC Peering Overview in the
    Amazon VPC Peering Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpcpeeringconnection.html
     */
    interface VPCPeeringConnection {
        Type: 'AWS::EC2::VPCPeeringConnection';
        Properties: AWS.EC2.VPCPeeringConnection.Properties;
    }
    /**
     * A VPC peering connection enables a network connection between two virtual private
    clouds
    (VPCs) so that you can route traffic between them using a private IP address. For
    more
    information about VPC peering and its limitations, see VPC Peering Overview in the
    Amazon VPC Peering Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpcpeeringconnection.html
     */
    function VPCPeeringConnection(props: C.Omit<VPCPeeringConnection, 'Type'>): VPCPeeringConnection;
}
export declare namespace AWS.EC2.VPCPeeringConnection {
    interface Properties {
        /**
         * String

         *
         * The AWS account ID of the owner of the VPC that you want to peer
 with.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpcpeeringconnection.html#cfn-ec2-vpcpeeringconnection-peerownerid
         */
        PeerOwnerId?: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the VPC peer role for the peering connection in
 another AWS account.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpcpeeringconnection.html#cfn-ec2-vpcpeeringconnection-peerrolearn
         */
        PeerRoleArn?: C.YieldsString;
        /**
         * String

         *
         * The ID of the VPC with which you are creating the peering connection.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpcpeeringconnection.html#cfn-ec2-vpcpeeringconnection-peervpcid
         */
        PeerVpcId: C.YieldsString;
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key–value pairs) for this
 resource.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpcpeeringconnection.html#cfn-ec2-vpcpeeringconnection-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * String

         *
         * The ID of the VPC that is requesting a peering connection.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpcpeeringconnection.html#cfn-ec2-vpcpeeringconnection-vpcid
         */
        VpcId: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * Creates a new VPN connection between an existing virtual private gateway and a VPN
    customer
    gateway.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpn-connection.html
     */
    interface VPNConnection {
        Type: 'AWS::EC2::VPNConnection';
        Properties: AWS.EC2.VPNConnection.Properties;
    }
    /**
     * Creates a new VPN connection between an existing virtual private gateway and a VPN
    customer
    gateway.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpn-connection.html
     */
    function VPNConnection(props: C.Omit<VPNConnection, 'Type'>): VPNConnection;
}
export declare namespace AWS.EC2.VPNConnection {
    interface Properties {
        /**
         * String

         *
         * The ID of the customer gateway. This can either be an embedded JSON object or a
 reference to a Gateway ID.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpn-connection.html#cfn-ec2-vpnconnection-customergatewayid
         */
        CustomerGatewayId: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether the VPN connection requires static routes.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpn-connection.html#cfn-ec2-vpnconnection-StaticRoutesOnly
         */
        StaticRoutesOnly?: C.YieldsBoolean;
        /**
         * AWS CloudFormation Resource Tags.

         *
         * The tags that you want to attach to the resource.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpn-connection.html#cfn-ec2-vpnconnection-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * String

         *
         * The type of VPN connection this virtual private gateway supports.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpn-connection.html#cfn-ec2-vpnconnection-type
         */
        Type: C.YieldsString;
        /**
         * String

         *
         * The ID of the virtual private gateway. This can either be an embedded JSON
 object or a reference to a Gateway ID.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpn-connection.html#cfn-ec2-vpnconnection-vpngatewayid
         */
        VpnGatewayId: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * A static route that is associated with a VPN connection between an existing virtual
    private
    gateway and a VPN customer gateway. The static route allows traffic to be routed from
    the
    virtual private gateway to the VPN customer gateway.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpn-connection-route.html
     */
    interface VPNConnectionRoute {
        Type: 'AWS::EC2::VPNConnectionRoute';
        Properties: AWS.EC2.VPNConnectionRoute.Properties;
    }
    /**
     * A static route that is associated with a VPN connection between an existing virtual
    private
    gateway and a VPN customer gateway. The static route allows traffic to be routed from
    the
    virtual private gateway to the VPN customer gateway.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpn-connection-route.html
     */
    function VPNConnectionRoute(props: C.Omit<VPNConnectionRoute, 'Type'>): VPNConnectionRoute;
}
export declare namespace AWS.EC2.VPNConnectionRoute {
    interface Properties {
        /**
         * String

         *
         * The CIDR block that is associated with the local subnet of the customer
 network.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpn-connection-route.html#cfn-ec2-vpnconnectionroute-cidrblock
         */
        DestinationCidrBlock: C.YieldsString;
        /**
         * String

         *
         * The ID of the VPN connection.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpn-connection-route.html#cfn-ec2-vpnconnectionroute-connectionid
         */
        VpnConnectionId: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * Creates a virtual private gateway. A virtual private gateway is the VPC-side endpoint
    for
    your VPN connection.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpn-gateway.html
     */
    interface VPNGateway {
        Type: 'AWS::EC2::VPNGateway';
        Properties: AWS.EC2.VPNGateway.Properties;
    }
    /**
     * Creates a virtual private gateway. A virtual private gateway is the VPC-side endpoint
    for
    your VPN connection.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpn-gateway.html
     */
    function VPNGateway(props: C.Omit<VPNGateway, 'Type'>): VPNGateway;
}
export declare namespace AWS.EC2.VPNGateway {
    interface Properties {
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key–value pairs) for this resource.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpn-gateway.html#cfn-ec2-vpngateway-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * String

         *
         * The type of VPN connection this virtual private gateway supports. The only valid value
 is
 "ipsec.1".

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpn-gateway.html#cfn-ec2-vpngateway-type
         */
        Type: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * Enables a virtual private gateway (VGW) to propagate routes to the routing tables
    of a
    VPC.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpn-gatewayrouteprop.html
     */
    interface VPNGatewayRoutePropagation {
        Type: 'AWS::EC2::VPNGatewayRoutePropagation';
        Properties: AWS.EC2.VPNGatewayRoutePropagation.Properties;
    }
    /**
     * Enables a virtual private gateway (VGW) to propagate routes to the routing tables
    of a
    VPC.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpn-gatewayrouteprop.html
     */
    function VPNGatewayRoutePropagation(props: C.Omit<VPNGatewayRoutePropagation, 'Type'>): VPNGatewayRoutePropagation;
}
export declare namespace AWS.EC2.VPNGatewayRoutePropagation {
    interface Properties {
        /**
         * List of route table IDs

         *
         * A list of routing table IDs that are associated with a VPC. The routing
 tables must be associated with the same VPC that the virtual private gateway is
 attached to.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpn-gatewayrouteprop.html#cfn-ec2-vpngatewayrouteprop-routetableids
         */
        RouteTableIds: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The ID of the virtual private gateway that is attached to a VPC. The virtual
 private gateway must be attached to the same VPC that the routing tables are
 associated with.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ec2-vpn-gatewayrouteprop.html#cfn-ec2-vpngatewayrouteprop-vpngatewayid
         */
        VpnGatewayId: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * The AWS::EC2::Volume type creates a new Amazon Elastic Block Store (Amazon EBS) volume.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-ebs-volume.html
     */
    interface Volume {
        Type: 'AWS::EC2::Volume';
        Properties: AWS.EC2.Volume.Properties;
    }
    /**
     * The AWS::EC2::Volume type creates a new Amazon Elastic Block Store (Amazon EBS) volume.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-ebs-volume.html
     */
    function Volume(props: C.Omit<Volume, 'Type'>): Volume;
}
export declare namespace AWS.EC2.Volume {
    interface Properties {
        /**
         * Boolean

         *
         * Indicates whether the volume is auto-enabled for I/O operations. By default,
 Amazon EBS disables I/O to the volume from attached EC2 instances when it determines
 that a volume's data is potentially inconsistent. If the consistency of the volume
 is not a concern, and you prefer that the volume be made available immediately if
 it's impaired, you can configure the volume to automatically enable I/O. For more
 information, see Working with the AutoEnableIO Volume Attribute in the
 Amazon EC2 User Guide for Linux Instances.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-ebs-volume.html#cfn-ec2-ebs-volume-autoenableio
         */
        AutoEnableIO?: C.YieldsBoolean;
        /**
         * String

         *
         * The Availability Zone in which to create the new volume.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-ebs-volume.html#cfn-ec2-ebs-volume-availabilityzone
         */
        AvailabilityZone: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether the volume is encrypted. You can attach encrypted Amazon EBS
 volumes only to instance types that support Amazon EBS encryption. Volumes that are
 created from encrypted snapshots are automatically encrypted. You can't create an
 encrypted volume from an unencrypted snapshot, or vice versa. If your AMI uses
 encrypted volumes, you can launch the AMI only on supported instance types. For
 more information, see Amazon EBS
    encryption in the Amazon EC2 User Guide for Linux Instances.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-ebs-volume.html#cfn-ec2-ebs-volume-encrypted
         */
        Encrypted?: C.YieldsBoolean;
        /**
         * Number

         *
         * The number of I/O operations per second (IOPS) that the volume supports. For more
 information about the valid sizes for each volume type, see the Iops parameter for the CreateVolume action in the Amazon EC2 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-ebs-volume.html#cfn-ec2-ebs-volume-iops
         */
        Iops?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the AWS Key Management Service master key that is
 used to
 create the encrypted volume, such as
 arn:aws:kms:us-east-2:012345678910:key/abcd1234-a123-456a-a12b-a123b4cd56ef.
 If you create an encrypted volume and don't specify this property, AWS CloudFormation
 uses the
 default master key.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-ebs-volume.html#cfn-ec2-ebs-volume-kmskeyid
         */
        KmsKeyId?: C.YieldsString;
        /**
         * Integer

         *
         * The size of the volume, in gibibytes (GiBs). For more information about the valid
 sizes for each volume type, see the Size parameter for the CreateVolume action in the Amazon EC2 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-ebs-volume.html#cfn-ec2-ebs-volume-size
         */
        Size?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The snapshot from which to create the new volume.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-ebs-volume.html#cfn-ec2-ebs-volume-snapshotid
         */
        SnapshotId?: C.YieldsString;
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key–value pairs) for this volume.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-ebs-volume.html#cfn-ec2-ebs-volume-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * String

         *
         * The volume type. If you set the type to io1, you must also set the Iops property. For valid values, see the VolumeType parameter for the CreateVolume action in the Amazon EC2 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-ebs-volume.html#cfn-ec2-ebs-volume-volumetype
         */
        VolumeType?: C.YieldsString;
    }
}
export declare namespace AWS.EC2 {
    /**
     * Attaches an Amazon EBS volume to a running instance and exposes it to the instance
    with the
    specified device name.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-ebs-volumeattachment.html
     */
    interface VolumeAttachment {
        Type: 'AWS::EC2::VolumeAttachment';
        Properties: AWS.EC2.VolumeAttachment.Properties;
    }
    /**
     * Attaches an Amazon EBS volume to a running instance and exposes it to the instance
    with the
    specified device name.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-ebs-volumeattachment.html
     */
    function VolumeAttachment(props: C.Omit<VolumeAttachment, 'Type'>): VolumeAttachment;
}
export declare namespace AWS.EC2.VolumeAttachment {
    interface Properties {
        /**
         * String

         *
         * How the device is exposed to the instance (e.g., /dev/sdh, or xvdh).
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-ebs-volumeattachment.html#cfn-ec2-ebs-volumeattachment-device
         */
        Device: C.YieldsString;
        /**
         * String

         *
         * The ID of the instance to which the volume attaches. This value can be a reference
 to an
 AWS::EC2::Instance resource, or it can be
 the physical ID of an existing EC2 instance.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-ebs-volumeattachment.html#cfn-ec2-ebs-volumeattachment-instanceid
         */
        InstanceId: C.YieldsString;
        /**
         * String

         *
         * The ID of the Amazon EBS volume. The volume and instance must be within the same
 Availability Zone. This value can be a reference to an AWS::EC2::Volume resource, or it can be the
 volume ID of an existing Amazon EBS volume.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-ebs-volumeattachment.html#cfn-ec2-ebs-volumeattachment-volumeid
         */
        VolumeId: C.YieldsString;
    }
}
export declare namespace AWS.ECR {
    /**
     * The AWS::ECR::Repository resource creates an Amazon EC2 Container Registry (Amazon ECR) repository,
    where users can push and pull Docker images. For more information, see Amazon ECR Repositories in the
    Amazon EC2 Container Registry User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecr-repository.html
     */
    interface Repository {
        Type: 'AWS::ECR::Repository';
        Properties: AWS.ECR.Repository.Properties;
    }
    /**
     * The AWS::ECR::Repository resource creates an Amazon EC2 Container Registry (Amazon ECR) repository,
    where users can push and pull Docker images. For more information, see Amazon ECR Repositories in the
    Amazon EC2 Container Registry User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecr-repository.html
     */
    function Repository(props: C.Omit<Repository, 'Type'>): Repository;
}
export declare namespace AWS.ECR.Repository {
    interface Properties {
        /**
         * String

         *
         * A name for the image repository. If you don't specify a name, AWS CloudFormation generates
 a
 unique physical ID and uses that ID for the repository name. For more information,
 see
 Name Type.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecr-repository.html#cfn-ecr-repository-repositoryname
         */
        RepositoryName?: C.YieldsString;
        /**
         * JSON object

         *
         * A policy that controls who has access to the repository and which actions they can
 perform on it. For more information, see Amazon ECR Repository Policies in the
 Amazon EC2 Container Registry User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecr-repository.html#cfn-ecr-repository-repositorypolicytext
         */
        RepositoryPolicyText?: C.Yields<C.CFJson>;
    }
}
export declare namespace AWS.ECS {
    /**
     * The AWS::ECS::Cluster resource creates an Amazon EC2 Container Service (Amazon ECS) cluster. This
    resource has no properties; use the Amazon ECS container agent to connect to the cluster.
    For more
    information, see Amazon ECS Container Agent in
    the Amazon EC2 Container Service Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-cluster.html
     */
    interface Cluster {
        Type: 'AWS::ECS::Cluster';
        Properties: AWS.ECS.Cluster.Properties;
    }
    /**
     * The AWS::ECS::Cluster resource creates an Amazon EC2 Container Service (Amazon ECS) cluster. This
    resource has no properties; use the Amazon ECS container agent to connect to the cluster.
    For more
    information, see Amazon ECS Container Agent in
    the Amazon EC2 Container Service Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-cluster.html
     */
    function Cluster(props: C.Omit<Cluster, 'Type'>): Cluster;
}
export declare namespace AWS.ECS.Cluster {
    interface Properties {
        /**
         * String

         *
         * A name for the cluster. If you don't specify a name, AWS CloudFormation generates
 a unique physical ID for the name.
 For more information, see Name Type.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-cluster.html#cfn-ecs-cluster-clustername
         */
        ClusterName?: C.YieldsString;
    }
}
export declare namespace AWS.ECS {
    /**
     * The AWS::ECS::Service resource creates an Amazon EC2 Container Service (Amazon ECS) service that runs
    and maintains the requested number of tasks and associated load balancers.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-service.html
     */
    interface Service {
        Type: 'AWS::ECS::Service';
        Properties: AWS.ECS.Service.Properties;
    }
    /**
     * The AWS::ECS::Service resource creates an Amazon EC2 Container Service (Amazon ECS) service that runs
    and maintains the requested number of tasks and associated load balancers.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-service.html
     */
    function Service(props: C.Omit<Service, 'Type'>): Service;
}
export declare namespace AWS.ECS.Service {
    interface Properties {
        /**
         * String

         *
         * The name or Amazon Resource Name (ARN) of the cluster that you want to run your
 Amazon ECS
 service on. If you do not specify a cluster, Amazon ECS uses the default
 cluster.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-service.html#cfn-ecs-service-cluster
         */
        Cluster?: C.YieldsString;
        /**
         * Amazon EC2 Container Service Service
    DeploymentConfiguration
         *
         * Configures how many tasks run during a deployment.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-service.html#cfn-ecs-service-deploymentconfiguration
         */
        DeploymentConfiguration?: C.Yields<AWS.ECS.Service.DeploymentConfiguration>;
        /**
         * Integer

         *
         * The number of simultaneous tasks that you want to run on the cluster. Specify
 the tasks with the TaskDefinition property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-service.html#cfn-ecs-service-desiredcount
         */
        DesiredCount?: C.Yields<C.CFInteger>;
        /**
         * List of Amazon EC2 Container Service Service
    LoadBalancers
         *
         * A list of load balancer objects to associate with the
 cluster. If you specify the Role property,
 LoadBalancers must be specified as well. For information
 about the number of load balancers that you can specify per service, see Service Load Balancing
 in the Amazon EC2 Container Service Developer Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-service.html#cfn-ecs-service-loadbalancers
         */
        LoadBalancers?: C.Yields<C.CFList<C.Yields<AWS.ECS.Service.LoadBalancer>>>;
        /**
         * Amazon EC2 Container Service Service
    PlacementConstraint
         *
         * The placement constraints for the tasks in the service.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-service.html#cfn-ecs-service-placementconstraints
         */
        PlacementConstraints?: C.Yields<C.CFList<C.Yields<AWS.ECS.Service.PlacementConstraint>>>;
        /**
         * Amazon EC2 Container Service Service
    PlacementStrategies
         *
         * The placement strategies that determine how tasks for the service are placed.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-service.html#cfn-ecs-service-placementstrategies
         */
        PlacementStrategies?: C.Yields<C.CFList<C.Yields<AWS.ECS.Service.PlacementStrategy>>>;
        /**
         * String

         *
         * The name or ARN of an AWS Identity and Access Management (IAM) role that allows your
 Amazon ECS container
 agent to make calls to your load balancer.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-service.html#cfn-ecs-service-role
         */
        Role?: C.YieldsString;
        /**
         * String

         *
         * The name of your
 service.
 The name is limited to 255 letters (uppercase and lowercase), numbers, hyphens,
 and underscores. Service names must be unique within a cluster, but you can have
 similarly named services in multiple clusters within a region or across multiple
 regions.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-service.html#cfn-ecs-service-servicename
         */
        ServiceName?: C.YieldsString;
        /**
         * String

         *
         * The ARN of the task definition (including the revision number) that you want to
 run on the cluster, such as
 arn:aws:ecs:us-east-1:123456789012:task-definition/mytask:3.
 You can't use :latest to specify a revision because it's ambiguous.
 For example, if AWS CloudFormation needed to roll back an update, it wouldn't know
 which
 revision to roll back to.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-service.html#cfn-ecs-service-taskdefinition
         */
        TaskDefinition: C.YieldsString;
    }
}
export declare namespace AWS.ECS.Service {
    interface DeploymentConfiguration {
        /**
         * Integer

         *
         * The maximum number of tasks, specified as a percentage of the Amazon ECS service's
 DesiredCount value, that can run in a service during a deployment. To calculate the maximum number
 of tasks, Amazon ECS uses this formula: the value of DesiredCount * (the value of the MaximumPercent/100), rounded down to the nearest integer value.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-service-deploymentconfiguration.html#cfn-ecs-service-deploymentconfiguration-maximumpercent
         */
        MaximumPercent?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The minimum number of tasks, specified as a percentage of the Amazon ECS service's
 DesiredCount value, that must continue to run and remain healthy during a deployment. To calculate
 the minimum number of tasks, Amazon ECS uses this formula: the value of DesiredCount * (the value of the MinimumHealthyPercent/100), rounded up to the nearest integer value.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-service-deploymentconfiguration.html#cfn-ecs-service-deploymentconfiguration-minimumhealthypercent
         */
        MinimumHealthyPercent?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.ECS.Service {
    interface LoadBalancer {
        /**
         * String

         *
         * The name of a container to use with the load balancer.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-service-loadbalancers.html#cfn-ecs-service-loadbalancers-containername
         */
        ContainerName?: C.YieldsString;
        /**
         * Integer

         *
         * The port number on the container to direct load balancer traffic to. Your
 container instances must allow ingress traffic on this port.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-service-loadbalancers.html#cfn-ecs-service-loadbalancers-containerport
         */
        ContainerPort: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The name of a Classic Load Balancer to associate with the Amazon ECS service.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-service-loadbalancers.html#cfn-ecs-service-loadbalancers-loadbalancername
         */
        LoadBalancerName?: C.YieldsString;
        /**
         * String

         *
         * An Application load balancer target group Amazon Resource Name (ARN) to associate
 with the Amazon ECS service.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-service-loadbalancers.html#cfn-ecs-service-loadbalancers-targetgrouparn
         */
        TargetGroupArn?: C.YieldsString;
    }
}
export declare namespace AWS.ECS.Service {
    interface PlacementConstraint {
        /**
         *
         *
         *
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-service-placementconstraint.html#cfn-ecs-service-placementconstraint-expression
         */
        Expression?: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-service-placementconstraint.html#cfn-ecs-service-placementconstraint-type
         */
        Type: C.YieldsString;
    }
}
export declare namespace AWS.ECS.Service {
    interface PlacementStrategy {
        /**
         *
         *
         *
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-service-placementstrategy.html#cfn-ecs-service-placementstrategy-field
         */
        Field?: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-service-placementstrategy.html#cfn-ecs-service-placementstrategy-type
         */
        Type: C.YieldsString;
    }
}
export declare namespace AWS.ECS {
    /**
     * The AWS::ECS::TaskDefinition
    resource
    describes the container and volume definitions of an Amazon EC2 Container Service (Amazon ECS) task.
    You can specify which Docker images to use, the required resources, and other configurations
    related to launching the task definition through an Amazon ECS service or task.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-taskdefinition.html
     */
    interface TaskDefinition {
        Type: 'AWS::ECS::TaskDefinition';
        Properties: AWS.ECS.TaskDefinition.Properties;
    }
    /**
     * The AWS::ECS::TaskDefinition
    resource
    describes the container and volume definitions of an Amazon EC2 Container Service (Amazon ECS) task.
    You can specify which Docker images to use, the required resources, and other configurations
    related to launching the task definition through an Amazon ECS service or task.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-taskdefinition.html
     */
    function TaskDefinition(props: C.Omit<TaskDefinition, 'Type'>): TaskDefinition;
}
export declare namespace AWS.ECS.TaskDefinition {
    interface Properties {
        /**
         * List of Amazon EC2 Container Service TaskDefinition
    ContainerDefinitions
         *
         * A list of container definitions in
 JSON
 format that describes the containers that make up your task.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-taskdefinition.html#cfn-ecs-taskdefinition-containerdefinitions
         */
        ContainerDefinitions?: C.Yields<C.CFList<C.Yields<AWS.ECS.TaskDefinition.ContainerDefinition>>>;
        /**
         * String

         *
         * The name of a family that this task definition is registered to. A
 family groups multiple versions of a task definition. Amazon ECS gives
 the first task definition that you registered to a family a revision number of 1.
 Amazon ECS
 gives sequential revision numbers to each task definition that you add.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-taskdefinition.html#cfn-ecs-taskdefinition-family
         */
        Family?: C.YieldsString;
        /**
         * String

         *
         * The Docker networking mode to use for the containers in the task, such as
 none, bridge, or host. For information about
 network modes, see NetworkMode in the Task Definition Parameters
 topic in the Amazon EC2 Container Service Developer Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-taskdefinition.html#cfn-ecs-taskdefinition-networkmode
         */
        NetworkMode?: C.YieldsString;
        /**
         * Amazon EC2 Container Service Service
    PlacementConstraint
         *
         * The placement constraints for the tasks in the service.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-taskdefinition.html#cfn-ecs-taskdefinition-placementconstraints
         */
        PlacementConstraints?: C.Yields<C.CFList<C.Yields<AWS.ECS.TaskDefinition.TaskDefinitionPlacementConstraint>>>;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of an AWS Identity and Access Management (IAM) role
 that grants containers in the task permission to call AWS APIs on your behalf. For
 more information, see IAM Roles for Tasks in the Amazon EC2 Container Service Developer Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-taskdefinition.html#cfn-ecs-taskdefinition-taskrolearn
         */
        TaskRoleArn?: C.YieldsString;
        /**
         * List of Amazon EC2 Container Service TaskDefinition
    Volumes
         *
         * A list of volume definitions
 in
 JSON format for the volumes that you can use in your container
 definitions.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ecs-taskdefinition.html#cfn-ecs-taskdefinition-volumes
         */
        Volumes?: C.Yields<C.CFList<C.Yields<AWS.ECS.TaskDefinition.Volume>>>;
    }
}
export declare namespace AWS.ECS.TaskDefinition {
    interface ContainerDefinition {
        /**
         * List of strings

         *
         * The CMD value to pass to the container. For more information
 about the Docker CMD parameter, see https://docs.docker.com/engine/reference/builder/#cmd.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-command
         */
        Command?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * Integer

         *
         * The minimum number of CPU units to reserve for the container. Containers share unallocated
 CPU units with other containers on the instance by using the same ratio as their allocated
 CPU units. For more information, see the cpu content for the ContainerDefinition data type in the Amazon EC2 Container Service API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-cpu
         */
        Cpu?: C.Yields<C.CFInteger>;
        /**
         * Boolean

         *
         * Indicates whether networking is disabled within the container.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-disablenetworking
         */
        DisableNetworking?: C.YieldsBoolean;
        /**
         * List of strings

         *
         * A list of DNS search domains that are provided to the container. The domain names
 that the DNS logic looks up when a process attempts to access a bare unqualified hostname.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-dnssearchdomains
         */
        DnsSearchDomains?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of strings

         *
         * A list of DNS servers that Amazon ECS provides to the container.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-dnsservers
         */
        DnsServers?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * Key-value pairs, with the name of the label as the key and the label value as the
 value.

         *
         * A key-value map of labels for the container.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-dockerlabels
         */
        DockerLabels?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         * List of strings

         *
         * A list of custom labels for SELinux and AppArmor multi-level security systems. For
 more information, see the dockerSecurityOptions content for the ContainerDefinition data type in the Amazon EC2 Container Service API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-dockersecurityoptions
         */
        DockerSecurityOptions?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of strings

         *
         * The ENTRYPOINT value to pass to the container. For more
 information about the Docker ENTRYPOINT parameter, see https://docs.docker.com/engine/reference/builder/#entrypoint.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-entrypoint
         */
        EntryPoint?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of Amazon EC2 Container Service
    TaskDefinition ContainerDefinitions Environment
         *
         * The environment variables to pass to the container.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-environment
         */
        Environment?: C.Yields<C.CFList<C.Yields<AWS.ECS.TaskDefinition.KeyValuePair>>>;
        /**
         * Boolean

         *
         * Indicates whether the task stops if this container fails. If you specify true and the container fails, all other containers in the task stop. If you specify false and the container fails, none of the other containers in the task is affected. This
 value is true by default.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-essential
         */
        Essential?: C.YieldsBoolean;
        /**
         * List of Amazon EC2 Container Service TaskDefinition ContainerDefinitions HostEntry
         *
         * A list of hostnames and IP address mappings to append to the /etc/hosts file on the container.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-extrahosts
         */
        ExtraHosts?: C.Yields<C.CFList<C.Yields<AWS.ECS.TaskDefinition.HostEntry>>>;
        /**
         * String

         *
         * The name that Docker will use for the container's hostname.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-hostname
         */
        Hostname?: C.YieldsString;
        /**
         * String

         *
         * The image to use for a container, which is passed directly to the Docker
 daemon. You can use images in the Docker Hub registry or specify other
 repositories
 (repository-url/image:tag).

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-image
         */
        Image?: C.YieldsString;
        /**
         * List of strings

         *
         * The name of another container to connect to. With links, containers can
 communicate with each other without using port mappings.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-links
         */
        Links?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * Amazon EC2 Container Service TaskDefinition ContainerDefinitions LogConfiguration
         *
         * Configures a custom log driver for the container. For more information, see the
 logConfiguration content for the ContainerDefinition data type
 in the Amazon EC2 Container Service API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-logconfiguration
         */
        LogConfiguration?: C.Yields<AWS.ECS.TaskDefinition.LogConfiguration>;
        /**
         * Integer

         *
         * The number of MiB of memory to reserve for the container. If your container attempts
 to exceed the allocated memory, the container is terminated.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-memory
         */
        Memory?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The number of MiB of memory to reserve for the container. When system
 memory is under contention, Docker attempts to keep the container memory
 within the limit. If the container requires more memory, it can consume up
 to the value specified by the Memory property or all of the
 available memory on the container instance, whichever comes first. This is
 called a soft limit.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-memoryreservation
         */
        MemoryReservation?: C.Yields<C.CFInteger>;
        /**
         * List of Amazon EC2 Container Service
    TaskDefinition ContainerDefinitions MountPoints
         *
         * The mount points for data volumes in the container.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-mountpoints
         */
        MountPoints?: C.Yields<C.CFList<C.Yields<AWS.ECS.TaskDefinition.MountPoint>>>;
        /**
         * String

         *
         * A name for the container.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-name
         */
        Name?: C.YieldsString;
        /**
         * List of Amazon EC2 Container Service
    TaskDefinition ContainerDefinitions PortMappings
         *
         * A mapping of the container port to a host port. Port mappings enable
 containers to access ports on the host container instance to send or receive
 traffic.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-portmappings
         */
        PortMappings?: C.Yields<C.CFList<C.Yields<AWS.ECS.TaskDefinition.PortMapping>>>;
        /**
         * Boolean

         *
         * Indicates whether the container is given full access to the host container instance.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-privileged
         */
        Privileged?: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Indicates whether the container's root file system is mounted as read only.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-readonlyrootfilesystem
         */
        ReadonlyRootFilesystem?: C.YieldsBoolean;
        /**
         * List of Amazon EC2 Container Service TaskDefinition ContainerDefinitions Ulimit
         *
         * A list of ulimits to set in the container. The ulimits set constraints on how much
 resources a container can consume so that it doesn't deplete all available resources
 on the host.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-ulimits
         */
        Ulimits?: C.Yields<C.CFList<C.Yields<AWS.ECS.TaskDefinition.Ulimit>>>;
        /**
         * String

         *
         * The user name to use inside the container.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-user
         */
        User?: C.YieldsString;
        /**
         * List of Amazon EC2 Container Service
    TaskDefinition ContainerDefinitions VolumesFrom
         *
         * The data volumes to mount from another container.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-volumesfrom
         */
        VolumesFrom?: C.Yields<C.CFList<C.Yields<AWS.ECS.TaskDefinition.VolumeFrom>>>;
        /**
         * String

         *
         * The working directory in the container in which to run commands.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions.html#cfn-ecs-taskdefinition-containerdefinition-workingdirectory
         */
        WorkingDirectory?: C.YieldsString;
    }
}
export declare namespace AWS.ECS.TaskDefinition {
    interface HostEntry {
        /**
         * String

         *
         * The hostname to use in the /etc/hosts file.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions-hostentry.html#cfn-ecs-taskdefinition-containerdefinition-hostentry-hostname
         */
        Hostname: C.YieldsString;
        /**
         * String

         *
         * The IP address to use in the /etc/hosts file.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions-hostentry.html#cfn-ecs-taskdefinition-containerdefinition-hostentry-ipaddress
         */
        IpAddress: C.YieldsString;
    }
}
export declare namespace AWS.ECS.TaskDefinition {
    interface HostVolumeProperties {
        /**
         * String

         *
         * The data volume path on the host container instance.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-volumes-host.html#cfn-ecs-taskdefinition-volumes-host-sourcepath
         */
        SourcePath?: C.YieldsString;
    }
}
export declare namespace AWS.ECS.TaskDefinition {
    interface KeyValuePair {
        /**
         * String

         *
         * The name of the environment variable.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions-environment.html#cfn-ecs-taskdefinition-containerdefinition-environment-name
         */
        Name?: C.YieldsString;
        /**
         * String

         *
         * The value of the environment variable.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions-environment.html#cfn-ecs-taskdefinition-containerdefinition-environment-value
         */
        Value?: C.YieldsString;
    }
}
export declare namespace AWS.ECS.TaskDefinition {
    interface LogConfiguration {
        /**
         * String

         *
         * The log driver to use for the container. This parameter requires that your container
 instance uses Docker Remote API Version 1.18 or greater. For more information, see
 the logDriver content for the LogConfiguration data type in the Amazon EC2 Container Service API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions-logconfiguration.html#cfn-ecs-taskdefinition-containerdefinition-logconfiguration-logdriver
         */
        LogDriver: C.YieldsString;
        /**
         * Key-value pairs, with the option name as the key and the option value as the value.

         *
         * The configuration options to send to the log driver. This parameter requires that
 your container instance uses Docker Remote API Version 1.18 or greater.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions-logconfiguration.html#cfn-ecs-taskdefinition-containerdefinition-logconfiguration-options
         */
        Options?: C.Yields<C.CFMap<C.YieldsString>>;
    }
}
export declare namespace AWS.ECS.TaskDefinition {
    interface MountPoint {
        /**
         * String

         *
         * The path on the container that indicates where you want to mount the
 volume.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions-mountpoints.html#cfn-ecs-taskdefinition-containerdefinition-mountpoints-containerpath
         */
        ContainerPath?: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether the container can write to the volume. If you specify
 true, the container has read-only access to the volume.
 If you specify false, the container can write to the
 volume. By default, the value is false.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions-mountpoints.html#cfn-ecs-taskdefinition-containerdefinition-mountpoints-readonly
         */
        ReadOnly?: C.YieldsBoolean;
        /**
         * String

         *
         * The name of the volume to mount.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions-mountpoints.html#cfn-ecs-taskdefinition-containerdefinition-mountpoints-sourcevolume
         */
        SourceVolume?: C.YieldsString;
    }
}
export declare namespace AWS.ECS.TaskDefinition {
    interface PortMapping {
        /**
         * Integer

         *
         * The port number on the container bound to the host port.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions-portmappings.html#cfn-ecs-taskdefinition-containerdefinition-portmappings-containerport
         */
        ContainerPort?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The host port number on the container instance that you want to reserve for
 your container. You can specify a non-reserved host port for your container port
 mapping, omit the host port, or set the host port to 0. If you
 specify a container port but no host port, your container host port is assigned
 automatically .

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions-portmappings.html#cfn-ecs-taskdefinition-containerdefinition-portmappings-readonly
         */
        HostPort?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The protocol used for the port mapping. For valid values, see the protocol parameter in the Amazon EC2 Container Service Developer Guide. By default, AWS CloudFormation specifies tcp.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions-portmappings.html#cfn-ecs-taskdefinition-containerdefinition-portmappings-sourcevolume
         */
        Protocol?: C.YieldsString;
    }
}
export declare namespace AWS.ECS.TaskDefinition {
    interface TaskDefinitionPlacementConstraint {
        /**
         *
         *
         *
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-taskdefinitionplacementconstraint.html#cfn-ecs-taskdefinition-taskdefinitionplacementconstraint-expression
         */
        Expression?: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-taskdefinitionplacementconstraint.html#cfn-ecs-taskdefinition-taskdefinitionplacementconstraint-type
         */
        Type: C.YieldsString;
    }
}
export declare namespace AWS.ECS.TaskDefinition {
    interface Ulimit {
        /**
         * Integer

         *
         * The hard limit for the ulimit type.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions-ulimit.html#cfn-ecs-taskdefinition-containerdefinition-ulimit-hardlimit
         */
        HardLimit: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The type of ulimit. For valid values, see the name content for the
 Ulimit data type in the
 Amazon EC2 Container Service API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions-ulimit.html#cfn-ecs-taskdefinition-containerdefinition-ulimit-name
         */
        Name: C.YieldsString;
        /**
         * Integer

         *
         * The soft limit for the ulimit type.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions-ulimit.html#cfn-ecs-taskdefinition-containerdefinition-ulimit-softlimit
         */
        SoftLimit: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.ECS.TaskDefinition {
    interface Volume {
        /**
         * Amazon EC2 Container Service TaskDefinition Volumes
    Host
         *
         * Determines whether your data volume persists on the host container instance and
 at the location where it is stored.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-volumes.html#cfn-ecs-taskdefinition-volumes-host
         */
        Host?: C.Yields<AWS.ECS.TaskDefinition.HostVolumeProperties>;
        /**
         * String

         *
         * The name of the volume. To specify mount points in your container definitions,
 use the value of this property.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-volumes.html#cfn-ecs-taskdefinition-volumes-name
         */
        Name?: C.YieldsString;
    }
}
export declare namespace AWS.ECS.TaskDefinition {
    interface VolumeFrom {
        /**
         * Boolean

         *
         * Indicates whether the container can write to the volume. If you specify
 true, the container has read-only access to the volume.
 If you specify false, the container can write to the
 volume. By default, the value is false.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions-volumesfrom.html#cfn-ecs-taskdefinition-containerdefinition-volumesfrom-readonly
         */
        ReadOnly?: C.YieldsBoolean;
        /**
         * String

         *
         * The name of the container that has the volumes to mount.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ecs-taskdefinition-containerdefinitions-volumesfrom.html#cfn-ecs-taskdefinition-containerdefinition-volumesfrom-sourcecontainer
         */
        SourceContainer?: C.YieldsString;
    }
}
export declare namespace AWS.EFS {
    /**
     * The AWS::EFS::FileSystem resource creates a new, empty file system in
    Amazon Elastic File System (Amazon EFS). You must create a mount target (AWS::EFS::MountTarget) to
    mount your Amazon EFS file system on an Amazon Elastic Compute Cloud (Amazon EC2)
    instance. For more information, see the
    CreateFileSystem API in the
    Amazon Elastic File System User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-efs-filesystem.html
     */
    interface FileSystem {
        Type: 'AWS::EFS::FileSystem';
        Properties: AWS.EFS.FileSystem.Properties;
    }
    /**
     * The AWS::EFS::FileSystem resource creates a new, empty file system in
    Amazon Elastic File System (Amazon EFS). You must create a mount target (AWS::EFS::MountTarget) to
    mount your Amazon EFS file system on an Amazon Elastic Compute Cloud (Amazon EC2)
    instance. For more information, see the
    CreateFileSystem API in the
    Amazon Elastic File System User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-efs-filesystem.html
     */
    function FileSystem(props: C.Omit<FileSystem, 'Type'>): FileSystem;
}
export declare namespace AWS.EFS.FileSystem {
    interface Properties {
        /**
         * Boolean

         *
         * A boolean value that, if true, creates an encrypted file system. For more information,
 see
 CreateFileSystem in the
 Amazon Elastic File System User Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-efs-filesystem.html#cfn-efs-filesystem-encrypted
         */
        Encrypted?: C.YieldsBoolean;
        /**
         * Amazon Elastic File System FileSystem
    FileSystemTags
         *
         * Tags to associate with the file system.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-efs-filesystem.html#cfn-efs-filesystem-filesystemtags
         */
        FileSystemTags?: C.Yields<C.CFList<C.Yields<AWS.EFS.FileSystem.ElasticFileSystemTag>>>;
        /**
         * String

         *
         * The ID of the AWS KMS customer master key (CMK) to use to protect the encrypted file
 system.  This parameter
 is only required if you want to use a non-default CMK. For more information, see
 CreateFileSystem in the
 Amazon Elastic File System User Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-efs-filesystem.html#cfn-efs-filesystem-kmskeyid
         */
        KmsKeyId?: C.YieldsString;
        /**
         * String

         *
         * The performance mode of the file system. For valid values, see the PerformanceMode parameter for the CreateFileSystem action in the Amazon Elastic File System User Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-efs-filesystem.html#cfn-efs-filesystem-performancemode
         */
        PerformanceMode?: C.YieldsString;
    }
}
export declare namespace AWS.EFS.FileSystem {
    interface ElasticFileSystemTag {
        /**
         * String

         *
         * The key name of the tag. You can specify a value that is from 1 to 128 Unicode
 characters in length, but you cannot use the prefix aws:.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-efs-filesystem-filesystemtags.html#cfn-efs-filesystem-filesystemtags-key
         */
        Key: C.YieldsString;
        /**
         * String

         *
         * The value of the tag key. You can specify a value that is from 0 to 128 Unicode
 characters in length.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-efs-filesystem-filesystemtags.html#cfn-efs-filesystem-filesystemtags-value
         */
        Value: C.YieldsString;
    }
}
export declare namespace AWS.EFS {
    /**
     * The AWS::EFS::MountTarget resource creates a mount target for an Amazon Elastic File System
    (Amazon EFS) file system (AWS::EFS::FileSystem). Use the mount target to mount file systems
    on Amazon Elastic Compute Cloud (Amazon EC2) instances.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-efs-mounttarget.html
     */
    interface MountTarget {
        Type: 'AWS::EFS::MountTarget';
        Properties: AWS.EFS.MountTarget.Properties;
    }
    /**
     * The AWS::EFS::MountTarget resource creates a mount target for an Amazon Elastic File System
    (Amazon EFS) file system (AWS::EFS::FileSystem). Use the mount target to mount file systems
    on Amazon Elastic Compute Cloud (Amazon EC2) instances.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-efs-mounttarget.html
     */
    function MountTarget(props: C.Omit<MountTarget, 'Type'>): MountTarget;
}
export declare namespace AWS.EFS.MountTarget {
    interface Properties {
        /**
         * String

         *
         * The ID of the file system for which you want to create the mount target.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-efs-mounttarget.html#cfn-efs-mounttarget-filesystemid
         */
        FileSystemId: C.YieldsString;
        /**
         * String

         *
         * An IPv4 address that is within the address range of the subnet that is
 specified in the SubnetId property. If you don't specify an IP
 address, Amazon EFS automatically assigns an address that is within the range of the
 subnet.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-efs-mounttarget.html#cfn-efs-mounttarget-ipaddress
         */
        IpAddress?: C.YieldsString;
        /**
         * List of strings

         *
         * A maximum of five VPC security group IDs that are in the same VPC as the subnet
 that is specified in the SubnetId property. For more information
 about security groups and mount targets, see Security in the
 Amazon Elastic File System User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-efs-mounttarget.html#cfn-efs-mounttarget-securitygroups
         */
        SecurityGroups: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The ID of the subnet in which you want to add the mount target.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-efs-mounttarget.html#cfn-efs-mounttarget-subnetid
         */
        SubnetId: C.YieldsString;
    }
}
export declare namespace AWS.EMR {
    /**
     * The AWS::EMR::Cluster resource creates an Amazon EMR cluster, which is a
    collection of EC2 instances on which you can run big data frameworks to process and
    analyze vast
    amounts of data. For more information, see Plan an Amazon EMR
       Cluster in the Amazon EMR Management Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-cluster.html
     */
    interface Cluster {
        Type: 'AWS::EMR::Cluster';
        Properties: AWS.EMR.Cluster.Properties;
    }
    /**
     * The AWS::EMR::Cluster resource creates an Amazon EMR cluster, which is a
    collection of EC2 instances on which you can run big data frameworks to process and
    analyze vast
    amounts of data. For more information, see Plan an Amazon EMR
       Cluster in the Amazon EMR Management Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-cluster.html
     */
    function Cluster(props: C.Omit<Cluster, 'Type'>): Cluster;
}
export declare namespace AWS.EMR.Cluster {
    interface Properties {
        /**
         * JSON object

         *
         * Additional
 features that you want to select.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-cluster.html#cfn-emr-cluster-additionalinfo
         */
        AdditionalInfo?: C.Yields<C.CFJson>;
        /**
         * List of Amazon EMR Cluster Application
         *
         * The software applications to deploy on the cluster, and the arguments that Amazon
 EMR
 passes to those applications.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-cluster.html#cfn-emr-cluster-applications
         */
        Applications?: C.Yields<C.CFList<C.Yields<AWS.EMR.Cluster.Application>>>;
        /**
         * String

         *
         * An AWS Identity and Access Management (IAM) role for automatic scaling policies. The
 default role is EMR_AutoScaling_DefaultRole. The IAM
 role provides permissions that the automatic scaling feature requires to launch and
 terminate Amazon EC2 instances in an instance group.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-cluster.html#cfn-elasticmapreduce-cluster-autoscalingrole
         */
        AutoScalingRole?: C.YieldsString;
        /**
         * List of Amazon EMR Cluster
    BootstrapActionConfig
         *
         * A list of bootstrap actions that Amazon EMR runs before starting applications on the
 cluster.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-cluster.html#cfn-emr-cluster-bootstrapactions
         */
        BootstrapActions?: C.Yields<C.CFList<C.Yields<AWS.EMR.Cluster.BootstrapActionConfig>>>;
        /**
         * List of Amazon EMR Cluster Configuration
         *
         * The software configuration of the Amazon EMR cluster.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-cluster.html#cfn-emr-cluster-configurations
         */
        Configurations?: C.Yields<C.CFList<C.Yields<AWS.EMR.Cluster.Configuration>>>;
        /**
         * Amazon EMR Cluster
    JobFlowInstancesConfig
         *
         * Configures the EC2 instances that will run jobs in the Amazon EMR cluster.
         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-cluster.html#cfn-emr-cluster-instances
         */
        Instances: C.Yields<AWS.EMR.Cluster.JobFlowInstancesConfig>;
        /**
         * String

         *
         * Also
 called instance profile and EC2 role. Accepts
 an instance profile associated with the role that you want to use. All EC2 instances
 in
 the cluster assume this role.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-cluster.html#cfn-emr-cluster-jobflowrole
         */
        JobFlowRole: C.YieldsString;
        /**
         * String

         *
         * An S3 bucket location to which Amazon EMR writes logs files from a job flow. If you
 don't
 specify a value, Amazon EMR doesn't write any log files.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-cluster.html#cfn-emr-cluster-loguri
         */
        LogUri?: C.YieldsString;
        /**
         * String

         *
         * A name for the Amazon EMR cluster.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-cluster.html#cfn-emr-cluster-name
         */
        Name: C.YieldsString;
        /**
         * String

         *
         * The Amazon EMR software release label. A release is a set of software applications
 and
 components that you can install and configure on an Amazon EMR cluster. For more information,
 see About Amazon EMR Releases in the Amazon EMR Release
    Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-cluster.html#cfn-emr-cluster-releaselabel
         */
        ReleaseLabel?: C.YieldsString;
        /**
         * String

         *
         * Indicates how individual EC2 instances terminate when an automatic
 scale-in
 activity occurs or an instance group is resized. For more
 information, see Cluster in
 the Amazon EMR API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-cluster.html#cfn-elasticmapreduce-cluster-scaledownbehavior
         */
        ScaleDownBehavior?: C.YieldsString;
        /**
         * String

         *
         * The name of the security configuration applied to the cluster.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-cluster.html#cfn-emr-securityconfiguration
         */
        SecurityConfiguration?: C.YieldsString;
        /**
         * String

         *
         * The IAM role that Amazon EMR assumes to access AWS resources on your behalf. For more
 information, see Configure IAM Roles for Amazon EMR in the Amazon EMR Management
    Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-cluster.html#cfn-emr-cluster-servicerole
         */
        ServiceRole: C.YieldsString;
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key–value pairs) to help you identify the Amazon EMR
 cluster.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-cluster.html#cfn-elasticmapreduce-cluster-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * Boolean

         *
         * Indicates whether the instances in the cluster are visible to all IAM users in the
 AWS account. If you specify true, all IAM users can view and (if they
 have permissions) manage the instances. If you specify false, only the
 IAM user that created the cluster can view and manage it. By default, AWS CloudFormation
 sets this
 property to false.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-cluster.html#cfn-emr-cluster-visibletoallusers
         */
        VisibleToAllUsers?: C.YieldsBoolean;
    }
}
export declare namespace AWS.EMR.Cluster {
    interface Application {
        /**
         * String-to-string map

         *
         * Metadata about third-party applications that third-party vendors use for testing
 purposes.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-application.html#cfn-emr-cluster-application-additionalinfo
         */
        AdditionalInfo?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         * List of strings

         *
         * Arguments that Amazon EMR passes to the application.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-application.html#cfn-emr-cluster-application-args
         */
        Args?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The name of the application to add to your cluster, such as Hadoop or
 Hive. For valid values, see the Applications parameter in the Amazon EMR API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-application.html#cfn-emr-cluster-application-name
         */
        Name?: C.YieldsString;
        /**
         * String

         *
         * The version of the application.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-application.html#cfn-emr-cluster-application-version
         */
        Version?: C.YieldsString;
    }
}
export declare namespace AWS.EMR.Cluster {
    interface AutoScalingPolicy {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-autoscalingpolicy.html#cfn-elasticmapreduce-cluster-autoscalingpolicy-constraints
         */
        Constraints: C.Yields<AWS.EMR.Cluster.ScalingConstraints>;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-autoscalingpolicy.html#cfn-elasticmapreduce-cluster-autoscalingpolicy-rules
         */
        Rules: C.Yields<C.CFList<C.Yields<AWS.EMR.Cluster.ScalingRule>>>;
    }
}
export declare namespace AWS.EMR.Cluster {
    interface BootstrapActionConfig {
        /**
         * String

         *
         * The name of the bootstrap action to add to your cluster.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-bootstrapactionconfig.html#cfn-emr-cluster-bootstrapactionconfig-name
         */
        Name: C.YieldsString;
        /**
         * Amazon EMR Cluster ScriptBootstrapActionConfig
         *
         * The script that the bootstrap action runs.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-bootstrapactionconfig.html#cfn-emr-cluster-bootstrapactionconfig-scriptbootstrapaction
         */
        ScriptBootstrapAction: C.Yields<AWS.EMR.Cluster.ScriptBootstrapActionConfig>;
    }
}
export declare namespace AWS.EMR.Cluster {
    interface CloudWatchAlarmDefinition {
        /**
         * String

         *
         * Determines how the metric specified by MetricName is compared to the value specified by Threshold.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-cloudwatchalarmdefinition.html#cfn-elasticmapreduce-cluster-cloudwatchalarmdefinition-comparisonoperator
         */
        ComparisonOperator: C.YieldsString;
        /**
         * List of Amazon EMR Cluster MetricDimension
         *
         * A list of CloudWatch metric dimensions.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-cloudwatchalarmdefinition.html#cfn-elasticmapreduce-cluster-cloudwatchalarmdefinition-dimensions
         */
        Dimensions?: C.Yields<C.CFList<C.Yields<AWS.EMR.Cluster.MetricDimension>>>;
        /**
         * Integer

         *
         * The number of periods, expressed in seconds using Period, during which the alarm condition must exist before the alarm triggers automatic
 scaling activity. The default value is 1.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-cloudwatchalarmdefinition.html#cfn-elasticmapreduce-cluster-cloudwatchalarmdefinition-evaluationperiods
         */
        EvaluationPeriods?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The name of the CloudWatch metric that is watched to determine an alarm condition.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-cloudwatchalarmdefinition.html#cfn-elasticmapreduce-cluster-cloudwatchalarmdefinition-metricname
         */
        MetricName: C.YieldsString;
        /**
         * String

         *
         * The namespace for the CloudWatch metric. The default is AWS/ElasticMapReduce.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-cloudwatchalarmdefinition.html#cfn-elasticmapreduce-cluster-cloudwatchalarmdefinition-namespace
         */
        Namespace?: C.YieldsString;
        /**
         * Integer

         *
         * The period, in seconds, over which the statistic is applied. EMR CloudWatch metrics
 are emitted every five minutes (300 seconds), so if an EMR CloudWatch metric is specified,
 specify 300.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-cloudwatchalarmdefinition.html#cfn-elasticmapreduce-cluster-cloudwatchalarmdefinition-period
         */
        Period: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The statistic to apply to the metric associated with the alarm. The default is AVERAGE.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-cloudwatchalarmdefinition.html#cfn-elasticmapreduce-cluster-cloudwatchalarmdefinition-statistic
         */
        Statistic?: C.YieldsString;
        /**
         * Double

         *
         * The value against which the specified statistic is compared.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-cloudwatchalarmdefinition.html#cfn-elasticmapreduce-cluster-cloudwatchalarmdefinition-threshold
         */
        Threshold: C.Yields<C.CFDouble>;
        /**
         * String

         *
         * The unit of measure associated with the CloudWatch metric being watched. The value
 specified for Unit must correspond to the units specified in the CloudWatch metric.
 

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-cloudwatchalarmdefinition.html#cfn-elasticmapreduce-cluster-cloudwatchalarmdefinition-unit
         */
        Unit?: C.YieldsString;
    }
}
export declare namespace AWS.EMR.Cluster {
    interface Configuration {
        /**
         * String

         *
         * The name of an application-specific configuration file. For more information see,
 Configuring Applications in the Amazon EMR Release
    Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-configuration.html#cfn-emr-cluster-configuration-classification
         */
        Classification?: C.YieldsString;
        /**
         * String-to-string map

         *
         * The settings that you want to change in the application-specific configuration file.
 For more information see, Configuring Applications in the Amazon EMR Release
    Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-configuration.html#cfn-emr-cluster-configuration-configurationproperties
         */
        ConfigurationProperties?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         * List of Amazon EMR Cluster Configuration
         *
         * A list of configurations to apply to this configuration. You can nest configurations
 so that a single configuration can have its own configurations. In other words, you
 can
 configure a configuration. For more information see, Configuring Applications in the Amazon EMR Release
    Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-configuration.html#cfn-emr-cluster-configuration-configurations
         */
        Configurations?: C.Yields<C.CFList<C.Yields<AWS.EMR.Cluster.Configuration>>>;
    }
}
export declare namespace AWS.EMR.Cluster {
    interface EbsBlockDeviceConfig {
        /**
         * Amazon EMR EbsConfiguration EbsBlockDeviceConfig VolumeSpecification
         *
         * The settings for the Amazon EBS volumes.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-ebsconfiguration-ebsblockdeviceconfig.html#cfn-emr-ebsconfiguration-ebsblockdeviceconfig-volumespecification
         */
        VolumeSpecification: C.Yields<AWS.EMR.Cluster.VolumeSpecification>;
        /**
         * Integer

         *
         * The number of Amazon EBS volumes that you want to create for each instance in the
 EMR cluster or instance
 group. The number cannot be 0.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-ebsconfiguration-ebsblockdeviceconfig.html#cfn-emr-ebsconfiguration-ebsblockdeviceconfig-volumesperinstance
         */
        VolumesPerInstance?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.EMR.Cluster {
    interface EbsConfiguration {
        /**
         * List of Amazon EMR EbsConfiguration EbsBlockDeviceConfigs
         *
         * Configures the block storage devices that are associated with your EMR instances.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-ebsconfiguration.html#cfn-emr-ebsconfiguration-ebsblockdeviceconfigs
         */
        EbsBlockDeviceConfigs?: C.Yields<C.CFList<C.Yields<AWS.EMR.Cluster.EbsBlockDeviceConfig>>>;
        /**
         * Boolean

         *
         * Indicates whether the instances are optimized for Amazon EBS I/O. This optimization
 provides dedicated throughput to Amazon EBS and an optimized configuration stack to
 provide optimal EBS I/O performance. For more information about fees and supported
 instance types, see EBS-Optimized Instances in the Amazon EC2 User Guide for Linux Instances.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-ebsconfiguration.html#cfn-emr-ebsconfiguration-ebsoptimized
         */
        EbsOptimized?: C.YieldsBoolean;
    }
}
export declare namespace AWS.EMR.Cluster {
    interface InstanceFleetConfig {
        /**
         * List of Amazon EMR
    Cluster InstanceTypeConfig
         *
         * The instance type configurations that define the EC2 instances in the instance fleet.
 Duplicates not allowed.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-instancefleetconfig.html#cfn-elasticmapreduce-cluster-instancefleetconfig-instancetypeconfigs
         */
        InstanceTypeConfigs?: C.Yields<C.CFList<C.Yields<AWS.EMR.Cluster.InstanceTypeConfig>>>;
        /**
         * Amazon EMR
    Cluster InstanceFleetProvisioningSpecifications
         *
         * The launch specification for the instance fleet.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-instancefleetconfig.html#cfn-elasticmapreduce-cluster-instancefleetconfig-launchspecifications
         */
        LaunchSpecifications?: C.Yields<AWS.EMR.Cluster.InstanceFleetProvisioningSpecifications>;
        /**
         * String

         *
         * The friendly name of the instance fleet.
 For constraints, see InstanceFleetConfig in the Amazon EMR API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-instancefleetconfig.html#cfn-elasticmapreduce-cluster-instancefleetconfig-name
         */
        Name?: C.YieldsString;
        /**
         * Integer

         *
         * The target capacity of On-Demand units for the instance fleet, which determines how
 many On-Demand instances to provision.
 For more information, see InstanceFleetConfig in the Amazon EMR API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-instancefleetconfig.html#cfn-elasticmapreduce-cluster-instancefleetconfig-targetondemandcapacity
         */
        TargetOnDemandCapacity?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The target capacity of Spot units for the instance fleet, which determines how many
 Spot instances to provision.
 For more information, see InstanceFleetConfig in the Amazon EMR API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-instancefleetconfig.html#cfn-elasticmapreduce-cluster-instancefleetconfig-targetspotcapacity
         */
        TargetSpotCapacity?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.EMR.Cluster {
    interface InstanceFleetProvisioningSpecifications {
        /**
         * Amazon EMR
    Cluster SpotProvisioningSpecification
         *
         * The launch specification for Spot instances in the fleet, which determines the defined
 duration and provisioning timeout behavior.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-instancefleetprovisioningspecifications.html#cfn-elasticmapreduce-cluster-instancefleetprovisioningspecifications-spotspecification
         */
        SpotSpecification: C.Yields<AWS.EMR.Cluster.SpotProvisioningSpecification>;
    }
}
export declare namespace AWS.EMR.Cluster {
    interface InstanceGroupConfig {
        /**
         * Amazon EMR
    Cluster AutoScalingPolicy
         *
         * An automatic scaling policy for a core instance group or task instance group in an
 Amazon EMR cluster. An automatic scaling policy defines how an instance group dynamically
 adds and terminates EC2  instances in response to the value of a CloudWatch
 metric.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-jobflowinstancesconfig-instancegroupconfig.html#cfn-elasticmapreduce-cluster-instancegroupconfig-autoscalingpolicy
         */
        AutoScalingPolicy?: C.Yields<AWS.EMR.Cluster.AutoScalingPolicy>;
        /**
         * String

         *
         * When launching instances as Spot Instances, the bid price in USD for each EC2 instance
 in the instance group.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-jobflowinstancesconfig-instancegroupconfig.html#cfn-emr-cluster-jobflowinstancesconfig-instancegroupconfig-bidprice
         */
        BidPrice?: C.YieldsString;
        /**
         * List of Amazon EMR Cluster Configuration
         *
         * A list of configurations to apply to this instance group. For more information see,
 Configuring Applications in the Amazon EMR Release Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-jobflowinstancesconfig-instancegroupconfig.html#cfn-emr-cluster-jobflowinstancesconfig-instancegroupconfig-configurations
         */
        Configurations?: C.Yields<C.CFList<C.Yields<AWS.EMR.Cluster.Configuration>>>;
        /**
         * Amazon EMR EbsConfiguration
         *
         * Configures Amazon Elastic Block Store (Amazon EBS) storage volumes to attach to your
 instances.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-jobflowinstancesconfig-instancegroupconfig.html#cfn-emr-cluster-jobflowinstancesconfig-instancegroupconfigConfigurations-ebsconfiguration
         */
        EbsConfiguration?: C.Yields<AWS.EMR.Cluster.EbsConfiguration>;
        /**
         * Integer

         *
         * The number of instances to launch in the instance group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-jobflowinstancesconfig-instancegroupconfig.html#cfn-emr-cluster-jobflowinstancesconfig-instancegroupconfig-instancecount
         */
        InstanceCount: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The EC2 instance type for all instances in the instance group. For more information,
 see Instance Configurations in the Amazon EMR Management
    Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-jobflowinstancesconfig-instancegroupconfig.html#cfn-emr-cluster-jobflowinstancesconfig-instancegroupconfig-instancetype
         */
        InstanceType: C.YieldsString;
        /**
         * String

         *
         * The type of marketplace from which your instances are provisioned into this group,
 either ON_DEMAND or SPOT. For more information, see Amazon EC2 Purchasing
    Options.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-jobflowinstancesconfig-instancegroupconfig.html#cfn-emr-cluster-jobflowinstancesconfig-instancegroupconfig-market
         */
        Market?: C.YieldsString;
        /**
         * String

         *
         * A name for the instance group.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-jobflowinstancesconfig-instancegroupconfig.html#cfn-emr-cluster-jobflowinstancesconfig-instancegroupconfig-name
         */
        Name?: C.YieldsString;
    }
}
export declare namespace AWS.EMR.Cluster {
    interface InstanceTypeConfig {
        /**
         * String

         *
         * The bid price for each EC2 Spot
 Instance
 type,
 as defined
 by
 InstanceType.
 BidPrice
 is expressed in USD. For more information, see InstanceTypeConfig in the
 Amazon EMR API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-instancetypeconfig.html#cfn-elasticmapreduce-cluster-instancetypeconfig-bidprice
         */
        BidPrice?: C.YieldsString;
        /**
         * Double

         *
         * The bid price, as a percentage of
 the
 On-Demand price, for each EC2 Spot instance as defined by
 InstanceType.
 BidPriceAsPercentageOfOnDemandPriceis
 expressed
 as a number. For more information, see InstanceTypeConfig in the
 Amazon EMR API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-instancetypeconfig.html#cfn-elasticmapreduce-cluster-instancetypeconfig-bidpriceaspercentageofondemandprice
         */
        BidPriceAsPercentageOfOnDemandPrice?: C.Yields<C.CFDouble>;
        /**
         * List of Amazon EMR Cluster Configuration
         *
         * A
 configuration
 classification that applies when provisioning cluster
 instances. This
 
 can include configurations for applications and software that run on the
 cluster.
 Duplicates
 are not allowed.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-instancetypeconfig.html#cfn-elasticmapreduce-cluster-instancetypeconfig-configurations
         */
        Configurations?: C.Yields<C.CFList<C.Yields<AWS.EMR.Cluster.Configuration>>>;
        /**
         * Amazon EMR EbsConfiguration
         *
         * The configuration of Amazon Elastic Block Store (Amazon EBS)
 that
 is attached to each instance as defined by
 InstanceType.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-instancetypeconfig.html#cfn-elasticmapreduce-cluster-instancetypeconfig-ebsconfiguration
         */
        EbsConfiguration?: C.Yields<AWS.EMR.Cluster.EbsConfiguration>;
        /**
         * String

         *
         * An EC2 instance type, such as m3.xlarge.
 For constraints, see InstanceTypeConfig in the Amazon EMR API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-instancetypeconfig.html#cfn-elasticmapreduce-cluster-instancetypeconfig-instancetype
         */
        InstanceType: C.YieldsString;
        /**
         * Integer

         *
         * The number of units that a provisioned instance of
 this
 type provides toward fulfilling the target capacities
 defined in InstanceFleetConfig. For more information, see
 InstanceTypeConfig in the
 Amazon EMR API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-instancetypeconfig.html#cfn-elasticmapreduce-cluster-instancetypeconfig-weightedcapacity
         */
        WeightedCapacity?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.EMR.Cluster {
    interface JobFlowInstancesConfig {
        /**
         * List of strings

         *
         * A list of additional EC2 security group IDs to assign to the master instance (master
 node) in your Amazon EMR cluster. Use this property to supplement the rules specified
 by the
 Amazon EMR managed master security group.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-jobflowinstancesconfig.html#cfn-emr-cluster-jobflowinstancesconfig-additionalmastersecuritygroups
         */
        AdditionalMasterSecurityGroups?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of strings

         *
         * A list of additional EC2 security group IDs to assign to the slave instances (slave
 nodes) in your Amazon EMR cluster. Use this property to supplement the rules specified
 by the
 Amazon EMR managed slave security group.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-jobflowinstancesconfig.html#cfn-emr-cluster-jobflowinstancesconfig-additionalslavesecuritygroups
         */
        AdditionalSlaveSecurityGroups?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * Amazon EMR
    Cluster InstanceFleetConfig
         *
         * The instance fleet settings for the
 core
 instances in your Amazon EMR cluster. Use this property with the MasterInstanceFleet property.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-jobflowinstancesconfig.html#cfn-elasticmapreduce-cluster-jobflowinstancesconfig-coreinstancefleet
         */
        CoreInstanceFleet?: C.Yields<AWS.EMR.Cluster.InstanceFleetConfig>;
        /**
         * Amazon EMR
    Cluster InstanceGroupConfig
         *
         * The settings for the
 core
 instances in your Amazon EMR cluster. Use this property with the MasterInstanceGroup property.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-jobflowinstancesconfig.html#cfn-emr-cluster-jobflowinstancesconfig-coreinstancegroup
         */
        CoreInstanceGroup?: C.Yields<AWS.EMR.Cluster.InstanceGroupConfig>;
        /**
         * String

         *
         * The name of an Amazon Elastic Compute Cloud (Amazon EC2) key pair, which you can use
 to access the instances
 in your Amazon EMR cluster.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-jobflowinstancesconfig.html#cfn-emr-cluster-jobflowinstancesconfig-ec2keyname
         */
        Ec2KeyName?: C.YieldsString;
        /**
         * String

         *
         * The ID of
 the
 subnet where you want to launch your instances.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-jobflowinstancesconfig.html#cfn-emr-cluster-jobflowinstancesconfig-ec2subnetid
         */
        Ec2SubnetId?: C.YieldsString;
        /**
         * String

         *
         * The ID of an EC2 security group (managed by Amazon EMR) that is assigned to the master
 instance (master node) in your Amazon EMR cluster.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-jobflowinstancesconfig.html#cfn-emr-cluster-jobflowinstancesconfig-emrmanagedmastersecuritygroup
         */
        EmrManagedMasterSecurityGroup?: C.YieldsString;
        /**
         * String

         *
         * The ID of an EC2 security group (managed by Amazon EMR) that is assigned to the slave
 instances (slave nodes) in your Amazon EMR cluster.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-jobflowinstancesconfig.html#cfn-emr-cluster-jobflowinstancesconfig-emrmanagedslavesecuritygroup
         */
        EmrManagedSlaveSecurityGroup?: C.YieldsString;
        /**
         * String

         *
         * The Hadoop version for the job flow. For valid values, see the HadoopVersion parameter in
 the Amazon EMR API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-jobflowinstancesconfig.html#cfn-emr-cluster-jobflowinstancesconfig-hadoopversion
         */
        HadoopVersion?: C.YieldsString;
        /**
         * Amazon EMR
    Cluster InstanceFleetConfig
         *
         * The instance fleet settings for the master instance (master node).
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-jobflowinstancesconfig.html#cfn-elasticmapreduce-cluster-jobflowinstancesconfig-masterinstancefleet
         */
        MasterInstanceFleet?: C.Yields<AWS.EMR.Cluster.InstanceFleetConfig>;
        /**
         * Amazon EMR
    Cluster InstanceGroupConfig
         *
         * The
 settings
 for the master instance (master node).

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-jobflowinstancesconfig.html#cfn-emr-cluster-jobflowinstancesconfig-coreinstancegroup
         */
        MasterInstanceGroup?: C.Yields<AWS.EMR.Cluster.InstanceGroupConfig>;
        /**
         * Amazon EMR
    Cluster PlacementType
         *
         * The Availability Zone (AZ) in which the job flow runs.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-jobflowinstancesconfig.html#cfn-emr-cluster-jobflowinstancesconfig-placement
         */
        Placement?: C.Yields<AWS.EMR.Cluster.PlacementType>;
        /**
         * String

         *
         * The ID of
 the
 EC2 security group (managed by Amazon EMR) that services use to access clusters in
 private
 subnets.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-jobflowinstancesconfig.html#cfn-emr-cluster-jobflowinstancesconfig-serviceaccesssecuritygroup
         */
        ServiceAccessSecurityGroup?: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether to prevent the EC2 instances from being terminated by an API call
 or user intervention. If you want to delete a stack with protected instances, update
 this value to false before you delete the stack. By default, AWS CloudFormation sets
 this property to false.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-jobflowinstancesconfig.html#cfn-emr-cluster-jobflowinstancesconfig-terminationprotected
         */
        TerminationProtected?: C.YieldsBoolean;
    }
}
export declare namespace AWS.EMR.Cluster {
    interface MetricDimension {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-metricdimension.html#cfn-elasticmapreduce-cluster-metricdimension-key
         */
        Key: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-metricdimension.html#cfn-elasticmapreduce-cluster-metricdimension-value
         */
        Value: C.YieldsString;
    }
}
export declare namespace AWS.EMR.Cluster {
    interface PlacementType {
        /**
         * String

         *
         * The Amazon Elastic Compute Cloud (Amazon EC2) AZ for the job flow. For more information,
 see http://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-regions-availability-zones.html in the Amazon EC2 User Guide for Linux Instances.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-jobflowinstancesconfig-placementtype.html#aws-properties-emr-cluster-jobflowinstancesconfig-placementtype
         */
        AvailabilityZone: C.YieldsString;
    }
}
export declare namespace AWS.EMR.Cluster {
    interface ScalingAction {
        /**
         * String

         *
         * Not available for instance groups. Instance groups use the market type specified for
 the group.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-scalingaction.html#cfn-elasticmapreduce-cluster-scalingaction-market
         */
        Market?: C.YieldsString;
        /**
         * Amazon EMR
    Cluster SimpleScalingPolicyConfiguration
         *
         * The type of adjustment the automatic scaling activity makes when triggered, and the
 periodicity of the adjustment.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-scalingaction.html#cfn-elasticmapreduce-cluster-scalingaction-simplescalingpolicyconfiguration
         */
        SimpleScalingPolicyConfiguration: C.Yields<AWS.EMR.Cluster.SimpleScalingPolicyConfiguration>;
    }
}
export declare namespace AWS.EMR.Cluster {
    interface ScalingConstraints {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-scalingconstraints.html#cfn-elasticmapreduce-cluster-scalingconstraints-maxcapacity
         */
        MaxCapacity: C.Yields<C.CFInteger>;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-scalingconstraints.html#cfn-elasticmapreduce-cluster-scalingconstraints-mincapacity
         */
        MinCapacity: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.EMR.Cluster {
    interface ScalingRule {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-scalingrule.html#cfn-elasticmapreduce-cluster-scalingrule-action
         */
        Action: C.Yields<AWS.EMR.Cluster.ScalingAction>;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-scalingrule.html#cfn-elasticmapreduce-cluster-scalingrule-description
         */
        Description?: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-scalingrule.html#cfn-elasticmapreduce-cluster-scalingrule-name
         */
        Name: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-scalingrule.html#cfn-elasticmapreduce-cluster-scalingrule-trigger
         */
        Trigger: C.Yields<AWS.EMR.Cluster.ScalingTrigger>;
    }
}
export declare namespace AWS.EMR.Cluster {
    interface ScalingTrigger {
        /**
         * Amazon EMR
    Cluster CloudWatchAlarmDefinition
         *
         * The definition of a CloudWatch metric alarm. When the defined alarm conditions are
 met along with other trigger parameters, scaling activity begins.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-scalingtrigger.html#cfn-elasticmapreduce-cluster-scalingtrigger-cloudwatchalarmdefinition
         */
        CloudWatchAlarmDefinition: C.Yields<AWS.EMR.Cluster.CloudWatchAlarmDefinition>;
    }
}
export declare namespace AWS.EMR.Cluster {
    interface ScriptBootstrapActionConfig {
        /**
         * List of strings

         *
         * A list of command line arguments to pass to the bootstrap action script.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-bootstrapactionconfig-scriptbootstrapactionconfig.html#cfn-emr-cluster-bootstrapactionconfig-scriptbootstrapaction-args
         */
        Args?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The location of the script that Amazon EMR runs during a bootstrap action. Specify
 a
 location in an S3 bucket or your local file system.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-bootstrapactionconfig-scriptbootstrapactionconfig.html#cfn-emr-cluster-bootstrapactionconfig-scriptbootstrapaction-path
         */
        Path: C.YieldsString;
    }
}
export declare namespace AWS.EMR.Cluster {
    interface SimpleScalingPolicyConfiguration {
        /**
         * String

         *
         * The way in which Amazon EC2 instances are added (if ScalingAdjustment is a positive number) or terminated (if ScalingAdjustment is a negative number) each time the scaling activity is triggered.
 CHANGE_IN_CAPACITY is the default.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-simplescalingpolicyconfiguration.html#cfn-elasticmapreduce-cluster-simplescalingpolicyconfiguration-adjustmenttype
         */
        AdjustmentType?: C.YieldsString;
        /**
         * Integer

         *
         * The amount of time, in seconds, after a scaling activity completes before any further
 trigger-related scaling activities can start. The default value is 0.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-simplescalingpolicyconfiguration.html#cfn-elasticmapreduce-cluster-simplescalingpolicyconfiguration-cooldown
         */
        CoolDown?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The amount by which to scale in or scale out, based on the specified AdjustmentType. A positive value adds to the instance group's Amazon EC2 instance count while a
 negative number removes instances.
 If AdjustmentType is set to EXACT_CAPACITY, the number should only be a positive integer. If AdjustmentType is set to PERCENT_CHANGE_IN_CAPACITY, the value
 should express the percentage as a decimal.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-simplescalingpolicyconfiguration.html#cfn-elasticmapreduce-cluster-simplescalingpolicyconfiguration-scalingadjustment
         */
        ScalingAdjustment: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.EMR.Cluster {
    interface SpotProvisioningSpecification {
        /**
         * Integer

         *
         * The defined duration for Spot instances (also known as Spot blocks) in minutes.
 For more information, see SpotProvisioningSpecification in the Amazon EMR API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-spotprovisioningspecification.html#cfn-elasticmapreduce-cluster-spotprovisioningspecification-blockdurationminutes
         */
        BlockDurationMinutes?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The action to take when TargetSpotCapacity has not been fulfilled when the TimeoutDurationMinutes
 has expired.
 For more information, see SpotProvisioningSpecification in the Amazon EMR API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-spotprovisioningspecification.html#cfn-elasticmapreduce-cluster-spotprovisioningspecification-timeoutaction
         */
        TimeoutAction: C.YieldsString;
        /**
         * Integer

         *
         * The spot provisioning timeout period in minutes.
 For more information, see SpotProvisioningSpecification in the Amazon EMR API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-cluster-spotprovisioningspecification.html#cfn-elasticmapreduce-cluster-spotprovisioningspecification-timeoutdurationminutes
         */
        TimeoutDurationMinutes: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.EMR.Cluster {
    interface VolumeSpecification {
        /**
         * Integer

         *
         * The number of I/O operations per second (IOPS) that the volume supports. For more
 information, see Iops for the EbsBlockDevice action in the Amazon EC2 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-ebsconfiguration-ebsblockdeviceconfig-volumespecification.html#cfn-emr-ebsconfiguration-ebsblockdeviceconfig-volumespecification-iops
         */
        Iops?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The volume size, in Gibibytes (GiB). For more information about specifying the volume
 size, see VolumeSize for the EbsBlockDevice action in the Amazon EC2 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-ebsconfiguration-ebsblockdeviceconfig-volumespecification.html#cfn-emr-ebsconfiguration-ebsblockdeviceconfig-volumespecification-sizeingb
         */
        SizeInGB: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The volume type, such as standard or io1. For more information about specifying the volume type, see VolumeType for the EbsBlockDevice action in the Amazon EC2 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-ebsconfiguration-ebsblockdeviceconfig-volumespecification.html#cfn-emr-ebsconfiguration-ebsblockdeviceconfig-volumespecification-volumetype
         */
        VolumeType: C.YieldsString;
    }
}
export declare namespace AWS.EMR {
    /**
     * Use
    the AWS::EMR::InstanceFleetConfig resource
    to
    configure
    a Spot
    Instance
    fleet
    for an Amazon EMR cluster. For more information, see Configure Instance
       Fleets in the Amazon EMR Management Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticmapreduce-instancefleetconfig.html
     */
    interface InstanceFleetConfig {
        Type: 'AWS::EMR::InstanceFleetConfig';
        Properties: AWS.EMR.InstanceFleetConfig.Properties;
    }
    /**
     * Use
    the AWS::EMR::InstanceFleetConfig resource
    to
    configure
    a Spot
    Instance
    fleet
    for an Amazon EMR cluster. For more information, see Configure Instance
       Fleets in the Amazon EMR Management Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticmapreduce-instancefleetconfig.html
     */
    function InstanceFleetConfig(props: C.Omit<InstanceFleetConfig, 'Type'>): InstanceFleetConfig;
}
export declare namespace AWS.EMR.InstanceFleetConfig {
    interface Properties {
        /**
         * String

         *
         * The ID of the target cluster.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticmapreduce-instancefleetconfig.html#cfn-elasticmapreduce-instancefleetconfig-clusterid
         */
        ClusterId: C.YieldsString;
        /**
         * String

         *
         * The node type that the instance fleet hosts.
 Valid
 values are MASTER, CORE, and TASK.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticmapreduce-instancefleetconfig.html#cfn-elasticmapreduce-instancefleetconfig-instancefleettype
         */
        InstanceFleetType: C.YieldsString;
        /**
         * List of Amazon EMR InstanceFleetConfig InstanceTypeConfig
         *
         * The instance type configurations that define the EC2 instances in the instance fleet.
 Duplicates not allowed.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticmapreduce-instancefleetconfig.html#cfn-elasticmapreduce-instancefleetconfig-instancetypeconfigs
         */
        InstanceTypeConfigs?: C.Yields<C.CFList<C.Yields<AWS.EMR.InstanceFleetConfig.InstanceTypeConfig>>>;
        /**
         * Amazon EMR InstanceFleetConfig InstanceFleetProvisioningSpecifications
         *
         * The
 launch
 specification for the instance fleet.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticmapreduce-instancefleetconfig.html#cfn-elasticmapreduce-instancefleetconfig-launchspecifications
         */
        LaunchSpecifications?: C.Yields<AWS.EMR.InstanceFleetConfig.InstanceFleetProvisioningSpecifications>;
        /**
         * String

         *
         * The friendly name of the instance fleet. For constraints, see InstanceFleetConfig in the Amazon EMR API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticmapreduce-instancefleetconfig.html#cfn-elasticmapreduce-instancefleetconfig-name
         */
        Name?: C.YieldsString;
        /**
         * Integer

         *
         * The target capacity of On-Demand
 units
 for the instance fleet.
 This
 
 determines how many
 On-Demand
 Instances
 to provision. For more information, see InstanceFleetConfig in the Amazon EMR API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticmapreduce-instancefleetconfig.html#cfn-elasticmapreduce-instancefleetconfig-targetondemandcapacity
         */
        TargetOnDemandCapacity?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The target capacity of Spot
 units
 for the instance
 fleet.
 This
 determines how many Spot
 Instances
 to provision. For more information, see InstanceFleetConfig in the Amazon EMR API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticmapreduce-instancefleetconfig.html#cfn-elasticmapreduce-instancefleetconfig-targetspotcapacity
         */
        TargetSpotCapacity?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.EMR.InstanceFleetConfig {
    interface Configuration {
        /**
         * String

         *
         * The
 application-specific
 configuration file.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancefleetconfig-configuration.html#cfn-elasticmapreduce-instancefleetconfig-configuration-classification
         */
        Classification?: C.YieldsString;
        /**
         * String to string map

         *
         * Within
 a configuration classification,
 a
 set of properties
 that
 represent the settings that you want to
 change
 in the configuration file. Duplicates not allowed.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancefleetconfig-configuration.html#cfn-elasticmapreduce-instancefleetconfig-configuration-configurationproperties
         */
        ConfigurationProperties?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         * List of Amazon EMR
    InstanceFleetConfig Configuration
         *
         * The list of
 additional
 configurations to apply within a
 configuration
 object. Duplicates not allowed.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancefleetconfig-configuration.html#cfn-elasticmapreduce-instancefleetconfig-configuration-configurations
         */
        Configurations?: C.Yields<C.CFList<C.Yields<AWS.EMR.InstanceFleetConfig.Configuration>>>;
    }
}
export declare namespace AWS.EMR.InstanceFleetConfig {
    interface EbsBlockDeviceConfig {
        /**
         * Amazon EMR InstanceFleetConfig VolumeSpecification
         *
         * Amazon EBS volume
 specifications,
 such as volume type, IOPS, and size
 (GiB),
 for the EBS volume attached to an EC2 instance in the fleet.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancefleetconfig-ebsblockdeviceconfig.html#cfn-elasticmapreduce-instancefleetconfig-ebsblockdeviceconfig-volumespecification
         */
        VolumeSpecification: C.Yields<AWS.EMR.InstanceFleetConfig.VolumeSpecification>;
        /**
         * Integer

         *
         * The number of Amazon EBS volumes with
 a
 specific volume configuration that are associated with
 every instance in the fleet.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancefleetconfig-ebsblockdeviceconfig.html#cfn-elasticmapreduce-instancefleetconfig-ebsblockdeviceconfig-volumesperinstance
         */
        VolumesPerInstance?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.EMR.InstanceFleetConfig {
    interface EbsConfiguration {
        /**
         * List of Amazon EMR InstanceFleetConfig EbsBlockDeviceConfig
         *
         * A
 list of Amazon EBS volume specifications
 that
 are attached to an instance. Duplicates not
 allowed.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancefleetconfig-ebsconfiguration.html#cfn-elasticmapreduce-instancefleetconfig-ebsconfiguration-ebsblockdeviceconfigs
         */
        EbsBlockDeviceConfigs?: C.Yields<C.CFList<C.Yields<AWS.EMR.InstanceFleetConfig.EbsBlockDeviceConfig>>>;
        /**
         * Boolean

         *
         * Indicates whether an Amazon EBS volume is
 EBS-optimized.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancefleetconfig-ebsconfiguration.html#cfn-elasticmapreduce-instancefleetconfig-ebsconfiguration-ebsoptimized
         */
        EbsOptimized?: C.YieldsBoolean;
    }
}
export declare namespace AWS.EMR.InstanceFleetConfig {
    interface InstanceFleetProvisioningSpecifications {
        /**
         * Amazon EMR InstanceFleetConfig SpotProvisioningSpecification
         *
         * The launch specification for Spot
 Instances
 in the
 fleet.
 This
 determines the defined duration and provisioning timeout behavior.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancefleetconfig-instancefleetprovisioningspecifications.html#cfn-elasticmapreduce-instancefleetconfig-instancefleetprovisioningspecifications-spotspecification
         */
        SpotSpecification: C.Yields<AWS.EMR.InstanceFleetConfig.SpotProvisioningSpecification>;
    }
}
export declare namespace AWS.EMR.InstanceFleetConfig {
    interface InstanceTypeConfig {
        /**
         * String

         *
         * The bid price for each EC2 Spot
 Instance
 type
 as
 defined by
 InstanceType.
 BidPrice
 is expressed in USD. For more information, see InstanceTypeConfig in the
 Amazon EMR API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancefleetconfig-instancetypeconfig.html#cfn-elasticmapreduce-instancefleetconfig-instancetypeconfig-bidprice
         */
        BidPrice?: C.YieldsString;
        /**
         * Double

         *
         * The bid price, as a percentage of
 the
 On-Demand price, for each EC2 Spot
 Instance
 as defined by
 InstanceType.
 BidPriceAsPercentageOfOnDemandPrice
 is expressed as a number. For more information, see InstanceTypeConfig in the
 Amazon EMR API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancefleetconfig-instancetypeconfig.html#cfn-elasticmapreduce-instancefleetconfig-instancetypeconfig-bidpriceaspercentageofondemandprice
         */
        BidPriceAsPercentageOfOnDemandPrice?: C.Yields<C.CFDouble>;
        /**
         * List of Amazon EMR
    InstanceFleetConfig Configuration
         *
         * A
 configuration
 classification that applies when provisioning cluster instances.
 You
 can
 use
 this property to configure
 applications
 and software that run on the cluster. Duplicates
 are
 not allowed.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancefleetconfig-instancetypeconfig.html#cfn-elasticmapreduce-instancefleetconfig-instancetypeconfig-configurations
         */
        Configurations?: C.Yields<C.CFList<C.Yields<AWS.EMR.InstanceFleetConfig.Configuration>>>;
        /**
         * Amazon EMR
    InstanceFleetConfig EbsConfiguration
         *
         * The configuration of Amazon Elastic Block Store (Amazon EBS)
 that
 is attached to each instance as defined by
 InstanceType.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancefleetconfig-instancetypeconfig.html#cfn-elasticmapreduce-instancefleetconfig-instancetypeconfig-ebsconfiguration
         */
        EbsConfiguration?: C.Yields<AWS.EMR.InstanceFleetConfig.EbsConfiguration>;
        /**
         * String

         *
         * An
 EC2 instance type, such as m3.xlarge. For
 constraints, see InstanceTypeConfig in the
 Amazon EMR API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancefleetconfig-instancetypeconfig.html#cfn-elasticmapreduce-instancefleetconfig-instancetypeconfig-instancetype
         */
        InstanceType: C.YieldsString;
        /**
         * Integer

         *
         * The number of units that a provisioned instance of
 this
 type provides toward fulfilling the target capacities
 defined in InstanceFleetConfig. For more information, see
 InstanceTypeConfig in the
 Amazon EMR API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancefleetconfig-instancetypeconfig.html#cfn-elasticmapreduce-instancefleetconfig-instancetypeconfig-weightedcapacity
         */
        WeightedCapacity?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.EMR.InstanceFleetConfig {
    interface SpotProvisioningSpecification {
        /**
         * Integer

         *
         * The
 defined
 duration for Spot
 Instances
 (also known as Spot blocks) in minutes. For more information, see SpotProvisioningSpecification in the
 Amazon EMR API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancefleetconfig-spotprovisioningspecification.html#cfn-elasticmapreduce-instancefleetconfig-spotprovisioningspecification-blockdurationminutes
         */
        BlockDurationMinutes?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The action to take when
 the
 capacity for the target Spot Instance, as specified in
 TargetSpotCapacity,
 has not been fulfilled
 before
 the
 time
 specified in
 TimeoutDurationMinutes
 has expired. For more information, see SpotProvisioningSpecification in the
 Amazon EMR API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancefleetconfig-spotprovisioningspecification.html#cfn-elasticmapreduce-instancefleetconfig-spotprovisioningspecification-timeoutaction
         */
        TimeoutAction: C.YieldsString;
        /**
         * Integer

         *
         * The
 timeout
 period for spot
 provisioning,
 in minutes. For more information, see SpotProvisioningSpecification in the
 Amazon EMR API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancefleetconfig-spotprovisioningspecification.html#cfn-elasticmapreduce-instancefleetconfig-spotprovisioningspecification-timeoutdurationminutes
         */
        TimeoutDurationMinutes: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.EMR.InstanceFleetConfig {
    interface VolumeSpecification {
        /**
         * Integer

         *
         * The number of I/O operations per second (IOPS) that the volume supports.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancefleetconfig-volumespecification.html#cfn-elasticmapreduce-instancefleetconfig-volumespecification-iops
         */
        Iops?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The volume size, in gibibytes (GiB).
 For valid values, see VolumeSpecification in the Amazon EMR API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancefleetconfig-volumespecification.html#cfn-elasticmapreduce-instancefleetconfig-volumespecification-sizeingb
         */
        SizeInGB: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The volume type.
 For valid values, see VolumeSpecification in the Amazon EMR API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancefleetconfig-volumespecification.html#cfn-elasticmapreduce-instancefleetconfig-volumespecification-volumetype
         */
        VolumeType: C.YieldsString;
    }
}
export declare namespace AWS.EMR {
    /**
     * The AWS::EMR::InstanceGroupConfig resource configures a task instance group for
    an Amazon EMR cluster.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-instancegroupconfig.html
     */
    interface InstanceGroupConfig {
        Type: 'AWS::EMR::InstanceGroupConfig';
        Properties: AWS.EMR.InstanceGroupConfig.Properties;
    }
    /**
     * The AWS::EMR::InstanceGroupConfig resource configures a task instance group for
    an Amazon EMR cluster.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-instancegroupconfig.html
     */
    function InstanceGroupConfig(props: C.Omit<InstanceGroupConfig, 'Type'>): InstanceGroupConfig;
}
export declare namespace AWS.EMR.InstanceGroupConfig {
    interface Properties {
        /**
         * Amazon EMR
    InstanceGroupConfig AutoScalingPolicy
         *
         * An automatic scaling policy for a core instance group or task instance group in an
 Amazon EMR cluster. An automatic scaling policy defines how an instance group dynamically
 adds and terminates EC2 instances in response to the value of a CloudWatch metric.
 For more
 information, see PutAutoScalingPolicy in the Amazon EMR API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-instancegroupconfig.html#cfn-elasticmapreduce-instancegroupconfig-autoscalingpolicy
         */
        AutoScalingPolicy?: C.Yields<AWS.EMR.InstanceGroupConfig.AutoScalingPolicy>;
        /**
         * String

         *
         * The bid price in USD for each Amazon EC2 instance in the instance group when launching
 instances (nodes) as Spot Instances.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-instancegroupconfig.html#cfn-emr-instancegroupconfig-bidprice
         */
        BidPrice?: C.YieldsString;
        /**
         * List of Amazon EMR Cluster Configuration
         *
         * A list of configurations to apply to this instance group. For more information see,
 Configuring Applications in the Amazon EMR Release
    Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-instancegroupconfig.html#cfn-emr-instancegroupconfig-configurations
         */
        Configurations?: C.Yields<C.CFList<C.Yields<AWS.EMR.InstanceGroupConfig.Configuration>>>;
        /**
         * Amazon EMR EbsConfiguration
         *
         * Configures Amazon Elastic Block Store (Amazon EBS) storage volumes to attach to your
 instances.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-instancegroupconfig.html#cfn-emr-instancegroupconfig-ebsconfiguration
         */
        EbsConfiguration?: C.Yields<AWS.EMR.InstanceGroupConfig.EbsConfiguration>;
        /**
         * Integer

         *
         * The number of instances to launch in the instance group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-instancegroupconfig.html#cfn-emr-instancegroupconfiginstancecount-
         */
        InstanceCount: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The role of the servers in the Amazon EMR cluster, such as TASK. For more
 information, see Instance Groups in the Amazon EMR Management
    Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-instancegroupconfig.html#cfn-emr-instancegroupconfig-instancerole
         */
        InstanceRole: C.YieldsString;
        /**
         * String

         *
         * The EC2 instance type for all instances in the instance group. For more information,
 see Instance Configurations in the Amazon EMR Management
    Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-instancegroupconfig.html#cfn-emr-instancegroupconfig-instancetype
         */
        InstanceType: C.YieldsString;
        /**
         * String

         *
         * The ID of an Amazon EMR cluster that you want to associate this instance group
 with.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-instancegroupconfig.html#cfn-emr-instancegroupconfig-jobflowid
         */
        JobFlowId: C.YieldsString;
        /**
         * String

         *
         * The type of marketplace from which your instances are provisioned into this group,
 either ON_DEMAND or SPOT. For more information, see Amazon EC2 Purchasing Options.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-instancegroupconfig.html#cfn-emr-instancegroupconfig-market
         */
        Market?: C.YieldsString;
        /**
         * String

         *
         * A name for the instance group.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-instancegroupconfig.html#cfn-emr-instancegroupconfig-name
         */
        Name?: C.YieldsString;
    }
}
export declare namespace AWS.EMR.InstanceGroupConfig {
    interface AutoScalingPolicy {
        /**
         * Amazon EMR
    InstanceGroupConfig ScalingConstraints
         *
         * The
 upper and lower Amazon EC2 instance limits for an automatic scaling
 policy. Automatic scaling activity doesn't cause an instance group to
 grow above or below these limits.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-autoscalingpolicy.html#cfn-elasticmapreduce-instancegroupconfig-autoscalingpolicy-constraints
         */
        Constraints: C.Yields<AWS.EMR.InstanceGroupConfig.ScalingConstraints>;
        /**
         * List of Amazon EMR
    InstanceGroupConfig ScalingRule
         *
         * The
 scale-in
 and scale-out rules that compose the automatic scaling policy.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-autoscalingpolicy.html#cfn-elasticmapreduce-instancegroupconfig-autoscalingpolicy-rules
         */
        Rules: C.Yields<C.CFList<C.Yields<AWS.EMR.InstanceGroupConfig.ScalingRule>>>;
    }
}
export declare namespace AWS.EMR.InstanceGroupConfig {
    interface CloudWatchAlarmDefinition {
        /**
         * String

         *
         * Determines how the metric specified by MetricName is compared to the value specified by Threshold.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-cloudwatchalarmdefinition.html#cfn-elasticmapreduce-instancegroupconfig-cloudwatchalarmdefinition-comparisonoperator
         */
        ComparisonOperator: C.YieldsString;
        /**
         * List of Amazon EMR
    InstanceGroupConfig MetricDimension
         *
         * A list of CloudWatch metric
 dimensions.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-cloudwatchalarmdefinition.html#cfn-elasticmapreduce-instancegroupconfig-cloudwatchalarmdefinition-dimensions
         */
        Dimensions?: C.Yields<C.CFList<C.Yields<AWS.EMR.InstanceGroupConfig.MetricDimension>>>;
        /**
         * Integer

         *
         * The number of periods, expressed in seconds using the Period property,
 during which the alarm condition must exist before the alarm triggers automatic scaling
 activity. The default value is 1.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-cloudwatchalarmdefinition.html#cfn-elasticmapreduce-instancegroupconfig-cloudwatchalarmdefinition-evaluationperiods
         */
        EvaluationPeriods?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The name of the CloudWatch metric that is
 watched
 to determine an alarm condition.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-cloudwatchalarmdefinition.html#cfn-elasticmapreduce-instancegroupconfig-cloudwatchalarmdefinition-metricname
         */
        MetricName: C.YieldsString;
        /**
         * String

         *
         * The namespace for the CloudWatch metric. The default is AWS/ElasticMapReduce.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-cloudwatchalarmdefinition.html#cfn-elasticmapreduce-instancegroupconfig-cloudwatchalarmdefinition-namespace
         */
        Namespace?: C.YieldsString;
        /**
         * Integer

         *
         * The period, in seconds, over which the
 statistic for
 applying the metric associated with the alarm is applied.
 You specify the
 statistic in the Statistic property.
 CloudWatch
 metrics for Amazon EMR
  are emitted every five minutes (300
 seconds).
 If
  you specify
 a CloudWatch
 metric for
 Amazon EMR,
 specify
 300.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-cloudwatchalarmdefinition.html#cfn-elasticmapreduce-instancegroupconfig-cloudwatchalarmdefinition-period
         */
        Period: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The statistic to apply to the metric associated with the alarm. The default is AVERAGE.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-cloudwatchalarmdefinition.html#cfn-elasticmapreduce-instancegroupconfig-cloudwatchalarmdefinition-statistic
         */
        Statistic?: C.YieldsString;
        /**
         * Double

         *
         * The value against which the specified
 statistic
 is compared.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-cloudwatchalarmdefinition.html#cfn-elasticmapreduce-instancegroupconfig-cloudwatchalarmdefinition-threshold
         */
        Threshold: C.Yields<C.CFDouble>;
        /**
         * String

         *
         * The
 unit of measure associated with the CloudWatch metric being watched.
 Specify
 the unit specified
 in the CloudWatch metric.
 

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-cloudwatchalarmdefinition.html#cfn-elasticmapreduce-instancegroupconfig-cloudwatchalarmdefinition-unit
         */
        Unit?: C.YieldsString;
    }
}
export declare namespace AWS.EMR.InstanceGroupConfig {
    interface Configuration {
        /**
         * String

         *
         * The name of an application-specific configuration file. For more information see,
 Configuring Applications in the Amazon EMR Release
    Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-configuration.html#cfn-emr-cluster-configuration-classification
         */
        Classification?: C.YieldsString;
        /**
         * String-to-string map

         *
         * The settings that you want to change in the application-specific configuration file.
 For more information see, Configuring Applications in the Amazon EMR Release
    Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-configuration.html#cfn-emr-cluster-configuration-configurationproperties
         */
        ConfigurationProperties?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         * List of Amazon EMR Cluster Configuration
         *
         * A list of configurations to apply to this configuration. You can nest configurations
 so that a single configuration can have its own configurations. In other words, you
 can
 configure a configuration. For more information see, Configuring Applications in the Amazon EMR Release
    Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-cluster-configuration.html#cfn-emr-cluster-configuration-configurations
         */
        Configurations?: C.Yields<C.CFList<C.Yields<AWS.EMR.InstanceGroupConfig.Configuration>>>;
    }
}
export declare namespace AWS.EMR.InstanceGroupConfig {
    interface EbsBlockDeviceConfig {
        /**
         * Amazon EMR EbsConfiguration EbsBlockDeviceConfig VolumeSpecification
         *
         * The settings for the Amazon EBS volumes.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-ebsconfiguration-ebsblockdeviceconfig.html#cfn-emr-ebsconfiguration-ebsblockdeviceconfig-volumespecification
         */
        VolumeSpecification: C.Yields<AWS.EMR.InstanceGroupConfig.VolumeSpecification>;
        /**
         * Integer

         *
         * The number of Amazon EBS volumes that you want to create for each instance in the
 EMR cluster or instance
 group. The number cannot be 0.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-ebsconfiguration-ebsblockdeviceconfig.html#cfn-emr-ebsconfiguration-ebsblockdeviceconfig-volumesperinstance
         */
        VolumesPerInstance?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.EMR.InstanceGroupConfig {
    interface EbsConfiguration {
        /**
         * List of Amazon EMR EbsConfiguration EbsBlockDeviceConfigs
         *
         * Configures the block storage devices that are associated with your EMR instances.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-ebsconfiguration.html#cfn-emr-ebsconfiguration-ebsblockdeviceconfigs
         */
        EbsBlockDeviceConfigs?: C.Yields<C.CFList<C.Yields<AWS.EMR.InstanceGroupConfig.EbsBlockDeviceConfig>>>;
        /**
         * Boolean

         *
         * Indicates whether the instances are optimized for Amazon EBS I/O. This optimization
 provides dedicated throughput to Amazon EBS and an optimized configuration stack to
 provide optimal EBS I/O performance. For more information about fees and supported
 instance types, see EBS-Optimized Instances in the Amazon EC2 User Guide for Linux Instances.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-ebsconfiguration.html#cfn-emr-ebsconfiguration-ebsoptimized
         */
        EbsOptimized?: C.YieldsBoolean;
    }
}
export declare namespace AWS.EMR.InstanceGroupConfig {
    interface MetricDimension {
        /**
         * String

         *
         * The dimension name.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-metricdimension.html#cfn-elasticmapreduce-instancegroupconfig-metricdimension-key
         */
        Key: C.YieldsString;
        /**
         * String

         *
         * The dimension value.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-metricdimension.html#cfn-elasticmapreduce-instancegroupconfig-metricdimension-value
         */
        Value: C.YieldsString;
    }
}
export declare namespace AWS.EMR.InstanceGroupConfig {
    interface ScalingAction {
        /**
         * String

         *
         * Not
 available for instance groups. Instance groups use the market type
 specified for the group.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-scalingaction.html#cfn-elasticmapreduce-instancegroupconfig-scalingaction-market
         */
        Market?: C.YieldsString;
        /**
         * Amazon EMR
    InstanceGroupConfig SimpleScalingPolicyConfiguration
         *
         * The type of adjustment that the automatic scaling activity makes when triggered, and
 the
 periodicity of the adjustment.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-scalingaction.html#cfn-elasticmapreduce-instancegroupconfig-scalingaction-simplescalingpolicyconfiguration
         */
        SimpleScalingPolicyConfiguration: C.Yields<AWS.EMR.InstanceGroupConfig.SimpleScalingPolicyConfiguration>;
    }
}
export declare namespace AWS.EMR.InstanceGroupConfig {
    interface ScalingConstraints {
        /**
         * Integer

         *
         * For autoscaling, the maximum number of EC2 instances in an instance group. Scale-out
 activities add instances only up to this boundary.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-scalingconstraints.html#cfn-elasticmapreduce-instancegroupconfig-scalingconstraints-maxcapacity
         */
        MaxCapacity: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * For autoscaling, the minimum number of EC2 instances in an instance group. Scale-in
 activities do not terminate instances below this boundary.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-scalingconstraints.html#cfn-elasticmapreduce-instancegroupconfig-scalingconstraints-mincapacity
         */
        MinCapacity: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.EMR.InstanceGroupConfig {
    interface ScalingRule {
        /**
         * Amazon EMR
    InstanceGroupConfig ScalingAction
         *
         * The conditions that trigger
 an
 automatic scaling activity.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-scalingrule.html#cfn-elasticmapreduce-instancegroupconfig-scalingrule-action
         */
        Action: C.Yields<AWS.EMR.InstanceGroupConfig.ScalingAction>;
        /**
         * String

         *
         * A friendly, more verbose description of the automatic scaling rule.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-scalingrule.html#cfn-elasticmapreduce-instancegroupconfig-scalingrule-description
         */
        Description?: C.YieldsString;
        /**
         * String

         *
         * The identifier of the automatic scaling rule. Rule names must be unique within a
 scaling policy.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-scalingrule.html#cfn-elasticmapreduce-instancegroupconfig-scalingrule-name
         */
        Name: C.YieldsString;
        /**
         * Amazon EMR
    InstanceGroupConfig ScalingTrigger
         *
         * The CloudWatch alarm definition that determines when automatic scaling activity is
 triggered.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-scalingrule.html#cfn-elasticmapreduce-instancegroupconfig-scalingrule-trigger
         */
        Trigger: C.Yields<AWS.EMR.InstanceGroupConfig.ScalingTrigger>;
    }
}
export declare namespace AWS.EMR.InstanceGroupConfig {
    interface ScalingTrigger {
        /**
         * Amazon EMR
    InstanceGroupConfig CloudWatchAlarmDefinition
         *
         * The definition of a CloudWatch metric alarm. When the defined alarm conditions are
 met along with other trigger parameters, scaling activity begins.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-scalingtrigger.html#cfn-elasticmapreduce-instancegroupconfig-scalingtrigger-cloudwatchalarmdefinition
         */
        CloudWatchAlarmDefinition: C.Yields<AWS.EMR.InstanceGroupConfig.CloudWatchAlarmDefinition>;
    }
}
export declare namespace AWS.EMR.InstanceGroupConfig {
    interface SimpleScalingPolicyConfiguration {
        /**
         * String

         *
         * The way in which EC2 instances are added (if ScalingAdjustment is a
 positive number) or terminated (if ScalingAdjustment is a negative number)
 when the scaling activity is triggered.
 CHANGE_IN_CAPACITY
 is the default value.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-simplescalingpolicyconfiguration.html#cfn-elasticmapreduce-instancegroupconfig-simplescalingpolicyconfiguration-adjustmenttype
         */
        AdjustmentType?: C.YieldsString;
        /**
         * Integer

         *
         * The amount of time, in seconds, after a scaling activity completes before any
 further trigger-related scaling activities can start. The default value is 0.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-simplescalingpolicyconfiguration.html#cfn-elasticmapreduce-instancegroupconfig-simplescalingpolicyconfiguration-cooldown
         */
        CoolDown?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The amount by which to scale the instance group, based on the specified
 AdjustmentType. A positive value adds to the instance group's EC2
 instance count. A negative number removes instances. If AdjustmentType is
 set to EXACT_CAPACITY, specify only a positive integer. If
 AdjustmentType is set to PERCENT_CHANGE_IN_CAPACITY, express
 the value of the percentage as a decimal. For example, -0.20 indicates a
 decrease in 20% increments of cluster capacity.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-instancegroupconfig-simplescalingpolicyconfiguration.html#cfn-elasticmapreduce-instancegroupconfig-simplescalingpolicyconfiguration-scalingadjustment
         */
        ScalingAdjustment: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.EMR.InstanceGroupConfig {
    interface VolumeSpecification {
        /**
         * Integer

         *
         * The number of I/O operations per second (IOPS) that the volume supports. For more
 information, see Iops for the EbsBlockDevice action in the Amazon EC2 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-ebsconfiguration-ebsblockdeviceconfig-volumespecification.html#cfn-emr-ebsconfiguration-ebsblockdeviceconfig-volumespecification-iops
         */
        Iops?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The volume size, in Gibibytes (GiB). For more information about specifying the volume
 size, see VolumeSize for the EbsBlockDevice action in the Amazon EC2 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-ebsconfiguration-ebsblockdeviceconfig-volumespecification.html#cfn-emr-ebsconfiguration-ebsblockdeviceconfig-volumespecification-sizeingb
         */
        SizeInGB: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The volume type, such as standard or io1. For more information about specifying the volume type, see VolumeType for the EbsBlockDevice action in the Amazon EC2 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-emr-ebsconfiguration-ebsblockdeviceconfig-volumespecification.html#cfn-emr-ebsconfiguration-ebsblockdeviceconfig-volumespecification-volumetype
         */
        VolumeType: C.YieldsString;
    }
}
export declare namespace AWS.EMR {
    /**
     * The AWS::EMR::SecurityConfiguration resource creates a security
    configuration that is stored in the Amazon EMR web service. You can specify the security
    configuration when creating a cluster. For more information, see
       Specifying Amazon EMR Encryption Options Using a Security Configuration in the Amazon EMR Release Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-securityconfiguration.html
     */
    interface SecurityConfiguration {
        Type: 'AWS::EMR::SecurityConfiguration';
        Properties: AWS.EMR.SecurityConfiguration.Properties;
    }
    /**
     * The AWS::EMR::SecurityConfiguration resource creates a security
    configuration that is stored in the Amazon EMR web service. You can specify the security
    configuration when creating a cluster. For more information, see
       Specifying Amazon EMR Encryption Options Using a Security Configuration in the Amazon EMR Release Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-securityconfiguration.html
     */
    function SecurityConfiguration(props: C.Omit<SecurityConfiguration, 'Type'>): SecurityConfiguration;
}
export declare namespace AWS.EMR.SecurityConfiguration {
    interface Properties {
        /**
         * String

         *
         * The name of the security configuration. For a list of valid parameters for
 encryption settings, see  AWS CLI Security Configuration JSON Reference in the Amazon EMR Release
    Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-securityconfiguration.html#cfn-emr-securityconfiguration-name
         */
        Name?: C.YieldsString;
        /**
         * String

         *
         * The security configuration details in JSON format.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-securityconfiguration.html#cfn-emr-securityconfiguration-securityconfiguration
         */
        SecurityConfiguration: C.Yields<C.CFJson>;
    }
}
export declare namespace AWS.EMR {
    /**
     * The AWS::EMR::Step resource creates a unit of work (a job flow step) that you
    submit to an Amazon EMR (Amazon EMR) cluster. The job flow step contains instructions
    for processing
    data on the cluster.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-step.html
     */
    interface Step {
        Type: 'AWS::EMR::Step';
        Properties: AWS.EMR.Step.Properties;
    }
    /**
     * The AWS::EMR::Step resource creates a unit of work (a job flow step) that you
    submit to an Amazon EMR (Amazon EMR) cluster. The job flow step contains instructions
    for processing
    data on the cluster.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-step.html
     */
    function Step(props: C.Omit<Step, 'Type'>): Step;
}
export declare namespace AWS.EMR.Step {
    interface Properties {
        /**
         * String

         *
         * The action to take if the job flow step fails. Currently, AWS CloudFormation supports
 CONTINUE and CANCEL_AND_WAIT. For more information, see Managing Cluster Termination in the Amazon EMR Management Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-step.html#cfn-elasticmapreduce-step-actiononfailure
         */
        ActionOnFailure: C.YieldsString;
        /**
         * Amazon EMR Step
    HadoopJarStepConfig
         *
         * The JAR file that includes the main function that Amazon EMR executes.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-step.html#cfn-elasticmapreduce-step-hadoopjarstep
         */
        HadoopJarStep: C.Yields<AWS.EMR.Step.HadoopJarStepConfig>;
        /**
         * String

         *
         * The ID of a cluster in which you want to run this job flow step.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-step.html#cfn-elasticmapreduce-step-jobflowid
         */
        JobFlowId: C.YieldsString;
        /**
         * String

         *
         * A name for the job flow step.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-emr-step.html#cfn-elasticmapreduce-step-name
         */
        Name: C.YieldsString;
    }
}
export declare namespace AWS.EMR.Step {
    interface HadoopJarStepConfig {
        /**
         * List of strings

         *
         * A list of command line arguments passed to the JAR file's main function when the
 function is executed.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-step-hadoopjarstepconfig.html#cfn-elasticmapreduce-step-hadoopjarstepconfig-args
         */
        Args?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * A path to the JAR file that Amazon EMR runs for the job flow step.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-step-hadoopjarstepconfig.html#cfn-elasticmapreduce-step-hadoopjarstepconfig-jar
         */
        Jar: C.YieldsString;
        /**
         * String

         *
         * The name of the main class in the specified JAR file. If you don't specify a value,
 you must specify a main class in the JAR file's manifest file.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-step-hadoopjarstepconfig.html#cfn-elasticmapreduce-step-hadoopjarstepconfig-mainclass
         */
        MainClass?: C.YieldsString;
        /**
         * List of Amazon EMR Step
    KeyValue
         *
         * A list of Java properties that are set when the job flow step runs. You can use
 these properties to pass key-value pairs to your main function in the JAR file.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-step-hadoopjarstepconfig.html#cfn-elasticmapreduce-step-hadoopjarstepconfig-stepproperties
         */
        StepProperties?: C.Yields<C.CFList<C.Yields<AWS.EMR.Step.KeyValue>>>;
    }
}
export declare namespace AWS.EMR.Step {
    interface KeyValue {
        /**
         * String

         *
         * The unique identifier of a key-value pair.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-step-keyvalue.html#cfn-elasticmapreduce-step-keyvalue-key
         */
        Key?: C.YieldsString;
        /**
         * String

         *
         * The value part of the identified key.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticmapreduce-step-keyvalue.html#cfn-elasticmapreduce-step-keyvalue-value
         */
        Value?: C.YieldsString;
    }
}
export declare namespace AWS.ElastiCache {
    /**
     * The AWS::ElastiCache::CacheCluster type creates an Amazon ElastiCache cache cluster.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-cache-cluster.html
     */
    interface CacheCluster {
        Type: 'AWS::ElastiCache::CacheCluster';
        Properties: AWS.ElastiCache.CacheCluster.Properties;
    }
    /**
     * The AWS::ElastiCache::CacheCluster type creates an Amazon ElastiCache cache cluster.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-cache-cluster.html
     */
    function CacheCluster(props: C.Omit<CacheCluster, 'Type'>): CacheCluster;
}
export declare namespace AWS.ElastiCache.CacheCluster {
    interface Properties {
        /**
         * String

         *
         * For Memcached cache clusters, indicates whether the nodes are created in a
 single Availability Zone or across multiple Availability Zones in the cluster's
 region. For valid values, see CreateCacheCluster in
 the Amazon ElastiCache API Reference.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-cache-cluster.html#cfn-elasticache-cachecluster-azmode
         */
        AZMode?: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates that minor engine upgrades will be applied automatically to the cache
 cluster during the maintenance window.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-cache-cluster.html#cfn-elasticache-cachecluster-autominorversionupgrade
         */
        AutoMinorVersionUpgrade?: C.YieldsBoolean;
        /**
         * String

         *
         * The compute and memory capacity of nodes in a cache cluster.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-cache-cluster.html#cfn-elasticache-cachecluster-cachenodetype
         */
        CacheNodeType: C.YieldsString;
        /**
         * String

         *
         * The name of the cache parameter group that is associated with this cache
 cluster.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-cache-cluster.html#cfn-elasticache-cachecluster-cacheparametergroupname
         */
        CacheParameterGroupName?: C.YieldsString;
        /**
         * List of strings

         *
         * A list of cache security group names that are associated with this cache
 cluster. If your cache cluster is in a VPC, specify the
 VpcSecurityGroupIds property instead.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-cache-cluster.html#cfn-elasticache-cachecluster-cachesecuritygroupnames
         */
        CacheSecurityGroupNames?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The cache subnet group that you associate with a cache cluster.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-cache-cluster.html#cfn-elasticache-cachecluster-cachesubnetgroupname
         */
        CacheSubnetGroupName?: C.YieldsString;
        /**
         * String

         *
         * A name for the cache cluster. If you don't specify a name, AWS CloudFormation generates
 a
 unique physical ID and uses that ID for the cache cluster. For more information,
 see Name Type.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-cache-cluster.html#cfn-elasticache-cachecluster-clustername
         */
        ClusterName?: C.YieldsString;
        /**
         * String

         *
         * The name of the cache engine to be used for this cache cluster, such as
 memcached or redis.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-cache-cluster.html#cfn-elasticache-cachecluster-engine
         */
        Engine: C.YieldsString;
        /**
         * String

         *
         * The version of the cache engine to be used for this cluster.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-cache-cluster.html#cfn-elasticache-cachecluster-engineversion
         */
        EngineVersion?: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the Amazon Simple Notification Service (SNS)
 topic to which notifications will be sent.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-cache-cluster.html#cfn-elasticache-cachecluster-notificationtopicarn
         */
        NotificationTopicArn?: C.YieldsString;
        /**
         * Integer

         *
         * The number of cache nodes that the cache cluster should have.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-cache-cluster.html#cfn-elasticache-cachecluster-numcachenodes
         */
        NumCacheNodes: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The port number on which each of the cache nodes will accept
 connections.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-cache-cluster.html#cfn-elasticache-cachecluster-port
         */
        Port?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The Amazon EC2 Availability Zone in which the cache cluster is created.
         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-cache-cluster.html#cfn-elasticache-cachecluster-preferredavailabilityzone
         */
        PreferredAvailabilityZone?: C.YieldsString;
        /**
         * List of strings

         *
         * For Memcached cache clusters, the list of Availability Zones in which cache
 nodes are created. The number of Availability Zones listed must equal the number
 of cache nodes. For example, if you want to create three nodes in two different
 Availability Zones, you can specify ["us-east-1a", "us-east-1a",
    "us-east-1b"], which would create two nodes in us-east-1a and one node
 in us-east-1b.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-cache-cluster.html#cfn-elasticache-cachecluster-preferredavailabilityzones
         */
        PreferredAvailabilityZones?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The weekly time range (in UTC) during which system maintenance can
 occur.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-cache-cluster.html#cfn-elasticache-cachecluster-preferredmaintenancewindow
         */
        PreferredMaintenanceWindow?: C.YieldsString;
        /**
         * List of strings

         *
         * The ARN of the snapshot file that you want to use to seed a new Redis cache
 cluster. If you manage a Redis instance outside of Amazon ElastiCache, you can create
 a new
 cache cluster in ElastiCache by using a snapshot file that is stored in an Amazon
 S3
 bucket.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-cache-cluster.html#cfn-elasticache-cachecluster-snapshotarns
         */
        SnapshotArns?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The name of a snapshot from which to restore data into a new Redis cache
 cluster.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-cache-cluster.html#cfn-elasticache-cachecluster-snapshotname
         */
        SnapshotName?: C.YieldsString;
        /**
         * Integer

         *
         * For Redis cache clusters, the number of days for which ElastiCache retains automatic
 snapshots before deleting them. For example, if you set the value to
 5, a snapshot that was taken today will be retained for 5 days
 before being deleted.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-cache-cluster.html#cfn-elasticache-cachecluster-snapshotretentionlimit
         */
        SnapshotRetentionLimit?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * For Redis cache clusters, the daily time range (in UTC) during which ElastiCache will
 begin taking a daily snapshot of your node group. For example, you can specify
 05:00-09:00.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-cache-cluster.html#cfn-elasticache-cachecluster-snapshotwindow
         */
        SnapshotWindow?: C.YieldsString;
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key–value pairs) for this cache
 cluster.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-cache-cluster.html#cfn-elasticache-cachecluster-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * List of strings

         *
         * A list of VPC security group IDs. If your cache cluster isn't in a VPC, specify
 the CacheSecurityGroupNames property instead.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-cache-cluster.html#cfn-elasticache-cachecluster-vpcsecuritygroupids
         */
        VpcSecurityGroupIds?: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.ElastiCache {
    /**
     * The AWS::ElastiCache::ParameterGroup type creates a new cache parameter group. Cache
    parameter groups control
    the parameters for a cache cluster.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-parameter-group.html
     */
    interface ParameterGroup {
        Type: 'AWS::ElastiCache::ParameterGroup';
        Properties: AWS.ElastiCache.ParameterGroup.Properties;
    }
    /**
     * The AWS::ElastiCache::ParameterGroup type creates a new cache parameter group. Cache
    parameter groups control
    the parameters for a cache cluster.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-parameter-group.html
     */
    function ParameterGroup(props: C.Omit<ParameterGroup, 'Type'>): ParameterGroup;
}
export declare namespace AWS.ElastiCache.ParameterGroup {
    interface Properties {
        /**
         * String

         *
         * The name of the cache parameter group family that the cache parameter group can be
 used
 with.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-parameter-group.html#cfn-elasticache-parametergroup-cacheparametergroupfamily
         */
        CacheParameterGroupFamily: C.YieldsString;
        /**
         * String

         *
         * The description for the Cache Parameter Group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-parameter-group.html#cfn-elasticache-parametergroup-description
         */
        Description: C.YieldsString;
        /**
         * Mapping of key-value pairs

         *
         * A comma-delimited list of parameter name/value pairs. For more information, go to
 ModifyCacheParameterGroup in the Amazon ElastiCache API Reference
    Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-parameter-group.html#cfn-elasticache-parametergroup-properties
         */
        Properties?: C.Yields<C.CFMap<C.YieldsString>>;
    }
}
export declare namespace AWS.ElastiCache {
    /**
     * The AWS::ElastiCache::ReplicationGroup resource creates an Amazon ElastiCache Redis
    replication group. A replication group is a
    collection of cache clusters, where one of the clusters is a primary read-write cluster
    and
    the others are read-only replicas.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html
     */
    interface ReplicationGroup {
        Type: 'AWS::ElastiCache::ReplicationGroup';
        Properties: AWS.ElastiCache.ReplicationGroup.Properties;
    }
    /**
     * The AWS::ElastiCache::ReplicationGroup resource creates an Amazon ElastiCache Redis
    replication group. A replication group is a
    collection of cache clusters, where one of the clusters is a primary read-write cluster
    and
    the others are read-only replicas.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html
     */
    function ReplicationGroup(props: C.Omit<ReplicationGroup, 'Type'>): ReplicationGroup;
}
export declare namespace AWS.ElastiCache.ReplicationGroup {
    interface Properties {
        /**
         * Boolean

         *
         * Currently, this property isn't used by ElastiCache.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-autominorversionupgrade
         */
        AutoMinorVersionUpgrade?: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Indicates whether Multi-AZ is enabled. When Multi-AZ is enabled, a read-only
 replica is automatically promoted to a read-write primary cluster if the existing
 primary cluster fails. If you specify true, you must specify a value
 greater than 1 for the NumCacheClusters property. By
 default, AWS CloudFormation sets the value to true.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-automaticfailoverenabled
         */
        AutomaticFailoverEnabled?: C.YieldsBoolean;
        /**
         * String

         *
         * The compute and memory capacity of nodes in the node group. For valid values,
 see CreateReplicationGroup in the Amazon ElastiCache API
    Reference Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-cachenodetype
         */
        CacheNodeType?: C.YieldsString;
        /**
         * String

         *
         * The name of the parameter group to associate with this replication group. For
 valid and default values, see CreateReplicationGroup in the Amazon ElastiCache API
    Reference Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-cacheparametergroupname
         */
        CacheParameterGroupName?: C.YieldsString;
        /**
         * List of strings

         *
         * A list of cache security group names to associate with this replication
 group.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-cachesecuritygroupnames
         */
        CacheSecurityGroupNames?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The name of a cache subnet group to use for this replication group.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-cachesubnetgroupname
         */
        CacheSubnetGroupName?: C.YieldsString;
        /**
         * String

         *
         * The name of the cache engine to use for the cache clusters in this replication
 group. Currently, you can specify only redis.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-engine
         */
        Engine?: C.YieldsString;
        /**
         * String

         *
         * The version number of the cache engine to use for the cache clusters in this
 replication group.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-engineversion
         */
        EngineVersion?: C.YieldsString;
        /**
         * List of Amazon ElastiCache
    ReplicationGroup NodeGroupConfiguration
         *
         * Configuration options for the node group (shard).
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-nodegroupconfiguration
         */
        NodeGroupConfiguration?: C.Yields<C.CFList<C.Yields<AWS.ElastiCache.ReplicationGroup.NodeGroupConfiguration>>>;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the Amazon Simple Notification Service topic to
 which notifications are
 sent.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-notificationtopicarn
         */
        NotificationTopicArn?: C.YieldsString;
        /**
         * Integer

         *
         * The number of cache clusters for this replication group. If automatic failover
 is enabled, you must specify a value greater than 1. For valid
 values, see CreateReplicationGroup in the Amazon ElastiCache API
    Reference Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-numcacheclusters
         */
        NumCacheClusters?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The number of node groups (shards) for this Redis (clustered mode enabled) replication
 group.
 For Redis (clustered mode disabled), omit this property.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-numnodegroups
         */
        NumNodeGroups?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The port number on which each member of the replication group accepts
 connections.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-port
         */
        Port?: C.Yields<C.CFInteger>;
        /**
         * List of strings

         *
         * A list of Availability Zones in which the cache clusters in this replication
 group are created.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-preferredcacheclusterazs
         */
        PreferredCacheClusterAZs?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The weekly time range during which system maintenance can occur. Use the
 following format to specify a time range: ddd:hh24:mi-ddd:hh24:mi
 (24H Clock UTC). For example, you can specify sun:22:00-sun:23:30 for
 Sunday from 10 PM to 11:30 PM.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-preferredmaintenancewindow
         */
        PreferredMaintenanceWindow?: C.YieldsString;
        /**
         * String

         *
         * The cache cluster that ElastiCache uses as the primary cluster for the replication
 group. The cache cluster must have a status of available.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-primaryclusterid
         */
        PrimaryClusterId?: C.YieldsString;
        /**
         * Integer

         *
         * The number of replica nodes in each node group (shard). For valid values, see
 CreateReplicationGroup in the Amazon ElastiCache API
    Reference Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-replicaspernodegroup
         */
        ReplicasPerNodeGroup?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * A description of the replication group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-replicationgroupdescription
         */
        ReplicationGroupDescription: C.YieldsString;
        /**
         * String

         *
         * An ID for the replication group. If you don't specify an ID, AWS CloudFormation generates
 a
 unique physical ID. For more information, see Name Type.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-replicationgroupid
         */
        ReplicationGroupId?: C.YieldsString;
        /**
         * List of strings

         *
         * A list of Amazon Virtual Private Cloud (Amazon VPC) security groups to associate with
 this replication
 group.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-securitygroupids
         */
        SecurityGroupIds?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of strings

         *
         * A single-element string list that specifies an ARN of a Redis
 .rdb snapshot file that is stored in Amazon Simple Storage Service (Amazon S3). The
 snapshot file populates the node group. The Amazon S3 object name in the ARN cannot
 contain commas. For example, you can specify
 arn:aws:s3:::my_bucket/snapshot1.rdb.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-snapshotarns
         */
        SnapshotArns?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The name of a snapshot from which to restore data into the replication
 group.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-snapshotname
         */
        SnapshotName?: C.YieldsString;
        /**
         * Integer

         *
         * The number of days that ElastiCache retains automatic snapshots before deleting
 them.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-snapshotretentionlimit
         */
        SnapshotRetentionLimit?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The time range (in UTC) when ElastiCache takes a daily snapshot of the node group
 that you specified in the SnapshottingClusterId property. For
 example, you can specify 05:00-09:00.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-snapshotwindow
         */
        SnapshotWindow?: C.YieldsString;
        /**
         * String

         *
         * The ID of the cache cluster that ElastiCache uses as the daily snapshot source for
 the replication group.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-snapshottingclusterid
         */
        SnapshottingClusterId?: C.YieldsString;
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key–value pairs) for this replication
 group.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html#cfn-elasticache-replicationgroup-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
    }
}
export declare namespace AWS.ElastiCache.ReplicationGroup {
    interface NodeGroupConfiguration {
        /**
         * String

         *
         * The Availability Zone where ElastiCache launches the node group's primary
 node.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-replicationgroup-nodegroupconfiguration.html#cfn-elasticache-replicationgroup-nodegroupconfiguration-primaryavailabilityzone
         */
        PrimaryAvailabilityZone?: C.YieldsString;
        /**
         * List of strings

         *
         * A list of Availability Zones where ElastiCache launches the read replicas. The number
 of Availability Zones must match the value of the ReplicaCount
 property or, if you don't specify the ReplicaCount property, the
 replication group's ReplicasPerNodeGroup property.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-replicationgroup-nodegroupconfiguration.html#cfn-elasticache-replicationgroup-nodegroupconfiguration-replicaavailabilityzones
         */
        ReplicaAvailabilityZones?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * Integer

         *
         * The number of read replica nodes in the node group.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-replicationgroup-nodegroupconfiguration.html#cfn-elasticache-replicationgroup-nodegroupconfiguration-replicacount
         */
        ReplicaCount?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * A string of comma-separated values where the first set of values are the slot
 numbers (zero based), and the second set of values are the keyspaces for each
 slot. The following example specifies three slots (numbered 0, 1, and 2):
 0,1,2,0-4999,5000-9999,10000-16,383.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-replicationgroup-nodegroupconfiguration.html#cfn-elasticache-replicationgroup-nodegroupconfiguration-slots
         */
        Slots?: C.YieldsString;
    }
}
export declare namespace AWS.ElastiCache {
    /**
     * The AWS::ElastiCache::SecurityGroup resource creates a cache security group.
    For more information about cache security groups, go to Cache
       Security Groups in the Amazon ElastiCache User Guide or go to
    CreateCacheSecurityGroup in the Amazon ElastiCache API Reference
       Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-security-group.html
     */
    interface SecurityGroup {
        Type: 'AWS::ElastiCache::SecurityGroup';
        Properties: AWS.ElastiCache.SecurityGroup.Properties;
    }
    /**
     * The AWS::ElastiCache::SecurityGroup resource creates a cache security group.
    For more information about cache security groups, go to Cache
       Security Groups in the Amazon ElastiCache User Guide or go to
    CreateCacheSecurityGroup in the Amazon ElastiCache API Reference
       Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-security-group.html
     */
    function SecurityGroup(props: C.Omit<SecurityGroup, 'Type'>): SecurityGroup;
}
export declare namespace AWS.ElastiCache.SecurityGroup {
    interface Properties {
        /**
         * String

         *
         * A description for the cache security group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-security-group.html#cfn-elasticache-securitygroup-description
         */
        Description: C.YieldsString;
    }
}
export declare namespace AWS.ElastiCache {
    /**
     * The AWS::ElastiCache::SecurityGroupIngress type authorizes ingress to a cache security
    group from hosts in specified Amazon EC2 security groups. For more information about
    ElastiCache
    security group ingress, go to AuthorizeCacheSecurityGroupIngress in the Amazon ElastiCache API
       Reference Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-security-group-ingress.html
     */
    interface SecurityGroupIngress {
        Type: 'AWS::ElastiCache::SecurityGroupIngress';
        Properties: AWS.ElastiCache.SecurityGroupIngress.Properties;
    }
    /**
     * The AWS::ElastiCache::SecurityGroupIngress type authorizes ingress to a cache security
    group from hosts in specified Amazon EC2 security groups. For more information about
    ElastiCache
    security group ingress, go to AuthorizeCacheSecurityGroupIngress in the Amazon ElastiCache API
       Reference Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-security-group-ingress.html
     */
    function SecurityGroupIngress(props: C.Omit<SecurityGroupIngress, 'Type'>): SecurityGroupIngress;
}
export declare namespace AWS.ElastiCache.SecurityGroupIngress {
    interface Properties {
        /**
         * String

         *
         * The name of the Cache Security Group to authorize.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-security-group-ingress.html#cfn-elasticache-securitygroupingress-cachesecuritygroupname
         */
        CacheSecurityGroupName: C.YieldsString;
        /**
         * String

         *
         * Name of the EC2 Security Group to include in the authorization.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-security-group-ingress.html#cfn-elasticache-securitygroupingress-ec2securitygroupname
         */
        EC2SecurityGroupName: C.YieldsString;
        /**
         * String

         *
         * Specifies the AWS Account ID of the owner of the EC2 security group specified
 in the EC2SecurityGroupName property. The AWS access key ID is not an acceptable
 value.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-security-group-ingress.html#cfn-elasticache-securitygroupingress-ec2securitygroupownerid
         */
        EC2SecurityGroupOwnerId?: C.YieldsString;
    }
}
export declare namespace AWS.ElastiCache {
    /**
     * Creates a cache subnet group. For more information about cache subnet groups, go to
    Cache Subnet Groups in the
    Amazon ElastiCache User Guide or go to CreateCacheSubnetGroup in the
    Amazon ElastiCache API Reference Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-subnetgroup.html
     */
    interface SubnetGroup {
        Type: 'AWS::ElastiCache::SubnetGroup';
        Properties: AWS.ElastiCache.SubnetGroup.Properties;
    }
    /**
     * Creates a cache subnet group. For more information about cache subnet groups, go to
    Cache Subnet Groups in the
    Amazon ElastiCache User Guide or go to CreateCacheSubnetGroup in the
    Amazon ElastiCache API Reference Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-subnetgroup.html
     */
    function SubnetGroup(props: C.Omit<SubnetGroup, 'Type'>): SubnetGroup;
}
export declare namespace AWS.ElastiCache.SubnetGroup {
    interface Properties {
        /**
         * String

         *
         * A name for the cache subnet group. If you don't specify a name, AWS CloudFormation
 generates
 a unique physical ID. For more information, see Name Type.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-subnetgroup.html#cfn-elasticache-subnetgroup-cachesubnetgroupname
         */
        CacheSubnetGroupName?: C.YieldsString;
        /**
         * String

         *
         * The description for the cache subnet group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-subnetgroup.html#cfn-elasticache-subnetgroup-description
         */
        Description: C.YieldsString;
        /**
         * String list

         *
         * The Amazon EC2 subnet IDs for the cache subnet group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticache-subnetgroup.html#cfn-elasticache-subnetgroup-subnetids
         */
        SubnetIds: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.ElasticBeanstalk {
    /**
     * Creates an Elastic Beanstalk application.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk.html
     */
    interface Application {
        Type: 'AWS::ElasticBeanstalk::Application';
        Properties: AWS.ElasticBeanstalk.Application.Properties;
    }
    /**
     * Creates an Elastic Beanstalk application.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk.html
     */
    function Application(props: C.Omit<Application, 'Type'>): Application;
}
export declare namespace AWS.ElasticBeanstalk.Application {
    interface Properties {
        /**
         * String

         *
         * A name for the Elastic Beanstalk application. If you don't specify a name, AWS CloudFormation
 generates
 a unique physical ID and uses that ID for the application name. For more
 information, see Name Type.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk.html#cfn-elasticbeanstalk-application-name
         */
        ApplicationName?: C.YieldsString;
        /**
         * String

         *
         * An optional description of this application.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk.html#cfn-elasticbeanstalk-application-description
         */
        Description?: C.YieldsString;
    }
}
export declare namespace AWS.ElasticBeanstalk {
    /**
     * Creates an application version, an iteration of deployable code, for an Elastic Beanstalk
    application.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-version.html
     */
    interface ApplicationVersion {
        Type: 'AWS::ElasticBeanstalk::ApplicationVersion';
        Properties: AWS.ElasticBeanstalk.ApplicationVersion.Properties;
    }
    /**
     * Creates an application version, an iteration of deployable code, for an Elastic Beanstalk
    application.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-version.html
     */
    function ApplicationVersion(props: C.Omit<ApplicationVersion, 'Type'>): ApplicationVersion;
}
export declare namespace AWS.ElasticBeanstalk.ApplicationVersion {
    interface Properties {
        /**
         * String

         *
         * Name of the Elastic Beanstalk application that is associated with this application
 version.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-version.html#cfn-elasticbeanstalk-applicationversion-applicationname
         */
        ApplicationName: C.YieldsString;
        /**
         * String

         *
         * A description of this application version.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-version.html#cfn-elasticbeanstalk-applicationversion-description
         */
        Description?: C.YieldsString;
        /**
         * Source Bundle
         *
         * The location of the source bundle for this version.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-version.html#cfn-elasticbeanstalk-applicationversion-sourcebundle
         */
        SourceBundle: C.Yields<AWS.ElasticBeanstalk.ApplicationVersion.SourceBundle>;
    }
}
export declare namespace AWS.ElasticBeanstalk.ApplicationVersion {
    interface SourceBundle {
        /**
         * String

         *
         * The Amazon S3 bucket where the data is located.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-sourcebundle.html#cfn-beanstalk-sourcebundle-s3bucket
         */
        S3Bucket: C.YieldsString;
        /**
         * String

         *
         * The Amazon S3 key where the data is located.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-sourcebundle.html#cfn-beanstalk-sourcebundle-s3key
         */
        S3Key: C.YieldsString;
    }
}
export declare namespace AWS.ElasticBeanstalk {
    /**
     * Creates a configuration template for an Elastic Beanstalk application. You can use
    configuration
    templates to deploy different versions of an application by using the configuration
    settings
    that you define in the configuration template.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-beanstalk-configurationtemplate.html
     */
    interface ConfigurationTemplate {
        Type: 'AWS::ElasticBeanstalk::ConfigurationTemplate';
        Properties: AWS.ElasticBeanstalk.ConfigurationTemplate.Properties;
    }
    /**
     * Creates a configuration template for an Elastic Beanstalk application. You can use
    configuration
    templates to deploy different versions of an application by using the configuration
    settings
    that you define in the configuration template.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-beanstalk-configurationtemplate.html
     */
    function ConfigurationTemplate(props: C.Omit<ConfigurationTemplate, 'Type'>): ConfigurationTemplate;
}
export declare namespace AWS.ElasticBeanstalk.ConfigurationTemplate {
    interface Properties {
        /**
         * String

         *
         * Name of the Elastic Beanstalk application that is associated with this configuration
 template.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-beanstalk-configurationtemplate.html#cfn-elasticbeanstalk-configurationtemplate-applicationname
         */
        ApplicationName: C.YieldsString;
        /**
         * String

         *
         * An optional description for this configuration.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-beanstalk-configurationtemplate.html#cfn-elasticbeanstalk-configurationtemplate-description
         */
        Description?: C.YieldsString;
        /**
         * String

         *
         * An environment whose settings you want to use to create the configuration
 template. You must specify this property if you don't specify the
 SolutionStackName or SourceConfiguration
 properties.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-beanstalk-configurationtemplate.html#cfn-elasticbeanstalk-configurationtemplate-environmentid
         */
        EnvironmentId?: C.YieldsString;
        /**
         * A list of OptionSettings.

         *
         * A list of
    OptionSettings for this Elastic Beanstalk configuration, such as the
 instance type. For a complete list of Elastic Beanstalk configuration options, see
 
    Option Values, in the AWS Elastic Beanstalk Developer
    Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-beanstalk-configurationtemplate.html#cfn-elasticbeanstalk-configurationtemplate-optionsettings
         */
        OptionSettings?: C.Yields<C.CFList<C.Yields<AWS.ElasticBeanstalk.ConfigurationTemplate.ConfigurationOptionSetting>>>;
        /**
         * String

         *
         * The name of an Elastic Beanstalk solution stack that this configuration will use.
 A
 solution stack specifies the operating system, architecture, and application
 server for a configuration template, such as 64bit Amazon Linux 2013.09
    running Tomcat 7 Java 7. For more information, see Supported Platforms in the
 AWS Elastic Beanstalk Developer Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-beanstalk-configurationtemplate.html#cfn-elasticbeanstalk-configurationtemplate-solutionstackname
         */
        SolutionStackName?: C.YieldsString;
        /**
         * Elastic Beanstalk
    SourceConfiguration Property Type
         *
         * A configuration template that is associated with another Elastic Beanstalk application.
 If
 you specify the SolutionStackName property and the
 SourceConfiguration property, the solution stack in the source
 configuration template must match the value that you specified for the
 SolutionStackName property.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-beanstalk-configurationtemplate.html#cfn-elasticbeanstalk-configurationtemplate-sourceconfiguration
         */
        SourceConfiguration?: C.Yields<AWS.ElasticBeanstalk.ConfigurationTemplate.SourceConfiguration>;
    }
}
export declare namespace AWS.ElasticBeanstalk.ConfigurationTemplate {
    interface ConfigurationOptionSetting {
        /**
         * String

         *
         * A unique namespace identifying the option's associated AWS resource. For a list
 of namespaces that you can use, see Configuration Options in the AWS Elastic Beanstalk
    Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-option-settings.html#cfn-beanstalk-optionsettings-namespace
         */
        Namespace: C.YieldsString;
        /**
         * String

         *
         * The name of the configuration option. For a list of options that you can use,
 see Configuration Options in the AWS Elastic Beanstalk
    Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-option-settings.html#cfn-beanstalk-optionsettings-optionname
         */
        OptionName: C.YieldsString;
        /**
         * String

         *
         * The value of the setting.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-option-settings.html#cfn-beanstalk-optionsettings-value
         */
        Value: C.YieldsString;
    }
}
export declare namespace AWS.ElasticBeanstalk.ConfigurationTemplate {
    interface SourceConfiguration {
        /**
         * String

         *
         * The name of the Elastic Beanstalk application that contains the configuration template
 that
 you want to use.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-configurationtemplate-sourceconfiguration.html#cfn-beanstalk-configurationtemplate-sourceconfiguration-applicationname
         */
        ApplicationName: C.YieldsString;
        /**
         * String

         *
         * The name of the configuration template.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-configurationtemplate-sourceconfiguration.html#cfn-beanstalk-configurationtemplate-sourceconfiguration-templatename
         */
        TemplateName: C.YieldsString;
    }
}
export declare namespace AWS.ElasticBeanstalk {
    /**
     * Creates or updates an AWS Elastic Beanstalk environment.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-environment.html
     */
    interface Environment {
        Type: 'AWS::ElasticBeanstalk::Environment';
        Properties: AWS.ElasticBeanstalk.Environment.Properties;
    }
    /**
     * Creates or updates an AWS Elastic Beanstalk environment.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-environment.html
     */
    function Environment(props: C.Omit<Environment, 'Type'>): Environment;
}
export declare namespace AWS.ElasticBeanstalk.Environment {
    interface Properties {
        /**
         * String

         *
         * The name of the application that is associated with this environment.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-environment.html#cfn-beanstalk-environment-applicationname
         */
        ApplicationName: C.YieldsString;
        /**
         * String

         *
         * A prefix for your Elastic Beanstalk environment URL.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-environment.html#cfn-beanstalk-environment-cnameprefix
         */
        CNAMEPrefix?: C.YieldsString;
        /**
         * String

         *
         * A description that helps you identify this environment.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-environment.html#cfn-beanstalk-environment-description
         */
        Description?: C.YieldsString;
        /**
         * String

         *
         * A name for the Elastic Beanstalk environment. If you don't specify a name, AWS CloudFormation
 generates
 a unique physical ID and uses that ID for the environment name. For more
 information, see Name Type.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-environment.html#cfn-beanstalk-environment-name
         */
        EnvironmentName?: C.YieldsString;
        /**
         * List of OptionSettings
         *
         * Key-value pairs defining configuration options for this environment, such as
 the instance type. These options override the values that are defined in the
 solution stack or the configuration template. If you remove any options during a stack
 update, the removed options revert to default values.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-environment.html#cfn-beanstalk-environment-optionsettings
         */
        OptionSettings?: C.Yields<C.CFList<C.Yields<AWS.ElasticBeanstalk.Environment.OptionSettings>>>;
        /**
         * String

         *
         * The name of an Elastic Beanstalk solution stack that this configuration will use.
 For more
 information, see Supported
    Platforms in the AWS Elastic Beanstalk Developer Guide. You must
 specify either this parameter or an Elastic Beanstalk configuration template name.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-environment.html#cfn-beanstalk-environment-solutionstackname
         */
        SolutionStackName?: C.YieldsString;
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key–value pairs) for this environment.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-environment.html#cfn-elasticbeanstalk-environment-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * String

         *
         * The name of the Elastic Beanstalk configuration template to use with the environment.
 You
 must specify either this parameter or a solution stack name.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-environment.html#cfn-beanstalk-environment-templatename
         */
        TemplateName?: C.YieldsString;
        /**
         * Elastic Beanstalk Environment Tier Property
    Type
         *
         * Specifies the tier to use in creating this environment. The environment tier
 that you choose determines whether Elastic Beanstalk provisions resources to support
 a web
 application that handles HTTP(S) requests or a web application that handles
 background-processing tasks.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-environment.html#cfn-beanstalk-environment-tier
         */
        Tier?: C.Yields<AWS.ElasticBeanstalk.Environment.Tier>;
        /**
         * String

         *
         * The version to associate with the environment.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-environment.html#cfn-beanstalk-environment-versionlabel
         */
        VersionLabel?: C.YieldsString;
    }
}
export declare namespace AWS.ElasticBeanstalk.Environment {
    interface OptionSettings {
        /**
         * String

         *
         * A unique namespace identifying the option's associated AWS resource. For a list
 of namespaces that you can use, see Configuration Options in the AWS Elastic Beanstalk
    Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-option-settings.html#cfn-beanstalk-optionsettings-namespace
         */
        Namespace: C.YieldsString;
        /**
         * String

         *
         * The name of the configuration option. For a list of options that you can use,
 see Configuration Options in the AWS Elastic Beanstalk
    Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-option-settings.html#cfn-beanstalk-optionsettings-optionname
         */
        OptionName: C.YieldsString;
        /**
         * String

         *
         * The value of the setting.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-option-settings.html#cfn-beanstalk-optionsettings-value
         */
        Value: C.YieldsString;
    }
}
export declare namespace AWS.ElasticBeanstalk.Environment {
    interface Tier {
        /**
         * String

         *
         * The name of the environment tier. You can specify WebServer or
 Worker.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-environment-tier.html#cfn-beanstalk-env-tier-name
         */
        Name?: C.YieldsString;
        /**
         * String

         *
         * The type of this environment tier. You can specify Standard for
 the WebServer tier or SQS/HTTP for the
 Worker tier.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-environment-tier.html#cfn-beanstalk-env-tier-type
         */
        Type?: C.YieldsString;
        /**
         * String

         *
         * The version of this environment tier.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-beanstalk-environment-tier.html#cfn-beanstalk-env-tier-version
         */
        Version?: C.YieldsString;
    }
}
export declare namespace AWS.ElasticLoadBalancing {
    /**
     * The AWS::ElasticLoadBalancing::LoadBalancer type creates a LoadBalancer.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb.html
     */
    interface LoadBalancer {
        Type: 'AWS::ElasticLoadBalancing::LoadBalancer';
        Properties: AWS.ElasticLoadBalancing.LoadBalancer.Properties;
    }
    /**
     * The AWS::ElasticLoadBalancing::LoadBalancer type creates a LoadBalancer.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb.html
     */
    function LoadBalancer(props: C.Omit<LoadBalancer, 'Type'>): LoadBalancer;
}
export declare namespace AWS.ElasticLoadBalancing.LoadBalancer {
    interface Properties {
        /**
         * Elastic Load Balancing AccessLoggingPolicy
         *
         * Captures detailed information for all requests made to your load balancer, such
 as the time a request was received, client’s IP address, latencies, request path,
 and server responses.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb.html#cfn-ec2-elb-accessloggingpolicy
         */
        AccessLoggingPolicy?: C.Yields<AWS.ElasticLoadBalancing.LoadBalancer.AccessLoggingPolicy>;
        /**
         * A list of AppCookieStickinessPolicy objects.

         *
         * Generates one or more stickiness policies with sticky session lifetimes that
 follow that of an application-generated cookie. These policies can be associated
 only with HTTP/HTTPS listeners.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb.html#cfn-ec2-elb-appcookiestickinesspolicy
         */
        AppCookieStickinessPolicy?: C.Yields<C.CFList<C.Yields<AWS.ElasticLoadBalancing.LoadBalancer.AppCookieStickinessPolicy>>>;
        /**
         * List of strings

         *
         * The Availability Zones in which to create the load balancer. You can specify
 the AvailabilityZones or Subnets property, but not
 both.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb.html#cfn-ec2-elb-availabilityzones
         */
        AvailabilityZones?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * Elastic Load Balancing
    ConnectionDrainingPolicy
         *
         * Whether deregistered or unhealthy instances can complete all in-flight
 requests.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb.html#cfn-ec2-elb-connectiondrainingpolicy
         */
        ConnectionDrainingPolicy?: C.Yields<AWS.ElasticLoadBalancing.LoadBalancer.ConnectionDrainingPolicy>;
        /**
         * Elastic Load Balancing ConnectionSettings
         *
         * Specifies how long front-end and back-end connections of your load balancer can
 remain idle.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb.html#cfn-ec2-elb-connectionsettings
         */
        ConnectionSettings?: C.Yields<AWS.ElasticLoadBalancing.LoadBalancer.ConnectionSettings>;
        /**
         * Boolean

         *
         * Whether cross-zone load balancing is enabled for the load balancer. With
 cross-zone load balancing, your load balancer nodes route traffic to the back-end
 instances across all Availability Zones. By default the CrossZone
 property is false.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb.html#cfn-ec2-elb-crosszone
         */
        CrossZone?: C.YieldsBoolean;
        /**
         * ElasticLoadBalancing LoadBalancer HealthCheck.

         *
         * Application health check for the instances.
         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb.html#cfn-ec2-elb-healthcheck
         */
        HealthCheck?: C.Yields<AWS.ElasticLoadBalancing.LoadBalancer.HealthCheck>;
        /**
         * List of strings

         *
         * A list of EC2 instance IDs for the load balancer.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb.html#cfn-ec2-elb-instances
         */
        Instances?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * A list of LBCookieStickinessPolicy objects.

         *
         * Generates a stickiness policy with sticky session lifetimes controlled by the
 lifetime of the browser (user-agent), or by a specified expiration period. This
 policy can be associated only with HTTP/HTTPS listeners.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb.html#cfn-ec2-elb-lbcookiestickinesspolicy
         */
        LBCookieStickinessPolicy?: C.Yields<C.CFList<C.Yields<AWS.ElasticLoadBalancing.LoadBalancer.LBCookieStickinessPolicy>>>;
        /**
         * A list of ElasticLoadBalancing Listener Property Type objects.

         *
         * One or more listeners for this load balancer. Each listener must be registered
 for a specific port, and you cannot have more than one listener for a given
 port.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb.html#cfn-ec2-elb-listeners
         */
        Listeners: C.Yields<C.CFList<C.Yields<AWS.ElasticLoadBalancing.LoadBalancer.Listeners>>>;
        /**
         * String

         *
         * A name for the load balancer. For valid values, see the LoadBalancerName parameter for the CreateLoadBalancer action in the Elastic Load Balancing API Reference version 2012-06-01.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb.html#cfn-ec2-elb-elbname
         */
        LoadBalancerName?: C.YieldsString;
        /**
         * A list of ElasticLoadBalancing policy objects.

         *
         * A list of elastic load balancing policies to apply to this elastic load balancer.
 Specify only back-end server policies. For more information, see DescribeLoadBalancerPolicyTypes in the Elastic Load Balancing API Reference version 2012-06-01.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb.html#cfn-ec2-elb-policies
         */
        Policies?: C.Yields<C.CFList<C.Yields<AWS.ElasticLoadBalancing.LoadBalancer.Policies>>>;
        /**
         * String

         *
         * For load balancers attached to an Amazon VPC, this parameter can be used to
 specify the type of load balancer to use. Specify internal to create
 an internal load balancer with a DNS name that resolves to private IP addresses or
 internet-facing to create a load balancer with a publicly
 resolvable DNS name, which resolves to public IP addresses.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb.html#cfn-ec2-elb-scheme
         */
        Scheme?: C.YieldsString;
        /**
         * A list of security groups assigned to your load balancer within your
 virtual private cloud (VPC).

         *
         * Required: No

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb.html#cfn-ec2-elb-securitygroups
         */
        SecurityGroups?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of strings

         *
         * A list of subnet IDs in your virtual private cloud (VPC) to attach to your load
 balancer. Do not specify multiple subnets that are in the same Availability Zone.
 You can specify the AvailabilityZones or Subnets
 property, but not both.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb.html#cfn-ec2-elb-subnets
         */
        Subnets?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key-value pairs) for this load balancer.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb.html#cfn-elasticloadbalancing-loadbalancer-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
    }
}
export declare namespace AWS.ElasticLoadBalancing.LoadBalancer {
    interface AccessLoggingPolicy {
        /**
         * Integer

         *
         * The interval for publishing access logs in minutes. You can specify an interval
 of either 5 minutes or 60 minutes.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-accessloggingpolicy.html#cfn-elb-accessloggingpolicy-emitinterval
         */
        EmitInterval?: C.Yields<C.CFInteger>;
        /**
         * Boolean

         *
         * Whether logging is enabled for the load balancer.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-accessloggingpolicy.html#cfn-elb-accessloggingpolicy-enabled
         */
        Enabled: C.YieldsBoolean;
        /**
         * String

         *
         * The name of an Amazon S3 bucket where access log files are stored.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-accessloggingpolicy.html#cfn-elb-accessloggingpolicy-s3bucketname
         */
        S3BucketName: C.YieldsString;
        /**
         * String

         *
         * A prefix for the all log object keys, such as
 my-load-balancer-logs/prod. If you store log files from multiple
 sources in a single bucket, you can use a prefix to distinguish each log file and
 its source.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-accessloggingpolicy.html#cfn-elb-accessloggingpolicy-s3bucketprefix
         */
        S3BucketPrefix?: C.YieldsString;
    }
}
export declare namespace AWS.ElasticLoadBalancing.LoadBalancer {
    interface AppCookieStickinessPolicy {
        /**
         * String

         *
         * Name of the application cookie used for stickiness.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-AppCookieStickinessPolicy.html#cfn-elb-appcookiestickinesspolicy-cookiename
         */
        CookieName: C.YieldsString;
        /**
         * String

         *
         * The name of the policy being created. The name must be unique within the set of
 policies for this Load Balancer.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-AppCookieStickinessPolicy.html#cfn-elb-appcookiestickinesspolicy-policyname
         */
        PolicyName: C.YieldsString;
    }
}
export declare namespace AWS.ElasticLoadBalancing.LoadBalancer {
    interface ConnectionDrainingPolicy {
        /**
         * Boolean

         *
         * Whether or not connection draining is enabled for the load balancer.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-connectiondrainingpolicy.html#cfn-elb-connectiondrainingpolicy-enabled
         */
        Enabled: C.YieldsBoolean;
        /**
         * Integer

         *
         * The time in seconds after the load balancer closes all connections to a
 deregistered or unhealthy instance.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-connectiondrainingpolicy.html#cfn-elb-connectiondrainingpolicy-timeout
         */
        Timeout?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.ElasticLoadBalancing.LoadBalancer {
    interface ConnectionSettings {
        /**
         * Integer

         *
         * The time (in seconds) that a connection to the load balancer can remain idle,
 which means no data is sent over the connection. After the specified time, the
 load balancer closes the connection.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-connectionsettings.html#cfn-elb-connectionsettings-idletimeout
         */
        IdleTimeout: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.ElasticLoadBalancing.LoadBalancer {
    interface HealthCheck {
        /**
         * String

         *
         * Specifies the number of consecutive health probe successes required before moving
 the instance to
 the Healthy state.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-health-check.html#cfn-elb-healthcheck-healthythreshold
         */
        HealthyThreshold: C.YieldsString;
        /**
         * String

         *
         * Specifies the approximate interval, in seconds, between health checks of an individual
 instance. Valid values are 5 to 300. The default is 30.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-health-check.html#cfn-elb-healthcheck-interval
         */
        Interval: C.YieldsString;
        /**
         * String

         *
         * Specifies the instance's protocol and port to check. The protocol can be TCP,
 HTTP, HTTPS, or SSL. The range of valid ports is 1 through 65535.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-health-check.html#cfn-elb-healthcheck-target
         */
        Target: C.YieldsString;
        /**
         * String

         *
         * Specifies the amount of time, in seconds, during which no response means a failed
 health probe.
 This value must be less than the value for Interval.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-health-check.html#cfn-elb-healthcheck-timeout
         */
        Timeout: C.YieldsString;
        /**
         * String

         *
         * Specifies the number of consecutive health probe failures required before moving the
 instance to
 the Unhealthy state.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-health-check.html#cfn-elb-healthcheck-unhealthythreshold
         */
        UnhealthyThreshold: C.YieldsString;
    }
}
export declare namespace AWS.ElasticLoadBalancing.LoadBalancer {
    interface LBCookieStickinessPolicy {
        /**
         * String

         *
         * The time period, in seconds, after which the cookie should be considered stale. If
 this parameter
 isn't specified, the sticky session will last for the duration of the browser session.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-LBCookieStickinessPolicy.html#cfn-elb-lbcookiestickinesspolicy-cookieexpirationperiod
         */
        CookieExpirationPeriod?: C.YieldsString;
        /**
         *
         *
         * The name of the policy being created. The name must be unique within the set of
 policies for this load balancer.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-LBCookieStickinessPolicy.html#cfn-elb-lbcookiestickinesspolicy-policyname
         */
        PolicyName?: C.YieldsString;
    }
}
export declare namespace AWS.ElasticLoadBalancing.LoadBalancer {
    interface Listeners {
        /**
         * String

         *
         * Specifies the TCP port on which the instance server listens. You can't modify
 this property during the life of the load balancer.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-listener.html#cfn-ec2-elb-listener-instanceport
         */
        InstancePort: C.YieldsString;
        /**
         * String

         *
         * Specifies the protocol to use for routing traffic to back-end instances: HTTP,
 HTTPS, TCP, or SSL. You can't modify this property during the life of the load
 balancer.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-listener.html#cfn-ec2-elb-listener-instanceprotocol
         */
        InstanceProtocol?: C.YieldsString;
        /**
         * String

         *
         * Specifies the external load balancer port number. You can't modify this
 property during the life of the load balancer.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-listener.html#cfn-ec2-elb-listener-loadbalancerport
         */
        LoadBalancerPort: C.YieldsString;
        /**
         * List of strings

         *
         * A list of ElasticLoadBalancing policy names to associate with the
 Listener. Specify only policies that are compatible with a
 Listener. For more information, see DescribeLoadBalancerPolicyTypes in the
 Elastic Load Balancing API Reference version 2012-06-01.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-listener.html#cfn-ec2-elb-listener-policynames
         */
        PolicyNames?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * Specifies the load balancer transport protocol to use for routing: HTTP, HTTPS,
 TCP or SSL. You can't modify this property during the life of the load
 balancer.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-listener.html#cfn-ec2-elb-listener-protocol
         */
        Protocol: C.YieldsString;
        /**
         * String

         *
         * The ARN of the SSL certificate to use. For more information about SSL
 certificates, see Managing
    Server Certificates in the AWS Identity and Access
    Management User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-listener.html#cfn-ec2-elb-listener-sslcertificateid
         */
        SSLCertificateId?: C.YieldsString;
    }
}
export declare namespace AWS.ElasticLoadBalancing.LoadBalancer {
    interface Policies {
        /**
         * List of JSON name-value pairs.

         *
         * A list of arbitrary attributes for this policy. If you don't need to specify
 any policy attributes, specify an empty list ([]).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-policy.html#cfn-ec2-elb-policy-attributes
         */
        Attributes: C.Yields<C.CFList<C.Yields<C.CFJson>>>;
        /**
         * List of String

         *
         * A list of instance ports for the policy. These are the ports associated with the back-end
 server.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-policy.html#cfn-ec2-elb-policy-instanceports
         */
        InstancePorts?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of String

         *
         * A list of external load balancer ports for the policy.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-policy.html#cfn-ec2-elb-policy-loadbalancerports
         */
        LoadBalancerPorts?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * A name for this policy that is unique to the load balancer.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-policy.html#cfn-ec2-elb-policy-policyname
         */
        PolicyName: C.YieldsString;
        /**
         * String

         *
         * The name of the policy type for this policy. This must be one of the types reported
 by the Elastic Load Balancing
 DescribeLoadBalancerPolicyTypes action.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-elb-policy.html#cfn-ec2-elb-policy-policytype
         */
        PolicyType: C.YieldsString;
    }
}
export declare namespace AWS.ElasticLoadBalancingV2 {
    /**
     * The AWS::ElasticLoadBalancingV2::Listener resource creates a listener for an
    Elastic Load Balancing Application or Network load balancer. The listener checks for
    connection requests and
    forwards them to one or more target groups. For more information, see Getting Started in the Elastic Load Balancing User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-listener.html
     */
    interface Listener {
        Type: 'AWS::ElasticLoadBalancingV2::Listener';
        Properties: AWS.ElasticLoadBalancingV2.Listener.Properties;
    }
    /**
     * The AWS::ElasticLoadBalancingV2::Listener resource creates a listener for an
    Elastic Load Balancing Application or Network load balancer. The listener checks for
    connection requests and
    forwards them to one or more target groups. For more information, see Getting Started in the Elastic Load Balancing User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-listener.html
     */
    function Listener(props: C.Omit<Listener, 'Type'>): Listener;
}
export declare namespace AWS.ElasticLoadBalancingV2.Listener {
    interface Properties {
        /**
         * List of Elastic Load Balancing Listener Certificates
         *
         * The SSL server certificate for the listener. With a certificate, you can encrypt traffic
 between the load balancer and the clients that initiate HTTPS sessions, and traffic
 between the load balancer and your targets.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-listener.html#cfn-elasticloadbalancingv2-listener-certificates
         */
        Certificates?: C.Yields<C.CFList<C.Yields<AWS.ElasticLoadBalancingV2.Listener.Certificate>>>;
        /**
         * List of Elastic Load Balancing Listener DefaultActions
         *
         * The default actions that the listener takes when handling incoming requests.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-listener.html#cfn-elasticloadbalancingv2-listener-defaultactions
         */
        DefaultActions: C.Yields<C.CFList<C.Yields<AWS.ElasticLoadBalancingV2.Listener.Action>>>;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the load balancer to associate with the listener.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-listener.html#cfn-elasticloadbalancingv2-listener-loadbalancerarn
         */
        LoadBalancerArn: C.YieldsString;
        /**
         * Integer

         *
         * The port on which the listener listens for requests.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-listener.html#cfn-elasticloadbalancingv2-listener-port
         */
        Port: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The protocol that clients must use to send requests to the listener.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-listener.html#cfn-elasticloadbalancingv2-listener-protocol
         */
        Protocol: C.YieldsString;
        /**
         * String

         *
         * The security policy that defines the ciphers and protocols that the load balancer
 supports.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-listener.html#cfn-elasticloadbalancingv2-listener-sslpolicy
         */
        SslPolicy?: C.YieldsString;
    }
}
export declare namespace AWS.ElasticLoadBalancingV2.Listener {
    interface Action {
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the target group to which Elastic Load Balancing
 routes the traffic.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticloadbalancingv2-listener-defaultactions.html#cfn-elasticloadbalancingv2-listener-defaultactions-targetgrouparn
         */
        TargetGroupArn: C.YieldsString;
        /**
         * String

         *
         * The type of action. For valid values, see the Type contents for the
 Action data type in the
 Elastic Load Balancing API Reference version 2015-12-01.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticloadbalancingv2-listener-defaultactions.html#cfn-elasticloadbalancingv2-listener-defaultactions-type
         */
        Type: C.YieldsString;
    }
}
export declare namespace AWS.ElasticLoadBalancingV2.Listener {
    interface Certificate {
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the certificate to associate with the listener.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticloadbalancingv2-listener-certificates.html#cfn-elasticloadbalancingv2-listener-certificates-certificatearn
         */
        CertificateArn?: C.YieldsString;
    }
}
export declare namespace AWS.ElasticLoadBalancingV2 {
    /**
     *
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-listenercertificate.html
     */
    interface ListenerCertificate {
        Type: 'AWS::ElasticLoadBalancingV2::ListenerCertificate';
        Properties: AWS.ElasticLoadBalancingV2.ListenerCertificate.Properties;
    }
    /**
     *
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-listenercertificate.html
     */
    function ListenerCertificate(props: C.Omit<ListenerCertificate, 'Type'>): ListenerCertificate;
}
export declare namespace AWS.ElasticLoadBalancingV2.ListenerCertificate {
    interface Properties {
        /**
         *
         *
         *
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-listenercertificate.html#cfn-elasticloadbalancingv2-listenercertificate-certificates
         */
        Certificates: C.Yields<C.CFList<C.Yields<AWS.ElasticLoadBalancingV2.ListenerCertificate.Certificate>>>;
        /**
         *
         *
         *
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-listenercertificate.html#cfn-elasticloadbalancingv2-listenercertificate-listenerarn
         */
        ListenerArn: C.YieldsString;
    }
}
export declare namespace AWS.ElasticLoadBalancingV2.ListenerCertificate {
    interface Certificate {
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the certificate to associate with the listener.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticloadbalancingv2-listener-certificates.html#cfn-elasticloadbalancingv2-listener-certificates-certificatearn
         */
        CertificateArn?: C.YieldsString;
    }
}
export declare namespace AWS.ElasticLoadBalancingV2 {
    /**
     * The AWS::ElasticLoadBalancingV2::ListenerRule resource defines which requests an Elastic Load Balancing listener takes action on
    and the action that it takes. For more information, see Getting Started in the Elastic Load Balancing User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-listenerrule.html
     */
    interface ListenerRule {
        Type: 'AWS::ElasticLoadBalancingV2::ListenerRule';
        Properties: AWS.ElasticLoadBalancingV2.ListenerRule.Properties;
    }
    /**
     * The AWS::ElasticLoadBalancingV2::ListenerRule resource defines which requests an Elastic Load Balancing listener takes action on
    and the action that it takes. For more information, see Getting Started in the Elastic Load Balancing User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-listenerrule.html
     */
    function ListenerRule(props: C.Omit<ListenerRule, 'Type'>): ListenerRule;
}
export declare namespace AWS.ElasticLoadBalancingV2.ListenerRule {
    interface Properties {
        /**
         * List of Elastic Load Balancing ListenerRule Actions
         *
         * The action that the listener takes when a request meets the specified condition.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-listenerrule.html#cfn-elasticloadbalancingv2-listenerrule-actions
         */
        Actions: C.Yields<C.CFList<C.Yields<AWS.ElasticLoadBalancingV2.ListenerRule.Action>>>;
        /**
         * List of Elastic Load Balancing ListenerRule Conditions
         *
         * The conditions under which a rule takes effect.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-listenerrule.html#cfn-elasticloadbalancingv2-listenerrule-conditions
         */
        Conditions: C.Yields<C.CFList<C.Yields<AWS.ElasticLoadBalancingV2.ListenerRule.RuleCondition>>>;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the listener that the rule applies to.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-listenerrule.html#cfn-elasticloadbalancingv2-listenerrule-listenerarn
         */
        ListenerArn: C.YieldsString;
        /**
         * Integer

         *
         * The priority for the rule. Elastic Load Balancing evaluates rules in priority order,
 from the lowest value to the highest value. If a request satisfies a rule, Elastic
 Load Balancing ignores all subsequent rules.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-listenerrule.html#cfn-elasticloadbalancingv2-listenerrule-priority
         */
        Priority: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.ElasticLoadBalancingV2.ListenerRule {
    interface Action {
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the target group to which Elastic Load Balancing
 routes the traffic.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticloadbalancingv2-listenerrule-actions.html#cfn-elasticloadbalancingv2-listener-actions-targetgrouparn
         */
        TargetGroupArn: C.YieldsString;
        /**
         * String

         *
         * The type of action. For valid values, see the Type contents for the
 Action data type in the
 Elastic Load Balancing API Reference version 2015-12-01.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticloadbalancingv2-listenerrule-actions.html#cfn-elasticloadbalancingv2-listener-actions-type
         */
        Type: C.YieldsString;
    }
}
export declare namespace AWS.ElasticLoadBalancingV2.ListenerRule {
    interface RuleCondition {
        /**
         * String

         *
         * The name of the condition that you want to define, such as path-pattern (which forwards requests based on the URL of the request).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticloadbalancingv2-listenerrule-conditions.html#cfn-elasticloadbalancingv2-listenerrule-conditions-field
         */
        Field?: C.YieldsString;
        /**
         * List of strings

         *
         * The value for the field that you specified in the Field property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticloadbalancingv2-listenerrule-conditions.html#cfn-elasticloadbalancingv2-listenerrule-conditions-values
         */
        Values?: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.ElasticLoadBalancingV2 {
    /**
     * The AWS::ElasticLoadBalancingV2::LoadBalancer resource creates an Elastic Load Balancing
    Application or Network Load Balancer. For more information, see the see
    Getting Started in the Elastic Load Balancing User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-loadbalancer.html
     */
    interface LoadBalancer {
        Type: 'AWS::ElasticLoadBalancingV2::LoadBalancer';
        Properties: AWS.ElasticLoadBalancingV2.LoadBalancer.Properties;
    }
    /**
     * The AWS::ElasticLoadBalancingV2::LoadBalancer resource creates an Elastic Load Balancing
    Application or Network Load Balancer. For more information, see the see
    Getting Started in the Elastic Load Balancing User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-loadbalancer.html
     */
    function LoadBalancer(props: C.Omit<LoadBalancer, 'Type'>): LoadBalancer;
}
export declare namespace AWS.ElasticLoadBalancingV2.LoadBalancer {
    interface Properties {
        /**
         * String

         *
         * [Application Load Balancers] The type of IP addresses that are used by the load
 balancer's subnets, such as ipv4 (for IPv4 addresses) or
 dualstack (for IPv4 and IPv6 addresses). For valid values, see the
 IpAddressType parameter for the CreateLoadBalancer
 action in the Elastic Load Balancing API Reference version 2015-12-01.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-loadbalancer.html#cfn-elasticloadbalancingv2-loadbalancer-ipaddresstype
         */
        IpAddressType?: C.YieldsString;
        /**
         * A list of Elastic Load Balancing LoadBalancer LoadBalancerAttributes
         *
         * Specifies the load balancer configuration.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-loadbalancer.html#cfn-elasticloadbalancingv2-loadbalancer-loadbalancerattributes
         */
        LoadBalancerAttributes?: C.Yields<C.CFList<C.Yields<AWS.ElasticLoadBalancingV2.LoadBalancer.LoadBalancerAttribute>>>;
        /**
         * String

         *
         * Specifies a name for the load balancer. This name must be unique within your AWS
 account and can have a maximum of 32 alphanumeric characters and hyphens. A name can't
 begin or end with a hyphen.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-loadbalancer.html#cfn-elasticloadbalancingv2-loadbalancer-name
         */
        Name?: C.YieldsString;
        /**
         * String

         *
         * Specifies whether the load balancer is internal or Internet-facing. Valid values are
 internet-facing and internal. The default is internet-facing.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-loadbalancer.html#cfn-elasticloadbalancingv2-loadbalancer-scheme
         */
        Scheme?: C.YieldsString;
        /**
         * List of strings

         *
         * [Application Load Balancers] Specifies a list of the IDs of the security groups to
 assign to the load balancer.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-loadbalancer.html#cfn-elasticloadbalancingv2-loadbalancer-securitygroups
         */
        SecurityGroups?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * A list of Elastic Load Balancing LoadBalancer SubnetMapping
         *
         * The IDs of the subnets to attach to the load balancer. You can specify only one subnet
 per Availability Zone. You must specify either subnets or subnet mappings.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-loadbalancer.html#cfn-elasticloadbalancingv2-loadbalancer-subnetmappings
         */
        SubnetMappings?: C.Yields<C.CFList<C.Yields<AWS.ElasticLoadBalancingV2.LoadBalancer.SubnetMapping>>>;
        /**
         * List of strings

         *
         * The IDs of the subnets to attach to the load balancer. You can specify only one subnet
 per Availability Zone. You must specify either subnets or subnet mappings.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-loadbalancer.html#cfn-elasticloadbalancingv2-loadbalancer-subnets
         */
        Subnets?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * AWS CloudFormation Resource Tags
         *
         * Specifies an arbitrary set of tags (key–value pairs) to associate with this
 load balancer. Use tags to manage your resources.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-loadbalancer.html#cfn-elasticloadbalancingv2-loadbalancer-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * String

         *
         * Specifies the type of load balancer to create. Valid values are application and network.The default is application.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-loadbalancer.html#cfn-elasticloadbalancingv2-loadbalancer-type
         */
        Type?: C.YieldsString;
    }
}
export declare namespace AWS.ElasticLoadBalancingV2.LoadBalancer {
    interface LoadBalancerAttribute {
        /**
         * String

         *
         * The name of an attribute that you want to configure. For the list of attributes that
 you can configure, see the Key contents for the LoadBalancerAttribute data type in the Elastic Load Balancing API Reference version 2015-12-01.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticloadbalancingv2-loadbalancer-loadbalancerattributes.html#cfn-elasticloadbalancingv2-loadbalancer-loadbalancerattributes-key
         */
        Key?: C.YieldsString;
        /**
         * String

         *
         * A value for the attribute.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticloadbalancingv2-loadbalancer-loadbalancerattributes.html#cfn-elasticloadbalancingv2-loadbalancer-loadbalancerattributes-value
         */
        Value?: C.YieldsString;
    }
}
export declare namespace AWS.ElasticLoadBalancingV2.LoadBalancer {
    interface SubnetMapping {
        /**
         * String

         *
         * [Network Load Balancer] The ID that represents the allocation of the Elastic IP address.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticloadbalancingv2-loadbalancer-subnetmapping.html#cfn-elasticloadbalancingv2-loadbalancer-subnetmapping-allocationid
         */
        AllocationId: C.YieldsString;
        /**
         * String

         *
         * The ID of the subnet.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticloadbalancingv2-loadbalancer-subnetmapping.html#cfn-elasticloadbalancingv2-loadbalancer-subnetmapping-subnetid
         */
        SubnetId: C.YieldsString;
    }
}
export declare namespace AWS.ElasticLoadBalancingV2 {
    /**
     * The AWS::ElasticLoadBalancingV2::TargetGroup resource creates an Elastic Load Balancing
    target group that routes requests to one or more registered targets, such as EC2 instances.
    For
    more information, see Getting Started in the Elastic Load Balancing User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-targetgroup.html
     */
    interface TargetGroup {
        Type: 'AWS::ElasticLoadBalancingV2::TargetGroup';
        Properties: AWS.ElasticLoadBalancingV2.TargetGroup.Properties;
    }
    /**
     * The AWS::ElasticLoadBalancingV2::TargetGroup resource creates an Elastic Load Balancing
    target group that routes requests to one or more registered targets, such as EC2 instances.
    For
    more information, see Getting Started in the Elastic Load Balancing User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-targetgroup.html
     */
    function TargetGroup(props: C.Omit<TargetGroup, 'Type'>): TargetGroup;
}
export declare namespace AWS.ElasticLoadBalancingV2.TargetGroup {
    interface Properties {
        /**
         * Integer

         *
         * The approximate number of seconds between health checks for an individual target.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-targetgroup.html#cfn-elasticloadbalancingv2-targetgroup-healthcheckintervalseconds
         */
        HealthCheckIntervalSeconds?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The ping path destination where Elastic Load Balancing sends health check requests.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-targetgroup.html#cfn-elasticloadbalancingv2-targetgroup-healthcheckpath
         */
        HealthCheckPath?: C.YieldsString;
        /**
         * String

         *
         * The port that the load balancer uses when performing health checks on the targets.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-targetgroup.html#cfn-elasticloadbalancingv2-targetgroup-healthcheckport
         */
        HealthCheckPort?: C.YieldsString;
        /**
         * String

         *
         * The protocol that the load balancer uses when performing health checks on the targets,
 such as HTTP or HTTPS.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-targetgroup.html#cfn-elasticloadbalancingv2-targetgroup-healthcheckprotocol
         */
        HealthCheckProtocol?: C.YieldsString;
        /**
         * Integer

         *
         * The number of seconds to wait for a response before considering that a health check
 has failed.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-targetgroup.html#cfn-elasticloadbalancingv2-targetgroup-healthchecktimeoutseconds
         */
        HealthCheckTimeoutSeconds?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The number of consecutive successful health checks that are required before an unhealthy
 target is considered healthy.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-targetgroup.html#cfn-elasticloadbalancingv2-targetgroup-healthythresholdcount
         */
        HealthyThresholdCount?: C.Yields<C.CFInteger>;
        /**
         * Elastic Load Balancing TargetGroup Matcher
         *
         * The HTTP codes that a healthy target uses when responding to a health check.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-targetgroup.html#cfn-elasticloadbalancingv2-targetgroup-matcher
         */
        Matcher?: C.Yields<AWS.ElasticLoadBalancingV2.TargetGroup.Matcher>;
        /**
         * String

         *
         * A name for the target group.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-targetgroup.html#cfn-elasticloadbalancingv2-targetgroup-name
         */
        Name?: C.YieldsString;
        /**
         * Integer

         *
         * The port on which the targets receive traffic.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-targetgroup.html#cfn-elasticloadbalancingv2-targetgroup-port
         */
        Port: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The protocol to use for routing traffic to the targets.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-targetgroup.html#cfn-elasticloadbalancingv2-targetgroup-protocol
         */
        Protocol: C.YieldsString;
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key–value pairs) for the target group. Use tags to help
 manage resources.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-targetgroup.html#cfn-elasticloadbalancingv2-targetgroup-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * List of Elastic Load Balancing TargetGroup TargetGroupAttributes
         *
         * Target group configurations.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-targetgroup.html#cfn-elasticloadbalancingv2-targetgroup-targetgroupattributes
         */
        TargetGroupAttributes?: C.Yields<C.CFList<C.Yields<AWS.ElasticLoadBalancingV2.TargetGroup.TargetGroupAttribute>>>;
        /**
         *                             Type: String

         *
         * The registration type of the targets in this target group. Valid values are instance and ip. The default is instance.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-targetgroup.html#cfn-elasticloadbalancingv2-targetgroup-targettype
         */
        TargetType?: C.YieldsString;
        /**
         * List of Elastic Load Balancing TargetGroup TargetDescription
         *
         * The targets to add to this target group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-targetgroup.html#cfn-elasticloadbalancingv2-targetgroup-targets
         */
        Targets?: C.Yields<C.CFList<C.Yields<AWS.ElasticLoadBalancingV2.TargetGroup.TargetDescription>>>;
        /**
         * Integer

         *
         * The number of consecutive failed health checks that are required before a target is
 considered unhealthy.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-targetgroup.html#cfn-elasticloadbalancingv2-targetgroup-unhealthythresholdcount
         */
        UnhealthyThresholdCount?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The ID of the VPC in which your targets are located.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-targetgroup.html#cfn-elasticloadbalancingv2-targetgroup-vpcid
         */
        VpcId: C.YieldsString;
    }
}
export declare namespace AWS.ElasticLoadBalancingV2.TargetGroup {
    interface Matcher {
        /**
         * String

         *
         * The HTTP codes that a healthy target must use when responding to a health check,
 such as 200,202 or 200-399.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticloadbalancingv2-targetgroup-matcher.html#cfn-elasticloadbalancingv2-targetgroup-matcher-httpcode
         */
        HttpCode: C.YieldsString;
    }
}
export declare namespace AWS.ElasticLoadBalancingV2.TargetGroup {
    interface TargetDescription {
        /**
         * String

         *
         * The ID of the target, such as an EC2 instance ID.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticloadbalancingv2-targetgroup-targetdescription.html#cfn-elasticloadbalancingv2-targetgroup-targetdescription-id
         */
        Id: C.YieldsString;
        /**
         * Integer

         *
         * The port number on which the target is listening for traffic.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticloadbalancingv2-targetgroup-targetdescription.html#cfn-elasticloadbalancingv2-targetgroup-targetdescription-port
         */
        Port?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.ElasticLoadBalancingV2.TargetGroup {
    interface TargetGroupAttribute {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticloadbalancingv2-targetgroup-targetgroupattribute.html#cfn-elasticloadbalancingv2-targetgroup-targetgroupattribute-key
         */
        Key?: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticloadbalancingv2-targetgroup-targetgroupattribute.html#cfn-elasticloadbalancingv2-targetgroup-targetgroupattribute-value
         */
        Value?: C.YieldsString;
    }
}
export declare namespace AWS.Elasticsearch {
    /**
     * The AWS::Elasticsearch::Domain resource creates an Amazon Elasticsearch Service (Amazon ES) domain
    that encapsulates the Amazon ES engine instances. For more information, see CreateElasticsearchDomain in the
    Amazon Elasticsearch Service Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticsearch-domain.html
     */
    interface Domain {
        Type: 'AWS::Elasticsearch::Domain';
        Properties: AWS.Elasticsearch.Domain.Properties;
    }
    /**
     * The AWS::Elasticsearch::Domain resource creates an Amazon Elasticsearch Service (Amazon ES) domain
    that encapsulates the Amazon ES engine instances. For more information, see CreateElasticsearchDomain in the
    Amazon Elasticsearch Service Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticsearch-domain.html
     */
    function Domain(props: C.Omit<Domain, 'Type'>): Domain;
}
export declare namespace AWS.Elasticsearch.Domain {
    interface Properties {
        /**
         * JSON object

         *
         * An AWS Identity and Access Management (IAM) policy document that specifies who can
 access the Amazon ES
 domain and their permissions. For more information, see Configuring Access Policies in the
 Amazon Elasticsearch Service Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticsearch-domain.html#cfn-elasticsearch-domain-accesspolicies
         */
        AccessPolicies?: C.Yields<C.CFJson>;
        /**
         * A JSON object consisting of a string key-value pair, such
 as:

         *
         * Additional options to specify for the Amazon ES domain. For more information, see
 Configuring Advanced Options in the
 Amazon Elasticsearch Service Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticsearch-domain.html#cfn-elasticsearch-domain-advancedoptions
         */
        AdvancedOptions?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         * String

         *
         * A name for the Amazon ES domain. For valid values, see the DomainName data type in the Amazon Elasticsearch Service Developer Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticsearch-domain.html#cfn-elasticsearch-domain-domainname
         */
        DomainName?: C.YieldsString;
        /**
         * Amazon Elasticsearch Service Domain
    EBSOptions
         *
         * The configurations of Amazon Elastic Block Store (Amazon EBS) volumes that are attached
 to data nodes in
 the Amazon ES domain. For more information, see Configuring EBS-based Storage in the
 Amazon Elasticsearch Service Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticsearch-domain.html#cfn-elasticsearch-domain-ebsoptions
         */
        EBSOptions?: C.Yields<AWS.Elasticsearch.Domain.EBSOptions>;
        /**
         * Amazon Elasticsearch Service Domain
    ElasticsearchClusterConfig
         *
         * The cluster configuration for the  Amazon ES domain. You can specify options such
 as
 the instance type and the number of instances. For more information, see Configuring Amazon ES Domains in the
 Amazon Elasticsearch Service Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticsearch-domain.html#cfn-elasticsearch-domain-elasticsearchclusterconfig
         */
        ElasticsearchClusterConfig?: C.Yields<AWS.Elasticsearch.Domain.ElasticsearchClusterConfig>;
        /**
         * String

         *
         * The version of Elasticsearch to use, such as 2.3. For information about the versions that Amazon ES supports, see the Elasticsearch-Version parameter for the CreateElasticsearchDomain action in the Amazon Elasticsearch Service Developer Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticsearch-domain.html#cfn-elasticsearch-domain-elasticsearchversion
         */
        ElasticsearchVersion?: C.YieldsString;
        /**
         * Amazon Elasticsearch Service Domain
    SnapshotOptions
         *
         * The automated snapshot configuration for the Amazon ES domain indices.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticsearch-domain.html#cfn-elasticsearch-domain-snapshotoptions
         */
        SnapshotOptions?: C.Yields<AWS.Elasticsearch.Domain.SnapshotOptions>;
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key–value pairs) to associate with the Amazon ES
 domain.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticsearch-domain.html#cfn-elasticsearch-domain-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
    }
}
export declare namespace AWS.Elasticsearch.Domain {
    interface EBSOptions {
        /**
         * Boolean

         *
         * Specifies whether Amazon EBS volumes are attached to data nodes in the Amazon ES
 domain.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticsearch-domain-ebsoptions.html#cfn-elasticsearch-domain-ebsoptions-ebsenabled
         */
        EBSEnabled?: C.YieldsBoolean;
        /**
         * Integer

         *
         * The number of I/O operations per second (IOPS) that the volume supports. This
 property applies only to the Provisioned IOPS (SSD) EBS volume type.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticsearch-domain-ebsoptions.html#cfn-elasticsearch-domain-ebsoptions-iops
         */
        Iops?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The size of the EBS volume for each data node. The minimum and maximum size of an
 EBS volume depends on the EBS volume type and the instance type to which it is attached.
 For more information, see Configuring EBS-based Storage in the
 Amazon Elasticsearch Service Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticsearch-domain-ebsoptions.html#cfn-elasticsearch-domain-ebsoptions-volumesize
         */
        VolumeSize?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The EBS volume type to use with the Amazon ES domain, such as standard,
 gp2, or io1. For more information about each type, see
 Amazon EBS Volume Types in the Amazon EC2 User Guide for Linux Instances.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticsearch-domain-ebsoptions.html#cfn-elasticsearch-domain-ebsoptions-volumetype
         */
        VolumeType?: C.YieldsString;
    }
}
export declare namespace AWS.Elasticsearch.Domain {
    interface ElasticsearchClusterConfig {
        /**
         * Integer

         *
         * The number of instances to use for the master node.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticsearch-domain-elasticsearchclusterconfig.html#cfn-elasticsearch-domain-elasticseachclusterconfig-dedicatedmastercount
         */
        DedicatedMasterCount?: C.Yields<C.CFInteger>;
        /**
         * Boolean

         *
         * Indicates whether to use a dedicated master node for the Amazon ES domain. A
 dedicated master node is a cluster node that performs cluster management tasks, but
 doesn't hold data or respond to data upload requests. Dedicated master nodes offload
 cluster management tasks to increase the stability of your search clusters.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticsearch-domain-elasticsearchclusterconfig.html#cfn-elasticsearch-domain-elasticseachclusterconfig-dedicatedmasterenabled
         */
        DedicatedMasterEnabled?: C.YieldsBoolean;
        /**
         * String

         *
         * The hardware configuration of the computer that hosts the dedicated master node,
 such as m3.medium.elasticsearch. For valid values, see Configuring Amazon ES Domains in the
 Amazon Elasticsearch Service Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticsearch-domain-elasticsearchclusterconfig.html#cfn-elasticsearch-domain-elasticseachclusterconfig-dedicatedmastertype
         */
        DedicatedMasterType?: C.YieldsString;
        /**
         * Integer

         *
         * The number of data nodes (instances) to use in the Amazon ES domain.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticsearch-domain-elasticsearchclusterconfig.html#cfn-elasticsearch-domain-elasticseachclusterconfig-instancecount
         */
        InstanceCount?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The instance type for your data nodes, such as m3.medium.elasticsearch.
 For valid values, see Configuring Amazon ES Domains in the
 Amazon Elasticsearch Service Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticsearch-domain-elasticsearchclusterconfig.html#cfn-elasticsearch-domain-elasticseachclusterconfig-instnacetype
         */
        InstanceType?: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether to enable zone awareness for the Amazon ES domain. When you enable
 zone awareness, Amazon ES allocates the nodes and replica index shards that belong
 to a
 cluster across two Availability Zones (AZs) in the same region to prevent data loss
 and
 minimize downtime in the event of node or data center failure. Don't enable zone
 awareness if your cluster has no replica index shards or is a single-node cluster.
 For
 more information, see Enabling Zone
    Awareness in the Amazon Elasticsearch Service Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticsearch-domain-elasticsearchclusterconfig.html#cfn-elasticsearch-domain-elasticseachclusterconfig-zoneawarenessenabled
         */
        ZoneAwarenessEnabled?: C.YieldsBoolean;
    }
}
export declare namespace AWS.Elasticsearch.Domain {
    interface SnapshotOptions {
        /**
         * Integer

         *
         * The hour in UTC during which the service takes an automated daily snapshot of the
 indices in the Amazon ES domain. For example, if you specify 0, Amazon ES
 takes an automated snapshot everyday between midnight and 1 am. You can specify a
 value
 between 0 and 23.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-elasticsearch-domain-snapshotoptions.html#cfn-elasticsearch-domain-snapshotoptions-automatedsnapshotstarthour
         */
        AutomatedSnapshotStartHour?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.Events {
    /**
     * The AWS::Events::Rule resource creates a rule that matches incoming Amazon CloudWatch Events (CloudWatch
    Events) events and routes them to one or more targets for processing. For more information,
    see
    Using CloudWatch Events in the Amazon CloudWatch User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-events-rule.html
     */
    interface Rule {
        Type: 'AWS::Events::Rule';
        Properties: AWS.Events.Rule.Properties;
    }
    /**
     * The AWS::Events::Rule resource creates a rule that matches incoming Amazon CloudWatch Events (CloudWatch
    Events) events and routes them to one or more targets for processing. For more information,
    see
    Using CloudWatch Events in the Amazon CloudWatch User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-events-rule.html
     */
    function Rule(props: C.Omit<Rule, 'Type'>): Rule;
}
export declare namespace AWS.Events.Rule {
    interface Properties {
        /**
         * String

         *
         * A description of the rule's purpose.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-events-rule.html#cfn-events-rule-description
         */
        Description?: C.YieldsString;
        /**
         * JSON object

         *
         * Describes which events CloudWatch Events routes to the specified target. These routed
 events are matched events. For more information, see Events and Event Patterns in the Amazon CloudWatch User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-events-rule.html#cfn-events-rule-eventpattern
         */
        EventPattern?: C.Yields<C.CFJson>;
        /**
         * String

         *
         * A name for the rule. If you don't specify a name, AWS CloudFormation generates a unique
 physical ID and uses that ID for the rule name. For more information, see Name Type.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-events-rule.html#cfn-events-rule-name
         */
        Name?: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-events-rule.html#cfn-events-rule-rolearn
         */
        RoleArn?: C.YieldsString;
        /**
         * String

         *
         * The schedule or rate (frequency) that determines when CloudWatch Events runs the rule.
 For more information, see Schedule Expression Syntax for Rules in the Amazon CloudWatch User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-events-rule.html#cfn-events-rule-scheduleexpression
         */
        ScheduleExpression?: C.YieldsString;
        /**
         * String

         *
         * Indicates whether the rule is enabled. For valid values, see the State parameter for the PutRule action in the Amazon CloudWatch Events API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-events-rule.html#cfn-events-rule-state
         */
        State?: C.YieldsString;
        /**
         * List of Amazon CloudWatch Events Rule Target
         *
         * The resources, such as Lambda functions or Kinesis streams, that CloudWatch Events
 routes events to
 and invokes when the rule is triggered. For information about valid targets, see the
 PutTargets action in the
 Amazon CloudWatch Events API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-events-rule.html#cfn-events-rule-targets
         */
        Targets?: C.Yields<C.CFList<C.Yields<AWS.Events.Rule.Target>>>;
    }
}
export declare namespace AWS.Events.Rule {
    interface EcsParameters {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-events-rule-ecsparameters.html#cfn-events-rule-ecsparameters-taskcount
         */
        TaskCount?: C.Yields<C.CFInteger>;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-events-rule-ecsparameters.html#cfn-events-rule-ecsparameters-taskdefinitionarn
         */
        TaskDefinitionArn: C.YieldsString;
    }
}
export declare namespace AWS.Events.Rule {
    interface InputTransformer {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-events-rule-inputtransformer.html#cfn-events-rule-inputtransformer-inputpathsmap
         */
        InputPathsMap?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-events-rule-inputtransformer.html#cfn-events-rule-inputtransformer-inputtemplate
         */
        InputTemplate: C.YieldsString;
    }
}
export declare namespace AWS.Events.Rule {
    interface KinesisParameters {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-events-rule-kinesisparameters.html#cfn-events-rule-kinesisparameters-partitionkeypath
         */
        PartitionKeyPath: C.YieldsString;
    }
}
export declare namespace AWS.Events.Rule {
    interface RunCommandParameters {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-events-rule-runcommandparameters.html#cfn-events-rule-runcommandparameters-runcommandtargets
         */
        RunCommandTargets: C.Yields<C.CFList<C.Yields<AWS.Events.Rule.RunCommandTarget>>>;
    }
}
export declare namespace AWS.Events.Rule {
    interface RunCommandTarget {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-events-rule-runcommandtarget.html#cfn-events-rule-runcommandtarget-key
         */
        Key: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-events-rule-runcommandtarget.html#cfn-events-rule-runcommandtarget-values
         */
        Values: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.Events.Rule {
    interface Target {
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the target.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-events-rule-target.html#cfn-events-rule-target-arn
         */
        Arn: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-events-rule-target.html#cfn-events-rule-target-ecsparameters
         */
        EcsParameters?: C.Yields<AWS.Events.Rule.EcsParameters>;
        /**
         * String

         *
         * A unique, user-defined identifier for the target. Acceptable values include
 alphanumeric characters, periods (.), hypens (-), and
 underscores (_).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-events-rule-target.html#cfn-events-rule-target-id
         */
        Id: C.YieldsString;
        /**
         * String

         *
         * A JSON-formatted text string that is passed to the target. This value overrides the
 matched event.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-events-rule-target.html#cfn-events-rule-target-input
         */
        Input?: C.YieldsString;
        /**
         * String

         *
         * When you don't want to pass the entire matched event, the JSONPath that describes
 which part of the event to pass to the target.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-events-rule-target.html#cfn-events-rule-target-inputpath
         */
        InputPath?: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-events-rule-target.html#cfn-events-rule-target-inputtransformer
         */
        InputTransformer?: C.Yields<AWS.Events.Rule.InputTransformer>;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-events-rule-target.html#cfn-events-rule-target-kinesisparameters
         */
        KinesisParameters?: C.Yields<AWS.Events.Rule.KinesisParameters>;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) role
 that grants CloudWatch Events
 permission to make calls to target services, such as AWS Lambda  or Kinesis
 streams.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-events-rule-target.html#cfn-events-rule-target-rolearn
         */
        RoleArn?: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-events-rule-target.html#cfn-events-rule-target-runcommandparameters
         */
        RunCommandParameters?: C.Yields<AWS.Events.Rule.RunCommandParameters>;
    }
}
export declare namespace AWS.GameLift {
    /**
     * The AWS::GameLift::Alias resource creates an alias for an Amazon GameLift (GameLift)
    fleet, which you can use to anonymize your fleet. You can reference the alias instead
    of a
    specific fleet when you create game sessions. For more information, see the CreateAlias action in the
    Amazon GameLift API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-alias.html
     */
    interface Alias {
        Type: 'AWS::GameLift::Alias';
        Properties: AWS.GameLift.Alias.Properties;
    }
    /**
     * The AWS::GameLift::Alias resource creates an alias for an Amazon GameLift (GameLift)
    fleet, which you can use to anonymize your fleet. You can reference the alias instead
    of a
    specific fleet when you create game sessions. For more information, see the CreateAlias action in the
    Amazon GameLift API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-alias.html
     */
    function Alias(props: C.Omit<Alias, 'Type'>): Alias;
}
export declare namespace AWS.GameLift.Alias {
    interface Properties {
        /**
         * String

         *
         * Information that helps you identify the purpose of this alias.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-alias.html#cfn-gamelift-alias-description
         */
        Description?: C.YieldsString;
        /**
         * String

         *
         * An identifier to associate with this alias. Alias names don't need to be
 unique.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-alias.html#cfn-gamelift-alias-name
         */
        Name: C.YieldsString;
        /**
         * Amazon GameLift Alias
    RoutingStrategy
         *
         * A routing configuration that specifies where traffic is directed for this alias,
 such as to a fleet or to a message.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-alias.html#cfn-gamelift-alias-routingstrategy
         */
        RoutingStrategy: C.Yields<AWS.GameLift.Alias.RoutingStrategy>;
    }
}
export declare namespace AWS.GameLift.Alias {
    interface RoutingStrategy {
        /**
         * String

         *
         * A unique identifier of a GameLift fleet to associate with the alias.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-alias-routingstrategy.html#cfn-gamelift-alias-routingstrategy-fleetid
         */
        FleetId?: C.YieldsString;
        /**
         * String

         *
         * A text message that GameLift displays for the Terminal routing
 type.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-alias-routingstrategy.html#cfn-gamelift-alias-routingstrategy-message
         */
        Message?: C.YieldsString;
        /**
         * String

         *
         * The type of routing strategy. For the SIMPLE type, traffic is routed to
 an active GameLift fleet. For the Terminal type, GameLift returns an exception
 with the message that you specified in the Message property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-alias-routingstrategy.html#cfn-gamelift-alias-routingstrategy-type
         */
        Type: C.YieldsString;
    }
}
export declare namespace AWS.GameLift {
    /**
     * The AWS::GameLift::Build resource creates a build that includes all of the
    components to run your game server in an Amazon GameLift (GameLift) fleet.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-build.html
     */
    interface Build {
        Type: 'AWS::GameLift::Build';
        Properties: AWS.GameLift.Build.Properties;
    }
    /**
     * The AWS::GameLift::Build resource creates a build that includes all of the
    components to run your game server in an Amazon GameLift (GameLift) fleet.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-build.html
     */
    function Build(props: C.Omit<Build, 'Type'>): Build;
}
export declare namespace AWS.GameLift.Build {
    interface Properties {
        /**
         * String

         *
         * An identifier to associate with this build. Build names don't need to be
 unique.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-build.html#cfn-gamelift-build-name
         */
        Name?: C.YieldsString;
        /**
         * Amazon GameLift Build
    StorageLocation
         *
         * The Amazon Simple Storage Service (Amazon S3) location where your build package files
 are located.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-build.html#cfn-gamelift-build-storagelocation
         */
        StorageLocation?: C.Yields<AWS.GameLift.Build.S3Location>;
        /**
         * String

         *
         * A version to associate with this build. Version is useful if you want to track
 updates to your build package files. Versions don't need to be unique.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-build.html#cfn-gamelift-build-version
         */
        Version?: C.YieldsString;
    }
}
export declare namespace AWS.GameLift.Build {
    interface S3Location {
        /**
         * String

         *
         * The S3 bucket where the GameLift build package files are stored.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-build-storagelocation.html#cfn-gamelift-build-storage-bucket
         */
        Bucket: C.YieldsString;
        /**
         * String

         *
         * The prefix (folder name) where the GameLift build package files are located.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-build-storagelocation.html#cfn-gamelift-build-storage-key
         */
        Key: C.YieldsString;
        /**
         * String

         *
         * An AWS Identity and Access Management (IAM) role Amazon Resource Name (ARN) that GameLift
 can assume to
 retrieve the build package files from Amazon Simple Storage Service (Amazon S3).

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-build-storagelocation.html#cfn-gamelift-build-storage-rolearn
         */
        RoleArn: C.YieldsString;
    }
}
export declare namespace AWS.GameLift {
    /**
     * The AWS::GameLift::Fleet resource creates an Amazon GameLift (GameLift) fleet to host
    game servers. A fleet is a set of EC2 instances, each of which is a host in the fleet.
    For more
    information, see the CreateFleet action in
    the Amazon GameLift API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-fleet.html
     */
    interface Fleet {
        Type: 'AWS::GameLift::Fleet';
        Properties: AWS.GameLift.Fleet.Properties;
    }
    /**
     * The AWS::GameLift::Fleet resource creates an Amazon GameLift (GameLift) fleet to host
    game servers. A fleet is a set of EC2 instances, each of which is a host in the fleet.
    For more
    information, see the CreateFleet action in
    the Amazon GameLift API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-fleet.html
     */
    function Fleet(props: C.Omit<Fleet, 'Type'>): Fleet;
}
export declare namespace AWS.GameLift.Fleet {
    interface Properties {
        /**
         * String

         *
         * The unique identifier for the build that you want to use with this fleet.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-fleet.html#cfn-gamelift-fleet-buildid
         */
        BuildId: C.YieldsString;
        /**
         * String

         *
         * Information that helps you identify the purpose of this fleet.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-fleet.html#cfn-gamelift-fleet-description
         */
        Description?: C.YieldsString;
        /**
         * Integer

         *
         * The number of EC2 instances that you want in this fleet.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-fleet.html#cfn-gamelift-fleet-desiredec2instances
         */
        DesiredEC2Instances: C.Yields<C.CFInteger>;
        /**
         * List of Amazon GameLift Fleet
    EC2InboundPermission
         *
         * The incoming traffic, expressed as IP ranges and port numbers, that is permitted to
 access the game server. If you don't specify values, no traffic is permitted to your
 game servers.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-fleet.html#cfn-gamelift-fleet-ec2inboundpermissions
         */
        EC2InboundPermissions?: C.Yields<C.CFList<C.Yields<AWS.GameLift.Fleet.IpPermission>>>;
        /**
         * String

         *
         * The type of EC2 instances that the fleet uses. EC2 instance types define the CPU,
 memory, storage, and networking capacity of the fleet's hosts. For more information
 about the instance types that are supported by GameLift, see the EC2InstanceType parameter in the
 Amazon GameLift API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-fleet.html#cfn-gamelift-fleet-ec2instancetype
         */
        EC2InstanceType: C.YieldsString;
        /**
         * List of strings

         *
         * The path to game-session log files that are generated by your game server, with the
 slashes (\) escaped. After a game session has been terminated, GameLift
 captures and stores the logs in an S3 bucket.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-fleet.html#cfn-gamelift-fleet-logpaths
         */
        LogPaths?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * Integer

         *
         * The maximum number of EC2 instances that you want to allow in this fleet. By default,
 AWS CloudFormation, sets this property to 1.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-fleet.html#cfn-gamelift-fleet-maxsize
         */
        MaxSize?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The minimum number of EC2 instances that you want to allow in this fleet. By default,
 AWS CloudFormation, sets this property to 0.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-fleet.html#cfn-gamelift-fleet-minsize
         */
        MinSize?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * An identifier to associate with this fleet. Fleet names don't need to be
 unique.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-fleet.html#cfn-gamelift-fleet-name
         */
        Name: C.YieldsString;
        /**
         * String

         *
         * The parameters that are required to launch your game server. Specify these
 parameters as a string of command-line parameters, such as +sv_port 33435
    +start_lobby.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-fleet.html#cfn-gamelift-fleet-serverlaunchparameters
         */
        ServerLaunchParameters?: C.YieldsString;
        /**
         * String

         *
         * The location of your game server that GameLift launches. You must escape the slashes
 (\) and use the following pattern:
 C:\\game\\launchpath. For example,
 if your game server files are in the MyGame folder, the path should be
 C:\\game\\MyGame\\server.exe.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-gamelift-fleet.html#cfn-gamelift-fleet-serverlaunchpath
         */
        ServerLaunchPath: C.YieldsString;
    }
}
export declare namespace AWS.GameLift.Fleet {
    interface IpPermission {
        /**
         * Integer

         *
         * The starting value for a range of allowed port numbers. This value must be lower
 than the ToPort value.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-fleet-ec2inboundpermission.html#cfn-gamelift-fleet-ec2inboundpermissions-fromport
         */
        FromPort: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The range of allowed IP addresses in CIDR notation.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-fleet-ec2inboundpermission.html#cfn-gamelift-fleet-ec2inboundpermissions-iprange
         */
        IpRange: C.YieldsString;
        /**
         * String

         *
         * The network communication protocol that is used by the fleet. For valid values, see
 the IpPermission data type in the
 Amazon GameLift API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-fleet-ec2inboundpermission.html#cfn-gamelift-fleet-ec2inboundpermissions-protocol
         */
        Protocol: C.YieldsString;
        /**
         * Integer

         *
         * The ending value for a range of allowed port numbers. This value must be higher than
 the FromPort value.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-gamelift-fleet-ec2inboundpermission.html#cfn-gamelift-fleet-ec2inboundpermissions-toport
         */
        ToPort: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.IAM {
    /**
     * The AWS::IAM::AccessKey resource type generates a secret access key and assigns it
    to an IAM user or AWS
    account.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-accesskey.html
     */
    interface AccessKey {
        Type: 'AWS::IAM::AccessKey';
        Properties: AWS.IAM.AccessKey.Properties;
    }
    /**
     * The AWS::IAM::AccessKey resource type generates a secret access key and assigns it
    to an IAM user or AWS
    account.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-accesskey.html
     */
    function AccessKey(props: C.Omit<AccessKey, 'Type'>): AccessKey;
}
export declare namespace AWS.IAM.AccessKey {
    interface Properties {
        /**
         * Integer

         *
         * This value is specific to AWS CloudFormation and can only be incremented. Incrementing this
 value notifies AWS CloudFormation that you want to rotate your access key. When you
 update your stack, AWS CloudFormation will
 replace the existing access key with a new key.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-accesskey.html#cfn-iam-accesskey-serial
         */
        Serial?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The status of the access key. By default, AWS CloudFormation sets this property value
 to
 Active.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-accesskey.html#cfn-iam-accesskey-status
         */
        Status?: C.YieldsString;
        /**
         * String

         *
         * The name of the user that the new key will belong to.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-accesskey.html#cfn-iam-accesskey-username
         */
        UserName: C.YieldsString;
    }
}
export declare namespace AWS.IAM {
    /**
     * The AWS::IAM::Group resource creates an AWS Identity and Access Management (IAM) group.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-group.html
     */
    interface Group {
        Type: 'AWS::IAM::Group';
        Properties: AWS.IAM.Group.Properties;
    }
    /**
     * The AWS::IAM::Group resource creates an AWS Identity and Access Management (IAM) group.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-group.html
     */
    function Group(props: C.Omit<Group, 'Type'>): Group;
}
export declare namespace AWS.IAM.Group {
    interface Properties {
        /**
         * String

         *
         * A name for the IAM group. For valid values, see the GroupName parameter for the CreateGroup action in the IAM API Reference. If you don't specify a name, AWS CloudFormation generates a unique physical ID and
 uses that ID for the group name.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-group.html#cfn-iam-group-groupname
         */
        GroupName?: C.YieldsString;
        /**
         * List of strings

         *
         * One or more managed policy ARNs to attach to this group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-group.html#cfn-iam-group-managepolicyarns
         */
        ManagedPolicyArns?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The path to the group. For more information about paths, see IAM Identifiers in the IAM User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-group.html#cfn-iam-group-path
         */
        Path?: C.YieldsString;
        /**
         * List of IAM Policies
         *
         * The policies to associate with this group. For information about policies, see Overview of IAM Policies in the IAM User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-group.html#cfn-iam-group-policies
         */
        Policies?: C.Yields<C.CFList<C.Yields<AWS.IAM.Group.Policy>>>;
    }
}
export declare namespace AWS.IAM.Group {
    interface Policy {
        /**
         * JSON object

         *
         * A policy document that describes what actions are allowed on which
 resources.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-policy.html#cfn-iam-policies-policydocument
         */
        PolicyDocument: C.Yields<C.CFJson>;
        /**
         * String

         *
         * The name of the policy.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-policy.html#cfn-iam-policies-policyname
         */
        PolicyName: C.YieldsString;
    }
}
export declare namespace AWS.IAM {
    /**
     * The AWS::IAM::InstanceProfile resource creates an AWS Identity and Access Management (IAM) instance
    profile that can be used with IAM roles for EC2 instances.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-instanceprofile.html
     */
    interface InstanceProfile {
        Type: 'AWS::IAM::InstanceProfile';
        Properties: AWS.IAM.InstanceProfile.Properties;
    }
    /**
     * The AWS::IAM::InstanceProfile resource creates an AWS Identity and Access Management (IAM) instance
    profile that can be used with IAM roles for EC2 instances.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-instanceprofile.html
     */
    function InstanceProfile(props: C.Omit<InstanceProfile, 'Type'>): InstanceProfile;
}
export declare namespace AWS.IAM.InstanceProfile {
    interface Properties {
        /**
         * String

         *
         * The name of the instance profile that you want to create. This parameter allows
 (per its regex pattern) a
 string consisting of upper and lowercase alphanumeric characters with no spaces.
 You can also include any of the following characters: = , . @
    -.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-instanceprofile.html#cfn-iam-instanceprofile-instanceprofilename
         */
        InstanceProfileName?: C.YieldsString;
        /**
         * String

         *
         * The path associated with this IAM instance profile. For information about IAM paths,
 see Friendly Names and Paths in the AWS Identity and Access Management User
    Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-instanceprofile.html#cfn-iam-instanceprofile-path
         */
        Path?: C.YieldsString;
        /**
         * List of strings

         *
         * The name of an existing IAM role to associate with this instance profile.
 Currently, you can assign a maximum of one role to an instance profile.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-instanceprofile.html#cfn-iam-instanceprofile-roles
         */
        Roles: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.IAM {
    /**
     * AWS::IAM::ManagedPolicy creates an AWS Identity and Access Management (IAM) managed policy for your
    AWS account, which you can use to apply permissions to IAM users, groups, and roles.
    For more
    information about managed policies, see Managed Policies and Inline
       Policies in the IAM User Guide guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-managedpolicy.html
     */
    interface ManagedPolicy {
        Type: 'AWS::IAM::ManagedPolicy';
        Properties: AWS.IAM.ManagedPolicy.Properties;
    }
    /**
     * AWS::IAM::ManagedPolicy creates an AWS Identity and Access Management (IAM) managed policy for your
    AWS account, which you can use to apply permissions to IAM users, groups, and roles.
    For more
    information about managed policies, see Managed Policies and Inline
       Policies in the IAM User Guide guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-managedpolicy.html
     */
    function ManagedPolicy(props: C.Omit<ManagedPolicy, 'Type'>): ManagedPolicy;
}
export declare namespace AWS.IAM.ManagedPolicy {
    interface Properties {
        /**
         * String

         *
         * A description of the IAM policy. For example, describe the permissions that
 are defined in the policy.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-managedpolicy.html#cfn-iam-managedpolicy-description
         */
        Description?: C.YieldsString;
        /**
         * List of strings

         *
         * The names of IAM groups to attach to this policy.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-managedpolicy.html#cfn-iam-managedpolicy-groups
         */
        Groups?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * A custom, friendly name for your IAM managed policy. For valid values, see the PolicyName parameter of the CreatePolicy action in the IAM API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-managedpolicy.html#cfn-iam-managedpolicy-managedpolicyname
         */
        ManagedPolicyName?: C.YieldsString;
        /**
         * String

         *
         * The path for the IAM policy. By default, the path is /. For more
 information, see IAM
    Identifiers in the IAM User Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-managedpolicy.html#cfn-ec2-dhcpoptions-path
         */
        Path?: C.YieldsString;
        /**
         * JSON object

         *
         * Policies that define the permissions for this managed policy. For more
 information about policy syntax, see IAM Policy Elements Reference in
 IAM User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-managedpolicy.html#cfn-iam-managedpolicy-policydocument
         */
        PolicyDocument: C.Yields<C.CFJson>;
        /**
         * List of strings

         *
         * The names of IAM roles to attach to this policy.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-managedpolicy.html#cfn-iam-managedpolicy-roles
         */
        Roles?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of strings

         *
         * The names of users to attach to this policy.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-managedpolicy.html#cfn-iam-managedpolicy-users
         */
        Users?: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.IAM {
    /**
     * The AWS::IAM::Policy resource associates an IAM policy with IAM users,
    roles, or groups. For more information about IAM policies, see Overview of IAM Policies in the
    IAM User Guide guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-policy.html
     */
    interface Policy {
        Type: 'AWS::IAM::Policy';
        Properties: AWS.IAM.Policy.Properties;
    }
    /**
     * The AWS::IAM::Policy resource associates an IAM policy with IAM users,
    roles, or groups. For more information about IAM policies, see Overview of IAM Policies in the
    IAM User Guide guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-policy.html
     */
    function Policy(props: C.Omit<Policy, 'Type'>): Policy;
}
export declare namespace AWS.IAM.Policy {
    interface Properties {
        /**
         * List of strings

         *
         * The names of groups to which you want to add the policy.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-policy.html#cfn-iam-policy-groups
         */
        Groups?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * JSON object

         *
         * A policy document that contains permissions to add to the specified users or
 groups.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-policy.html#cfn-iam-policy-policydocument
         */
        PolicyDocument: C.Yields<C.CFJson>;
        /**
         * String

         *
         * The name of the policy. If you specify multiple policies for an entity, specify
 unique names. For example, if you specify a list of policies for an IAM role,
 each policy must have a unique name.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-policy.html#cfn-iam-policy-policyname
         */
        PolicyName: C.YieldsString;
        /**
         * List of strings

         *
         * The names of AWS::IAM::Roles to
 which this policy will be attached.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-policy.html#cfn-iam-policy-roles
         */
        Roles?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of strings

         *
         * The names of users for whom you want to add the policy.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-policy.html#cfn-iam-policy-users
         */
        Users?: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.IAM {
    /**
     * Creates an AWS Identity and Access Management (IAM) role. Use an IAM role to enable
    applications running on an EC2 instance to securely access your AWS resources.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-role.html
     */
    interface Role {
        Type: 'AWS::IAM::Role';
        Properties: AWS.IAM.Role.Properties;
    }
    /**
     * Creates an AWS Identity and Access Management (IAM) role. Use an IAM role to enable
    applications running on an EC2 instance to securely access your AWS resources.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-role.html
     */
    function Role(props: C.Omit<Role, 'Type'>): Role;
}
export declare namespace AWS.IAM.Role {
    interface Properties {
        /**
         * A JSON policy document

         *
         * The trust policy that is associated with this role. You can associate only one assume
 role policy with a role. For an example of
 an assume role policy, see Template Examples. For more information about the elements that you can use in an IAM policy, see IAM Policy Elements Reference in the IAM User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-role.html#cfn-iam-role-assumerolepolicydocument
         */
        AssumeRolePolicyDocument: C.Yields<C.CFJson>;
        /**
         * List of strings

         *
         * One or more managed policy ARNs to attach to this role.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-role.html#cfn-iam-role-managepolicyarns
         */
        ManagedPolicyArns?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The path associated with this role. For information about IAM paths, see
 Friendly Names and Paths in
 IAM User Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-role.html#cfn-iam-role-path
         */
        Path?: C.YieldsString;
        /**
         * List of IAM Policies
         *
         * The policies to associate with this role. For sample templates, see Template Examples.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-role.html#cfn-iam-role-policies
         */
        Policies?: C.Yields<C.CFList<C.Yields<AWS.IAM.Role.Policy>>>;
        /**
         * String

         *
         * A name for the IAM role. For valid values, see the RoleName parameter for the CreateRole action in the IAM API Reference. If you don't specify a name, AWS CloudFormation generates a unique physical ID and
 uses that ID for the group name.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-role.html#cfn-iam-role-rolename
         */
        RoleName?: C.YieldsString;
    }
}
export declare namespace AWS.IAM.Role {
    interface Policy {
        /**
         * JSON object

         *
         * A policy document that describes what actions are allowed on which
 resources.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-policy.html#cfn-iam-policies-policydocument
         */
        PolicyDocument: C.Yields<C.CFJson>;
        /**
         * String

         *
         * The name of the policy.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-policy.html#cfn-iam-policies-policyname
         */
        PolicyName: C.YieldsString;
    }
}
export declare namespace AWS.IAM {
    /**
     * The AWS::IAM::User type creates a user.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-user.html
     */
    interface User {
        Type: 'AWS::IAM::User';
        Properties: AWS.IAM.User.Properties;
    }
    /**
     * The AWS::IAM::User type creates a user.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-user.html
     */
    function User(props: C.Omit<User, 'Type'>): User;
}
export declare namespace AWS.IAM.User {
    interface Properties {
        /**
         * List of strings

         *
         * A name of a group to which you want to add the user.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-user.html#cfn-iam-user-groups
         */
        Groups?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * IAM User LoginProfile
         *
         * Creates a login profile so that the user can access the AWS Management Console.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-user.html#cfn-iam-user-loginprofile
         */
        LoginProfile?: C.Yields<AWS.IAM.User.LoginProfile>;
        /**
         * List of strings

         *
         * One or more managed policy ARNs to attach to this user.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-user.html#cfn-iam-user-managepolicyarns
         */
        ManagedPolicyArns?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The path for the user name. For more information about paths, see IAM Identifiers in the IAM User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-user.html#cfn-iam-user-path
         */
        Path?: C.YieldsString;
        /**
         * List of IAM Policies
         *
         * The policies to associate with this user. For information about policies, see Overview of IAM Policies in the IAM User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-user.html#cfn-iam-user-policies
         */
        Policies?: C.Yields<C.CFList<C.Yields<AWS.IAM.User.Policy>>>;
        /**
         * String

         *
         * A name for the IAM user. For valid values, see the UserName parameter for the CreateUser action in the IAM API Reference. If you don't specify a name, AWS CloudFormation generates a unique physical ID and
 uses that ID for the user name.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-user.html#cfn-iam-user-username
         */
        UserName?: C.YieldsString;
    }
}
export declare namespace AWS.IAM.User {
    interface LoginProfile {
        /**
         * String

         *
         * The password for the user.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-user-loginprofile.html#cfn-iam-user-loginprofile-password
         */
        Password: C.YieldsString;
        /**
         * Boolean

         *
         * Specifies whether the user is required to set a new password the next time the
 user logs in to the AWS Management Console.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-user-loginprofile.html#cfn-iam-user-loginprofile-passwordresetrequired
         */
        PasswordResetRequired?: C.YieldsBoolean;
    }
}
export declare namespace AWS.IAM.User {
    interface Policy {
        /**
         * JSON object

         *
         * A policy document that describes what actions are allowed on which
 resources.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-policy.html#cfn-iam-policies-policydocument
         */
        PolicyDocument: C.Yields<C.CFJson>;
        /**
         * String

         *
         * The name of the policy.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-policy.html#cfn-iam-policies-policyname
         */
        PolicyName: C.YieldsString;
    }
}
export declare namespace AWS.IAM {
    /**
     * The AWS::IAM::UserToGroupAddition type adds AWS Identity and Access Management (IAM) users to a
    group.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-addusertogroup.html
     */
    interface UserToGroupAddition {
        Type: 'AWS::IAM::UserToGroupAddition';
        Properties: AWS.IAM.UserToGroupAddition.Properties;
    }
    /**
     * The AWS::IAM::UserToGroupAddition type adds AWS Identity and Access Management (IAM) users to a
    group.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-addusertogroup.html
     */
    function UserToGroupAddition(props: C.Omit<UserToGroupAddition, 'Type'>): UserToGroupAddition;
}
export declare namespace AWS.IAM.UserToGroupAddition {
    interface Properties {
        /**
         * String

         *
         * The name of group to add users to.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-addusertogroup.html#cfn-iam-addusertogroup-groupname
         */
        GroupName: C.YieldsString;
        /**
         * List of users

         *
         * Required: Yes

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iam-addusertogroup.html#cfn-iam-addusertogroup-users
         */
        Users: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.IoT {
    /**
     * Use the AWS::IoT::Certificate resource to declare an X.509 certificate.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-certificate.html
     */
    interface Certificate {
        Type: 'AWS::IoT::Certificate';
        Properties: AWS.IoT.Certificate.Properties;
    }
    /**
     * Use the AWS::IoT::Certificate resource to declare an X.509 certificate.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-certificate.html
     */
    function Certificate(props: C.Omit<Certificate, 'Type'>): Certificate;
}
export declare namespace AWS.IoT.Certificate {
    interface Properties {
        /**
         * String

         *
         * The certificate signing request (CSR).
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-certificate.html#cfn-iot-certificate-certificatesigningrequest
         */
        CertificateSigningRequest: C.YieldsString;
        /**
         * String

         *
         * The status of the certificate.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-certificate.html#cfn-iot-certificate-status
         */
        Status: C.YieldsString;
    }
}
export declare namespace AWS.IoT {
    /**
     * Use the AWS::IoT::Policy resource to declare an AWS IoT policy.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-policy.html
     */
    interface Policy {
        Type: 'AWS::IoT::Policy';
        Properties: AWS.IoT.Policy.Properties;
    }
    /**
     * Use the AWS::IoT::Policy resource to declare an AWS IoT policy.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-policy.html
     */
    function Policy(props: C.Omit<Policy, 'Type'>): Policy;
}
export declare namespace AWS.IoT.Policy {
    interface Properties {
        /**
         * JSON object

         *
         * The JSON document that describes the policy.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-policy.html#cfn-iot-policy-policydocument
         */
        PolicyDocument: C.Yields<C.CFJson>;
        /**
         * String

         *
         * The name (the physical ID) of the AWS IoT policy.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-policy.html#cfn-iot-policy-policyname
         */
        PolicyName?: C.YieldsString;
    }
}
export declare namespace AWS.IoT {
    /**
     * Use the AWS::IoT::PolicyPrincipalAttachment resource to attach an AWS IoT policy
    to a principal (an X.509 certificate or other credential).
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-policyprincipalattachment.html
     */
    interface PolicyPrincipalAttachment {
        Type: 'AWS::IoT::PolicyPrincipalAttachment';
        Properties: AWS.IoT.PolicyPrincipalAttachment.Properties;
    }
    /**
     * Use the AWS::IoT::PolicyPrincipalAttachment resource to attach an AWS IoT policy
    to a principal (an X.509 certificate or other credential).
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-policyprincipalattachment.html
     */
    function PolicyPrincipalAttachment(props: C.Omit<PolicyPrincipalAttachment, 'Type'>): PolicyPrincipalAttachment;
}
export declare namespace AWS.IoT.PolicyPrincipalAttachment {
    interface Properties {
        /**
         * String

         *
         * The name of the policy.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-policyprincipalattachment.html#cfn-iot-policyprincipalattachment-policyname
         */
        PolicyName: C.YieldsString;
        /**
         * String

         *
         * The principal, which can be a certificate ARN (as returned from the
 CreateCertificate operation) or an Amazon Cognito ID.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-policyprincipalattachment.html#cfn-iot-policyprincipalattachment-principal
         */
        Principal: C.YieldsString;
    }
}
export declare namespace AWS.IoT {
    /**
     * Use the AWS::IoT::Thing resource to declare an AWS IoT thing.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-thing.html
     */
    interface Thing {
        Type: 'AWS::IoT::Thing';
        Properties: AWS.IoT.Thing.Properties;
    }
    /**
     * Use the AWS::IoT::Thing resource to declare an AWS IoT thing.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-thing.html
     */
    function Thing(props: C.Omit<Thing, 'Type'>): Thing;
}
export declare namespace AWS.IoT.Thing {
    interface Properties {
        /**
         * AWS IoT Thing AttributePayload
         *
         * The attribute payload.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-thing.html#cfn-iot-thing-attributepayload
         */
        AttributePayload?: C.Yields<AWS.IoT.Thing.AttributePayload>;
        /**
         * String

         *
         * The name (the physical ID) of the AWS IoT thing.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-thing.html#cfn-iot-thing-thingname
         */
        ThingName?: C.YieldsString;
    }
}
export declare namespace AWS.IoT.Thing {
    interface AttributePayload {
        /**
         * String to string map

         *
         * A string that contains up to three key–value pairs. Maximum length of 800. Duplicates
 not allowed.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-thing-attributepayload.html#cfn-iot-thing-attributepayload-attributes
         */
        Attributes?: C.Yields<C.CFMap<C.YieldsString>>;
    }
}
export declare namespace AWS.IoT {
    /**
     * Use the AWS::IoT::ThingPrincipalAttachment resource to attach a principal (an
    X.509 certificate or another credential) to a thing.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-thingprincipalattachment.html
     */
    interface ThingPrincipalAttachment {
        Type: 'AWS::IoT::ThingPrincipalAttachment';
        Properties: AWS.IoT.ThingPrincipalAttachment.Properties;
    }
    /**
     * Use the AWS::IoT::ThingPrincipalAttachment resource to attach a principal (an
    X.509 certificate or another credential) to a thing.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-thingprincipalattachment.html
     */
    function ThingPrincipalAttachment(props: C.Omit<ThingPrincipalAttachment, 'Type'>): ThingPrincipalAttachment;
}
export declare namespace AWS.IoT.ThingPrincipalAttachment {
    interface Properties {
        /**
         * String

         *
         * The principal, which can be a certificate ARN (as returned from the
 CreateCertificate operation) or an Amazon Cognito ID.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-thingprincipalattachment.html#cfn-iot-thingprincipalattachment-principal
         */
        Principal: C.YieldsString;
        /**
         * String

         *
         * The name of the AWS IoT thing.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-thingprincipalattachment.html#cfn-iot-thingprincipalattachment-thingname
         */
        ThingName: C.YieldsString;
    }
}
export declare namespace AWS.IoT {
    /**
     * Use the AWS::IoT::TopicRule resource to declare an AWS IoT rule.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-topicrule.html
     */
    interface TopicRule {
        Type: 'AWS::IoT::TopicRule';
        Properties: AWS.IoT.TopicRule.Properties;
    }
    /**
     * Use the AWS::IoT::TopicRule resource to declare an AWS IoT rule.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-topicrule.html
     */
    function TopicRule(props: C.Omit<TopicRule, 'Type'>): TopicRule;
}
export declare namespace AWS.IoT.TopicRule {
    interface Properties {
        /**
         * String

         *
         * The name (the physical ID) of the AWS IoT rule.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-topicrule.html#cfn-iot-topicrule-rulename
         */
        RuleName?: C.YieldsString;
        /**
         * TopicRulePayload object

         *
         * The actions associated with the AWS IoT rule.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iot-topicrule.html#cfn-iot-topicrule-topicrulepayload
         */
        TopicRulePayload: C.Yields<AWS.IoT.TopicRule.TopicRulePayload>;
    }
}
export declare namespace AWS.IoT.TopicRule {
    interface Action {
        /**
         * AWS IoT TopicRule CloudwatchAlarmAction
         *
         * Changes the state of a CloudWatch alarm.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-action.html#cfn-iot-topicrule-action-cloudwatchalarm
         */
        CloudwatchAlarm?: C.Yields<AWS.IoT.TopicRule.CloudwatchAlarmAction>;
        /**
         * AWS IoT TopicRule CloudwatchMetricAction
         *
         * Captures a CloudWatch metric.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-action.html#cfn-iot-topicrule-action-cloudwatchmetric
         */
        CloudwatchMetric?: C.Yields<AWS.IoT.TopicRule.CloudwatchMetricAction>;
        /**
         * AWS IoT TopicRule DynamoDBAction
         *
         * Writes data to a DynamoDB table.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-action.html#cfn-iot-topicrule-action-dynamodb
         */
        DynamoDB?: C.Yields<AWS.IoT.TopicRule.DynamoDBAction>;
        /**
         * AWS IoT TopicRule DynamoDBv2Action
         *
         * Writes data to a DynamoDB table.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-action.html#cfn-iot-topicrule-action-dynamodbv2
         */
        DynamoDBv2?: C.Yields<AWS.IoT.TopicRule.DynamoDBv2Action>;
        /**
         * AWS IoT TopicRule ElasticsearchAction
         *
         * Writes data to an Elasticsearch domain.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-action.html#cfn-iot-topicrule-action-elasticsearch
         */
        Elasticsearch?: C.Yields<AWS.IoT.TopicRule.ElasticsearchAction>;
        /**
         * AWS IoT TopicRule FirehoseAction
         *
         * Writes data to a Kinesis Firehose stream.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-action.html#cfn-iot-topicrule-action-firehose
         */
        Firehose?: C.Yields<AWS.IoT.TopicRule.FirehoseAction>;
        /**
         * AWS IoT TopicRule KinesisAction
         *
         * Writes data to an Kinesis stream.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-action.html#cfn-iot-topicrule-action-kinesis
         */
        Kinesis?: C.Yields<AWS.IoT.TopicRule.KinesisAction>;
        /**
         * AWS IoT TopicRule LambdaAction
         *
         * Invokes a Lambda function.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-action.html#cfn-iot-topicrule-action-lambda
         */
        Lambda?: C.Yields<AWS.IoT.TopicRule.LambdaAction>;
        /**
         * AWS IoT TopicRule RepublishAction
         *
         * Publishes data to an MQ Telemetry Transport (MQTT) topic different from the one
 currently specified.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-action.html#cfn-iot-topicrule-action-republish
         */
        Republish?: C.Yields<AWS.IoT.TopicRule.RepublishAction>;
        /**
         * AWS IoT TopicRule S3Action
         *
         * Writes data to an S3 bucket.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-action.html#cfn-iot-topicrule-action-s3
         */
        S3?: C.Yields<AWS.IoT.TopicRule.S3Action>;
        /**
         * AWS IoT TopicRule SnsAction
         *
         * Publishes data to an SNS topic.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-action.html#cfn-iot-topicrule-action-sns
         */
        Sns?: C.Yields<AWS.IoT.TopicRule.SnsAction>;
        /**
         * AWS IoT TopicRule SqsAction
         *
         * Publishes data to an SQS queue.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-action.html#cfn-iot-topicrule-action-sqs
         */
        Sqs?: C.Yields<AWS.IoT.TopicRule.SqsAction>;
    }
}
export declare namespace AWS.IoT.TopicRule {
    interface CloudwatchAlarmAction {
        /**
         * String

         *
         * The CloudWatch alarm name.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-cloudwatchalarmaction.html#cfn-iot-topicrule-cloudwatchalarmaction-alarmname
         */
        AlarmName: C.YieldsString;
        /**
         * String

         *
         * The IAM role that allows access to the CloudWatch alarm.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-cloudwatchalarmaction.html#cfn-iot-topicrule-cloudwatchalarmaction-rolearn
         */
        RoleArn: C.YieldsString;
        /**
         * String

         *
         * The reason for the change of the alarm state.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-cloudwatchalarmaction.html#cfn-iot-topicrule-cloudwatchalarmaction-statereason
         */
        StateReason: C.YieldsString;
        /**
         * String

         *
         * The value of the alarm state.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-cloudwatchalarmaction.html#cfn-iot-topicrule-cloudwatchalarmaction-statevalue
         */
        StateValue: C.YieldsString;
    }
}
export declare namespace AWS.IoT.TopicRule {
    interface CloudwatchMetricAction {
        /**
         * String

         *
         * The name of the CloudWatch metric.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-cloudwatchmetricaction.html#cfn-iot-topicrule-cloudwatchmetricaction-metricname
         */
        MetricName: C.YieldsString;
        /**
         * String

         *
         * The name of the CloudWatch metric namespace.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-cloudwatchmetricaction.html#cfn-iot-topicrule-cloudwatchmetricaction-metricnamespace
         */
        MetricNamespace: C.YieldsString;
        /**
         * String

         *
         * An optional Unix timestamp.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-cloudwatchmetricaction.html#cfn-iot-topicrule-cloudwatchmetricaction-metrictimestamp
         */
        MetricTimestamp?: C.YieldsString;
        /**
         * String

         *
         * The metric unit
 supported by Amazon CloudWatch.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-cloudwatchmetricaction.html#cfn-iot-topicrule-cloudwatchmetricaction-metricunit
         */
        MetricUnit: C.YieldsString;
        /**
         * String

         *
         * The value to publish to the metric. For example, if you count the occurrences
 of a particular term such as Error, the value will be 1
 for each occurrence.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-cloudwatchmetricaction.html#cfn-iot-topicrule-cloudwatchmetricaction-metricvalue
         */
        MetricValue: C.YieldsString;
        /**
         * String

         *
         * The ARN of the IAM role that grants access to the CloudWatch metric.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-cloudwatchmetricaction.html#cfn-iot-topicrule-cloudwatchmetricaction-rolearn
         */
        RoleArn: C.YieldsString;
    }
}
export declare namespace AWS.IoT.TopicRule {
    interface DynamoDBAction {
        /**
         * String

         *
         * The name of the hash key.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-dynamodbaction.html#cfn-iot-topicrule-dynamodbaction-hashkeyfield
         */
        HashKeyField: C.YieldsString;
        /**
         * String

         *
         * The data type of the hash key (also called the partition key). Valid values are: "STRING" or "NUMBER".

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-dynamodbaction.html#cfn-iot-topicrule-dynamodbaction-hashkeytype
         */
        HashKeyType?: C.YieldsString;
        /**
         * String

         *
         * The value of the hash key.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-dynamodbaction.html#cfn-iot-topicrule-dynamodbaction-hashkeyvalue
         */
        HashKeyValue: C.YieldsString;
        /**
         * String

         *
         * The name of the column in the DynamoDB table that contains the result of the query.
 You
 can customize this name.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-dynamodbaction.html#cfn-iot-topicrule-dynamodbaction-payloadfield
         */
        PayloadField?: C.YieldsString;
        /**
         * String

         *
         * The name of the range key.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-dynamodbaction.html#cfn-iot-topicrule-dynamodbaction-rangekeyfield
         */
        RangeKeyField: C.YieldsString;
        /**
         * String

         *
         * The data type of the range key (also called the sort key). Valid values are: "STRING" or "NUMBER".

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-dynamodbaction.html#cfn-iot-topicrule-dynamodbaction-rangekeytype
         */
        RangeKeyType?: C.YieldsString;
        /**
         * String

         *
         * The value of the range key.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-dynamodbaction.html#cfn-iot-topicrule-dynamodbaction-rangekeyvalue
         */
        RangeKeyValue: C.YieldsString;
        /**
         * String

         *
         * The ARN of the IAM role that grants access to the DynamoDB table.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-dynamodbaction.html#cfn-iot-topicrule-dynamodbaction-rolearn
         */
        RoleArn: C.YieldsString;
        /**
         * String

         *
         * The name of the DynamoDB table.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-dynamodbaction.html#cfn-iot-topicrule-dynamodbaction-tablename
         */
        TableName: C.YieldsString;
    }
}
export declare namespace AWS.IoT.TopicRule {
    interface DynamoDBv2Action {
        /**
         *                             Type: AWS IoT TopicRule PutItemInput
         *
         * Specifies the database table to which to write the item for an AWS IoT topic rule.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-dynamodbv2action.html#cfn-iot-topicrule-dynamodbv2action-putitem
         */
        PutItem?: C.Yields<AWS.IoT.TopicRule.PutItemInput>;
        /**
         *                             Type: String

         *
         * The IAM role that allows access to the DynamoDB table. At a minimum, the role must
 allow the dynamoDB:PutItem IAM action.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-dynamodbv2action.html#cfn-iot-topicrule-dynamodbv2action-rolearn
         */
        RoleArn?: C.YieldsString;
    }
}
export declare namespace AWS.IoT.TopicRule {
    interface ElasticsearchAction {
        /**
         * String

         *
         * The endpoint of your Elasticsearch domain.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-elasticsearchaction.html#cfn-iot-topicrule-elasticsearchaction-endpoint
         */
        Endpoint: C.YieldsString;
        /**
         * String

         *
         * A unique identifier for the stored data.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-elasticsearchaction.html#cfn-iot-topicrule-elasticsearchaction-id
         */
        Id: C.YieldsString;
        /**
         * String

         *
         * The Elasticsearch index where the data is stored.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-elasticsearchaction.html#cfn-iot-topicrule-elasticsearchaction-index
         */
        Index: C.YieldsString;
        /**
         * String

         *
         * The ARN of the IAM role that grants access to Elasticsearch.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-elasticsearchaction.html#cfn-iot-topicrule-elasticsearchaction-rolearn
         */
        RoleArn: C.YieldsString;
        /**
         * String

         *
         * The type of stored data.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-elasticsearchaction.html#cfn-iot-topicrule-elasticsearchaction-type
         */
        Type: C.YieldsString;
    }
}
export declare namespace AWS.IoT.TopicRule {
    interface FirehoseAction {
        /**
         * String

         *
         * The delivery stream name.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-firehoseaction.html#cfn-iot-topicrule-firehoseaction-deliverystreamname
         */
        DeliveryStreamName: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the IAM role that grants access to the
 Kinesis Firehose stream.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-firehoseaction.html#cfn-iot-topicrule-firehoseaction-rolearn
         */
        RoleArn: C.YieldsString;
        /**
         * String

         *
         * A character separator that's used to separate records written to the Kinesis Firehose
 stream. For valid values, see Firehose Action in the AWS IoT Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-firehoseaction.html#cfn-iot-topicrule-firehoseaction-separator
         */
        Separator?: C.YieldsString;
    }
}
export declare namespace AWS.IoT.TopicRule {
    interface KinesisAction {
        /**
         * String

         *
         * The partition key (the grouping of data by shard within an Kinesis
 stream).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-kinesisaction.html#cfn-iot-topicrule-kinesisaction-partitionkey
         */
        PartitionKey?: C.YieldsString;
        /**
         * String

         *
         * The ARN of the IAM role that grants access to an Kinesis stream.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-kinesisaction.html#cfn-iot-topicrule-kinesisaction-rolearn
         */
        RoleArn: C.YieldsString;
        /**
         * String

         *
         * The name of the Kinesis stream.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-kinesisaction.html#cfn-iot-topicrule-kinesisaction-streamname
         */
        StreamName: C.YieldsString;
    }
}
export declare namespace AWS.IoT.TopicRule {
    interface LambdaAction {
        /**
         * String

         *
         * The ARN of the Lambda function.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-lambdaaction.html#cfn-iot-topicrule-lambdaaction-functionarn
         */
        FunctionArn?: C.YieldsString;
    }
}
export declare namespace AWS.IoT.TopicRule {
    interface PutItemInput {
        /**
         *                             Type: String

         *
         * The name of the DynamoDB table.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-putiteminput.html#cfn-iot-topicrule-putiteminput-tablename
         */
        TableName: C.YieldsString;
    }
}
export declare namespace AWS.IoT.TopicRule {
    interface RepublishAction {
        /**
         * String

         *
         * The ARN of the IAM role that grants publishing access.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-republishaction.html#cfn-iot-topicrule-republishaction-rolearn
         */
        RoleArn: C.YieldsString;
        /**
         * String

         *
         * The name of the MQTT topic topic different from the one currently
 specified.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-republishaction.html#cfn-iot-topicrule-republishaction-topic
         */
        Topic: C.YieldsString;
    }
}
export declare namespace AWS.IoT.TopicRule {
    interface S3Action {
        /**
         * String

         *
         * The name of the S3 bucket.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-s3action.html#cfn-iot-topicrule-s3action-bucketname
         */
        BucketName: C.YieldsString;
        /**
         * String

         *
         * The object key (the name of an object in the S3 bucket).
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-s3action.html#cfn-iot-topicrule-s3action-key
         */
        Key: C.YieldsString;
        /**
         * String

         *
         * The ARN of the IAM role that grants access to Amazon S3.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-s3action.html#cfn-iot-topicrule-s3action-rolearn
         */
        RoleArn: C.YieldsString;
    }
}
export declare namespace AWS.IoT.TopicRule {
    interface SnsAction {
        /**
         * String

         *
         * The format of the published message.  Amazon SNS uses this setting to determine
 whether it should parse the payload and extract the platform-specific bits from
 the payload.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-snsaction.html#cfn-iot-topicrule-snsaction-messageformat
         */
        MessageFormat?: C.YieldsString;
        /**
         * String

         *
         * The ARN of the IAM role that grants access to Amazon SNS.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-snsaction.html#cfn-iot-topicrule-snsaction-rolearn
         */
        RoleArn: C.YieldsString;
        /**
         * String

         *
         * The ARN of the Amazon SNS topic.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-snsaction.html#cfn-iot-topicrule-snsaction-targetarn
         */
        TargetArn: C.YieldsString;
    }
}
export declare namespace AWS.IoT.TopicRule {
    interface SqsAction {
        /**
         * String

         *
         * The URL of the Amazon Simple Queue Service (Amazon SQS) queue.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-sqsaction.html#cfn-iot-topicrule-sqsaction-queueurl
         */
        QueueUrl: C.YieldsString;
        /**
         * String

         *
         * The ARN of the IAM role that grants access to Amazon SQS.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-sqsaction.html#cfn-iot-topicrule-sqsaction-rolearn
         */
        RoleArn: C.YieldsString;
        /**
         * Boolean

         *
         * Specifies whether Base64 encoding should be used.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-sqsaction.html#cfn-iot-topicrule-sqsaction-usebase64
         */
        UseBase64?: C.YieldsBoolean;
    }
}
export declare namespace AWS.IoT.TopicRule {
    interface TopicRulePayload {
        /**
         * Array of Action objects

         *
         * The actions associated with the rule.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-topicrulepayload.html#cfn-iot-topicrule-topicrulepayload-actions
         */
        Actions: C.Yields<C.CFList<C.Yields<AWS.IoT.TopicRule.Action>>>;
        /**
         * String

         *
         * The version of the SQL rules engine to use when evaluating the rule.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-topicrulepayload.html#cfn-iot-topicrule-topicrulepayload-awsiotsqlversion
         */
        AwsIotSqlVersion?: C.YieldsString;
        /**
         * String

         *
         * The description of the rule.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-topicrulepayload.html#cfn-iot-topicrule-topicrulepayload-description
         */
        Description?: C.YieldsString;
        /**
         * Boolean

         *
         * Specifies whether the rule is disabled.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-topicrulepayload.html#cfn-iot-topicrule-topicrulepayload-ruledisabled
         */
        RuleDisabled: C.YieldsBoolean;
        /**
         * String

         *
         * The SQL statement that queries the topic. For more information, see Rules for AWS IoT
 in the AWS IoT Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iot-topicrule-topicrulepayload.html#cfn-iot-topicrule-topicrulepayload-sql
         */
        Sql: C.YieldsString;
    }
}
export declare namespace AWS.KMS {
    /**
     * The AWS::KMS::Alias resource creates a display name for a customer master key (CMK) in AWS Key Management
    Service (AWS KMS). Using an alias to refer to a key can help you simplify key management.
    For example, when rotating keys, you can just update the alias mapping instead of
    tracking and changing key IDs. For more information, see Working with Aliases in the AWS Key Management Service Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kms-alias.html
     */
    interface Alias {
        Type: 'AWS::KMS::Alias';
        Properties: AWS.KMS.Alias.Properties;
    }
    /**
     * The AWS::KMS::Alias resource creates a display name for a customer master key (CMK) in AWS Key Management
    Service (AWS KMS). Using an alias to refer to a key can help you simplify key management.
    For example, when rotating keys, you can just update the alias mapping instead of
    tracking and changing key IDs. For more information, see Working with Aliases in the AWS Key Management Service Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kms-alias.html
     */
    function Alias(props: C.Omit<Alias, 'Type'>): Alias;
}
export declare namespace AWS.KMS.Alias {
    interface Properties {
        /**
         * String

         *
         * The name of the alias. The name must start with alias followed by a forward slash, such as alias/. You can't specify aliases that begin with alias/AWS. These aliases are reserved.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kms-alias.html#cfn-kms-alias-aliasname
         */
        AliasName: C.YieldsString;
        /**
         * String

         *
         * The ID of the key for which you are creating the alias. Specify the key's globally
 unique identifier or Amazon Resource Name (ARN). You can't specify another alias.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kms-alias.html#cfn-kms-alias-targetkeyid
         */
        TargetKeyId: C.YieldsString;
    }
}
export declare namespace AWS.KMS {
    /**
     * The AWS::KMS::Key resource creates a customer master key (CMK) in AWS Key Management Service
    (AWS KMS). Users (customers) can use the master key to encrypt their data stored in
    AWS services
    that are integrated with AWS KMS or within their applications. For more information,
    see What is the AWS Key Management Service? in the
    AWS Key Management Service Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kms-key.html
     */
    interface Key {
        Type: 'AWS::KMS::Key';
        Properties: AWS.KMS.Key.Properties;
    }
    /**
     * The AWS::KMS::Key resource creates a customer master key (CMK) in AWS Key Management Service
    (AWS KMS). Users (customers) can use the master key to encrypt their data stored in
    AWS services
    that are integrated with AWS KMS or within their applications. For more information,
    see What is the AWS Key Management Service? in the
    AWS Key Management Service Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kms-key.html
     */
    function Key(props: C.Omit<Key, 'Type'>): Key;
}
export declare namespace AWS.KMS.Key {
    interface Properties {
        /**
         * String

         *
         * A description of the key. Use a description that helps your users decide
 whether the key is appropriate for a particular task.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kms-key.html#cfn-kms-key-description
         */
        Description?: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether AWS KMS rotates the key. AWS CloudFormation sets this value to
 false by default.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kms-key.html#cfn-kms-key-enablekeyrotation
         */
        EnableKeyRotation?: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Indicates whether the key is available for use. AWS CloudFormation sets this value
 to
 true by default.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kms-key.html#cfn-kms-key-enabled
         */
        Enabled?: C.YieldsBoolean;
        /**
         * JSON object

         *
         * An AWS KMS key policy to attach to the key. Use a policy to specify who has permission
 to use the key and which actions they can perform. For more information, see Key Policies in the AWS Key Management Service Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kms-key.html#cfn-kms-key-keypolicy
         */
        KeyPolicy: C.Yields<C.CFJson>;
        /**
         *
         *
         *
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kms-key.html#cfn-kms-key-keyusage
         */
        KeyUsage?: C.YieldsString;
    }
}
export declare namespace AWS.Kinesis {
    /**
     * Creates an Kinesis stream that captures and transports data records that are emitted
    from data
    sources. For information about creating streams, see CreateStream in the
    Amazon Kinesis API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesis-stream.html
     */
    interface Stream {
        Type: 'AWS::Kinesis::Stream';
        Properties: AWS.Kinesis.Stream.Properties;
    }
    /**
     * Creates an Kinesis stream that captures and transports data records that are emitted
    from data
    sources. For information about creating streams, see CreateStream in the
    Amazon Kinesis API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesis-stream.html
     */
    function Stream(props: C.Omit<Stream, 'Type'>): Stream;
}
export declare namespace AWS.Kinesis.Stream {
    interface Properties {
        /**
         * String

         *
         * The name of the Kinesis stream. If you don't specify a name, AWS CloudFormation generates
 a unique physical
 ID and uses that ID for the stream name. For more information, see
 Name Type.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesis-stream.html#cfn-kinesis-stream-name
         */
        Name?: C.YieldsString;
        /**
         * Integer

         *
         * The number of hours for the data records that are stored in shards to remain accessible.
 The default value is 24. For more
 information about the stream retention period, see Changing
    the Data Retention Period in the Amazon Kinesis Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesis-stream.html#cfn-kinesis-stream-retentionperiodhours
         */
        RetentionPeriodHours?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The number of shards that the stream uses. For greater provisioned throughput,
 increase the number of shards.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesis-stream.html#cfn-kinesis-stream-shardcount
         */
        ShardCount: C.Yields<C.CFInteger>;
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key–value pairs) to associate with the Kinesis
 stream. For information about constraints for this property, see Tag Restrictions in the
 Amazon Kinesis Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesis-stream.html#cfn-kinesis-stream-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
    }
}
export declare namespace AWS.KinesisAnalytics {
    /**
     * The AWS::KinesisAnalytics::Application resource creates an Amazon Kinesis Analytics
    application. For more information, see the Amazon Kinesis Analytics Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisanalytics-application.html
     */
    interface Application {
        Type: 'AWS::KinesisAnalytics::Application';
        Properties: AWS.KinesisAnalytics.Application.Properties;
    }
    /**
     * The AWS::KinesisAnalytics::Application resource creates an Amazon Kinesis Analytics
    application. For more information, see the Amazon Kinesis Analytics Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisanalytics-application.html
     */
    function Application(props: C.Omit<Application, 'Type'>): Application;
}
export declare namespace AWS.KinesisAnalytics.Application {
    interface Properties {
        /**
         *                             Type: String

         *
         * The name of your Amazon Kinesis Analytics application.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisanalytics-application.html#cfn-kinesisanalytics-application-applicationname
         */
        ApplicationName?: C.YieldsString;
        /**
         *                             Type: List of Kinesis Analytics
    Application Input
         *
         * Use this parameter to configure the application input.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisanalytics-application.html#cfn-kinesisanalytics-application-inputs
         */
        Inputs: C.Yields<C.CFList<C.Yields<AWS.KinesisAnalytics.Application.Input>>>;
        /**
         *                             Type: String

         *
         * The summary description of the application.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisanalytics-application.html#cfn-kinesisanalytics-application-applicationdescription
         */
        ApplicationDescription?: C.YieldsString;
        /**
         *                             Type: String

         *
         * One or more SQL statements that read input data, transform it, and generate
 output.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisanalytics-application.html#cfn-kinesisanalytics-application-applicationcode
         */
        ApplicationCode?: C.YieldsString;
    }
}
export declare namespace AWS.KinesisAnalytics.Application {
    interface CSVMappingParameters {
        /**
         *                             Type: String

         *
         * The
 row
 delimiter. For example, in a CSV format,
 "\n"
 is the typical row delimiter.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-csvmappingparameters.html#cfn-kinesisanalytics-application-csvmappingparameters-recordrowdelimiter
         */
        RecordRowDelimiter: C.YieldsString;
        /**
         *                             Type: String

         *
         * The
 column
 delimiter. For example, in a CSV format, a comma (",") is the typical column delimiter.
 

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-csvmappingparameters.html#cfn-kinesisanalytics-application-csvmappingparameters-recordcolumndelimiter
         */
        RecordColumnDelimiter: C.YieldsString;
    }
}
export declare namespace AWS.KinesisAnalytics.Application {
    interface Input {
        /**
         *                             Type: String

         *
         * The
 name
 prefix to use when creating the in-application streams.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-input.html#cfn-kinesisanalytics-application-input-nameprefix
         */
        NamePrefix: C.YieldsString;
        /**
         *                             Type: Kinesis Analytics
    Application InputSchema
         *
         * Describes the format of the data in the streaming source, and how each data element
 maps to corresponding columns in the in-application stream that is being created.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-input.html#cfn-kinesisanalytics-application-input-inputschema
         */
        InputSchema: C.Yields<AWS.KinesisAnalytics.Application.InputSchema>;
        /**
         *                             Type: Kinesis Analytics Application KinesisStreamsInput
         *
         * If the streaming source is an
 Amazon Kinesis
 stream, identifies the stream's
 ARN
 and an
 IAM
 role that enables
 Kinesis Analytics
 to access the stream on your behalf.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-input.html#cfn-kinesisanalytics-application-input-kinesisstreamsinput
         */
        KinesisStreamsInput?: C.Yields<AWS.KinesisAnalytics.Application.KinesisStreamsInput>;
        /**
         *                             Type: Kinesis Analytics Application KinesisFirehoseInput
         *
         * If the streaming source is an
 Amazon Kinesis Firehose
 delivery stream, identifies the
 Kinesis Firehose
 delivery stream's
 Amazon Resource
 Name
 (ARN)
 and an
 IAM
 role that enables
 Kinesis Analytics
 to access the stream on your behalf.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-input.html#cfn-kinesisanalytics-application-input-kinesisfirehoseinput
         */
        KinesisFirehoseInput?: C.Yields<AWS.KinesisAnalytics.Application.KinesisFirehoseInput>;
        /**
         *                             Type: Kinesis Analytics
    Application InputParallelism
         *
         * Describes the number of in-application streams to create.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-input.html#cfn-kinesisanalytics-application-input-inputparallelism
         */
        InputParallelism?: C.Yields<AWS.KinesisAnalytics.Application.InputParallelism>;
    }
}
export declare namespace AWS.KinesisAnalytics.Application {
    interface InputParallelism {
        /**
         *                             Type: Integer

         *
         * The number of in-application streams to create.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-inputparallelism.html#cfn-kinesisanalytics-application-inputparallelism-count
         */
        Count?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.KinesisAnalytics.Application {
    interface InputSchema {
        /**
         *                             Type: String

         *
         * Specifies the encoding of the records in the streaming source;
 for
 example, UTF-8.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-inputschema.html#cfn-kinesisanalytics-application-inputschema-recordencoding
         */
        RecordEncoding?: C.YieldsString;
        /**
         *                             Type: List of Kinesis Analytics
    Application RecordColumn
         *
         * A list of
 RecordColumn
 objects.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-inputschema.html#cfn-kinesisanalytics-application-inputschema-recordcolumns
         */
        RecordColumns: C.Yields<C.CFList<C.Yields<AWS.KinesisAnalytics.Application.RecordColumn>>>;
        /**
         *                             Type: Kinesis Analytics
    Application RecordFormat
         *
         * Specifies the format of the records on the streaming source.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-inputschema.html#cfn-kinesisanalytics-application-inputschema-recordformat
         */
        RecordFormat: C.Yields<AWS.KinesisAnalytics.Application.RecordFormat>;
    }
}
export declare namespace AWS.KinesisAnalytics.Application {
    interface JSONMappingParameters {
        /**
         *                             Type: String

         *
         * The
 path
 to the top-level parent that contains the records (e.g.,
 "$".)

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-jsonmappingparameters.html#cfn-kinesisanalytics-application-jsonmappingparameters-recordrowpath
         */
        RecordRowPath: C.YieldsString;
    }
}
export declare namespace AWS.KinesisAnalytics.Application {
    interface KinesisFirehoseInput {
        /**
         *                             Type: String

         *
         * The
 Amazon Resource Name
 (ARN)
 of the input
 Kinesis Firehose
 delivery stream.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-kinesisfirehoseinput.html#cfn-kinesisanalytics-application-kinesisfirehoseinput-resourcearn
         */
        ResourceARN: C.YieldsString;
        /**
         *                             Type: String

         *
         * The
 ARN of the
 IAM
 role that
 Kinesis Analytics
 can assume to access the stream on your behalf.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-kinesisfirehoseinput.html#cfn-kinesisanalytics-application-kinesisfirehoseinput-rolearn
         */
        RoleARN: C.YieldsString;
    }
}
export declare namespace AWS.KinesisAnalytics.Application {
    interface KinesisStreamsInput {
        /**
         *                             Type: String

         *
         * The
 Amazon Resource Name
 (ARN)
 of the input
 Amazon Kinesis
 stream to read.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-kinesisstreamsinput.html#cfn-kinesisanalytics-application-kinesisstreamsinput-resourcearn
         */
        ResourceARN: C.YieldsString;
        /**
         *                             Type: String

         *
         * The
 ARN of the IAM role that
 Kinesis Analytics
 can assume to access the stream on your behalf.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-kinesisstreamsinput.html#cfn-kinesisanalytics-application-kinesisstreamsinput-rolearn
         */
        RoleARN: C.YieldsString;
    }
}
export declare namespace AWS.KinesisAnalytics.Application {
    interface MappingParameters {
        /**
         *                             Type: Kinesis Analytics Application JSONMappingParameters
         *
         * Provides additional mapping information when JSON is the record format on the streaming
 source.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-mappingparameters.html#cfn-kinesisanalytics-application-mappingparameters-jsonmappingparameters
         */
        JSONMappingParameters?: C.Yields<AWS.KinesisAnalytics.Application.JSONMappingParameters>;
        /**
         *                             Type: Kinesis Analytics Application CSVMappingParameters
         *
         * Provides additional mapping information when the record format uses delimiters (for
 example, CSV).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-mappingparameters.html#cfn-kinesisanalytics-application-mappingparameters-csvmappingparameters
         */
        CSVMappingParameters?: C.Yields<AWS.KinesisAnalytics.Application.CSVMappingParameters>;
    }
}
export declare namespace AWS.KinesisAnalytics.Application {
    interface RecordColumn {
        /**
         *                             Type: String

         *
         * Reference to the data element in the streaming input of the reference data source.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-recordcolumn.html#cfn-kinesisanalytics-application-recordcolumn-mapping
         */
        Mapping?: C.YieldsString;
        /**
         *                             Type: String

         *
         * The
 type
 of column created in the in-application input stream or reference table.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-recordcolumn.html#cfn-kinesisanalytics-application-recordcolumn-sqltype
         */
        SqlType: C.YieldsString;
        /**
         *                             Type: String

         *
         * The
 name
 of the column created in the in-application input stream or reference table.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-recordcolumn.html#cfn-kinesisanalytics-application-recordcolumn-name
         */
        Name: C.YieldsString;
    }
}
export declare namespace AWS.KinesisAnalytics.Application {
    interface RecordFormat {
        /**
         *                             Type: Kinesis Analytics Application MappingParameters
         *
         * When configuring application input at the time of creating or updating an
 application, provides additional mapping information specific to the record format
 (such
 as JSON, CSV, or record fields delimited by some delimiter) on the streaming source.
 

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-recordformat.html#cfn-kinesisanalytics-application-recordformat-mappingparameters
         */
        MappingParameters?: C.Yields<AWS.KinesisAnalytics.Application.MappingParameters>;
        /**
         *                             Type: String

         *
         * The type of record format (e.g CSV or JSON.)

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-recordformat.html#cfn-kinesisanalytics-application-recordformat-recordformattype
         */
        RecordFormatType: C.YieldsString;
    }
}
export declare namespace AWS.KinesisAnalytics {
    /**
     * The
    AWS::KinesisAnalytics::ApplicationOutput resource adds an external destination to your Amazon Kinesis Analytics application.
    For more information, see
    AddApplicationOutput in the
    Amazon Kinesis Analytics Developer Guide.
    
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisanalytics-applicationoutput.html
     */
    interface ApplicationOutput {
        Type: 'AWS::KinesisAnalytics::ApplicationOutput';
        Properties: AWS.KinesisAnalytics.ApplicationOutput.Properties;
    }
    /**
     * The
    AWS::KinesisAnalytics::ApplicationOutput resource adds an external destination to your Amazon Kinesis Analytics application.
    For more information, see
    AddApplicationOutput in the
    Amazon Kinesis Analytics Developer Guide.
    
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisanalytics-applicationoutput.html
     */
    function ApplicationOutput(props: C.Omit<ApplicationOutput, 'Type'>): ApplicationOutput;
}
export declare namespace AWS.KinesisAnalytics.ApplicationOutput {
    interface Properties {
        /**
         * String

         *
         * The
 name
 of the application to which you want to add the output configuration.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisanalytics-applicationoutput.html#cfn-kinesisanalytics-applicationoutput-applicationname
         */
        ApplicationName: C.YieldsString;
        /**
         *                             Type: Kinesis Analytics
    ApplicationOutput Output
         *
         * An array of objects, each describing one output configuration.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisanalytics-applicationoutput.html#cfn-kinesisanalytics-applicationoutput-output
         */
        Output: C.Yields<AWS.KinesisAnalytics.ApplicationOutput.Output>;
    }
}
export declare namespace AWS.KinesisAnalytics.ApplicationOutput {
    interface DestinationSchema {
        /**
         *                             Type: String

         *
         * Specifies the format of the records on the output stream.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationoutput-destinationschema.html#cfn-kinesisanalytics-applicationoutput-destinationschema-recordformattype
         */
        RecordFormatType?: C.YieldsString;
    }
}
export declare namespace AWS.KinesisAnalytics.ApplicationOutput {
    interface KinesisFirehoseOutput {
        /**
         *                             Type: String

         *
         * The
 Amazon Resource
 Name
 (ARN)
 of the destination Amazon Kinesis Firehose delivery stream to write to.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationoutput-kinesisfirehoseoutput.html#cfn-kinesisanalytics-applicationoutput-kinesisfirehoseoutput-resourcearn
         */
        ResourceARN: C.YieldsString;
        /**
         *                             Type: String

         *
         * The ARN of the IAM role that Amazon Kinesis Analytics can assume to write to the destination
 stream on your behalf.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationoutput-kinesisfirehoseoutput.html#cfn-kinesisanalytics-applicationoutput-kinesisfirehoseoutput-rolearn
         */
        RoleARN: C.YieldsString;
    }
}
export declare namespace AWS.KinesisAnalytics.ApplicationOutput {
    interface KinesisStreamsOutput {
        /**
         *                             Type: String

         *
         * The
 Amazon Resource
 Name
 (ARN)
 of the destination Amazon Kinesis stream to write to.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationoutput-kinesisstreamsoutput.html#cfn-kinesisanalytics-applicationoutput-kinesisstreamsoutput-resourcearn
         */
        ResourceARN: C.YieldsString;
        /**
         *                             Type: String

         *
         * The ARN of the IAM role that Amazon Kinesis Analytics can assume to write to the destination
 stream on your behalf.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationoutput-kinesisstreamsoutput.html#cfn-kinesisanalytics-applicationoutput-kinesisstreamsoutput-rolearn
         */
        RoleARN: C.YieldsString;
    }
}
export declare namespace AWS.KinesisAnalytics.ApplicationOutput {
    interface Output {
        /**
         *                             Type: Kinesis Analytics
    ApplicationOutput DestinationSchema
         *
         * The data format when records are written to the destination.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationoutput-output.html#cfn-kinesisanalytics-applicationoutput-output-destinationschema
         */
        DestinationSchema: C.Yields<AWS.KinesisAnalytics.ApplicationOutput.DestinationSchema>;
        /**
         *                             Type: Kinesis Analytics
    ApplicationOutput KinesisFirehoseOutput
         *
         * Identifies an
 Amazon Kinesis Firehose
 delivery stream as the destination.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationoutput-output.html#cfn-kinesisanalytics-applicationoutput-output-kinesisfirehoseoutput
         */
        KinesisFirehoseOutput?: C.Yields<AWS.KinesisAnalytics.ApplicationOutput.KinesisFirehoseOutput>;
        /**
         *                             Type: Kinesis Analytics
    ApplicationOutput KinesisStreamsOutput
         *
         * Identifies an
 Amazon Kinesis
 stream as the destination.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationoutput-output.html#cfn-kinesisanalytics-applicationoutput-output-kinesisstreamsoutput
         */
        KinesisStreamsOutput?: C.Yields<AWS.KinesisAnalytics.ApplicationOutput.KinesisStreamsOutput>;
        /**
         *                             Type: String

         *
         * The name of the in-application stream.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationoutput-output.html#cfn-kinesisanalytics-applicationoutput-output-name
         */
        Name?: C.YieldsString;
    }
}
export declare namespace AWS.KinesisAnalytics {
    /**
     * Use the AWS CloudFormation
    AWS::KinesisAnalytics::ApplicationReferenceDataSource resource  to add a reference data source to an existing  Amazon Kinesis Analytics
    application.
    For more information, see AddApplicationReferenceDataSource in the
    Amazon Kinesis Analytics Developer Guide.
    
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisanalytics-applicationreferencedatasource.html
     */
    interface ApplicationReferenceDataSource {
        Type: 'AWS::KinesisAnalytics::ApplicationReferenceDataSource';
        Properties: AWS.KinesisAnalytics.ApplicationReferenceDataSource.Properties;
    }
    /**
     * Use the AWS CloudFormation
    AWS::KinesisAnalytics::ApplicationReferenceDataSource resource  to add a reference data source to an existing  Amazon Kinesis Analytics
    application.
    For more information, see AddApplicationReferenceDataSource in the
    Amazon Kinesis Analytics Developer Guide.
    
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisanalytics-applicationreferencedatasource.html
     */
    function ApplicationReferenceDataSource(props: C.Omit<ApplicationReferenceDataSource, 'Type'>): ApplicationReferenceDataSource;
}
export declare namespace AWS.KinesisAnalytics.ApplicationReferenceDataSource {
    interface Properties {
        /**
         *                             Type: String

         *
         * The name of an existing application.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisanalytics-applicationreferencedatasource.html#cfn-kinesisanalytics-applicationreferencedatasource-applicationname
         */
        ApplicationName: C.YieldsString;
        /**
         *                             Type: Kinesis Analytics ApplicationReferenceDataSource ReferenceDataSource
         *
         * The reference data source, which is an object in your
 Amazon Simple Storage Service
 (Amazon S3) bucket.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisanalytics-applicationreferencedatasource.html#cfn-kinesisanalytics-applicationreferencedatasource-referencedatasource
         */
        ReferenceDataSource: C.Yields<AWS.KinesisAnalytics.ApplicationReferenceDataSource.ReferenceDataSource>;
    }
}
export declare namespace AWS.KinesisAnalytics.ApplicationReferenceDataSource {
    interface CSVMappingParameters {
        /**
         *                             Type: String

         *
         * The row delimiter. For example, in a CSV format,
 "\n"
 is the typical row delimiter.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-csvmappingparameters.html#cfn-kinesisanalytics-applicationreferencedatasource-csvmappingparameters-recordrowdelimiter
         */
        RecordRowDelimiter: C.YieldsString;
        /**
         *                             Type: String

         *
         * The column delimiter. For example, in a CSV format, a comma (",") is the typical column
 delimiter.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-csvmappingparameters.html#cfn-kinesisanalytics-applicationreferencedatasource-csvmappingparameters-recordcolumndelimiter
         */
        RecordColumnDelimiter: C.YieldsString;
    }
}
export declare namespace AWS.KinesisAnalytics.ApplicationReferenceDataSource {
    interface JSONMappingParameters {
        /**
         *                             Type: String;

         *
         * Path to the top-level parent that contains the records
 (e.g.,
 "$".)

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-jsonmappingparameters.html#cfn-kinesisanalytics-applicationreferencedatasource-jsonmappingparameters-recordrowpath
         */
        RecordRowPath: C.YieldsString;
    }
}
export declare namespace AWS.KinesisAnalytics.ApplicationReferenceDataSource {
    interface MappingParameters {
        /**
         *                             Type: Kinesis Analytics ApplicationReferenceDataSource JSONMappingParameters
         *
         * Provides additional mapping information when JSON is the record format on the streaming
 source.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-mappingparameters.html#cfn-kinesisanalytics-applicationreferencedatasource-mappingparameters-jsonmappingparameters
         */
        JSONMappingParameters?: C.Yields<AWS.KinesisAnalytics.ApplicationReferenceDataSource.JSONMappingParameters>;
        /**
         *                             Type: Kinesis Analytics ApplicationReferenceDataSource CSVMappingParameters
         *
         * Provides additional mapping information when the record format uses delimiters (for
 example, CSV).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-mappingparameters.html#cfn-kinesisanalytics-applicationreferencedatasource-mappingparameters-csvmappingparameters
         */
        CSVMappingParameters?: C.Yields<AWS.KinesisAnalytics.ApplicationReferenceDataSource.CSVMappingParameters>;
    }
}
export declare namespace AWS.KinesisAnalytics.ApplicationReferenceDataSource {
    interface RecordColumn {
        /**
         *                             Type: String;

         *
         * The reference to the data element in the streaming input of the reference data source.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-recordcolumn.html#cfn-kinesisanalytics-applicationreferencedatasource-recordcolumn-mapping
         */
        Mapping?: C.YieldsString;
        /**
         *                             Type: String;

         *
         * The SQL
 data type of
 the column created in the in-application input stream or reference table.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-recordcolumn.html#cfn-kinesisanalytics-applicationreferencedatasource-recordcolumn-sqltype
         */
        SqlType: C.YieldsString;
        /**
         *                             Type: String;

         *
         * The name of the column created in the in-application input stream or reference table.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-recordcolumn.html#cfn-kinesisanalytics-applicationreferencedatasource-recordcolumn-name
         */
        Name: C.YieldsString;
    }
}
export declare namespace AWS.KinesisAnalytics.ApplicationReferenceDataSource {
    interface RecordFormat {
        /**
         *                             Type: Kinesis Analytics ApplicationReferenceDataSource MappingParameters
         *
         * When configuring application input at the time of creating or updating an application,
 provides additional mapping information specific to the record format (such as JSON,
 CSV, or record fields delimited by some delimiter) on the streaming source.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-recordformat.html#cfn-kinesisanalytics-applicationreferencedatasource-recordformat-mappingparameters
         */
        MappingParameters?: C.Yields<AWS.KinesisAnalytics.ApplicationReferenceDataSource.MappingParameters>;
        /**
         *                             Type: String;

         *
         * The type of record format (CSV or JSON).
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-recordformat.html#cfn-kinesisanalytics-applicationreferencedatasource-recordformat-recordformattype
         */
        RecordFormatType: C.YieldsString;
    }
}
export declare namespace AWS.KinesisAnalytics.ApplicationReferenceDataSource {
    interface ReferenceDataSource {
        /**
         *                             Type: Kinesis Analytics ApplicationReferenceDataSource ReferenceSchema
         *
         * Describes the format of the data in the streaming source, and how each data
 element maps to corresponding columns
 that are
 created in the in-application stream.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-referencedatasource.html#cfn-kinesisanalytics-applicationreferencedatasource-referencedatasource-referenceschema
         */
        ReferenceSchema: C.Yields<AWS.KinesisAnalytics.ApplicationReferenceDataSource.ReferenceSchema>;
        /**
         *                             Type: String;

         *
         * The
 name
 of the in-application table to create.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-referencedatasource.html#cfn-kinesisanalytics-applicationreferencedatasource-referencedatasource-tablename
         */
        TableName?: C.YieldsString;
        /**
         *                             Type: Kinesis Analytics ApplicationReferenceDataSource S3ReferenceDataSource
         *
         * Identifies the
 Amazon S3
 bucket and object that contains the reference data. Also identifies the
 IAM
 role that
 Amazon Kinesis Analytics
 can assume to read this object on your behalf.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-referencedatasource.html#cfn-kinesisanalytics-applicationreferencedatasource-referencedatasource-s3referencedatasource
         */
        S3ReferenceDataSource?: C.Yields<AWS.KinesisAnalytics.ApplicationReferenceDataSource.S3ReferenceDataSource>;
    }
}
export declare namespace AWS.KinesisAnalytics.ApplicationReferenceDataSource {
    interface ReferenceSchema {
        /**
         *                             Type: String;

         *
         * Specifies the encoding of the records in the streaming source; For example,
 UTF-8.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-referenceschema.html#cfn-kinesisanalytics-applicationreferencedatasource-referenceschema-recordencoding
         */
        RecordEncoding?: C.YieldsString;
        /**
         *                             Type: List of Kinesis Analytics ApplicationReferenceDataSource RecordColumn
         *
         * A list of Amazon Kinesis Analytics ApplicationReferenceDataSource RecordColumn objects.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-referenceschema.html#cfn-kinesisanalytics-applicationreferencedatasource-referenceschema-recordcolumns
         */
        RecordColumns: C.Yields<C.CFList<C.Yields<AWS.KinesisAnalytics.ApplicationReferenceDataSource.RecordColumn>>>;
        /**
         *                             Type: Kinesis Analytics ApplicationReferenceDataSource RecordFormat
         *
         * Specifies the format of the records on the streaming source.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-referenceschema.html#cfn-kinesisanalytics-applicationreferencedatasource-referenceschema-recordformat
         */
        RecordFormat: C.Yields<AWS.KinesisAnalytics.ApplicationReferenceDataSource.RecordFormat>;
    }
}
export declare namespace AWS.KinesisAnalytics.ApplicationReferenceDataSource {
    interface S3ReferenceDataSource {
        /**
         *                             Type: String;

         *
         * The Amazon Resource Name (ARN) of the
 Amazon S3
 bucket.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-s3referencedatasource.html#cfn-kinesisanalytics-applicationreferencedatasource-s3referencedatasource-bucketarn
         */
        BucketARN: C.YieldsString;
        /**
         *                             Type: String;

         *
         * The object key name containing reference data.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-s3referencedatasource.html#cfn-kinesisanalytics-applicationreferencedatasource-s3referencedatasource-filekey
         */
        FileKey: C.YieldsString;
        /**
         *                             Type: String;

         *
         * The ARN of the
 IAM
 role that the service can assume to read data on your behalf.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-s3referencedatasource.html#cfn-kinesisanalytics-applicationreferencedatasource-s3referencedatasource-referencerolearn
         */
        ReferenceRoleARN: C.YieldsString;
    }
}
export declare namespace AWS.KinesisFirehose {
    /**
     * The AWS::KinesisFirehose::DeliveryStream resource creates an Amazon Kinesis Firehose (Kinesis Firehose) delivery stream that
    delivers real-time streaming data to an Amazon Simple Storage Service (Amazon S3),
    Amazon Redshift, or Amazon Elasticsearch Service (Amazon ES) destination. For more
    information, see Creating an Amazon Kinesis Firehose Delivery Stream in the Amazon Kinesis Firehose Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisfirehose-deliverystream.html
     */
    interface DeliveryStream {
        Type: 'AWS::KinesisFirehose::DeliveryStream';
        Properties: AWS.KinesisFirehose.DeliveryStream.Properties;
    }
    /**
     * The AWS::KinesisFirehose::DeliveryStream resource creates an Amazon Kinesis Firehose (Kinesis Firehose) delivery stream that
    delivers real-time streaming data to an Amazon Simple Storage Service (Amazon S3),
    Amazon Redshift, or Amazon Elasticsearch Service (Amazon ES) destination. For more
    information, see Creating an Amazon Kinesis Firehose Delivery Stream in the Amazon Kinesis Firehose Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisfirehose-deliverystream.html
     */
    function DeliveryStream(props: C.Omit<DeliveryStream, 'Type'>): DeliveryStream;
}
export declare namespace AWS.KinesisFirehose.DeliveryStream {
    interface Properties {
        /**
         * String

         *
         * A name for the delivery stream.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisfirehose-deliverystream.html#cfn-kinesisfirehose-deliverystream-deliverystreamname
         */
        DeliveryStreamName?: C.YieldsString;
        /**
         * Kinesis Firehose DeliveryStream ElasticsearchDestinationConfiguration
         *
         * An Amazon ES destination for the delivery stream.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisfirehose-deliverystream.html#cfn-kinesisfirehose-deliverystream-elasticsearchdestinationconfiguration
         */
        ElasticsearchDestinationConfiguration?: C.Yields<AWS.KinesisFirehose.DeliveryStream.ElasticsearchDestinationConfiguration>;
        /**
         *                             Type: Kinesis Firehose DeliveryStream ExtendedS3DestinationConfiguration
         *
         * An Amazon S3 destination for the delivery stream.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisfirehose-deliverystream.html#cfn-kinesisfirehose-deliverystream-extendeds3destinationconfiguration
         */
        ExtendedS3DestinationConfiguration?: C.Yields<AWS.KinesisFirehose.DeliveryStream.ExtendedS3DestinationConfiguration>;
        /**
         * Kinesis Firehose DeliveryStream RedshiftDestinationConfiguration
         *
         * An Amazon Redshift destination for the delivery stream.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisfirehose-deliverystream.html#cfn-kinesisfirehose-deliverystream-redshiftdestinationconfiguration
         */
        RedshiftDestinationConfiguration?: C.Yields<AWS.KinesisFirehose.DeliveryStream.RedshiftDestinationConfiguration>;
        /**
         * Kinesis Firehose DeliveryStream S3DestinationConfiguration
         *
         * An Amazon S3 destination for the delivery stream.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisfirehose-deliverystream.html#cfn-kinesisfirehose-deliverystream-s3destinationconfiguration
         */
        S3DestinationConfiguration?: C.Yields<AWS.KinesisFirehose.DeliveryStream.S3DestinationConfiguration>;
    }
}
export declare namespace AWS.KinesisFirehose.DeliveryStream {
    interface BufferingHints {
        /**
         * Integer

         *
         * The length of time, in seconds, that Kinesis Firehose buffers incoming data before
 delivering it to the destination. For valid values, see the IntervalInSeconds content for the BufferingHints data type in the Amazon Kinesis Firehose API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-bufferinghints.html#cfn-kinesisfirehose-deliverystream-bufferinghints-intervalinseconds
         */
        IntervalInSeconds: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The size of the buffer, in MBs, that Kinesis Firehose uses for incoming data before
 delivering it to the destination. For valid values, see the SizeInMBs content for the BufferingHints data type in the Amazon Kinesis Firehose API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-bufferinghints.html#cfn-kinesisfirehose-deliverystream-bufferinghints-sizeinmbs
         */
        SizeInMBs: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.KinesisFirehose.DeliveryStream {
    interface CloudWatchLoggingOptions {
        /**
         * Boolean

         *
         * Indicates whether CloudWatch Logs logging is enabled.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-cloudwatchloggingoptions.html#cfn-kinesisfirehose-deliverystream-cloudwatchloggingoptions-enabled
         */
        Enabled?: C.YieldsBoolean;
        /**
         * String

         *
         * The name of the CloudWatch Logs log group that contains the log stream that Kinesis
 Firehose will use.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-cloudwatchloggingoptions.html#cfn-kinesisfirehose-deliverystream-cloudwatchloggingoptions-loggroupname
         */
        LogGroupName?: C.YieldsString;
        /**
         * String

         *
         * The name of the CloudWatch Logs log stream that Kinesis Firehose uses to send logs
 about data delivery.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-cloudwatchloggingoptions.html#cfn-kinesisfirehose-deliverystream-cloudwatchloggingoptions-logstreamname
         */
        LogStreamName?: C.YieldsString;
    }
}
export declare namespace AWS.KinesisFirehose.DeliveryStream {
    interface CopyCommand {
        /**
         * String

         *
         * Parameters to use with the Amazon Redshift COPY command. For examples, see the CopyOptions content for the CopyCommand data type in the Amazon Kinesis Firehose API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-copycommand.html#cfn-kinesisfirehose-deliverystream-copycommand-copyoptions
         */
        CopyOptions?: C.YieldsString;
        /**
         * String

         *
         * A comma-separated list of the column names in the table that Kinesis Firehose copies
 data to.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-copycommand.html#cfn-kinesisfirehose-deliverystream-copycommand-datatablecolumns
         */
        DataTableColumns?: C.YieldsString;
        /**
         * String

         *
         * The name of the table where Kinesis Firehose adds the copied data.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-copycommand.html#cfn-kinesisfirehose-deliverystream-copycommand-datatablename
         */
        DataTableName: C.YieldsString;
    }
}
export declare namespace AWS.KinesisFirehose.DeliveryStream {
    interface ElasticsearchBufferingHints {
        /**
         * Integer

         *
         * The length of time, in seconds, that Kinesis Firehose buffers incoming data before
 delivering it to the destination. For valid values, see the IntervalInSeconds content for the BufferingHints data type in the Amazon Kinesis Firehose API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-elasticsearchbufferinghints.html#cfn-kinesisfirehose-deliverystream-elasticsearchbufferinghints-intervalinseconds
         */
        IntervalInSeconds: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The size of the buffer, in MBs, that Kinesis Firehose uses for incoming data before
 delivering it to the destination. For valid values, see the SizeInMBs content for the BufferingHints data type in the Amazon Kinesis Firehose API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-elasticsearchbufferinghints.html#cfn-kinesisfirehose-deliverystream-elasticsearchbufferinghints-sizeinmbs
         */
        SizeInMBs: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.KinesisFirehose.DeliveryStream {
    interface ElasticsearchDestinationConfiguration {
        /**
         * Kinesis Firehose DeliveryStream ElasticsearchBufferingHints
         *
         * Configures how Kinesis Firehose buffers incoming data while delivering it to the Amazon
 ES domain.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-elasticsearchdestinationconfiguration.html#cfn-kinesisfirehose-deliverystream-elasticsearchdestinationconfiguration-bufferinghints
         */
        BufferingHints: C.Yields<AWS.KinesisFirehose.DeliveryStream.ElasticsearchBufferingHints>;
        /**
         * Kinesis Firehose DeliveryStream CloudWatchLoggingOptions
         *
         * The Amazon CloudWatch Logs logging options for the delivery stream.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-elasticsearchdestinationconfiguration.html#cfn-kinesisfirehose-deliverystream-elasticsearchdestinationconfiguration-cloudwatchloggingoptions
         */
        CloudWatchLoggingOptions?: C.Yields<AWS.KinesisFirehose.DeliveryStream.CloudWatchLoggingOptions>;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the Amazon ES domain that Kinesis Firehose delivers
 data to.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-elasticsearchdestinationconfiguration.html#cfn-kinesisfirehose-deliverystream-elasticsearchdestinationconfiguration-domainarn
         */
        DomainARN: C.YieldsString;
        /**
         * String

         *
         * The name of the Elasticsearch index to which Kinesis Firehose adds data for indexing.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-elasticsearchdestinationconfiguration.html#cfn-kinesisfirehose-deliverystream-elasticsearchdestinationconfiguration-indexname
         */
        IndexName: C.YieldsString;
        /**
         * String

         *
         * The frequency of Elasticsearch index rotation. If you enable index rotation, Kinesis
 Firehose appends a portion of the UTC arrival timestamp to the specified index name,
 and rotates the appended timestamp accordingly. For more information, see Index Rotation for the Amazon ES Destination in the Amazon Kinesis Firehose Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-elasticsearchdestinationconfiguration.html#cfn-kinesisfirehose-deliverystream-elasticsearchdestinationconfiguration-indexrotationperiod
         */
        IndexRotationPeriod: C.YieldsString;
        /**
         *                             Type: Kinesis Firehose DeliveryStream ProcessingConfiguration
         *
         * The data processing configuration for the Kinesis Firehose delivery stream.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-elasticsearchdestinationconfiguration.html#cfn-kinesisfirehose-deliverystream-elasticsearchdestinationconfiguration-processingconfiguration
         */
        ProcessingConfiguration?: C.Yields<AWS.KinesisFirehose.DeliveryStream.ProcessingConfiguration>;
        /**
         * Kinesis Firehose DeliveryStream ElasticsearchRetryOptionsType: String

         *
         * The retry behavior when Kinesis Firehose is unable to deliver data to Amazon ES.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-elasticsearchdestinationconfiguration.html#cfn-kinesisfirehose-deliverystream-elasticsearchdestinationconfiguration-retryoptions
         */
        RetryOptions: C.Yields<AWS.KinesisFirehose.DeliveryStream.ElasticsearchRetryOptions>;
        /**
         * String

         *
         * The ARN of the AWS Identity and Access Management (IAM) role that grants Kinesis Firehose
 access to your Amazon S3 bucket, AWS KMS (if you enable data encryption), and Amazon
 CloudWatch Logs (if you enable logging).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-elasticsearchdestinationconfiguration.html#cfn-kinesisfirehose-deliverystream-elasticsearchdestinationconfiguration-rolearn
         */
        RoleARN: C.YieldsString;
        /**
         * String

         *
         * The condition under which Kinesis Firehose delivers data to Amazon Simple Storage
 Service (Amazon S3). You can send Amazon S3 all documents (all data) or only the documents
 that Kinesis Firehose could not deliver to the Amazon ES destination.
 For more information and valid values, see the S3BackupMode content for the ElasticsearchDestinationConfiguration data type in the Amazon Kinesis Firehose API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-elasticsearchdestinationconfiguration.html#cfn-kinesisfirehose-deliverystream-elasticsearchdestinationconfiguration-s3backupmode
         */
        S3BackupMode: C.YieldsString;
        /**
         * Kinesis Firehose DeliveryStream S3DestinationConfigurationType: String

         *
         * The S3 bucket where Kinesis Firehose backs up incoming data.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-elasticsearchdestinationconfiguration.html#cfn-kinesisfirehose-deliverystream-elasticsearchdestinationconfiguration-s3configuration
         */
        S3Configuration: C.Yields<AWS.KinesisFirehose.DeliveryStream.S3DestinationConfiguration>;
        /**
         * String

         *
         * The Elasticsearch type name that Amazon ES adds to documents when indexing data.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-elasticsearchdestinationconfiguration.html#cfn-kinesisfirehose-deliverystream-elasticsearchdestinationconfiguration-typename
         */
        TypeName: C.YieldsString;
    }
}
export declare namespace AWS.KinesisFirehose.DeliveryStream {
    interface ElasticsearchRetryOptions {
        /**
         * Integer

         *
         * After an initial failure to deliver to Amazon ES, the total amount of time during
 which Kinesis Firehose re-attempts delivery (including the first attempt). If Kinesis
 Firehose can't deliver the data within the specified time, it writes the data to the
 backup S3 bucket. For valid values, see the DurationInSeconds content for the ElasticsearchRetryOptions data type in the Amazon Kinesis Firehose API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-elasticsearchretryoptions.html#cfn-kinesisfirehose-deliverystream-elasticsearchretryoptions-durationinseconds
         */
        DurationInSeconds: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.KinesisFirehose.DeliveryStream {
    interface EncryptionConfiguration {
        /**
         * Amazon Kinesis Firehose DeliveryStream KMSEncryptionConfig
         *
         * The AWS Key Management Service (AWS KMS) encryption key that Amazon S3 uses to encrypt
 your data.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-encryptionconfiguration.html#cfn-kinesisfirehose-deliverystream-encryptionconfiguration-kmsencryptionconfig
         */
        KMSEncryptionConfig?: C.Yields<AWS.KinesisFirehose.DeliveryStream.KMSEncryptionConfig>;
        /**
         * String

         *
         * Disables encryption. For valid values, see the NoEncryptionConfig content for the EncryptionConfiguration data type in the Amazon Kinesis Firehose API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-encryptionconfiguration.html#cfn-kinesisfirehose-deliverystream-encryptionconfiguration-noencryptionconfig
         */
        NoEncryptionConfig?: C.YieldsString;
    }
}
export declare namespace AWS.KinesisFirehose.DeliveryStream {
    interface ExtendedS3DestinationConfiguration {
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the Amazon S3 bucket. For constraints, see ExtendedS3DestinationConfiguration
 in the Amazon Kinesis Firehose API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-extendeds3destinationconfiguration.html#cfn-kinesisfirehose-deliverystream-extendeds3destinationconfiguration-bucketarn
         */
        BucketARN: C.YieldsString;
        /**
         *                             Type: Kinesis Firehose DeliveryStream BufferingHints
         *
         * The buffering option.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-extendeds3destinationconfiguration.html#cfn-kinesisfirehose-deliverystream-extendeds3destinationconfiguration-bufferinghints
         */
        BufferingHints: C.Yields<AWS.KinesisFirehose.DeliveryStream.BufferingHints>;
        /**
         *                             Type: Kinesis Firehose DeliveryStream CloudWatchLoggingOptions
         *
         * The CloudWatch logging options for the Kinesis Firehose delivery stream.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-extendeds3destinationconfiguration.html#cfn-kinesisfirehose-deliverystream-extendeds3destinationconfiguration-cloudwatchloggingoptions
         */
        CloudWatchLoggingOptions?: C.Yields<AWS.KinesisFirehose.DeliveryStream.CloudWatchLoggingOptions>;
        /**
         * String

         *
         * The compression format for the Kinesis Firehose delivery stream. The default value
 is UNCOMPRESSED. For valid values, see ExtendedS3DestinationConfiguration
 in the Amazon Kinesis Firehose API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-extendeds3destinationconfiguration.html#cfn-kinesisfirehose-deliverystream-extendeds3destinationconfiguration-compressionformat
         */
        CompressionFormat: C.YieldsString;
        /**
         *                             Type: Kinesis Firehose DeliveryStream EncryptionConfiguration
         *
         * The encryption configuration for the Kinesis Firehose delivery stream. The default
 value is NoEncryption.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-extendeds3destinationconfiguration.html#cfn-kinesisfirehose-deliverystream-extendeds3destinationconfiguration-encryptionconfiguration
         */
        EncryptionConfiguration?: C.Yields<AWS.KinesisFirehose.DeliveryStream.EncryptionConfiguration>;
        /**
         * String

         *
         * The YYYY/MM/DD/HH time format prefix is automatically used for delivered Amazon S3 files. For more
 information, see ExtendedS3DestinationConfiguration
 in the Amazon Kinesis Firehose API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-extendeds3destinationconfiguration.html#cfn-kinesisfirehose-deliverystream-extendeds3destinationconfiguration-prefix
         */
        Prefix: C.YieldsString;
        /**
         *                             Type: Kinesis Firehose DeliveryStream ProcessingConfiguration
         *
         * The data processing configuration for the Kinesis Firehose delivery stream.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-extendeds3destinationconfiguration.html#cfn-kinesisfirehose-deliverystream-extendeds3destinationconfiguration-processingconfiguration
         */
        ProcessingConfiguration?: C.Yields<AWS.KinesisFirehose.DeliveryStream.ProcessingConfiguration>;
        /**
         * String

         *
         * The ARN of the AWS credentials. For constraints, see ExtendedS3DestinationConfiguration
 in the Amazon Kinesis Firehose API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-extendeds3destinationconfiguration.html#cfn-kinesisfirehose-deliverystream-extendeds3destinationconfiguration-rolearn
         */
        RoleARN: C.YieldsString;
        /**
         *                             Type: Kinesis Firehose DeliveryStream S3DestinationConfiguration
         *
         * The configuration for backup in Amazon S3.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-extendeds3destinationconfiguration.html#cfn-kinesisfirehose-deliverystream-extendeds3destinationconfiguration-s3backupconfiguration
         */
        S3BackupConfiguration?: C.Yields<AWS.KinesisFirehose.DeliveryStream.S3DestinationConfiguration>;
        /**
         * String

         *
         * The Amazon S3 backup mode. For valid values, see ExtendedS3DestinationConfiguration
 in the Amazon Kinesis Firehose API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-extendeds3destinationconfiguration.html#cfn-kinesisfirehose-deliverystream-extendeds3destinationconfiguration-s3backupmode
         */
        S3BackupMode?: C.YieldsString;
    }
}
export declare namespace AWS.KinesisFirehose.DeliveryStream {
    interface KMSEncryptionConfig {
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the AWS KMS encryption key that Amazon S3 uses to
 encrypt data delivered by the Kinesis Firehose stream. The key must belong to the
 same region as the destination S3 bucket.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-kmsencryptionconfig.html#cfn-kinesisfirehose-deliverystream-kmsencryptionconfig-awskmskeyarn
         */
        AWSKMSKeyARN: C.YieldsString;
    }
}
export declare namespace AWS.KinesisFirehose.DeliveryStream {
    interface ProcessingConfiguration {
        /**
         * Boolean

         *
         * Indicates whether data processing is enabled (true) or disabled (false).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-processingconfiguration.html#cfn-kinesisfirehose-deliverystream-processingconfiguration-enabled
         */
        Enabled: C.YieldsBoolean;
        /**
         *                             Type: List of Kinesis Firehose DeliveryStream Processor
         *
         * The data processors.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-processingconfiguration.html#cfn-kinesisfirehose-deliverystream-processingconfiguration-processors
         */
        Processors: C.Yields<C.CFList<C.Yields<AWS.KinesisFirehose.DeliveryStream.Processor>>>;
    }
}
export declare namespace AWS.KinesisFirehose.DeliveryStream {
    interface Processor {
        /**
         *                             Type: List of Amazon Kinesis Firehose DeliveryStream ProcessorParameter
         *
         * The processor parameters.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-processor.html#cfn-kinesisfirehose-deliverystream-processor-parameters
         */
        Parameters: C.Yields<C.CFList<C.Yields<AWS.KinesisFirehose.DeliveryStream.ProcessorParameter>>>;
        /**
         * String

         *
         * The type of processor. Valid values: Lambda.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-processor.html#cfn-kinesisfirehose-deliverystream-processor-type
         */
        Type: C.YieldsString;
    }
}
export declare namespace AWS.KinesisFirehose.DeliveryStream {
    interface ProcessorParameter {
        /**
         * String

         *
         * The name of the parameter.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-processorparameter.html#cfn-kinesisfirehose-deliverystream-processorparameter-parametername
         */
        ParameterName: C.YieldsString;
        /**
         * String

         *
         * The parameter value.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-processorparameter.html#cfn-kinesisfirehose-deliverystream-processorparameter-parametervalue
         */
        ParameterValue: C.YieldsString;
    }
}
export declare namespace AWS.KinesisFirehose.DeliveryStream {
    interface RedshiftDestinationConfiguration {
        /**
         * Kinesis Firehose DeliveryStream CloudWatchLoggingOptions
         *
         * The Amazon CloudWatch Logs logging options for the delivery stream.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-redshiftdestinationconfiguration.html#cfn-kinesisfirehose-deliverystream-redshiftdestinationconfiguration-cloudwatchloggingoptions
         */
        CloudWatchLoggingOptions?: C.Yields<AWS.KinesisFirehose.DeliveryStream.CloudWatchLoggingOptions>;
        /**
         * String

         *
         * The connection string that Kinesis Firehose uses to connect to the Amazon Redshift
 cluster.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-redshiftdestinationconfiguration.html#cfn-kinesisfirehose-deliverystream-redshiftdestinationconfiguration-clusterjdbcurl
         */
        ClusterJDBCURL: C.YieldsString;
        /**
         * Kinesis Firehose DeliveryStream CopyCommand
         *
         * Configures the Amazon Redshift COPY command that Kinesis Firehose uses to load data into the cluster from the Amazon
 S3 bucket.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-redshiftdestinationconfiguration.html#cfn-kinesisfirehose-deliverystream-redshiftdestinationconfiguration-copycommand
         */
        CopyCommand: C.Yields<AWS.KinesisFirehose.DeliveryStream.CopyCommand>;
        /**
         * String

         *
         * The password for the Amazon Redshift user that you specified in the Username property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-redshiftdestinationconfiguration.html#cfn-kinesisfirehose-deliverystream-redshiftdestinationconfiguration-password
         */
        Password: C.YieldsString;
        /**
         *                             Type: Kinesis Firehose DeliveryStream ProcessingConfiguration
         *
         * The data processing configuration for the Kinesis Firehose delivery stream.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-redshiftdestinationconfiguration.html#cfn-kinesisfirehose-deliverystream-redshiftdestinationconfiguration-processingconfiguration
         */
        ProcessingConfiguration?: C.Yields<AWS.KinesisFirehose.DeliveryStream.ProcessingConfiguration>;
        /**
         * String

         *
         * The ARN of the AWS Identity and Access Management (IAM) role that grants Kinesis Firehose
 access to your Amazon S3 bucket and AWS KMS (if you enable data encryption).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-redshiftdestinationconfiguration.html#cfn-kinesisfirehose-deliverystream-redshiftdestinationconfiguration-rolearn
         */
        RoleARN: C.YieldsString;
        /**
         * Kinesis Firehose DeliveryStream S3DestinationConfiguration
         *
         * The S3 bucket where Kinesis Firehose first delivers data. After the data is in the
 bucket, Kinesis Firehose uses the COPY command to load the data into the Amazon Redshift cluster.
 For the Amazon S3 bucket's compression format, don't specify SNAPPY or ZIP because the Amazon Redshift COPY command doesn't support them.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-redshiftdestinationconfiguration.html#cfn-kinesisfirehose-deliverystream-redshiftdestinationconfiguration-s3configuration
         */
        S3Configuration: C.Yields<AWS.KinesisFirehose.DeliveryStream.S3DestinationConfiguration>;
        /**
         * String

         *
         * The Amazon Redshift user that has permission to access the Amazon Redshift cluster.
 This user must have INSERT privileges for copying data from the Amazon S3 bucket to the cluster.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-redshiftdestinationconfiguration.html#cfn-kinesisfirehose-deliverystream-redshiftdestinationconfiguration-username
         */
        Username: C.YieldsString;
    }
}
export declare namespace AWS.KinesisFirehose.DeliveryStream {
    interface S3DestinationConfiguration {
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the Amazon S3 bucket to send data to.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-s3destinationconfiguration.html#cfn-kinesisfirehose-deliverystream-s3destinationconfiguration-bucketarn
         */
        BucketARN: C.YieldsString;
        /**
         * Kinesis Firehose DeliveryStream BufferingHints
         *
         * Configures how Kinesis Firehose buffers incoming data while delivering it to the Amazon
 S3 bucket.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-s3destinationconfiguration.html#cfn-kinesisfirehose-deliverystream-s3destinationconfiguration-bufferinghints
         */
        BufferingHints: C.Yields<AWS.KinesisFirehose.DeliveryStream.BufferingHints>;
        /**
         * Amazon Kinesis Firehose DeliveryStream CloudWatchLoggingOptions
         *
         * The Amazon CloudWatch Logs logging options for the delivery stream.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-s3destinationconfiguration.html#cfn-kinesisfirehose-deliverystream-s3destinationconfiguration-cloudwatchloggingoptions
         */
        CloudWatchLoggingOptions?: C.Yields<AWS.KinesisFirehose.DeliveryStream.CloudWatchLoggingOptions>;
        /**
         * String

         *
         * The type of compression that  Kinesis Firehose uses to compress the data that it delivers
 to the Amazon S3 bucket.
 For valid values, see the CompressionFormat content for the S3DestinationConfiguration data type in the Amazon Kinesis Firehose API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-s3destinationconfiguration.html#cfn-kinesisfirehose-deliverystream-s3destinationconfiguration-compressionformat
         */
        CompressionFormat: C.YieldsString;
        /**
         * Amazon Kinesis Firehose DeliveryStream EncryptionConfiguration
         *
         * Configures Amazon Simple Storage Service (Amazon S3) server-side encryption. Kinesis
 Firehose uses AWS Key Management Service (AWS KMS) to encrypt the data that it delivers
 to your Amazon S3 bucket.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-s3destinationconfiguration.html#cfn-kinesisfirehose-deliverystream-s3destinationconfiguration-encryptionconfiguration
         */
        EncryptionConfiguration?: C.Yields<AWS.KinesisFirehose.DeliveryStream.EncryptionConfiguration>;
        /**
         * String

         *
         * A prefix that Kinesis Firehose adds to the files that it delivers to the Amazon S3
 bucket. The prefix helps you identify the files that Kinesis Firehose delivered.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-s3destinationconfiguration.html#cfn-kinesisfirehose-deliverystream-s3destinationconfiguration-prefix
         */
        Prefix: C.YieldsString;
        /**
         * String

         *
         * The ARN of an AWS Identity and Access Management (IAM) role that grants Kinesis Firehose
 access to your Amazon S3 bucket and AWS KMS (if you enable data encryption).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisfirehose-deliverystream-s3destinationconfiguration.html#cfn-kinesisfirehose-deliverystream-s3destinationconfiguration-rolearn
         */
        RoleARN: C.YieldsString;
    }
}
export declare namespace AWS.Lambda {
    /**
     * The AWS::Lambda::Alias resource creates an alias that points to the version of an AWS Lambda (Lambda) function
    that you specify. Use aliases when you want to control which version of your function
    other services or applications invoke. Those services or applications can use your
    function's alias so that they don't need to be updated whenever you release a new
    version of your function. For more information, see Introduction to
       AWS Lambda Aliases in the AWS Lambda Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-alias.html
     */
    interface Alias {
        Type: 'AWS::Lambda::Alias';
        Properties: AWS.Lambda.Alias.Properties;
    }
    /**
     * The AWS::Lambda::Alias resource creates an alias that points to the version of an AWS Lambda (Lambda) function
    that you specify. Use aliases when you want to control which version of your function
    other services or applications invoke. Those services or applications can use your
    function's alias so that they don't need to be updated whenever you release a new
    version of your function. For more information, see Introduction to
       AWS Lambda Aliases in the AWS Lambda Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-alias.html
     */
    function Alias(props: C.Omit<Alias, 'Type'>): Alias;
}
export declare namespace AWS.Lambda.Alias {
    interface Properties {
        /**
         * String

         *
         * Information about the alias, such as its purpose or the Lambda function that is associated
 with it.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-alias.html#cfn-lambda-alias-description
         */
        Description?: C.YieldsString;
        /**
         * String

         *
         * The Lambda function that you want to associate with this alias. You can specify the
 function's name or its Amazon Resource Name (ARN).

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-alias.html#cfn-lambda-alias-functionname
         */
        FunctionName: C.YieldsString;
        /**
         * String

         *
         * The version of the Lambda function that you want to associate with this alias.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-alias.html#cfn-lambda-alias-functionversion
         */
        FunctionVersion: C.YieldsString;
        /**
         * String

         *
         * A name for the alias.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-alias.html#cfn-lambda-alias-name
         */
        Name: C.YieldsString;
    }
}
export declare namespace AWS.Lambda {
    /**
     * The AWS::Lambda::EventSourceMapping resource specifies a stream as an event
    source for an AWS Lambda (Lambda) function. The stream can be an Kinesis stream or
    an Amazon DynamoDB
    (DynamoDB) stream. Lambda invokes the associated function when records are posted
    to the stream.
    For more information, see CreateEventSourceMapping in the AWS Lambda Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-eventsourcemapping.html
     */
    interface EventSourceMapping {
        Type: 'AWS::Lambda::EventSourceMapping';
        Properties: AWS.Lambda.EventSourceMapping.Properties;
    }
    /**
     * The AWS::Lambda::EventSourceMapping resource specifies a stream as an event
    source for an AWS Lambda (Lambda) function. The stream can be an Kinesis stream or
    an Amazon DynamoDB
    (DynamoDB) stream. Lambda invokes the associated function when records are posted
    to the stream.
    For more information, see CreateEventSourceMapping in the AWS Lambda Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-eventsourcemapping.html
     */
    function EventSourceMapping(props: C.Omit<EventSourceMapping, 'Type'>): EventSourceMapping;
}
export declare namespace AWS.Lambda.EventSourceMapping {
    interface Properties {
        /**
         * Integer

         *
         * The largest number of records that Lambda retrieves from your event source when
 invoking your function. Your function receives an event with all the retrieved
 records. For the default and valid values, see CreateEventSourceMapping in the
 AWS Lambda Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-eventsourcemapping.html#cfn-lambda-eventsourcemapping-batchsize
         */
        BatchSize?: C.Yields<C.CFInteger>;
        /**
         * Boolean

         *
         * Indicates whether Lambda begins polling the event source.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-eventsourcemapping.html#cfn-lambda-eventsourcemapping-enabled
         */
        Enabled?: C.YieldsBoolean;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the Kinesis or DynamoDB stream that is the source
 of events. Any record added to this stream can invoke the Lambda function. For more
 information, see CreateEventSourceMapping in the
 AWS Lambda Developer Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-eventsourcemapping.html#cfn-lambda-eventsourcemapping-eventsourcearn
         */
        EventSourceArn: C.YieldsString;
        /**
         * String

         *
         * The name or ARN of a Lambda function to invoke when Lambda detects an event on
 the stream.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-eventsourcemapping.html#cfn-lambda-eventsourcemapping-functionname
         */
        FunctionName: C.YieldsString;
        /**
         * String

         *
         * The position in the stream where Lambda starts reading. For valid values, see
 CreateEventSourceMapping in the
 AWS Lambda Developer Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-eventsourcemapping.html#cfn-lambda-eventsourcemapping-startingposition
         */
        StartingPosition: C.YieldsString;
    }
}
export declare namespace AWS.Lambda {
    /**
     * The AWS::Lambda::Function resource creates an AWS Lambda (Lambda) function that
    can run code in response to events. For more information, see CreateFunction in the
    AWS Lambda Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-function.html
     */
    interface Function {
        Type: 'AWS::Lambda::Function';
        Properties: AWS.Lambda.Function.Properties;
    }
    /**
     * The AWS::Lambda::Function resource creates an AWS Lambda (Lambda) function that
    can run code in response to events. For more information, see CreateFunction in the
    AWS Lambda Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-function.html
     */
    function Function(props: C.Omit<Function, 'Type'>): Function;
}
export declare namespace AWS.Lambda.Function {
    interface Properties {
        /**
         * AWS Lambda Function Code
         *
         * The source code of your Lambda function. You can point to a file in an Amazon Simple
 Storage Service
 (Amazon S3) bucket or specify your source code as inline text.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-function.html#cfn-lambda-function-code
         */
        Code: C.Yields<AWS.Lambda.Function.Code>;
        /**
         * AWS Lambda Function
    DeadLetterConfig
         *
         * Configures how Lambda handles events that it can't process. If you don't specify
 a Dead Letter Queue (DLQ) configuration, Lambda discards events after the maximum
 number of retries. For more information, see Dead Letter Queues in the
 AWS Lambda Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-function.html#cfn-lambda-function-deadletterconfig
         */
        DeadLetterConfig?: C.Yields<AWS.Lambda.Function.DeadLetterConfig>;
        /**
         * String

         *
         * A description of the function.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-function.html#cfn-lambda-function-description
         */
        Description?: C.YieldsString;
        /**
         * AWS Lambda Function
    Environment
         *
         * Key-value pairs that Lambda caches and makes available for your Lambda functions.
 Use environment variables to apply configuration changes, such as test and
 production environment configurations, without changing your Lambda function source
 code.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-function.html#cfn-lambda-function-environment
         */
        Environment?: C.Yields<AWS.Lambda.Function.Environment>;
        /**
         * String

         *
         * A name for the function. If you don't specify a name, AWS CloudFormation generates
 a unique physical ID and uses that ID for the function's name. For more information,
 see Name Type.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-function.html#cfn-lambda-function-functionname
         */
        FunctionName?: C.YieldsString;
        /**
         * String

         *
         * The name of the function (within your source code) that Lambda calls to start running
 your code. For more information, see the Handler property
 in the AWS Lambda Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-function.html#cfn-lambda-function-handler
         */
        Handler: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of an AWS Key Management Service (AWS KMS) key that
 Lambda uses to
 encrypt and decrypt environment variable values.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-function.html#cfn-lambda-function-kmskeyarn
         */
        KmsKeyArn?: C.YieldsString;
        /**
         * Integer

         *
         * The amount of memory, in MB, that is allocated to your Lambda function. Lambda
 uses this value to proportionally allocate the amount of CPU power. For more
 information, see Resource
    Model in the AWS Lambda Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-function.html#cfn-lambda-function-memorysize
         */
        MemorySize?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) execution
 role that
 Lambda assumes when it runs your code to access AWS services.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-function.html#cfn-lambda-function-role
         */
        Role: C.YieldsString;
        /**
         * String

         *
         * The runtime environment for the Lambda function that you are uploading. For valid
 values, see the Runtime property in the AWS Lambda Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-function.html#cfn-lambda-function-runtime
         */
        Runtime: C.YieldsString;
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key–value pairs) for this Lambda function.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-function.html#cfn-lambda-function-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * Integer

         *
         * The function execution time (in seconds) after which Lambda terminates the
 function. Because the execution time affects cost, set this value based on the
 function's expected execution time. By default,  Timeout is set to
 3 seconds. For more information, see the FAQs.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-function.html#cfn-lambda-function-timeout
         */
        Timeout?: C.Yields<C.CFInteger>;
        /**
         * AWS Lambda Function TracingConfig
         *
         * The parent object that contains your Lambda function's tracing settings. By default,
 the Mode property is set to PassThrough. For valid values, see the TracingConfig data type in the AWS Lambda Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-function.html#cfn-lambda-function-tracingconfig
         */
        TracingConfig?: C.Yields<AWS.Lambda.Function.TracingConfig>;
        /**
         * AWS Lambda Function VpcConfig
         *
         * If the Lambda function requires access to resources in a VPC, specify a VPC configuration
 that Lambda uses to set up an elastic network interface (ENI). The ENI enables your
 function to connect to other resources in your VPC, but it doesn't provide public
 Internet access. If your function requires Internet access (for example, to access
 AWS services that don't have VPC endpoints), configure a Network Address Translation
 (NAT) instance inside your VPC or use an Amazon Virtual Private Cloud
 (Amazon VPC) NAT gateway. For more information, see NAT Gateways in the Amazon VPC User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-function.html#cfn-lambda-function-vpcconfig
         */
        VpcConfig?: C.Yields<AWS.Lambda.Function.VpcConfig>;
    }
}
export declare namespace AWS.Lambda.Function {
    interface Code {
        /**
         * String

         *
         * The name of the S3 bucket that contains the source code of your Lambda function. The
 S3 bucket must be in the same region as the stack.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lambda-function-code.html#cfn-lambda-function-code-s3bucket
         */
        S3Bucket?: C.YieldsString;
        /**
         * String

         *
         * The location and name of the .zip file that contains your source code. If you specify this property, you must also
 specify the S3Bucket property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lambda-function-code.html#cfn-lambda-function-code-s3key
         */
        S3Key?: C.YieldsString;
        /**
         * String

         *
         * If you have S3 versioning enabled, the version ID of the.zip file that contains your source code. You can specify this property only if you specify
 the S3Bucket and S3Key properties.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lambda-function-code.html#cfn-lambda-function-code-s3objectversion
         */
        S3ObjectVersion?: C.YieldsString;
        /**
         * String

         *
         * For nodejs4.3, nodejs6.10, python2.7, and python3.6 runtime environments, the
 source code of your Lambda function. You can't use this property with other runtime
 environments.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lambda-function-code.html#cfn-lambda-function-code-zipfile
         */
        ZipFile?: C.YieldsString;
    }
}
export declare namespace AWS.Lambda.Function {
    interface DeadLetterConfig {
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of a resource where Lambda delivers unprocessed
 events, such as an Amazon SNS topic or Amazon Simple Queue Service (Amazon SQS) queue.
 For the Lambda function execution role, you must explicitly provide the relevant permissions so that
 access to your DLQ resource is part of the execution role for your Lambda function.
 

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lambda-function-deadletterconfig.html#cfn-lambda-function-deadletterconfig-targetarn
         */
        TargetArn?: C.YieldsString;
    }
}
export declare namespace AWS.Lambda.Function {
    interface Environment {
        /**
         * Mapping of key-value pairs

         *
         * A map of key-value pairs that the Lambda function can access.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lambda-function-environment.html#cfn-lambda-function-environment-variables
         */
        Variables?: C.Yields<C.CFMap<C.YieldsString>>;
    }
}
export declare namespace AWS.Lambda.Function {
    interface TracingConfig {
        /**
         * String

         *
         * Specifies how Lambda traces a request. The default mode is PassThrough. For more information, see TracingConfig in the AWS Lambda Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lambda-function-tracingconfig.html#cfn-lambda-function-tracingconfig-mode
         */
        Mode?: C.YieldsString;
    }
}
export declare namespace AWS.Lambda.Function {
    interface VpcConfig {
        /**
         * List of strings

         *
         * A list of one or more security groups IDs in the VPC that includes the resources to
 which your Lambda function requires access.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lambda-function-vpcconfig.html#cfn-lambda-function-vpcconfig-securitygroupids
         */
        SecurityGroupIds: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of strings

         *
         * A list of one or more subnet IDs in the VPC that includes the resources to which your
 Lambda function requires access.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lambda-function-vpcconfig.html#cfn-lambda-function-vpcconfig-subnetids
         */
        SubnetIds: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.Lambda {
    /**
     * The AWS::Lambda::Permission resource associates a policy statement with a
    specific AWS Lambda (Lambda) function's access policy. The function policy grants
    a specific AWS
    service or application permission to invoke the function. For more information, see
    AddPermission in the
    AWS Lambda Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-permission.html
     */
    interface Permission {
        Type: 'AWS::Lambda::Permission';
        Properties: AWS.Lambda.Permission.Properties;
    }
    /**
     * The AWS::Lambda::Permission resource associates a policy statement with a
    specific AWS Lambda (Lambda) function's access policy. The function policy grants
    a specific AWS
    service or application permission to invoke the function. For more information, see
    AddPermission in the
    AWS Lambda Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-permission.html
     */
    function Permission(props: C.Omit<Permission, 'Type'>): Permission;
}
export declare namespace AWS.Lambda.Permission {
    interface Properties {
        /**
         * String

         *
         * The Lambda actions that you want to allow in this statement. For example, you can
 specify lambda:CreateFunction to specify a certain action, or use a wildcard (lambda:*) to grant permission to all Lambda actions. For a list of actions, see Actions and Condition Context Keys for AWS Lambda in the IAM User Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-permission.html#cfn-lambda-permission-action
         */
        Action: C.YieldsString;
        /**
         * String

         *
         * A unique token that must be supplied by the principal invoking the function.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-permission.html#cfn-lambda-permission-eventsourcetoken
         */
        EventSourceToken?: C.YieldsString;
        /**
         * String

         *
         * The name (physical ID), Amazon Resource Name (ARN), or alias ARN of the Lambda
 function that you want to associate with this statement. Lambda adds this statement
 to the function's access policy.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-permission.html#cfn-lambda-permission-functionname
         */
        FunctionName: C.YieldsString;
        /**
         * String

         *
         * The entity for which you are granting permission to invoke the Lambda function.
 This entity can be any valid AWS service principal, such as
 s3.amazonaws.com or sns.amazonaws.com, or, if you are
 granting cross-account permission, an AWS account ID. For example, you might want
 to allow a custom application in another AWS account to push events to Lambda by
 invoking your function.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-permission.html#cfn-lambda-permission-principal
         */
        Principal: C.YieldsString;
        /**
         * String

         *
         * The AWS account ID (without hyphens) of the source owner. For example, if you
 specify an S3 bucket in the SourceArn property, this value is the
 bucket owner's account ID. You can use this property to ensure that all source
 principals are owned by a specific account.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-permission.html#cfn-lambda-permission-sourceaccount
         */
        SourceAccount?: C.YieldsString;
        /**
         * String

         *
         * The ARN of a resource that is invoking your function. When granting Amazon Simple
 Storage Service
 (Amazon S3) permission to invoke your function, specify this property with the bucket
 ARN as its value. This ensures that events generated only from the specified
 bucket, not just any bucket from any AWS account that creates a mapping to your
 function, can invoke the function.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-permission.html#cfn-lambda-permission-sourcearn
         */
        SourceArn?: C.YieldsString;
    }
}
export declare namespace AWS.Lambda {
    /**
     * The AWS::Lambda::Version resource publishes a specified version of an AWS Lambda (Lambda) function. When publishing
    a new version of your function, Lambda copies the latest version of your function.
    For more information, see Introduction to AWS Lambda Versioning in the AWS Lambda Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-version.html
     */
    interface Version {
        Type: 'AWS::Lambda::Version';
        Properties: AWS.Lambda.Version.Properties;
    }
    /**
     * The AWS::Lambda::Version resource publishes a specified version of an AWS Lambda (Lambda) function. When publishing
    a new version of your function, Lambda copies the latest version of your function.
    For more information, see Introduction to AWS Lambda Versioning in the AWS Lambda Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-version.html
     */
    function Version(props: C.Omit<Version, 'Type'>): Version;
}
export declare namespace AWS.Lambda.Version {
    interface Properties {
        /**
         * String

         *
         * The SHA-256 hash of the deployment package that you want to publish. This value must
 match the SHA-256 hash of the $LATEST version of the function. Specify this property to validate that you are publishing
 the correct package.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-version.html#cfn-lambda-version-codesha256
         */
        CodeSha256?: C.YieldsString;
        /**
         * String

         *
         * A description of the version you are publishing. If you don't specify a value, Lambda
 copies the description from the $LATEST version of the function.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-version.html#cfn-lambda-version-description
         */
        Description?: C.YieldsString;
        /**
         * String

         *
         * The Lambda function for which you want to publish a version. You can specify the function's
 name or its Amazon Resource Name (ARN).

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lambda-version.html#cfn-lambda-version-functionname
         */
        FunctionName: C.YieldsString;
    }
}
export declare namespace AWS.Logs {
    /**
     * The AWS::Logs::Destination resource creates an Amazon CloudWatch Logs (CloudWatch Logs) destination,
    which enables you to specify a physical resource (such as an Kinesis stream) that
    subscribes to
    CloudWatch Logs log events from another AWS account. For more information, see Cross-Account Log Data Sharing with
       Subscriptions in the Amazon CloudWatch User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-destination.html
     */
    interface Destination {
        Type: 'AWS::Logs::Destination';
        Properties: AWS.Logs.Destination.Properties;
    }
    /**
     * The AWS::Logs::Destination resource creates an Amazon CloudWatch Logs (CloudWatch Logs) destination,
    which enables you to specify a physical resource (such as an Kinesis stream) that
    subscribes to
    CloudWatch Logs log events from another AWS account. For more information, see Cross-Account Log Data Sharing with
       Subscriptions in the Amazon CloudWatch User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-destination.html
     */
    function Destination(props: C.Omit<Destination, 'Type'>): Destination;
}
export declare namespace AWS.Logs.Destination {
    interface Properties {
        /**
         * String

         *
         * The name of the CloudWatch Logs destination.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-destination.html#cfn-logs-destination-destinationname
         */
        DestinationName: C.YieldsString;
        /**
         * String

         *
         * An AWS Identity and Access Management (IAM) policy that specifies who can write to
 your
 destination.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-destination.html#cfn-logs-destination-destinationpolicy
         */
        DestinationPolicy: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of an IAM role that permits CloudWatch Logs to send
 data
 to the specified AWS resource (TargetArn).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-destination.html#cfn-logs-destination-rolearn
         */
        RoleArn: C.YieldsString;
        /**
         * String

         *
         * The ARN of the AWS resource that receives log events. Currently, you can
 specify only an Kinesis stream.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-destination.html#cfn-logs-destination-targetarn
         */
        TargetArn: C.YieldsString;
    }
}
export declare namespace AWS.Logs {
    /**
     * The AWS::Logs::LogGroup resource creates an Amazon CloudWatch Logs log group that defines
    common properties for log streams, such as their retention and access control rules.
    Each log
    stream must belong to one log group.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-loggroup.html
     */
    interface LogGroup {
        Type: 'AWS::Logs::LogGroup';
        Properties: AWS.Logs.LogGroup.Properties;
    }
    /**
     * The AWS::Logs::LogGroup resource creates an Amazon CloudWatch Logs log group that defines
    common properties for log streams, such as their retention and access control rules.
    Each log
    stream must belong to one log group.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-loggroup.html
     */
    function LogGroup(props: C.Omit<LogGroup, 'Type'>): LogGroup;
}
export declare namespace AWS.Logs.LogGroup {
    interface Properties {
        /**
         * String

         *
         * A name for the log group. If you don't specify a name, AWS CloudFormation generates
 a unique
 physical ID and uses that ID for the log group. For more information, see Name Type.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-loggroup.html#cfn-cwl-loggroup-loggroupname
         */
        LogGroupName?: C.YieldsString;
        /**
         * Integer

         *
         * The number of days log events are kept in CloudWatch Logs. When a log event expires,
 CloudWatch Logs automatically deletes it. For valid values, see PutRetentionPolicy in
 the Amazon CloudWatch Logs API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-loggroup.html#cfn-cwl-loggroup-retentionindays
         */
        RetentionInDays?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.Logs {
    /**
     * The AWS::Logs::LogStream resource creates an Amazon CloudWatch Logs log stream in a log
    group. A log stream represents the sequence of events coming from an application instance
    or
    resource that you are monitoring. For more information, see Monitoring Log Files in the
    Amazon CloudWatch User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-logstream.html
     */
    interface LogStream {
        Type: 'AWS::Logs::LogStream';
        Properties: AWS.Logs.LogStream.Properties;
    }
    /**
     * The AWS::Logs::LogStream resource creates an Amazon CloudWatch Logs log stream in a log
    group. A log stream represents the sequence of events coming from an application instance
    or
    resource that you are monitoring. For more information, see Monitoring Log Files in the
    Amazon CloudWatch User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-logstream.html
     */
    function LogStream(props: C.Omit<LogStream, 'Type'>): LogStream;
}
export declare namespace AWS.Logs.LogStream {
    interface Properties {
        /**
         * String

         *
         * The name of the log group where the log stream is created.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-logstream.html#cfn-logs-logstream-loggroupname
         */
        LogGroupName: C.YieldsString;
        /**
         * String

         *
         * The name of the log stream to create. The name must be unique within the log
 group.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-logstream.html#cfn-logs-logstream-logstreamname
         */
        LogStreamName?: C.YieldsString;
    }
}
export declare namespace AWS.Logs {
    /**
     * The AWS::Logs::MetricFilter resource creates a metric filter that describes
    how Amazon CloudWatch Logs extracts information from logs that you specify and transforms
    it into Amazon CloudWatch
    metrics. If you have multiple metric filters that are associated with a log group,
    all the
    filters are applied to the log streams in that group.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-metricfilter.html
     */
    interface MetricFilter {
        Type: 'AWS::Logs::MetricFilter';
        Properties: AWS.Logs.MetricFilter.Properties;
    }
    /**
     * The AWS::Logs::MetricFilter resource creates a metric filter that describes
    how Amazon CloudWatch Logs extracts information from logs that you specify and transforms
    it into Amazon CloudWatch
    metrics. If you have multiple metric filters that are associated with a log group,
    all the
    filters are applied to the log streams in that group.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-metricfilter.html
     */
    function MetricFilter(props: C.Omit<MetricFilter, 'Type'>): MetricFilter;
}
export declare namespace AWS.Logs.MetricFilter {
    interface Properties {
        /**
         * String

         *
         * Describes the pattern that CloudWatch Logs follows to interpret each entry in a log.
 A log
 entry might contain fields such as timestamps, IP addresses, error
 codes, bytes transferred, and so on. You use the pattern to specify those fields
 and to specify what to look for in the log file. For example, if you're interested
 in error codes that begin with 1234, your filter pattern might be
 [timestamps, ip_addresses, error_codes = 1234*, size, ...].
 For more information, see
    Filter and Pattern Syntax in the Amazon CloudWatch User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-metricfilter.html#cfn-cwl-metricfilter-filterpattern
         */
        FilterPattern: C.YieldsString;
        /**
         * String

         *
         * The name of an existing log group that you want to associate with this metric
 filter.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-metricfilter.html#cfn-cwl-metricfilter-loggroupname
         */
        LogGroupName: C.YieldsString;
        /**
         * A list of CloudWatch Logs MetricFilter
    MetricTransformation Property
         *
         * Describes how to transform data from a log into a CloudWatch metric.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-metricfilter.html#cfn-cwl-metricfilter-metrictransformations
         */
        MetricTransformations: C.Yields<C.CFList<C.Yields<AWS.Logs.MetricFilter.MetricTransformation>>>;
    }
}
export declare namespace AWS.Logs.MetricFilter {
    interface MetricTransformation {
        /**
         * String

         *
         * The name of the CloudWatch metric to which the log information will be
 published.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-logs-metricfilter-metrictransformation.html#cfn-cwl-metricfilter-metrictransformation-metricname
         */
        MetricName: C.YieldsString;
        /**
         * String

         *
         * The destination namespace of the CloudWatch metric. Namespaces are containers for
 metrics. For example, you can add related metrics in the same namespace.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-logs-metricfilter-metrictransformation.html#cfn-cwl-metricfilter-metrictransformation-metricnamespace
         */
        MetricNamespace: C.YieldsString;
        /**
         * String

         *
         * The value that is published to the CloudWatch metric. For example, if you're counting
 the occurrences of a particular term like Error, specify
 1 for the metric value. If you're counting the number of bytes
 transferred, reference the value that is in the log event by using $
 followed by the name of the field that you specified in the filter pattern, such
 as $size.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-logs-metricfilter-metrictransformation.html#cfn-cwl-metricfilter-metrictransformation-metricvalue
         */
        MetricValue: C.YieldsString;
    }
}
export declare namespace AWS.Logs {
    /**
     * The AWS::Logs::SubscriptionFilter resource creates an Amazon CloudWatch Logs (CloudWatch Logs)
    subscription filter that defines which log events are delivered to your Kinesis stream
    or
    AWS Lambda (Lambda) function and where to send them.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-subscriptionfilter.html
     */
    interface SubscriptionFilter {
        Type: 'AWS::Logs::SubscriptionFilter';
        Properties: AWS.Logs.SubscriptionFilter.Properties;
    }
    /**
     * The AWS::Logs::SubscriptionFilter resource creates an Amazon CloudWatch Logs (CloudWatch Logs)
    subscription filter that defines which log events are delivered to your Kinesis stream
    or
    AWS Lambda (Lambda) function and where to send them.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-subscriptionfilter.html
     */
    function SubscriptionFilter(props: C.Omit<SubscriptionFilter, 'Type'>): SubscriptionFilter;
}
export declare namespace AWS.Logs.SubscriptionFilter {
    interface Properties {
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the Kinesis stream or Lambda function that you
 want to use as the subscription feed destination.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-subscriptionfilter.html#cfn-cwl-subscriptionfilter-destinationarn
         */
        DestinationArn: C.YieldsString;
        /**
         * String

         *
         * The filtering expressions that restrict what gets delivered to the destination
 AWS resource. For more information about the filter pattern syntax, see Filter and Pattern
    Syntax in the Amazon CloudWatch User Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-subscriptionfilter.html#cfn-cwl-subscriptionfilter-filterpattern
         */
        FilterPattern: C.YieldsString;
        /**
         * String

         *
         * The log group to associate with the subscription filter. All log events that
 are uploaded to this log group are filtered and delivered to the specified AWS
 resource if the filter pattern matches the log events.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-subscriptionfilter.html#cfn-cwl-subscriptionfilter-loggroupname
         */
        LogGroupName: C.YieldsString;
        /**
         * String

         *
         * An IAM role that grants CloudWatch Logs permission to put data into the specified
 Kinesis stream. For Lambda and CloudWatch Logs destinations, don't specify this property
 because CloudWatch Logs gets the necessary permissions from the destination resource.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-logs-subscriptionfilter.html#cfn-cwl-subscriptionfilter-rolearn
         */
        RoleArn?: C.YieldsString;
    }
}
export declare namespace AWS.OpsWorks {
    /**
     * Defines an AWS OpsWorks app for an AWS OpsWorks stack. The app specifies the code
    that you want to run on
    an application server.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-app.html
     */
    interface App {
        Type: 'AWS::OpsWorks::App';
        Properties: AWS.OpsWorks.App.Properties;
    }
    /**
     * Defines an AWS OpsWorks app for an AWS OpsWorks stack. The app specifies the code
    that you want to run on
    an application server.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-app.html
     */
    function App(props: C.Omit<App, 'Type'>): App;
}
export declare namespace AWS.OpsWorks.App {
    interface Properties {
        /**
         * AWS OpsWorks Source Type
         *
         * The information required to retrieve an app from a repository.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-app.html#cfn-opsworks-app-appsource
         */
        AppSource?: C.Yields<AWS.OpsWorks.App.Source>;
        /**
         * A list of key-value pairs

         *
         * One or more user-defined key-value pairs to be added to the app attributes
 bag.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-app.html#cfn-opsworks-app-attributes
         */
        Attributes?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         * List of DataSource
         *
         * A list of databases to associate with the AWS OpsWorks app.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-app.html#cfn-opsworks-app-datasources
         */
        DataSources?: C.Yields<C.CFList<C.Yields<AWS.OpsWorks.App.DataSource>>>;
        /**
         * String

         *
         * A description of the app.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-app.html#cfn-opsworks-app-description
         */
        Description?: C.YieldsString;
        /**
         * List of strings

         *
         * The app virtual host settings, with multiple domains separated by commas. For
 example, 'www.example.com, example.com'.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-app.html#cfn-opsworks-app-domains
         */
        Domains?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * Boolean

         *
         * Whether to enable SSL for this app.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-app.html#cfn-opsworks-app-enablessl
         */
        EnableSsl?: C.YieldsBoolean;
        /**
         * List of AWS OpsWorks App Environment
         *
         * The environment variables to associate with the AWS OpsWorks app.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-app.html#cfn-opsworks-app-environment
         */
        Environment?: C.Yields<C.CFList<C.Yields<AWS.OpsWorks.App.EnvironmentVariable>>>;
        /**
         * String

         *
         * The name of the AWS OpsWorks app.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-app.html#cfn-opsworks-app-name
         */
        Name: C.YieldsString;
        /**
         * String

         *
         * The app short name, which is used internally by AWS OpsWorks and by Chef
 recipes.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-app.html#cfn-opsworks-app-shortname
         */
        Shortname?: C.YieldsString;
        /**
         * AWS OpsWorks SslConfiguration
    Type
         *
         * The SSL configuration
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-app.html#cfn-opsworks-app-sslconfiguration
         */
        SslConfiguration?: C.Yields<AWS.OpsWorks.App.SslConfiguration>;
        /**
         * String

         *
         * The ID of the AWS OpsWorks stack to associate this app with.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-app.html#cfn-opsworks-app-stackid
         */
        StackId: C.YieldsString;
        /**
         * String

         *
         * The app type. Each supported type is associated with a particular layer. For
 more information, see CreateApp in the AWS OpsWorks Stacks API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-app.html#cfn-opsworks-app-type
         */
        Type: C.YieldsString;
    }
}
export declare namespace AWS.OpsWorks.App {
    interface DataSource {
        /**
         * String

         *
         * The ARN of the data source.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-app-datasource.html#cfn-opsworks-app-datasource-arn
         */
        Arn?: C.YieldsString;
        /**
         * String

         *
         * The name of the database.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-app-datasource.html#cfn-opsworks-app-datasource-databasename
         */
        DatabaseName?: C.YieldsString;
        /**
         * String

         *
         * The type of the data source, such as
 AutoSelectOpsworksMysqlInstance,
 OpsworksMysqlInstance, or RdsDbInstance. For valid
 values, see the DataSource
 type in the AWS OpsWorks Stacks API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-app-datasource.html#cfn-opsworks-app-datasource-type
         */
        Type?: C.YieldsString;
    }
}
export declare namespace AWS.OpsWorks.App {
    interface EnvironmentVariable {
        /**
         * String

         *
         * The name of the environment variable, which can consist of up to 64 characters.
 You can use upper and lowercase letters, numbers, and underscores (_), but the
 name must start with a letter or underscore.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-app-environment.html#cfn-opsworks-app-environment-key
         */
        Key: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether the value of the environment variable is concealed, such as
 with a DescribeApps
 response. To conceal an environment variable's value, set the value to
 true.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-app-environment.html#cfn-opsworks-app-environment-secure
         */
        Secure?: C.YieldsBoolean;
        /**
         * String

         *
         * The value of the environment variable, which can be empty. You can specify a
 value of up to 256 characters.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-app-environment.html#value
         */
        Value: C.YieldsString;
    }
}
export declare namespace AWS.OpsWorks.App {
    interface Source {
        /**
         * String

         *
         * This parameter depends on the repository type. For Amazon S3 bundles, set
 Password to the appropriate IAM secret access key. For HTTP
 bundles, Git repositories, and Subversion repositories, set Password
 to the appropriate password.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-source.html#cfn-opsworks-custcookbooksource-pw
         */
        Password?: C.YieldsString;
        /**
         * String

         *
         * The application's version. With AWS OpsWorks, you can deploy new versions of an
 application. One of the simplest approaches is to have branches or revisions in
 your repository that represent different versions that can potentially be
 deployed.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-source.html#cfn-opsworks-custcookbooksource-revision
         */
        Revision?: C.YieldsString;
        /**
         * String

         *
         * The repository's SSH key. For more information, see Using Git Repository SSH
    Keys in the AWS OpsWorks User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-source.html#cfn-opsworks-custcookbooksource-sshkey
         */
        SshKey?: C.YieldsString;
        /**
         * String

         *
         * The repository type.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-source.html#cfn-opsworks-custcookbooksource-type
         */
        Type?: C.YieldsString;
        /**
         * String

         *
         * The source URL.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-source.html#cfn-opsworks-custcookbooksource-url
         */
        Url?: C.YieldsString;
        /**
         * String

         *
         * This parameter depends on the repository type. For Amazon S3 bundles, set
 Username to the appropriate IAM access key ID. For HTTP bundles,
 Git repositories, and Subversion repositories, set Username to the
 appropriate user name.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-source.html#cfn-opsworks-custcookbooksource-username
         */
        Username?: C.YieldsString;
    }
}
export declare namespace AWS.OpsWorks.App {
    interface SslConfiguration {
        /**
         * String

         *
         * The contents of the certificate's domain.crt file.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-app-sslconfiguration.html#cfn-opsworks-app-sslconfig-certificate
         */
        Certificate?: C.YieldsString;
        /**
         * String

         *
         * An intermediate certificate authority key or client authentication.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-app-sslconfiguration.html#cfn-opsworks-app-sslconfig-chain
         */
        Chain?: C.YieldsString;
        /**
         * String

         *
         * The private key; the contents of the certificate's
 domain.kex file.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-app-sslconfiguration.html#cfn-opsworks-app-sslconfig-privatekey
         */
        PrivateKey?: C.YieldsString;
    }
}
export declare namespace AWS.OpsWorks {
    /**
     * Attaches an Elastic Load Balancing load balancer to an AWS OpsWorks layer that you
    specify.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-elbattachment.html
     */
    interface ElasticLoadBalancerAttachment {
        Type: 'AWS::OpsWorks::ElasticLoadBalancerAttachment';
        Properties: AWS.OpsWorks.ElasticLoadBalancerAttachment.Properties;
    }
    /**
     * Attaches an Elastic Load Balancing load balancer to an AWS OpsWorks layer that you
    specify.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-elbattachment.html
     */
    function ElasticLoadBalancerAttachment(props: C.Omit<ElasticLoadBalancerAttachment, 'Type'>): ElasticLoadBalancerAttachment;
}
export declare namespace AWS.OpsWorks.ElasticLoadBalancerAttachment {
    interface Properties {
        /**
         * String

         *
         * Elastic Load Balancing load balancer name.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-elbattachment.html#cfn-opsworks-elbattachment-elbname
         */
        ElasticLoadBalancerName: C.YieldsString;
        /**
         * String

         *
         * The AWS OpsWorks layer ID that the Elastic Load Balancing load balancer will be attached
 to.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-elbattachment.html#cfn-opsworks-elbattachment-layerid
         */
        LayerId: C.YieldsString;
    }
}
export declare namespace AWS.OpsWorks {
    /**
     * Creates an Amazon Elastic Compute Cloud (Amazon EC2) instance for an AWS OpsWorks
    stack. Instances for  AWS OpsWorks stacks
    handle the work of serving applications and balancing traffic, for example.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html
     */
    interface Instance {
        Type: 'AWS::OpsWorks::Instance';
        Properties: AWS.OpsWorks.Instance.Properties;
    }
    /**
     * Creates an Amazon Elastic Compute Cloud (Amazon EC2) instance for an AWS OpsWorks
    stack. Instances for  AWS OpsWorks stacks
    handle the work of serving applications and balancing traffic, for example.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html
     */
    function Instance(props: C.Omit<Instance, 'Type'>): Instance;
}
export declare namespace AWS.OpsWorks.Instance {
    interface Properties {
        /**
         * String

         *
         * The version of the  AWS OpsWorks agent that AWS OpsWorks installs on each instance.
 AWS OpsWorks
 sends commands to the agent to performs tasks on your instances, such as starting
 Chef runs. For valid values, see the AgentVersion parameter for the
 CreateInstance action
 in the AWS OpsWorks Stacks API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html#cfn-opsworks-instance-agentversion
         */
        AgentVersion?: C.YieldsString;
        /**
         * String

         *
         * The ID of the custom Amazon Machine Image (AMI) to be used to create the
 instance. For more information about custom AMIs, see Using Custom AMIs in the
 AWS OpsWorks User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html#cfn-opsworks-instance-amiid
         */
        AmiId?: C.YieldsString;
        /**
         * String

         *
         * The instance architecture.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html#cfn-opsworks-instance-architecture
         */
        Architecture?: C.YieldsString;
        /**
         * String

         *
         * For scaling instances, the type of scaling. If you specify load-based scaling,
 do not specify a time-based scaling configuration. For valid values, see CreateInstance in the
 AWS OpsWorks Stacks API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html#cfn-opsworks-instance-autoscalingtype
         */
        AutoScalingType?: C.YieldsString;
        /**
         * String

         *
         * The instance Availability Zone.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html#cfn-opsworks-instance-availabilityzone
         */
        AvailabilityZone?: C.YieldsString;
        /**
         * List of AWS OpsWorks Instance
    BlockDeviceMapping
         *
         * A list of block devices that are mapped to the AWS OpsWorks instance. For more
 information, see the BlockDeviceMappings parameter for the CreateInstance action in the
 AWS OpsWorks Stacks API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html#cfn-opsworks-instance-blockdevicemappings
         */
        BlockDeviceMappings?: C.Yields<C.CFList<C.Yields<AWS.OpsWorks.Instance.BlockDeviceMapping>>>;
        /**
         * Boolean

         *
         * Whether the instance is optimized for Amazon Elastic Block Store (Amazon EBS) I/O.
 If you specify an
 Amazon EBS-optimized instance type, AWS OpsWorks enables EBS optimization by default.
 For more
 information, see Amazon EBS–Optimized
    Instances in the Amazon EC2 User Guide for Linux Instances.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html#cfn-opsworks-instance-ebsoptimized
         */
        EbsOptimized?: C.YieldsBoolean;
        /**
         * List of strings

         *
         * A list of Elastic IP addresses to associate with the instance.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html#cfn-opsworks-instance-elasticips
         */
        ElasticIps?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The name of the instance host.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html#cfn-opsworks-instance-hostname
         */
        Hostname?: C.YieldsString;
        /**
         * Boolean

         *
         * Whether to install operating system and package updates when the instance
 boots.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html#cfn-opsworks-instance-installupdatesonboot
         */
        InstallUpdatesOnBoot?: C.YieldsBoolean;
        /**
         * String

         *
         * The instance type, which must be supported by AWS OpsWorks. For more information,
 see
 CreateInstance in the
 AWS OpsWorks Stacks API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html#cfn-opsworks-instance-instancetype
         */
        InstanceType: C.YieldsString;
        /**
         * List of strings

         *
         * The IDs of the AWS OpsWorks layers to associate with this instance.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html#cfn-opsworks-instance-layerids
         */
        LayerIds: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The instance operating system. For more information, see CreateInstance in the
 AWS OpsWorks Stacks API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html#cfn-opsworks-instance-os
         */
        Os?: C.YieldsString;
        /**
         * String

         *
         * The root device type of the instance.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html#cfn-opsworks-instance-rootdevicetype
         */
        RootDeviceType?: C.YieldsString;
        /**
         * String

         *
         * The SSH key name of the instance.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html#cfn-opsworks-instance-sshkeyname
         */
        SshKeyName?: C.YieldsString;
        /**
         * String

         *
         * The ID of the AWS OpsWorks stack that this instance will be associated with.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html#cfn-opsworks-instance-stackid
         */
        StackId: C.YieldsString;
        /**
         * String

         *
         * The ID of the instance's subnet. If the stack is running in a VPC, you can use
 this parameter to override the stack's default subnet ID value and direct AWS OpsWorks
 to
 launch the instance in a different subnet.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html#cfn-opsworks-instance-subnetid
         */
        SubnetId?: C.YieldsString;
        /**
         * String

         *
         * The tenancy of the instance. For more information, see the Tenancy
 parameter for the CreateInstance action in the
 AWS OpsWorks Stacks API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html#cfn-opsworks-instance-tenancy
         */
        Tenancy?: C.YieldsString;
        /**
         * AWS OpsWorks
    TimeBasedAutoScaling Type
         *
         * The time-based scaling configuration for the instance.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html#cfn-opsworks-instance-timebasedautoscaling
         */
        TimeBasedAutoScaling?: C.Yields<AWS.OpsWorks.Instance.TimeBasedAutoScaling>;
        /**
         * String

         *
         * The instance's virtualization type, paravirtual or
 hvm.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html#cfn-opsworks-instance-virtualizationtype
         */
        VirtualizationType?: C.YieldsString;
        /**
         * List of strings

         *
         * A list of AWS OpsWorks volume IDs to associate with the instance. For more
 information, see AWS::OpsWorks::Volume.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-instance.html#cfn-opsworks-instance-volumes
         */
        Volumes?: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.OpsWorks.Instance {
    interface BlockDeviceMapping {
        /**
         * String

         *
         * The name of the device that is exposed to the instance, such as
 /dev/dsh or xvdh. For the root device, you can use
 the explicit device name or you can set this parameter to
 ROOT_DEVICE. If you set the parameter to ROOT_DEVICE,
 AWS OpsWorks provides the correct device name.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-instance-blockdevicemapping.html#cfn-opsworks-instance-blockdevicemapping-devicename
         */
        DeviceName?: C.YieldsString;
        /**
         * AWS OpsWorks
    Instance BlockDeviceMapping EbsBlockDevice
         *
         * Configuration information about the Amazon Elastic Block Store (Amazon EBS) volume.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-instance-blockdevicemapping.html#cfn-opsworks-instance-blockdevicemapping-ebs
         */
        Ebs?: C.Yields<AWS.OpsWorks.Instance.EbsBlockDevice>;
        /**
         * String

         *
         * Suppresses the device that is specified in the block device mapping of the
 AWS OpsWorks instance Amazon Machine Image (AMI).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-instance-blockdevicemapping.html#cfn-opsworks-instance-blockdevicemapping-nodevice
         */
        NoDevice?: C.YieldsString;
        /**
         * String

         *
         * The name of the virtual device. The name must be in the form
 ephemeralX, where
 X is a number equal to or greater than zero (0), for
 example, ephemeral0.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-instance-blockdevicemapping.html#cfn-opsworks-instance-blockdevicemapping-virtualname
         */
        VirtualName?: C.YieldsString;
    }
}
export declare namespace AWS.OpsWorks.Instance {
    interface EbsBlockDevice {
        /**
         * Boolean

         *
         * Indicates whether to delete the volume when the instance is terminated.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-instance-ebsblockdevice.html#cfn-opsworks-instance-ebsblockdevice-deleteontermination
         */
        DeleteOnTermination?: C.YieldsBoolean;
        /**
         * Integer

         *
         * The number of I/O operations per second (IOPS) that the volume supports. For
 more information, see Iops for the EbsBlockDevice action in the
 Amazon EC2 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-instance-ebsblockdevice.html#cfn-opsworks-instance-ebsblockdevice-iops
         */
        Iops?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The snapshot ID of the volume that you want to use. If you specify both the
 SnapshotId and VolumeSize properties,
 VolumeSize must be equal to or greater than the size of the
 snapshot.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-instance-ebsblockdevice.html#cfn-opsworks-instance-ebsblockdevice-snapshotid
         */
        SnapshotId?: C.YieldsString;
        /**
         * Integer

         *
         * The volume size, in Gibibytes (GiB). If you specify both the
 SnapshotId and VolumeSize properties,
 VolumeSize must be equal to or greater than the size of the
 snapshot. For more information about specifying volume size, see VolumeSize for the
 EbsBlockDevice action in the
 Amazon EC2 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-instance-ebsblockdevice.html#cfn-opsworks-instance-ebsblockdevice-volumesize
         */
        VolumeSize?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The volume type. For more information about specifying the volume type, see
 VolumeType for the
 EbsBlockDevice action in the
 Amazon EC2 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-instance-ebsblockdevice.html#cfn-opsworks-instance-ebsblockdevice-volumetype
         */
        VolumeType?: C.YieldsString;
    }
}
export declare namespace AWS.OpsWorks.Instance {
    interface TimeBasedAutoScaling {
        /**
         * String to string map

         *
         * The schedule for Friday.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-instance-timebasedautoscaling.html#cfn-opsworks-instance-timebasedautoscaling-friday
         */
        Friday?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         * String to string map

         *
         * The schedule for Monday.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-instance-timebasedautoscaling.html#cfn-opsworks-instance-timebasedautoscaling-monday
         */
        Monday?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         * String to string map

         *
         * The schedule for Saturday.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-instance-timebasedautoscaling.html#cfn-opsworks-instance-timebasedautoscaling-saturday
         */
        Saturday?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         * String to string map

         *
         * The schedule for Sunday.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-instance-timebasedautoscaling.html#cfn-opsworks-instance-timebasedautoscaling-sunday
         */
        Sunday?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         * String to string map

         *
         * The schedule for Thursday.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-instance-timebasedautoscaling.html#cfn-opsworks-instance-timebasedautoscaling-thursday
         */
        Thursday?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         * String to string map

         *
         * The schedule for Tuesday.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-instance-timebasedautoscaling.html#cfn-opsworks-instance-timebasedautoscaling-tuesday
         */
        Tuesday?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         * String to string map

         *
         * The schedule for Wednesday.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-instance-timebasedautoscaling.html#cfn-opsworks-instance-timebasedautoscaling-wednesday
         */
        Wednesday?: C.Yields<C.CFMap<C.YieldsString>>;
    }
}
export declare namespace AWS.OpsWorks {
    /**
     * Creates an AWS OpsWorks layer. A layer defines, for example, which packages and applications
    are
    installed and how they are configured.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-layer.html
     */
    interface Layer {
        Type: 'AWS::OpsWorks::Layer';
        Properties: AWS.OpsWorks.Layer.Properties;
    }
    /**
     * Creates an AWS OpsWorks layer. A layer defines, for example, which packages and applications
    are
    installed and how they are configured.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-layer.html
     */
    function Layer(props: C.Omit<Layer, 'Type'>): Layer;
}
export declare namespace AWS.OpsWorks.Layer {
    interface Properties {
        /**
         * A list of key-value pairs

         *
         * One or more user-defined key-value pairs to be added to the stack attributes
 bag.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-layer.html#cfn-opsworks-layer-attributes
         */
        Attributes?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         * Boolean

         *
         * Whether to automatically assign an Elastic IP address to Amazon EC2 instances in
 this layer.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-layer.html#cfn-opsworks-layer-autoassignelasticips
         */
        AutoAssignElasticIps: C.YieldsBoolean;
        /**
         * Boolean

         *
         * For AWS OpsWorks stacks that are running in a VPC, whether to automatically assign
 a
 public IP address to Amazon EC2 instances in this layer.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-layer.html#cfn-opsworks-layer-autoassignpublicips
         */
        AutoAssignPublicIps: C.YieldsBoolean;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of an IAM instance profile that is to be used
 for the Amazon EC2 instances in this layer.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-layer.html#cfn-opsworks-layer-custominstanceprofilearn
         */
        CustomInstanceProfileArn?: C.YieldsString;
        /**
         * JSON object

         *
         * A custom stack configuration and deployment attributes that AWS OpsWorks installs
 on
 the layer's instances. For more information, see the CustomJson
 parameter for the CreateLayer action in the
 AWS OpsWorks Stacks API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-layer.html#cfn-opsworks-layer-customjson
         */
        CustomJson?: C.Yields<C.CFJson>;
        /**
         * AWS OpsWorks Recipes Type
         *
         * Custom event recipes for this layer.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-layer.html#cfn-opsworks-layer-customrecipes
         */
        CustomRecipes?: C.Yields<AWS.OpsWorks.Layer.Recipes>;
        /**
         * List of strings

         *
         * Custom security group IDs for this layer.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-layer.html#cfn-opsworks-layer-customsecuritygroupids
         */
        CustomSecurityGroupIds?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * Boolean

         *
         * Whether to automatically heal Amazon EC2 instances that have become disconnected or
 timed out.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-layer.html#cfn-opsworks-layer-enableautohealing
         */
        EnableAutoHealing: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Whether to install operating system and package updates when the instance
 boots.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-layer.html#cfn-opsworks-layer-installupdatesonboot
         */
        InstallUpdatesOnBoot?: C.YieldsBoolean;
        /**
         * AWS OpsWorks Layer
    LifeCycleConfiguration
         *
         * The lifecycle events for the AWS OpsWorks layer.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-layer.html#cfn-opsworks-layer-lifecycleeventconfiguration
         */
        LifecycleEventConfiguration?: C.Yields<AWS.OpsWorks.Layer.LifecycleEventConfiguration>;
        /**
         * AWS OpsWorks LoadBasedAutoScaling
    Type
         *
         * The load-based scaling configuration for the AWS OpsWorks layer.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-layer.html#cfn-opsworks-layer-loadbasedautoscaling
         */
        LoadBasedAutoScaling?: C.Yields<AWS.OpsWorks.Layer.LoadBasedAutoScaling>;
        /**
         * String

         *
         * The AWS OpsWorks layer name.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-layer.html#cfn-opsworks-layer-name
         */
        Name: C.YieldsString;
        /**
         * List of strings

         *
         * The packages for this layer.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-layer.html#cfn-opsworks-layer-packages
         */
        Packages?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The layer short name, which is used internally by AWS OpsWorks and by Chef recipes.
 The short name is also used as the name for the directory where your app files are
 installed.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-layer.html#cfn-opsworks-layer-shortname
         */
        Shortname: C.YieldsString;
        /**
         * String

         *
         * The ID of the AWS OpsWorks stack that this layer will be associated with.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-layer.html#cfn-opsworks-layer-stackid
         */
        StackId: C.YieldsString;
        /**
         * String

         *
         * The layer type. A stack cannot have more than one layer of the same type,
 except for the custom type. You can have any number of
 custom types. For more information, see CreateLayer in the
 AWS OpsWorks Stacks API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-layer.html#cfn-opsworks-layer-type
         */
        Type: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-layer.html#cfn-opsworks-layer-useebsoptimizedinstances
         */
        UseEbsOptimizedInstances?: C.YieldsBoolean;
        /**
         * A list of AWS OpsWorks VolumeConfiguration
    Type
         *
         * Describes the Amazon EBS volumes for this layer.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-layer.html#cfn-opsworks-layer-volumeconfigurations
         */
        VolumeConfigurations?: C.Yields<C.CFList<C.Yields<AWS.OpsWorks.Layer.VolumeConfiguration>>>;
    }
}
export declare namespace AWS.OpsWorks.Layer {
    interface AutoScalingThresholds {
        /**
         * Number

         *
         * The percentage of CPU utilization that triggers the starting or stopping of
 instances (scaling).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-loadbasedautoscaling-autoscalingthresholds.html#cfn-opsworks-layer-loadbasedautoscaling-autoscalingthresholds-cputhreshold
         */
        CpuThreshold?: C.Yields<C.CFDouble>;
        /**
         * Integer

         *
         * The amount of time (in minutes) after a scaling event occurs that AWS OpsWorks should
 ignore metrics and not start any additional scaling events.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-loadbasedautoscaling-autoscalingthresholds.html#cfn-opsworks-layer-loadbasedautoscaling-autoscalingthresholds-ignoremetricstime
         */
        IgnoreMetricsTime?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The number of instances to add or remove when the load exceeds a
 threshold.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-loadbasedautoscaling-autoscalingthresholds.html#cfn-opsworks-layer-loadbasedautoscaling-autoscalingthresholds-instancecount
         */
        InstanceCount?: C.Yields<C.CFInteger>;
        /**
         * Number

         *
         * The degree of system load that triggers the starting or stopping of instances
 (scaling). For more information about how load is computed, see Load
    (computing).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-loadbasedautoscaling-autoscalingthresholds.html#cfn-opsworks-layer-loadbasedautoscaling-autoscalingthresholds-loadthreshold
         */
        LoadThreshold?: C.Yields<C.CFDouble>;
        /**
         * Number

         *
         * The percentage of memory consumption that triggers the starting or stopping of
 instances (scaling).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-loadbasedautoscaling-autoscalingthresholds.html#cfn-opsworks-layer-loadbasedautoscaling-autoscalingthresholds-memorythreshold
         */
        MemoryThreshold?: C.Yields<C.CFDouble>;
        /**
         * Integer

         *
         * The amount of time, in minutes, that the load must exceed a threshold before
 instances are added or removed.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-loadbasedautoscaling-autoscalingthresholds.html#cfn-opsworks-layer-loadbasedautoscaling-autoscalingthresholds-thresholdwaittime
         */
        ThresholdsWaitTime?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.OpsWorks.Layer {
    interface LifecycleEventConfiguration {
        /**
         * AWS OpsWorks Layer LifeCycleConfiguration ShutdownEventConfiguration
         *
         * Specifies the shutdown event configuration for a layer.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-lifecycleeventconfiguration.html#cfn-opsworks-layer-lifecycleconfiguration-shutdowneventconfiguration
         */
        ShutdownEventConfiguration?: C.Yields<AWS.OpsWorks.Layer.ShutdownEventConfiguration>;
    }
}
export declare namespace AWS.OpsWorks.Layer {
    interface LoadBasedAutoScaling {
        /**
         * AWS OpsWorks
    AutoScalingThresholds Type
         *
         * The threshold below which the instances are scaled down (stopped). If the load
 falls below this threshold for a specified amount of time, AWS OpsWorks stops a specified
 number of instances.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-loadbasedautoscaling.html#cfn-opsworks-layer-loadbasedautoscaling-downscaling
         */
        DownScaling?: C.Yields<AWS.OpsWorks.Layer.AutoScalingThresholds>;
        /**
         * Boolean

         *
         * Whether to enable automatic load-based scaling for the layer.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-loadbasedautoscaling.html#cfn-opsworks-layer-loadbasedautoscaling-enable
         */
        Enable?: C.YieldsBoolean;
        /**
         * AWS OpsWorks
    AutoScalingThresholds Type
         *
         * The threshold above which the instances are scaled up (added). If the load
 exceeds this thresholds for a specified amount of time, AWS OpsWorks starts a specified
 number of instances.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-loadbasedautoscaling.html#cfn-opsworks-layer-loadbasedautoscaling-upscaling
         */
        UpScaling?: C.Yields<AWS.OpsWorks.Layer.AutoScalingThresholds>;
    }
}
export declare namespace AWS.OpsWorks.Layer {
    interface Recipes {
        /**
         * List of strings

         *
         * Custom recipe names to be run following a Configure event. The event occurs on
 all of the stack's instances when an instance enters or leaves the online
 state.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-recipes.html#cfn-opsworks-layer-customrecipes-configure
         */
        Configure?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of strings

         *
         * Custom recipe names to be run following a Deploy event. The event occurs when
 you run a deploy command, typically to deploy an application to a set
 of application server instances.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-recipes.html#cfn-opsworks-layer-customrecipes-deploy
         */
        Deploy?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of strings

         *
         * Custom recipe names to be run following a Setup event. This event
 occurs on a new instance after it successfully boots.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-recipes.html#cfn-opsworks-layer-customrecipes-setup
         */
        Setup?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of strings

         *
         * Custom recipe names to be run following a Shutdown event. This
 event occurs after you direct AWS OpsWorks to shut an instance down before the associated
 Amazon EC2 instance is actually terminated.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-recipes.html#cfn-opsworks-layer-customrecipes-shutdown
         */
        Shutdown?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of strings

         *
         * Custom recipe names to be run following a Undeploy event. This
 event occurs when you delete an app or run an undeploy command to
 remove an app from a set of application server instances.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-recipes.html#cfn-opsworks-layer-customrecipes-undeploy
         */
        Undeploy?: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.OpsWorks.Layer {
    interface ShutdownEventConfiguration {
        /**
         * Boolean

         *
         * Indicates whether to wait for connections to drain from the Elastic Load Balancing
 load
 balancers.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-lifecycleeventconfiguration-shutdowneventconfiguration.html#cfn-opsworks-layer-lifecycleconfiguration-shutdowneventconfiguration-delayuntilelbconnectionsdrained
         */
        DelayUntilElbConnectionsDrained?: C.YieldsBoolean;
        /**
         * Integer

         *
         * The time, in seconds, that AWS OpsWorks waits after a shutdown event has been
 triggered before shutting down an instance.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-lifecycleeventconfiguration-shutdowneventconfiguration.html#cfn-opsworks-layer-lifecycleconfiguration-shutdowneventconfiguration-executiontimeout
         */
        ExecutionTimeout?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.OpsWorks.Layer {
    interface VolumeConfiguration {
        /**
         * Integer

         *
         * The number of I/O operations per second (IOPS) to provision for the
 volume.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-volumeconfiguration.html#cfn-opsworks-layer-volconfig-iops
         */
        Iops?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The volume mount point, such as /dev/sdh.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-volumeconfiguration.html#cfn-opsworks-layer-volconfig-mountpoint
         */
        MountPoint?: C.YieldsString;
        /**
         * Integer

         *
         * The number of disks in the volume.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-volumeconfiguration.html#cfn-opsworks-layer-volconfig-numberofdisks
         */
        NumberOfDisks?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The volume RAID level.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-volumeconfiguration.html#cfn-opsworks-layer-volconfig-raidlevel
         */
        RaidLevel?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The volume size.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-volumeconfiguration.html#cfn-opsworks-layer-volconfig-size
         */
        Size?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The type of volume, such as magnetic or SSD. For valid values, see VolumeConfiguration in
 the AWS OpsWorks Stacks API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-layer-volumeconfiguration.html#cfn-opsworks-layer-volconfig-volumetype
         */
        VolumeType?: C.YieldsString;
    }
}
export declare namespace AWS.OpsWorks {
    /**
     * Creates an AWS OpsWorks stack. An AWS OpsWorks stack represents a set of instances
    that you want to
    manage collectively, typically because they have a common purpose such as serving
    PHP
    applications.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html
     */
    interface Stack {
        Type: 'AWS::OpsWorks::Stack';
        Properties: AWS.OpsWorks.Stack.Properties;
    }
    /**
     * Creates an AWS OpsWorks stack. An AWS OpsWorks stack represents a set of instances
    that you want to
    manage collectively, typically because they have a common purpose such as serving
    PHP
    applications.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html
     */
    function Stack(props: C.Omit<Stack, 'Type'>): Stack;
}
export declare namespace AWS.OpsWorks.Stack {
    interface Properties {
        /**
         * String

         *
         * The AWS OpsWorks agent version that you want to use. The agent communicates with the
 service and handles tasks such as initiating Chef runs in response to lifecycle
 events. For valid values, see the AgentVersion parameter for the CreateStack action in the
 AWS OpsWorks Stacks API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html#cfn-opsworks-stack-agentversion
         */
        AgentVersion?: C.YieldsString;
        /**
         * A list of key-value pairs

         *
         * One or more user-defined key-value pairs to be added to the stack attributes
 bag.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html#cfn-opsworks-stack-attributes
         */
        Attributes?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         * AWS OpsWorks ChefConfiguration
    Type
         *
         * Describes the Chef configuration. For more information, see the CreateStack ChefConfiguration parameter in the
 AWS OpsWorks Stacks API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html#cfn-opsworks-stack-chefconfiguration
         */
        ChefConfiguration?: C.Yields<AWS.OpsWorks.Stack.ChefConfiguration>;
        /**
         * List of strings

         *
         * If you're cloning an AWS OpsWorks stack, a list of AWS OpsWorks application stack
 IDs from the
 source stack to include in the cloned stack.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html#cfn-opsworks-stack-cloneappids
         */
        CloneAppIds?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * Boolean

         *
         * If you're cloning an AWS OpsWorks stack, indicates whether to clone the source stack's
 permissions.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html#cfn-opsworks-stack-clonepermissions
         */
        ClonePermissions?: C.YieldsBoolean;
        /**
         * AWS OpsWorks
    StackConfigurationManager Type
         *
         * Describes the configuration manager. When you create a stack, you use the
 configuration manager to specify the Chef version. For supported Chef versions,
 see the CreateStack ConfigurationManager parameter in the
 AWS OpsWorks Stacks API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html#cfn-opsworks-stack-configmanager
         */
        ConfigurationManager?: C.Yields<AWS.OpsWorks.Stack.StackConfigurationManager>;
        /**
         * AWS OpsWorks Source Type
         *
         * Contains the information required to retrieve a cookbook from a
 repository.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html#cfn-opsworks-stack-custcookbooksource
         */
        CustomCookbooksSource?: C.Yields<AWS.OpsWorks.Stack.Source>;
        /**
         * JSON object

         *
         * A user-defined custom JSON object. The custom JSON is used to override the
 corresponding default stack configuration JSON values. For more information, see
 CreateStack in the
 AWS OpsWorks Stacks API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html#cfn-opsworks-stack-custjson
         */
        CustomJson?: C.Yields<C.CFJson>;
        /**
         * String

         *
         * The stack's default Availability Zone, which must be in the specified
 region.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html#cfn-opsworks-stack-defaultaz
         */
        DefaultAvailabilityZone?: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of an IAM instance profile that is the default
 profile for all of the stack's Amazon EC2 instances.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html#cfn-opsworks-stack-defaultinstanceprof
         */
        DefaultInstanceProfileArn: C.YieldsString;
        /**
         * String

         *
         * The stack's default operating system. For more information, see CreateStack in the
 AWS OpsWorks Stacks API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html#cfn-opsworks-stack-defaultos
         */
        DefaultOs?: C.YieldsString;
        /**
         * String

         *
         * The default root device type. This value is used by default for all instances
 in the stack, but you can override it when you create an instance. For more
 information, see CreateStack in the AWS OpsWorks Stacks API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html#cfn-opsworks-stack-defaultrootdevicetype
         */
        DefaultRootDeviceType?: C.YieldsString;
        /**
         * String

         *
         * A default SSH key for the stack instances. You can override this value when you
 create or update an instance.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html#cfn-opsworks-stack-defaultsshkeyname
         */
        DefaultSshKeyName?: C.YieldsString;
        /**
         * String

         *
         * The stack's default subnet ID. All instances are launched into this subnet
 unless you specify another subnet ID when you create the instance.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html#defaultsubnet
         */
        DefaultSubnetId?: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the Amazon EC2 Container Service (Amazon ECS) cluster
 to register
 with the AWS OpsWorks stack.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html#cfn-opsworks-stack-ecsclusterarn
         */
        EcsClusterArn?: C.YieldsString;
        /**
         * List of AWS OpsWorks Stack ElasticIp
         *
         * A list of Elastic IP addresses to register with the AWS OpsWorks stack.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html#cfn-opsworks-stack-elasticips
         */
        ElasticIps?: C.Yields<C.CFList<C.Yields<AWS.OpsWorks.Stack.ElasticIp>>>;
        /**
         * String

         *
         * The stack's host name theme, with spaces replaced by underscores. The theme is
 used to generate host names for the stack's instances. For more information, see
 CreateStack in the
 AWS OpsWorks Stacks API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html#cfn-opsworks-stack-hostnametheme
         */
        HostnameTheme?: C.YieldsString;
        /**
         * String

         *
         * The name of the AWS OpsWorks stack.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html#cfn-opsworks-stack-name
         */
        Name: C.YieldsString;
        /**
         * List of AWS OpsWorks Stack RdsDbInstance
         *
         * The Amazon Relational Database Service (Amazon RDS) DB instance to register with the
 AWS OpsWorks stack.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html#cfn-opsworks-stack-rdsdbinstances
         */
        RdsDbInstances?: C.Yields<C.CFList<C.Yields<AWS.OpsWorks.Stack.RdsDbInstance>>>;
        /**
         * String

         *
         * The AWS Identity and Access Management (IAM) role that AWS OpsWorks uses to work with
 AWS resources on your
 behalf. You must specify an Amazon Resource Name (ARN) for an existing IAM
 role.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html#cfn-opsworks-stack-servicerolearn
         */
        ServiceRoleArn: C.YieldsString;
        /**
         * String

         *
         * If you're cloning an AWS OpsWorks stack, the stack ID of the source AWS OpsWorks stack
 to
 clone.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html#cfn-opsworks-stack-sourcestackid
         */
        SourceStackId?: C.YieldsString;
        /**
         * Boolean

         *
         * Whether the stack uses custom cookbooks.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html#usecustcookbooks
         */
        UseCustomCookbooks?: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Whether to associate the AWS OpsWorks built-in security groups with the stack's
 layers.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html#cfn-opsworks-stack-useopsworkssecuritygroups
         */
        UseOpsworksSecurityGroups?: C.YieldsBoolean;
        /**
         * String

         *
         * The ID of the VPC that the stack is to be launched into, which must be in the
 specified region. All instances are launched into this VPC. If you specify this
 property, you must specify the DefaultSubnetId property.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-stack.html#cfn-opsworks-stack-vpcid
         */
        VpcId?: C.YieldsString;
    }
}
export declare namespace AWS.OpsWorks.Stack {
    interface ChefConfiguration {
        /**
         * String

         *
         * The Berkshelf version.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-chefconfiguration.html#cfn-opsworks-chefconfiguration-berkshelfversion
         */
        BerkshelfVersion?: C.YieldsString;
        /**
         * Boolean

         *
         * Whether to enable Berkshelf.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-chefconfiguration.html#cfn-opsworks-chefconfiguration-berkshelfversion
         */
        ManageBerkshelf?: C.YieldsBoolean;
    }
}
export declare namespace AWS.OpsWorks.Stack {
    interface ElasticIp {
        /**
         * String

         *
         * The Elastic IP address.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-elasticip.html#cfn-opsworks-stack-elasticip-ip
         */
        Ip: C.YieldsString;
        /**
         * String

         *
         * A name for the Elastic IP address.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-elasticip.html#cfn-opsworks-stack-elasticip-name
         */
        Name?: C.YieldsString;
    }
}
export declare namespace AWS.OpsWorks.Stack {
    interface RdsDbInstance {
        /**
         * String

         *
         * The password of the registered database.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-rdsdbinstance.html#cfn-opsworks-stack-rdsdbinstance-dbpassword
         */
        DbPassword: C.YieldsString;
        /**
         * String

         *
         * The master user name of the registered database.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-rdsdbinstance.html#cfn-opsworks-stack-rdsdbinstance-dbuser
         */
        DbUser: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the Amazon RDS DB instance to register with the
 AWS OpsWorks stack.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-rdsdbinstance.html#cfn-opsworks-stack-rdsdbinstance-rdsdbinstancearn
         */
        RdsDbInstanceArn: C.YieldsString;
    }
}
export declare namespace AWS.OpsWorks.Stack {
    interface Source {
        /**
         * String

         *
         * This parameter depends on the repository type. For Amazon S3 bundles, set
 Password to the appropriate IAM secret access key. For HTTP
 bundles, Git repositories, and Subversion repositories, set Password
 to the appropriate password.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-source.html#cfn-opsworks-custcookbooksource-password
         */
        Password?: C.YieldsString;
        /**
         * String

         *
         * The application's version. With AWS OpsWorks, you can deploy new versions of an
 application. One of the simplest approaches is to have branches or revisions in
 your repository that represent different versions that can potentially be
 deployed.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-source.html#cfn-opsworks-custcookbooksource-revision
         */
        Revision?: C.YieldsString;
        /**
         * String

         *
         * The repository's SSH key. For more information, see Using Git Repository SSH
    Keys in the AWS OpsWorks User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-source.html#cfn-opsworks-custcookbooksource-sshkey
         */
        SshKey?: C.YieldsString;
        /**
         * String

         *
         * The repository type.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-source.html#cfn-opsworks-custcookbooksource-type
         */
        Type?: C.YieldsString;
        /**
         * String

         *
         * The source URL.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-source.html#cfn-opsworks-custcookbooksource-url
         */
        Url?: C.YieldsString;
        /**
         * String

         *
         * This parameter depends on the repository type. For Amazon S3 bundles, set
 Username to the appropriate IAM access key ID. For HTTP bundles,
 Git repositories, and Subversion repositories, set Username to the
 appropriate user name.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-source.html#cfn-opsworks-custcookbooksource-username
         */
        Username?: C.YieldsString;
    }
}
export declare namespace AWS.OpsWorks.Stack {
    interface StackConfigurationManager {
        /**
         * String

         *
         * The name of the configuration manager.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-stackconfigmanager.html#cfn-opsworks-configmanager-name
         */
        Name?: C.YieldsString;
        /**
         * String

         *
         * The Chef version.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-opsworks-stack-stackconfigmanager.html#cfn-opsworks-configmanager-version
         */
        Version?: C.YieldsString;
    }
}
export declare namespace AWS.OpsWorks {
    /**
     * The AWS::OpsWorks::UserProfile resource configures SSH access for users who
    require access to instances in an AWS OpsWorks stack.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-userprofile.html
     */
    interface UserProfile {
        Type: 'AWS::OpsWorks::UserProfile';
        Properties: AWS.OpsWorks.UserProfile.Properties;
    }
    /**
     * The AWS::OpsWorks::UserProfile resource configures SSH access for users who
    require access to instances in an AWS OpsWorks stack.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-userprofile.html
     */
    function UserProfile(props: C.Omit<UserProfile, 'Type'>): UserProfile;
}
export declare namespace AWS.OpsWorks.UserProfile {
    interface Properties {
        /**
         * Boolean

         *
         * Indicates whether users can use the AWS OpsWorks My Settings page to
 specify their own SSH public key. For more information, see Setting an IAM User's Public
    SSH Key in the AWS OpsWorks User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-userprofile.html#cfn-opsworks-userprofile-allowselfmanagement
         */
        AllowSelfManagement?: C.YieldsBoolean;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the AWS Identity and Access Management (IAM) user
 to associate with
 this configuration.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-userprofile.html#cfn-opsworks-userprofile-iamuserarn
         */
        IamUserArn: C.YieldsString;
        /**
         * String

         *
         * The public SSH key that is associated with the IAM user. To access instances,
 the IAM user must have or be given the corresponding private key.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-userprofile.html#cfn-opsworks-userprofile-sshpublickey
         */
        SshPublicKey?: C.YieldsString;
        /**
         * String

         *
         * The user's SSH user name.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-userprofile.html#cfn-opsworks-userprofile-sshusername
         */
        SshUsername?: C.YieldsString;
    }
}
export declare namespace AWS.OpsWorks {
    /**
     * The AWS::OpsWorks::Volume resource registers an Amazon Elastic Block Store (Amazon EBS) volume with
    an AWS OpsWorks stack.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-volume.html
     */
    interface Volume {
        Type: 'AWS::OpsWorks::Volume';
        Properties: AWS.OpsWorks.Volume.Properties;
    }
    /**
     * The AWS::OpsWorks::Volume resource registers an Amazon Elastic Block Store (Amazon EBS) volume with
    an AWS OpsWorks stack.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-volume.html
     */
    function Volume(props: C.Omit<Volume, 'Type'>): Volume;
}
export declare namespace AWS.OpsWorks.Volume {
    interface Properties {
        /**
         * String

         *
         * The ID of the Amazon EBS volume to register with the AWS OpsWorks stack.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-volume.html#cfn-opsworks-volume-ec2volumeid
         */
        Ec2VolumeId: C.YieldsString;
        /**
         * String

         *
         * The mount point for the Amazon EBS volume, such as /mnt/disk1.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-volume.html#cfn-opsworks-volume-mountpoint
         */
        MountPoint?: C.YieldsString;
        /**
         * String

         *
         * A name for the Amazon EBS volume.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-volume.html#cfn-opsworks-volume-name
         */
        Name?: C.YieldsString;
        /**
         * String

         *
         * The ID of the AWS OpsWorks stack that AWS OpsWorks registers the volume to.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-opsworks-volume.html#cfn-opsworks-volume-stackid
         */
        StackId: C.YieldsString;
    }
}
export declare namespace AWS.RDS {
    /**
     * The AWS::RDS::DBCluster resource creates a cluster, such as an Aurora for
    Amazon RDS (Amazon Aurora) DB cluster. Amazon Aurora is a fully managed, MySQL-compatible,
    relational database engine. For more information, see Aurora on Amazon RDS in the
    Amazon Relational Database Service User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbcluster.html
     */
    interface DBCluster {
        Type: 'AWS::RDS::DBCluster';
        Properties: AWS.RDS.DBCluster.Properties;
    }
    /**
     * The AWS::RDS::DBCluster resource creates a cluster, such as an Aurora for
    Amazon RDS (Amazon Aurora) DB cluster. Amazon Aurora is a fully managed, MySQL-compatible,
    relational database engine. For more information, see Aurora on Amazon RDS in the
    Amazon Relational Database Service User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbcluster.html
     */
    function DBCluster(props: C.Omit<DBCluster, 'Type'>): DBCluster;
}
export declare namespace AWS.RDS.DBCluster {
    interface Properties {
        /**
         * String

         *
         * A list of Availability Zones (AZs) in which DB instances in the cluster can be
 created.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbcluster.html#cfn-rds-dbcluster-availabilityzones
         */
        AvailabilityZones?: C.YieldsString;
        /**
         * Integer

         *
         * The number of days for which automatic backups are retained. For more
 information, see CreateDBCluster in the Amazon Relational Database Service API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbcluster.html#cfn-rds-dbcluster-backuprententionperiod
         */
        BackupRetentionPeriod?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The name of the DB cluster parameter group to associate with this DB cluster.
 For the default value, see the DBClusterParameterGroupName parameter
 of the CreateDBCluster
 action in the Amazon Relational Database Service API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbcluster.html#cfn-rds-dbcluster-dbclusterparametergroupname
         */
        DBClusterParameterGroupName?: C.YieldsString;
        /**
         * String

         *
         * A DB subnet group that you want to associate with this DB cluster.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbcluster.html#cfn-rds-dbcluster-dbsubnetgroupname
         */
        DBSubnetGroupName?: C.YieldsString;
        /**
         * String

         *
         * The name of your database. You can specify a name of up to eight alpha-numeric
 characters. If you do not provide a name, Amazon Relational Database Service (Amazon
 RDS) won't create a
 database in this DB cluster.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbcluster.html#cfn-rds-dbcluster-databasename
         */
        DatabaseName?: C.YieldsString;
        /**
         * String

         *
         * The name of the database engine that you want to use for this DB
 cluster.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbcluster.html#cfn-rds-dbcluster-engine
         */
        Engine: C.YieldsString;
        /**
         * String

         *
         * The version number of the database engine that you want to use.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbcluster.html#cfn-rds-dbcluster-engineversion
         */
        EngineVersion?: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the AWS Key Management Service master key that is
 used to
 encrypt the database instances in the DB cluster, such as
 arn:aws:kms:us-east-1:012345678910:key/abcd1234-a123-456a-a12b-a123b4cd56ef.
 If you enable the StorageEncrypted property but don't specify this
 property, the default master key is used. If you specify this property, you must
 set the StorageEncrypted property to true.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbcluster.html#cfn-rds-dbcluster-kmskeyid
         */
        KmsKeyId?: C.YieldsString;
        /**
         * String

         *
         * The password for the master database user.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbcluster.html#cfn-rds-dbcluster-masteruserpassword
         */
        MasterUserPassword?: C.YieldsString;
        /**
         * String

         *
         * The master user name for the DB instance.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbcluster.html#cfn-rds-dbcluster-masterusername
         */
        MasterUsername?: C.YieldsString;
        /**
         * Integer

         *
         * The port number on which the DB instances in the cluster can accept
 connections.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbcluster.html#cfn-rds-dbcluster-port
         */
        Port?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * if automated backups are enabled (see the BackupRetentionPeriod
 property), the daily time range in UTC during which you want to create automated
 backups.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbcluster.html#cfn-rds-dbcluster-preferredbackupwindow
         */
        PreferredBackupWindow?: C.YieldsString;
        /**
         * String

         *
         * The weekly time range (in UTC) during which system maintenance can
 occur.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbcluster.html#cfn-rds-dbcluster-preferredmaintenancewindow
         */
        PreferredMaintenanceWindow?: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the source Amazon RDS MySQL DB instance or DB cluster,
 
 if this DB cluster is created as a Read Replica.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbcluster.html#cfn-rds-dbcluster-replicationsourceidentifier
         */
        ReplicationSourceIdentifier?: C.YieldsString;
        /**
         * String

         *
         * The identifier for the DB cluster snapshot from which you want to
 restore.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbcluster.html#cfn-rds-dbcluster-snapshotidentifier
         */
        SnapshotIdentifier?: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether the DB instances in the cluster are encrypted.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbcluster.html#cfn-rds-dbcluster-storageencrypted
         */
        StorageEncrypted?: C.YieldsBoolean;
        /**
         * A list of resource tags
         *
         * The tags that you want to attach to this DB cluster.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbcluster.html#cfn-rds-dbcluster-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * List of strings

         *
         * A list of VPC security groups to associate with this DB cluster.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbcluster.html#cfn-rds-dbcluster-vpcsecuritygroupids
         */
        VpcSecurityGroupIds?: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.RDS {
    /**
     * The AWS::RDS::DBClusterParameterGroup resource creates a new Amazon Relational Database Service (Amazon RDS)
    database (DB) cluster parameter group. For more information about DB cluster parameter
    groups,
    see Appendix: DB Cluster and
       DB Instance Parameters in the Amazon Relational Database Service User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbclusterparametergroup.html
     */
    interface DBClusterParameterGroup {
        Type: 'AWS::RDS::DBClusterParameterGroup';
        Properties: AWS.RDS.DBClusterParameterGroup.Properties;
    }
    /**
     * The AWS::RDS::DBClusterParameterGroup resource creates a new Amazon Relational Database Service (Amazon RDS)
    database (DB) cluster parameter group. For more information about DB cluster parameter
    groups,
    see Appendix: DB Cluster and
       DB Instance Parameters in the Amazon Relational Database Service User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbclusterparametergroup.html
     */
    function DBClusterParameterGroup(props: C.Omit<DBClusterParameterGroup, 'Type'>): DBClusterParameterGroup;
}
export declare namespace AWS.RDS.DBClusterParameterGroup {
    interface Properties {
        /**
         *
         *
         * A friendly description for this DB cluster parameter group.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbclusterparametergroup.html#cfn-rds-dbclusterparametergroup-description
         */
        Description: C.YieldsString;
        /**
         *
         *
         * The database family of this DB cluster parameter group, such as
 aurora5.6.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbclusterparametergroup.html#cfn-rds-dbclusterparametergroup-family
         */
        Family: C.YieldsString;
        /**
         *
         *
         * The parameters to set for this DB cluster parameter group. For a list of
 parameter keys, see Appendix: DB Cluster
    and DB Instance Parameters in the
 Amazon Relational Database Service User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbclusterparametergroup.html#cfn-rds-dbclusterparametergroup-parameters
         */
        Parameters: C.Yields<C.CFJson>;
        /**
         * A list of resource tags
         *
         * The tags that you want to attach to this parameter group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbclusterparametergroup.html#cfn-rds-dbclusterparametergroup-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
    }
}
export declare namespace AWS.RDS {
    /**
     * The AWS::RDS::DBInstance type creates an Amazon Relational Database Service (Amazon RDS) DB instance. For
    detailed information about configuring RDS DB instances, see CreateDBInstance.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html
     */
    interface DBInstance {
        Type: 'AWS::RDS::DBInstance';
        Properties: AWS.RDS.DBInstance.Properties;
    }
    /**
     * The AWS::RDS::DBInstance type creates an Amazon Relational Database Service (Amazon RDS) DB instance. For
    detailed information about configuring RDS DB instances, see CreateDBInstance.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html
     */
    function DBInstance(props: C.Omit<DBInstance, 'Type'>): DBInstance;
}
export declare namespace AWS.RDS.DBInstance {
    interface Properties {
        /**
         * String

         *
         * The allocated storage size, specified in gigabytes (GB).
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-allocatedstorage
         */
        AllocatedStorage?: C.YieldsString;
        /**
         * Boolean

         *
         * If you update the EngineVersion property to a version that's
 different from the DB instance's current major version, set this property to
 true. For more information, see ModifyDBInstance in the
 Amazon Relational Database Service API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-allowmajorversionupgrade
         */
        AllowMajorVersionUpgrade?: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Indicates that minor engine upgrades are applied automatically to the DB
 instance during the maintenance window. The default value is
 true.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-autominorversionupgrade
         */
        AutoMinorVersionUpgrade?: C.YieldsBoolean;
        /**
         * String

         *
         * The name of the Availability Zone where the DB instance is located. You cannot
 set the AvailabilityZone parameter if the
 MultiAZ parameter is set to true.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-availabilityzone
         */
        AvailabilityZone?: C.YieldsString;
        /**
         * String

         *
         * The number of days during which automatic DB snapshots are retained.
 

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-backupretentionperiod
         */
        BackupRetentionPeriod?: C.YieldsString;
        /**
         * String

         *
         * For supported engines, specifies the character set to associate with the DB
 instance. For more information, see Appendix: Oracle
    Character Sets Supported in Amazon RDS in the
 Amazon Relational Database Service User Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-charactersetname
         */
        CharacterSetName?: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether to copy all of the user-defined tags from the DB instance to
 snapshots of the DB instance. By default, Amazon RDS doesn't copy tags to snapshots.
 Amazon RDS doesn't copy tags with the aws:: prefix unless it's the DB
 instance's final snapshot (the snapshot when you delete the DB instance).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-copytagstosnapshot
         */
        CopyTagsToSnapshot?: C.YieldsBoolean;
        /**
         * String

         *
         * The name of an existing DB cluster that this instance will be associated with.
 If you specify this property, specify aurora for the
 Engine property and do not specify any of the following
 properties: AllocatedStorage, BackupRetentionPeriod,
 CharacterSetName,
 DBSecurityGroups, MasterUsername,
 MasterUserPassword, OptionGroupName,
 PreferredBackupWindow, PreferredMaintenanceWindow,
 Port, SourceDBInstanceIdentifier,
 StorageType, or VPCSecurityGroups.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-dbclusteridentifier
         */
        DBClusterIdentifier?: C.YieldsString;
        /**
         * String

         *
         * The name of the compute and memory capacity classes of the DB instance.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-dbinstanceclass
         */
        DBInstanceClass: C.YieldsString;
        /**
         * String

         *
         * A name for the DB instance. If you specify a name, AWS CloudFormation converts it
 to lower
 case. If you don't specify a name, AWS CloudFormation generates a unique physical
 ID and uses
 that ID for the DB instance. For more information, see Name Type.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-dbinstanceidentifier
         */
        DBInstanceIdentifier?: C.YieldsString;
        /**
         * String

         *
         * The name of the DB instance that was provided at the time of creation, if one
 was specified. This same name is returned for the life of the DB instance.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-dbname
         */
        DBName?: C.YieldsString;
        /**
         * String

         *
         * The name of an existing DB parameter group or a reference to an AWS::RDS::DBParameterGroup
 resource created in the template.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-dbparametergroupname
         */
        DBParameterGroupName?: C.YieldsString;
        /**
         * List of strings

         *
         * A list of the DB security groups to assign to the DB instance. The list can
 include both the name of existing DB security groups or references to AWS::RDS::DBSecurityGroup
 resources created in the template.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-dbsecuritygroups
         */
        DBSecurityGroups?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The name or ARN of the DB snapshot used to restore the DB instance. If you are
 restoring from a shared manual DB snapshot, you must specify the Amazon Resource
 Name (ARN) of the snapshot.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-dbsnapshotidentifier
         */
        DBSnapshotIdentifier?: C.YieldsString;
        /**
         * String

         *
         * A DB subnet group to associate with the DB instance. If you
 update this value, the new subnet group must be a subnet group in a new VPC.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-dbsubnetgroupname
         */
        DBSubnetGroupName?: C.YieldsString;
        /**
         * String

         *
         * For an Amazon RDS DB instance that is running Microsoft SQL Server, the Active Directory
 directory ID to create the instance in. Amazon RDS uses Windows Authentication to
 authenticate users that connect to the DB instance. For more information, see
 Using Windows
    Authentication with an Amazon RDS DB Instance Running Microsoft SQL Server
 in the Amazon Relational Database Service User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-domain
         */
        Domain?: C.YieldsString;
        /**
         * String

         *
         * The name of an IAM role that Amazon RDS uses when calling the Directory Service
 APIs.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-domainiamrolename
         */
        DomainIAMRoleName?: C.YieldsString;
        /**
         * String

         *
         * The database engine that the DB instance uses. This property is optional when
 you specify the DBSnapshotIdentifier property to create DB
 instances.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-engine
         */
        Engine?: C.YieldsString;
        /**
         * String

         *
         * The version number of the database engine that the DB instance uses.
         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-engineversion
         */
        EngineVersion?: C.YieldsString;
        /**
         * Number

         *
         * The number of I/O operations per second (IOPS) that the database provisions.
 The value must be equal to or greater than 1000.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-iops
         */
        Iops?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The ARN of the AWS Key Management Service (AWS KMS) master key that is used to encrypt
 the DB
 instance, such as
 arn:aws:kms:us-east-1:012345678910:key/abcd1234-a123-456a-a12b-a123b4cd56ef.
 If you enable the StorageEncrypted property but don't specify this
 property, AWS CloudFormation uses the default master key. If you specify this property,
 you
 must set the StorageEncrypted property to true.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-kmskeyid
         */
        KmsKeyId?: C.YieldsString;
        /**
         * String

         *
         * The license model of the DB instance.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-licensemodel
         */
        LicenseModel?: C.YieldsString;
        /**
         * String

         *
         * The master password for the DB instance.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-masteruserpassword
         */
        MasterUserPassword?: C.YieldsString;
        /**
         * String

         *
         * The master user name for the DB instance.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-masterusername
         */
        MasterUsername?: C.YieldsString;
        /**
         * Integer

         *
         * The interval, in seconds, between points when Amazon RDS collects enhanced
 monitoring metrics for the DB instance. To disable metrics collection, specify
 0.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-monitoringinterval
         */
        MonitoringInterval?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The ARN of the AWS Identity and Access Management (IAM) role that permits Amazon RDS
 to send enhanced
 monitoring metrics to Amazon CloudWatch, for example,
 arn:aws:iam:123456789012:role/emaccess. For information on creating a
 monitoring role, see To create an IAM
    role for Amazon RDS Enhanced Monitoring in the
 Amazon Relational Database Service User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-monitoringrolearn
         */
        MonitoringRoleArn?: C.YieldsString;
        /**
         * Boolean

         *
         * Specifies if the database instance is a multiple
 Availability Zone deployment. You cannot set the
 AvailabilityZone parameter if the
 MultiAZ parameter is set to true. Amazon Aurora
 storage is replicated across all the availability zones and doesn't require the MultiAZ option to be set.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-multiaz
         */
        MultiAZ?: C.YieldsBoolean;
        /**
         * String

         *
         * The option group that this DB instance is associated with.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-optiongroupname
         */
        OptionGroupName?: C.YieldsString;
        /**
         * String

         *
         * The port for the instance.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-port
         */
        Port?: C.YieldsString;
        /**
         * String

         *
         * The daily time range during which automated backups are performed if automated
 backups are enabled, as determined by the BackupRetentionPeriod
 property. For valid values, see the PreferredBackupWindow parameter
 for the CreateDBInstance action in the
 Amazon Relational Database Service API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-preferredbackupwindow
         */
        PreferredBackupWindow?: C.YieldsString;
        /**
         * String

         *
         * The weekly time range (in UTC) during which system maintenance can occur. For valid
 values, see the PreferredMaintenanceWindow parameter for the CreateDBInstance action in the Amazon Relational Database Service API Reference.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-preferredmaintenancewindow
         */
        PreferredMaintenanceWindow?: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether the DB instance is an Internet-facing instance. If you
 specify true, AWS CloudFormation creates an instance with a publicly resolvable
 DNS name, which resolves to a public IP address. If you specify
 false, AWS CloudFormation creates an internal instance with a DNS name that
 resolves to a private IP address.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-publiclyaccessible
         */
        PubliclyAccessible?: C.YieldsBoolean;
        /**
         * String

         *
         * If you want to create a read replica DB instance, specify the ID of the source
 DB instance. Each DB instance can have a limited number of read replicas. For more
 information, see Working with Read
    Replicas in the Amazon Relational Database Service Developer Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-sourcedbinstanceidentifier
         */
        SourceDBInstanceIdentifier?: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether the DB instance is encrypted.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-storageencrypted
         */
        StorageEncrypted?: C.YieldsBoolean;
        /**
         * String

         *
         * The storage type associated with this DB instance.
         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-storagetype
         */
        StorageType?: C.YieldsString;
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key–value pairs) for this DB instance.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * String

         *
         * The time zone of the DB instance, which you can specify to match the time zone
 of your applications. To see which engines supports time zones, see the
 Timezone parameter for the CreateDBInstance action in the
 Amazon Relational Database Service API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-timezone
         */
        Timezone?: C.YieldsString;
        /**
         * List of strings

         *
         * A list of the VPC security group IDs to assign to the DB instance. The list can
 include both the physical IDs of existing VPC security groups and references to
 AWS::EC2::SecurityGroup resources created in the
 template.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-database-instance.html#cfn-rds-dbinstance-vpcsecuritygroups
         */
        VPCSecurityGroups?: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.RDS {
    /**
     * Creates a custom parameter group for an RDS database family. For more information
    about RDS parameter groups,
    see Working with
       DB Parameter Groups in the Amazon Relational Database Service User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-dbparametergroup.html
     */
    interface DBParameterGroup {
        Type: 'AWS::RDS::DBParameterGroup';
        Properties: AWS.RDS.DBParameterGroup.Properties;
    }
    /**
     * Creates a custom parameter group for an RDS database family. For more information
    about RDS parameter groups,
    see Working with
       DB Parameter Groups in the Amazon Relational Database Service User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-dbparametergroup.html
     */
    function DBParameterGroup(props: C.Omit<DBParameterGroup, 'Type'>): DBParameterGroup;
}
export declare namespace AWS.RDS.DBParameterGroup {
    interface Properties {
        /**
         *
         *
         * A friendly description of the RDS parameter group. For example, "My Parameter Group".

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-dbparametergroup.html#cfn-rds-dbparametergroup-description
         */
        Description: C.YieldsString;
        /**
         *
         *
         * The database family of this RDS parameter group. For example, "MySQL5.1".

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-dbparametergroup.html#cfn-rds-dbparametergroup-family
         */
        Family: C.YieldsString;
        /**
         *
         *
         * The parameters to set for this RDS parameter group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-dbparametergroup.html#cfn-rds-dbparametergroup-parameters
         */
        Parameters?: C.Yields<C.CFMap<C.YieldsString>>;
        /**
         * A list of resource tags.

         *
         * The tags that you want to attach to the RDS parameter group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-dbparametergroup.html#cfn-rds-dbparametergroup-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
    }
}
export declare namespace AWS.RDS {
    /**
     * The AWS::RDS::DBSecurityGroup type is used to create or update an Amazon RDS DB Security
    Group.
    For more information about DB security groups, see Working with DB Security Groups in the Amazon Relational Database Service Developer Guide. For
    details on the settings for DB security groups, see CreateDBSecurityGroup.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-security-group.html
     */
    interface DBSecurityGroup {
        Type: 'AWS::RDS::DBSecurityGroup';
        Properties: AWS.RDS.DBSecurityGroup.Properties;
    }
    /**
     * The AWS::RDS::DBSecurityGroup type is used to create or update an Amazon RDS DB Security
    Group.
    For more information about DB security groups, see Working with DB Security Groups in the Amazon Relational Database Service Developer Guide. For
    details on the settings for DB security groups, see CreateDBSecurityGroup.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-security-group.html
     */
    function DBSecurityGroup(props: C.Omit<DBSecurityGroup, 'Type'>): DBSecurityGroup;
}
export declare namespace AWS.RDS.DBSecurityGroup {
    interface Properties {
        /**
         * List of RDS
    Security Group Rules.

         *
         * Network ingress authorization for an Amazon EC2 security group or an IP address range.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-security-group.html#cfn-rds-dbsecuritygroup-dbsecuritygroupingress
         */
        DBSecurityGroupIngress: C.Yields<C.CFList<C.Yields<AWS.RDS.DBSecurityGroup.Ingress>>>;
        /**
         * String

         *
         * The Id of the VPC. Indicates which VPC this DB Security Group should belong to.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-security-group.html#cfn-rds-dbsecuritygroup-ec2vpcid
         */
        EC2VpcId?: C.YieldsString;
        /**
         * String

         *
         * Description of the security group.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-security-group.html#cfn-rds-dbsecuritygroup-groupdescription
         */
        GroupDescription: C.YieldsString;
        /**
         * A list of resource tags.

         *
         * The tags that you want to attach to the Amazon RDS DB security group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-security-group.html#cfn-rds-dbsecuritygroup-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
    }
}
export declare namespace AWS.RDS.DBSecurityGroup {
    interface Ingress {
        /**
         * String

         *
         * The IP range to authorize.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-security-group-rule.html#cfn-rds-securitygroup-cidrip
         */
        CIDRIP?: C.YieldsString;
        /**
         * String

         *
         * Id of the VPC or EC2 Security Group to authorize.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-security-group-rule.html#cfn-rds-securitygroup-ec2securitygroupid
         */
        EC2SecurityGroupId?: C.YieldsString;
        /**
         * String

         *
         * Name of the EC2 Security Group to authorize.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-security-group-rule.html#cfn-rds-securitygroup-ec2securitygroupname
         */
        EC2SecurityGroupName?: C.YieldsString;
        /**
         * String

         *
         * AWS Account Number of the owner of the EC2 Security Group specified in the EC2SecurityGroupName
 parameter. The AWS Access Key ID is not an acceptable value.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-security-group-rule.html#cfn-rds-securitygroup-ec2securitygroupownerid
         */
        EC2SecurityGroupOwnerId?: C.YieldsString;
    }
}
export declare namespace AWS.RDS {
    /**
     * The AWS::RDS::DBSecurityGroupIngress type enables ingress to a DBSecurityGroup using
    one of two forms of
    authorization. First, EC2 or VPC security groups can be added to the DBSecurityGroup
    if the application using the
    database is running on EC2 or VPC instances. Second, IP ranges are available if the
    application accessing your
    database is running on the Internet. For more information about DB security groups,
    see Working
       with DB security groups
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-security-group-ingress.html
     */
    interface DBSecurityGroupIngress {
        Type: 'AWS::RDS::DBSecurityGroupIngress';
        Properties: AWS.RDS.DBSecurityGroupIngress.Properties;
    }
    /**
     * The AWS::RDS::DBSecurityGroupIngress type enables ingress to a DBSecurityGroup using
    one of two forms of
    authorization. First, EC2 or VPC security groups can be added to the DBSecurityGroup
    if the application using the
    database is running on EC2 or VPC instances. Second, IP ranges are available if the
    application accessing your
    database is running on the Internet. For more information about DB security groups,
    see Working
       with DB security groups
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-security-group-ingress.html
     */
    function DBSecurityGroupIngress(props: C.Omit<DBSecurityGroupIngress, 'Type'>): DBSecurityGroupIngress;
}
export declare namespace AWS.RDS.DBSecurityGroupIngress {
    interface Properties {
        /**
         * String

         *
         * The IP range to authorize.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-security-group-ingress.html#cfn-rds-securitygroup-ingress-cidrip
         */
        CIDRIP?: C.YieldsString;
        /**
         * String

         *
         * The name (ARN) of the AWS::RDS::DBSecurityGroup to which this ingress
 will be added.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-security-group-ingress.html#cfn-rds-securitygroup-ingress-dbsecuritygroupname
         */
        DBSecurityGroupName: C.YieldsString;
        /**
         * String

         *
         * The ID of the VPC or EC2 security group to authorize.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-security-group-ingress.html#cfn-rds-securitygroup-ingress-ec2securitygroupid
         */
        EC2SecurityGroupId?: C.YieldsString;
        /**
         * String

         *
         * The name of the EC2 security group to authorize.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-security-group-ingress.html#cfn-rds-securitygroup-ingress-ec2securitygroupname
         */
        EC2SecurityGroupName?: C.YieldsString;
        /**
         * String

         *
         * The AWS Account Number of the owner of the EC2 security group specified in the EC2SecurityGroupName
 parameter. The AWS Access Key ID is not an acceptable value.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-security-group-ingress.html#cfn-rds-securitygroup-ingress-ec2securitygroupownerid
         */
        EC2SecurityGroupOwnerId?: C.YieldsString;
    }
}
export declare namespace AWS.RDS {
    /**
     * The AWS::RDS::DBSubnetGroup type creates an RDS database subnet group.
    Subnet groups must contain at least two subnets in two different
    Availability Zones in the same region.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbsubnet-group.html
     */
    interface DBSubnetGroup {
        Type: 'AWS::RDS::DBSubnetGroup';
        Properties: AWS.RDS.DBSubnetGroup.Properties;
    }
    /**
     * The AWS::RDS::DBSubnetGroup type creates an RDS database subnet group.
    Subnet groups must contain at least two subnets in two different
    Availability Zones in the same region.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbsubnet-group.html
     */
    function DBSubnetGroup(props: C.Omit<DBSubnetGroup, 'Type'>): DBSubnetGroup;
}
export declare namespace AWS.RDS.DBSubnetGroup {
    interface Properties {
        /**
         * String

         *
         * The description for the DB Subnet Group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbsubnet-group.html#cfn-rds-dbsubnetgroup-dbsubnetgroupdescription
         */
        DBSubnetGroupDescription: C.YieldsString;
        /**
         * List of strings

         *
         * The EC2 Subnet IDs for the DB Subnet Group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbsubnet-group.html#cfn-rds-dbsubnetgroup-subnetids
         */
        SubnetIds: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * A list of resource tags in key-value
 format.

         *
         * The tags that you want to attach to the RDS database subnet group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbsubnet-group.html#cfn-rds-dbsubnetgroup-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
    }
}
export declare namespace AWS.RDS {
    /**
     * Use the AWS::RDS::EventSubscription resource to get notifications for
    Amazon Relational Database Service events through the Amazon Simple Notification Service.
    For more information, see  Using Amazon RDS Event Notification in the
    Amazon Relational Database Service User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-eventsubscription.html
     */
    interface EventSubscription {
        Type: 'AWS::RDS::EventSubscription';
        Properties: AWS.RDS.EventSubscription.Properties;
    }
    /**
     * Use the AWS::RDS::EventSubscription resource to get notifications for
    Amazon Relational Database Service events through the Amazon Simple Notification Service.
    For more information, see  Using Amazon RDS Event Notification in the
    Amazon Relational Database Service User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-eventsubscription.html
     */
    function EventSubscription(props: C.Omit<EventSubscription, 'Type'>): EventSubscription;
}
export declare namespace AWS.RDS.EventSubscription {
    interface Properties {
        /**
         * Boolean

         *
         * Indicates whether to activate the subscription. If you don't specify this
 property, AWS CloudFormation activates the subscription.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-eventsubscription.html#cfn-rds-eventsubscription-enabled
         */
        Enabled?: C.YieldsBoolean;
        /**
         * List of strings

         *
         * A list of event categories that you want to subscribe to for a given source type.
 If you don't specify this property, you are notified about all event categories. For
 more information, see  Using Amazon RDS Event Notification in the Amazon Relational Database Service User Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-eventsubscription.html#cfn-rds-eventsubscription-eventcategories
         */
        EventCategories?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of an Amazon SNS topic that you want to send event
 notifications to.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-eventsubscription.html#cfn-rds-eventsubscription-snstopicarn
         */
        SnsTopicArn: C.YieldsString;
        /**
         * List of strings

         *
         * A list of identifiers for which Amazon RDS provides notification events.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-eventsubscription.html#cfn-rds-eventsubscription-sourceids
         */
        SourceIds?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The type of source for which Amazon RDS provides notification events. For example,
 if you want to be notified of events generated by a database instance, set this parameter
 to db-instance. If you don't specify a value, notifications are provided for all source types. For
 valid values, see the SourceType parameter for the CreateEventSubscription action in the
 Amazon Relational Database Service API Reference.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-eventsubscription.html#cfn-rds-eventsubscription-sourcetype
         */
        SourceType?: C.YieldsString;
    }
}
export declare namespace AWS.RDS {
    /**
     * Use the AWS::RDS::OptionGroup resource to create an option group that can make
    managing data and databases easier. For more information about option groups, see
    Working with Option Groups in the
    Amazon Relational Database Service User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-optiongroup.html
     */
    interface OptionGroup {
        Type: 'AWS::RDS::OptionGroup';
        Properties: AWS.RDS.OptionGroup.Properties;
    }
    /**
     * Use the AWS::RDS::OptionGroup resource to create an option group that can make
    managing data and databases easier. For more information about option groups, see
    Working with Option Groups in the
    Amazon Relational Database Service User Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-optiongroup.html
     */
    function OptionGroup(props: C.Omit<OptionGroup, 'Type'>): OptionGroup;
}
export declare namespace AWS.RDS.OptionGroup {
    interface Properties {
        /**
         * String

         *
         * The name of the database engine that this option group is associated
 with.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-optiongroup.html#cfn-rds-optiongroup-enginename
         */
        EngineName: C.YieldsString;
        /**
         * String

         *
         * The major version number of the database engine that this option group is
 associated with.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-optiongroup.html#cfn-rds-optiongroup-majorengineversion
         */
        MajorEngineVersion: C.YieldsString;
        /**
         * Amazon RDS OptionGroup
    OptionConfigurations
         *
         * The configurations for this option group.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-optiongroup.html#cfn-rds-optiongroup-optionconfigurations
         */
        OptionConfigurations: C.Yields<C.CFList<C.Yields<AWS.RDS.OptionGroup.OptionConfiguration>>>;
        /**
         * String

         *
         * A description of the option group.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-optiongroup.html#cfn-rds-optiongroup-optiongroupdescription
         */
        OptionGroupDescription: C.YieldsString;
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key–value pairs) for this option group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-optiongroup.html#cfn-rds-optiongroup-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
    }
}
export declare namespace AWS.RDS.OptionGroup {
    interface OptionConfiguration {
        /**
         * List of strings

         *
         * A list of database security group names for this option. If the option requires
 access to a port, the security groups must allow access to that port. If you specify
 this property, don't specify the VPCSecurityGroupMemberships
 property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-optiongroup-optionconfigurations.html#cfn-rds-optiongroup-optionconfigurations-dbsecuritygroupmemberships
         */
        DBSecurityGroupMemberships?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The name of the option. For more information about options, see Working with Option
    Groups in the Amazon Relational Database Service User
    Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-optiongroup-optionconfigurations.html#cfn-rds-optiongroup-optionconfigurations-optionname
         */
        OptionName: C.YieldsString;
        /**
         * Amazon RDS
    OptionGroup OptionConfigurations OptionSettings
         *
         * The settings for this option.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-optiongroup-optionconfigurations.html#cfn-rds-optiongroup-optionconfigurations-optionsettings
         */
        OptionSettings?: C.Yields<AWS.RDS.OptionGroup.OptionSetting>;
        /**
         * Integer

         *
         * The port number that this option uses.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-optiongroup-optionconfigurations.html#cfn-rds-optiongroup-optionconfigurations-port
         */
        Port?: C.Yields<C.CFInteger>;
        /**
         * List of strings

         *
         * A list of VPC security group IDs for this option. If the option requires access to
 a
 port, the security groups must allow access to that port. If you specify this property,
 don't specify the DBSecurityGroupMemberships property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-optiongroup-optionconfigurations.html#cfn-rds-optiongroup-optionconfigurations-vpcsecuritygroupmemberships
         */
        VpcSecurityGroupMemberships?: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.RDS.OptionGroup {
    interface OptionSetting {
        /**
         * String

         *
         * The name of the option setting that you want to specify.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-optiongroup-optionconfigurations-optionsettings.html#cfn-rds-optiongroup-optionconfigurations-optionsettings-name
         */
        Name?: C.YieldsString;
        /**
         * String

         *
         * The value of the option setting.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-rds-optiongroup-optionconfigurations-optionsettings.html#cfn-rds-optiongroup-optionconfigurations-optionsettings-value
         */
        Value?: C.YieldsString;
    }
}
export declare namespace AWS.Redshift {
    /**
     * Use
    the AWS::Redshift::Cluster resource
    to
    create
    an Amazon Redshift cluster. A cluster is a fully managed data warehouse that
    consists of a set of compute nodes. For more information about default and valid values,
    see
    CreateCluster in the
    Amazon Redshift API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html
     */
    interface Cluster {
        Type: 'AWS::Redshift::Cluster';
        Properties: AWS.Redshift.Cluster.Properties;
    }
    /**
     * Use
    the AWS::Redshift::Cluster resource
    to
    create
    an Amazon Redshift cluster. A cluster is a fully managed data warehouse that
    consists of a set of compute nodes. For more information about default and valid values,
    see
    CreateCluster in the
    Amazon Redshift API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html
     */
    function Cluster(props: C.Omit<Cluster, 'Type'>): Cluster;
}
export declare namespace AWS.Redshift.Cluster {
    interface Properties {
        /**
         * Boolean

         *
         * When a new version of  Amazon Redshift is released, tells whether upgrades can be
 applied
 to the engine that is running on the cluster. The upgrades are applied during the
 maintenance window. The default value is true.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-allowversionupgrade
         */
        AllowVersionUpgrade?: C.YieldsBoolean;
        /**
         * Integer

         *
         * The number of days that automated snapshots are retained. The default value is
 1. To disable automated snapshots, set the value to
 0.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-automatedsnapshotretentionperiod
         */
        AutomatedSnapshotRetentionPeriod?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The Amazon Elastic Compute Cloud (Amazon EC2) Availability Zone in which you want
 to provision your
 Amazon Redshift cluster. For example, if you have several EC2 instances running in
 a specific
 Availability Zone, you might want the cluster to be provisioned in the same zone
 to decrease network latency.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-availabilityzone
         */
        AvailabilityZone?: C.YieldsString;
        /**
         * String

         *
         * The name of
 the
 parameter group that you want to associate with this
 cluster.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-clusterparametergroupname
         */
        ClusterParameterGroupName?: C.YieldsString;
        /**
         * List of strings

         *
         * A list of security groups that you want to associate with
 this cluster. Applies to EC2-Classic.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-clustersecuritygroups
         */
        ClusterSecurityGroups?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The name of a cluster subnet group that you want to associate with this
 cluster.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-clustersubnetgroupname
         */
        ClusterSubnetGroupName?: C.YieldsString;
        /**
         * String

         *
         * The type of cluster. Specify single-node or
 multi-node (default).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-clustertype
         */
        ClusterType: C.YieldsString;
        /**
         * String

         *
         * The version of the Amazon Redshift engine that you want to deploy on the cluster.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-clusterversion
         */
        ClusterVersion?: C.YieldsString;
        /**
         * String

         *
         * The name of the first database that
 will
 be created when the cluster is created.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-dbname
         */
        DBName: C.YieldsString;
        /**
         * String

         *
         * The Elastic IP (EIP) address for the cluster.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-elasticip
         */
        ElasticIp?: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether the data in the cluster is encrypted at rest. The default value
 is false.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-encrypted
         */
        Encrypted?: C.YieldsBoolean;
        /**
         * String

         *
         * Specifies the name of the hardware security module (HSM) client certificate
 that the Amazon Redshift cluster uses to retrieve the data encryption keys stored
 in an
 HSM.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-hsmclientcertidentifier
         */
        HsmClientCertificateIdentifier?: C.YieldsString;
        /**
         * String

         *
         * The name of the HSM configuration that contains the information that the Amazon Redshift
 cluster can use to retrieve and store keys in an HSM.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-HsmConfigurationIdentifier
         */
        HsmConfigurationIdentifier?: C.YieldsString;
        /**
         * String

         *
         * A list of AWS Identity and Access Management (IAM) roles that the cluster can use
 to access other
 AWS services. Supply the IAM roles by their Amazon Resource Name (ARN). You
 can provide a maximum of 10 IAM roles in a single request. A cluster can have a
 maximum of 10 IAM roles associated with it at a time.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-iamroles
         */
        IamRoles?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * The ID of the AWS Key Management Service (AWS KMS) key that you want to use to encrypt
 data in the
 cluster.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-kmskeyid
         */
        KmsKeyId?: C.YieldsString;
        /**
         * Amazon Redshift LoggingProperties
         *
         * Configures
 Amazon Redshift to create audit log files, containing logging information such as
 queries and
 connection attempts, for this cluster.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-loggingproperties
         */
        LoggingProperties?: C.Yields<AWS.Redshift.Cluster.LoggingProperties>;
        /**
         * String

         *
         * The password associated with the master user account for this cluster.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-masteruserpassword
         */
        MasterUserPassword: C.YieldsString;
        /**
         * String

         *
         * The user name that is associated with the master user account for this
 cluster.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-masterusername
         */
        MasterUsername: C.YieldsString;
        /**
         * String

         *
         * The node type that is provisioned for this cluster.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-nodetype
         */
        NodeType: C.YieldsString;
        /**
         * Integer

         *
         * The number of compute nodes in the cluster. If you specify
 multi-node for the ClusterType parameter, you must
 specify a number greater than 1.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-nodetype
         */
        NumberOfNodes?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * When you
 restore
 from a snapshot from another AWS account, the 12-digit AWS
 account ID that contains that snapshot.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-owneraccount
         */
        OwnerAccount?: C.YieldsString;
        /**
         * Integer

         *
         * The port number on which the cluster accepts incoming connections. The default value
 is 5439.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-port
         */
        Port?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The weekly time range (in UTC) during which automated cluster maintenance can
 occur. The format of the time range is
 ddd:hh24:mi-ddd:hh24:mi.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-preferredmaintenancewindow
         */
        PreferredMaintenanceWindow?: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether the cluster can be accessed from a public network.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-publiclyaccessible
         */
        PubliclyAccessible?: C.YieldsBoolean;
        /**
         *
         *
         * The name of the cluster that the
 source
 snapshot was created from. For more information about restoring
 from a snapshot, see the RestoreFromClusterSnapshot action in the
 Amazon Redshift API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-snapshotclusteridentifier
         */
        SnapshotClusterIdentifier?: C.YieldsString;
        /**
         * String

         *
         * The name of the snapshot from which to create a new cluster.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-snapshotidentifier
         */
        SnapshotIdentifier?: C.YieldsString;
        /**
         * AWS CloudFormation Resource Tags
         *
         * Specifies an arbitrary set of tags (key–value pairs) to associate with this
 cluster. Use tags to manage your resources.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * List of strings

         *
         * A list of VPC security groups that are associated with this cluster.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-cluster.html#cfn-redshift-cluster-vpcsecuritygroupids
         */
        VpcSecurityGroupIds?: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.Redshift.Cluster {
    interface LoggingProperties {
        /**
         * String

         *
         * The name of an existing S3 bucket where the log files are to be stored.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-redshift-cluster-loggingproperties.html#cfn-redshift-cluster-loggingproperties-bucketname
         */
        BucketName: C.YieldsString;
        /**
         * String

         *
         * The
 prefix applied to the log file names.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-redshift-cluster-loggingproperties.html#cfn-redshift-cluster-loggingproperties-s3keyprefix
         */
        S3KeyPrefix?: C.YieldsString;
    }
}
export declare namespace AWS.Redshift {
    /**
     * Creates an Amazon Redshift parameter group that you can associate with an Amazon Redshift
    cluster. The parameters
    in the group apply to all the databases that you create in the cluster.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clusterparametergroup.html
     */
    interface ClusterParameterGroup {
        Type: 'AWS::Redshift::ClusterParameterGroup';
        Properties: AWS.Redshift.ClusterParameterGroup.Properties;
    }
    /**
     * Creates an Amazon Redshift parameter group that you can associate with an Amazon Redshift
    cluster. The parameters
    in the group apply to all the databases that you create in the cluster.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clusterparametergroup.html
     */
    function ClusterParameterGroup(props: C.Omit<ClusterParameterGroup, 'Type'>): ClusterParameterGroup;
}
export declare namespace AWS.Redshift.ClusterParameterGroup {
    interface Properties {
        /**
         * String

         *
         * A description of the parameter group.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clusterparametergroup.html#cfn-redshift-clusterparametergroup-description
         */
        Description: C.YieldsString;
        /**
         * String

         *
         * The Amazon Redshift engine version that applies to this cluster parameter group. The
 cluster engine version determines the set of parameters that you can specify in
 the Parameters property.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clusterparametergroup.html#cfn-redshift-clusterparametergroup-parametergroupfamily
         */
        ParameterGroupFamily: C.YieldsString;
        /**
         * Amazon Redshift Parameter
    Type
         *
         * A list of parameter names and values that are allowed by the Amazon Redshift engine
 version that you specified in the ParameterGroupFamily property. For
 more information, see Amazon Redshift Parameter Groups in the
 Amazon Redshift Cluster Management Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clusterparametergroup.html#cfn-redshift-clusterparametergroup-parameters
         */
        Parameters?: C.Yields<C.CFList<C.Yields<AWS.Redshift.ClusterParameterGroup.Parameter>>>;
        /**
         * AWS CloudFormation Resource Tags
         *
         * Specifies an arbitrary set of tags (key–value pairs) to associate with this
 parameter group. Use tags to manage your resources.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clusterparametergroup.html#cfn-redshift-clusterparametergroup-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
    }
}
export declare namespace AWS.Redshift.ClusterParameterGroup {
    interface Parameter {
        /**
         * String

         *
         * The name of the parameter.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-property-redshift-clusterparametergroup-parameter.html#cfn-redshift-clusterparametergroup-parameter-parametername
         */
        ParameterName: C.YieldsString;
        /**
         * String

         *
         * The value of the parameter.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-property-redshift-clusterparametergroup-parameter.html#cfn-redshift-clusterparametergroup-parameter-parametervalue
         */
        ParameterValue: C.YieldsString;
    }
}
export declare namespace AWS.Redshift {
    /**
     * Creates an Amazon Redshift security group. You use security groups to control access
    to Amazon Redshift clusters
    that are not in a VPC.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clustersecuritygroup.html
     */
    interface ClusterSecurityGroup {
        Type: 'AWS::Redshift::ClusterSecurityGroup';
        Properties: AWS.Redshift.ClusterSecurityGroup.Properties;
    }
    /**
     * Creates an Amazon Redshift security group. You use security groups to control access
    to Amazon Redshift clusters
    that are not in a VPC.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clustersecuritygroup.html
     */
    function ClusterSecurityGroup(props: C.Omit<ClusterSecurityGroup, 'Type'>): ClusterSecurityGroup;
}
export declare namespace AWS.Redshift.ClusterSecurityGroup {
    interface Properties {
        /**
         * String

         *
         * A description of the security group.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clustersecuritygroup.html#cfn-redshift-clustersecuritygroup-description
         */
        Description: C.YieldsString;
        /**
         * AWS CloudFormation Resource Tags
         *
         * Specifies an arbitrary set of tags (key–value pairs) to associate with this
 security group. Use tags to manage your resources.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clustersecuritygroup.html#cfn-redshift-clustersecuritygroup-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
    }
}
export declare namespace AWS.Redshift {
    /**
     * Specifies inbound (ingress) rules for an Amazon Redshift security group.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clustersecuritygroupingress.html
     */
    interface ClusterSecurityGroupIngress {
        Type: 'AWS::Redshift::ClusterSecurityGroupIngress';
        Properties: AWS.Redshift.ClusterSecurityGroupIngress.Properties;
    }
    /**
     * Specifies inbound (ingress) rules for an Amazon Redshift security group.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clustersecuritygroupingress.html
     */
    function ClusterSecurityGroupIngress(props: C.Omit<ClusterSecurityGroupIngress, 'Type'>): ClusterSecurityGroupIngress;
}
export declare namespace AWS.Redshift.ClusterSecurityGroupIngress {
    interface Properties {
        /**
         * String

         *
         * The IP address range that has inbound access to the Amazon Redshift security group.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clustersecuritygroupingress.html#cfn-redshift-clustersecuritygroupingress-cidrip
         */
        CIDRIP?: C.YieldsString;
        /**
         * String

         *
         * The name of the Amazon Redshift security group that will be associated with the ingress
 rule.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clustersecuritygroupingress.html#cfn-redshift-clustersecuritygroupingress-clustersecuritygroupname
         */
        ClusterSecurityGroupName: C.YieldsString;
        /**
         * String

         *
         * The Amazon EC2 security group that will be added the Amazon Redshift security group.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clustersecuritygroupingress.html#cfn-redshift-clustersecuritygroupingress-ec2securitygroupname
         */
        EC2SecurityGroupName?: C.YieldsString;
        /**
         * String

         *
         * The 12-digit AWS account number of the owner of the Amazon EC2 security group that
 is specified by the EC2SecurityGroupName parameter.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clustersecuritygroupingress.html#cfn-redshift-clustersecuritygroupingress-ec2securitygroupownerid
         */
        EC2SecurityGroupOwnerId?: C.YieldsString;
    }
}
export declare namespace AWS.Redshift {
    /**
     * Creates an Amazon Redshift subnet group. You must provide a list of one or more subnets
    in your
    existing Amazon VPC when creating an Amazon Redshift subnet group.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clustersubnetgroup.html
     */
    interface ClusterSubnetGroup {
        Type: 'AWS::Redshift::ClusterSubnetGroup';
        Properties: AWS.Redshift.ClusterSubnetGroup.Properties;
    }
    /**
     * Creates an Amazon Redshift subnet group. You must provide a list of one or more subnets
    in your
    existing Amazon VPC when creating an Amazon Redshift subnet group.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clustersubnetgroup.html
     */
    function ClusterSubnetGroup(props: C.Omit<ClusterSubnetGroup, 'Type'>): ClusterSubnetGroup;
}
export declare namespace AWS.Redshift.ClusterSubnetGroup {
    interface Properties {
        /**
         * String

         *
         * A description of the subnet group.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clustersubnetgroup.html#cfn-redshift-clustersubnetgroup-description
         */
        Description: C.YieldsString;
        /**
         * List of strings

         *
         * A list of VPC subnet IDs. You can modify a maximum of 20 subnets.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clustersubnetgroup.html#cfn-redshift-clustersubnetgroup-subnetids
         */
        SubnetIds: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * AWS CloudFormation Resource Tags
         *
         * Specifies an arbitrary set of tags (key–value pairs) to associate with this
 subnet group. Use tags to manage your resources.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-redshift-clustersubnetgroup.html#cfn-redshift-clustersubnetgroup-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
    }
}
export declare namespace AWS.Route53 {
    /**
     * Use the AWS::Route53::HealthCheck resource to check the health of your
    resources before Amazon Route 53 responds to a DNS query. For more information, see
    How Health Checks Work in Simple Amazon
       Route 53 Configurations in the Amazon Route 53 Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-route53-healthcheck.html
     */
    interface HealthCheck {
        Type: 'AWS::Route53::HealthCheck';
        Properties: AWS.Route53.HealthCheck.Properties;
    }
    /**
     * Use the AWS::Route53::HealthCheck resource to check the health of your
    resources before Amazon Route 53 responds to a DNS query. For more information, see
    How Health Checks Work in Simple Amazon
       Route 53 Configurations in the Amazon Route 53 Developer Guide.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-route53-healthcheck.html
     */
    function HealthCheck(props: C.Omit<HealthCheck, 'Type'>): HealthCheck;
}
export declare namespace AWS.Route53.HealthCheck {
    interface Properties {
        /**
         * Amazon Route 53
    HealthCheckConfig
         *
         * An Amazon Route 53 health check.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-route53-healthcheck.html#cfn-route53-healthcheck-healthcheckconfig
         */
        HealthCheckConfig: C.Yields<AWS.Route53.HealthCheck.HealthCheckConfig>;
        /**
         * A list of Amazon Route 53
    HealthCheckTags
         *
         * An arbitrary set of tags (key–value pairs) for this health check.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-route53-healthcheck.html#cfn-route53-healthcheck-healthchecktags
         */
        HealthCheckTags?: C.Yields<C.CFList<C.Yields<AWS.Route53.HealthCheck.HealthCheckTag>>>;
    }
}
export declare namespace AWS.Route53.HealthCheck {
    interface AlarmIdentifier {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-healthcheck-alarmidentifier.html#cfn-route53-healthcheck-alarmidentifier-name
         */
        Name: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-healthcheck-alarmidentifier.html#cfn-route53-healthcheck-alarmidentifier-region
         */
        Region: C.YieldsString;
    }
}
export declare namespace AWS.Route53.HealthCheck {
    interface HealthCheckConfig {
        /**
         * Amazon Route 53 AlarmIdentifier
         *
         * Identifies the CloudWatch alarm that you want Amazon Route 53 health checkers to use
 to determine whether this health check is healthy.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-healthcheck-healthcheckconfig.html#cfn-route53-healthcheck-healthcheckconfig-alarmidentifier
         */
        AlarmIdentifier?: C.Yields<AWS.Route53.HealthCheck.AlarmIdentifier>;
        /**
         * List of strings

         *
         * (CALCULATED Health Checks Only) A complex type that contains one ChildHealthCheck element for each
 health check that you want to associate with a CALCULATED health check.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-healthcheck-healthcheckconfig.html#cfn-route53-healthcheck-healthcheckconfig-childhealthchecks
         */
        ChildHealthChecks?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * Boolean

         *
         * Specifies whether you want Amazon Route 53 to send the value of
 FullyQualifiedDomainName to the endpoint in the
 client_hello message during TLS negotiation. This allows the
 endpoint to respond to HTTPS health check requests with the
 applicable SSL/TLS certificate. For more information, see http://docs.aws.amazon.com/Route53/latest/APIReference/API_HealthCheckConfig.html.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-healthcheck-healthcheckconfig.html#cfn-route53-healthcheck-healthcheckconfig-enablesni
         */
        EnableSNI?: C.YieldsBoolean;
        /**
         * Integer

         *
         * The number of consecutive health checks that an endpoint must pass or fail for
 Amazon Route 53 to change the current status of the endpoint from unhealthy to healthy
 or
 healthy to unhealthy.  For more information, see How Amazon Route 53 Determines Whether an Endpoint Is Healthy in the
 Amazon Route 53 Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-healthcheck-healthcheckconfig.html#cfn-route53-healthcheck-healthcheckconfig-failurethreshold
         */
        FailureThreshold?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * If you specified the IPAddress property, the value that you want
 Amazon Route 53 to pass in the host header in all health checks except for TCP health
 checks. If you don't specify an IP address, the domain that Amazon Route 53 sends
 a DNS
 request to. Amazon Route 53 uses the IP address that the DNS returns to check the
 health of
 the endpoint.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-healthcheck-healthcheckconfig.html#cfn-route53-healthcheck-healthcheckconfig-fullyqualifieddomainname
         */
        FullyQualifiedDomainName?: C.YieldsString;
        /**
         * Integer

         *
         * The number of child health checks that are associated with a CALCULATED health that
 Amazon Route 53 must consider healthy for the CALCULATED health check to be considered healthy.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-healthcheck-healthcheckconfig.html#cfn-route53-healthcheck-healthcheckconfig-healththreshold
         */
        HealthThreshold?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The IPv4 IP address of the endpoint on which you want Amazon Route 53 to perform health
 checks. If you don't specify an IP address, Amazon Route 53 sends a DNS request to
 resolve
 the domain name that you specify in the FullyQualifiedDomainName
 property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-healthcheck-healthcheckconfig.html#cfn-route53-healthcheck-healthcheckconfig-ipaddress
         */
        IPAddress?: C.YieldsString;
        /**
         * String

         *
         * When Amazon CloudWatch has insufficient data about the metric to determine the alarm
 state, the status that you want
 Amazon Route 53 to assign to the health check (Healthy, Unhealthy, or LastKnownStatus).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-healthcheck-healthcheckconfig.html#cfn-route53-healthcheck-healthcheckconfig-insufficientdatahealthstatus
         */
        InsufficientDataHealthStatus?: C.YieldsString;
        /**
         * Boolean

         *
         * Specifies whether you want Amazon Route 53 to invert the status of a health check,
 for
 example, to consider a health check unhealthy when it otherwise would be
 considered healthy.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-healthcheck-healthcheckconfig.html#cfn-route53-healthcheck-healthcheckconfig-inverted
         */
        Inverted?: C.YieldsBoolean;
        /**
         * Boolean

         *
         * Specifies whether you want Amazon Route 53 to measure the latency between health checkers
 in multiple AWS regions and your endpoint and display CloudWatch latency graphs on
 the
 Health Checks page in the Amazon Route 53 console.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-healthcheck-healthcheckconfig.html#cfn-route53-healthcheck-healthcheckconfig-measurelatency
         */
        MeasureLatency?: C.YieldsBoolean;
        /**
         * Integer

         *
         * The port on the endpoint on which you want Amazon Route 53 to perform health
 checks.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-healthcheck-healthcheckconfig.html#cfn-route53-healthcheck-healthcheckconfig-port
         */
        Port?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The number of seconds between the time that Amazon Route 53 gets a response from your
 endpoint and the time that it sends the next health check request. Each Amazon Route 53
 health checker makes requests at this interval. For valid values, see the RequestInterval element in
 the Amazon Route 53 API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-healthcheck-healthcheckconfig.html#cfn-route53-healthcheck-healthcheckconfig-requestinterval
         */
        RequestInterval?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The path that you want Amazon Route 53 to request when performing health checks. The
 path
 can be any value for which your endpoint returns an HTTP status code of
 2xx or
 3xx when the endpoint is healthy,
 such as /docs/route53-health-check.html.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-healthcheck-healthcheckconfig.html#cfn-route53-healthcheck-healthcheckconfig-resourcepath
         */
        ResourcePath?: C.YieldsString;
        /**
         * String

         *
         * If the value of the Type property is HTTP_STR_MATCH
 or HTTPS_STR_MATCH, the string that you want Amazon Route 53 to search for in
 the response body from the specified resource. If the string appears in the
 response body, Amazon Route 53 considers the resource healthy.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-healthcheck-healthcheckconfig.html#cfn-route53-healthcheck-healthcheckconfig-searchstring
         */
        SearchString?: C.YieldsString;
        /**
         * String

         *
         * The type of health check that you want to create. This indicates how Amazon Route 53
 determines whether an endpoint is healthy. You can specify HTTP,
 HTTPS, HTTP_STR_MATCH, HTTPS_STR_MATCH,
 or TCP. For information about the different types, see the Type element
 in the Amazon Route 53 API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-healthcheck-healthcheckconfig.html#cfn-route53-healthcheck-healthcheckconfig-type
         */
        Type: C.YieldsString;
    }
}
export declare namespace AWS.Route53.HealthCheck {
    interface HealthCheckTag {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-healthcheck-healthchecktag.html#cfn-route53-healthchecktags-key
         */
        Key: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-healthcheck-healthchecktag.html#cfn-route53-healthchecktags-value
         */
        Value: C.YieldsString;
    }
}
export declare namespace AWS.Route53 {
    /**
     * The AWS::Route53::HostedZone resource creates a hosted zone, which can contain
    a collection of record sets for a domain. You cannot create a hosted zone for a top-level
    domain (TLD). For more information, see POST CreateHostedZone or POST CreateHostedZone
       (Private) in the Amazon Route 53 API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-route53-hostedzone.html
     */
    interface HostedZone {
        Type: 'AWS::Route53::HostedZone';
        Properties: AWS.Route53.HostedZone.Properties;
    }
    /**
     * The AWS::Route53::HostedZone resource creates a hosted zone, which can contain
    a collection of record sets for a domain. You cannot create a hosted zone for a top-level
    domain (TLD). For more information, see POST CreateHostedZone or POST CreateHostedZone
       (Private) in the Amazon Route 53 API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-route53-hostedzone.html
     */
    function HostedZone(props: C.Omit<HostedZone, 'Type'>): HostedZone;
}
export declare namespace AWS.Route53.HostedZone {
    interface Properties {
        /**
         * Amazon Route 53 HostedZoneConfig
    Property
         *
         * A complex type that contains an optional comment about your hosted zone.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-route53-hostedzone.html#cfn-route53-hostedzone-hostedzoneconfig
         */
        HostedZoneConfig?: C.Yields<AWS.Route53.HostedZone.HostedZoneConfig>;
        /**
         * List of Amazon Route 53
    HostedZoneTags
         *
         * An arbitrary set of tags (key–value pairs) for this hosted zone.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-route53-hostedzone.html#cfn-route53-hostedzone-hostedzonetags
         */
        HostedZoneTags?: C.Yields<C.CFList<C.Yields<AWS.Route53.HostedZone.HostedZoneTag>>>;
        /**
         * String

         *
         * The name of the domain. For resource record types that include a domain name,
 specify a fully qualified domain name.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-route53-hostedzone.html#cfn-route53-hostedzone-name
         */
        Name: C.YieldsString;
        /**
         * List of Amazon Route 53 HostedZoneVPCs
         *
         * One or more VPCs that you want to associate with this hosted zone. When you
 specify this property, AWS CloudFormation creates a private hosted zone.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-route53-hostedzone.html#cfn-route53-hostedzone-vpcs
         */
        VPCs?: C.Yields<C.CFList<C.Yields<AWS.Route53.HostedZone.VPC>>>;
    }
}
export declare namespace AWS.Route53.HostedZone {
    interface HostedZoneConfig {
        /**
         * String

         *
         * Any comments that you want to include about the hosted zone.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-hostedzone-hostedzoneconfig.html#cfn-route53-hostedzone-hostedzoneconfig-comment
         */
        Comment?: C.YieldsString;
    }
}
export declare namespace AWS.Route53.HostedZone {
    interface HostedZoneTag {
        /**
         * String

         *
         * The key name of the tag.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-hostedzone-hostedzonetags.html#cfn-route53-hostedzonetags-key
         */
        Key: C.YieldsString;
        /**
         * String

         *
         * The value for the tag.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-hostedzone-hostedzonetags.html#cfn-route53-hostedzonetags-value
         */
        Value: C.YieldsString;
    }
}
export declare namespace AWS.Route53.HostedZone {
    interface VPC {
        /**
         * String

         *
         * The ID of the Amazon VPC that you want to associate with the hosted zone.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-route53-hostedzone-hostedzonevpcs.html#cfn-route53-hostedzone-hostedzonevpcs-vpcid
         */
        VPCId: C.YieldsString;
        /**
         * String

         *
         * The region in which the Amazon VPC was created as specified in the
 VPCId property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-route53-hostedzone-hostedzonevpcs.html#cfn-route53-hostedzone-hostedzonevpcs-vpcregion
         */
        VPCRegion: C.YieldsString;
    }
}
export declare namespace AWS.Route53 {
    /**
     * The AWS::Route53::RecordSet type can be used as a standalone resource or as an
    embedded property in the AWS::Route53::RecordSetGroup type. Note that some
    AWS::Route53::RecordSet properties are valid only when used within
    AWS::Route53::RecordSetGroup.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html
     */
    interface RecordSet {
        Type: 'AWS::Route53::RecordSet';
        Properties: AWS.Route53.RecordSet.Properties;
    }
    /**
     * The AWS::Route53::RecordSet type can be used as a standalone resource or as an
    embedded property in the AWS::Route53::RecordSetGroup type. Note that some
    AWS::Route53::RecordSet properties are valid only when used within
    AWS::Route53::RecordSetGroup.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html
     */
    function RecordSet(props: C.Omit<RecordSet, 'Type'>): RecordSet;
}
export declare namespace AWS.Route53.RecordSet {
    interface Properties {
        /**
         * AliasTarget
         *
         * Alias resource record sets only: Information about the
 domain to which you are redirecting traffic.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-aliastarget
         */
        AliasTarget?: C.Yields<AWS.Route53.RecordSet.AliasTarget>;
        /**
         * String

         *
         * Any comments that you want to include about the hosted zone.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-comment
         */
        Comment?: C.YieldsString;
        /**
         * String

         *
         * Designates the record set as a PRIMARY or SECONDARY
 failover record set. When you have more than one resource performing the same
 function, you can configure Amazon Route 53 to check the health of your resources
 and use
 only health resources to respond to DNS queries. You cannot create nonfailover
 resource record sets that have the same Name and Type
 property values as failover resource record sets. For more information, see the
 Failover content in the Amazon Route 53 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-failover
         */
        Failover?: C.YieldsString;
        /**
         * Amazon Route 53 Record Set GeoLocation
    Property
         *
         * Describes how Amazon Route 53 responds to DNS queries based on the
 geographic origin of the query. This property is not
 compatible with the Region
 property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-geolocation
         */
        GeoLocation?: C.Yields<AWS.Route53.RecordSet.GeoLocation>;
        /**
         * String

         *
         * The health check ID that you want to apply to this record set. Amazon Route 53 returns
 this resource record set in response to a DNS query only while record set is
 healthy.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-healthcheckid
         */
        HealthCheckId?: C.YieldsString;
        /**
         * String

         *
         * The ID of the hosted zone.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-hostedzoneid
         */
        HostedZoneId?: C.YieldsString;
        /**
         * String

         *
         * The name of the domain for the hosted zone where you want to add the record
 set.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-hostedzonename
         */
        HostedZoneName?: C.YieldsString;
        /**
         * String

         *
         * The name of the domain. You must specify a fully qualified domain name that
 ends with a period as the last label indication. If you omit the final period,
 Amazon Route 53 adds it.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-name
         */
        Name: C.YieldsString;
        /**
         *
         *
         * Latency resource record sets only: The Amazon EC2 region where the resource that is
 specified in this resource record set resides. The resource typically is an AWS
 resource, for example, Amazon EC2 instance or an Elastic Load Balancing load balancer,
 and is referred
 to by an IP address or a DNS domain name, depending on the record type.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-region
         */
        Region?: C.YieldsString;
        /**
         * List of strings

         *
         * List of resource records to add. Each record should be in the format
 appropriate for the record type specified by the Type
 property. For information about different record types and their record formats,
 see Values for Basic Resource Record Sets and
 Appendix: Domain Name Format in the Amazon Route 53 Developer
    Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-resourcerecords
         */
        ResourceRecords?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * A unique identifier that differentiates among multiple resource record sets
 that have the same combination of DNS name and type.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-setidentifier
         */
        SetIdentifier?: C.YieldsString;
        /**
         * String

         *
         * The resource record cache time to live (TTL), in seconds. If you specify this
 property, do not specify the AliasTarget property. For alias target
 records, the alias uses a TTL value from the target.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-ttl
         */
        TTL?: C.YieldsString;
        /**
         * String

         *
         * The type of records to add. For valid values, see the Type content in the Amazon Route 53 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-type
         */
        Type: C.YieldsString;
        /**
         * Number. Weight expects integer values.

         *
         * Weighted resource record sets only: Among resource record
 sets that have the same combination of DNS name and type, a value that determines
 what portion of traffic for the current resource record set is routed to the
 associated location.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-weight
         */
        Weight?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.Route53.RecordSet {
    interface AliasTarget {
        /**
         * String

         *
         * The DNS name of the load balancer, the domain name of the CloudFront distribution,
 the website endpoint of the Amazon S3 bucket, or another record set in the same hosted
 zone that is the target of the alias.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-aliastarget.html#cfn-route53-aliastarget-dnshostname
         */
        DNSName: C.YieldsString;
        /**
         * Boolean

         *
         * Whether Amazon Route 53 checks the health of the resource record sets in the alias
 target
 when responding to DNS queries. For more information about using this property,
 see EvaluateTargetHealth in the
 Amazon Route 53 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-aliastarget.html#cfn-route53-aliastarget-evaluatetargethealth
         */
        EvaluateTargetHealth?: C.YieldsBoolean;
        /**
         * String

         *
         * The hosted zone ID. For load balancers, use the canonical hosted zone ID of the
 load balancer. For Amazon S3, use the hosted zone ID for your bucket's website
 endpoint. For CloudFront, use Z2FDTNDATAQYW2. For a list of hosted zone IDs
 of other services, see the relevant service in the AWS Regions and Endpoints.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-aliastarget.html#cfn-route53-aliastarget-hostedzoneid
         */
        HostedZoneId: C.YieldsString;
    }
}
export declare namespace AWS.Route53.RecordSet {
    interface GeoLocation {
        /**
         * String

         *
         * All DNS queries from the continent that you specified are routed to this
 resource record set. If you specify this property, omit the
 CountryCode and SubdivisionCode properties.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset-geolocation.html#cfn-route53-recordset-geolocation-continentcode
         */
        ContinentCode?: C.YieldsString;
        /**
         * String

         *
         * All DNS queries from the country that you specified are routed to this resource
 record set. If you specify this property, omit the ContinentCode
 property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset-geolocation.html#cfn-route53-recordset-geolocation-countrycode
         */
        CountryCode?: C.YieldsString;
        /**
         * String

         *
         * If you specified US for the country code, you can specify a state
 in the United States. All DNS queries from the state that you specified are routed
 to this resource record set. If you specify this property, you must specify
 US for the CountryCode and omit the
 ContinentCode property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset-geolocation.html#cfn-route53-recordset-geolocation-subdivisioncode
         */
        SubdivisionCode?: C.YieldsString;
    }
}
export declare namespace AWS.Route53 {
    /**
     * The AWS::Route53::RecordSetGroup resource creates record sets for a hosted zone. For
    more
    information about constraints and values for each property, see POST
       CreateHostedZone for hosted zones and POST
       ChangeResourceRecordSet for resource record sets.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-route53-recordsetgroup.html
     */
    interface RecordSetGroup {
        Type: 'AWS::Route53::RecordSetGroup';
        Properties: AWS.Route53.RecordSetGroup.Properties;
    }
    /**
     * The AWS::Route53::RecordSetGroup resource creates record sets for a hosted zone. For
    more
    information about constraints and values for each property, see POST
       CreateHostedZone for hosted zones and POST
       ChangeResourceRecordSet for resource record sets.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-route53-recordsetgroup.html
     */
    function RecordSetGroup(props: C.Omit<RecordSetGroup, 'Type'>): RecordSetGroup;
}
export declare namespace AWS.Route53.RecordSetGroup {
    interface Properties {
        /**
         * String

         *
         * Any comments you want to include about the hosted zone.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-route53-recordsetgroup.html#cfn-route53-recordsetgroup-comment
         */
        Comment?: C.YieldsString;
        /**
         * String

         *
         * The ID of the hosted zone.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-route53-recordsetgroup.html#cfn-route53-recordsetgroup-hostedzoneid
         */
        HostedZoneId?: C.YieldsString;
        /**
         * String

         *
         * The name of the domain for the hosted zone where you want to add the record set.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-route53-recordsetgroup.html#cfn-route53-recordsetgroup-hostedzonename
         */
        HostedZoneName?: C.YieldsString;
        /**
         *  List of AWS::Route53::RecordSet objects, as shown in the
 following example:
 

         *
         * List of resource record sets to add.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-route53-recordsetgroup.html#cfn-route53-recordsetgroup-recordsets
         */
        RecordSets?: C.Yields<C.CFList<C.Yields<AWS.Route53.RecordSetGroup.RecordSet>>>;
    }
}
export declare namespace AWS.Route53.RecordSetGroup {
    interface AliasTarget {
        /**
         * String

         *
         * The DNS name of the load balancer, the domain name of the CloudFront distribution,
 the website endpoint of the Amazon S3 bucket, or another record set in the same hosted
 zone that is the target of the alias.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-aliastarget.html#cfn-route53-aliastarget-dnshostname
         */
        DNSName: C.YieldsString;
        /**
         * Boolean

         *
         * Whether Amazon Route 53 checks the health of the resource record sets in the alias
 target
 when responding to DNS queries. For more information about using this property,
 see EvaluateTargetHealth in the
 Amazon Route 53 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-aliastarget.html#cfn-route53-aliastarget-evaluatetargethealth
         */
        EvaluateTargetHealth?: C.YieldsBoolean;
        /**
         * String

         *
         * The hosted zone ID. For load balancers, use the canonical hosted zone ID of the
 load balancer. For Amazon S3, use the hosted zone ID for your bucket's website
 endpoint. For CloudFront, use Z2FDTNDATAQYW2. For a list of hosted zone IDs
 of other services, see the relevant service in the AWS Regions and Endpoints.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-aliastarget.html#cfn-route53-aliastarget-hostedzoneid
         */
        HostedZoneId: C.YieldsString;
    }
}
export declare namespace AWS.Route53.RecordSetGroup {
    interface GeoLocation {
        /**
         * String

         *
         * All DNS queries from the continent that you specified are routed to this
 resource record set. If you specify this property, omit the
 CountryCode and SubdivisionCode properties.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset-geolocation.html#cfn-route53-recordsetgroup-geolocation-continentcode
         */
        ContinentCode?: C.YieldsString;
        /**
         * String

         *
         * All DNS queries from the country that you specified are routed to this resource
 record set. If you specify this property, omit the ContinentCode
 property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset-geolocation.html#cfn-route53-recordset-geolocation-countrycode
         */
        CountryCode?: C.YieldsString;
        /**
         * String

         *
         * If you specified US for the country code, you can specify a state
 in the United States. All DNS queries from the state that you specified are routed
 to this resource record set. If you specify this property, you must specify
 US for the CountryCode and omit the
 ContinentCode property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset-geolocation.html#cfn-route53-recordset-geolocation-subdivisioncode
         */
        SubdivisionCode?: C.YieldsString;
    }
}
export declare namespace AWS.Route53.RecordSetGroup {
    interface RecordSet {
        /**
         * AliasTarget
         *
         * Alias resource record sets only: Information about the
 domain to which you are redirecting traffic.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-aliastarget
         */
        AliasTarget?: C.Yields<AWS.Route53.RecordSetGroup.AliasTarget>;
        /**
         * String

         *
         * Any comments that you want to include about the hosted zone.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-comment
         */
        Comment?: C.YieldsString;
        /**
         * String

         *
         * Designates the record set as a PRIMARY or SECONDARY
 failover record set. When you have more than one resource performing the same
 function, you can configure Amazon Route 53 to check the health of your resources
 and use
 only health resources to respond to DNS queries. You cannot create nonfailover
 resource record sets that have the same Name and Type
 property values as failover resource record sets. For more information, see the
 Failover content in the Amazon Route 53 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-failover
         */
        Failover?: C.YieldsString;
        /**
         * Amazon Route 53 Record Set GeoLocation
    Property
         *
         * Describes how Amazon Route 53 responds to DNS queries based on the
 geographic origin of the query. This property is not
 compatible with the Region
 property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-geolocation
         */
        GeoLocation?: C.Yields<AWS.Route53.RecordSetGroup.GeoLocation>;
        /**
         * String

         *
         * The health check ID that you want to apply to this record set. Amazon Route 53 returns
 this resource record set in response to a DNS query only while record set is
 healthy.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-healthcheckid
         */
        HealthCheckId?: C.YieldsString;
        /**
         * String

         *
         * The ID of the hosted zone.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-hostedzoneid
         */
        HostedZoneId?: C.YieldsString;
        /**
         * String

         *
         * The name of the domain for the hosted zone where you want to add the record
 set.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-hostedzonename
         */
        HostedZoneName?: C.YieldsString;
        /**
         * String

         *
         * The name of the domain. You must specify a fully qualified domain name that
 ends with a period as the last label indication. If you omit the final period,
 Amazon Route 53 adds it.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-name
         */
        Name: C.YieldsString;
        /**
         *
         *
         * Latency resource record sets only: The Amazon EC2 region where the resource that is
 specified in this resource record set resides. The resource typically is an AWS
 resource, for example, Amazon EC2 instance or an Elastic Load Balancing load balancer,
 and is referred
 to by an IP address or a DNS domain name, depending on the record type.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-region
         */
        Region?: C.YieldsString;
        /**
         * List of strings

         *
         * List of resource records to add. Each record should be in the format
 appropriate for the record type specified by the Type
 property. For information about different record types and their record formats,
 see Values for Basic Resource Record Sets and
 Appendix: Domain Name Format in the Amazon Route 53 Developer
    Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-resourcerecords
         */
        ResourceRecords?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * A unique identifier that differentiates among multiple resource record sets
 that have the same combination of DNS name and type.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-setidentifier
         */
        SetIdentifier?: C.YieldsString;
        /**
         * String

         *
         * The resource record cache time to live (TTL), in seconds. If you specify this
 property, do not specify the AliasTarget property. For alias target
 records, the alias uses a TTL value from the target.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-ttl
         */
        TTL?: C.YieldsString;
        /**
         * String

         *
         * The type of records to add. For valid values, see the Type content in the Amazon Route 53 API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-type
         */
        Type: C.YieldsString;
        /**
         * Number. Weight expects integer values.

         *
         * Weighted resource record sets only: Among resource record
 sets that have the same combination of DNS name and type, a value that determines
 what portion of traffic for the current resource record set is routed to the
 associated location.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-route53-recordset.html#cfn-route53-recordset-weight
         */
        Weight?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.S3 {
    /**
     * The AWS::S3::Bucket resource creates an Amazon Simple Storage Service (Amazon S3) bucket in the same
    AWS
    Region in which you create the AWS CloudFormation stack.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket.html
     */
    interface Bucket {
        Type: 'AWS::S3::Bucket';
        Properties: AWS.S3.Bucket.Properties;
    }
    /**
     * The AWS::S3::Bucket resource creates an Amazon Simple Storage Service (Amazon S3) bucket in the same
    AWS
    Region in which you create the AWS CloudFormation stack.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket.html
     */
    function Bucket(props: C.Omit<Bucket, 'Type'>): Bucket;
}
export declare namespace AWS.S3.Bucket {
    interface Properties {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket.html#cfn-s3-bucket-accelerateconfiguration
         */
        AccelerateConfiguration?: C.Yields<AWS.S3.Bucket.AccelerateConfiguration>;
        /**
         * String

         *
         * A canned access control list (ACL) that grants predefined permissions to the
 bucket. For more information about canned ACLs, see Canned ACLs in the Amazon S3
    documentation  in the
 Amazon Simple Storage Service Developer Guide..

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket.html#cfn-s3-bucket-accesscontrol
         */
        AccessControl?: C.YieldsString;
        /**
         * String

         *
         * A name for the bucket. If you don't specify a name, AWS CloudFormation generates a
 unique
 physical ID and uses that ID for the bucket name. For more information, see Name Type. The bucket
 name must contain only lowercase letters, numbers, periods (.), and dashes
 (-).

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket.html#cfn-s3-bucket-name
         */
        BucketName?: C.YieldsString;
        /**
         * Amazon S3 Cors Configuration
         *
         * Rules that define cross-origin resource sharing of objects in this bucket.
 For more information, see Enabling
    Cross-Origin Resource Sharing in the
 Amazon Simple Storage Service Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket.html#cfn-s3-bucket-crossoriginconfig
         */
        CorsConfiguration?: C.Yields<AWS.S3.Bucket.CorsConfiguration>;
        /**
         * Amazon S3 Lifecycle Configuration
         *
         * Rules that define how Amazon S3 manages objects during their lifetime. For more
 information, see Object
    Lifecycle Management in the
 Amazon Simple Storage Service Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket.html#cfn-s3-bucket-lifecycleconfig
         */
        LifecycleConfiguration?: C.Yields<AWS.S3.Bucket.LifecycleConfiguration>;
        /**
         * Amazon S3 Logging Configuration
         *
         * Settings that define where logs are stored.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket.html#cfn-s3-bucket-loggingconfig
         */
        LoggingConfiguration?: C.Yields<AWS.S3.Bucket.LoggingConfiguration>;
        /**
         *                             Type: List of Amazon S3 Bucket MetricsConfiguration
         *
         * Settings that define a metrics configuration for the CloudWatch request metrics from
 the bucket.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket.html#cfn-s3-bucket-metricsconfigurations
         */
        MetricsConfigurations?: C.Yields<C.CFList<C.Yields<AWS.S3.Bucket.MetricsConfiguration>>>;
        /**
         * Amazon S3
    NotificationConfiguration
         *
         * Configuration that defines how Amazon S3 handles bucket notifications.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket.html#cfn-s3-bucket-notification
         */
        NotificationConfiguration?: C.Yields<AWS.S3.Bucket.NotificationConfiguration>;
        /**
         * Amazon S3
    ReplicationConfiguration
         *
         * Configuration for replicating objects in an S3 bucket. To enable replication, you
 must also enable versioning by using the VersioningConfiguration property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket.html#cfn-s3-bucket-replicationconfiguration
         */
        ReplicationConfiguration?: C.Yields<AWS.S3.Bucket.ReplicationConfiguration>;
        /**
         * AWS CloudFormation Resource Tags
         *
         * An arbitrary set of tags (key-value pairs) for this S3 bucket.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket.html#cfn-s3-bucket-tags
         */
        Tags?: C.Yields<C.CFList<C.Yields<Tag>>>;
        /**
         * Amazon S3 Versioning Configuration
         *
         * Enables multiple variants of all objects in this bucket. You might enable
 versioning to prevent objects from being deleted or overwritten by mistake
 or to archive objects so that you can retrieve previous versions of
 them.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket.html#cfn-s3-bucket-versioning
         */
        VersioningConfiguration?: C.Yields<AWS.S3.Bucket.VersioningConfiguration>;
        /**
         * Website Configuration
    Type
         *
         * Information used to configure the bucket as a static website. For more information,
 see Hosting Websites on Amazon
    S3.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket.html#cfn-s3-bucket-websiteconfiguration
         */
        WebsiteConfiguration?: C.Yields<AWS.S3.Bucket.WebsiteConfiguration>;
    }
}
export declare namespace AWS.S3.Bucket {
    interface AbortIncompleteMultipartUpload {
        /**
         *                             Type: Integer

         *
         * The number of days after the upload is initiated before aborting the upload.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-abortincompletemultipartupload.html#cfn-s3-bucket-abortincompletemultipartupload-daysafterinitiation
         */
        DaysAfterInitiation: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.S3.Bucket {
    interface AccelerateConfiguration {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-accelerateconfiguration.html#cfn-s3-bucket-accelerateconfiguration-accelerationstatus
         */
        AccelerationStatus: C.YieldsString;
    }
}
export declare namespace AWS.S3.Bucket {
    interface CorsConfiguration {
        /**
         * Amazon S3 Cors Configuration Rule
         *
         * A set of origins and methods that you allow.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-cors.html#cfn-s3-bucket-cors-corsrule
         */
        CorsRules: C.Yields<C.CFList<C.Yields<AWS.S3.Bucket.CorsRule>>>;
    }
}
export declare namespace AWS.S3.Bucket {
    interface CorsRule {
        /**
         * List of strings

         *
         * Headers that are specified in the Access-Control-Request-Headers
 header. These headers are allowed in a preflight OPTIONS request. In response to
 any preflight OPTIONS request, Amazon S3 returns any requested headers that are
 allowed.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-cors-corsrule.html#cfn-s3-bucket-cors-corsrule-allowedheaders
         */
        AllowedHeaders?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of strings

         *
         * An HTTP method that you allow the origin to execute. The valid values are
 GET, PUT, HEAD, POST, and
 DELETE.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-cors-corsrule.html#cfn-s3-bucket-cors-corsrule-allowedmethods
         */
        AllowedMethods: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of strings

         *
         * An origin that you allow to send cross-domain requests.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-cors-corsrule.html#cfn-s3-bucket-cors-corsrule-allowedorigins
         */
        AllowedOrigins: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * List of strings

         *
         * One or more headers in the response that are accessible to client applications
 (for example, from a JavaScript XMLHttpRequest object).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-cors-corsrule.html#cfn-s3-bucket-cors-corsrule-exposedheaders
         */
        ExposedHeaders?: C.Yields<C.CFList<C.YieldsString>>;
        /**
         * String

         *
         * A unique identifier for this rule. The value cannot be more than 255
 characters.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-cors-corsrule.html#cfn-s3-bucket-cors-corsrule-id
         */
        Id?: C.YieldsString;
        /**
         * Integer

         *
         * The time in seconds that your browser is to cache the preflight response for
 the specified resource.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-cors-corsrule.html#cfn-s3-bucket-cors-corsrule-maxage
         */
        MaxAge?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.S3.Bucket {
    interface FilterRule {
        /**
         * String

         *
         * Whether the filter matches the prefix or suffix of object key names. For valid
 values, see the Name request element of the PUT Bucket
    notification action in the
 Amazon Simple Storage Service API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-notificationconfiguration-config-filter-s3key-rules.html#cfn-s3-bucket-notificationconfiguraiton-config-filter-s3key-rules-name
         */
        Name: C.YieldsString;
        /**
         * String

         *
         * The value that the filter searches for in object key names.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-notificationconfiguration-config-filter-s3key-rules.html#cfn-s3-bucket-notificationconfiguraiton-config-filter-s3key-rules-value
         */
        Value: C.YieldsString;
    }
}
export declare namespace AWS.S3.Bucket {
    interface LambdaConfiguration {
        /**
         * String

         *
         * The S3 bucket event for which to invoke the Lambda function.
 For more information, see Supported Event Types in the
 Amazon Simple Storage Service Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-notificationconfig-lambdaconfig.html#cfn-s3-bucket-notificationconfig-lambdaconfig-event
         */
        Event: C.YieldsString;
        /**
         * Amazon S3
    NotificationConfiguration Config Filter
         *
         * The filtering rules that determine which objects invoke the Lambda function. For
 example, you can create a filter so that only image files with a
 .jpg extension invoke the function when they are added to
 the S3 bucket.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-notificationconfig-lambdaconfig.html#cfn-s3-bucket-notificationconfig-lambdaconfig-filter
         */
        Filter?: C.Yields<AWS.S3.Bucket.NotificationFilter>;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the Lambda function that Amazon S3 invokes when
 the
 specified event type occurs.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-notificationconfig-lambdaconfig.html#cfn-s3-bucket-notificationconfig-lambdaconfig-function
         */
        Function: C.YieldsString;
    }
}
export declare namespace AWS.S3.Bucket {
    interface LifecycleConfiguration {
        /**
         * Amazon S3 Lifecycle Rule
         *
         * A lifecycle rule for individual objects in an S3 bucket.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-lifecycleconfig.html#cfn-s3-bucket-lifecycleconfig-rules
         */
        Rules: C.Yields<C.CFList<C.Yields<AWS.S3.Bucket.Rule>>>;
    }
}
export declare namespace AWS.S3.Bucket {
    interface LoggingConfiguration {
        /**
         * String

         *
         * The name of an Amazon S3 bucket where Amazon S3 store server access log files. You
 can
 store log files in any bucket that you own. By default, logs are stored in the
 bucket where the LoggingConfiguration property is defined.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-loggingconfig.html#cfn-s3-bucket-loggingconfig-destinationbucketname
         */
        DestinationBucketName?: C.YieldsString;
        /**
         * String

         *
         * A prefix for the all log object keys. If you store log files from multiple Amazon
 S3
 buckets in a single bucket, you can use a prefix to distinguish which log files
 came from which bucket.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-loggingconfig.html#cfn-s3-bucket-loggingconfig-logfileprefix
         */
        LogFilePrefix?: C.YieldsString;
    }
}
export declare namespace AWS.S3.Bucket {
    interface MetricsConfiguration {
        /**
         *                             Type: String

         *
         * The ID used to identify the metrics configuration.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-metricsconfiguration.html#cfn-s3-bucket-metricsconfiguration-id
         */
        Id: C.YieldsString;
        /**
         *                             Type: String

         *
         * The prefix that an object must have to be included in the metrics results.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-metricsconfiguration.html#cfn-s3-bucket-metricsconfiguration-prefix
         */
        Prefix?: C.YieldsString;
        /**
         *                             Type: List of Amazon S3  Bucket TagFilter
         *
         * Specifies a list of tag filters to use as a metrics configuration filter. The metrics
 configuration includes only objects that meet the filter's criteria.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-metricsconfiguration.html#cfn-s3-bucket-metricsconfiguration-tagfilters
         */
        TagFilters?: C.Yields<C.CFList<C.Yields<AWS.S3.Bucket.TagFilter>>>;
    }
}
export declare namespace AWS.S3.Bucket {
    interface NoncurrentVersionTransition {
        /**
         * String

         *
         * The storage class to which you want the object to transition, such as
 GLACIER. For valid values, see the StorageClass
 request element of the PUT
    Bucket lifecycle action in the
 Amazon Simple Storage Service API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-lifecycleconfig-rule-noncurrentversiontransition.html#cfn-s3-bucket-lifecycleconfig-rule-noncurrentversiontransition-storageclass
         */
        StorageClass: C.YieldsString;
        /**
         * Integer

         *
         * The number of days between the time that a new version of the object is
 uploaded to the bucket and when old versions of the object are transitioned to the
 specified storage class.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-lifecycleconfig-rule-noncurrentversiontransition.html#cfn-s3-bucket-lifecycleconfig-rule-noncurrentversiontransition-transitionindays
         */
        TransitionInDays: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.S3.Bucket {
    interface NotificationConfiguration {
        /**
         * Amazon Simple Storage Service
    NotificationConfiguration LambdaConfigurations
         *
         * The AWS Lambda functions to invoke and the events for which to invoke the
 functions.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-notificationconfig.html#cfn-s3-bucket-notificationconfig-lambdaconfig
         */
        LambdaConfigurations?: C.Yields<C.CFList<C.Yields<AWS.S3.Bucket.LambdaConfiguration>>>;
        /**
         * Amazon Simple Storage Service
    NotificationConfiguration QueueConfigurations
         *
         * The Amazon Simple Queue Service queues to publish messages to and the events for which
 to publish
 messages.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-notificationconfig.html#cfn-s3-bucket-notificationconfig-queueconfig
         */
        QueueConfigurations?: C.Yields<C.CFList<C.Yields<AWS.S3.Bucket.QueueConfiguration>>>;
        /**
         * Amazon S3
    NotificationConfiguration TopicConfigurations
         *
         * The topic to which notifications are sent and the events for which notification
 are generated.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-notificationconfig.html#cfn-s3-bucket-notificationconfig-topicconfig
         */
        TopicConfigurations?: C.Yields<C.CFList<C.Yields<AWS.S3.Bucket.TopicConfiguration>>>;
    }
}
export declare namespace AWS.S3.Bucket {
    interface NotificationFilter {
        /**
         * Amazon S3
    NotificationConfiguration Config Filter S3Key
         *
         * Amazon S3 filtering rules that describe for which object key names to send
 notifications.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-notificationconfiguration-config-filter.html#cfn-s3-bucket-notificationconfiguraiton-config-filter-s3key
         */
        S3Key: C.Yields<AWS.S3.Bucket.S3KeyFilter>;
    }
}
export declare namespace AWS.S3.Bucket {
    interface QueueConfiguration {
        /**
         * String

         *
         * The S3 bucket event about which you want to publish messages to Amazon Simple Queue
 Service (
 Amazon SQS). For more information, see Supported Event Types in the
 Amazon Simple Storage Service Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-notificationconfig-queueconfig.html#cfn-s3-bucket-notificationconfig-queueconfig-event
         */
        Event: C.YieldsString;
        /**
         * Amazon S3
    NotificationConfiguration Config Filter
         *
         * The filtering rules that determine for which objects to send notifications. For
 example, you can create a filter so that Amazon Simple Storage Service (Amazon S3)
 sends notifications only
 when image files with a .jpg extension are added to the
 bucket.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-notificationconfig-queueconfig.html#cfn-s3-bucket-notificationconfig-queueconfig-filter
         */
        Filter?: C.Yields<AWS.S3.Bucket.NotificationFilter>;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the Amazon SQS queue that Amazon S3 publishes messages
 to when the specified event type occurs.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-notificationconfig-queueconfig.html#cfn-s3-bucket-notificationconfig-queueconfig-queue
         */
        Queue: C.YieldsString;
    }
}
export declare namespace AWS.S3.Bucket {
    interface RedirectAllRequestsTo {
        /**
         * String

         *
         * Name of the host where requests are redirected.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-websiteconfiguration-redirectallrequeststo.html#cfn-s3-websiteconfiguration-redirectallrequeststo-hostname
         */
        HostName: C.YieldsString;
        /**
         * String

         *
         * Protocol to use (http or https) when redirecting
 requests. The default is the protocol that is used in the original request.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-websiteconfiguration-redirectallrequeststo.html#cfn-s3-websiteconfiguration-redirectallrequeststo-protocol
         */
        Protocol?: C.YieldsString;
    }
}
export declare namespace AWS.S3.Bucket {
    interface RedirectRule {
        /**
         * String

         *
         * Name of the host where requests are redirected.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-websiteconfiguration-routingrules-redirectrule.html#cfn-s3-websiteconfiguration-redirectrule-hostname
         */
        HostName?: C.YieldsString;
        /**
         * String

         *
         * The HTTP redirect code to use on the response.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-websiteconfiguration-routingrules-redirectrule.html#cfn-s3-websiteconfiguration-redirectrule-httpredirectcode
         */
        HttpRedirectCode?: C.YieldsString;
        /**
         * String

         *
         * The protocol to use in the redirect request.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-websiteconfiguration-routingrules-redirectrule.html#cfn-s3-websiteconfiguration-redirectrule-protocol
         */
        Protocol?: C.YieldsString;
        /**
         * String

         *
         * The object key prefix to use in the redirect request. For example, to redirect
 requests for all pages with the prefix docs/ (objects in the
 docs/ folder) to the documents/ prefix, you can set
 the KeyPrefixEquals property in routing condition property to
 docs/, and set the ReplaceKeyPrefixWith property to documents/.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-websiteconfiguration-routingrules-redirectrule.html#cfn-s3-websiteconfiguration-redirectrule-replacekeyprefixwith
         */
        ReplaceKeyPrefixWith?: C.YieldsString;
        /**
         * String

         *
         * The specific object key to use in the redirect request. For example, redirect
 request to error.html.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-websiteconfiguration-routingrules-redirectrule.html#cfn-s3-websiteconfiguration-redirectrule-replacekeywith
         */
        ReplaceKeyWith?: C.YieldsString;
    }
}
export declare namespace AWS.S3.Bucket {
    interface ReplicationConfiguration {
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of an AWS Identity and Access Management (IAM) role
 that Amazon S3 assumes
 when replicating objects. For more information, see How to Set Up Cross-Region
    Replication in the Amazon Simple Storage Service Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-replicationconfiguration.html#cfn-s3-bucket-replicationconfiguration-role
         */
        Role: C.YieldsString;
        /**
         * List of Amazon S3
    ReplicationConfiguration Rule
         *
         * A replication rule that specifies which objects to replicate and where they are
 stored.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-replicationconfiguration.html#cfn-s3-bucket-replicationconfiguration-rules
         */
        Rules: C.Yields<C.CFList<C.Yields<AWS.S3.Bucket.ReplicationRule>>>;
    }
}
export declare namespace AWS.S3.Bucket {
    interface ReplicationDestination {
        /**
         * String

         *
         * The Amazon resource name (ARN) of an S3 bucket where Amazon S3 stores replicated
 objects. This destination bucket must be in a different region than your source
 bucket.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-replicationconfiguration-rules-destination.html#cfn-s3-bucket-replicationconfiguration-rules-destination-bucket
         */
        Bucket: C.YieldsString;
        /**
         * String

         *
         * The storage class to use when replicating objects, such as standard or reduced
 redundancy. By default, Amazon S3 uses the storage class of the source object to create
 object replica. For valid values, see the StorageClass element of the
 PUT Bucket
    replication action in the Amazon Simple Storage Service API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-replicationconfiguration-rules-destination.html#cfn-s3-bucket-replicationconfiguration-rules-destination-storageclass
         */
        StorageClass?: C.YieldsString;
    }
}
export declare namespace AWS.S3.Bucket {
    interface ReplicationRule {
        /**
         * Amazon S3
    ReplicationConfiguration Rules Destination
         *
         * Defines the destination where Amazon S3 stores replicated objects.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-replicationconfiguration-rules.html#cfn-s3-bucket-replicationconfiguration-rules-destination
         */
        Destination: C.Yields<AWS.S3.Bucket.ReplicationDestination>;
        /**
         * String

         *
         * A unique identifier for the rule. If you don't specify a value, AWS CloudFormation
 generates
 a random ID.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-replicationconfiguration-rules.html#cfn-s3-bucket-replicationconfiguration-rules-id
         */
        Id?: C.YieldsString;
        /**
         * String

         *
         * An object prefix. This rule applies to all Amazon S3 objects with this prefix. To
 specify all objects in an S3 bucket, specify an empty string.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-replicationconfiguration-rules.html#cfn-s3-bucket-replicationconfiguration-rules-prefix
         */
        Prefix: C.YieldsString;
        /**
         * String

         *
         * Whether the rule is enabled. For valid values, see the Status
 element of the PUT Bucket
    replication action in the Amazon Simple Storage Service API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-replicationconfiguration-rules.html#cfn-s3-bucket-replicationconfiguration-rules-status
         */
        Status: C.YieldsString;
    }
}
export declare namespace AWS.S3.Bucket {
    interface RoutingRule {
        /**
         * Amazon S3 Website
    Configuration Routing Rules Redirect Rule Property
         *
         * Redirect requests to another host, to another page, or with another
 protocol.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-websiteconfiguration-routingrules.html#cfn-s3-websiteconfiguration-routingrules-redirectrule
         */
        RedirectRule: C.Yields<AWS.S3.Bucket.RedirectRule>;
        /**
         * Amazon S3
    Website Configuration Routing Rules Routing Rule Condition Property
         *
         * Rules that define when a redirect is applied.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-websiteconfiguration-routingrules.html#cfn-s3-websiteconfiguration-routingrules-routingrulecondition
         */
        RoutingRuleCondition?: C.Yields<AWS.S3.Bucket.RoutingRuleCondition>;
    }
}
export declare namespace AWS.S3.Bucket {
    interface RoutingRuleCondition {
        /**
         * String

         *
         * Applies this redirect if the error code equals this value in the event of an
 error.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-websiteconfiguration-routingrules-routingrulecondition.html#cfn-s3-websiteconfiguration-routingrules-routingrulecondition-httperrorcodereturnedequals
         */
        HttpErrorCodeReturnedEquals?: C.YieldsString;
        /**
         * String

         *
         * The object key name prefix when the redirect is applied. For example, to
 redirect requests for ExamplePage.html, set the key prefix to
 ExamplePage.html. To redirect request for all pages with the
 prefix docs/, set the key prefix to docs/, which
 identifies all objects in the docs/ folder.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-websiteconfiguration-routingrules-routingrulecondition.html#cfn-s3-websiteconfiguration-routingrules-routingrulecondition-keyprefixequals
         */
        KeyPrefixEquals?: C.YieldsString;
    }
}
export declare namespace AWS.S3.Bucket {
    interface Rule {
        /**
         * Amazon S3 Bucket AbortIncompleteMultipartUpload
         *
         * Specifies a lifecycle rule that aborts incomplete multipart uploads to an Amazon S3
 bucket.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-lifecycleconfig-rule.html#cfn-s3-bucket-rule-abortincompletemultipartupload
         */
        AbortIncompleteMultipartUpload?: C.Yields<AWS.S3.Bucket.AbortIncompleteMultipartUpload>;
        /**
         * String

         *
         * Indicates when objects are deleted from Amazon S3 and Amazon Glacier. The date value
 must be
 in ISO 8601 format. The time is always midnight UTC. If you specify an expiration
 and transition time, you must use the same time unit for both properties (either
 in days or by date). The expiration time must also be later than the transition
 time.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-lifecycleconfig-rule.html#cfn-s3-bucket-lifecycleconfig-rule-expirationdate
         */
        ExpirationDate?: C.Yields<C.CFTimestamp>;
        /**
         * Integer

         *
         * Indicates the number of days after creation when objects are deleted from Amazon S3
 and Amazon Glacier. If you specify an expiration and transition time, you must use
 the same
 time unit for both properties (either in days or by date). The expiration time
 must also be later than the transition time.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-lifecycleconfig-rule.html#cfn-s3-bucket-lifecycleconfig-rule-expirationindays
         */
        ExpirationInDays?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * A unique identifier for this rule. The value cannot be more than 255
 characters.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-lifecycleconfig-rule.html#cfn-s3-bucket-lifecycleconfig-rule-id
         */
        Id?: C.YieldsString;
        /**
         * Integer

         *
         * For buckets with versioning enabled (or suspended), specifies the time, in
 days, between when a new version of the object is uploaded to the bucket and when
 old versions of the object expire. When object versions expire, Amazon S3 permanently
 deletes them. If you specify a transition and expiration time, the expiration time
 must be later than the transition time.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-lifecycleconfig-rule.html#cfn-s3-bucket-lifecycleconfig-rule-noncurrentversionexpirationindays
         */
        NoncurrentVersionExpirationInDays?: C.Yields<C.CFInteger>;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-lifecycleconfig-rule.html#cfn-s3-bucket-lifecycleconfig-rule-noncurrentversiontransition
         */
        NoncurrentVersionTransition?: C.Yields<AWS.S3.Bucket.NoncurrentVersionTransition>;
        /**
         * List of Amazon S3
    Lifecycle Rule NoncurrentVersionTransition
         *
         * For buckets with versioning enabled (or suspended), one or more transition rules that
 specify when non-current objects transition to a specified storage class. If you specify
 a transition and expiration time, the expiration time must be later than the transition
 time. If you specify this property, don't specify the NoncurrentVersionTransition property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-lifecycleconfig-rule.html#cfn-s3-bucket-lifecycleconfig-rule-noncurrentversiontransitions
         */
        NoncurrentVersionTransitions?: C.Yields<C.CFList<C.Yields<AWS.S3.Bucket.NoncurrentVersionTransition>>>;
        /**
         * String

         *
         * Object key prefix that identifies one or more objects to which this rule
 applies.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-lifecycleconfig-rule.html#cfn-s3-bucket-lifecycleconfig-rule-prefix
         */
        Prefix?: C.YieldsString;
        /**
         * String

         *
         * Specify either Enabled or Disabled. If you specify
 Enabled, Amazon S3 executes this rule as scheduled. If you specify
 Disabled, Amazon S3 ignores this rule.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-lifecycleconfig-rule.html#cfn-s3-bucket-lifecycleconfig-rule-status
         */
        Status: C.YieldsString;
        /**
         *                             Type: List of Amazon S3  Bucket TagFilter
         *
         * Tags to use to identify a subset of objects to which the lifecycle rule applies.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-lifecycleconfig-rule.html#cfn-s3-bucket-rule-tagfilters
         */
        TagFilters?: C.Yields<C.CFList<C.Yields<AWS.S3.Bucket.TagFilter>>>;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-lifecycleconfig-rule.html#cfn-s3-bucket-lifecycleconfig-rule-transition
         */
        Transition?: C.Yields<AWS.S3.Bucket.Transition>;
        /**
         * List of Amazon S3 Lifecycle Rule
    Transition
         *
         * One or more transition rules that specify when an object transitions to a specified
 storage class. If you specify an expiration and transition time, you must use the
 same time unit for both properties (either in days or by date). The expiration time
 must also be later than the transition time. If you specify this property, don't specify
 the Transition property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-lifecycleconfig-rule.html#cfn-s3-bucket-lifecycleconfig-rule-transitions
         */
        Transitions?: C.Yields<C.CFList<C.Yields<AWS.S3.Bucket.Transition>>>;
    }
}
export declare namespace AWS.S3.Bucket {
    interface S3KeyFilter {
        /**
         * List of Amazon S3 NotificationConfiguration Config Filter S3Key Rules
         *
         * The object key name to filter on and whether to filter on the suffix or prefix
 of the key name.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-notificationconfiguration-config-filter-s3key.html#cfn-s3-bucket-notificationconfiguraiton-config-filter-s3key-rules
         */
        Rules: C.Yields<C.CFList<C.Yields<AWS.S3.Bucket.FilterRule>>>;
    }
}
export declare namespace AWS.S3.Bucket {
    interface TagFilter {
        /**
         *                             Type: String

         *
         * The tag key.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-tagfilter.html#cfn-s3-bucket-tagfilter-key
         */
        Key: C.YieldsString;
        /**
         *                             Type: String

         *
         * The tag value.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-tagfilter.html#cfn-s3-bucket-tagfilter-value
         */
        Value: C.YieldsString;
    }
}
export declare namespace AWS.S3.Bucket {
    interface TopicConfiguration {
        /**
         * String

         *
         * The Amazon Simple Storage Service (Amazon S3) bucket event about which to send notifications.
 For more
 information, see Supported Event
    Types in the Amazon Simple Storage Service Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-notificationconfig-topicconfig.html#cfn-s3-bucket-notificationconfig-topicconfig-event
         */
        Event: C.YieldsString;
        /**
         * Amazon S3
    NotificationConfiguration Config Filter
         *
         * The filtering rules that determine for which objects to send notifications. For
 example, you can create a filter so that Amazon Simple Storage Service (Amazon S3)
 sends notifications only
 when image files with a .jpg extension are added to the
 bucket.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-notificationconfig-topicconfig.html#cfn-s3-bucket-notificationconfig-topicconfig-filter
         */
        Filter?: C.Yields<AWS.S3.Bucket.NotificationFilter>;
        /**
         * String

         *
         * The Amazon SNS topic Amazon Resource Name (ARN) to which Amazon S3 reports the specified
 events.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-notificationconfig-topicconfig.html#cfn-s3-bucket-notificationconfig-topicconfig-topic
         */
        Topic: C.YieldsString;
    }
}
export declare namespace AWS.S3.Bucket {
    interface Transition {
        /**
         * String

         *
         * The storage class to which you want the object to transition, such as
 GLACIER. For valid values, see the StorageClass
 request element of the PUT
    Bucket lifecycle action in the
 Amazon Simple Storage Service API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-lifecycleconfig-rule-transition.html#cfn-s3-bucket-lifecycleconfig-rule-transition-storageclass
         */
        StorageClass: C.YieldsString;
        /**
         * String

         *
         * Indicates when objects are transitioned to the specified storage class. The
 date value must be in ISO 8601 format. The time is always midnight UTC.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-lifecycleconfig-rule-transition.html#cfn-s3-bucket-lifecycleconfig-rule-transition-transitiondate
         */
        TransitionDate?: C.Yields<C.CFTimestamp>;
        /**
         * Integer

         *
         * Indicates the number of days after creation when objects are transitioned to
 the specified storage class.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-lifecycleconfig-rule-transition.html#cfn-s3-bucket-lifecycleconfig-rule-transition-transitionindays
         */
        TransitionInDays?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.S3.Bucket {
    interface VersioningConfiguration {
        /**
         * String

         *
         * The versioning state of an Amazon S3 bucket. If you enable versioning, you must
 suspend versioning to disable it.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-versioningconfig.html#cfn-s3-bucket-versioningconfig-status
         */
        Status: C.YieldsString;
    }
}
export declare namespace AWS.S3.Bucket {
    interface WebsiteConfiguration {
        /**
         * String

         *
         * The name of the error document for the website.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-websiteconfiguration.html#cfn-s3-websiteconfiguration-errordocument
         */
        ErrorDocument?: C.YieldsString;
        /**
         * String

         *
         * The name of the index document for the website.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-websiteconfiguration.html#cfn-s3-websiteconfiguration-indexdocument
         */
        IndexDocument?: C.YieldsString;
        /**
         * Amazon S3 Website
    Configuration Redirect All Requests To Property
         *
         * The redirect behavior for every request to this bucket's website endpoint.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-websiteconfiguration.html#cfn-s3-websiteconfiguration-redirectallrequeststo
         */
        RedirectAllRequestsTo?: C.Yields<AWS.S3.Bucket.RedirectAllRequestsTo>;
        /**
         * List of Amazon S3 Website Configuration
    Routing Rules Property
         *
         * Rules that define when a redirect is applied and the redirect behavior.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-websiteconfiguration.html#cfn-s3-websiteconfiguration-routingrules
         */
        RoutingRules?: C.Yields<C.CFList<C.Yields<AWS.S3.Bucket.RoutingRule>>>;
    }
}
export declare namespace AWS.S3 {
    /**
     * The AWS::S3::BucketPolicy type applies an Amazon S3 bucket policy to an Amazon S3
    bucket.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-policy.html
     */
    interface BucketPolicy {
        Type: 'AWS::S3::BucketPolicy';
        Properties: AWS.S3.BucketPolicy.Properties;
    }
    /**
     * The AWS::S3::BucketPolicy type applies an Amazon S3 bucket policy to an Amazon S3
    bucket.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-policy.html
     */
    function BucketPolicy(props: C.Omit<BucketPolicy, 'Type'>): BucketPolicy;
}
export declare namespace AWS.S3.BucketPolicy {
    interface Properties {
        /**
         * String

         *
         * The Amazon S3 bucket that the policy applies to.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-policy.html#aws-properties-s3-policy-bucket
         */
        Bucket: C.YieldsString;
        /**
         * JSON object

         *
         * A policy document containing permissions to add to the specified bucket. For
 more information, see Access Policy Language Overview in the
 Amazon Simple Storage Service Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-policy.html#aws-properties-s3-policy-policydocument
         */
        PolicyDocument: C.Yields<C.CFJson>;
    }
}
export declare namespace AWS.SDB {
    /**
     * Use the AWS::SDB::Domain resource to declare an Amazon SimpleDB domain. When you
    specify AWS::SDB::Domain as an argument in a Ref function, AWS CloudFormation
    returns the value of the DomainName.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-simpledb.html
     */
    interface Domain {
        Type: 'AWS::SDB::Domain';
        Properties: AWS.SDB.Domain.Properties;
    }
    /**
     * Use the AWS::SDB::Domain resource to declare an Amazon SimpleDB domain. When you
    specify AWS::SDB::Domain as an argument in a Ref function, AWS CloudFormation
    returns the value of the DomainName.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-simpledb.html
     */
    function Domain(props: C.Omit<Domain, 'Type'>): Domain;
}
export declare namespace AWS.SDB.Domain {
    interface Properties {
        /**
         * String

         *
         * Information about the Amazon SimpleDB domain.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-simpledb.html#cfn-sdb-domain-description
         */
        Description?: C.YieldsString;
    }
}
export declare namespace AWS.SNS {
    /**
     * The AWS::SNS::Subscription resource subscribes an endpoint to an Amazon Simple Notification Service
    (Amazon SNS) topic. The owner of the endpoint must confirm the subscription before
    Amazon SNS creates
    the subscription.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-sns-subscription.html
     */
    interface Subscription {
        Type: 'AWS::SNS::Subscription';
        Properties: AWS.SNS.Subscription.Properties;
    }
    /**
     * The AWS::SNS::Subscription resource subscribes an endpoint to an Amazon Simple Notification Service
    (Amazon SNS) topic. The owner of the endpoint must confirm the subscription before
    Amazon SNS creates
    the subscription.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-sns-subscription.html
     */
    function Subscription(props: C.Omit<Subscription, 'Type'>): Subscription;
}
export declare namespace AWS.SNS.Subscription {
    interface Properties {
        /**
         * String

         *
         * The endpoint that receives notifications from the Amazon SNS topic. The endpoint
 value depends on the protocol that you specify. For more information, see the
 Subscribe Endpoint
 parameter in the Amazon Simple Notification Service API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-sns-subscription.html#cfn-sns-endpoint
         */
        Endpoint?: C.YieldsString;
        /**
         * String

         *
         * The subscription's protocol. For more information, see the Subscribe Protocol parameter in
 the Amazon Simple Notification Service API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-sns-subscription.html#cfn-sns-protocol
         */
        Protocol?: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the topic to subscribe to.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-sns-subscription.html#topicarn
         */
        TopicArn?: C.YieldsString;
    }
}
export declare namespace AWS.SNS {
    /**
     * The AWS::SNS::Topic type creates an Amazon Simple Notification Service (Amazon SNS) topic.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sns-topic.html
     */
    interface Topic {
        Type: 'AWS::SNS::Topic';
        Properties: AWS.SNS.Topic.Properties;
    }
    /**
     * The AWS::SNS::Topic type creates an Amazon Simple Notification Service (Amazon SNS) topic.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sns-topic.html
     */
    function Topic(props: C.Omit<Topic, 'Type'>): Topic;
}
export declare namespace AWS.SNS.Topic {
    interface Properties {
        /**
         * String

         *
         * A developer-defined string that can be used to identify this SNS topic.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sns-topic.html#cfn-sns-topic-displayname
         */
        DisplayName?: C.YieldsString;
        /**
         * List of SNS
    Subscriptions
         *
         * The SNS subscriptions (endpoints) for this topic.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sns-topic.html#cfn-sns-topic-subscription
         */
        Subscription?: C.Yields<C.CFList<C.Yields<AWS.SNS.Topic.Subscription>>>;
        /**
         * String

         *
         * A name for the topic. If you don't specify a name, AWS CloudFormation generates a
 unique
 physical ID and uses that ID for the topic name. For more information, see Name Type.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sns-topic.html#cfn-sns-topic-topicname
         */
        TopicName?: C.YieldsString;
    }
}
export declare namespace AWS.SNS.Topic {
    interface Subscription {
        /**
         * String

         *
         * The subscription's endpoint (format depends on the protocol). For more
 information, see the Subscribe
    Endpoint parameter in the
 Amazon Simple Notification Service API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sns-subscription.html#cfn-sns-topic-subscription-endpoint
         */
        Endpoint: C.YieldsString;
        /**
         * String

         *
         * The subscription's protocol. For more information, see the Subscribe Protocol parameter in
 the Amazon Simple Notification Service API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sns-subscription.html#cfn-sns-topic-subscription-protocol
         */
        Protocol: C.YieldsString;
    }
}
export declare namespace AWS.SNS {
    /**
     * The AWS::SNS::TopicPolicy resource associates Amazon SNS topics with a
    policy.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sns-policy.html
     */
    interface TopicPolicy {
        Type: 'AWS::SNS::TopicPolicy';
        Properties: AWS.SNS.TopicPolicy.Properties;
    }
    /**
     * The AWS::SNS::TopicPolicy resource associates Amazon SNS topics with a
    policy.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sns-policy.html
     */
    function TopicPolicy(props: C.Omit<TopicPolicy, 'Type'>): TopicPolicy;
}
export declare namespace AWS.SNS.TopicPolicy {
    interface Properties {
        /**
         *
         *
         * A policy document that contains permissions to add to the specified SNS
 topics.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sns-policy.html#cfn-sns-topicpolicy-policydocument
         */
        PolicyDocument: C.Yields<C.CFJson>;
        /**
         * A list of Amazon SNS topics ARNs

         *
         * The Amazon Resource Names (ARN) of the topics to which you want to add the policy.
 You can use the Ref function to
 specify an AWS::SNS::Topic
 resource.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sns-policy.html#cfn-sns-topicpolicy-topics
         */
        Topics: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.SQS {
    /**
     * The AWS::SQS::Queue resource creates an Amazon Simple Queue Service (Amazon SQS) queue.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sqs-queues.html
     */
    interface Queue {
        Type: 'AWS::SQS::Queue';
        Properties: AWS.SQS.Queue.Properties;
    }
    /**
     * The AWS::SQS::Queue resource creates an Amazon Simple Queue Service (Amazon SQS) queue.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sqs-queues.html
     */
    function Queue(props: C.Omit<Queue, 'Type'>): Queue;
}
export declare namespace AWS.SQS.Queue {
    interface Properties {
        /**
         * Boolean

         *
         * For first-in-first-out (FIFO) queues, specifies whether to enable content-based
 deduplication. During the deduplication interval, Amazon SQS treats messages that
 are sent
 with identical content as duplicates and delivers only one copy of the message. For
 more
 information, see the ContentBasedDeduplication attribute for the CreateQueue action in the
 Amazon Simple Queue Service API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sqs-queues.html#aws-sqs-queue-contentbaseddeduplication
         */
        ContentBasedDeduplication?: C.YieldsBoolean;
        /**
         * Integer

         *
         * The time in seconds that the delivery of all messages in the queue is delayed. You
 can specify an integer value of 0 to 900 (15 minutes). The
 default value is 0.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sqs-queues.html#aws-sqs-queue-delayseconds
         */
        DelaySeconds?: C.Yields<C.CFInteger>;
        /**
         * Boolean

         *
         * Indicates whether this queue is a FIFO queue. For more information, see FIFO (First-In-First-Out) Queues in the
 Amazon Simple Queue Service Developer Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sqs-queues.html#aws-sqs-queue-fifoqueue
         */
        FifoQueue?: C.YieldsBoolean;
        /**
         * Integer

         *
         * The length of time in seconds that Amazon SQS can reuse a data key to encrypt or decrypt
 messages before calling AWS KMS again.
 The value must be an integer between 60 (1 minute) and 86,400 (24 hours). The default
 is 300 (5 minutes).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sqs-queues.html#aws-sqs-queue-kmsdatakeyreuseperiodseconds
         */
        KmsDataKeyReusePeriodSeconds?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The ID of an AWS managed customer master key (CMK) for Amazon SQS or a custom CMK.
 To use the AWS managed CMK for Amazon SQS,
 specify the alias alias/aws/sqs.
 For more information, see CreateQueue in the Amazon Simple Queue Service API Reference, Protecting Data Using Server-Side Encryption (SSE) and AWS KMS in the
 Amazon Simple Queue Service Developer Guide, or Customer Master Keys in the AWS Key Management Service Best Practices whitepaper.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sqs-queues.html#aws-sqs-queue-kmsmasterkeyid
         */
        KmsMasterKeyId?: C.YieldsString;
        /**
         * Integer

         *
         * The limit of how many bytes that a message can contain before Amazon SQS rejects it.
 You
 can specify an integer value from 1024 bytes (1 KiB) to 262144
 bytes (256 KiB). The default value is 262144 (256 KiB).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sqs-queues.html#aws-sqs-queue-maxmesgsize
         */
        MaximumMessageSize?: C.Yields<C.CFInteger>;
        /**
         * Integer

         *
         * The number of seconds that Amazon SQS retains a message. You can specify an integer
 value
 from 60 seconds (1 minute) to 1209600 seconds (14 days). The
 default value is 345600 seconds (4 days).

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sqs-queues.html#aws-sqs-queue-msgretentionperiod
         */
        MessageRetentionPeriod?: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * A name for the queue. To create a FIFO queue, the name of your FIFO queue must end
 with the .fifo suffix. For more information, see FIFO (First-In-First-Out) Queues in the Amazon Simple Queue Service Developer Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sqs-queues.html#aws-sqs-queue-name
         */
        QueueName?: C.YieldsString;
        /**
         * Integer

         *
         * Specifies the duration, in seconds, that the ReceiveMessage action call
 waits until a message is in the queue in order to include it in the
 response, as opposed to returning an empty response if a message isn't
 yet available. You can specify an integer from 1 to 20. The
 short polling is used as the default or when you specify 0 for this
 property. For more information, see Amazon SQS Long Poll.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sqs-queues.html#aws-sqs-queue-receivemsgwaittime
         */
        ReceiveMessageWaitTimeSeconds?: C.Yields<C.CFInteger>;
        /**
         * Amazon SQS RedrivePolicy
         *
         * Specifies an existing dead letter queue to receive messages after the source queue
 (this queue) fails to process a message a specified number of times.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sqs-queues.html#aws-sqs-queue-redrive
         */
        RedrivePolicy?: C.Yields<C.CFJson>;
        /**
         * Integer

         *
         * The length of time during which a message will be unavailable after a message
 is delivered from the queue. This blocks other components from receiving the same
 message and gives the initial component time to process and delete the message from
 the
 queue.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sqs-queues.html#aws-sqs-queue-visiblitytimeout
         */
        VisibilityTimeout?: C.Yields<C.CFInteger>;
    }
}
export declare namespace AWS.SQS {
    /**
     * The AWS::SQS::QueuePolicy type applies a policy to Amazon SQS queues.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sqs-policy.html
     */
    interface QueuePolicy {
        Type: 'AWS::SQS::QueuePolicy';
        Properties: AWS.SQS.QueuePolicy.Properties;
    }
    /**
     * The AWS::SQS::QueuePolicy type applies a policy to Amazon SQS queues.
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sqs-policy.html
     */
    function QueuePolicy(props: C.Omit<QueuePolicy, 'Type'>): QueuePolicy;
}
export declare namespace AWS.SQS.QueuePolicy {
    interface Properties {
        /**
         * JSON object

         *
         * A policy document that contains the permissions for the specified Amazon SQS queues.
 For
 more information about Amazon SQS policies, see Creating Custom Policies Using
    the Access Policy Language in the
 Amazon Simple Queue Service Developer Guide.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sqs-policy.html#cfn-sqs-queuepolicy-policydoc
         */
        PolicyDocument: C.Yields<C.CFJson>;
        /**
         * List of strings

         *
         * The URLs of the queues to which you want to add the policy. You can use the Ref function to specify an AWS::SQS::Queue resource.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-sqs-policy.html#cfn-sqs-queuepolicy-queues
         */
        Queues: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.SSM {
    /**
     * The AWS::SSM::Association resource associates an Amazon EC2 Systems Manager (SSM) document
    with EC2 instances that contain a configuration agent to process the document.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-association.html
     */
    interface Association {
        Type: 'AWS::SSM::Association';
        Properties: AWS.SSM.Association.Properties;
    }
    /**
     * The AWS::SSM::Association resource associates an Amazon EC2 Systems Manager (SSM) document
    with EC2 instances that contain a configuration agent to process the document.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-association.html
     */
    function Association(props: C.Omit<Association, 'Type'>): Association;
}
export declare namespace AWS.SSM.Association {
    interface Properties {
        /**
         * String

         *
         * The version of the SSM document to associate with the target.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-association.html#cfn-ssm-association-documentversion
         */
        DocumentVersion?: C.YieldsString;
        /**
         * String

         *
         * The ID of the instance that the SSM document is associated with.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-association.html#cfn-ssm-association-instanceid
         */
        InstanceId?: C.YieldsString;
        /**
         * String

         *
         * The name of the SSM document.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-association.html#cfn-ssm-association-name
         */
        Name: C.YieldsString;
        /**
         * String to list-of-strings map

         *
         * Parameter values that the SSM document uses at runtime.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-association.html#cfn-ssm-association-parameters
         */
        Parameters?: C.Yields<C.CFMap<C.Yields<AWS.SSM.Association.ParameterValues>>>;
        /**
         * String

         *
         * A Cron expression that specifies when the association is applied to the target. For
 supported expressions, see the ScheduleExpression parameter for the CreateAssociation action in the
 Amazon EC2 Systems Manager API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-association.html#cfn-ssm-association-scheduleexpression
         */
        ScheduleExpression?: C.YieldsString;
        /**
         * List of Amazon EC2 Systems Manager Association Targets
         *
         * The targets that the SSM document sends commands to.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-association.html#cfn-ssm-association-targets
         */
        Targets?: C.Yields<C.CFList<C.Yields<AWS.SSM.Association.Target>>>;
    }
}
export declare namespace AWS.SSM.Association {
    interface ParameterValues {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ssm-association-parametervalues.html#cfn-ssm-association-parametervalues-parametervalues
         */
        ParameterValues: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.SSM.Association {
    interface Target {
        /**
         * String

         *
         * The name of the criteria that EC2 instances must meet. For valid keys, see the
 Target data type in the
 Amazon EC2 Systems Manager API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ssm-association-target.html#cfn-ssm-association-target-key
         */
        Key: C.YieldsString;
        /**
         * List of strings

         *
         * The value of the criteria. SSM runs targeted commands on EC2 instances that match
 the criteria. For more information, see the Target data type in the Amazon EC2 Systems Manager API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ssm-association-target.html#cfn-ssm-association-target-values
         */
        Values: C.Yields<C.CFList<C.YieldsString>>;
    }
}
export declare namespace AWS.SSM {
    /**
     * The AWS::SSM::Document resource creates an Amazon EC2 Systems Manager (SSM) document that
    describes an instance configuration, which you can use to set up and run commands
    on your
    instances.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-document.html
     */
    interface Document {
        Type: 'AWS::SSM::Document';
        Properties: AWS.SSM.Document.Properties;
    }
    /**
     * The AWS::SSM::Document resource creates an Amazon EC2 Systems Manager (SSM) document that
    describes an instance configuration, which you can use to set up and run commands
    on your
    instances.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-document.html
     */
    function Document(props: C.Omit<Document, 'Type'>): Document;
}
export declare namespace AWS.SSM.Document {
    interface Properties {
        /**
         * JSON object

         *
         * A JSON object that describes an instance configuration. For more information,
 see Creating SSM Documents in
 the Amazon EC2 User Guide for Linux Instances.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-document.html#cfn-ssm-document-content
         */
        Content: C.Yields<C.CFJson>;
        /**
         * String

         *
         * The type of document to create that relates to the purpose of your document, such
 as
 running commands, bootstrapping software, or automating tasks. For valid values, see
 the
 CreateDocument action in the
 Amazon EC2 Systems Manager API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-document.html#cfn-ssm-document-documenttype
         */
        DocumentType?: C.YieldsString;
    }
}
export declare namespace AWS.SSM {
    /**
     * The AWS::SSM::Parameter resource creates an Amazon EC2 Systems Manager (SSM) parameter in
    Parameter Store.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-parameter.html
     */
    interface Parameter {
        Type: 'AWS::SSM::Parameter';
        Properties: AWS.SSM.Parameter.Properties;
    }
    /**
     * The AWS::SSM::Parameter resource creates an Amazon EC2 Systems Manager (SSM) parameter in
    Parameter Store.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-parameter.html
     */
    function Parameter(props: C.Omit<Parameter, 'Type'>): Parameter;
}
export declare namespace AWS.SSM.Parameter {
    interface Properties {
        /**
         * String

         *
         * The type of parameter. Valid values include the following: String or StringList.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-parameter.html#cfn-ssm-parameter-type
         */
        Type: C.YieldsString;
        /**
         * String

         *
         * Information about the parameter that you want to add to the system.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-parameter.html#cfn-ssm-parameter-description
         */
        Description?: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-parameter.html#cfn-ssm-parameter-allowedpattern
         */
        AllowedPattern?: C.YieldsString;
        /**
         * String

         *
         * The parameter value. Value must not nest another parameter. Do not use {{}} in the value.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-parameter.html#cfn-ssm-parameter-value
         */
        Value: C.YieldsString;
        /**
         * String

         *
         * The name of the parameter. Names must not be prefixed with aws or ssm.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ssm-parameter.html#cfn-ssm-parameter-name
         */
        Name?: C.YieldsString;
    }
}
export declare namespace AWS.StepFunctions {
    /**
     * Use the AWS::StepFunctions::Activity resource to create an AWS Step Functions
    activity.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-stepfunctions-activity.html
     */
    interface Activity {
        Type: 'AWS::StepFunctions::Activity';
        Properties: AWS.StepFunctions.Activity.Properties;
    }
    /**
     * Use the AWS::StepFunctions::Activity resource to create an AWS Step Functions
    activity.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-stepfunctions-activity.html
     */
    function Activity(props: C.Omit<Activity, 'Type'>): Activity;
}
export declare namespace AWS.StepFunctions.Activity {
    interface Properties {
        /**
         * String

         *
         * The name of the activity to create. This name must be unique for your AWS
 account and region.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-stepfunctions-activity.html#cfn-stepfunctions-activity-name
         */
        Name: C.YieldsString;
    }
}
export declare namespace AWS.StepFunctions {
    /**
     * Use the AWS::StepFunctions::StateMachine resource to create an AWS Step Functions state
    machine.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-stepfunctions-statemachine.html
     */
    interface StateMachine {
        Type: 'AWS::StepFunctions::StateMachine';
        Properties: AWS.StepFunctions.StateMachine.Properties;
    }
    /**
     * Use the AWS::StepFunctions::StateMachine resource to create an AWS Step Functions state
    machine.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-stepfunctions-statemachine.html
     */
    function StateMachine(props: C.Omit<StateMachine, 'Type'>): StateMachine;
}
export declare namespace AWS.StepFunctions.StateMachine {
    interface Properties {
        /**
         * String

         *
         * The Amazon States Language definition of the state machine. For more information,
 see Amazon States Language in the AWS Step Functions Developer Guide.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-stepfunctions-statemachine.html#cfn-stepfunctions-statemachine-definitionstring
         */
        DefinitionString: C.YieldsString;
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the IAM role to use for this state
 machine.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-stepfunctions-statemachine.html#cfn-stepfunctions-statemachine-rolearn
         */
        RoleArn: C.YieldsString;
    }
}
export declare namespace AWS.WAF {
    /**
     * The AWS::WAF::ByteMatchSet resource creates an AWS WAF
    ByteMatchSet that identifies a part of a web request that you want to inspect.
    For more information, see CreateByteMatchSet in the AWS WAF API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-bytematchset.html
     */
    interface ByteMatchSet {
        Type: 'AWS::WAF::ByteMatchSet';
        Properties: AWS.WAF.ByteMatchSet.Properties;
    }
    /**
     * The AWS::WAF::ByteMatchSet resource creates an AWS WAF
    ByteMatchSet that identifies a part of a web request that you want to inspect.
    For more information, see CreateByteMatchSet in the AWS WAF API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-bytematchset.html
     */
    function ByteMatchSet(props: C.Omit<ByteMatchSet, 'Type'>): ByteMatchSet;
}
export declare namespace AWS.WAF.ByteMatchSet {
    interface Properties {
        /**
         * List of AWS WAF ByteMatchSet
    ByteMatchTuples
         *
         * Settings for the ByteMatchSet, such as the bytes (typically a
 string that corresponds with ASCII characters) that you want AWS WAF to search for
 in web
 requests.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-bytematchset.html#cfn-waf-bytematchset-bytematchtuples
         */
        ByteMatchTuples?: C.Yields<C.CFList<C.Yields<AWS.WAF.ByteMatchSet.ByteMatchTuple>>>;
        /**
         * String

         *
         * A friendly name or description of the ByteMatchSet.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-bytematchset.html#cfn-waf-bytematchset-name
         */
        Name: C.YieldsString;
    }
}
export declare namespace AWS.WAF.ByteMatchSet {
    interface ByteMatchTuple {
        /**
         * AWS WAF ByteMatchSet
    ByteMatchTuples FieldToMatch
         *
         * The part of a web request that you want AWS WAF to search, such as a specific
 header or a query string.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-bytematchset-bytematchtuples.html#cfn-waf-bytematchset-bytematchtuples-fieldtomatch
         */
        FieldToMatch: C.Yields<AWS.WAF.ByteMatchSet.FieldToMatch>;
        /**
         * String

         *
         * How AWS WAF finds matches within the web request part in which you are searching.
 For valid values, see the PositionalConstraint content for the ByteMatchTuple data type in
 the AWS WAF API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-bytematchset-bytematchtuples.html#cfn-waf-bytematchset-bytematchtuples-positionalconstraint
         */
        PositionalConstraint: C.YieldsString;
        /**
         * String

         *
         * The value that AWS WAF searches for. AWS CloudFormation base64 encodes this value
 before
 sending it to AWS WAF.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-bytematchset-bytematchtuples.html#cfn-waf-bytematchset-bytematchtuples-targetstring
         */
        TargetString?: C.YieldsString;
        /**
         * String

         *
         * The base64-encoded value that AWS WAF searches for. AWS CloudFormation sends this
 value to
 AWS WAF without encoding it.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-bytematchset-bytematchtuples.html#cfn-waf-bytematchset-bytematchtuples-targetstringbase64
         */
        TargetStringBase64?: C.YieldsString;
        /**
         * String

         *
         * Specifies how AWS WAF processes the target string value. Text transformations
 eliminate some of the unusual formatting that attackers use in web requests in an
 effort to bypass AWS WAF. If you specify a transformation, AWS WAF transforms the
 target string value before inspecting a web request for a match.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-bytematchset-bytematchtuples.html#cfn-waf-bytematchset-bytematchtuples-texttransformation
         */
        TextTransformation: C.YieldsString;
    }
}
export declare namespace AWS.WAF.ByteMatchSet {
    interface FieldToMatch {
        /**
         * String

         *
         * If you specify HEADER for the Type property, the name
 of the header that AWS WAF searches for, such as User-Agent or
 Referer. If you specify any other value for the Type
 property, do not specify this property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-bytematchset-bytematchtuples-fieldtomatch.html#cfn-waf-bytematchset-bytematchtuples-fieldtomatch-data
         */
        Data?: C.YieldsString;
        /**
         * String

         *
         * The part of the web request in which AWS WAF searches for the target string. For
 valid values, see FieldToMatch
 in the AWS WAF API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-bytematchset-bytematchtuples-fieldtomatch.html#cfn-waf-bytematchset-bytematchtuples-fieldtomatch-type
         */
        Type: C.YieldsString;
    }
}
export declare namespace AWS.WAF {
    /**
     * The AWS::WAF::IPSet resource creates an AWS WAF IPSet that
    specifies which web requests to permit or block based on the IP addresses from which
    the
    requests originate. For more information, see CreateIPSet in the AWS WAF API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-ipset.html
     */
    interface IPSet {
        Type: 'AWS::WAF::IPSet';
        Properties: AWS.WAF.IPSet.Properties;
    }
    /**
     * The AWS::WAF::IPSet resource creates an AWS WAF IPSet that
    specifies which web requests to permit or block based on the IP addresses from which
    the
    requests originate. For more information, see CreateIPSet in the AWS WAF API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-ipset.html
     */
    function IPSet(props: C.Omit<IPSet, 'Type'>): IPSet;
}
export declare namespace AWS.WAF.IPSet {
    interface Properties {
        /**
         * List of AWS WAF IPSet IPSetDescriptors
         *
         * The IP address type and IP address range (in CIDR notation) from which web
 requests originate. If you associate the IPSet with a web ACL that is associated with a Amazon CloudFront
 (CloudFront) distribution, this descriptor is the value of one of the following fields
 in the
 CloudFront access logs:

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-ipset.html#cfn-waf-ipset-ipsetdescriptors
         */
        IPSetDescriptors?: C.Yields<C.CFList<C.Yields<AWS.WAF.IPSet.IPSetDescriptor>>>;
        /**
         * String

         *
         * A friendly name or description of the IPSet.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-ipset.html#cfn-waf-ipset-name
         */
        Name: C.YieldsString;
    }
}
export declare namespace AWS.WAF.IPSet {
    interface IPSetDescriptor {
        /**
         * String

         *
         * The IP address type, such as IPV4. For valid values, see the
 Type contents of the IPSetDescriptor data type
 in the AWS WAF API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-ipset-ipsetdescriptors.html#cfn-waf-ipset-ipsetdescriptors-type
         */
        Type: C.YieldsString;
        /**
         * String

         *
         * An IP address (in CIDR notation) that AWS WAF permits, blocks, or counts. For
 example, to specify a single IP address such as 192.0.2.44, specify
 192.0.2.44/32. To specify a range of IP addresses such as
 192.0.2.0 to 192.0.2.255, specify
 192.0.2.0/24.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-ipset-ipsetdescriptors.html#cfn-waf-ipset-ipsetdescriptors-value
         */
        Value: C.YieldsString;
    }
}
export declare namespace AWS.WAF {
    /**
     * The AWS::WAF::Rule resource creates an AWS WAF rule that specifies a
    combination of IPSet, ByteMatchSet, and
    SqlInjectionMatchSet objects that identify the web requests to allow, block, or
    count. To implement rules, you must associate them with a web ACL.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-rule.html
     */
    interface Rule {
        Type: 'AWS::WAF::Rule';
        Properties: AWS.WAF.Rule.Properties;
    }
    /**
     * The AWS::WAF::Rule resource creates an AWS WAF rule that specifies a
    combination of IPSet, ByteMatchSet, and
    SqlInjectionMatchSet objects that identify the web requests to allow, block, or
    count. To implement rules, you must associate them with a web ACL.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-rule.html
     */
    function Rule(props: C.Omit<Rule, 'Type'>): Rule;
}
export declare namespace AWS.WAF.Rule {
    interface Properties {
        /**
         * String

         *
         * A friendly name or description for the metrics of the rule. For valid values,
 see the MetricName parameter for the CreateRule
 action in the AWS WAF API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-rule.html#cfn-waf-rule-metricname
         */
        MetricName: C.YieldsString;
        /**
         * String

         *
         * A friendly name or description of the rule.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-rule.html#cfn-waf-rule-name
         */
        Name: C.YieldsString;
        /**
         * List of AWS WAF Rule Predicates
         *
         * The ByteMatchSet, IPSet, SizeConstraintSet, SqlInjectionMatchSet, or XssMatchSet objects to include in a rule. If you add more than one predicate to a rule, a request
 must match all conditions in order to be allowed or blocked.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-rule.html#cfn-waf-rule-predicates
         */
        Predicates?: C.Yields<C.CFList<C.Yields<AWS.WAF.Rule.Predicate>>>;
    }
}
export declare namespace AWS.WAF.Rule {
    interface Predicate {
        /**
         * String

         *
         * The unique identifier of a predicate, such as the ID of a ByteMatchSet or IPSet.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-rule-predicates.html#cfn-waf-rule-predicates-dataid
         */
        DataId: C.YieldsString;
        /**
         * Boolean

         *
         * Whether to use the settings or the negated settings that you specified in the ByteMatchSet, IPSet, SizeConstraintSet, SqlInjectionMatchSet, or XssMatchSet objects.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-rule-predicates.html#cfn-waf-rule-predicates-negated
         */
        Negated: C.YieldsBoolean;
        /**
         * String

         *
         * The type of predicate in a rule, such as an IPSet
 (IPMatch). For valid values, see the Type contents of the
 Predicate data type in the
 AWS WAF API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-rule-predicates.html#cfn-waf-rule-predicates-type
         */
        Type: C.YieldsString;
    }
}
export declare namespace AWS.WAF {
    /**
     * The AWS::WAF::SizeConstraintSet resource specifies a size constraint that AWS WAF uses to check the size of a web
    request and which parts of the request to check. For more information, see CreateSizeConstraintSet in the AWS WAF API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-sizeconstraintset.html
     */
    interface SizeConstraintSet {
        Type: 'AWS::WAF::SizeConstraintSet';
        Properties: AWS.WAF.SizeConstraintSet.Properties;
    }
    /**
     * The AWS::WAF::SizeConstraintSet resource specifies a size constraint that AWS WAF uses to check the size of a web
    request and which parts of the request to check. For more information, see CreateSizeConstraintSet in the AWS WAF API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-sizeconstraintset.html
     */
    function SizeConstraintSet(props: C.Omit<SizeConstraintSet, 'Type'>): SizeConstraintSet;
}
export declare namespace AWS.WAF.SizeConstraintSet {
    interface Properties {
        /**
         * String

         *
         * A friendly name or description for the SizeConstraintSet.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-sizeconstraintset.html#cfn-waf-sizeconstraintset-name
         */
        Name: C.YieldsString;
        /**
         * List of AWS WAF SizeConstraintSet SizeConstraint
         *
         * The size constraint and the part of the web request to check.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-sizeconstraintset.html#cfn-waf-sizeconstraintset-sizeconstraints
         */
        SizeConstraints: C.Yields<C.CFList<C.Yields<AWS.WAF.SizeConstraintSet.SizeConstraint>>>;
    }
}
export declare namespace AWS.WAF.SizeConstraintSet {
    interface FieldToMatch {
        /**
         * String

         *
         * If you specify HEADER for the Type property, the name of the header that AWS WAF searches for, such as User-Agent or Referer. If you specify any other value for the Type property, do not specify this property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-sizeconstraintset-sizeconstraint-fieldtomatch.html#cfn-waf-sizeconstraintset-sizeconstraint-fieldtomatch-data
         */
        Data?: C.YieldsString;
        /**
         * String

         *
         * The part of the web request in which AWS WAF searches for the target string. For valid
 values, see FieldToMatch in the AWS WAF API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-sizeconstraintset-sizeconstraint-fieldtomatch.html#cfn-waf-sizeconstraintset-sizeconstraint-fieldtomatch-type
         */
        Type: C.YieldsString;
    }
}
export declare namespace AWS.WAF.SizeConstraintSet {
    interface SizeConstraint {
        /**
         * String

         *
         * The type of comparison that you want AWS WAF to perform. AWS WAF uses this value in
 combination with the Size and FieldToMatch property values to check if the size constraint is a match. For more information
 and valid values, see the ComparisonOperator content for the SizeConstraint data type in the AWS WAF API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-sizeconstraintset-sizeconstraint.html#cfn-waf-sizeconstraintset-sizeconstraint-comparisonoperator
         */
        ComparisonOperator: C.YieldsString;
        /**
         * AWS WAF SizeConstraintSet SizeConstraint FieldToMatch
         *
         * The part of a web request that you want AWS WAF to search, such as a specific header
 or a query string.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-sizeconstraintset-sizeconstraint.html#cfn-waf-sizeconstraintset-sizeconstraint-fieldtomatch
         */
        FieldToMatch: C.Yields<AWS.WAF.SizeConstraintSet.FieldToMatch>;
        /**
         * Integer

         *
         * The size in bytes that you want AWS WAF to compare against the size of the specified
 FieldToMatch. AWS WAF uses Size in combination with the ComparisonOperator and FieldToMatch property values to check if the size constraint of a web request is a match. For
 more information and valid values, see the Size content for the SizeConstraint data type in the
 AWS WAF API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-sizeconstraintset-sizeconstraint.html#cfn-waf-sizeconstraintset-sizeconstraint-size
         */
        Size: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * Specifies how AWS WAF processes the FieldToMatch property before inspecting a request for a match. Text transformations eliminate
 some of the unusual formatting that attackers use in web requests in an effort to
 bypass AWS WAF. If you specify a transformation, AWS WAF transforms the FieldToMatch before inspecting a web request for a match.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-sizeconstraintset-sizeconstraint.html#cfn-waf-sizeconstraintset-sizeconstraint-texttransformation
         */
        TextTransformation: C.YieldsString;
    }
}
export declare namespace AWS.WAF {
    /**
     * The AWS::WAF::SqlInjectionMatchSet resource creates an AWS WAF
    SqlInjectionMatchSet, which you use to allow, block, or count requests that
    contain malicious SQL code in a specific part of web requests. For more information,
    see CreateSqlInjectionMatchSet in
    the AWS WAF API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-sqlinjectionmatchset.html
     */
    interface SqlInjectionMatchSet {
        Type: 'AWS::WAF::SqlInjectionMatchSet';
        Properties: AWS.WAF.SqlInjectionMatchSet.Properties;
    }
    /**
     * The AWS::WAF::SqlInjectionMatchSet resource creates an AWS WAF
    SqlInjectionMatchSet, which you use to allow, block, or count requests that
    contain malicious SQL code in a specific part of web requests. For more information,
    see CreateSqlInjectionMatchSet in
    the AWS WAF API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-sqlinjectionmatchset.html
     */
    function SqlInjectionMatchSet(props: C.Omit<SqlInjectionMatchSet, 'Type'>): SqlInjectionMatchSet;
}
export declare namespace AWS.WAF.SqlInjectionMatchSet {
    interface Properties {
        /**
         * String

         *
         * A friendly name or description of the SqlInjectionMatchSet.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-sqlinjectionmatchset.html#cfn-waf-sqlinjectionmatchset-name
         */
        Name: C.YieldsString;
        /**
         * List of AWS WAF
    SqlInjectionMatchSet SqlInjectionMatchTuples
         *
         * The parts of web requests that you want AWS WAF to inspect for malicious SQL code
 and, if you want AWS WAF to inspect a header, the name of the header.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-sqlinjectionmatchset.html#cfn-waf-sqlinjectionmatchset-sqlinjectionmatchtuples
         */
        SqlInjectionMatchTuples?: C.Yields<C.CFList<C.Yields<AWS.WAF.SqlInjectionMatchSet.SqlInjectionMatchTuple>>>;
    }
}
export declare namespace AWS.WAF.SqlInjectionMatchSet {
    interface FieldToMatch {
        /**
         * String

         *
         * If you specify HEADER for the Type property, the name
 of the header that AWS WAF searches for, such as User-Agent or
 Referer. If you specify any other value for the Type
 property, do not specify this property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-bytematchset-bytematchtuples-fieldtomatch.html#cfn-waf-sizeconstraintset-sizeconstraint-fieldtomatch-data
         */
        Data?: C.YieldsString;
        /**
         * String

         *
         * The part of the web request in which AWS WAF searches for the target string. For
 valid values, see FieldToMatch
 in the AWS WAF API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-bytematchset-bytematchtuples-fieldtomatch.html#cfn-waf-sizeconstraintset-sizeconstraint-fieldtomatch-type
         */
        Type: C.YieldsString;
    }
}
export declare namespace AWS.WAF.SqlInjectionMatchSet {
    interface SqlInjectionMatchTuple {
        /**
         * AWS WAF ByteMatchSet
    ByteMatchTuples FieldToMatch
         *
         * The part of a web request that you want AWS WAF to search, such as a specific
 header or a query string.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-sqlinjectionmatchset-sqlinjectionmatchtuples.html#cfn-waf-sqlinjectionmatchset-sqlinjectionmatchtuples-fieldtomatch
         */
        FieldToMatch: C.Yields<AWS.WAF.SqlInjectionMatchSet.FieldToMatch>;
        /**
         * String

         *
         * Text transformations eliminate some of the unusual formatting that attackers
 use in web requests in an effort to bypass AWS WAF. If you specify a transformation,
 AWS WAF transforms the target string value before inspecting a web request for a
 match. For valid values, see the TextTransformation content for the
 SqlInjectionMatchTuple data type in the
 AWS WAF API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-sqlinjectionmatchset-sqlinjectionmatchtuples.html#cfn-waf-sqlinjectionmatchset-sqlinjectionmatchtuples-texttransformation
         */
        TextTransformation: C.YieldsString;
    }
}
export declare namespace AWS.WAF {
    /**
     * The AWS::WAF::WebACL resource creates an AWS WAF web access control group
    (ACL) containing the rules that identify the Amazon CloudFront (CloudFront) web requests
    that you want to
    allow, block, or count. For more information, see CreateWebACL in the
    AWS WAF API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-webacl.html
     */
    interface WebACL {
        Type: 'AWS::WAF::WebACL';
        Properties: AWS.WAF.WebACL.Properties;
    }
    /**
     * The AWS::WAF::WebACL resource creates an AWS WAF web access control group
    (ACL) containing the rules that identify the Amazon CloudFront (CloudFront) web requests
    that you want to
    allow, block, or count. For more information, see CreateWebACL in the
    AWS WAF API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-webacl.html
     */
    function WebACL(props: C.Omit<WebACL, 'Type'>): WebACL;
}
export declare namespace AWS.WAF.WebACL {
    interface Properties {
        /**
         * AWS WAF WebACL Action
         *
         * The action that you want AWS WAF to take when a request doesn't match the
 criteria in any of the rules that are associated with the web ACL.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-webacl.html#cfn-waf-webacl-defaultaction
         */
        DefaultAction: C.Yields<AWS.WAF.WebACL.WafAction>;
        /**
         * String

         *
         * A friendly name or description for the Amazon CloudWatch metric of this web ACL. For
 valid values, see the MetricName parameter of the CreateWebACL action in the AWS WAF API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-webacl.html#cfn-waf-webacl-metricname
         */
        MetricName: C.YieldsString;
        /**
         * String

         *
         * A friendly name or description of the web ACL.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-webacl.html#cfn-waf-webacl-name
         */
        Name: C.YieldsString;
        /**
         * List of AWS WAF WebACL Rules
         *
         * The rules to associate with the web ACL and the settings for each rule.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-webacl.html#cfn-waf-webacl-rules
         */
        Rules?: C.Yields<C.CFList<C.Yields<AWS.WAF.WebACL.ActivatedRule>>>;
    }
}
export declare namespace AWS.WAF.WebACL {
    interface ActivatedRule {
        /**
         * AWS WAF WebACL Action
         *
         * The action that Amazon CloudFront (CloudFront) or AWS WAF takes when a web request
 matches all
 conditions in the rule, such as allow, block, or count the request.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-webacl-rules.html#cfn-waf-webacl-rules-action
         */
        Action: C.Yields<AWS.WAF.WebACL.WafAction>;
        /**
         * Integer

         *
         * The order in which AWS WAF evaluates the rules in a web ACL. AWS WAF evaluates
 rules with a lower value before rules with a higher value. The value must be a
 unique integer. If you have multiple rules in a web ACL, the priority numbers do
 not need to be consecutive.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-webacl-rules.html#cfn-waf-webacl-rules-priority
         */
        Priority: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * The ID of an AWS WAF rule to
 associate with a web ACL.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-webacl-rules.html#cfn-waf-webacl-rules-ruleid
         */
        RuleId: C.YieldsString;
    }
}
export declare namespace AWS.WAF.WebACL {
    interface WafAction {
        /**
         * String

         *
         * For actions that are associated with a rule, the action that AWS WAF takes when a
 web request matches all conditions in a rule.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-webacl-action.html#cfn-waf-webacl-action-type
         */
        Type: C.YieldsString;
    }
}
export declare namespace AWS.WAF {
    /**
     * The AWS::WAF::XssMatchSet resource specifies the parts of web requests that you want AWS WAF to inspect for
    cross-site scripting attacks and the name of the header to inspect. For more information,
    see XssMatchSet in the AWS WAF API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-xssmatchset.html
     */
    interface XssMatchSet {
        Type: 'AWS::WAF::XssMatchSet';
        Properties: AWS.WAF.XssMatchSet.Properties;
    }
    /**
     * The AWS::WAF::XssMatchSet resource specifies the parts of web requests that you want AWS WAF to inspect for
    cross-site scripting attacks and the name of the header to inspect. For more information,
    see XssMatchSet in the AWS WAF API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-xssmatchset.html
     */
    function XssMatchSet(props: C.Omit<XssMatchSet, 'Type'>): XssMatchSet;
}
export declare namespace AWS.WAF.XssMatchSet {
    interface Properties {
        /**
         * String

         *
         * A friendly name or description for the XssMatchSet.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-xssmatchset.html#cfn-waf-xssmatchset-name
         */
        Name: C.YieldsString;
        /**
         * List of AWS WAF XssMatchSet XssMatchTuple
         *
         * The parts of web requests that you want to inspect for cross-site scripting attacks.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-waf-xssmatchset.html#cfn-waf-xssmatchset-xssmatchtuples
         */
        XssMatchTuples: C.Yields<C.CFList<C.Yields<AWS.WAF.XssMatchSet.XssMatchTuple>>>;
    }
}
export declare namespace AWS.WAF.XssMatchSet {
    interface FieldToMatch {
        /**
         * String

         *
         * If you specify HEADER for the Type property, the name of the header that AWS WAF searches for, such as User-Agent or Referer. If you specify any other value for the Type property, do not specify this property.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-xssmatchset-xssmatchtuple-fieldtomatch.html#cfn-waf-xssmatchset-xssmatchtuple-fieldtomatch-data
         */
        Data?: C.YieldsString;
        /**
         * String

         *
         * The part of the web request in which AWS WAF searches for the target string. For valid
 values, see FieldToMatch in the AWS WAF API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-xssmatchset-xssmatchtuple-fieldtomatch.html#cfn-waf-xssmatchset-xssmatchtuple-fieldtomatch-type
         */
        Type: C.YieldsString;
    }
}
export declare namespace AWS.WAF.XssMatchSet {
    interface XssMatchTuple {
        /**
         * AWS WAF XssMatchSet XssMatchTuple FieldToMatch
         *
         * The part of a web request that you want AWS WAF to search, such as a specific header
 or a query string.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-xssmatchset-xssmatchtuple.html#cfn-waf-xssmatchset-xssmatchtuple-fieldtomatch
         */
        FieldToMatch: C.Yields<AWS.WAF.XssMatchSet.FieldToMatch>;
        /**
         * String

         *
         * Specifies how AWS WAF processes the FieldToMatch property before inspecting a request for a match. Text transformations eliminate
 some of the unusual formatting that attackers use in web requests in an effort to
 bypass AWS WAF. If you specify a transformation, AWS WAF transforms the FieldToMatch parameter before inspecting a web request for a match.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-waf-xssmatchset-xssmatchtuple.html#cfn-waf-xssmatchset-xssmatchtuple-texttransformation
         */
        TextTransformation: C.YieldsString;
    }
}
export declare namespace AWS.WAFRegional {
    /**
     * The AWS::WAFRegional::ByteMatchSet resource creates an AWS WAF Regional
    ByteMatchSet that identifies a part of a web request that you want to inspect.
    For more information, see CreateByteMatchSet in the AWS WAF Regional API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-bytematchset.html
     */
    interface ByteMatchSet {
        Type: 'AWS::WAFRegional::ByteMatchSet';
        Properties: AWS.WAFRegional.ByteMatchSet.Properties;
    }
    /**
     * The AWS::WAFRegional::ByteMatchSet resource creates an AWS WAF Regional
    ByteMatchSet that identifies a part of a web request that you want to inspect.
    For more information, see CreateByteMatchSet in the AWS WAF Regional API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-bytematchset.html
     */
    function ByteMatchSet(props: C.Omit<ByteMatchSet, 'Type'>): ByteMatchSet;
}
export declare namespace AWS.WAFRegional.ByteMatchSet {
    interface Properties {
        /**
         * List of AWS WAF Regional ByteMatchSet
    ByteMatchTuples
         *
         * Settings for the ByteMatchSet, such as the bytes (typically a
 string that corresponds with ASCII characters) that you want AWS WAF to search for
 in web
 requests.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-bytematchset.html#cfn-wafregional-bytematchset-bytematchtuples
         */
        ByteMatchTuples?: C.Yields<C.CFList<C.Yields<AWS.WAFRegional.ByteMatchSet.ByteMatchTuple>>>;
        /**
         * String

         *
         * A friendly name or description of the ByteMatchSet.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-bytematchset.html#cfn-wafregional-bytematchset-name
         */
        Name: C.YieldsString;
    }
}
export declare namespace AWS.WAFRegional.ByteMatchSet {
    interface ByteMatchTuple {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-bytematchset-bytematchtuple.html#cfn-wafregional-bytematchset-bytematchtuple-targetstring
         */
        TargetString?: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-bytematchset-bytematchtuple.html#cfn-wafregional-bytematchset-bytematchtuple-targetstringbase64
         */
        TargetStringBase64?: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-bytematchset-bytematchtuple.html#cfn-wafregional-bytematchset-bytematchtuple-positionalconstraint
         */
        PositionalConstraint: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-bytematchset-bytematchtuple.html#cfn-wafregional-bytematchset-bytematchtuple-texttransformation
         */
        TextTransformation: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-bytematchset-bytematchtuple.html#cfn-wafregional-bytematchset-bytematchtuple-fieldtomatch
         */
        FieldToMatch: C.Yields<AWS.WAFRegional.ByteMatchSet.FieldToMatch>;
    }
}
export declare namespace AWS.WAFRegional.ByteMatchSet {
    interface FieldToMatch {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-bytematchset-fieldtomatch.html#cfn-wafregional-bytematchset-fieldtomatch-type
         */
        Type: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-bytematchset-fieldtomatch.html#cfn-wafregional-bytematchset-fieldtomatch-data
         */
        Data?: C.YieldsString;
    }
}
export declare namespace AWS.WAFRegional {
    /**
     * The AWS::WAFRegional::IPSet resource creates an AWS WAF Regional IPSet that
    specifies which web requests to permit or block based on the IP addresses from which
    the
    requests originate. For more information, see CreateIPSet in the AWS WAF Regional API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-ipset.html
     */
    interface IPSet {
        Type: 'AWS::WAFRegional::IPSet';
        Properties: AWS.WAFRegional.IPSet.Properties;
    }
    /**
     * The AWS::WAFRegional::IPSet resource creates an AWS WAF Regional IPSet that
    specifies which web requests to permit or block based on the IP addresses from which
    the
    requests originate. For more information, see CreateIPSet in the AWS WAF Regional API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-ipset.html
     */
    function IPSet(props: C.Omit<IPSet, 'Type'>): IPSet;
}
export declare namespace AWS.WAFRegional.IPSet {
    interface Properties {
        /**
         * List of AWS WAF Regional IPSet IPSetDescriptors
         *
         * The IP address type and IP address range (in CIDR notation) from which web
 requests originate. If you associate the IPSet with a web ACL that is associated with a Amazon CloudFront
 (CloudFront) distribution, this descriptor is the value of one of the following fields
 in the
 CloudFront access logs:

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-ipset.html#cfn-wafregional-ipset-ipsetdescriptors
         */
        IPSetDescriptors?: C.Yields<C.CFList<C.Yields<AWS.WAFRegional.IPSet.IPSetDescriptor>>>;
        /**
         * String

         *
         * A friendly name or description of the IPSet.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-ipset.html#cfn-wafregional-ipset-name
         */
        Name: C.YieldsString;
    }
}
export declare namespace AWS.WAFRegional.IPSet {
    interface IPSetDescriptor {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-ipset-ipsetdescriptor.html#cfn-wafregional-ipset-ipsetdescriptor-type
         */
        Type: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-ipset-ipsetdescriptor.html#cfn-wafregional-ipset-ipsetdescriptor-value
         */
        Value: C.YieldsString;
    }
}
export declare namespace AWS.WAFRegional {
    /**
     * The AWS::WAFRegional::Rule resource creates an AWS WAF Regional rule that specifies a
    combination of IPSet, ByteMatchSet, and
    SqlInjectionMatchSet objects that identify the web requests to allow, block, or
    count. To implement rules, you must associate them with a web ACL.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-rule.html
     */
    interface Rule {
        Type: 'AWS::WAFRegional::Rule';
        Properties: AWS.WAFRegional.Rule.Properties;
    }
    /**
     * The AWS::WAFRegional::Rule resource creates an AWS WAF Regional rule that specifies a
    combination of IPSet, ByteMatchSet, and
    SqlInjectionMatchSet objects that identify the web requests to allow, block, or
    count. To implement rules, you must associate them with a web ACL.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-rule.html
     */
    function Rule(props: C.Omit<Rule, 'Type'>): Rule;
}
export declare namespace AWS.WAFRegional.Rule {
    interface Properties {
        /**
         * String

         *
         * A friendly name or description for the metrics of the rule. For valid values,
 see the MetricName parameter for the CreateRule
 action in the AWS WAF Regional API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-rule.html#cfn-wafregional-rule-metricname
         */
        MetricName: C.YieldsString;
        /**
         * List of AWS WAF Regional Rule Predicates
         *
         * The ByteMatchSet, IPSet, SizeConstraintSet, SqlInjectionMatchSet, or XssMatchSet objects to include in a rule. If you add more than one predicate to a rule, a request
 must match all conditions in order to be allowed or blocked.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-rule.html#cfn-wafregional-rule-predicates
         */
        Predicates?: C.Yields<C.CFList<C.Yields<AWS.WAFRegional.Rule.Predicate>>>;
        /**
         * String

         *
         * A friendly name or description of the rule.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-rule.html#cfn-wafregional-rule-name
         */
        Name: C.YieldsString;
    }
}
export declare namespace AWS.WAFRegional.Rule {
    interface Predicate {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-rule-predicate.html#cfn-wafregional-rule-predicate-type
         */
        Type: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-rule-predicate.html#cfn-wafregional-rule-predicate-dataid
         */
        DataId: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-rule-predicate.html#cfn-wafregional-rule-predicate-negated
         */
        Negated: C.YieldsBoolean;
    }
}
export declare namespace AWS.WAFRegional {
    /**
     * The AWS::WAFRegional::SizeConstraintSet resource specifies a size constraint that AWS WAF uses to check the size of a web
    request and which parts of the request to check. For more information, see CreateSizeConstraintSet in the AWS WAF Regional API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-sizeconstraintset.html
     */
    interface SizeConstraintSet {
        Type: 'AWS::WAFRegional::SizeConstraintSet';
        Properties: AWS.WAFRegional.SizeConstraintSet.Properties;
    }
    /**
     * The AWS::WAFRegional::SizeConstraintSet resource specifies a size constraint that AWS WAF uses to check the size of a web
    request and which parts of the request to check. For more information, see CreateSizeConstraintSet in the AWS WAF Regional API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-sizeconstraintset.html
     */
    function SizeConstraintSet(props: C.Omit<SizeConstraintSet, 'Type'>): SizeConstraintSet;
}
export declare namespace AWS.WAFRegional.SizeConstraintSet {
    interface Properties {
        /**
         * List of AWS WAF Regional SizeConstraintSet SizeConstraint
         *
         * The size constraint and the part of the web request to check.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-sizeconstraintset.html#cfn-wafregional-sizeconstraintset-sizeconstraints
         */
        SizeConstraints?: C.Yields<C.CFList<C.Yields<AWS.WAFRegional.SizeConstraintSet.SizeConstraint>>>;
        /**
         * String

         *
         * A friendly name or description for the SizeConstraintSet.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-sizeconstraintset.html#cfn-wafregional-sizeconstraintset-name
         */
        Name: C.YieldsString;
    }
}
export declare namespace AWS.WAFRegional.SizeConstraintSet {
    interface FieldToMatch {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-sizeconstraintset-fieldtomatch.html#cfn-wafregional-sizeconstraintset-fieldtomatch-type
         */
        Type: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-sizeconstraintset-fieldtomatch.html#cfn-wafregional-sizeconstraintset-fieldtomatch-data
         */
        Data?: C.YieldsString;
    }
}
export declare namespace AWS.WAFRegional.SizeConstraintSet {
    interface SizeConstraint {
        /**
         * String

         *
         * The type of comparison that you want AWS WAF to perform. AWS WAF uses this value in
 combination with the Size and FieldToMatch property values to check if the size constraint is a match. For more information
 and valid values, see the ComparisonOperator content for the SizeConstraint data type in the AWS WAF Regional API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-sizeconstraintset-sizeconstraint.html#cfn-wafregional-sizeconstraintset-sizeconstraint-comparisonoperator
         */
        ComparisonOperator: C.YieldsString;
        /**
         * Integer

         *
         * The size in bytes that you want AWS WAF to compare against the size of the specified
 FieldToMatch. AWS WAF uses Size in combination with the ComparisonOperator and FieldToMatch property values to check if the size constraint of a web request is a match. For
 more information and valid values, see the Size content for the SizeConstraint data type in the
 AWS WAF Regional API Reference.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-sizeconstraintset-sizeconstraint.html#cfn-wafregional-sizeconstraintset-sizeconstraint-size
         */
        Size: C.Yields<C.CFInteger>;
        /**
         * String

         *
         * Specifies how AWS WAF processes the FieldToMatch property before inspecting a request for a match. Text transformations eliminate
 some of the unusual formatting that attackers use in web requests in an effort to
 bypass AWS WAF. If you specify a transformation, AWS WAF transforms the FieldToMatch before inspecting a web request for a match.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-sizeconstraintset-sizeconstraint.html#cfn-wafregional-sizeconstraintset-sizeconstraint-texttransformation
         */
        TextTransformation: C.YieldsString;
        /**
         * AWS WAF Regional SizeConstraintSet SizeConstraint FieldToMatch
         *
         * The part of a web request that you want AWS WAF to search, such as a specific header
 or a query string.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-sizeconstraintset-sizeconstraint.html#cfn-wafregional-sizeconstraintset-sizeconstraint-fieldtomatch
         */
        FieldToMatch: C.Yields<AWS.WAFRegional.SizeConstraintSet.FieldToMatch>;
    }
}
export declare namespace AWS.WAFRegional {
    /**
     * The AWS::WAFRegional::SqlInjectionMatchSet resource creates an AWS WAF Regional
    SqlInjectionMatchSet, which you use to allow, block, or count requests that
    contain malicious SQL code in a specific part of web requests. For more information,
    see CreateSqlInjectionMatchSet in
    the AWS WAF Regional API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-sqlinjectionmatchset.html
     */
    interface SqlInjectionMatchSet {
        Type: 'AWS::WAFRegional::SqlInjectionMatchSet';
        Properties: AWS.WAFRegional.SqlInjectionMatchSet.Properties;
    }
    /**
     * The AWS::WAFRegional::SqlInjectionMatchSet resource creates an AWS WAF Regional
    SqlInjectionMatchSet, which you use to allow, block, or count requests that
    contain malicious SQL code in a specific part of web requests. For more information,
    see CreateSqlInjectionMatchSet in
    the AWS WAF Regional API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-sqlinjectionmatchset.html
     */
    function SqlInjectionMatchSet(props: C.Omit<SqlInjectionMatchSet, 'Type'>): SqlInjectionMatchSet;
}
export declare namespace AWS.WAFRegional.SqlInjectionMatchSet {
    interface Properties {
        /**
         * List of AWS WAF Regional
    SqlInjectionMatchSet SqlInjectionMatchTuples
         *
         * The parts of web requests that you want AWS WAF to inspect for malicious SQL code
 and, if you want AWS WAF to inspect a header, the name of the header.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-sqlinjectionmatchset.html#cfn-wafregional-sqlinjectionmatchset-sqlinjectionmatchtuples
         */
        SqlInjectionMatchTuples?: C.Yields<C.CFList<C.Yields<AWS.WAFRegional.SqlInjectionMatchSet.SqlInjectionMatchTuple>>>;
        /**
         * String

         *
         * A friendly name or description of the SqlInjectionMatchSet.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-sqlinjectionmatchset.html#cfn-wafregional-sqlinjectionmatchset-name
         */
        Name: C.YieldsString;
    }
}
export declare namespace AWS.WAFRegional.SqlInjectionMatchSet {
    interface FieldToMatch {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-sqlinjectionmatchset-fieldtomatch.html#cfn-wafregional-sqlinjectionmatchset-fieldtomatch-type
         */
        Type: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-sqlinjectionmatchset-fieldtomatch.html#cfn-wafregional-sqlinjectionmatchset-fieldtomatch-data
         */
        Data?: C.YieldsString;
    }
}
export declare namespace AWS.WAFRegional.SqlInjectionMatchSet {
    interface SqlInjectionMatchTuple {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-sqlinjectionmatchset-sqlinjectionmatchtuple.html#cfn-wafregional-sqlinjectionmatchset-sqlinjectionmatchtuple-texttransformation
         */
        TextTransformation: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-sqlinjectionmatchset-sqlinjectionmatchtuple.html#cfn-wafregional-sqlinjectionmatchset-sqlinjectionmatchtuple-fieldtomatch
         */
        FieldToMatch: C.Yields<AWS.WAFRegional.SqlInjectionMatchSet.FieldToMatch>;
    }
}
export declare namespace AWS.WAFRegional {
    /**
     * The AWS::WAFRegional::WebACL resource creates an AWS WAF Regional web access control group
    (ACL) containing the rules that identify the Amazon CloudFront (CloudFront) web requests
    that you want to
    allow, block, or count. For more information, see CreateWebACL in the
    AWS WAF Regional API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-webacl.html
     */
    interface WebACL {
        Type: 'AWS::WAFRegional::WebACL';
        Properties: AWS.WAFRegional.WebACL.Properties;
    }
    /**
     * The AWS::WAFRegional::WebACL resource creates an AWS WAF Regional web access control group
    (ACL) containing the rules that identify the Amazon CloudFront (CloudFront) web requests
    that you want to
    allow, block, or count. For more information, see CreateWebACL in the
    AWS WAF Regional API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-webacl.html
     */
    function WebACL(props: C.Omit<WebACL, 'Type'>): WebACL;
}
export declare namespace AWS.WAFRegional.WebACL {
    interface Properties {
        /**
         * String

         *
         * A friendly name or description for the Amazon CloudWatch metric of this web ACL. For
 valid values, see the MetricName parameter of the CreateWebACL action in the AWS WAF Regional API Reference.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-webacl.html#cfn-wafregional-webacl-metricname
         */
        MetricName: C.YieldsString;
        /**
         * AWS WAF Regional WebACL Action
         *
         * The action that you want AWS WAF to take when a request doesn't match the
 criteria in any of the rules that are associated with the web ACL.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-webacl.html#cfn-wafregional-webacl-defaultaction
         */
        DefaultAction: C.Yields<AWS.WAFRegional.WebACL.Action>;
        /**
         * List of AWS WAF Regional WebACL Rules
         *
         * The rules to associate with the web ACL and the settings for each rule.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-webacl.html#cfn-wafregional-webacl-rules
         */
        Rules?: C.Yields<C.CFList<C.Yields<AWS.WAFRegional.WebACL.Rule>>>;
        /**
         * String

         *
         * A friendly name or description of the web ACL.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-webacl.html#cfn-wafregional-webacl-name
         */
        Name: C.YieldsString;
    }
}
export declare namespace AWS.WAFRegional.WebACL {
    interface Action {
        /**
         * String

         *
         * For actions that are associated with a rule, the action that AWS WAF takes when a
 web request matches all conditions in a rule.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-webacl-action.html#cfn-wafregional-webacl-action-type
         */
        Type: C.YieldsString;
    }
}
export declare namespace AWS.WAFRegional.WebACL {
    interface Rule {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-webacl-rule.html#cfn-wafregional-webacl-rule-action
         */
        Action: C.Yields<AWS.WAFRegional.WebACL.Action>;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-webacl-rule.html#cfn-wafregional-webacl-rule-priority
         */
        Priority: C.Yields<C.CFInteger>;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-webacl-rule.html#cfn-wafregional-webacl-rule-ruleid
         */
        RuleId: C.YieldsString;
    }
}
export declare namespace AWS.WAFRegional {
    /**
     * The AWS::WAFRegional::WebACLAssociation resource associates an AWS WAF Regional web access control group
    (ACL) with a resource. For more information, see AssociateWebACL in the
    AWS WAF Regional API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-webaclassociation.html
     */
    interface WebACLAssociation {
        Type: 'AWS::WAFRegional::WebACLAssociation';
        Properties: AWS.WAFRegional.WebACLAssociation.Properties;
    }
    /**
     * The AWS::WAFRegional::WebACLAssociation resource associates an AWS WAF Regional web access control group
    (ACL) with a resource. For more information, see AssociateWebACL in the
    AWS WAF Regional API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-webaclassociation.html
     */
    function WebACLAssociation(props: C.Omit<WebACLAssociation, 'Type'>): WebACLAssociation;
}
export declare namespace AWS.WAFRegional.WebACLAssociation {
    interface Properties {
        /**
         * String

         *
         * The Amazon Resource Name (ARN) of the resource to protect with the web ACL.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-webaclassociation.html#cfn-wafregional-webaclassociation-resourcearn
         */
        ResourceArn: C.YieldsString;
        /**
         * String

         *
         * A unique identifier (ID) for the web ACL.
         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-webaclassociation.html#cfn-wafregional-webaclassociation-webaclid
         */
        WebACLId: C.YieldsString;
    }
}
export declare namespace AWS.WAFRegional {
    /**
     * The AWS::WAFRegional::XssMatchSet resource specifies the parts of web requests that you want AWS WAF to inspect for
    cross-site scripting attacks and the name of the header to inspect. For more information,
    see XssMatchSet in the AWS WAF Regional API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-xssmatchset.html
     */
    interface XssMatchSet {
        Type: 'AWS::WAFRegional::XssMatchSet';
        Properties: AWS.WAFRegional.XssMatchSet.Properties;
    }
    /**
     * The AWS::WAFRegional::XssMatchSet resource specifies the parts of web requests that you want AWS WAF to inspect for
    cross-site scripting attacks and the name of the header to inspect. For more information,
    see XssMatchSet in the AWS WAF Regional API Reference.
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-xssmatchset.html
     */
    function XssMatchSet(props: C.Omit<XssMatchSet, 'Type'>): XssMatchSet;
}
export declare namespace AWS.WAFRegional.XssMatchSet {
    interface Properties {
        /**
         * List of AWS WAF Regional XssMatchSet XssMatchTuple
         *
         * The parts of web requests that you want to inspect for cross-site scripting attacks.
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-xssmatchset.html#cfn-wafregional-xssmatchset-xssmatchtuples
         */
        XssMatchTuples?: C.Yields<C.CFList<C.Yields<AWS.WAFRegional.XssMatchSet.XssMatchTuple>>>;
        /**
         * String

         *
         * A friendly name or description for the XssMatchSet.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-wafregional-xssmatchset.html#cfn-wafregional-xssmatchset-name
         */
        Name: C.YieldsString;
    }
}
export declare namespace AWS.WAFRegional.XssMatchSet {
    interface FieldToMatch {
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-xssmatchset-fieldtomatch.html#cfn-wafregional-xssmatchset-fieldtomatch-type
         */
        Type: C.YieldsString;
        /**
         *
         *
         *
         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-xssmatchset-fieldtomatch.html#cfn-wafregional-xssmatchset-fieldtomatch-data
         */
        Data?: C.YieldsString;
    }
}
export declare namespace AWS.WAFRegional.XssMatchSet {
    interface XssMatchTuple {
        /**
         * String

         *
         * Specifies how AWS WAF processes the FieldToMatch property before inspecting a request for a match. Text transformations eliminate
 some of the unusual formatting that attackers use in web requests in an effort to
 bypass AWS WAF. If you specify a transformation, AWS WAF transforms theFieldToMatch parameter before inspecting a web request for a match.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-xssmatchset-xssmatchtuple.html#cfn-wafregional-xssmatchset-xssmatchtuple-texttransformation
         */
        TextTransformation: C.YieldsString;
        /**
         * AWS WAF Regional XssMatchSet XssMatchTuple FieldToMatch
         *
         * The part of a web request that you want AWS WAF to search, such as a specific header
 or a query string.

         *
         * UpdateType: Mutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-wafregional-xssmatchset-xssmatchtuple.html#cfn-wafregional-xssmatchset-xssmatchtuple-fieldtomatch
         */
        FieldToMatch: C.Yields<AWS.WAFRegional.XssMatchSet.FieldToMatch>;
    }
}
export declare namespace AWS.WorkSpaces {
    /**
     * The AWS::WorkSpaces::Workspace resource creates an Amazon WorkSpaces workspace, which is a
    cloud-based desktop experience for end users. Before creating a Workspace
    in CloudFormation, you must register a Directory Service directory with
    Workspaces. This process is documented at Register a Directory with Amazon WorkSpaces. For more information, see the Amazon WorkSpaces Administration Guide.
    
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-workspaces-workspace.html
     */
    interface Workspace {
        Type: 'AWS::WorkSpaces::Workspace';
        Properties: AWS.WorkSpaces.Workspace.Properties;
    }
    /**
     * The AWS::WorkSpaces::Workspace resource creates an Amazon WorkSpaces workspace, which is a
    cloud-based desktop experience for end users. Before creating a Workspace
    in CloudFormation, you must register a Directory Service directory with
    Workspaces. This process is documented at Register a Directory with Amazon WorkSpaces. For more information, see the Amazon WorkSpaces Administration Guide.
    
 
     *
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-workspaces-workspace.html
     */
    function Workspace(props: C.Omit<Workspace, 'Type'>): Workspace;
}
export declare namespace AWS.WorkSpaces.Workspace {
    interface Properties {
        /**
         * String

         *
         * The identifier of the bundle from which you want to create the workspace. A
 bundle specifies the details of the workspace, such as the installed applications
 and the size of CPU, memory, and storage. Use the DescribeWorkspaceBundles action to list the bundles that AWS
 offers.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-workspaces-workspace.html#cfn-workspaces-workspace-bundleid
         */
        BundleId: C.YieldsString;
        /**
         * String

         *
         * The identifier of the AWS Directory Service directory in which you want to create
 the
 workspace. The directory must already be registered with Amazon WorkSpaces. Use the
 DescribeWorkspaceDirectories action to list the directories that are
 available.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-workspaces-workspace.html#cfn-workspaces-workspace-directoryid
         */
        DirectoryId: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether Amazon WorkSpaces encrypts data stored on the root volume
 (C: drive).

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-workspaces-workspace.html#cfn-workspaces-workspace-rootvolumeencryptionenabled
         */
        RootVolumeEncryptionEnabled?: C.YieldsBoolean;
        /**
         * String

         *
         * The name of the user to which the workspace is assigned. This user name must
 exist in the specified AWS Directory Service directory.

         *
         * UpdateType: Immutable
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-workspaces-workspace.html#cfn-workspaces-workspace-username
         */
        UserName: C.YieldsString;
        /**
         * Boolean

         *
         * Indicates whether Amazon WorkSpaces encrypts data stored on the user volume
 (D: drive).

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-workspaces-workspace.html#cfn-workspaces-workspace-uservolumeencryptionenabled
         */
        UserVolumeEncryptionEnabled?: C.YieldsBoolean;
        /**
         * String

         *
         * The AWS Key Management Service (AWS KMS) key ID that Amazon WorkSpaces uses to encrypt
 data stored on your
 workspace.

         *
         * UpdateType: Conditional
         * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-workspaces-workspace.html#cfn-workspaces-workspace-volumeencryptionkey
         */
        VolumeEncryptionKey?: C.YieldsString;
    }
}
export interface Tag {
    /**
     * String

     *
     * The key name of the tag. You can specify a value that is 1 to 127 Unicode characters
in length and cannot be prefixed with aws:. You can use any of the following characters: the set of Unicode letters, digits,
whitespace, _, ., /, =, +, and -.

     *
     * UpdateType: Mutable
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-resource-tags.html#cfn-resource-tags-key
     */
    Key: C.YieldsString;
    /**
     * String

     *
     * The value for the tag. You can specify a value that is 1 to 255 Unicode characters
in length and cannot be prefixed with aws:. You can use any of the following characters: the set of Unicode letters, digits,
whitespace, _, ., /, =, +, and -.

     *
     * UpdateType: Mutable
     * Documentation: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-resource-tags.html#cfn-resource-tags-value
     */
    Value: C.YieldsString;
}
export declare type Resource = AWS.ApiGateway.Account | AWS.ApiGateway.ApiKey | AWS.ApiGateway.Authorizer | AWS.ApiGateway.BasePathMapping | AWS.ApiGateway.ClientCertificate | AWS.ApiGateway.Deployment | AWS.ApiGateway.DocumentationPart | AWS.ApiGateway.DocumentationVersion | AWS.ApiGateway.DomainName | AWS.ApiGateway.GatewayResponse | AWS.ApiGateway.Method | AWS.ApiGateway.Model | AWS.ApiGateway.RequestValidator | AWS.ApiGateway.Resource | AWS.ApiGateway.RestApi | AWS.ApiGateway.Stage | AWS.ApiGateway.UsagePlan | AWS.ApiGateway.UsagePlanKey | AWS.ApplicationAutoScaling.ScalableTarget | AWS.ApplicationAutoScaling.ScalingPolicy | AWS.AutoScaling.AutoScalingGroup | AWS.AutoScaling.LaunchConfiguration | AWS.AutoScaling.LifecycleHook | AWS.AutoScaling.ScalingPolicy | AWS.AutoScaling.ScheduledAction | AWS.Batch.ComputeEnvironment | AWS.Batch.JobDefinition | AWS.Batch.JobQueue | AWS.CertificateManager.Certificate | AWS.CloudFormation.CustomResource | AWS.CloudFormation.Stack | AWS.CloudFormation.WaitCondition | AWS.CloudFormation.WaitConditionHandle | AWS.CloudFront.Distribution | AWS.CloudTrail.Trail | AWS.CloudWatch.Alarm | AWS.CloudWatch.Dashboard | AWS.CodeBuild.Project | AWS.CodeCommit.Repository | AWS.CodeDeploy.Application | AWS.CodeDeploy.DeploymentConfig | AWS.CodeDeploy.DeploymentGroup | AWS.CodePipeline.CustomActionType | AWS.CodePipeline.Pipeline | AWS.Cognito.IdentityPool | AWS.Cognito.IdentityPoolRoleAttachment | AWS.Cognito.UserPool | AWS.Cognito.UserPoolClient | AWS.Cognito.UserPoolGroup | AWS.Cognito.UserPoolUser | AWS.Cognito.UserPoolUserToGroupAttachment | AWS.Config.ConfigRule | AWS.Config.ConfigurationRecorder | AWS.Config.DeliveryChannel | AWS.DAX.Cluster | AWS.DAX.ParameterGroup | AWS.DAX.SubnetGroup | AWS.DMS.Certificate | AWS.DMS.Endpoint | AWS.DMS.EventSubscription | AWS.DMS.ReplicationInstance | AWS.DMS.ReplicationSubnetGroup | AWS.DMS.ReplicationTask | AWS.DataPipeline.Pipeline | AWS.DirectoryService.MicrosoftAD | AWS.DirectoryService.SimpleAD | AWS.DynamoDB.Table | AWS.EC2.CustomerGateway | AWS.EC2.DHCPOptions | AWS.EC2.EIP | AWS.EC2.EIPAssociation | AWS.EC2.EgressOnlyInternetGateway | AWS.EC2.FlowLog | AWS.EC2.Host | AWS.EC2.Instance | AWS.EC2.InternetGateway | AWS.EC2.NatGateway | AWS.EC2.NetworkAcl | AWS.EC2.NetworkAclEntry | AWS.EC2.NetworkInterface | AWS.EC2.NetworkInterfaceAttachment | AWS.EC2.NetworkInterfacePermission | AWS.EC2.PlacementGroup | AWS.EC2.Route | AWS.EC2.RouteTable | AWS.EC2.SecurityGroup | AWS.EC2.SecurityGroupEgress | AWS.EC2.SecurityGroupIngress | AWS.EC2.SpotFleet | AWS.EC2.Subnet | AWS.EC2.SubnetCidrBlock | AWS.EC2.SubnetNetworkAclAssociation | AWS.EC2.SubnetRouteTableAssociation | AWS.EC2.TrunkInterfaceAssociation | AWS.EC2.VPC | AWS.EC2.VPCCidrBlock | AWS.EC2.VPCDHCPOptionsAssociation | AWS.EC2.VPCEndpoint | AWS.EC2.VPCGatewayAttachment | AWS.EC2.VPCPeeringConnection | AWS.EC2.VPNConnection | AWS.EC2.VPNConnectionRoute | AWS.EC2.VPNGateway | AWS.EC2.VPNGatewayRoutePropagation | AWS.EC2.Volume | AWS.EC2.VolumeAttachment | AWS.ECR.Repository | AWS.ECS.Cluster | AWS.ECS.Service | AWS.ECS.TaskDefinition | AWS.EFS.FileSystem | AWS.EFS.MountTarget | AWS.EMR.Cluster | AWS.EMR.InstanceFleetConfig | AWS.EMR.InstanceGroupConfig | AWS.EMR.SecurityConfiguration | AWS.EMR.Step | AWS.ElastiCache.CacheCluster | AWS.ElastiCache.ParameterGroup | AWS.ElastiCache.ReplicationGroup | AWS.ElastiCache.SecurityGroup | AWS.ElastiCache.SecurityGroupIngress | AWS.ElastiCache.SubnetGroup | AWS.ElasticBeanstalk.Application | AWS.ElasticBeanstalk.ApplicationVersion | AWS.ElasticBeanstalk.ConfigurationTemplate | AWS.ElasticBeanstalk.Environment | AWS.ElasticLoadBalancing.LoadBalancer | AWS.ElasticLoadBalancingV2.Listener | AWS.ElasticLoadBalancingV2.ListenerCertificate | AWS.ElasticLoadBalancingV2.ListenerRule | AWS.ElasticLoadBalancingV2.LoadBalancer | AWS.ElasticLoadBalancingV2.TargetGroup | AWS.Elasticsearch.Domain | AWS.Events.Rule | AWS.GameLift.Alias | AWS.GameLift.Build | AWS.GameLift.Fleet | AWS.IAM.AccessKey | AWS.IAM.Group | AWS.IAM.InstanceProfile | AWS.IAM.ManagedPolicy | AWS.IAM.Policy | AWS.IAM.Role | AWS.IAM.User | AWS.IAM.UserToGroupAddition | AWS.IoT.Certificate | AWS.IoT.Policy | AWS.IoT.PolicyPrincipalAttachment | AWS.IoT.Thing | AWS.IoT.ThingPrincipalAttachment | AWS.IoT.TopicRule | AWS.KMS.Alias | AWS.KMS.Key | AWS.Kinesis.Stream | AWS.KinesisAnalytics.Application | AWS.KinesisAnalytics.ApplicationOutput | AWS.KinesisAnalytics.ApplicationReferenceDataSource | AWS.KinesisFirehose.DeliveryStream | AWS.Lambda.Alias | AWS.Lambda.EventSourceMapping | AWS.Lambda.Function | AWS.Lambda.Permission | AWS.Lambda.Version | AWS.Logs.Destination | AWS.Logs.LogGroup | AWS.Logs.LogStream | AWS.Logs.MetricFilter | AWS.Logs.SubscriptionFilter | AWS.OpsWorks.App | AWS.OpsWorks.ElasticLoadBalancerAttachment | AWS.OpsWorks.Instance | AWS.OpsWorks.Layer | AWS.OpsWorks.Stack | AWS.OpsWorks.UserProfile | AWS.OpsWorks.Volume | AWS.RDS.DBCluster | AWS.RDS.DBClusterParameterGroup | AWS.RDS.DBInstance | AWS.RDS.DBParameterGroup | AWS.RDS.DBSecurityGroup | AWS.RDS.DBSecurityGroupIngress | AWS.RDS.DBSubnetGroup | AWS.RDS.EventSubscription | AWS.RDS.OptionGroup | AWS.Redshift.Cluster | AWS.Redshift.ClusterParameterGroup | AWS.Redshift.ClusterSecurityGroup | AWS.Redshift.ClusterSecurityGroupIngress | AWS.Redshift.ClusterSubnetGroup | AWS.Route53.HealthCheck | AWS.Route53.HostedZone | AWS.Route53.RecordSet | AWS.Route53.RecordSetGroup | AWS.S3.Bucket | AWS.S3.BucketPolicy | AWS.SDB.Domain | AWS.SNS.Subscription | AWS.SNS.Topic | AWS.SNS.TopicPolicy | AWS.SQS.Queue | AWS.SQS.QueuePolicy | AWS.SSM.Association | AWS.SSM.Document | AWS.SSM.Parameter | AWS.StepFunctions.Activity | AWS.StepFunctions.StateMachine | AWS.WAF.ByteMatchSet | AWS.WAF.IPSet | AWS.WAF.Rule | AWS.WAF.SizeConstraintSet | AWS.WAF.SqlInjectionMatchSet | AWS.WAF.WebACL | AWS.WAF.XssMatchSet | AWS.WAFRegional.ByteMatchSet | AWS.WAFRegional.IPSet | AWS.WAFRegional.Rule | AWS.WAFRegional.SizeConstraintSet | AWS.WAFRegional.SqlInjectionMatchSet | AWS.WAFRegional.WebACL | AWS.WAFRegional.WebACLAssociation | AWS.WAFRegional.XssMatchSet | AWS.WorkSpaces.Workspace;
